/*
 *++
 *  FACILITY:
 *
 *      anagram.c
 *
 *  ABSTRACT:
 *
 *	Module contain anagram functions.  This replace the old anagram module
 *	which was full of bugs and wasn't mine anyhow!
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      6 February 1995
 *--
 */


/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "main.h"
#include "messages.h"
#include "memman.h"
#include "match.h"
#include "utils.h"
#include "word.h"
#include "strll.h"

#define SUBANAGRAM_MIN_LEN	3



/*
 *  ?? Code total anagram generator should go here ??
 */



/*
 *  ?? Add in max_size too ??
 */

PUBLIC int cword_anagram_get_subanagrams (
	char			*target_str,
	int			min_size,
	char			centre_letter,
	CWORD_T_STR_LL		**str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is used to generate all the possible full or partial
 *	anagrams of the string inputted through the parameter list.  I think it
 *	uses the 'TARGET' model of anagram generaton.
 *
 *  FORMAL PARAMETERS:
 *
 *      *target_str		(r) String containing characters to generate anagrams of
 *	min_size		(r) Minimum size of the resultant anagram
 *	centre_letter		(r) Letter which must be present in each anagram if any
 *	**str_ll		(w) Linked list of sub anagrams found.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    char *chp, 
	 word_str[CWORD_SIZEOF_WORD + 1];

    int count = 0, 
	matches, 
	*mark, 
	length_target, 
	i, 
	wl,
	status;

    BOOLEAN is_centre_found = FALSE,
	    is_centre = FALSE;

    cword_messages_output_message (
	CWORD_CLOCKBEFORE,
	TRACE2,
	clock ());

    *str_ll = (CWORD_T_STR_LL *) 0;

    if (cword_word_is_alphabetics_only (target_str) != CWORD_S_NORMAL)
    {
	cword_messages_output_message (
		CWORD_INVALIDCHAR,
		(TRACE2 | SCREEN0),
		target_str);

    	return (CWORD_S_FAILURE);
    }

    cword_word_lower_str (target_str);

    /*
     *  Verification of input of correct parameters!
     */

    if (centre_letter != (char) 0)
    {
	if (isupper (centre_letter) != 0)
	    centre_letter = tolower (centre_letter);

	/*
	 *  If centre letter specified then check for it!
	 */

	for (count = 0; count < strlen (target_str); count++)
	{
	    if (target_str[count] == centre_letter)
	    {
		is_centre_found = TRUE;
		break;
	    }
	}

	if (is_centre_found  == FALSE)
	{
	    cword_messages_output_message (
		CWORD_CENTRENOTFOUND,
		(TRACE2 | SCREEN0),
		centre_letter);

	    return (CWORD_S_FAILURE);
	}
    }

    if (min_size > strlen (target_str))
    {
	cword_messages_output_message (
	    CWORD_MINSIZETOOBIG,
	    (TRACE2 | SCREEN0),
	    min_size, 
	    target_str);

	return (CWORD_S_FAILURE);
    }

    cword_messages_output_message (
	CWORD_MATCHSUBANAGRAMS,
	(TRACE2 | SCREEN0),
	target_str);

    /*
     *  Initialise parameters
     */

    if (strlen (target_str) == 0)
    {
	cword_messages_output_message (
	    CWORD_OUTFILEMSG,
	    (TRACE2 | SCREEN0));

	return (CWORD_S_FAILURE);
    }
    else
    {
	length_target = strlen (target_str);

    	if (min_size > length_target)
    	{
	    cword_messages_output_message (
		CWORD_OUTFILEMSG,
		(TRACE2 | SCREEN0));

    	    return (CWORD_S_FAILURE);
    	}
    }

    if ( (min_size == 0) && (strlen (target_str) > SUBANAGRAM_MIN_LEN) )
    {
	min_size = SUBANAGRAM_MIN_LEN;
    }

    if (0 == (int) centre_letter)
    {
	cword_messages_output_message (
	    CWORD_MATCHNOTARGETSTAT,
	    (TRACE2 | SCREEN0),
	    min_size - 1, 
	    target_str);
    }
    else
    {
	cword_messages_output_message (
	    CWORD_MATCHTARGETSTAT,
	    (TRACE2 | SCREEN0),
	    centre_letter,
	    min_size - 1, 
	    target_str);
    }

    /*
     *	Dynamically allocate a pointer to an array of integers.  Remember to
     *	de-allocate the array.
     */

    mark = (int *) calloc ((size_t) length_target, sizeof (int));

    DEBUG_TRACE (
	"min = %d max = %d\n", 
	min_size, 
	strlen (target_str));

    /*
     *	Start the ball rolling with the initialisation of the read_word_files
     *	function.
     */

    status = cword_match_read_word_files (
		min_size,
		strlen (target_str),
		word_str);

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    cword_messages_output_message (
	CWORD_SUBANAGRAMSNOW,
	SCREEN0);

    /*
     *  Read until EOF of word files.
     */

    do 
    {
	for (i = 0; i < length_target; i++)
	    mark[i] = 0;

	matches = 0;

	is_centre = (centre_letter == (char) 0) ? TRUE : FALSE;

	wl = strlen (word_str);

	/*
	 *  Check that the length of word_str is greater than the minimum size
	 *  of anagrams expected.  This will always be true in here as I'm only
	 *  using words within specification.
	 */

	if (wl >= min_size) 
        {
	    for (chp = word_str; *chp != (char) 0; chp++)
	    {
		is_centre = (is_centre || (*chp == centre_letter)) ? TRUE : FALSE;

		for (i = 0; i < length_target; i++)
		{
		    if ( (mark[i] == 0) && (*chp == target_str[i]) )
		    {
			/*
			 *  Match for character[i] of target_str in word_str
			 */

			mark[i] = 1;

			/*
			 *  Number of matches++
			 */

			matches++;

			break;
		    }
		}
	    }
	}

	/*
	 *  If the number of matches found equals the strlen of word_str and
	 *  is_centre == TRUE.
	 */

	if ( (wl == matches) && (is_centre != FALSE) )
	{
	    cword_strll_add_str (
		SORTED | UNIQUE,
		word_str,
		str_ll);

	    if (*str_ll == (CWORD_T_STR_LL *) 0)
	    {
		CWORD_MATCH_WAIT_AND_CLOCK;

		free (mark);

		return (CWORD_S_FAILURE);
	    }

	    count++;
	}
    }
    while (CWORD_S_NORMAL == cword_match_read_word_files (0, 0, word_str));

    free (mark);

    cword_match_read_word_files (RESET, RESET, 0);

    CWORD_MATCH_WAIT_AND_CLOCK;

    count = (int) cword_strll_number_str_ll (*str_ll);

    if (count == 1)
	cword_messages_output_message (
	    CWORD_S_SUBANAGRAMFOUND,
	    (TRACE2 | SCREEN0),
	    count);
    else
	cword_messages_output_message (
	    CWORD_SUBANAGRAMSFOUND,
	    (TRACE2 | SCREEN0),
	    count);

    if (*str_ll == (CWORD_T_STR_LL *) 0)
    {
	cword_messages_output_message (
	    CWORD_NOSUBANAGRAMS,
	    (TRACE2 | SCREEN0));

	return (CWORD_S_NORMAL);	
    }

    cword_messages_output_message (
	CWORD_CLOCKAFTER,
	TRACE2,
	clock ());

    return (CWORD_S_NORMAL);

}
