/*
 *  ?? Check that all functions called from here do not accept pointers to
 *  empty or undefined link lists.  These loose ends can cause some major
 *  problems that are hard to track down when code is moved around ??
 */

/*
 *++
 *  FACILITY:
 *
 *      charui.c
 *
 *  ABSTRACT:
 *
 *	This module contains all the functions that implement the Character
 *	Cell User Interface (CCUI) of Cword.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      19 July 1993
 *--
 */

/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>

#include <ctype.h>

#if defined(VMS) || defined(__VMS)

#include <unixio.h>

#endif

#if defined(__MSDOS__) || defined(MSDOS)

/*
 *  Visual C++ header to prototype system ().
 */

#include <process.h>

#endif

#include "main.h"
#include "print.h"
#include "utils.h"
#include "os.h"
#include "grid.h"
#include "webster.h"
#include "anagram.h"
#include "messages.h"
#include "generate.h"
#include "charui.h"
#include "fileio.h"
#include "memman.h"
#include "match.h"
#include "stats.h"
#include "soundex.h"
#include "clue.h"
#include "word.h"
#include "defaults.h"
#include "verify.h"
#include "breakdic.h"
#include "cl.h"
#include "strll.h"

#if defined(VMS) || defined (__VMS)

#include "rdb.h"

#endif

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#ifndef CHARUI

/*
 *  Various hotkeys to used at Cword's command line.  More will be added here
 *  when necessary.
 */

#define EXIT_CHAR	'E'
#define SPAWN1_CHAR	'!'
#define SPAWN2_CHAR	'%'
#define SPAWN3_CHAR	'$'
#define BACK_CHAR	'B'
#define DEFAULT_CHAR	'D'
#define HELP_CHAR	'H'
#define QUESTION_CHAR   '?'

#define ADD_CHAR	'A'
#define DELETE_CHAR	'D'
#define QUIT_CHAR	'Q'

#define CD_CHAR		'C'
#define CD_STR		"CD "

#define BELL_CHAR	'\007'

/*
 *  Macros containing format of menu output
 */

#define MAIN_MENU_CHOICE(input_str) choice++; \
				    cword_messages_output_message ( \
					    CWORD_S_MAINMENUCHOICE, \
					    SCREEN0, \
					    choice, \
					    input_str);

#define EXIT_MAIN_MENU_CHOICE(input_str) cword_messages_output_message ( \
					    CWORD_S_MAINMENUCHOICE, \
					    SCREEN0, \
					    0, \
					    input_str);

#define SUB_MENU_CHOICE(input_str) choice++; \
				   if (choice < 10) \
				   { \
				       cword_messages_output_message ( \
						CWORD_S_SUBMENUCHOICE0, \
						SCREEN0, \
						choice, \
						input_str); \
				   } \
				   else \
				   { \
				       cword_messages_output_message ( \
						CWORD_S_SUBMENUCHOICE1, \
						SCREEN0, \
						choice, \
						input_str); \
			           }

#define EXIT_SUB_MENU_CHOICE(input_str) cword_messages_output_message ( \
					    CWORD_S_SUBMENUCHOICE0, \
					    SCREEN0, \
					    0, \
					    input_str);

#define MAIN_MENU_HEADER /* ?? Not needed ?? */

#define SUB_MENU_HEADER(input_str) cword_messages_output_message ( \
					    CWORD_S_SUBMENUHEADER, \
					    SCREEN0, \
					    cword_charui_l_main_menu_choice, \
					    input_str);



/*
 *  PRIVATE function prototypes.
 */

PRIVATE int cword_charui__display_1_menu ();

PRIVATE int cword_charui__display_2_menu ();

PRIVATE int cword_charui__display_3_menu ();

PRIVATE int cword_charui__display_4_menu ();

PRIVATE int cword_charui__display_5_menu ();

PRIVATE int cword_charui__display_6_menu ();

PRIVATE int cword_charui__alter_defaults ();

PRIVATE int cword_charui__display_defaults (
	int			message_id);

PRIVATE int cword_charui_modify_default (
	int			default_option);

PRIVATE void cword_charui__masthead_print ();

PRIVATE int cword_charui__output_help_screen (
	int			max_choice);

/*
 *  These two module statics are used to determine where the user is at any
 *  particular time in the CCUI menu interface.
 *
 *  ?? Not used in the code for any constructive purpose yet.  Could be useful
 *  in the future depending on the development of extra functionality in the
 *  CCUI area ??
 */

LOCAL unsigned int cword_charui_l_main_menu_choice = 0;
LOCAL unsigned int cword_charui_l_sub_menu_choice = 0;



/*
 *  ?? Make sure that no menu screen has more than LINESPERSCREEN of output ??
 */

PUBLIC int cword_charui_display_main ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This is the main user interface function called from main ().  The top
 *	level menu is displayed within this function and this calls all
 *	subordinate menus from here.
 *
 *	This functionality has to be made flexible enough to allow easy
 *	movement of the menu options.
 *	
 *  FORMAL PARAMETERS:
 *
 *      None
 *
 *  IMPLICIT INPUTS:
 *
 *      Not yet
 *
 *  IMPLICIT OUTPUTS:
 *
 *      Not yet
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *--
 */

{

    int choice = 0,
	total_choice = 0,
	status;

    do
    {
	CWORD_OS_CLEAR_SCREEN ();

	cword_charui__masthead_print ();

	choice = 0;
	MAIN_MENU_CHOICE("Generate grid");
	MAIN_MENU_CHOICE("Fill grid");
	MAIN_MENU_CHOICE("Complete crossword");
	MAIN_MENU_CHOICE("Word utilities");
	MAIN_MENU_CHOICE("Misc. other utilities");
	EXIT_MAIN_MENU_CHOICE("Exit!");

	/*
	 *  Save number of choices in total_choice.
	 */
	total_choice = choice;

	status = cword_charui_get_choice (
		    CWORD_INPUTCHOICE,
		    total_choice,
		    &choice);

	/*
	 *  Save the main menu choice selected in
	 *  cword_charui_l_main_menu_choice
	 */
	cword_charui_l_main_menu_choice = choice;

	if (CWORD_CHOICENOTVALID != status)
	{
	    /*
	     *  Output where we are currently in the menu interface.
	     */

	    cword_messages_output_message (
		CWORD_S_MENUNUMTRACEMSG,
		TRACE0,
		CWORD_MAIN_STR,
		cword_charui_l_main_menu_choice);

	    switch (choice)
	    {
	    case (1):

		status = cword_charui__display_1_menu ();
		break;

	    case (2):

		status = cword_charui__display_2_menu ();
		break;

	    case (3):

		status = cword_charui__display_3_menu ();
		break;

	    case (4):

		status = cword_charui__display_4_menu ();
		break;

	    case (5):

		status = cword_charui__display_5_menu ();
		break;

	    case (0):
		cword_messages_output_message (
		    CWORD_FINISHMSG,
		    SCREEN0);

		break;

	    default:

		break;
	    }

	    if (choice != 0)
		cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
	}
    }
    while (0 != choice);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_charui__display_1_menu ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is given over to functions concerned with the generation
 *	and printing of empty crossword grids.  No other activities will take
 *	place in this function.
 *
 *  FORMAL PARAMETERS:
 *
 *      None
 *
 *  IMPLICIT INPUTS:
 *
 *      Not yet
 *
 *  IMPLICIT OUTPUTS:
 *
 *      Not yet
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *--
 */

{

    CROSSWORD_RECORD *crossword_record = (CROSSWORD_RECORD *) 0;

    CWORD_T_STR_LL *str_ll = (CWORD_T_STR_LL *) 0;

    char file_str[CWORD_SIZEOF_FILESPEC + 1],
	 tempfile_str[CWORD_SIZEOF_FILESPEC + 1];

    int	total_choice = 0,
	choice = 0,
	status;

    do
    {
	CWORD_OS_CLEAR_SCREEN ();

	cword_charui__masthead_print ();

	SUB_MENU_HEADER ("Generate grid");

	choice = 0;

	SUB_MENU_CHOICE("Generate grid");

	cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);

	SUB_MENU_CHOICE("List grids suitable for printing");

	SUB_MENU_CHOICE("Report on grid definition");

	SUB_MENU_CHOICE("Write grid to text file");
	SUB_MENU_CHOICE("Write grid to postscript file");

#ifdef DB
	/*
	 *  Database code used only if /define=DB or -DDB is specified in
	 *  compile.
	 */

	cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);

	SUB_MENU_CHOICE("Read grid from database");
	SUB_MENU_CHOICE("Write grid to database");
	SUB_MENU_CHOICE("Delete grid from database");
	SUB_MENU_CHOICE("List grids in database");
#endif

	EXIT_SUB_MENU_CHOICE("Back to main menu!");

	total_choice = choice;

	status = cword_charui_get_choice (
		    CWORD_INPUTCHOICE,
		    total_choice,
		    &choice);

	cword_charui_l_sub_menu_choice = choice;

	if (CWORD_CHOICENOTVALID != status)
	{
	    switch (choice)
	    {
	    case (1):
		/*
		 *  Generate an empty grid for later filling.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Generate an empty grid) selected\n",
		    cword_charui_l_sub_menu_choice);

		crossword_record = cword_memman_make_crossword_rec ();

		status = cword_generate_create_grid (crossword_record);

		if ( (CWORD_S_NORMAL != status) || (crossword_record->clue_sum == 0) )
		{
		    cword_memman_free_cross_rec (crossword_record);

		    break;
		}

		cword_messages_output_message (
		    CWORD_GRDFILEWRITE,
		    SCREEN0,
		    cword_g_defaults_record.temp_grid_name, 
		    CWORD_SUFFIX_GRID_FILE);

		if (cword_charui_get_file_spec (
			FALSE,
			CWORD_SUFFIX_GRID_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_fileio_write_grid (
				file_str,
				crossword_record);		
		}
		else
		{
		    if (file_str[0] == (char) 0)
		    {
			sprintf (
			    tempfile_str,
			    "%s%s",
			    cword_g_defaults_record.temp_grid_name, 
			    CWORD_SUFFIX_GRID_FILE);

			status = cword_fileio_write_grid (
				    tempfile_str,
				    crossword_record);
		    }
		    else
		    {
			status = cword_fileio_write_grid (
				    file_str,
				    crossword_record);
		    }			
		}

		cword_memman_free_cross_rec (crossword_record);

		break;

	    case (2):
		/*
		 *  List grid files suitable for printing.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (List grid files suitable for printing) selected\n",
		    cword_charui_l_sub_menu_choice);

		sprintf (
		    file_str,
		    "*%s",
		    CWORD_SUFFIX_GRID_FILE);

		status = cword_stats_grid_files_info (file_str);

		break;

	    case (3):
		/*
		 *  Report on grid definition and associated statistics.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Report on grid definition) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_GETSTATSGRDMSG,
		    SCREEN0,
		    CWORD_SUFFIX_GRID_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_GRID_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    crossword_record = cword_memman_make_crossword_rec ();

		    status = cword_fileio_read_grid (
				file_str,
				crossword_record);

		    if (CWORD_S_NORMAL != status)
		    {
			cword_memman_free_cross_rec (crossword_record);

			break;
		    }	    

		    /*
		     *  ?? Why not a file_str rather than a crossword_record ??
		     */

		    status = cword_stats_report_on_grid (crossword_record);

		    cword_memman_free_cross_rec (crossword_record);
		}

		break;

	    case (4):
		/*
		 *  Output selected grid to text file.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Output grid definition to text file) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_OUTFILEMSG,
		    SCREEN0,
		    CWORD_SUFFIX_GRID_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_GRID_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_print_grid (
				TEXT,
				file_str);
		}

		break;

	    case (5):
		/*
		 *  Output selected grid to postscript file.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Output grid definition to postscript file!) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_OUTFILEMSG,
		    SCREEN0,
		    CWORD_SUFFIX_GRID_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_GRID_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_print_grid (
				POSTSCRIPT,
				file_str);
		}

		break;

#ifdef DB
	    case (6):
		/*
		 *  Read grid definition from the local Cword database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Read grid from database) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_READFROMDBMSG,
		    SCREEN0,
		    GRID_STR);

		if (cword_charui_get_str (
			ALL,
			BLANK_NOTOK,
			sizeof (file_str) - 1,
			file_str) == CWORD_S_NORMAL)
		{
		    crossword_record = cword_memman_make_crossword_rec ();

		    status = cword_rdb_get_grd (
				file_str,
				crossword_record);

		    if (CWORD_S_NORMAL == status)
		    {
			cword_messages_output_message (
			    CWORD_RDBREADSUCC,
			    (TRACE0 | SCREEN0),
			    GRID_STR,
			    file_str);
		    }
		    else
		    {
			cword_messages_output_message (
			    CWORD_RDBREADFAIL,
			    (TRACE0 | SCREEN0),
			    GRID_STR,
			    file_str);

			cword_memman_free_cross_rec (crossword_record);

			break;
		    }

		    cword_messages_output_message (
			CWORD_S_DBTOFILEMSG,
			SCREEN0,
			GRID_STR,
			CWORD_SUFFIX_GRID_FILE);

		    if (cword_charui_get_file_spec (
			    TRUE,
			    CWORD_SUFFIX_GRID_FILE,
			    sizeof (file_str),
			    file_str) == CWORD_S_NORMAL)
		    {
			status = cword_fileio_write_grid (
				    file_str,
				    crossword_record);

			if (CWORD_S_NORMAL != status)
			{
			    cword_memman_free_cross_rec (crossword_record);

			    break;
			}
		    }

		    cword_memman_free_cross_rec (crossword_record);
		}

		break;

	    case (7):
		/*
		 *  Write grid definition to the local Cword database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Write grid to database) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_WRITETODB,
		    SCREEN0,
		    GRID_STR,
		    CWORD_SUFFIX_GRID_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_GRID_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    crossword_record = cword_memman_make_crossword_rec ();

		    status = cword_fileio_read_grid (
				file_str,
				crossword_record);

		    if (CWORD_S_NORMAL != status)
		    {
			cword_memman_free_cross_rec (crossword_record);

			break;
		    }
		    
		    status = cword_rdb_add_grd (crossword_record);

		    if (CWORD_S_NORMAL == status)
		    {
			cword_messages_output_message (
			    CWORD_RDBADDSUCC,
			    (TRACE0 | SCREEN0),
			    GRID_STR,
			    crossword_record->name);
		    }	    
		    else
		    {
			cword_messages_output_message (
			    CWORD_RDBADDFAIL,
			    (TRACE0 | SCREEN0),
			    GRID_STR,
			    crossword_record->name);
		    }

		    cword_memman_free_cross_rec (crossword_record);
		}

		break;

	    case (8):
		/*
		 *  Delete grid definition from the local Cword database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Delete grid from database) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_DELFROMDB,
		    SCREEN0,
		    GRID_STR);

		if (cword_charui_get_str (
			ALL,
			BLANK_NOTOK,
			sizeof (file_str) - 1,
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_rdb_delete_grd (file_str);

		    if (CWORD_S_NORMAL == status)
		    {
			cword_messages_output_message (
			    CWORD_RDBDELSUCC,
			    (TRACE0 | SCREEN0),
			    GRID_STR,
			    file_str);
		    }
		    else
		    {
			cword_messages_output_message (
			    CWORD_S_RDBDELFAIL,
			    (TRACE0 | SCREEN0),
			    GRID_STR,
			    file_str);
		    }
		}

		break;

	    case (9):
		/*
		 *  List grid definitions in local Cword database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (List grids in database) selected\n",
		    cword_charui_l_sub_menu_choice);

		status = cword_rdb_get_grd_ll (&str_ll);

		if (status == CWORD_S_NORMAL)
		{
		    if (str_ll != (CWORD_T_STR_LL *) 0)
		    {
			cword_messages_output_message (
			    CWORD_S_DEFNSINDB,
			    SCREEN0,
			    GRIDS_STR);

			cword_strll_output_str_ll (
			    str_ll,
			    1);
		    }
		    else
		    {
			cword_messages_output_message (
			    CWORD_S_NODEFNSINDB,
			    SCREEN0,
			    GRIDS_STR);
		    }
		}

		cword_strll_free_str_ll (&str_ll);

		break;
#endif

	    case (0):
		WRITE_TRACE ("## MENU: option 0 (Back to main menu!) selected\n");

		break;

	    default:
		break;
	    }

	    if (choice != 0)
		cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
	}
    }
    while (0 != choice);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_charui__display_2_menu ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is given over to functions concerned with the filling and
 *	printing of empty/filled crossword definitions.  No other activities
 *	will take place in this function.
 *
 *  FORMAL PARAMETERS:
 *
 *      None
 *
 *  IMPLICIT INPUTS:
 *
 *      Not yet
 *
 *  IMPLICIT OUTPUTS:
 *
 *      Not yet
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *--
 */

{

    CROSSWORD_RECORD *crossword_record;

    char file_str[CWORD_SIZEOF_FILESPEC + 1],
	 tempfile_str[CWORD_SIZEOF_FILESPEC + 1];

    int	total_choice = 0,
	choice = 0,
	status;

    do
    {
	CWORD_OS_CLEAR_SCREEN ();

	cword_charui__masthead_print ();

	SUB_MENU_HEADER("Fill grid");

	choice = 0;

	SUB_MENU_CHOICE("Fill grid with lights");
	SUB_MENU_CHOICE("List grids suitable for filling");

	cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);

	SUB_MENU_CHOICE("List crossword files suitable for printing");

	SUB_MENU_CHOICE("Report on crossword definition");
	SUB_MENU_CHOICE("Write crossword to text file");
	SUB_MENU_CHOICE("Write crossword to postscript file");

	EXIT_SUB_MENU_CHOICE("Back to main menu!");

	total_choice = choice;

	status = cword_charui_get_choice (
		    CWORD_INPUTCHOICE,
		    total_choice,
		    &choice);

	cword_charui_l_sub_menu_choice = choice;

	if (CWORD_CHOICENOTVALID != status)
	{
	    switch (choice)
	    {
	    case (1):
		/*
		 *  fill grid file with lights.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Fill a grid with lights) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_FILLXWORD,
		    SCREEN0,
		    CWORD_SUFFIX_GRID_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_GRID_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    crossword_record = cword_memman_make_crossword_rec ();

		    status = cword_fileio_read_grid (
				file_str,
				crossword_record);

		    if (CWORD_S_NORMAL != status)
		    {
			cword_memman_free_cross_rec (crossword_record);

			break;
		    }

		    status = cword_grid_fill_desc (crossword_record);

		    if (CWORD_S_NORMAL != status)
		    {
			/*
			 *  Quitting during seeding is not considered an error
			 *  in the strict sense of the word.
			 */

			if (CWORD_S_SEEDQUITNOW != status)
			    cword_messages_output_message (
				CWORD_S_GRIDERROR, 
				(SCREEN0 | TRACE0));

			cword_memman_free_cross_rec (crossword_record);

			break;
		    }
		    else 
		    {
			cword_messages_output_message (
			    CWORD_XWORDFILEWRITE,
			    SCREEN0,
			    crossword_record->file_name,
			    CWORD_SUFFIX_XWORD_FILE);

			if (cword_charui_get_file_spec (
				FALSE,
				CWORD_SUFFIX_XWORD_FILE,
				sizeof (file_str),
				file_str) == CWORD_S_NORMAL)
			{
			    status = cword_fileio_write_crossword (
					file_str,
					crossword_record);
			}
			else
			{
			    if (file_str[0] == (char) 0)
			    {
				sprintf (
				    tempfile_str,
				    "%s%s",
				    crossword_record->file_name,
				    CWORD_SUFFIX_XWORD_FILE);

				status = cword_fileio_write_crossword (
					    tempfile_str,
					    crossword_record);
			    }
			    else
			    {
				status = cword_fileio_write_crossword (
					    file_str,
					    crossword_record);
			    }
			}
		    }

		    cword_memman_free_cross_rec (crossword_record);
		}

		break;

	    case (2):
		/*
		 *  List grid files suitable for printing.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (List grid files suitable for filling) selected\n",
		    cword_charui_l_sub_menu_choice);

		sprintf (
		    file_str,
		    "*%s",
		    CWORD_SUFFIX_GRID_FILE);

		status = cword_stats_grid_files_info (file_str);

		break;

	    case (3):
		/*
		 *  List crossword files suitable for processing.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (List crossword files suitable for processing) selected\n",
		    cword_charui_l_sub_menu_choice);

		sprintf (
		    file_str,
		    "*%s",
		    CWORD_SUFFIX_XWORD_FILE);

		/*
		 *  ?? Why not a file_str rather than a crossword_record ??
		 */

		status = cword_stats_crosswrd_files_info (file_str);

		break;

	    case (4):
		/*
		 *  Report on crossword definition.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Report on crossword definition) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
			    CWORD_GETSTATSXRDMSG,
			    SCREEN0,
			    CWORD_SUFFIX_XWORD_FILE);

		if (cword_charui_get_file_spec (
			    TRUE,
			    CWORD_SUFFIX_XWORD_FILE,
			    sizeof (file_str),
			    file_str) == CWORD_S_NORMAL)
		{
		    crossword_record = cword_memman_make_crossword_rec ();

		    status = cword_fileio_read_crossword (
				file_str,
				crossword_record);

		    if (CWORD_S_NORMAL != status)
		    {
			cword_memman_free_cross_rec (crossword_record);

			break;
		    }	    

		    /*
		     *  ?? Why not a file_str rather than a crossword_record ??
		     */

		    status = cword_stats_report_on_crossword (crossword_record);

		    cword_memman_free_cross_rec (crossword_record);
		}

		break;

	    case (5):
		/*
		 *  Output crossword to text file.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Output crossword definition to text file) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_OUTFILEMSG,
		    SCREEN0,
		    CWORD_SUFFIX_XWORD_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_XWORD_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_print_crossword (
				TEXT,
				file_str);
		}

		break;

	    case (6):
		/*
		 *  Output crossword to postscript file.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Output crossword definition to postscript file) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_OUTFILEMSG,
		    SCREEN0,
		    CWORD_SUFFIX_XWORD_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_XWORD_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_print_crossword (
				POSTSCRIPT,
				file_str);
		}

		break;

	    case (0):
		WRITE_TRACE ("## MENU: option 0 (Back to main menu!) selected\n");

		break;

	    default:

		break;
	    }

	    if (choice != 0)
		cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
	}
    }
    while (0 != choice);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_charui__display_3_menu ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is given over to functions concerned with the completion
 *	and printing of filled crossword definitions.  No other activities will
 *	take place in this function.
 *
 *  FORMAL PARAMETERS:
 *
 *      None
 *
 *  IMPLICIT INPUTS:
 *
 *      Not yet
 *
 *  IMPLICIT OUTPUTS:
 *
 *      Not yet
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *--
 */

{

    CROSSWORD_RECORD *crossword_record;

    CWORD_T_STR_LL *str_ll = (CWORD_T_STR_LL *) 0;

    char file_str[CWORD_SIZEOF_FILESPEC + 1],
	 clue_str[CWORD_SIZEOF_CLUE + 1];

    int	total_choice = 0,
	choice = 0,
	status;

    do
    {
	CWORD_OS_CLEAR_SCREEN ();

	cword_charui__masthead_print ();

	SUB_MENU_HEADER("Complete crossword");

	choice = 0;

	SUB_MENU_CHOICE("Complete crossword definition");
	SUB_MENU_CHOICE("List crossword files suitable for completion");

	cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);

	SUB_MENU_CHOICE("Write crossword to text file");
	SUB_MENU_CHOICE("Write crossword to postscript file");

#ifdef DB
	/*
	 *  Database code used only if /define=DB or -DDB is specified in
	 *  compile.
	 */

	cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);

	SUB_MENU_CHOICE("Read crossword from database");
	SUB_MENU_CHOICE("Write crossword to database");
	SUB_MENU_CHOICE("Delete crossword from database");
	SUB_MENU_CHOICE("List crosswords in database");

	cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);

	SUB_MENU_CHOICE("Add clue to database");
	SUB_MENU_CHOICE("Get clue from database");
	SUB_MENU_CHOICE("Delete clue from database");
#endif

	EXIT_SUB_MENU_CHOICE("Back to main menu!");

	total_choice = choice;

	status = cword_charui_get_choice (
		    CWORD_INPUTCHOICE,
		    total_choice,
		    &choice);

	cword_charui_l_sub_menu_choice = choice;

	if (CWORD_CHOICENOTVALID != status)
	{
	    switch (choice)
	    {
	    case (1):
		/*
		 *  Complete crossword defintion with clues.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Complete crossword definition) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_XWORDCLUEGEN,
		    SCREEN0,
		    CWORD_SUFFIX_XWORD_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_XWORD_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    crossword_record = cword_memman_make_crossword_rec ();

		    status = cword_fileio_read_crossword (
				file_str,
				crossword_record);

		    if (CWORD_S_NORMAL != status)
		    {
			cword_memman_free_cross_rec (crossword_record);

			break;
		    }

		    status = cword_clue_fill_desc (crossword_record);

		    if (CWORD_S_NORMAL != status)
		    {
			cword_memman_free_cross_rec (crossword_record);

			break;
		    }

		    cword_fileio_write_crossword (
				file_str,
				crossword_record);

		    cword_memman_free_cross_rec (crossword_record);
		}

		break;

	    case (2):
		/*
		 *  List crossword files suitable for printing.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (List crossword files suitable for printing) selected\n",
		    cword_charui_l_sub_menu_choice);

		sprintf (
		    file_str,
		    "*%s",
		    CWORD_SUFFIX_XWORD_FILE);

		/*
		 *  ?? Why not a file_str rather than a crossword_record ??
		 */

		status = cword_stats_crosswrd_files_info (file_str);

		break;

	    case (3):
		/*
		 *  Output crossword to text file.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Output crossword definition to text file) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_OUTFILEMSG,
		    SCREEN0,
		    CWORD_SUFFIX_XWORD_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_XWORD_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_print_crossword (
				TEXT,
				file_str);
		}

		break;

	    case (4):
		/*
		 *  Output crossword to postscript file.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Output crossword definition to postscript file) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_OUTFILEMSG,
		    SCREEN0,
		    CWORD_SUFFIX_XWORD_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_XWORD_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_print_crossword (
				POSTSCRIPT,
				file_str);
		}

		break;

#ifdef DB
	    case (5):
		/*
		 *  Read crossword definition from the local Cword database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Read crossword from database) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_READFROMDBMSG,
		    SCREEN0,
		    CROSSWORD_STR);

		if (cword_charui_get_str (
			ALL,
			BLANK_NOTOK,
			sizeof (file_str) - 1,
			file_str) == CWORD_S_NORMAL)
		{
		    crossword_record = cword_memman_make_crossword_rec ();

		    status = cword_rdb_get_xrd (
				file_str,
				crossword_record);

		    if (CWORD_S_NORMAL == status)
		    {
			cword_messages_output_message (
			    CWORD_RDBREADSUCC,
			    (TRACE0 | SCREEN0),
			    CROSSWORD_STR,
			    file_str);
		    }
		    else
		    {
			cword_messages_output_message (
			    CWORD_RDBREADFAIL,
			    (TRACE0 | SCREEN0),
			    CROSSWORD_STR,
			    file_str);

			cword_memman_free_cross_rec (crossword_record);

			break;
		    }

		    cword_messages_output_message (
			CWORD_S_DBTOFILEMSG,
			SCREEN0,
			CROSSWORD_STR,
			CWORD_SUFFIX_XWORD_FILE);

		    if (cword_charui_get_file_spec (
			    TRUE,
			    CWORD_SUFFIX_XWORD_FILE,
			    sizeof (file_str),
			    file_str) == CWORD_S_NORMAL)
		    {
			status = cword_fileio_write_crossword (
				    file_str,
				    crossword_record);

			if (CWORD_S_NORMAL != status)
			{
			    cword_memman_free_cross_rec (crossword_record);

			    break;
			}
		    }

		    cword_memman_free_cross_rec (crossword_record);
		}

		break;

	    case (6):
		/*
		 *  Write crossword definition to the local Cword database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Write crossword to database) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_WRITETODB,
		    SCREEN0,
		    CROSSWORD_STR,
		    CWORD_SUFFIX_XWORD_FILE);

		if (cword_charui_get_file_spec (
			TRUE,
			CWORD_SUFFIX_XWORD_FILE,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    crossword_record = cword_memman_make_crossword_rec ();

		    status = cword_fileio_read_crossword (
				file_str,
				crossword_record);

		    if (CWORD_S_NORMAL != status)
		    {
			cword_memman_free_cross_rec (crossword_record);

			break;
		    }
		    
		    status = cword_rdb_add_xrd (crossword_record);

		    if (CWORD_S_NORMAL == status)
		    {
			cword_messages_output_message (
			    CWORD_RDBADDSUCC,
			    (TRACE0 | SCREEN0),
			    CROSSWORD_STR,
			    crossword_record->name);
		    }	    
		    else
		    {
			cword_messages_output_message (
			    CWORD_RDBADDFAIL,
			    (TRACE0 | SCREEN0),
			    CROSSWORD_STR,
			    crossword_record->name);
		    }

		    cword_memman_free_cross_rec (crossword_record);
		}

		break;

	    case (7):
		/*
		 *  Delete crossword definition from the local Cword database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Delete crossword from database) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_DELFROMDB,
		    SCREEN0,
		    CROSSWORD_STR);

		if (cword_charui_get_str (
			ALL,
			BLANK_NOTOK,
			sizeof (file_str) - 1,
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_rdb_delete_xrd (file_str);

		    if (CWORD_S_NORMAL == status)
		    {
			cword_messages_output_message (
			    CWORD_RDBDELSUCC,
			    (TRACE0 | SCREEN0),
			    CROSSWORD_STR,
			    file_str);
		    }
		    else
		    {
			cword_messages_output_message (
			    CWORD_S_RDBDELFAIL,
			    (TRACE0 | SCREEN0),
			    CROSSWORD_STR,
			    file_str);
		    }
		}

		break;

	    case (8):
		/*
		 *  List crossword definitions in local Cword database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Delete crossword from database) selected\n",
		    cword_charui_l_sub_menu_choice);

		status = cword_rdb_get_xrd_ll (&str_ll);

		if (status == CWORD_S_NORMAL)
		{
		    if (str_ll != (CWORD_T_STR_LL *) 0)
		    {
			cword_messages_output_message (
			    CWORD_S_DEFNSINDB,
			    SCREEN0,
			    CROSSWORDS_STR);

			cword_strll_output_str_ll (
			    str_ll,
			    1);
		    }
		    else
		    {
			cword_messages_output_message (
			    CWORD_S_NODEFNSINDB,
			    SCREEN0,
			    CROSSWORDS_STR);
		    }
		}

		cword_strll_free_str_ll (&str_ll);

		break;

	    case (9):
		/*
		 *  Add a single clue to the database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Add clue to DB) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_RDBPUTCLUEDB,
		    SCREEN0);

		cword_messages_output_message (
		    CWORD_S_RDBPUTLIGHTSTR,
		    SCREEN0);

		if (cword_charui_get_str (
			CHARACTER, 
			BLANK_NOTOK,
			sizeof (file_str) - 1,
			file_str) == CWORD_S_NORMAL)
		{
		    cword_messages_output_message (
			CWORD_S_RDBPUTANSSTR,
			SCREEN0);

		    if (cword_charui_get_str (
			    ALL,
			    BLANK_NOTOK,
			    sizeof (clue_str) - 1,
			    clue_str) == CWORD_S_NORMAL)
		    {
			if (clue_str[0] != (char) 0)
			{
			    status = cword_rdb_add_clue (
					file_str,
					clue_str);

			    if (CWORD_S_NORMAL == status)
			    {
				printf ("## Clue/answer pair added successfully\n");
			    }
			}
			else
			{
			    printf ("## No clue inputted\n");
			}
		    }
		}

		break;

	    case (10):
		/*
		 *  Get a single clue from the database using the 
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Get clue from DB) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_RDBGETCLUEDB,
		    SCREEN0);

		cword_messages_output_message (
		    CWORD_S_RDBGETLIGHTSTR,
		    SCREEN0);

		/*
		 *  Get light str and then check the database for a clue
		 *  matching the light entered.
		 */

		if (cword_charui_get_str (
			CHARACTER, 
			BLANK_NOTOK,
			sizeof (file_str) - 1,
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_rdb_get_clue (
				file_str,
				clue_str);

		    if (CWORD_S_NORMAL == status)
		    {
			printf (
			    "## Clue found in db for \"%s\" is \"%s\"\n",
			    file_str,
			    clue_str);
		    }
		    else if (CWORD_S_FAILURE == status)
		    {
			cword_messages_output_message (
			    CWORD_S_NOCLUEFORLIGHT,
			    SCREEN0,
			    file_str);
		    }
		    else
		    {
			/*
			 *  Some other sort of error that should already be out
			 *  in the open.
			 */
		    }
		}

		break;

	    case (11):
		/*
		 *  Delete a single clue from the database.
		 */

		WRITE_TRACE (
		    "## MENU: option %u (Delete clue from DB) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_RDBDELCLUEDB,
		    SCREEN0);

		cword_messages_output_message (
		    CWORD_S_RDBGETLIGHTSTR,
		    SCREEN0);

		/*
		 *  Get light str and then check the database for a clue
		 *  matching the light entered.
		 */

		if (cword_charui_get_str (
			CHARACTER, 
			BLANK_NOTOK,
			sizeof (file_str) - 1,
			file_str) == CWORD_S_NORMAL)
		{
		    status = cword_rdb_delete_clue (file_str);

		    if (CWORD_S_NORMAL == status)
		    {
			printf ("## Clue successfully deleted\n");
		    }
		    else if (CWORD_S_FAILURE == status)
		    {
			printf (
			    "## Light %s not found in database\n",
			    file_str);
		    }
		    else
		    {
			/*
			 *  Some other sort of error that should already be out
			 *  in the open.
			 */
		    }
		}

		break;
#endif

	    case (0):
		WRITE_TRACE ("## MENU: option (Back to main menu!) selected\n");

		break;

	    default:

		break;
	    }

	    if (choice != 0)
		cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
	}
    }
    while (0 != choice);

#if 0
    cword_messages_output_message (
	CWORD_DBNOTPORTABLE,
	(TRACE0 | SCREEN0));
#endif

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_charui__display_4_menu ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is given over to functions concerned with word utilities
 *	and other useful utilites than may come in useful while writing or
 *	solving crosswords.  No other activities will take place in this
 *	function.
 *
 *  FORMAL PARAMETERS:
 *
 *      None
 *
 *  IMPLICIT INPUTS:
 *
 *      Not yet
 *
 *  IMPLICIT OUTPUTS:
 *
 *      Not yet
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion.
 *--
 */

{

    char temp_str[CWORD_SIZEOF_WORD + 1],
	 word_str[CWORD_SIZEOF_WORD + 1],
	 word1_str[CWORD_SIZEOF_WORD + 1],
         temp_char;

    int total_choice = 0,
	choice = 0,
        status;

    unsigned int min;

    do
    {
	CWORD_OS_CLEAR_SCREEN ();

	cword_charui__masthead_print ();

	SUB_MENU_HEADER("Word functions");

	choice = 0;

	SUB_MENU_CHOICE("Spell check");
	SUB_MENU_CHOICE("Template match");
	SUB_MENU_CHOICE("Intersect template match");
	SUB_MENU_CHOICE("Wildcard match");

	SUB_MENU_CHOICE("Sub-anagram generator");
	SUB_MENU_CHOICE("Total anagram generator");
	SUB_MENU_CHOICE("Super-anagram generator");

	SUB_MENU_CHOICE("ROT 13 decoder");
	SUB_MENU_CHOICE("Soundex generator");
	SUB_MENU_CHOICE("Webster definition");
	EXIT_SUB_MENU_CHOICE("Back to main menu!");

	total_choice = choice;

	status = cword_charui_get_choice (
		    CWORD_INPUTCHOICE,
		    total_choice,
		    &choice);

	cword_charui_l_sub_menu_choice = choice;

	if (CWORD_CHOICENOTVALID != status)
	{
	    switch (choice)
	    {
	    case (1):
		WRITE_TRACE (
		    "## MENU: option %u (Spell check) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_SPELLCHECKSTR,
		    SCREEN0);

		if (cword_charui_get_str (
			CHARACTER, 
			BLANK_NOTOK,
			sizeof (word_str) - 1,
			word_str) == CWORD_S_NORMAL)
		{
		    cword_cl_exe_spell (word_str);
		}

		break;

	    case (2):
		WRITE_TRACE (
		    "## MENU: option %u (Template match) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_TEMPLATESTR,
		    SCREEN0);

		if (cword_charui_get_str (
			(CHARACTER | SPACE | QUESTION | PERCENT | DOT), 
			BLANK_NOTOK,
			sizeof (word_str) - 1,
			word_str) == CWORD_S_NORMAL)
		{
		    cword_cl_exe_template (word_str);
		}

		break;

	    case (3):
		WRITE_TRACE (
		    "## MENU: option %u (Intersect template match) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_INTERSECTEXPLAIN,
		    SCREEN0);

		cword_messages_output_message (
		    CWORD_INTERSECTSTR1,
		    SCREEN0);

		if (cword_charui_get_str (
			(CHARACTER | SPACE | QUESTION | PERCENT | DOT), 
			BLANK_NOTOK,
			sizeof (word_str) - 1,
			word_str) == CWORD_S_NORMAL)
		{
		    cword_messages_output_message (
			CWORD_INTERSECTSTR2,
			SCREEN0);

		    if (cword_charui_get_str (
			    (CHARACTER | SPACE | QUESTION | PERCENT | DOT), 
			    BLANK_NOTOK,
			    sizeof (word1_str) - 1,
			    word1_str) == CWORD_S_NORMAL)
		    {
			cword_cl_exe_intersect (
			    word_str,
			    word1_str);
		    }
		}

		break;

	    case (4):
		WRITE_TRACE (
		    "## MENU: option %u (Wildcard match) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_INPUTSTRQ,
		    SCREEN0);

		if (cword_charui_get_str (
			(CHARACTER | WILDCARD | QUESTION | SPACE),
			BLANK_NOTOK,
			sizeof (word_str) - 1,
			word_str) == CWORD_S_NORMAL)
		{
		    cword_cl_exe_wordwild (word_str);
		}

		break;

	    case (5):
		WRITE_TRACE (
		    "## MENU: option %u (Sub-anagram generator) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_ANAGRAMMINSTR,
		    SCREEN0);

		if (cword_charui_get_str (
			DIGIT,
			BLANK_NOTOK,
			sizeof (temp_str) - 1,
			temp_str) == CWORD_S_NORMAL)
		{
		    min = atoi (temp_str);

		    if (min > CWORD_SIZEOF_WORD)
		    {
			cword_messages_output_message (
			    CWORD_LENGREATERMAX,
			    (TRACE0 | SCREEN0),
			    CWORD_SIZEOF_WORD);

			break;
		    }

		    cword_messages_output_message (
			CWORD_ANAGRAMGETSTR,
			SCREEN0,
			word_str);

		    if (cword_charui_get_str (
			    CHARACTER,
			    BLANK_NOTOK,
			    sizeof (word_str) - 1,
			    word_str) == CWORD_S_NORMAL)
		    {
			/*
			 *  ?? Some extra checking before calling
			 *  cword_utils_exe_subanagram - maybe move into that
			 *  function altogether ??
			 */

			if (strlen (word_str) < min)
			{
			    cword_messages_output_message (
				CWORD_MINSIZETOOBIG,
				(TRACE0 | SCREEN0),
				min,
				word_str);

			    break;
			}

			cword_messages_output_message (
			    CWORD_ANAGRAMALWAYSSTR,
			    SCREEN0,
			    word_str);

			temp_char = (char) cword_charui_get_ch (FALSE);

			if (min == strlen (word_str))
			{
			    cword_messages_output_message (
				CWORD_ANAGRAMALL,
				SCREEN0,
				word_str);
			}

			cword_cl_exe_subanagram (
    			    min,
			    CWORD_SIZEOF_WORD,
    			    temp_char,
    			    word_str);
		    }
		}

		break;

	    case (6):
		WRITE_TRACE (
		    "## MENU: option %u (Total anagram generator) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_NOTYETIMPLEMENT,
		    SCREEN0);

		break;

	    case (7):
		WRITE_TRACE (
		    "## MENU: option %u (Super-anagram generator) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_SUPERANAGRAMSTR,
		    SCREEN0);

		if (cword_charui_get_str (
			CHARACTER, 
			BLANK_NOTOK,
			sizeof (word_str) - 1,
			word_str) == CWORD_S_NORMAL)
		{
		    cword_cl_exe_superanagram (
			sizeof (word_str),
			CWORD_SIZEOF_WORD,
			(char) 0,
			word_str);
		}

		break;

	    case (8):
		WRITE_TRACE (
		    "## MENU: option %u (Rot 13!) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_ROTSTR,
		    SCREEN0);

		if (cword_charui_get_str (
			ALL,
			BLANK_NOTOK,
			sizeof (word_str) - 1,
			word_str) == CWORD_S_NORMAL)
		{
		    cword_word_rot_string (
			13,
			(unsigned char *) word_str);
		}

		break;

	    case (9):
		WRITE_TRACE (
		    "## MENU: option %u (Soundex generator) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_SOUNDEXSTR,
		    SCREEN0);

		if (cword_charui_get_str (
			CHARACTER,
			BLANK_NOTOK,
			sizeof (word_str) - 1,
			word_str) == CWORD_S_NORMAL)
		{
		    cword_cl_exe_soundex (word_str);
		}

		break;

	    case (10):
		WRITE_TRACE (
		    "## MENU: option %u (Webster definition) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_WEBSTERSTR,
		    SCREEN0);

		/*
		 *  ?? Should not accept input string if the websters server
		 *  functionality is not available, or not defined ??
		 *
		 *  ?? This should be checked way back during the defaults file
		 *  processing ??
		 */

		if (cword_charui_get_str (
			CHARACTER,
			BLANK_NOTOK,
			sizeof (word_str) - 1,
			word_str) == CWORD_S_NORMAL)
		{
		    cword_webster_get_defn (word_str);
		}

		break;

	    case (0):
		WRITE_TRACE ("## MENU: option %u (Back to main menu!) selected\n");

		break;

	    default:

		break;
	    }

	    if (choice != 0)
		cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
	}
    }
    while (0 != choice);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_charui__display_5_menu ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is given over to functions concerned with misc. utilities
 *	that maybe be useful to run during the runtime life of cword.  No other
 *	activities will take place in this function.
 *
 *  FORMAL PARAMETERS:
 *
 *	None.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    char file_str[CWORD_SIZEOF_FILESPEC + 1],
	 word_str[CWORD_SIZEOF_WORD + 1];

    int total_choice = 0,
	choice = 0,
	status;

    do
    {
	CWORD_OS_CLEAR_SCREEN ();

	cword_charui__masthead_print ();

	SUB_MENU_HEADER("Misc. other utilities");

	choice = 0;
	SUB_MENU_CHOICE("Alter defaults");
	SUB_MENU_CHOICE("Break up input dictionary");
	SUB_MENU_CHOICE("Add word to word file");
	SUB_MENU_CHOICE("Estimate size of word files");

#ifdef DB
	/*
	 *  Database code used only if /define=DB or -DDB is specified in
	 *  compile.
	 */

	SUB_MENU_CHOICE("Database statistics");
#endif

	EXIT_SUB_MENU_CHOICE("Back to main menu!");

	total_choice = choice;

	status = cword_charui_get_choice (
		    CWORD_INPUTCHOICE,
		    total_choice,
		    &choice);

	cword_charui_l_sub_menu_choice = choice;

	if (CWORD_CHOICENOTVALID != status)
	{
	    switch (choice)
	    {
	    case (1):
		WRITE_TRACE (
		    "## MENU: option %u (Alter defaults) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_charui__alter_defaults ();

		break;

	    case (2):
		WRITE_TRACE (
		    "## MENU: option %u (Break up input dictionary) selected\n",
		    cword_charui_l_sub_menu_choice);

		cword_messages_output_message (
		    CWORD_S_BREAKUPDIC,
		    SCREEN0);

		if (cword_charui_get_file_spec (
			TRUE,
			(char *) 0,
			sizeof (file_str),
			file_str) == CWORD_S_NORMAL)
		{
		    cword_messages_output_message (
			CWORD_S_BREAKUPDICSPEC,
			TRACE0,
			file_str);

		    status = breakdic_process_input_dict (file_str);

		    if (status == TRUE)
		    {
			WRITE_TRACE (
			    "File %s successfully broken up using breakdic\n",
			    file_str);
		    }
		    else
		    {
			WRITE_TRACE (
			    "Error breaking up %s using breakdic\n",
			    file_str);
		    }
		}

		break;

	    case (3):
		WRITE_TRACE (
		    "## MENU: option %u (Add word to word files) selected\n",
		    cword_charui_l_sub_menu_choice);

		printf ("## Please input word to add to word file : ");

		if (cword_charui_get_str (
			    CHARACTER,
			    BLANK_NOTOK,
			    sizeof (word_str) - 1,
			    word_str) == CWORD_S_NORMAL)
		{
		    status = cword_match_add_word_to_file (word_str);
		}

		break;

	    case (4):
		WRITE_TRACE (
		    "## MENU: option %u (Estimate size of word files) selected\n",
		    cword_charui_l_sub_menu_choice);

		status = cword_stats_get_word_file_stats ();

		break;

#ifdef DB
	    case (5):
		WRITE_TRACE (
		    "## MENU: option %u (Database statistics) selected\n",
		    cword_charui_l_sub_menu_choice);

		status = cword_rdb_output_stats ();

		break;
#endif

	    case (0):
		WRITE_TRACE ("## MENU: option (Back to main menu!) selected\n");

		break;

	    default:

		break;
	    }

	    if (choice != 0)
		cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
	}
    }
    while (0 != choice);


    return (CWORD_S_NORMAL);

}



/*
 *  ?? Write altered defaults to file ??
 */

PRIVATE int cword_charui__alter_defaults ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function displays the screen of defaults and collects input from
 *	the user determining which default is to be altered.
 *
 *  FORMAL PARAMETERS:
 *
 *      None!
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion.
 *--
 */

{

    int choice,
	status;

    do
    {
	/*
	 *  Display all modifible defaults to UI and then allow user select one
	 *  to modify.  Once selected the option is displayed the user is
	 *  invited to input a new value for that default.  There are three
	 *  types of defaults possible and Cword should be able to handle each
	 *  of them equally well:
	 *
	 *  1. TRUE/FALSE - has to be one or the other.
	 *  2. Numeric - (X <= value <= Y).
	 *  3. Character string - could be checked - or not - as the case may be.
	 */

	cword_charui__display_defaults (CWORD_S_MODYDEFAULTMSG);

	/*
	 *  ?? Change the second parameter of this function as the number of
	 *  options go up or down ??
	 */

	status = cword_charui_get_choice (
		    CWORD_S_INPUTDEFCHOICE,
		    28,
		    &choice);

	if ( (CWORD_CHOICENOTVALID != status) && (choice != 0) )
	{
	    if (choice != -1)
	    {
		/*
		 *  Only get in here if valid choice has been found!  When
		 *  finished in cword_charui_modify_default "pause" screen :-).
		 */

		/*
		 *  This extra check is to make sure that we are not entering
		 *  in here if choice is -1 i.e. something other than an option
		 *  was chosen.
		 */

		cword_charui_modify_default (choice);
	    }

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
	}
    }
    while (choice != 0);

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Can I make the output of the defaults screen any more user friendly.
 *  It's really ugly at the moment ??
 *
 *  ?? Can I help it if the defaults being output are over 80 columns wide ??
 *
 *  ?? Proper messages in this function ??
 *
 *  ?? Instead of the TRUE/FALSE couplets, I could have: YES/NO or ON/OFF ??
 */

PRIVATE int cword_charui__display_defaults (
	int			message_id)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Simple function to display current defaults!
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully display current defaults.
 *--
 */

{

    CWORD_T_STR_LL *temp_ll;

    char temp_str[CWORD_SIZEOF_MESSAGE + 1];

    long temp_val,
	 num = 1;

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	message_id,
	SCREEN0);

    /*
     *  Grid creation (5 options).
     */

    /*
     *  ?? Cannot be absolutely sure of the evaluation order when 
     *  using num in the printf statements so they have to be 
     *  separate ??
     */

    printf (
	"%2u. %s - %u ",
	num++,
	"SMALLEST_ALLOWABLE_LIGHT",
	cword_g_defaults_record.smallest_allowable_light);


    printf (
	"%2u. %s - %u\n",
	num++,
	"SYMMETRY_TYPE",
	cword_g_defaults_record.symmetry_type);

    printf (
	"%2u. %s - %u ",
	num++,
	"X_DIMENSIONS",
	cword_g_defaults_record.x_dimensions);

    printf (
	"%2u. %s - %u ",
	num++,
	"Y_DIMENSIONS",
	cword_g_defaults_record.y_dimensions);

    printf (
	"%2u. %s - %s\n",
	num++,
	"TEMP_GRID_NAME",
	cword_g_defaults_record.temp_grid_name);

    /*
     *  Grid verification (2 options).
     */

    printf (
	"%2u. %s - %s ",
	num++,
	"UNCHES_CHECK",
	cword_g_defaults_record.unches_check == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

    printf (
	"%2u. %s - %s\n",
	num++,
	"SYMMETRY_CHECK",
	cword_g_defaults_record.symmetry_check == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

    /*
     *  Word files (1 option).
     */

    temp_ll = cword_g_defaults_record.word_file_prefix_list;
    temp_str[0] = (char) 0;

    if (temp_ll != (CWORD_T_STR_LL *) 0)
    {
        while (temp_ll != (CWORD_T_STR_LL *) 0)
        {
	    if (temp_str[0] == (char) 0)
                strcpy (temp_str, temp_ll->str);
            else
            {
                strcat (temp_str, ", ");
                strcat (temp_str, temp_ll->str);
            }

            temp_ll = temp_ll->next;
        }
    }
    else
    {
        strcpy (temp_str, "no files found"); /* ?? #define ?? */
    }

    printf ("\n");

    printf (
	"%2u. %s - %s\n",
	num++,
	"WORD_FILE_PREFIX",
	temp_str);

    /*
     *  Fill robot (9 options).
     */

    printf ("\n");

    printf (
	"%2u. %s - %u ",
	num++,
	"LIGHT_GENERATION_ALGORITHM",
	cword_g_defaults_record.light_generation_algorithm);

    printf (
	"%2u. %s - %s\n",
	num++,
	"AUTOMATIC_LIGHT_GENERATION",
	cword_g_defaults_record.automatic_light_generation == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

    printf (
	"%2u. %s - %u ",
	num++,
	"LETTER_WEIGHT_ALGORITHM",
	cword_g_defaults_record.letter_weight_algorithm);

    printf (
	"%2u. %s - %u\n",
	num++,
	"NEXTLIGHT_SELECT_ALGORITHM",
	cword_g_defaults_record.nextlight_select_algorithm);

    printf (
	"%2u. %s - %u ",
	num++,
	"FILL_FROM_WHERE",
	cword_g_defaults_record.fill_from_where);

    printf (
	"%2u. %s - %u \n",
	num++,
	"RESET_AFTER_NUM_ITERATIONS",
	cword_g_defaults_record.reset_after_num_iterations);

    printf (
	"%2u. %s - %s ",
	num++,
	"TEMP_CROSSWORD_NAME",
	cword_g_defaults_record.temp_crossword_name);

    printf (
	"%2u. %s - %u\n",
	num++,
	"MAX_LIGHT_CHOICES",
	cword_g_defaults_record.max_light_choices);

    printf (
	"%2u. %s - %u\n",
	num++,
	"MAX_LIGHT_ITERATIONS",
	cword_g_defaults_record.max_light_iterations);

    /*
     *  Webster server (2 options).
     */

    printf ("\n");

    printf (
	"%2u. %s - %s ",
	num++,
	"WEBSTER_SERVER_NAME",
	cword_g_defaults_record.webster_server_name);

    printf (
	"%2u. %s - %u\n",
	num++,
	"WEBSTER_SERVER_PORT",
	cword_g_defaults_record.webster_server_port);

    /*
     *  Log file usage defaults (3 options).
     */

    if (cword_g_defaults_record.trace_level == LEVEL3)
	temp_val = 3;
    else if (cword_g_defaults_record.trace_level == LEVEL2)
	temp_val = 2;
    else if (cword_g_defaults_record.trace_level == LEVEL1)
	temp_val = 1;
    else
	temp_val = 0;

    printf (
	"%2u. %s - %s ",
	num++,
	"WRITE_LOG_FILE",
	cword_g_defaults_record.write_log_file == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

    printf (
	"%2u. %s - %s ",
	num++,
	"LOG_FILE_PREFIX",
	cword_g_defaults_record.log_file_prefix);

    printf (
	"%2u. %s - %u\n",
	num++,
	"TRACE_LEVEL",
	temp_val);

    /*
     *  Misc (6 optons).
     */

    printf ("\n");

    printf (
	"%2u. %s - %u ",
	num++,
	"SOUNDEX_ALGORITHM",
	cword_g_defaults_record.soundex_algorithm);

    printf (
	"%2u. %s - %u ",
	num++,
	"WORDS_PER_DOT",
	cword_g_defaults_record.words_per_dot);

    printf (
	"%2u. %s - %s\n",
	num++,
	"RING_BELL",
	cword_g_defaults_record.ring_bell == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

    printf (
	"%2u. %s - %s ",
	num++,
	"CHECK_SPELLING",
	cword_g_defaults_record.check_spelling == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

    printf (
	"%2u. %s - \"%s\" ",
	num++,
	"USERNAME",
	cword_g_defaults_record.username);

    printf (
	"%2u. %s - %u\n",
	num++,
	"MAX_MATCHES",
	cword_g_defaults_record.max_matches);

    printf ("\n");

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Include a one line description of each of the defaults when it has 
 *  been selected for alteration ??
 *
 *  ?? Option to create a new options file altogether from the altered 
 *  defaults ??
 *
 *  ?? Re-generate word_file_prefix statistics if the word_file_prefixes 
 *  list has been added/deleted to/from ??
 */

PRIVATE int cword_charui_modify_default (
	int			default_option)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to modify Cword default given option via default_option.  It
 *	will prompt the user with necessary information to do the actual
 *	modifications.
 *
 *  FORMAL PARAMETERS:
 *
 *      default_option		(r) Default option to modify
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    char temp_str[CWORD_SIZEOF_MESSAGE + 1],
	 temp_char;

    long temp_val;

    WRITE_TRACE (
	"## Option %u selected\n",
	default_option);

    printf (
	"\n\n## Please refer to %s for more information about user defaults\n\n",
    	cword_g_defaults_record.defaults_file_spec); 

    /*
     *  Ring bell to wake up user - in case she/he is falling asleep ;-).
     */

    cword_charui_ring_bell (1);

    switch (default_option)
    {
    case (1): /* SMALLEST_ALLOWABLE_LIGHT */
	printf (
	    "## Current smallest allowable light is length %u\n",
	    cword_g_defaults_record.smallest_allowable_light);

	printf (
	    "## Input smallest allowable light [%u] : ",
	    cword_g_defaults_record.smallest_allowable_light);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > CWORD_SIZEOF_WORD) || (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    if ( (temp_val >= cword_g_defaults_record.x_dimensions) &&
	          (temp_val >= cword_g_defaults_record.y_dimensions) )
	    {
		/*
		 *  Warning!
		 */

		printf (
		    "## Smallest allowable light is larger than the default value for X or Y\n");
	    }

	    cword_g_defaults_record.smallest_allowable_light = temp_val;

	    printf (
		"## Changing smallest allowable light to %u\n",
		cword_g_defaults_record.smallest_allowable_light);
	}

	break;

    case (2):  /* SYMMETRY_TYPE */
	printf (
	    "## Current default symmetry type is %u\n",
	    cword_g_defaults_record.symmetry_type);

	printf (
	    "## Input new default symmetry type [%u]: ",
	    cword_g_defaults_record.symmetry_type);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > 5) || (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.symmetry_type = temp_val;

	    printf (
		"## Changing default symmetry type to %u\n",
		cword_g_defaults_record.symmetry_type);
	}

	break;

    case (3): /* X_DIMENSIONS */
	printf (
	    "## Current default X dimensions is %u\n",
	    cword_g_defaults_record.x_dimensions);

	printf (
	    "## Input new default X dimensions [%u]: ",
	    cword_g_defaults_record.x_dimensions);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if (temp_val > CWORD_SIZEOF_WORD)
	    {
		printf ("## New default is too large for current set up\n");

		break;
	    }

	    if (temp_val < cword_g_defaults_record.smallest_allowable_light)
	    {
		printf ("## New default is smaller than the smallest allowable light size\n");

		break;
	    }

	    cword_g_defaults_record.x_dimensions = temp_val;

	    printf (
		"## Changing default X dimensions to %u\n",
		cword_g_defaults_record.x_dimensions);
	}

	break;

    case (4): /* Y_DIMENSIONS */
	printf (
	    "## Current default Y dimensions is %u\n",
	    cword_g_defaults_record.y_dimensions);

	printf (
	    "## Input new default Y dimensions [%u]: ",
	    cword_g_defaults_record.y_dimensions);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if (temp_val > CWORD_SIZEOF_WORD)
	    {
		printf ("## New default is too large for current set up\n");

		break;
	    }

	    if (temp_val < cword_g_defaults_record.smallest_allowable_light)
	    {
		printf ("## New default is smaller than the smallest allowable light size\n");

		break;
	    }

	    cword_g_defaults_record.y_dimensions = temp_val;

	    printf (
		"## Changing default Y dimensions to %u\n",
		cword_g_defaults_record.y_dimensions);
	}

	break;

    case (5): /* TEMP_GRID_NAME */
	/*
	 *  ?? Turn into function ??
	 */

	printf (
	    "## Temporary grid name is %s\n",
	    cword_g_defaults_record.temp_grid_name);

	printf (
	    "## Input new temporary grid name [%s] : ",
	    cword_g_defaults_record.temp_grid_name);

	if (cword_charui_get_str (
		VALID_FILE_SPEC,
		BLANK_NOTOK,
		sizeof (temp_str) - 1,
		temp_str) == CWORD_S_NORMAL)
	{
	    printf (
		"## New temporary grid name is \"%s\"\n",
		temp_str);

	    strcpy (cword_g_defaults_record.temp_grid_name, temp_str);
	}
	else
	{
	    printf ("## Unaltered\n");
	}

	break;

    case (6): /* UNCHES CHECK */
	printf (
	    "## Check for unches is %s\n",
	    cword_g_defaults_record.unches_check == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

	printf (
	    "## Change unches check to %s (%c/%c) [%c] : ",
	    cword_g_defaults_record.unches_check == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR,
	    CWORD_Y_CHAR,
	    CWORD_N_CHAR,
	    CWORD_N_CHAR);

	temp_char = (char) cword_charui_get_ch (FALSE);

	if (toupper (temp_char) == CWORD_Y_CHAR)
	{
	    printf (
		"## Changing Unches check to %s\n",
		cword_g_defaults_record.unches_check == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR);

	    if (cword_g_defaults_record.unches_check == TRUE)
		cword_g_defaults_record.unches_check = FALSE;
	    else
		cword_g_defaults_record.unches_check = TRUE;
	}
	else
	{
	    printf ("## Default remains unaltered\n");
	}

	break;

    case (7): /* SYMMETRY_CHECK */
	/*
	 *  ?? Turn into function ??
	 */

	printf (
	    "## Check for symmetry is %s\n",
	    cword_g_defaults_record.symmetry_check == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

	printf (
	    "## Change symmetry check to %s (%c/%c) [%c] : ",
	    cword_g_defaults_record.symmetry_check == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR,
	    CWORD_Y_CHAR,
	    CWORD_N_CHAR,
	    CWORD_N_CHAR);

	temp_char = (char) cword_charui_get_ch (FALSE);

	if (toupper (temp_char) == CWORD_Y_CHAR)
	{
	    printf (
		"## Changing symmetry check to %s\n",
		cword_g_defaults_record.symmetry_check == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR);

	    if (cword_g_defaults_record.symmetry_check == TRUE)
		cword_g_defaults_record.symmetry_check = FALSE;
	    else
		cword_g_defaults_record.symmetry_check = TRUE;
	}
	else
	{
	    printf ("## Default remains unaltered\n");
	}

	break;

    case (8): /* WORD_FILE_PREFIX */
	/*
	 *  Only support "ADD" or "DELETE" a word file prefix!
	 *
	 *  List the available word file prefixes and display a menu allowing
	 *  the user to select the prefix to delete.  Only one action will be
	 *  allowed each time this option is selected.
	 */

	printf (
	    "## Add/Delete word file prefix (%c/%c) : ",
	    ADD_CHAR,
	    DELETE_CHAR);

	temp_char = (char) cword_charui_get_ch (FALSE);

	/*
	 *  Either A or D.  Nothing else is accepted.
	 *
	 *  ?? Function of its own ??
	 */

	if (toupper (temp_char) == ADD_CHAR)
	{
	    printf ("## Please input word file prefix to be added : ");

	    if (cword_charui_get_str (
		    VALID_FILE_SPEC,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	    {
		if (cword_strll_is_str_in_str_ll (
			temp_str,
			cword_g_defaults_record.word_file_prefix_list) == TRUE)
		{
		    printf ("## Think you could fool me by adding a file prefix already in the list\n");

		    break;
		}

		/*
		 *  Check whether the file prefix entered is valid.
		 */

		if (cword_match_verify_file_prefix (temp_str) == FALSE)
		{
		    cword_messages_output_message (
			CWORD_S_NOWORDFILEFND,
			(TRACE0 | SCREEN0),
			temp_str);

		    break;
		}

		printf (
		    "## New word file prefix is \"%s\"\n",
		    temp_str);

		/* 
		 *  ?? Add inputted word file prefix to list.  First check that
		 *  the prefix exists and is not part of the list of word
		 *  prefixes already defined ??
		 */

		if (CWORD_S_NORMAL != cword_strll_add_str (
					SORTED | UNIQUE,
					temp_str,
					&cword_g_defaults_record.word_file_prefix_list))
		{
		    printf (
			"## Error adding %s to word file prefix list\n",
			temp_str);
		}
	    }
	}
	else if (toupper (temp_char) == DELETE_CHAR)
	{
	    if (cword_strll_is_str_ll_empty (cword_g_defaults_record.word_file_prefix_list) == TRUE)
	    {
		printf ("## word_file_prefix list is empty - therefore cannot delete entries :-)\n");

		break;
	    }

	    printf ("## Please input word file to be deleted : ");

	    if (cword_charui_get_str (
		    VALID_FILE_SPEC,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	    {
		/*
		 *  Check whether the file prefix entered is in the existing list.
		 */

		if (cword_strll_is_str_in_str_ll (
			temp_str,
			cword_g_defaults_record.word_file_prefix_list) == FALSE)
		{
		    printf ("## Word file prefix entered is not in word file prefix list\n");

		    break;
		}

		printf (
		    "## Existing word file prefix to be deleted is \"%s\"\n",
		    temp_str);

		/* 
		 *  ?? Delete word file prefix from list but first check that
		 *  it is part of the list of word prefixes already defined ??
		 */

		if (cword_strll_del_str (
			temp_str,
			&cword_g_defaults_record.word_file_prefix_list) != CWORD_S_NORMAL)
		{
		    printf (
			"## Error deleting %s from word file prefix list\n",
			temp_str);
		}
	    }
	}
	else
	{
	    printf ("## Invalid selection made - cannot continue\n");
	}

	break;

    case (9): /* LIGHT_GENERATION_ALGORITHM */
	printf (
	    "## Current light generation algorithm is %u\n",
	    cword_g_defaults_record.light_generation_algorithm);

	printf (
	    "## Input new light generation algorithm [%u]: ",
	    cword_g_defaults_record.light_generation_algorithm);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > 2) || (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.light_generation_algorithm = temp_val;

	    printf (
		"## Changing light generation algorithm to %u\n",
		cword_g_defaults_record.light_generation_algorithm);
	}

	break;

    case (10): /* AUTOMATIC_LIGHT_GENERATION */
	printf (
	    "## Automatic light generation is %s\n",
	    cword_g_defaults_record.automatic_light_generation == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

	printf (
	    "## Change automatic light generation to %s (%c/%c) [%c] : ",
	    cword_g_defaults_record.automatic_light_generation == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR,
	    CWORD_Y_CHAR,
	    CWORD_N_CHAR,
	    CWORD_N_CHAR);

	temp_char = (char) cword_charui_get_ch (FALSE);

	if (toupper (temp_char) == CWORD_Y_CHAR)
	{
	    printf (
		"## Changing automatic light generation to %s\n",
		cword_g_defaults_record.automatic_light_generation == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR);

	    if (cword_g_defaults_record.automatic_light_generation == TRUE)
		cword_g_defaults_record.automatic_light_generation = FALSE;
	    else
		cword_g_defaults_record.automatic_light_generation = TRUE;
	}
	else
	{
	    printf ("## Default remains unaltered\n");
	}

	break;

    case (11): /* LETTER_WEIGHT_ALGORITHM */
	printf (
	    "## Current letter weight algorithm is %u\n",
	    cword_g_defaults_record.letter_weight_algorithm);

	printf (
	    "## Input new letter weight algorithm [%u]: ",
	    cword_g_defaults_record.letter_weight_algorithm);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > 3) || (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.letter_weight_algorithm = temp_val;

	    printf (
		"## Changing letter weight algorithm to %u\n",
		cword_g_defaults_record.letter_weight_algorithm);
	}

	break;

    case (12): /* NEXTLIGHT_SELECT_ALGORITHM */
	printf (
	    "## Current next light selection algorithm is %u\n",
	    cword_g_defaults_record.nextlight_select_algorithm);

	printf (
	    "## Input new next light selection algorithm [%u]: ",
	    cword_g_defaults_record.nextlight_select_algorithm);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > 4) || (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.nextlight_select_algorithm = temp_val;

	    printf (
		"## Changing next light selection algorithm to %u\n",
		cword_g_defaults_record.letter_weight_algorithm);
	}

	break;

    case (13): /* FILL_FROM_WHERE */
	/*
	 *  ?? Can only be changed on OpenVMS ??
	 */

	printf (
	    "## Currently fill from where is %u\n",
	    cword_g_defaults_record.fill_from_where);

	printf (
	    "## Input new file from where [%u]: ",
	    cword_g_defaults_record.fill_from_where);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > 2) || (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.fill_from_where = temp_val;

	    printf (
		"## Changing fill from where to %u\n",
		cword_g_defaults_record.fill_from_where);
	}

	break;

    case (14): /* RESET_AFTER_NUM_ITERATIONS */
	printf (
	    "## Current reset grid after number of iterations is %u\n",
	    cword_g_defaults_record.reset_after_num_iterations);

	printf (
	    "## Input new reset grid after number of iterations [%u]: ",
	    cword_g_defaults_record.reset_after_num_iterations);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    /*
	     *  ?? Better granularity needed here ??
	     */

	    if ( (temp_val > cword_g_defaults_record.max_light_iterations) || 
                  (temp_val < 100) )
	    {
		printf ("## Error getting new reset grid after number of iterations\n");

		break;
	    }

	    cword_g_defaults_record.reset_after_num_iterations = temp_val;

	    printf (
		"## Changing reset grid after number of iterations to %u\n",
		cword_g_defaults_record.reset_after_num_iterations);
	}

	break;

    case (15): /* TEMP_CROSSWORD_NAME */
	printf (
	    "## Temporary crossword name is %s\n",
	    cword_g_defaults_record.temp_crossword_name);

	printf (
	    "## Input new temporary crossword name [%s] : ",
	    cword_g_defaults_record.temp_crossword_name);

	if (cword_charui_get_str (
		VALID_FILE_SPEC,
		BLANK_NOTOK,
		sizeof (temp_str) - 1,
		temp_str) == CWORD_S_NORMAL)
	{
	    printf (
		"## New temporary crossword name is \"%s\"\n",
		temp_str);

	    strcpy (cword_g_defaults_record.temp_crossword_name, temp_str);
	}
	else
	{
	    printf ("## Unaltered\n");
	}

	break;

    case (16): /* MAX_LIGHT_CHOICES */
	printf (
	    "## Current maximum light choices is %u\n",
	    cword_g_defaults_record.max_light_choices);

	printf (
	    "## Input new maximum light choices [%u]: ",
	    cword_g_defaults_record.max_light_choices);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > CWORD_SIZEOF_MAX_ARRAY_SIZE) || 
                  (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.max_light_choices = temp_val;

	    printf (
		"## Changing maximum light choices to %u\n",
		cword_g_defaults_record.max_light_choices);
	}

	break;

    case (17): /* MAX_LIGHT_ITERATIONS */
	printf (
	    "## Current maximum light fill iterations is %u\n",
	    cword_g_defaults_record.max_light_iterations);

	printf (
	    "## Input new maximum light fill iterations [%u]: ",
	    cword_g_defaults_record.max_light_iterations);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if (temp_val < 0)
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    /*
	     *	?? Should check the value of reset_after_num_iterations too if
	     *	this is being changed ??
	     */

	    cword_g_defaults_record.max_light_iterations = temp_val;

	    printf (
		"## Changing maximum light fill iterations counter to %u\n",
		cword_g_defaults_record.max_light_iterations);
	}

	break;

    case (18): /* WEBSTER_SERVER_NAME */
	printf (
	    "## Websters server IP address is %s\n",
	    cword_g_defaults_record.webster_server_name);

	printf (
	    "## Input new webster server IP address [%s] : ",
	    cword_g_defaults_record.webster_server_name);

	if (cword_charui_get_str (
		VALID_FILE_SPEC,
		BLANK_NOTOK,
		sizeof (temp_str) - 1,
		temp_str) == CWORD_S_NORMAL)
	{
	    printf (
		"## New webster server IP number is \"%s\"\n",
		temp_str);

	    /*
	     *	?? Check that the machine exists before allowing the field to
	     *	be modified - gethostbyname ??
	     */

	    strcpy (cword_g_defaults_record.webster_server_name, temp_str);
	}
	else
	{
	    printf ("## Unaltered\n");
	}

	break;

    case (19): /* WEBSTER_SERVER_PORT */
	printf (
	    "## Current Websters server port is %u\n",
	    cword_g_defaults_record.webster_server_port);

	printf (
	    "## Input new Websters sever port [%u]: ",
	    cword_g_defaults_record.webster_server_port);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > 99999) || 
                  (temp_val < 0) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.webster_server_port = temp_val;

	    /*
	     *	?? Check that webster works with this new value before
	     *	continuing ??
	     */

	    printf (
		"## Changing Websters server port to %u\n",
		cword_g_defaults_record.webster_server_port);
	}

	break;

    case (20): /* WRITE_LOG_FILE */
	printf (
	    "## Write log file is %s\n",
	    cword_g_defaults_record.write_log_file == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

	printf (
	    "## Change write log file to %s (%c/%c) [%c] : ",
	    cword_g_defaults_record.write_log_file == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR,
	    CWORD_Y_CHAR,
	    CWORD_N_CHAR,
	    CWORD_N_CHAR);

	temp_char = (char) cword_charui_get_ch (FALSE);

	if (toupper (temp_char) == CWORD_Y_CHAR)
	{
	    printf (
		"## Changing write log file to %s\n",
		cword_g_defaults_record.write_log_file == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR);

	    if (cword_g_defaults_record.write_log_file == TRUE)
		cword_g_defaults_record.write_log_file = FALSE;
	    else
		cword_g_defaults_record.write_log_file = TRUE;
	}
	else
	{
	    printf ("## Default remains unaltered\n");
	}

	break;

    case (21): /* LOG_FILE_PREFIX */
	/*
	 *  ?? No point changing this as it will make no difference during this
	 *  session.  Tell user as much and continue as normal ??
	 */

	printf ("## Changes to the value of LOG_FILE_PREFIX cannot be made dynamically\n");

	break;

#if 0
	printf (
	    "## Log file prefix is %s\n",
	    cword_g_defaults_record.log_file_prefix);

	printf (
	    "## Input new log file prefix [%s] : ",
	    cword_g_defaults_record.log_file_prefix);

	if (cword_charui_get_str (
		VALID_FILE_SPEC,
		BLANK_NOTOK,
		sizeof (temp_str) - 1,
		temp_str) == CWORD_S_NORMAL)
	{
	    printf (
		"## New log file prefix is \"%s\"\n",
		temp_str);

	    strcpy (cword_g_defaults_record.log_file_prefix, temp_str);
	}
	else
	{
	    printf ("## Unaltered\n");
	}

	break;
#endif

    case (22): /* TRACE_LEVEL */
	if (cword_g_defaults_record.trace_level == LEVEL3)
	    temp_val = 3;
	else if (cword_g_defaults_record.trace_level == LEVEL2)
	    temp_val = 2;
	else if (cword_g_defaults_record.trace_level == LEVEL1)
	    temp_val = 1;
	else
	    temp_val = 0;

	printf (
	    "## Current output trace level is %u\n",
	    temp_val);

	printf (
	    "## Input new output trace level [%u]: ",
	    temp_val);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > 3) || 
                  (temp_val < 0) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    if (temp_val == 3)
		cword_g_defaults_record.trace_level = LEVEL3;
	    else if (temp_val == 2)
		cword_g_defaults_record.trace_level = LEVEL2;
	    else if (temp_val == 1)
		cword_g_defaults_record.trace_level = LEVEL1;
	    else
		cword_g_defaults_record.trace_level = LEVEL0;

	    printf (
		"## Changing output trace level to %u\n",
		temp_val);
	}

	break;

    case (23): /* SOUNDEX_ALGORITHM */
	printf (
	    "## Current soundex algorithm is %u\n",
	    cword_g_defaults_record.soundex_algorithm);

	printf (
	    "## Input new soundex algorithm [%u]: ",
	    cword_g_defaults_record.soundex_algorithm);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > 2) || (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.soundex_algorithm = temp_val;

	    printf (
		"## Changing soundex algorithm to %u\n",
		cword_g_defaults_record.soundex_algorithm);
	}

	break;

    case (24): /* WORDS_PER_DOT */
	printf (
	    "## Current words per dot counter is %u\n",
	    cword_g_defaults_record.words_per_dot);

	printf (
	    "## Input new word per dot counter [%u]: ",
	    cword_g_defaults_record.words_per_dot);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > CWORD_SIZEOF_MAX_WORDS_PER_DOT) || 
                  (temp_val < CWORD_SIZEOF_MIN_WORDS_PER_DOT) )
	    {
		printf ("## Error getting new word per dot counter\n");

		break;
	    }

	    cword_g_defaults_record.words_per_dot = temp_val;

	    printf (
		"## Changing word per dot counter to %u\n",
		cword_g_defaults_record.words_per_dot);
	}

	break;

    case (25): /* RING_BELL */
	printf (
	    "## Ring bell on error/warning is %s\n",
	    cword_g_defaults_record.ring_bell == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

	printf (
	    "## Change ring bell on error/warning to %s (%c/%c) [%c] : ",
	    cword_g_defaults_record.ring_bell == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR,
	    CWORD_Y_CHAR,
	    CWORD_N_CHAR,
	    CWORD_N_CHAR);

	temp_char = (char) cword_charui_get_ch (FALSE);

	if (toupper (temp_char) == CWORD_Y_CHAR)
	{
	    printf (
		"## Changing ring bell on error to %s\n",
		cword_g_defaults_record.ring_bell == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR);

	    if (cword_g_defaults_record.ring_bell == TRUE)
		cword_g_defaults_record.ring_bell = FALSE;
	    else
		cword_g_defaults_record.ring_bell = TRUE;
	}
	else
	{
	    printf ("## Default remains unaltered\n");
	}

	break;

    case (26): /* CHECK_SPELLING */
	printf (
	    "## Check spelling of all lights is %s\n",
	    cword_g_defaults_record.check_spelling == TRUE ? CWORD_TRUE_STR : CWORD_FALSE_STR);

	printf (
	    "## Change check spelling of all lights to %s (%c/%c) [%c] : ",
	    cword_g_defaults_record.check_spelling == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR,
	    CWORD_Y_CHAR,
	    CWORD_N_CHAR,
	    CWORD_N_CHAR);

	temp_char = (char) cword_charui_get_ch (FALSE);

	if (toupper (temp_char) == CWORD_Y_CHAR)
	{
	    printf (
		"## Changing check spelling of all lights to %s\n",
		cword_g_defaults_record.check_spelling == TRUE ? CWORD_FALSE_STR : CWORD_TRUE_STR);

	    if (cword_g_defaults_record.check_spelling == TRUE)
		cword_g_defaults_record.check_spelling = FALSE;
	    else
		cword_g_defaults_record.check_spelling = TRUE;
	}
	else
	{
	    printf ("## Default remains unaltered\n");
	}

	break;

    case (27): /* USERNAME */
	printf (
	    "## Username is %s\n",
	    cword_g_defaults_record.username);

	printf (
	    "## Input new username [%s] : ",
	    cword_g_defaults_record.username);

	if (cword_charui_get_str (
		    ALL,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    printf (
		"## New username is \"%s\"\n",
		temp_str);

	    strcpy (cword_g_defaults_record.username, temp_str);
	}
	else
	{
	    printf ("## Unaltered\n");
	}

	break;

    case (28): /* MAX_MATCHES */
	printf (
	    "## Current maximum return matches is %u\n",
	    cword_g_defaults_record.max_matches);

	printf (
	    "## Input new maximum return matches [%u]: ",
	    cword_g_defaults_record.max_matches);

	if (cword_charui_get_str (
		    DIGIT,
		    BLANK_NOTOK,
		    sizeof (temp_str) - 1,
		    temp_str) == CWORD_S_NORMAL)
	{
	    temp_val = atoi (temp_str);

	    if ( (temp_val > CWORD_SIZEOF_MAX_ARRAY_SIZE) || 
                  (temp_val < 1) )
	    {
		printf ("## Error getting allowable value for option\n");

		break;
	    }

	    cword_g_defaults_record.max_matches = temp_val;

	    printf (
		"## Changing maximum matches to %u\n",
		cword_g_defaults_record.max_matches);
	}

	break;

    default:
	printf ("Valid default not chosen - cannot continue\n");

	break;
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? If this function returns anything other than CWORD_S_NORMAL at the main
 *  menu level it will exit cword.  If this is called returning an error in a
 *  lower level menu it will cause Cword to go back up a level.  Handle more
 *  effectively ??
 */

PUBLIC int cword_charui_get_choice (
	int			message_id,
	int			max_choice,
	int			*choice)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function obtains a valid input from the user concerning what
 *	option number is required in the current menu.
 *
 *  FORMAL PARAMETERS:
 *
 *      max_choice		(r) Total number of options in the current menu
 *	*choice			(w) What number option is selected
 *
 *  IMPLICIT INPUTS:
 *
 *      Not yet
 *
 *  IMPLICIT OUTPUTS:
 *
 *      Not yet
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *	CWORD_CHOICENOTVALID	Invalid option number chosen
 *--
 */

{

    char temp_str[CWORD_SIZEOF_MESSAGE + 1];

    int status;

    /*
     *  Mark *choice's card before continuing.
     */

    *choice = -1;

    /*
     *  Output user defined prompt string.
     */

    cword_messages_output_message (
	message_id,
	SCREEN0);

    status = cword_charui_get_str (
		ALL, 
		BLANK_OK,
		sizeof (temp_str) - 1,
		temp_str);

    /*
     *	Take out any leading and trailing spaces from the input string before
     *	continuing!
     */

    cword_word_del_lead_spaces (temp_str);
    cword_word_del_trail_spaces (temp_str);

    /*
     *	No point going any further if there's nothing in temp_str to process.
     */

    if (strlen (temp_str) == 0)
	return (CWORD_CHOICENOTVALID);

    /*
     *  ?? Add VERSION_CHAR to display build time and version number ??
     */

    if ( (toupper (temp_str[0]) == EXIT_CHAR) || 
          (toupper (temp_str[0]) == QUIT_CHAR) )
    {
	WRITE_TRACE ("## MENU: EXIT/QUIT option chosen\n");

	/* 
	 *  EXIT/QUIT application now.  Probably against the laws 
	 *  of least astonishment - but should be OK.
	 */

	cword_messages_output_message (
		CWORD_S_NONUMTRACEMSG,
		TRACE0,
		"EXIT/QUIT option");

	cword_messages_output_message (
		CWORD_FINISHMSG,
		SCREEN0);

	exit (EXIT_SUCCESS);
    }
    else if (toupper (temp_str[0]) == BACK_CHAR)
    {
	/*
	 *  Go back up the menu tree
	 */

	WRITE_TRACE ("## MENU: BACK option chosen\n");

	/*
	 *  User has implicitly select option 0.
	 */

	*choice = 0;

	return (CWORD_S_NORMAL);
    }
    else if ( (temp_str[0] == SPAWN1_CHAR) ||
  	       (temp_str[0] == SPAWN2_CHAR) ||
	        (temp_str[0] == SPAWN3_CHAR) )
    {
	/*
	 *  Goto system and execute OS command.
	 */

#if !defined (_WINDOWS)
	/*
	 *  Execute an operating system specific command when either !, %, or,
	 *  $ entered plus the command to be executed via the system () RTL
	 *  function!
	 */

	char system_command[CWORD_SIZEOF_MESSAGE + 1];

	char count = 1;

	WRITE_TRACE ("## MENU: OS command option chosen\n");

	if (strlen (temp_str) == 1)
	{
	    /*
	     *	Note: Special case where system will create a new sub-shell in
	     *	OpenVMS.  On U*X system will report an error!
	     *
	     *	For the moment it will do nothing!
	     */

	    printf ("# Invalid spawn command entered - please re-try\n");

	    return (CWORD_S_NORMAL);
	}

	while (temp_str[count] != (char) 0)
	{
	    system_command[count - 1] = temp_str[count];
	    count++;
	}
	system_command[count - 1] = (char) 0;

	cword_messages_output_message (
		CWORD_SYSTEMCALLMSG,
		SCREEN0,
		system_command);

	status = system (system_command);

#if 0
    	/*
         *  For the time being I'm going to disable the checking of 
    	 *  the return status to system ().
    	 */

	/*
	 *  ?? Return value of system varies from system to system.  This 
    	 *  may have to be recoded appropriately ??
	 */

	if (status == 0)
	{
	    cword_messages_output_message (
		CWORD_S_SYSTEMCALLERR, 
		(SCREEN0 | TRACE0));
	}
#endif

	cword_messages_output_message (CWORD_SYSTEMCALLRETURN, SCREEN0);

#else

	/*
	 *  If Quickwin: do nothing.
	 */

	cword_messages_output_message (CWORD_S_NOTYETIMPLEMENT, SCREEN0);
#endif

	/*
	 *  No choice made but it is OK since the default in the case statement
	 *  which the calling function is using will allow normal operation
	 *  continue when we return a -1 in *choice.
	 */

	return (CWORD_S_NORMAL);
    }
    else if (toupper (temp_str[0]) == DEFAULT_CHAR)
    {
	WRITE_TRACE ("## MENU: DEFAULT option chosen\n");

	/*
	 *  Display defaults screen.
	 */

	cword_charui__display_defaults (CWORD_S_VIEWDEFAULTMSG);

	return (CWORD_S_NORMAL);
    }
    else if (toupper (temp_str[0]) == CD_CHAR)
    {
	char parse_str[CWORD_SIZEOF_LINE + 1];

	/*
	 *  Assume for the time being that the command is a change directory
	 *  command.
	 *
	 *  Figure out whether it is a valid change directory directive before
	 *  continuing.
	 */

	strcpy (parse_str, temp_str);

	cword_word_upper_str (parse_str);

	if (strncmp (parse_str, CD_STR, 3) == 0)
	{
    	    /*
             *  ?? if cd on its own is entered output current working 
             *  directory ??
             */

	    /*
	     *	Having checked the first three characters of the input string,
	     *	I have determined that a "change directory" directive has
	     *	probably been inputted.
	     *
	     *	Parse the rest of the string, taking out the directory
	     *	specification.  Call chdir using this string and return success
	     *	or failure on the result of this function.
	     */

	    WRITE_TRACE ("## MENU: CD option chosen\n");

	    strcpy (parse_str, &temp_str[3]);

	    cword_word_del_lead_spaces (parse_str);
	    cword_word_del_trail_spaces (parse_str);

	    cword_messages_output_message (CWORD_S_CHDIRMSG, SCREEN0);

    	    /*
             *  ?? Make chdir into VMS/UNIX/DOS specific function ??
             *
             *  _chdir in Visual C++.
             */

	    if (chdir (parse_str) == 0)
	    {
		cword_messages_output_message (
		    CWORD_S_CHDIRSUCCESS, 
		    (SCREEN0 | TRACE0),
		    parse_str);
	    }
	    else
	    {
		cword_messages_output_message (
		    CWORD_S_CHDIRFAILURE, 
		    (SCREEN0 | TRACE0),
		    parse_str);
	    }

	    return (CWORD_S_NORMAL);
	}

	/*
	 *  Continue if we get down to here - it will probably result in an
	 *  error but what the hey!
	 */
    }
    else if ( (toupper (temp_str[0]) == HELP_CHAR) || (toupper (temp_str[0]) == QUESTION_CHAR) )
    {
	WRITE_TRACE ("## MENU: HELP option chosen\n");

	/*
	 *  Display help screen.
	 */

    	cword_charui__output_help_screen (max_choice);

	return (CWORD_S_NORMAL);
    }

    /*
     *  If it's none of the above then it must be a number - I hope!
     */

    if (cword_word_is_digits_only (temp_str) == TRUE)
    {
	/*
	 *  String contains all digits *only* so it's OK to come in here!
	 */

	*choice = atoi (temp_str);    

	if ( (*choice < 0) || (*choice > max_choice) )
	{
	    /*
	     *  This is an invalid choice.
	     */

	    cword_messages_output_message (
		CWORD_S_INVALIDCHOICE,
		SCREEN0);

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    return (CWORD_CHOICENOTVALID);
	}
	else
	{
	    /*
	     *	A valid numberic option (0 to max_choice) was inputted by the
	     *	user.  This value is being returned through *choice.
	     */

	    return (CWORD_S_NORMAL);
	}
    }
    else
    {
	/*
	 *  String contains at least one character or embedded space - this is
	 *  an error in the context!
	 *
	 *  This is invalid input.
	 */

	cword_messages_output_message (
	    CWORD_S_INVALIDINPUT,
	    SCREEN0,
	    temp_str);

	cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	return (CWORD_CHOICENOTVALID);
    }

}



PRIVATE int cword_charui__output_help_screen (
	int			max_choice)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to output the short-cut options that are always available in
 *	cword.
 *
 *  FORMAL PARAMETERS:
 *
 *	max_choice		(r) Maximum choices available in the specific menu
 *
 *  IMPLICIT INPUTS:
 *
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL
 *--
 */

{

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	CWORD_S_CWORDCOMMANDS,
	SCREEN0);

    printf (
	"## %c, %c - Exit/Quit Cword completely\n",
	EXIT_CHAR,
        QUIT_CHAR);

    printf (
	"## %c - Back up to the calling menu or exit if in main menu\n",
	BACK_CHAR);

    printf (
	"## %c, %c - Gets this screen\n",
	HELP_CHAR,
	QUESTION_CHAR);

    printf (
	"## %c - Displays current defaults\n",
	DEFAULT_CHAR);

    printf (
	"## 1 to %d - Valid menu options\n",
	max_choice);

    printf (
	"## %c, %c, %c [OS command] - Execute an OS/shell command\n",
	SPAWN1_CHAR,
	SPAWN2_CHAR,
	SPAWN3_CHAR);

    printf (
	"## %s[directory] - Set directory into [directory]\n",
	CD_STR);

    printf ("## 0 - Back up menu or exit Cword\n");

    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

    return (CWORD_S_NORMAL);

}



PUBLIC BOOLEAN cword_charui_pause_ui (
	int			message_id,
	BOOLEAN			check_input)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	??
 *
 *  FORMAL PARAMETERS:
 *
 *	message_id		(r) Message ID of message to be outputted
 *	check_input		(r) TRUE - check user input using CWORD_PRESSRETURNORQUIT or
 *				    FALSE - don't check user input.
 *
 *  IMPLICIT INPUTS:
 *
 *      none
 *
 *  IMPLICIT OUTPUTS:
 *
 *      none
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Continue processing.
 *	FALSE			User wants out as QUIT_CHAR pressed.
 *--
 */

{

    char temp_str[CWORD_SIZEOF_MESSAGE + 1];

    if (message_id == 0)
    {
	/*
	 *  If message is NULL then eat any user input!
	 */

	cword_charui_get_str (
		ALL, 
		BLANK_OK,
		sizeof (temp_str) - 1,
		temp_str);

	/*
	 *  Return that everything is fine!
	 */

	return (TRUE);
    }	
    else
    {
	/*
	 *  Output message defined in message_id.
	 *
	 *  Use QUIT_CHAR irrespective of whether it's needed or not!
	 */

	cword_messages_output_message (
		message_id,
		SCREEN0,
		QUIT_CHAR);

	/*
	 *  Grab user input!
	 */

	cword_charui_get_str (
		ALL, 
		BLANK_OK,
		sizeof (temp_str) - 1,
		temp_str);

	/*
	 *  Now check check_input for TRUE or FALSE and act accordingly.
	 */

	if (check_input == TRUE)
	{
	    /*
	     *	I will assume that '\n' (or <return>) is OK and that 'q' is
	     *	not!
	     */

	    if (toupper (temp_str[0]) == QUIT_CHAR)
	    {
		return (FALSE);
	    }
	    else
	    {
		return (TRUE);
	    }
	}
	else
	{
	    return (TRUE);
	}
    }

}



PUBLIC int cword_charui_check_page_break (
	void			*check_list,
	int			message_id,
	int			*line_count)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	??
 *
 *  FORMAL PARAMETERS:
 *
 *	*check_list		()
 *	message_id		()
 *	*line_count		()
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Continue processing.
 *	CWORD_S_FAILURE		Stop processing as negative user input found.
 *--
 */

{

    if (check_list == (void *) 0)
    {
	/*
	 *  If nothing is left on check_list then there is nothing for us to do
	 *  either except return as normal.
	 */

	return (CWORD_S_NORMAL);
    }

    if ( ((*line_count % CWORD_SIZEOF_LINESPERSCREEN) == 0) && 
	  (*line_count != 0) )
    {
	/*
	 *  Reset *line_count and continuing.
	 */

	*line_count = 0;

	/*
	 *  Check user input and return if user is finished viewing output.
	 */

	if (cword_charui_pause_ui (message_id, TRUE) == FALSE)
	{
	    return (CWORD_S_FAILURE);
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE void cword_charui__masthead_print ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to print out the mast header proclaiming the name of the
 *	product and the producer ;-).
 *
 *  FORMAL PARAMETERS:
 *
 *      None.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None
 *--
 */

{

    cword_messages_output_message (
	CWORD_WAITINGENDED,
	SCREEN0);

    cword_messages_output_message (
	CWORD_S_DASHES,
	SCREEN0);

    cword_messages_output_message (
	CWORD_MAINBANNER,
	SCREEN0,
	CWORD_VERSION_ID);

    cword_messages_output_message (
	CWORD_S_DASHES,
	SCREEN0);

    cword_messages_output_message (
	CWORD_WAITINGENDED,
	SCREEN0);

    return;

}



PUBLIC int cword_charui_get_file_spec (
	int			msg_output,
	char			*file_suffix,
	int			sizeof_file_str,
	char			*file_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to get a valid, or not, file specification from standard
 *	input.
 *
 *  FORMAL PARAMETERS:
 *
 *	msg_output		(r) Whether to output error message or not
 *      *file_suffix		(r) file suffix expected from input
 *	sizeof_file_str		(r) Size of file_str
 *	*file_str		(w) file spec returning from this function
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Valid file spec found
 *	CWORD_S_FAILURE		Invalid information entered to standard input
 *--
 */

{

    int status;

    file_str[0] = (char) 0;

    /*
     *	Use cword_charui_get_str to get a string containing a file
     *	specification from the user.
     */

    status = cword_charui_get_str (
		VALID_FILE_SPEC, 
		BLANK_OK,
		sizeof_file_str - 1,
		file_str);

    if (CWORD_S_NORMAL != status)
    {
	return (status);
    }

    if (file_str[0] != (char) 0)
    {
#if defined(VMS) || defined (__VMS)
	/*
	 *  Switching to lower case string file_str as I like seeing lower
	 *  case letters only in a file specification.
	 */

	cword_word_lower_str (file_str);
#endif

	if (file_suffix != (char *) 0)
	{
	    /*
	     *	If the file spec inputted doesn't exist *or* the file spec
	     *	inputted doesn't ends in a dot, proceed to do further
	     *	processing to find a valid file ??
	     */

	    if ( (cword_utils_does_file_exist (file_str) != TRUE) &&
	          (file_str[strlen (file_str) - 1] != DOT_CHAR) )
	    {
		if (strstr (file_str, file_suffix) == (char *) 0)
		{
		    strcat (file_str, file_suffix);
		}
		else
		{
		    printf (
			"## file_suffix %s found in %s\n",
			file_suffix,
			file_str);
		}
	    }
	}

	/*
	 *  Check if the file name is valid?
	 */

	status = cword_utils_is_filename_legal (file_str);

	if (CWORD_S_NORMAL != status)
	{
	    file_str[0] = (char) 0;

	    return (CWORD_S_FAILURE);
	}
	else
	    return (CWORD_S_NORMAL);
    }
    else
    {
	if (msg_output == TRUE)
	{
	    cword_messages_output_message (
		CWORD_IVDINPUT,
		SCREEN0);
	}

	file_str[0] = (char) 0;

	return (CWORD_S_FAILURE);
    }

}



/*
 *  ?? Handle invalid '%' characters more correctly.  If '%' is found in input
 *  string then a second percent character should accompany it during any
 *  further processing.  This means the coding of a function to check 
 *  the input string for '%' characters ??
 *
 *  ?? Make sure that all uses of this function convert from 
 *  TRUE/FALSE to BLANK_OK/BLANK_NOTOK ??
 */

PUBLIC int cword_charui_get_str (
	unsigned long		char_mask,
	BOOLEAN_BLANK		blank_msg,
	int			sizeof_input_str,
	char			*input_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to get a string from standard input.  This function has been
 *	beefed up to accept/not accept certain charaacters other than the
 *	digits (0-9) and characters (a-z, A-Z).  At the moment it will accept
 *	the following:
 *
 *	   description		CONSTANT 
 *
 *	1.  a-z, A-Z		CHARACTER
 *	2.  1-9			DIGIT
 *	3.  ?			QUESTION	    
 *	4.  *			WILDCARD
 *	5.  %			PERCENT
 *	6.  space		SPACE
 *	7.  -			DASH
 *	8.  _			UNDERSCORE
 *	9.  .			DOT
 *      10. '                   APOSTROPHE
 *	10. everything		ALL
 *
 *  FORMAL PARAMETERS:
 *
 *	char_mask		(r) type of input to allow
 *	msg_output		(r) Whether to output error messages to the screen
 *	*input_str		(w) String to obtain from user
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Input from user OK in the context required
 *	CWORD_S_FAILURE		Input from user not OK in the context required
 *--
 */

{

    char temp_str[CWORD_SIZEOF_MESSAGE + 1],
	 message_str[CWORD_SIZEOF_MESSAGE + 1];

    input_str[0] = (char) 0;

    /*
     *	This will allow a string of size <= CWORD_SIZEOF_MESSAGE be inputted
     *	through stdin.
     */

    errno = 0;
    if ((char *) 0 == fgets (
			temp_str, 
			sizeof (temp_str) - 1,
			stdin))
    {
	cword_messages_get_message (
	    CWORD_S_FGETSPERRORMSG,
	    message_str);

	/*
	 *  ?? strerror ??
	 */

	perror (message_str);

	/*
	 *  Should really exit if fgets returns an error as it's probably only
	 *  a symptom of something deeper that is wrong!
	 */

	temp_str[0] = (char) 0;

	return (CWORD_S_FAILURE);
    }

    if (temp_str[strlen (temp_str) - 1] != '\n')
    {
	/*
	 *  This will chew up all extra input that may be waiting from stdin
	 *  over and above sizeof (temp_str) - 1.
	 */

	while ((char) '\n' != fgetc (stdin))
	{}
    }

    temp_str[strlen (temp_str) - 1] = (char) 0;

    /*
     *	If nothing is found then depending on the value of blank_msg, this
     *	function will return CWORD_S_NORMAL or CWORD_S_FAILURE.
     */

    if (temp_str[0] == (char) 0)
    {
	if (blank_msg == BLANK_OK)
	{
	    return (CWORD_S_NORMAL);
	}
	else
	{
	    printf ("## Nothing entered by user - NOTOK - returning FAILURE and message ##\n");

	    /*
	     *  ?? Error message ??
	     *
	     *	cword_messages_output_message (
	     *	    CWORD_IVDINPUT,
	     *	    SCREEN0);
	     */

	    return (CWORD_S_FAILURE);
	}
    }

    if (sizeof_input_str < strlen (temp_str) + 1)
    {
	cword_messages_output_message (
	    CWORD_S_TRUNCATINGINPUT,
	    SCREEN0,
	    sizeof_input_str);

	strncpy (
	    input_str, 
	    temp_str, 
	    sizeof_input_str);

	input_str[sizeof_input_str] = (char) 0;
    }
    else
    {
	/*
	 *  Copy temp_str into input_str and now check whether temp_str is
	 *  valid, or not.
	 */

	strcpy (input_str, temp_str);
    }

    /*
     *	Validate inputted string against defined mask.
     */

    if (cword_word_check_input_str (
		char_mask,
		temp_str) != CWORD_S_NORMAL)
    {
	cword_messages_output_message (
		CWORD_INVALIDCHAR,
		(TRACE2 | SCREEN0),
		input_str);

	input_str[0] = (char) 0;

	return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



PUBLIC char cword_charui_get_ch (
	int			msg_output)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that will return to the calling function the first valid
 *	character inputted by the user.
 *
 *  FORMAL PARAMETERS:
 *
 *	msg_output		(r) Output message if msg_output == TRUE
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      character		First character inputted by user
 *	(char) 0		Error or terminator inputted by user
 *--
 */

{

    char word_str[CWORD_SIZEOF_MESSAGE + 1];

    /*
     *  Call cword_charui_get_str to actually get the string you require
     */

    cword_charui_get_str (
	ALL, 
	BLANK_OK,
	sizeof (word_str) - 1,
	word_str);

    if (word_str[0] != (char) 0)
    {
	cword_word_del_lead_spaces (word_str);

	return (word_str[0]);
    }
    else
    {
	if (msg_output == TRUE)
	{
	    cword_messages_output_message (
		    CWORD_IVDINPUT,
		    (TRACE2 | SCREEN0));
	}	

	return ((char) 0);
    }

}



PUBLIC int cword_charui_highlight_dots (
	unsigned int		length)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    /*
     *  ?? VMS only at the moment ??
     */

#if defined(VMS) || defined (__VMS)

    char temp_str[CWORD_SIZEOF_LINE + 1],
	 temp1_str[CWORD_SIZEOF_LINE + 1];

    int count1;

    count1 = 0;
    while (count1 < length)
    {
	temp_str[count1] = (char) DOT_CHAR;
	count1++;
    }
    temp_str[count1] = (char) 0;
    
    sprintf (
	temp1_str,
	"%s?\033[%dD",
	temp_str,
	length + 1);

    cword_messages_output_message (
	CWORD_USERDEFINEDSTRING,
	SCREEN0,
	temp1_str);

    return (CWORD_S_NORMAL);

#else

    /*
     *  Nothing at the moment!
     */

    return (CWORD_S_NORMAL);

#endif

}



PUBLIC int cword_charui_ring_bell (
	unsigned int		times_to_ring)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    /* Only need a few characters as the bell is one character long */
    char temp_str[3];

    unsigned int count;

    if (cword_g_defaults_record.ring_bell == FALSE)
    {
	/*
	 *  No bell ringing (unless it's your own) if the global variable
	 *  ring_bell is FALSE.
	 */

	return (CWORD_S_NORMAL);
    }

    sprintf (
	temp_str,
	"%c",
	BELL_CHAR);

    for (count = 0; count < times_to_ring; count++)
    {
	cword_messages_output_message (
	    CWORD_USERDEFINEDSTRING,
	    SCREEN0,
	    temp_str);
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Move to stats as it's the only module using it!
 */

PUBLIC int cword_charui_display_definition (
	int			grid_or_crossword,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This is a function which will display either a crossword or a grid
 *	depnding on the value of the variable grid_or_crossword.
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CROSSWORD_GRID output_grid;

    if (grid_or_crossword == OUTPUT_GRID)
    {
	/*
	 *  Call cword_verify_get_block_props to get grid properties.
	 */

	cword_verify_get_block_props (
	    crossword_record,
	    output_grid);
    }
    else
    {
	/*
	 *  Call cword_verify_get_letters_grid to get cell contents.
	 */

	cword_verify_get_letters_grid (
	    crossword_record,
	    output_grid);
    }

    cword_charui_output_grid (
	0,
	(SCREEN0 | TRACE0),
	FALSE,
	crossword_record->x_dimensions,
	crossword_record->y_dimensions,
	output_grid);

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Change the method of outputting the evolving grid to space out 
 *  the grid contents for better readability ??
 *
 *  ?? Centre the output grid ??
 */

PUBLIC int cword_charui_output_grid (
	int			message_id,
	int			trace_level,
	BOOLEAN			is_grid_fill,
	int			x_dimensions,
	int			y_dimensions,
	CROSSWORD_GRID		crossword_grid)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function will output to stdout the specified CROSSWORD_GRID
 *	inputted to it via the argument list.  The user can specify the trace
 *	level together with how the grid should be outputted: is_grid_filled.
 *
 *	This function is called from GRID and SEED only.
 *
 *  FORMAL PARAMETERS:
 *
 *	message_id		() Message to be outputted before grid is displayed.
 *	trace_level		() How the grid is the be outputted.
 *	is_grid_fill		() TRUE/FALSE depending upon whether GRID or SEED is calling.
 *	x_dimensions		() X dimensions of inputted grid.
 *	y_dimensions		() Y dimensions of inputted grid.
 *	crossword_grid		() grid values to be outputted.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Normal completion.
 *--
 */

{

    char output_str[CWORD_SIZEOF_MESSAGE + 1];

    int count1,
	count2,
	line_num = 0;

    cword_messages_output_message (CWORD_OUTPUTGRID, TRACE2);

    CWORD_OS_CLEAR_SCREEN ();

    if (message_id != 0)
    {
	/*
	 *  Display the message (hopefully) if msg_id != 0.
	 */

	cword_messages_output_message (
	    message_id,
	    SCREEN0);

	line_num++;
    }

    cword_messages_output_message (
	CWORD_WAITINGENDED,
	trace_level);
    line_num++;

    memset (output_str, '\0', sizeof (output_str));
    strcpy (output_str, "+");

    for (count1 = 0; count1 < x_dimensions; count1++)
	output_str[1 + count1] = '-';

    strcat (output_str, "+\n");

    cword_messages_output_message (
	CWORD_USERDEFINEDSTRING,
	trace_level,
	output_str);

    for (count2 = 0; count2 < y_dimensions; count2++)
    {
	memset (output_str, 0, sizeof (output_str));
	strcpy (output_str, "|");

	for (count1 = 0; count1 < x_dimensions; count1++)
	{
	    /*
	     *	All processing that is necessary before outputting grid should
	     *	be completed before coming in here!
	     */

	    output_str[1 + count1] = crossword_grid[count1][y_dimensions - 1 - count2];
	}
	strcat (output_str, "|\n");

	cword_messages_output_message (
	    CWORD_USERDEFINEDSTRING,
	    trace_level,
	    output_str);

	line_num++;

	if (is_grid_fill == FALSE)
	{
	    /*
	     *  This should not be called if we are in grid fill mode.  Pass in
	     *  parameter at command line to handle checking this.
	     */

	    (void) cword_charui_check_page_break (
		    NOT_FINISHED,
		    CWORD_S_PRESSRETURN,
		    &line_num);
	}
    }

    memset (output_str, 0, sizeof (output_str));

    strcpy (output_str, "+");

    for (count1 = 0; count1 < x_dimensions; count1++)
	output_str[1 + count1] = '-';
    strcat (output_str, "+\n\n");

    cword_messages_output_message (
	CWORD_USERDEFINEDSTRING,
	trace_level,
	output_str);

    return (CWORD_S_NORMAL);

}



#else

main (
	int argv, 
	char *argc[])	

{

    int	status;

    OPEN_TRACE ("charui_test");

    WRITE_TRACE ("*** Executing CHARUI_TEST User Interface ***\n");

    status = cword_charui_display_main ();

    CLOSE_TRACE ();

    exit (EXIT_SUCCESS);

}

#endif
