#ifndef CWORD_CHARUI_LOADED
#define CWORD_CHARUI_LOADED 1

#include "main.h"

/*
 *  CHARACTER    a-z, A-Z
 *  DIGIT        0-9
 *  QUESTION     ?
 *  WILDCARD     *
 *  PERCENT      %
 *  SPACE        ' '
 *  DASH         -
 *  UNDERSCORE   _
 *  DOT          .
 *  APOSTROPHE   '
 *  BACKSLASH    \
 *  FORWARDSLASH /
 *  COLON        :
 *  DOLLAR       $
 *  SEMICOLON    ;
 *  
 *  ALL          Everything you want!
 */

/*
 *  Possible problems with sizeof (int) or sizeof (long)
 */

#define CHARACTER       0X00001
#define DIGIT           0X00002

#define PERCENT         0X00004
#define WILDCARD        0X00008
#define QUESTION        0X00010
#define SPACE           0X00020
#define DASH            0X00040
#define UNDERSCORE      0X00080
#define APOSTROPHE      0X00100
#define DOT             0X00200

#define BACKSLASH       0X00400
#define FORWARDSLASH    0X00800
#define COLON           0X01000
#define DOLLAR          0X02000
#define OPENSQUARE      0X04000
#define CLOSESQUARE     0X08000
#define SEMICOLON       0X10000

#define ALL		0X20000

/*
 *  Breaking down an input file specification into its constituent parts.
 *
 *  I am enforcing fairly normal file naming conventions on my audience.  If
 *  there are any complaints I suppose you could mess around in here to allow
 *  more liberal inputs be accepted.
 */

#define VALID_FILE_PREFIX	CHARACTER | DIGIT | DASH | UNDERSCORE

#define VALID_FILE_SUFFIX	CHARACTER | DIGIT | DASH | UNDERSCORE

/*
 *  Each OS has its own concept of what is a valid file specification.  I
 *  cannot cover all possibitlties but I can make a fairly good effort to stop
 *  invalid input being received from the user using these four definitions.
 */

#if defined(VMS) || defined (__VMS)

#define VALID_FILE_NAME		VALID_FILE_PREFIX | VALID_FILE_SUFFIX | DOT | SEMICOLON
#define VALID_FILE_SPEC		VALID_FILE_NAME | DOLLAR | COLON | OPENSQUARE | CLOSESQUARE

#endif

#if defined(__unix) || defined(__unix__)

#define VALID_FILE_NAME		VALID_FILE_PREFIX | VALID_FILE_SUFFIX | DOT
#define VALID_FILE_SPEC		VALID_FILE_NAME | FORWARDSLASH

#endif

#if defined(__MSDOS__) || defined(MSDOS)

#define VALID_FILE_NAME		VALID_FILE_PREFIX | VALID_FILE_SUFFIX | DOT
#define VALID_FILE_SPEC		VALID_FILE_NAME | COLON | BACKSLASH

#endif

/*
 *  What the various characters look like!
 */

#define PERCENT_CHAR	  '%'
#define WILDCARD_CHAR	  '*'
#define QUESTION_CHAR	  '?'
#define SPACE_CHAR	  ' '
#define DASH_CHAR	  '-'
#define UNDERSCORE_CHAR	  '_'
#define DOT_CHAR	  '.'
#define APOSTROPHE_CHAR	  '\''

#define BACKSLASH_CHAR    '\\'
#define FORWARDSLASH_CHAR '/'
#define COLON_CHAR        ':'
#define DOLLAR_CHAR       '$'
#define OPENSQUARE_CHAR   '['
#define CLOSESQUARE_CHAR  ']'
#define SEMICOLON_CHAR	  ';'

#define NOT_FINISHED	(void *) 1

/*
 *  ?? Change TRUE and FALSE to more appropriate values in grid, generate and
 *  clue modules - anything that used the get_str () function ??
 */

typedef enum
{
    BLANK_NOTOK = 0,
    BLANK_OK = 1
} BOOLEAN_BLANK;

PUBLIC int cword_charui_display_main ();

PUBLIC int cword_charui_get_choice (
	int			message_id,
	int			max_choice,
	int			*choice);

PUBLIC int cword_charui_get_str (
	unsigned long		char_mask,
	BOOLEAN_BLANK		blank_msg,
	int			sizeof_input_str,
	char			*input_str);

PUBLIC BOOLEAN cword_charui_pause_ui (
	int			message_id,
	BOOLEAN			check_input);

PUBLIC int cword_charui_check_page_break (
	void			*check_list,
	int			message_id,
	int			*line_count);

PUBLIC char cword_charui_get_ch (
	int			msg_output);

PUBLIC int cword_charui_get_file_spec (
	int			msg_output,
	char			*file_suffix,
	int			sizeof_file_spec,
	char			*file_spec);

PUBLIC int cword_charui_highlight_dots (
	unsigned int		length);

PUBLIC int cword_charui_ring_bell (
	unsigned int		times_to_ring);

PUBLIC int cword_charui_display_definition (
	int			grid_or_crossword,
	CROSSWORD_RECORD	*crossword_record);

PUBLIC int cword_charui_output_grid (
	int			message_id,
	int			trace_level,
	BOOLEAN			is_grid_fill,
	int			x_dimensions,
	int			y_dimensions,
	CROSSWORD_GRID		crossword_grid);

#endif
