/*
 *++
 *  FACILITY:
 *
 *      cl.c
 *
 *  ABSTRACT:
 *
 *      Module containing Command Line parsing functionality.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      18 March 1995
 *--
 */


/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>

#include "messages.h"
#include "charui.h"
#include "utils.h"
#include "word.h"
#include "defaults.h"
#include "soundex.h"
#include "match.h"
#include "memman.h"
#include "webster.h"
#include "cl.h"

#include "strll.h"

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#define CL_STRNCMP_CHECK(command_argument, defined_argument)   strncmp ( \
								command_argument, \
								defined_argument, \
								strlen (command_argument))

#if 0

/*
 *  ?? Obsolete ??
 */

#define CL_PARAM_MISSING(argument)	    if (argument == (char *) 0) \
					    { \
						cword_messages_output_message ( \
						    CWORD_S_COMLINEMISSING, \
						    (TRACE0 | SCREEN0)); \
						return (CWORD_S_FAILURE); \
					    } \
					    0;

#define CL_PARAM_CHECK(check_mask, argument)    if (cword_word_check_input_str ( \
							    check_mask, \
							    argument) != CWORD_S_NORMAL) \
						    { \
							cword_messages_output_message ( \
							    CWORD_S_COMLINEINVD, \
							    (TRACE0 | SCREEN0)); \
							return (CWORD_S_FAILURE); \
						    } \
						    0;
#endif



/*
 *  PRIVATE function prototypes.
 */

PRIVATE BOOLEAN cword_cl_is_param_missing (
	char			*param_str);

PRIVATE BOOLEAN cword_cl_is_param_ok (
	long			check_mask,
	char			*param_str);

PRIVATE int cword_cl__preproc_arguments (
	char			*arguments[]);

PRIVATE int cword_cl__get_nb_arguments (
	char			*arguments[],
	int			*min,
	int			*max,
	char			*centre_ch);



PRIVATE BOOLEAN cword_cl_is_param_missing (
	char			*param_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Checks wheth the input parameter - *param_str - is not NULL.  If it is
 *	NULL then a message is outputted and TRUE is returned, otherwise FALSE
 *	is returned.
 *
 *  FORMAL PARAMETERS:
 *
 *      *param_str		(r) Parameter to check for validity
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Parameter missing - print message and return error (TRUE)
 *	FALSE			Parameter not missing - continue
 *--
 */

{

    if (param_str == (char *) 0)
    {
	cword_messages_output_message (
	    CWORD_S_COMLINEMISSING,
	    (TRACE0 | SCREEN0));

	return (TRUE);
    }

    return (FALSE);

}



PRIVATE BOOLEAN cword_cl_is_param_ok (
	long			check_mask,
	char			*param_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Checks the input str - *param_str - against the check_mask inputted.
 *	If there are characters that are not allowed in the string then a
 *	message is outputted and FALSE is returned, otherwise TRUE is returned.
 *
 *  FORMAL PARAMETERS:
 *
 *	check_mask		()
 *	*param_str		()
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			
 *	FALSE			
 *--
 */

{

    if (cword_word_check_input_str (
	    check_mask,
	    param_str) != CWORD_S_NORMAL)
    {
	cword_messages_output_message (
	    CWORD_S_COMLINEINVD,
	    (TRACE0 | SCREEN0));

	return (FALSE);
    }

    return (TRUE);

}



PRIVATE int cword_cl__preproc_arguments (
	char			*arguments[])

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Do a bit of pre-processing on the arguments entered from the command
 *	line.  Only "lower-case" those command line inputs that start with a
 *	'-'!
 *
 *  FORMAL PARAMETERS:
 *
 *	*arguments[]		(r/w)
 *
 *  IMPLICIT INPUTS:
 *
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Command line pre-process correctly.
 *	CWORD_S_FAILURE		Command line processing error found.
 *--
 */

{

    int count = 1;

    /*
     *  Disregard *argument[0] as it's the image's name.
     */

    if (arguments[count] == (char *) 0) 
    {
	return (CWORD_S_NORMAL);
    }

    while (arguments[count] != (char *) 0)
    {
	if (arguments[count][0] == (char) '-')
	{
	    if (strlen (arguments[count]) == 1)
	    {
		/*
		 *  If an input of "-" is found immediately exit as error has
		 *  been found.
		 */

		cword_messages_output_message (
		    CWORD_S_COMLINEINVD,
		    (TRACE0 | SCREEN0));

		return (CWORD_S_FAILURE);
	    }
	    else
	    {
		/*
		 *  ?? &argument[count][1] ??
		 *
		 *  ?? Check that they these inputs are alphabetic only ??
		 */

		cword_word_lower_str (arguments[count]);

		DEBUG_TRACE (
		    "Reset command line argument %s to lower case\n",
		    arguments[count]);

		if ( ((strncmp (arguments[count], "-m", 2) == 0) || 
    		       (strncmp (arguments[count], "-s", 2) == 0)) 
    				&& 
                      (strlen (arguments[count]) == 2) )
		{
		    cword_messages_output_message (
			CWORD_S_COMLINENOTUNQ,
			(TRACE0 | SCREEN0),
			arguments[count]);

		    return (CWORD_S_FAILURE);
		}
		else if ( (strncmp (arguments[count], "-su", 3) == 0) && 
                      (strlen (arguments[count]) == 3) )
		{
		    cword_messages_output_message (
			CWORD_S_COMLINENOTUNQ,
			(TRACE0 | SCREEN0),
			arguments[count]);

		    return (CWORD_S_FAILURE);
		}
	    }
	}

	count++;
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_cl__get_nb_arguments (
	char			*arguments[],
	int			*min,
	int			*max,
	char			*centre_ch)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Parse the command line (*arguments[]) looking for -max, -min, and
 *	-defaults before continuing parsing the command line in general.
 *
 *  FORMAL PARAMETERS:
 *
 *	*arguments[]		(r) Command line context
 *	*min			(w) min. value for anagramming functionality.
 *	*max			(w) max. value for anagramming functionality.
 *	*centre_ch		(w) Centre character for some anagramming functionality.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      cword_g_defaults_record.defaults_file_spec
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful processing.
 *	CWORD_S_FAILURE		Error processing command line, or inputs held therein.
 *--
 */

{

    /*
     *	Parse through the complete command line looking for -max, -min,
     *	-centre, or/and -defaults only.  These are all needed before any of the
     *	other options can be executed.
     */

    int count = 1;

    /*
     *	Disregard *argument[0] as it's the image's name.
     */

    if (arguments[count] == (char *) 0) 
    {
	return (CWORD_S_NORMAL);
    }

    while (arguments[count] != (char *) 0)
    {
	if (CL_STRNCMP_CHECK (arguments[count], "-maximum") == 0)
	{
	    count++;

	    if (cword_cl_is_param_missing (arguments[count]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    DIGIT,
		    arguments[count]) == FALSE)
		return (CWORD_S_FAILURE);

	    *max = atoi (arguments[count]);

	    printf (
		"## Getting max = %u\n",
		*max);
	}
	else if (CL_STRNCMP_CHECK (arguments[count], "-minimum") == 0)
	{
	    count++;

	    if (cword_cl_is_param_missing (arguments[count]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    DIGIT,
		    arguments[count]) == FALSE)
		return (CWORD_S_FAILURE);

	    *min = atoi (arguments[count]);

	    printf (
		"## Getting min = %u\n",
		*min);
	}
	else if (CL_STRNCMP_CHECK (arguments[count], "-centre") == 0)
	{
	    count++;

	    if (cword_cl_is_param_missing (arguments[count]) == TRUE)
		return (CWORD_S_FAILURE);

	    if ( (strlen (arguments[count]) > 1) || (isalpha (arguments[count][0]) == 0) )
	    {
		cword_messages_output_message (
		    CWORD_S_COMLINECENONLY,
		    (TRACE0 | SCREEN0));

		return (CWORD_S_FAILURE);
	    }

	    *centre_ch = arguments[count][0];

	    printf (
		"## Getting centre character = %c\n",
		*centre_ch);
	}
	else if (CL_STRNCMP_CHECK (arguments[count], "-defaults") == 0)
	{
	    count++;

	    if (cword_cl_is_param_missing (arguments[count]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    VALID_FILE_SPEC,
		    arguments[count]) == FALSE)
		return (CWORD_S_FAILURE);

	    if (cword_utils_does_file_exist (arguments[count]) == FALSE)
	    {
		cword_messages_output_message (
		    CWORD_S_FILENOTFOUNDERR,
		    (TRACE0 | SCREEN0),
		    arguments[count]);

		return (CWORD_S_FAILURE);
	    }

	    /*
	     *  Altering the global variable default_file_str.
	     */

	    strcpy (cword_g_defaults_record.defaults_file_spec, arguments[count]);

	    printf (
		"# Obtained defaults file name: %s from command line\n", 
		cword_g_defaults_record.defaults_file_spec);
	}

	count++;
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Allow support for the major pieces of functionality that Cword provides ??
 *  i.e.
 *  
 *	1. grid creation
 *	2. grid filling
 *	3. crossword completion
 *
 *  ?? List the valid command line options available ??
 *
 *  ?? Check are there any other possible valid command line parameters being
 *  ignored because of the execution path being taken.  If so then use
 *  CWORD_S_COMLINIGNORE ??
 */

PUBLIC int cword_cl_parse_command_line (
	int			arg_no,
	char			*arguments[])

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to parse the Cword command line obtaining the users commands
 *	and executing them if so required..
 *
 *  FORMAL PARAMETERS:
 *
 *      arg_no			(r) Number of arguments read from command line.
 *	*arguments[]		(r) Array of pointers to character strings containing 
 *                                  command line input.
 *
 *  IMPLICIT OUTPUTS:
 *
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    char centre_ch = 0;

    int input_no = 1,
	max = 0,
	min = 0;

    /*
     *	Disregard *argument[0] as it's the image's name.
     */

    if (arguments[input_no] == (char *) 0)
    {
	/*
	 *  No command line options provided at all.
	 */

    	DEBUG_TRACE ("No command line options provided - clean run at UI possible\n");

	return (CWORD_S_NORMAL);
    }

    /*
     *	To begin: lowercase all command line inputs beginning with a '-' and do
     *	some checking of the command line parameters
     */

    if (cword_cl__preproc_arguments (arguments) != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *	Look for -maximum, -minimum, -centre, or/and -defaults before
     *	continuing any further.
     */

    if (cword_cl__get_nb_arguments (
	    arguments,
	    &min,
	    &max,
	    &centre_ch) != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *	Read the defaults file before going any further.  This assumes that any
     *	defaults file other than the "default" defaults file will already be
     *	known, before getting to here!
     */

    cword_defaults_read_defaults ();

    while (input_no < arg_no)
    {
	if (CL_STRNCMP_CHECK (arguments[input_no], "-webster") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    CHARACTER,
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    cword_webster_get_defn (arguments[input_no]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-subanagram") == 0)
	{
	    /*
	     *	?? Use -max, -min, and -centre definitions ??
	     */

	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    CHARACTER,
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Using subanagram word: %s\n",
		arguments[input_no]);

    	    cword_cl_exe_subanagram (
    		min,
    		max,
    		centre_ch,
    		arguments[input_no]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-superanagram") == 0)
	{
	    /*
	     *	?? Use -max, -min, and -centre definitions ??
	     */

	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    CHARACTER,
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting superanagram word: %s\n",
		arguments[input_no]);

    	    cword_cl_exe_superanagram (
    		min,
    		max,
    		centre_ch,
    		arguments[input_no]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-soundex") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    CHARACTER,
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting soundex word: %s\n",
		arguments[input_no]);

    	    cword_cl_exe_soundex (arguments[input_no]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-template") == 0)
	{
	    /*
	     *	?? Check whether the user has correctly defined the template to
	     *	be searched for i.e. 
	     *
	     *  $ cword -template j   s
	     *
	     *  would be incorrect while 
	     *
	     *  $ cword -template "j   s"
	     *
	     *  would be OK ??
	     */

	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    /*
	     *  Allow only QUESTION_CHAR, SPACE_CHAR or a character as input.
	     */

	    if (cword_cl_is_param_ok (
		    (CHARACTER | SPACE | QUESTION | PERCENT | DOT),
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting template search word: %s\n",
		arguments[input_no]);

    	    cword_cl_exe_template (arguments[input_no]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-intersect") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    (CHARACTER | SPACE | QUESTION | PERCENT | DOT),
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting intersect word 1: %s\n",
		arguments[input_no]);

	    if (cword_cl_is_param_missing (arguments[input_no + 1]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    (CHARACTER | SPACE | QUESTION | PERCENT | DOT),
		    arguments[input_no + 1]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting intersect word 2: %s\n",
		arguments[input_no + 1]);

    	    cword_cl_exe_intersect (
    		arguments[input_no],
    		arguments[input_no + 1]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-addword") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    CHARACTER,
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting word to add to word files: %s\n",
		arguments[input_no]);

    	    /*
     	     *  Not happy with output message if word is already in 
             *  word files
    	     */

    	    /*
             *  ?? Not happy with function name - add_to_word_file 
             *  (already exists) ??
             */

    	    if (cword_match_add_word_to_file (arguments[input_no]) != CWORD_S_NORMAL)
    	    {
		cword_messages_output_message (
		    CWORD_S_WORDFILEERR,
		    SCREEN0,
		    arguments[input_no]);
    	    }

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-rot13") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    (SPACE | CHARACTER),
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting rot13 word: %s\n",
		arguments[input_no]);

	    cword_word_rot_string (
		13,
		(unsigned char *) arguments[input_no]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-wordwild") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    (CHARACTER | WILDCARD | QUESTION),
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting wordwild word: %s\n",
		arguments[input_no]);

    	    cword_cl_exe_wordwild (arguments[input_no]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-spell") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    CHARACTER,
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    printf (
		"## Getting word to spell: %s\n",
		arguments[input_no]);

	    cword_cl_exe_spell (arguments[input_no]);

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-text") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    VALID_FILE_SPEC,
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    if (cword_utils_does_file_exist (arguments[input_no]) == FALSE)
	    {
		cword_messages_output_message (
		    CWORD_S_FILENOTFOUNDERR,
		    (TRACE0 | SCREEN0),
		    arguments[input_no]);

		return (CWORD_S_FAILURE);
	    }

	    printf (
		"## Getting grid/crossword file to print as text: %s\n",
		arguments[input_no]);

	    /*
	     *	?? Not very powerful - should have a function to autosense the
	     *	correct function to call in such circumstances ??
	     */

	    if (strstr (arguments[input_no], CWORD_SUFFIX_XWORD_FILE) != 0)
	    {
		/*
		 *  Found CWORD_SUFFIX_XWORD_FILE in the input file's
		 *  specification.
		 */

		cword_print_crossword (
		    TEXT,
		    arguments[input_no]);
	    }
	    else
	    {
		cword_print_grid (
		    TEXT,
		    arguments[input_no]);
	    }

	    return (CWORD_S_FAILURE);
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-postscript") == 0)
	{
	    input_no++;

	    if (cword_cl_is_param_missing (arguments[input_no]) == TRUE)
		return (CWORD_S_FAILURE);

	    if (cword_cl_is_param_ok (
		    VALID_FILE_SPEC,
		    arguments[input_no]) == FALSE)
		return (CWORD_S_FAILURE);

	    if (cword_utils_does_file_exist (arguments[input_no]) == FALSE)
	    {
		cword_messages_output_message (
		    CWORD_S_FILENOTFOUNDERR,
		    (TRACE0 | SCREEN0),
		    arguments[input_no]);

		return (CWORD_S_FAILURE);
	    }

	    printf (
		"## Getting grid/crossword file to print as postscript: %s\n",
		arguments[input_no]);

	    /*
	     *	?? Not very powerful - should have a function to autosense the
	     *	correct function to call in such circumstances ??
	     */

	    if (strstr (arguments[input_no], CWORD_SUFFIX_XWORD_FILE) != 0)
	    {
		/*
		 *  Found CWORD_SUFFIX_XWORD_FILE in the input file's
		 *  specification.
		 */

		cword_print_crossword (
		    POSTSCRIPT,
		    arguments[input_no]);
	    }
	    else
	    {
		cword_print_grid (
		    POSTSCRIPT,
		    arguments[input_no]);
	    }

	    return (CWORD_S_FAILURE);
	}
	else if ( (CL_STRNCMP_CHECK (arguments[input_no], "-help") == 0) ||
		   (CL_STRNCMP_CHECK (arguments[input_no], "-?") == 0) ||
		    (CL_STRNCMP_CHECK (arguments[input_no], "?") == 0) ||
		     (CL_STRNCMP_CHECK (arguments[input_no], "help") == 0) )
	{
	    /*
	     *  ?? Put into messages.h ??
	     */

	    printf ("Usage: cword [-defaults defaults_file] [-max n] [-min n] [-centre ch]\n");
	    printf ("       [-webster word] [-subanagram word] [-superanagram word]\n");
	    printf ("       [-soundex word] [-template template_str] [-spell word]\n");
	    printf ("       [-addword word] [-rot13 string] [-wordwild string]\n");
	    printf ("       [-text crossword/grid file] [-postscript crossword/grid file]\n");
	    printf ("       [-intersect template1_str template2_str] [-help] [help] [-?] [?]\n");

	    return (CWORD_S_FAILURE);
	}
	/*
	 *  Ignore -defaults. -minimum, and -maximum as they have already been
	 *  processed!
	 */
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-defaults") == 0)
	{
	    input_no++;
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-centre") == 0)
	{
	    input_no++;
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-minimum") == 0)
	{
	    input_no++;
	}
	else if (CL_STRNCMP_CHECK (arguments[input_no], "-maximum") == 0)
	{
	    input_no++;
	}
	else
	{
	    cword_messages_output_message (
		CWORD_S_COMLINEUNSUPT,
		(TRACE0 | SCREEN0),
		arguments[input_no]);

	    /*
	     *	?? Should I exit, or continue with a warning ??
	     */

	    return (CWORD_S_FAILURE);
	}

	input_no++;
    }

    if ( (max != 0) || (min != 0) || (centre_ch != 0) )
    {
	cword_messages_output_message (
	    CWORD_S_COMLINEMAXMIN,
	    (TRACE0 | SCREEN0));
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Use these functions in CHARUI to replace functionality already existing ??
 */

PUBLIC int cword_cl_exe_subanagram (
    	int			min,
    	int			max,
    	char			centre_ch,
    	char			*check_str)    

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *
 *  FORMAL PARAMETERS:
 *
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *--
 */

{

    CWORD_T_STR_LL *str_ll = (CWORD_T_STR_LL *) 0;

    int status;

    /*
     *  ?? Not using max at the moment ??
     */

    status = cword_anagram_get_subanagrams (
		    check_str,
		    min,
		    centre_ch,
		    &str_ll);

    if (CWORD_S_NORMAL == status)
    {
	/*
	 *  Output link list of matched elements
	 */

	 if (str_ll != (CWORD_T_STR_LL *) 0)
	 {
	    cword_messages_output_message (
		    CWORD_RETURNEDWORDS,
		    SCREEN0);

	    cword_strll_output_str_ll (
		    str_ll,
		    0);
	 }
    }

    cword_strll_free_str_ll (&str_ll);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_cl_exe_superanagram (
    	int			min,
    	int			max,
    	char			centre_ch,
    	char			*check_str)    

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *
 *  FORMAL PARAMETERS:
 *
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *--
 */

{

    CWORD_T_STR_LL *str_ll = (CWORD_T_STR_LL *) 0;

    int status;

    /*
     *	?? Not using max, min or centre_ch at the moment ??
     *
     *  ?? Move to anagram - cword_anagram_get_super_anags ??
     */

    status = cword_match_generic_read_func (
		cword_word_super_anagram_str,
		CWORD_S_SUPERANAGRAMNOW,
		check_str,
		&str_ll);

    if (CWORD_S_NORMAL == status)
    {
	cword_messages_output_message (
		CWORD_RETURNEDWORDS,
		SCREEN0);

	cword_strll_output_str_ll (
		str_ll,
		0);
    }

    cword_strll_free_str_ll (&str_ll);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_cl_exe_soundex (
    	char			*check_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *
 *  FORMAL PARAMETERS:
 *
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *--
 */

{

    CWORD_T_STR_LL *soundex_ll = (CWORD_T_STR_LL *) 0;

    int status;

    if (cword_g_defaults_record.soundex_algorithm == SOUNDEX_MATCH_START)
    {
    	status = cword_soundex_samelet_matches (
		    check_str,
		    &soundex_ll);
    }
    else
    {
    	status = cword_soundex_getall_matches (
		    check_str,
		    &soundex_ll);
    }

    if (CWORD_S_NORMAL == status)
    {
    	/*
    	 *  Output link list of matched Soundex elements.
	 */

	if (soundex_ll != (CWORD_T_STR_LL *) 0)
	{
	    cword_messages_output_message (
	        CWORD_SOUNDEXLIST,
		SCREEN0);

    	    cword_strll_output_str_ll (
	        soundex_ll,
		0);
    	}
    }

    cword_strll_free_str_ll (&soundex_ll);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_cl_exe_template (
    	char			*check_str)    

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *
 *  FORMAL PARAMETERS:
 *
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *--
 */

{

    CWORD_T_STR_LL *word_ll = (CWORD_T_STR_LL *) 0;

    int status;

    status = cword_match_template_list (
		check_str,
		&word_ll);

    if (CWORD_S_NORMAL == status)
    {
	/*
	 *  Output link list of elements found to match template.
	 */

    	if (word_ll != (CWORD_T_STR_LL *) 0)
	{
	    cword_messages_output_message (
	        CWORD_RETURNEDWORDS,
		SCREEN0);

	    cword_strll_output_str_ll (
		word_ll,
		0);
	}
    }
    else
    {
        cword_messages_output_message (
	    CWORD_TEMPLATEFAILMSG,
	    (TRACE0 | SCREEN0));
    }

    cword_strll_free_str_ll (&word_ll);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_cl_exe_intersect (
    	char			*check1_str,
    	char			*check2_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *
 *  FORMAL PARAMETERS:
 *
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *--
 */

{

    CWORD_T_STR_LL *double_list = (CWORD_T_STR_LL *) 0;

    /*
     *  ?? Check return status of this function call ??
     */

    cword_match_find_intersects (
	check1_str,
	check2_str,
	&double_list);

    if (double_list != (CWORD_T_STR_LL *) 0)
    {
    	cword_messages_output_message (
	    CWORD_INTERSECTLIST,
	    SCREEN0);

	/*
	 *  Output using double format!
	 */

	cword_strll_output_twin_str_ll (
	    double_list,
	    4);
    }

    cword_strll_free_str_ll (&double_list);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_cl_exe_wordwild (
    	char			*check_str)    

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *
 *  FORMAL PARAMETERS:
 *
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *--
 */

{

    CWORD_T_STR_LL *word_ll = (CWORD_T_STR_LL *) 0;

    int status;

    /*
     *  ?? Allow spaces and replace with '?'s ??
     */

    status = cword_match_generic_read_func (
                cword_word_str_wild,
		CWORD_STRWILDNOW,
		check_str,
		&word_ll);

    if (CWORD_S_NORMAL == status)
    {
        cword_messages_output_message (
	    CWORD_RETURNEDWORDS,
	    SCREEN0);

	/*
         *  Output link list of items
	 */

	cword_strll_output_str_ll (
	    word_ll,
	    0);
    }

    cword_strll_free_str_ll (&word_ll);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_cl_exe_spell (
	char			*check_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int status;

    status = cword_match_spell_check (check_str);

    if (CWORD_S_NORMAL == status)
	cword_messages_output_message (
	    CWORD_SPELLSUCCESS,
	    (TRACE0 | SCREEN0),
	    check_str);
    else if (CWORD_S_FAILURE == status)
	cword_messages_output_message (
	    CWORD_SPELLFAILURE,
	    (TRACE0 | SCREEN0),
	    check_str);
    else
	cword_messages_output_message (
	    status,
	    (TRACE0 | SCREEN0));
	
    return (CWORD_S_NORMAL);

}

