/*
 *  ?? Option to view the grid graphically or list the lights so as 
 *  to get a better idea what's happening in the grid:
 *
 *  L = should view clues that have been user defined (chang from V 
 *      currently)
 *
 *  V = should view the lights' grid or at least view the light 
 *      definitions sequentially ??
 */

/*
 *++
 *  FACILITY:
 *
 *      clue.c
 *
 *  ABSTRACT:
 *
 *	Module containing the functionality to allow the user insert clues into
 *	an already filled grid.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      23 September 1994 (Should have been done a lot sooner)
 *--
 */


/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "main.h"

#ifdef CLUE
#define MESSAGES_DEFINED 1 
#endif

#include "messages.h"
#include "utils.h"
#include "os.h"
#include "charui.h"
#include "fileio.h"
#include "stats.h"
#include "clue.h"

#if 0

/*
 *  This is not needed right now but could be useful in the future
 */

#include "defaults.h"
extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;
#endif

/*
 *  Module globals used to determine whether crossword definition has been
 *  altered or not
 */

static BOOLEAN cword_clue_l_defn_updated;
static BOOLEAN cword_clue_l_defn_downdated;
static BOOLEAN cword_clue_l_defn_full;

/*
 *  ?? Possibly use an array for the more complete storage of previously
 *  generated clues ??
 */

static int cword_clue_l_last_clue_num;
static int cword_clue_l_last_clue_dir;



#ifndef CLUE

/* 
 *  Use these constants in the menu that constitutes the functionality of this
 *  module
 */

#define ADD_CHAR	'A'
#define QUIT_CHAR	'Q'
#define SAVE_CHAR	'S'
#define VIEW_CHAR	'V'
#define RESET_CHAR	'R'
#define LIST_CHAR	'L'
#define HELP_CHAR	'H'
#define INVALID_CHAR    '\0'

/*
 *  These string constants are used when constructing the clue description
 */

#define LIGHT_OPEN_BRACKET	"("
#define LIGHT_CLOSE_BRACKET	")"

#define CLUE_CLUES_PER_LINE     5

/*
 *  PRIVATE function prototypes
 */

PRIVATE int cword_clue__output_clue_opts (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_clue__get_clue_coords (
	CROSSWORD_RECORD	*crossword_record,
	int			next_clue_number,
	int			next_clue_direction,
	int			*clue_number,
	int			*clue_direction);

PRIVATE int cword_clue__get_valid_clue (
        CROSSWORD_RECORD	*crossword_record,
	int			*clue_number,
	int			*clue_direction);

PRIVATE int cword_clue__check_clue_desc (
	char			*clue_ans,
	char			*clue_str);

PRIVATE int cword_clue__get_light_len (
	int			light_len,
	char			*clue_str);

PRIVATE int cword_clue__update_clue_desc (
        int			clue_number,
	int			clue_direction,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_clue__view_grid (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_clue__list_clues (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE void cword_clue__output_help ();

PRIVATE int cword_clue__reset_clue (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE void cword_clue__get_option (
	char			*option);



PRIVATE int cword_clue__output_clue_opts (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to output to user an idea of the current state of the
 *	crossword definition.  Information includes how many of the clues have
 *	been generated by the user.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r) Pointer to CROSSWORD_RECORD
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *	cword_clue_l_defn_full
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion but clues left to generate
 *	CWORD_S_CLUESALLDEFD	Crossword defintion is completely defined
 *--
 */

{

    int count,
	line_num;

    BOOLEAN is_clue_valid,
	    are_all_clues_filled = TRUE,
	    are_over_99_clues;

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	    CWORD_S_CLUESTARTTEXT,
	    SCREEN0);

    if (cword_stats_get_high_clue_num (
	    BOTH,
	    crossword_record) > 99)
        are_over_99_clues = TRUE;
    else
        are_over_99_clues = FALSE;

    line_num = 2;
    for (count = 0; count < crossword_record->clue_sum; count++)
    {
	/*
	 *  How many clues should be one a single line?
	 */

	if ( !(count % CLUE_CLUES_PER_LINE) )
	{
	    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

	    line_num++;
	}

	/*
	 *  Powerful way of handling the correct screening of output to CCUI.
	 *
	 *  ?? Shitty way of handling this - NOT_FINISHED ??
	 */

	(void) cword_charui_check_page_break (
		    NOT_FINISHED,
		    CWORD_S_PRESSRETURN,
		    &line_num);

	is_clue_valid = TRUE;
	if (strcmp (
		crossword_record->clue_details[count].clue_description,
		crossword_record->clue_details[count].clue_answer) == 0)
        {
	    are_all_clues_filled = FALSE;
	    is_clue_valid = FALSE;
        }

	cword_messages_output_message (
		CWORD_GRIDOUTPUTCLUE,
		SCREEN0,
		are_over_99_clues == FALSE ? 2 : 3,
		crossword_record->clue_details[count].clue_number,
		crossword_record->clue_details[count].clue_direction 
				    ? CWORD_ACROSS_STR : CWORD_DOWN_STR,  
		is_clue_valid ? CWORD_CLUED_CHAR : CWORD_NOTCLUED_CHAR);

    }

    if (are_all_clues_filled == TRUE)
    {
	/*
	 *  Is the grid completely generated - YES!
	 */

	cword_clue_l_defn_full = TRUE;

	cword_messages_output_message (
		CWORD_S_CLUESALLDEFD,
		SCREEN0);

	return (CWORD_S_CLUESALLDEFD);
    }
    else
    {
	/*
	 *  Is the grid completely generated - NO!
	 */

	cword_clue_l_defn_full = FALSE;
    }

    cword_messages_output_message (
	    CWORD_USERDEFINEDSTRING,
	    SCREEN0,
	    "\n\n");

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_clue__get_clue_coords (
	CROSSWORD_RECORD	*crossword_record,
	int			next_clue_number,
	int			next_clue_direction,
	int			*clue_number,
	int			*clue_direction)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to get the clue definition (clue number and clue direction)
 *	and check whether the inputted clue definition exists of not.  This
 *	function is used with adding and resetting clues during the clue
 *	generation phase.
 *
 *  FORMAL PARAMETERS:
 *
 *	*crossword_record	(r) Pointer to CROSSWORD_RECORD
 *	next_clue_number	(r) Suggested clue number
 *	next_clue_direction	(r) Suggested clue direction
 *	*clue_number            (w) Actual clue number chosen
 *	*clue_direction         (w) Actual clue direction chosen
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_clue_l_defn_updated 
 *	cword_clue_l_defn_downdated
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Successful completion
 *	CWORD_S_FAILURE		Serious error recorded
 *      CWORD_S_INVALIDCLUEINFO	Invaliud clue information entered - can continue
 *--
 */

{

    char    temp_str[CWORD_SIZEOF_MESSAGE + 1],
	    temp_ch;

    static int	highest_clue = 0;

    int count,
	status,
        down,
        across,
	times_around;

    BOOLEAN is_valid_clue;

    *clue_number = *clue_direction = 0;

    /*
     *	Get a valid clue definition so that it can be used to:
     *
     *	1. add or
     *	2. reset a clue description
     */

    /*
     *  ?? Could be a better way to do this but I don't know it yet ??
     */

    if ( (cword_clue_l_defn_updated == FALSE) && (cword_clue_l_defn_downdated == FALSE) )
	highest_clue = 0;

    if (highest_clue == 0)
    {
	/*
	 *  Only executed once!
	 */

	for (count = 0; count < crossword_record->clue_sum; count++)
	{
	    if (crossword_record->clue_details[count].clue_number > highest_clue)
		highest_clue = crossword_record->clue_details[count].clue_number;
	}

	if (highest_clue == 0)
	{
	    /*
	     *	?? Some errors are so unlikely that they may never happen so
	     *	they will be categorised by a single '#' at the beginning of
	     *	the line of text to be output ??
	     */

	    printf ("# Serious error: highest clue not found.  Cannot continue.\n");

	    return (CWORD_S_FAILURE);
	}
    }

    times_around = 0;
    is_valid_clue = FALSE;
    while (is_valid_clue != TRUE)
    {
	is_valid_clue = TRUE;

        times_around++;

	if (times_around == 4)
	{
	    cword_messages_output_message (
		    CWORD_S_INVALIDCLUEINFO,
		    SCREEN0);

	    return (CWORD_S_INVALIDCLUEINFO);
	}

	cword_messages_output_message (
		CWORD_S_CLUENUMBERMSG,
		SCREEN0,
		1, 
		highest_clue,
		next_clue_number);

	/*
	 *  Just get a valid integer without mapping it to a light
	 *  location in the grid for the time being
	 */

	status = cword_charui_get_str (
		    DIGIT,
		    BLANK_OK,
		    sizeof (temp_str) - 1,
		    temp_str);

	if (CWORD_S_NORMAL != status)
	{
	    cword_messages_output_message (
		    CWORD_S_INVALIDCLUEINFO,
		    SCREEN0);

	    return (CWORD_S_INVALIDCLUEINFO);
	}

	if (temp_str[0] == (char) 0)
	    *clue_number = next_clue_number;
	else
	    *clue_number = atoi (temp_str);

	/*
	 *  Check if there is a 24 Across or 24 Down before outputting to
	 *  user the possibilities
	 */

	down = FALSE; across = FALSE;
	for (count = 0; count < crossword_record->clue_sum; count++)
	{
	    if (crossword_record->clue_details[count].clue_number == *clue_number)
		if (crossword_record->clue_details[count].clue_direction == ACROSS)
		    across = TRUE;
		else
		    down = TRUE;
	}

	if ( (across == FALSE) && (down == FALSE) )
	{
	    /* 
	     *  No valid clue found for clue_num - warning
	     */

	    cword_messages_output_message (
			CWORD_INVALIDLIGHTNUM,
			SCREEN0,
			*clue_number,
			CWORD_ACROSS_STR,
			*clue_number, 
			CWORD_DOWN_STR);

	    is_valid_clue = FALSE;
	}

	if (is_valid_clue == TRUE)
	{
	    temp_ch = (char) 0;

	    if ( (across == TRUE) && (down == TRUE) )
	    {
		cword_messages_output_message (
			    CWORD_S_CLUEDIRECTMSG,
			    SCREEN0,
			    CWORD_A_CHAR, 
			    CWORD_D_CHAR,
			    next_clue_direction ? CWORD_A_CHAR : CWORD_D_CHAR);

		temp_ch = cword_charui_get_ch (FALSE);

		if (toupper (temp_ch) == CWORD_A_CHAR)
		    *clue_direction = ACROSS;
		else if (toupper(temp_ch) == CWORD_D_CHAR)
		    *clue_direction = DOWN;
		else if (temp_ch == (char) 0)
		    *clue_direction = next_clue_direction;
		else
		{
		    /* default */

		    cword_messages_output_message (
				CWORD_S_DEFOPTCHOSEN,
				SCREEN0);

		    *clue_direction = ACROSS;
		}
	    }
	    else
	    if (across == TRUE)
		*clue_direction = ACROSS;
	    else
	    if (down == TRUE)    
		*clue_direction = DOWN;

	    cword_messages_output_message (
		    CWORD_LIGHTDIRCHOSEN,
		    SCREEN0,
		    *clue_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR);
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_clue__get_valid_clue (
        CROSSWORD_RECORD	*crossword_record,
	int			*clue_number,
	int			*clue_direction)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to get a valid clue number and clue direction which will then
 *	be clued by the user.  Calls cword_clue__get_clue_coords after
 *	suggested clue data is obtained.
 *
 *  FORMAL PARAMETERS:
 *
 *	*crossword_record	(r) Pointer to CROSSWORD_RECORD
 *	*clue_number		(w) number of clue to be updated
 *	*clue_direction		(w) direction of clue to be updated
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *	CWORD_S_FAILURE		Valid clue number/description not entered
 *--
 */

{

    int count,
	status,
	next_clue_number = 0,
	next_clue_direction = 0;

    BOOLEAN is_clue_found;

    /*
     *  Get the clue that is highest up the list and is still undefined by the user
     */

    count = 0;
    is_clue_found = FALSE;
    while (count < crossword_record->clue_sum)
    {
	if (strcmp (
		crossword_record->clue_details[count].clue_answer,
		crossword_record->clue_details[count].clue_description) == 0)
	{
	    next_clue_number = crossword_record->clue_details[count].clue_number;
	    next_clue_direction = crossword_record->clue_details[count].clue_direction;
	    is_clue_found = TRUE;

	    break;
	}

	count++;
    }

    if (is_clue_found == FALSE)
    {
	cword_messages_output_message (
	    CWORD_S_NOUNDCLUESFND,
	    SCREEN0,
	    crossword_record->clue_details[0].clue_number,
	    crossword_record->clue_details[0].clue_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

	next_clue_number = crossword_record->clue_details[0].clue_number;
	next_clue_direction = crossword_record->clue_details[0].clue_direction;
    }

    DEBUG_TRACE (
	"Successfully found %d %s\n",
	next_clue_number,
	next_clue_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

    status = cword_clue__get_clue_coords (
		crossword_record,
		next_clue_number,
		next_clue_direction,
		clue_number,
		clue_direction);

    return (status);

}



PRIVATE int cword_clue__check_clue_desc (
	char			*clue_ans,
	char			*clue_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check the inputted clue string.  This function will alter
 *	the user to problems in the string, if necessary.  These problems
 *	include:
 *
 *	1. The first character of the clue description is not in uppercase
 *	2. The answer to the clue is contained in the string given that the
 *         description is now different
 *	3. The clue description is blank or contains all spaces
 *
 *  FORMAL PARAMETERS:
 *
 *	*clue_ans		(r) Answer to clue description
 *	*clue_str		(r/w) clue description
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Inputted clue description is OK
 *	CWORD_S_FAILURE		Inputted clue description is not OK
 *--
 */

{

    int count;

    BOOLEAN is_clue_empty;

    /*
     *	Check various attributes of the inputted clue for completeness
     */

    if (clue_str[0] == (char) 0)
    {
	cword_messages_output_message (
		CWORD_S_NULLCLUEENTERED,
		SCREEN0);

	return (CWORD_S_FAILURE);
    }
    else
    {
	/*
	 *  Check that the inputted clue contains at least one alpanumeric
	 *  character
	 */

	count = 0;
	is_clue_empty = TRUE;
	while (count < strlen (clue_str))
	{
	    if (isalnum (clue_str[0]))
	    {
		is_clue_empty = FALSE;

		break;
	    }

	    count++;
	}

	if (is_clue_empty == TRUE)
	{
	    cword_messages_output_message (
		    CWORD_S_CLUEISBLANKS,
		    SCREEN0);
	    
	    return (CWORD_S_FAILURE);
	}

	/*
	 *  Check the first letter of the clue and ask whether the user
	 *  wants it to be uppercased
	 */

	if ( (clue_str[0] >= (char) 'a') && (clue_str[0] <= (char) 'z') )
	{
	    cword_messages_output_message (
		    CWORD_S_FIRSTCHARUPPER,
		    SCREEN0,
		    clue_str[0]);

	    clue_str[0] = clue_str[0] - 32;
	}

    	/*
         *  Check whether the answer is in the clue - give or take
         *  some clue pre-processing!
         */

    	if (cword_word_is_light_in_clue (
    		clue_str,
    		clue_ans) == TRUE)
    	{
	    cword_messages_output_message (
		CWORD_S_ANSINCLUEMSG,
		SCREEN0);

	    /*
	     *  ?? Prompt user whether to keep the clue or not ??
	     */
    	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_clue__get_light_len (
	int			light_len,
	char			*clue_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to help with the logistics of determining the dimensions of
 *	the light which the clue *clue_str is describing.  Once the light data
 *	is obtained it is appended onto the clue_str.
 *
 *  FORMAL PARAMETERS:
 *
 *	light_len		(r) Clue light length is needed for validation
 *	*clue_str		(r/w) clue_str character string to be added to
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Normal completion
 *--
 */

{

    /*
     *	If all is said and done this function shouldn't really exist.  It tries
     *	to close a fairly major loophole/bug in Cword of not being able to
     *	dynamically know what the light consists of in the clue description.
     *
     *	This gap will be filled by allowing the user define what words the
     *	light consists of.  e.g. If the light in question was sonofagun - or
     *	more readably "son of a gun" - then the user will use this function to
     *	define how the light will be solved in the clue description.
     *
     *  Further e.g:
     *
     *     clue description = "bastard (3,2,1,3)"
     *	   clue answer/light = "sonofagun"
     *
     *	This function helps the setter lay out how the "(3,2,1,4)" is seen.
     */

    char temp_str[CWORD_SIZEOF_MESSAGE + 1];

    int count,
	len_array[CWORD_SIZEOF_WORD],
	total_curr_len,
	status;

    BOOLEAN is_zero_entered = FALSE;

    cword_messages_output_message (
	    CWORD_S_CLUELIGHTDIMS,
	    SCREEN0);

    for (count = 0; count < CWORD_SIZEOF_WORD; count++)
    {
	len_array[count] = -1;
    }

    count = 0;
    total_curr_len = 0;
    while (total_curr_len < light_len)
    {
	cword_messages_output_message (
		CWORD_S_CLUEDIMSINPUT,
		SCREEN0,
		(light_len - total_curr_len),
		(light_len - total_curr_len));

	status = cword_charui_get_str (
		    DIGIT,
		    BLANK_OK,
		    sizeof (temp_str) - 1,
		    temp_str);

	if (CWORD_S_NORMAL != status)
	{
	    cword_messages_output_message (
		    CWORD_S_INVALIDCLUEINFO,
		    SCREEN0);

	    len_array[count] = light_len - total_curr_len;

	    cword_messages_output_message (
		    CWORD_S_CLUEDIMSDEF,
		    SCREEN0,
		    len_array[count]);

	    break;
	}

	if (temp_str[0] == (char) 0)
	{
	    /*
	     *  Total length found!
	     */

	    len_array[count] = light_len - total_curr_len;

	    break;
	}
	else
	{
	    if (atoi (temp_str) > (light_len - total_curr_len))
	    {
		/*
		 *  Invalid length entered
		 */

		cword_messages_output_message (
			CWORD_S_CLUEDIMERR,
			SCREEN0);
	    }
	    else if (atoi (temp_str) == 0)
	    {
		/*
		 *  Special case if 0 is entered
		 */

		cword_messages_output_message (
			CWORD_S_ZEROLENCLUE,
			SCREEN0);

		is_zero_entered = TRUE;

		break;
	    }
	    {
		len_array[count] = atoi (temp_str);

		total_curr_len += len_array[count];

		count++;
	    }
	}
    }

    if (is_zero_entered == FALSE)
    {
	count = 0;
	while (len_array[count] != -1)
	{
	    if (count == 0)
	    {
		sprintf (
		    temp_str,
		    " %s%d",
		    LIGHT_OPEN_BRACKET,
		    len_array[count]);
	    }
	    else
	    {
		sprintf (
		    temp_str,
		    "%d",
		    len_array[count]);
	    }

	    strcat (clue_str, temp_str);

	    if (len_array[count + 1] != -1)
		strcat (clue_str, ",");

	    count++;
	}

	strcat (clue_str, LIGHT_CLOSE_BRACKET);
    }

    cword_messages_output_message (
	    CWORD_S_TOTALCLUEDESC,
	    SCREEN0,
	    clue_str);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_clue__update_clue_desc (
        int			clue_number,
	int			clue_direction,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that controls the process of getting and adding the clue
 *	description data to the crossword definition.
 *
 *  FORMAL PARAMETERS:
 *
 *	clue_number		(r) Clue number of clue to add
 *	clue_direction		(r) Clue direction of clue to add
 *	*crossword_record	(r/w) Pointer to CROSSWORD_RECORD
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *	cword_clue_l_defn_updated
 *	cword_clue_l_last_clue_num
 *	cword_clue_l_last_clue_dir
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *      CWORD_S_INVALIDCLUEINFO Clue description found to be invalid
 *--
 */

{

    /*
     *	Use clue_number and clue_direction to reference the crossword record
     */

    char clue_str[CWORD_SIZEOF_CLUE + 1];

    int	index_val,
	status;

    BOOLEAN is_clue_found = FALSE;

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	    CWORD_S_ADDCLUEHEADMSG,
	    SCREEN0);

    for (index_val = 0; index_val < crossword_record->clue_sum; index_val++)
    {
	if ( (crossword_record->clue_details[index_val].clue_number == clue_number) &&
	      (crossword_record->clue_details[index_val].clue_direction == clue_direction) )
	{
	    is_clue_found = TRUE;

	    cword_messages_output_message (
		CWORD_S_PROCESSCLUEMSG,
		SCREEN0,
		crossword_record->clue_details[index_val].clue_number,
		crossword_record->clue_details[index_val].clue_direction 
							  ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

	    cword_messages_output_message (
		CWORD_S_LIGHTCLUEMSG,
		SCREEN0,
		crossword_record->clue_details[index_val].clue_answer);

	    if (strcmp (
		    crossword_record->clue_details[index_val].clue_answer,
		    crossword_record->clue_details[index_val].clue_description) == 0)
	    {
		cword_messages_output_message (
		    CWORD_S_DEFAULTCLUEMSG,
		    SCREEN0,
		    crossword_record->clue_details[index_val].clue_description);
	    }
	    else
	    {
		/*
		 *  ?? If the clue is longer than 50 characters - what then ??
		 */

		cword_messages_output_message (
		    CWORD_S_CURRENTCLUEMSG,
		    SCREEN0,
		    crossword_record->clue_details[index_val].clue_description);
	    }

	    cword_messages_output_message (
		CWORD_S_PROCESSINGCLUE,
		SCREEN0,
		crossword_record->clue_details[index_val].clue_number,
		crossword_record->clue_details[index_val].clue_direction 
                                                      ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

    	    /*
	     *  Tell user to wake up!
	     */

	    cword_charui_ring_bell (1);

	    cword_messages_output_message (
		    CWORD_S_INPUTCLUEMSG,
		    SCREEN0);

    	    /*
	     *	?? Could give the user another chance to input clue if NULL is
	     *	entered first time around ??
      	     */

	    /*
	     *	Only allow a clue of length sizeof(clue_str) - 5 to be entered
	     *	as there has to be some room left for the extra light length
	     *	information to be inputted at the end of the user defined clue.
	     */

	    cword_charui_get_str (
		ALL,
		BLANK_OK,
		sizeof (clue_str) - 5,
		clue_str);

	    break;
	}
    }

    if (is_clue_found != TRUE)
    {
	cword_messages_output_message (
		CWORD_S_NOCLUESFOUNDMSG,
		SCREEN0);

	return (CWORD_S_NORMAL);
    }

    if (clue_str[0] != (char) 0)
    {
	/*
	 *  Check the format of the inputted clue for correctness
	 */

	status = cword_clue__check_clue_desc (
			crossword_record->clue_details[index_val].clue_answer,			
			clue_str);

	if (CWORD_S_NORMAL != status)
	{
	    /*
	     *	Only errors here are concerned with the clue being invalid and
	     *	not with any other sort of error!
	     */

	    return (CWORD_S_INVALIDCLUEINFO);
	}

	status = cword_clue__get_light_len (
		    strlen (crossword_record->clue_details[index_val].clue_answer),
		    clue_str);

	if (CWORD_S_NORMAL == status)
	{
	    cword_messages_output_message (
		    CWORD_S_CLUEVALIDMSG,
		    (TRACE0 | SCREEN0),
		    crossword_record->clue_details[index_val].clue_number,
		    crossword_record->clue_details[index_val].clue_direction 
                                                      ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

	    /*
	     *	Check that clue_str is allowed in clue_description by checking
	     *	its string length against the sizeof clue_description
	     */

	    if (strlen (clue_str) > sizeof (crossword_record->clue_details[index_val].clue_description))
	    {
		printf ("# Possible memory corruption error as strlen (clue_str) > sizeof (clue_desc)\n");
	    }

	    strcpy (
		crossword_record->clue_details[index_val].clue_description,
		clue_str);

	    cword_clue_l_defn_updated = TRUE;
	    cword_clue_l_last_clue_num = crossword_record->clue_details[index_val].clue_number;
	    cword_clue_l_last_clue_dir = crossword_record->clue_details[index_val].clue_direction;
	}
    }
    else
    {
	cword_messages_output_message (
		CWORD_S_CLUELEFTALONE,
		SCREEN0);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_clue__list_clues (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Function to display the clues/light generated to date.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r) Pointer to CROSSWORD_RECORD
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      CWORD_S_NORMAL		Normal completion
 *--
 */

{

    int index_val,
	line_num;

    BOOLEAN are_some_gen = FALSE,
	    are_all_gen = TRUE;

    for (index_val = 0; index_val < crossword_record->clue_sum; index_val++)
    {
	if (strcmp (
		crossword_record->clue_details[index_val].clue_answer,
		crossword_record->clue_details[index_val].clue_description) != 0)
	{
	    are_some_gen = TRUE;
	}
	else
	{
	    are_all_gen = FALSE;
	}

	if ( (are_all_gen == FALSE) && (are_some_gen == TRUE) )
	{
	    break;
	}
    }

    if (are_some_gen == FALSE)
    {
	cword_messages_output_message (
		CWORD_S_NOCLUEYETUSRGEN,
		SCREEN0);

	cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	return (CWORD_S_NORMAL);
    }

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	    CWORD_S_REVIEWCLUESMSG,
	    SCREEN0);

    line_num = 1;
    for (index_val = 0; index_val < crossword_record->clue_sum; index_val++)
    {
	if (strcmp (
		crossword_record->clue_details[index_val].clue_answer,
		crossword_record->clue_details[index_val].clue_description) != 0)
	{
	    if (strlen (crossword_record->clue_details[index_val].clue_description) > 50)
	    {
		cword_messages_output_message (
		    CWORD_S_DISPLAYSOMECLUE,
		    SCREEN0,
		    crossword_record->clue_details[index_val].clue_number,
		    crossword_record->clue_details[index_val].clue_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
		    crossword_record->clue_details[index_val].clue_description,
		    crossword_record->clue_details[index_val].clue_answer);
	    }
	    else
	    {
		cword_messages_output_message (
		    CWORD_S_DISPLAYALLCLUE,
		    SCREEN0,
		    crossword_record->clue_details[index_val].clue_number,
		    crossword_record->clue_details[index_val].clue_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
		    crossword_record->clue_details[index_val].clue_description,
		    crossword_record->clue_details[index_val].clue_answer);
	    }

	    line_num++;

	    /*
	     *	?? Check whether there are extra clues left to be reviewed and
	     *	allow user the option to continue or not if there are ??
	     */

	    (void) cword_charui_check_page_break (
			NOT_FINISHED,
			CWORD_S_PRESSRETURN,
			&line_num);
	}
    }

    if (are_all_gen == FALSE)
    {
	cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);

	cword_messages_output_message (
	    CWORD_S_CLUENOTALLDEFN,
	    SCREEN0);

	cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);
    }

    /*
     *	If line_num not equal to zero then output another pause for
     *	completeness.
     */

    if (line_num != 0)
    {
	cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_clue__view_grid (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    /*
     *	?? Include more sophisticated scheme where the lights and their
     *	locations are also displayed ??
     */

    /*
     *  Nothing much to it or is it?
     */

    cword_charui_display_definition (
	OUTPUT_CROSSWORD,
	crossword_record);

    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

    return (CWORD_S_NORMAL);

}



PRIVATE void cword_clue__get_option (
	char			*option)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that controls the command line input phase of clue generation.
 *
 *  FORMAL PARAMETERS:
 *
 *	*option			(w) Get the option that the user wants processed
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      void
 *--
 */

{

    char temp_ch;

    /*
     *	?? Could have the default action be what the last action was rather
     *	than just having the default action being Add ??
     */

    cword_messages_output_message (
	CWORD_S_CLUECREATECOMM,
	SCREEN0,
	ADD_CHAR);

    temp_ch = cword_charui_get_ch (FALSE);

    if ( (toupper (temp_ch) == ADD_CHAR) || (temp_ch == (char) 0) )
	*option = ADD_CHAR;
    else if (toupper (temp_ch) == QUIT_CHAR)
	*option = QUIT_CHAR;
    else if (toupper (temp_ch) == SAVE_CHAR)
	*option = SAVE_CHAR;
    else if (toupper (temp_ch) == VIEW_CHAR)
	*option = VIEW_CHAR;
    else if (toupper (temp_ch) == LIST_CHAR)
	*option = LIST_CHAR;
    else if (toupper (temp_ch) == RESET_CHAR)
	*option = RESET_CHAR;
    else if (toupper (temp_ch) == HELP_CHAR)
	*option = HELP_CHAR;
    else
    {
	/*
	 *  Do nothing
	 */

	*option = INVALID_CHAR;
    }

    /*
     *  No return value really necessary
     */

    return;

}



/*
 *  ?? Make into a generic container function akin to those alreadt in
 *  generate.c and grid.c ??
 */

PRIVATE void cword_clue__output_help ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	CWORD_S_GENERICHELP,
	SCREEN0);

    printf ("## A - Add a clue description\n");
    printf ("## Q - Quit the clue generation phase\n");
    printf ("## S - Save the crossword description with the clue descriptions added\n");
    printf ("## V - View grid filled with light definitions\n");
    printf ("## L - List clue descriptions that have been user defined\n");
    printf ("## R - Reset description of selected user defined clue\n");
    printf ("## H - Get this screen :-)\n");

    cword_messages_output_message (
	CWORD_WAITINGENDED,
	SCREEN0);

    return;

}



PRIVATE int cword_clue__reset_clue (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to reset the contents of the selected clue to the contents of
 *	the light
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record       (r/w) Pointer to CROSSWORD_RECORD
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_clue_l_defn_downdated
 *      cword_clue_l_last_clue_num
 *      cword_clue_l_last_clue_dir
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal Completion
 *      CWORD_S_FAILURE         Failed to reset clue for some reason 
 *      CWORD_S_INVALIDCLUEINFO Invalid clue info entered
 *--
 */

{

    int next_clue_number,
	next_clue_direction,
	clue_number,
	clue_direction,
	count,
	status;

    BOOLEAN is_reset,
	    is_clue_avail;

    /*
     *	Check if there is anything to reset from the local module globals
     *	footprint.  If nothing is forthcoming there check wherther there are
     *	any clues to reset in the grid definition itself.
     */

    is_clue_avail = FALSE;
    next_clue_number = next_clue_direction = 0;

    if ( (cword_clue_l_last_clue_num == 0) && (cword_clue_l_last_clue_dir == 0) )
    {
	/*
	 *  No clues have been defined by user during the current session.
	 *  Have to check whether the definition has already some clues
	 *  defined.
	 */

	for (count = 0; count < crossword_record->clue_sum; count++)
	{
	    if (strcmp (
		    crossword_record->clue_details[count].clue_answer,
		    crossword_record->clue_details[count].clue_description) != 0)
	    {
		next_clue_number = crossword_record->clue_details[count].clue_number;
		next_clue_direction = crossword_record->clue_details[count].clue_direction;

		is_clue_avail = TRUE;

		break;
	    }
	}
    }
    else
    {
	next_clue_number = cword_clue_l_last_clue_num;
	next_clue_direction = cword_clue_l_last_clue_dir;

	is_clue_avail = TRUE;
    }

    if (is_clue_avail == FALSE)
    {
	cword_messages_output_message (
	    CWORD_S_RESETNONETODO,
	    SCREEN0);

	cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	return (CWORD_S_NORMAL);
    }
    else
    {
	cword_messages_output_message (
	    CWORD_S_RESETCLUEMSG,
	    SCREEN0);

	status = cword_clue__get_clue_coords (
			crossword_record,
			next_clue_number,
			next_clue_direction,
			&clue_number,
			&clue_direction);

	if ( (CWORD_S_FAILURE == status) || (CWORD_S_INVALIDCLUEINFO == status) )
	{
	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    return (status);
	}
    }

    is_reset = FALSE;
    for (count = 0; count < crossword_record->clue_sum; count++)
    {
	if ( (crossword_record->clue_details[count].clue_number == clue_number) &&
	      (crossword_record->clue_details[count].clue_direction == clue_direction) )
	{
	    if (strcmp (
		    crossword_record->clue_details[count].clue_answer,
		    crossword_record->clue_details[count].clue_description) != 0)
	    {
		strcpy (
		    crossword_record->clue_details[count].clue_description,
		    crossword_record->clue_details[count].clue_answer);

		cword_messages_output_message (
		    CWORD_S_RESERCLUESUCC,
		    (TRACE0 | SCREEN0),
		    crossword_record->clue_details[count].clue_number,
		    crossword_record->clue_details[count].clue_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		cword_clue_l_defn_downdated = TRUE;

		if ( (clue_number == cword_clue_l_last_clue_num) &&
		      (clue_direction == cword_clue_l_last_clue_dir) )
		{
		    /*
		     *  If this clue was re-set then it can't be used again!
		     */

		    cword_clue_l_last_clue_num = 0;
		    cword_clue_l_last_clue_dir = 0;
		}

		is_reset = TRUE;

		break;
	    }
	}
    }

    if (is_reset == FALSE)
    {
	cword_messages_output_message (
	    CWORD_S_RESETCLUEISDEFN,
	    SCREEN0);
    }

    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

    return (CWORD_S_NORMAL);

}



/*
 *  ?? MAJOR UPGRADE IN FUNCTIONALITY REQUIRED: how to add in webster and
 *  anagramming abilities to this phase of Cword ??
 */

PUBLIC int cword_clue_fill_desc (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to help the user fill in the clue descriptions for already
 *	filled in grids.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r/w) Pointer to a CROSSWORD_RECORD
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_clue_l_defn_updated
 *	cword_clue_l_defn_downdated
 *	cword_clue_l_defn_full
 *	cword_clue_l_last_clue_num
 *	cword_clue_l_last_clue_dir
 *
 *  IMPLICIT OUTPUTS:
 *
 *	cword_clue_l_defn_updated
 *	cword_clue_l_defn_downdated
 *	cword_clue_l_defn_full
 *	cword_clue_l_last_clue_num
 *	cword_clue_l_last_clue_dir
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully filled
 *	CWORD_S_FAILURE		Unsuccessfully filled
 *--
 */

{

    char option;

    int status,
	clue_number,
	clue_direction;

    static BOOLEAN is_first_time;

    BOOLEAN are_still_clueing = TRUE,
	    is_save_file_ok = FALSE;

    /*
     *	Reset BOOLEAN each time function is executed!
     */

    is_first_time = TRUE;

    /*
     *  Re-set this module global each time this is executed.
     */

    cword_clue_l_defn_updated = FALSE;
    cword_clue_l_defn_downdated = FALSE;
    cword_clue_l_defn_full = FALSE;
    cword_clue_l_last_clue_num = 0;
    cword_clue_l_last_clue_dir = 0;

    CWORD_OS_CLEAR_SCREEN ();

    while (are_still_clueing == TRUE)
    {
	/*
	 *  First output the clueing options to the screen for the user to
	 *  select from
	 */ 

	status = cword_clue__output_clue_opts (crossword_record);

	if (CWORD_S_NORMAL != status)
	{
	    if (CWORD_S_CLUESALLDEFD == status)
	    {
		/*
		 *  Keep going until user is finished and selected Save or Quit!
		 */
	    }
	    else
	    {
		/*
		 *  ?? Other tidying up ??
		 */

		return (CWORD_S_FAILURE);
	    }
	}

	if (is_first_time == TRUE)
	{
	    cword_messages_output_message (
		CWORD_S_HELPAVAILABLE,
		SCREEN0,
		HELP_CHAR);

	    is_first_time = FALSE;
	}

	cword_clue__get_option (&option);

	switch (option)
	{
	case (QUIT_CHAR):
	    /*
	     *	Quit this phase without saving any changes that were made but
	     *	first check if the grid has been altered before exitting.
	     */

	    /*
	     *	?? Quitting here is slightly different than quitting while
	     *	creating a grid.  Maybe I need to make sure than both function
	     *	perform similar functions ??
	     */

	    if ( (cword_clue_l_defn_downdated == TRUE) || 
                  (cword_clue_l_defn_updated == TRUE) )
	    {
		char temp_ch;

		cword_messages_output_message (
			CWORD_S_QUITCHANGESMADE,
			SCREEN0);

		cword_messages_output_message (
			CWORD_S_QUITCONTINUE,
			SCREEN0,
			CWORD_Y_CHAR,
			CWORD_N_CHAR,
			CWORD_Y_CHAR);

		temp_ch = cword_charui_get_ch (FALSE);

		/*
		 *  If CWORD_N_CHAR is entered then we will not quit otherwise
		 *  we do quite without saving if necessary.
		 */

		if (toupper (temp_ch) != CWORD_N_CHAR)
		{
		    cword_messages_output_message (
			    CWORD_S_CLUEQUITUNSAVE,
			    (TRACE0 | SCREEN0));

		    are_still_clueing  = FALSE;
		}
	    }
	    else
	    {
		/*
		 *  Can exit safely without losing any info built up during
		 *  session.
		 */

		cword_messages_output_message (
			CWORD_S_CLUEQUITMSG,
			(SCREEN0 | TRACE0));

		are_still_clueing  = FALSE;
	    }

	    break;

	case (SAVE_CHAR):
	    /*
	     *	Check if the grid has changed before saving?
	     */

	    cword_messages_output_message (
		    CWORD_S_CLUESAVEGENMSG,
		    (SCREEN0 | TRACE0));

	    if ( (cword_clue_l_defn_downdated == TRUE) || 
                  (cword_clue_l_defn_updated == TRUE) )
	    {
		cword_messages_output_message (
			CWORD_S_SAVECLUESMSG,
			(SCREEN0 | TRACE0));

		/*
		 *  ?? More functional response if clues not completely
		 *  generated - maybe have a Y/N with a chance to return back
		 *  to generating ??
		 */

		if (cword_clue_l_defn_full != TRUE)
		{
		    cword_messages_output_message (
			    CWORD_S_CLUESNOTCOMP,
			    (SCREEN0 | TRACE0));
		}

		is_save_file_ok = TRUE;
	    }
	    else
	    {
		cword_messages_output_message (
			CWORD_S_NOTSAVECLUESMSG,
			(SCREEN0 | TRACE0));
	    }

	    are_still_clueing  = FALSE;

	    break;

	case (VIEW_CHAR):
	    status = cword_clue__view_grid (crossword_record);

	    if (CWORD_S_FAILURE == status)
	    {
		/*
		 *  ?? Other tidying up ??
		 */

		return (CWORD_S_FAILURE);
	    }

	    break;

	case (LIST_CHAR):
	    status = cword_clue__list_clues (crossword_record);

	    if (CWORD_S_FAILURE == status)
	    {
		/*
		 *  ?? Other tidying up ??
		 */

		return (CWORD_S_FAILURE);
	    }

	    break;

	case (RESET_CHAR):
	    status = cword_clue__reset_clue (crossword_record);

	    if (CWORD_S_FAILURE == status)
	    {
		/*
		 *  ?? Other tidying up ??
		 */

		return (CWORD_S_FAILURE);
	    }

	    break;

	case (ADD_CHAR):
	    status = cword_clue__get_valid_clue (
			crossword_record,
			&clue_number,
			&clue_direction);

	    if (CWORD_S_FAILURE == status)
	    {
		/*
		 *  ?? Other tidying up ??
		 */

		return (CWORD_S_FAILURE);
	    }
	    else if (CWORD_S_INVALIDCLUEINFO == status)
	    {
		/*
		 *  Do nothing, just continue
		 */
	    }
	    else
	    {
		/*
		 *  Successful status recorded!
		 */

		status = cword_clue__update_clue_desc (
			    clue_number,
			    clue_direction,
			    crossword_record);

		if (CWORD_S_FAILURE == status)
		{
		    /*
		     *  ?? Other tidying up ??
		     */

		    return (CWORD_S_FAILURE);
		}
		else if (CWORD_S_INVALIDCLUEINFO == status)
		{
		    /*
		     *  Do nothing, just continue
		     */
		}
	    }

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    break;

	case (HELP_CHAR): 

	    cword_clue__output_help ();

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    break;

	default:
	    cword_messages_output_message (
			CWORD_CHOICENOTVALID,
			SCREEN0);

	    /*
	     *  ?? Just to see how it works ??
	     */

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    break;
	}
    }

    if (is_save_file_ok == TRUE)
    {
	return (CWORD_S_NORMAL);
    }
    else
    {
	return (CWORD_S_FAILURE);
    }

}



#else

main (
	int			argc, 
	char			*argv[])

{

    return (CWORD_S_NORMAL);

}

#endif

