#!/bin/sh
#
#  FACILITY
#  --------
#
#	cword.sh
#
#  ABSTRACT
#  --------
#
#       Simple Bourne Shell script for building Cword on a U*X system.
#
#	***** Note *****
#       The user may have to 'chmod a+x cword.sh' to get this file working. 
#       The reason for this is that Cword is/was primarily developed on
#       OpenVMS and it is/was not possible to preserve the correct U*X 
#       properties for immediate execution of the file.  It's left to the
#       user as an exercise.
#       ***** Note *****
#
#  AUTHORS
#  -------
#
#	James A. Lundon
#
#  CREATION DATE
#  -------------
#
#	1 March 1993
#
#  Defining some environment variables for compilation.
#
CC=/pub/bin/gcc
CFLAGS="-g -O"
#
#  Output each command to stdout while it's being executed.
#
set -x
#
#  Actual compilation of sources.
#
$CC $CFLAGS -c print.c -o print.o
$CC $CFLAGS -c fileio.c -o fileio.o
$CC $CFLAGS -c utils.c -o utils.o
$CC $CFLAGS -c generate.c -o generate.o
$CC $CFLAGS -c match.c -o match.o
$CC $CFLAGS -c grid.c -o grid.o
$CC $CFLAGS -c word.c -o word.o
$CC $CFLAGS -c charui.c -o charui.o
$CC $CFLAGS -c verify.c -o verify.o
$CC $CFLAGS -c soundex.c -o soundex.o
$CC $CFLAGS -c messages.c -o messages.o
$CC $CFLAGS -c webster.c -o webster.o
$CC $CFLAGS -c stats.c -o stats.o
$CC $CFLAGS -c memman.c -o memman.o
$CC $CFLAGS -c unix.c -o unix.o
$CC $CFLAGS -c anagram.c -o anagram.o
$CC $CFLAGS -c charui.c -o charui.o 
$CC $CFLAGS -c defaults.c -o defaults.o 
$CC $CFLAGS -c clue.c -o clue.o 
$CC $CFLAGS -c seed.c -o seed.o 
$CC $CFLAGS -c breakdic.c -o breakdic.o 
$CC $CFLAGS -c cl.c -o cl.o 
$CC $CFLAGS -c strll.c -o strll.o 
$CC $CFLAGS -c main.c -o main.o
#
#  Cword is the main UI in which most - if not all - of the work will be
#  completed by the user.
#
$CC $CFLAGS -o cword main.o strll.o cl.o breakdic.o seed.o clue.o defaults.o \
                     charui.o anagram.o unix.o memman.o stats.o \
                     webster.o messages.o soundex.o verify.o word.o \
                     grid.o match.o generate.o utils.o fileio.o print.o \
            -lm
#
#  breakdic is used to break up an input ASCII dictionary into a series of 
#  word files compatible to cword.  
#
$CC $CFLAGS -DMAIN breakdic.c -o breakdic
#
#  Disable outputting of command line options.
#
set -
#
#  Check that the Cword executables were built correctly.
#
if [ -f cword -a -f breakdic ]
then
   echo "** You've successfully built both Cword and breakdic **"
#
#  Delete all object files lying around after the compilation - if the 
#  compilation has gone successfully.
#
   rm -f *.o
else
   echo "**** There were problems compiling either Cword or breakdic or both *****"
fi
