$!
$!  FACILITY :
$!
$!      cwordkit.com
$!
$!  ABSTRACT :
$!
$!      DCL command file to build Cword into a series of
$!      mailable UUencoded, gzipped/compressed, tar files!
$!
$!      The following symbols have to be defined before this
$!      command file will work: SPLITFILE, VMS2TAR, GZIP,
$!      COMPRESS and UUENCODE.
$!
$!  AUTHORS :
$!
$!      James A. Lundon
$!
$!  CREATION DATE :
$!
$!      10 May 1994
$!
$  set control=(t,y)
$!
$!  On control or error then exit!
$!
$  on control then goto abort_cwordkit
$  on error then goto error_cwordkit
$  on warning then continue
$!
$  build_dir = "CWORD$ROOT:[KIT.build]"
$  final_dir = "CWORD$ROOT:[KIT.final]"
$  work_dir = "CWORD$ROOT:[000000]"
$!
$  purge :== purge
$!
$!  ?? All questions at the beginning of the command file ??
$!
$  inquire buildkit "Are you sure you want to build Cword [Y/N] [N]"
$  if (buildkit .eqs. "Y" .or. buildkit .eqs. "y")
$  then
$      write sys$output "CWORD-I-BUILD, Building Cword kit from sources"
$  else
$      write sys$output "CWORD-I-NBUILD, Not building Cword kit"
$      exit 5
$  endif
$!
$  start_time = f$time ()
$  write sys$output "CWORD-I-STARTT, Time Cword kit build started was ", start_time
$!
$  set def 'build_dir
$  define in_build_dir "1"
$  write sys$output "CWORD-W-DELETING, Deleting contents of ''build_dir directory"
$  delete/noconfirm/nolog *.*;*
$!
$!  ?? Build in the concept of building from a particular CMS class ??
$!
$  write sys$output "CWORD-I-REFERS, Copying Cword sources from reference directory"
$  copy cword_sources:*.* []/log
$!
$!  Copy the current sources from the working directory into 
$!  the build kit directory - if required?
$!
$  inquire option "Get sources from working directory [Y/N] [N]"
$  if (option .eqs. "Y" .or. option .eqs. "y")
$  then
$      write sys$output "CWORD-I-COPYNG, Copying sources from working directory"
$      on error then continue
$!
$!  Other files that are currently in the working directory may get 
$!  excluded from this list.  However, it's better to have files 
$!  exclued than include files that should not be in the distribution at 
$!  all.  It might get people confused :-).
$!
$      write sys$output "CWORD-W-FILES, Warning: only copying *.c, *.h, *.sc - may find no files"
$      copy 'work_dir'*.c, 'work_dir'*.h, 'work_dir'*.sc []/log
$      on error then goto error_cwordkit
$      write sys$output "CWORD-I-PURGNG, Purging build directory"
$      purge/log
$  endif
$!
$  write sys$output "CWORD-I-RENAMG, Renaming all sources to *.*;1"
$  rename/nolog/noconfirm *.* *.*;1
$!
$!  Tar the sources into one large blob for future mailing.
$!
$  write sys$output "CWORD-I-TARRNG, Creating tar file of Cword sources"
$!
$!  ?? Store the build number AKA Microsoft and add it into the sources.
$!  This will be a lot of extra work but might be worth the effort in 
$!  the long run ??
$!
$!  ?? Grab the current version of Cword to create tar file with from 
$!  where ??
$!  
$!  ?? 010 could be a little confusing as it could signify either V1.0 
$!  or what I want it to signify V0.10 ??
$!
$!  ?? Automate this process by putting Cword's current version ID into 
$!  a separate file which is then parsed by this command procedure to
$!  obtain a value for version ??
$!
$!  ?? Do straight gzip of the sources ??
$!
$  version = "010"
$  define tape 'final_dir'cword'version'.tar
$  vms2tar cv
$!
$  set default 'final_dir
$!
$!  Compress/gzip the tarred glob.
$!
$  write sys$output " "
$  inquire compression "Select compression type [1. compress - 2. gzip] [1]"
$  if (compression .eqs. "2")
$  then
$      write sys$output "CWORD-I-GZIPPG, Creating gzip file of Cword sources"
$!
$!  Will create a file called cword'version.tar-GZ
$!
$      if (f$search("cword''version'.tar-GZ") .nes. "")
$      then
$          delete/log cword'version.tar-GZ;*
$      endif
$      gzip cword'version.tar 
$  else
$      write sys$output "CWORD-I-COMPRG, Creating compressed file of Cword sources"
$!
$      if (f$search("cword''version'.tar-Z") .nes. "")
$      then
$          delete/log cword'version.tar-Z;*
$      endif
$      compress cword'version.tar cword'version.tar-Z
$  endif
$!
$!  UUencode the compressed tarred glob.
$!
$!  The only way of knowing which compression algorithm was used is by 
$!  looking at the first line of the UUencoded file :-).
$!
$  write sys$output "CWORD-I-UUENNG, Creating UUencoded file of Cword sources"
$  if (compression .eqs. "2")
$  then
$      uuencode cword'version.tar-GZ "cword''version'.tar.gz" cword'version.GZ-uue
$  else
$      uuencode cword'version.tar-Z "cword''version'.tar.Z" cword'version.Z-uue
$  endif
$!
$!  Split the UUencoded compressed/gzipped tarred glob into a series
$!  of files that any E-mailer can handle.
$!
$  write sys$output "CWORD-I-SPLITG, Creating mail files of Cword sources"
$  if (compression .eqs. "2")
$  then
$!
$      splitfile cword'version.GZ-uue 2000
$  else
$      splitfile cword'version.Z-uue 3000
$  endif
$!
$  write sys$output "CWORD-I-FINALF, Directory listing of final Cword kit files"
$!
$  if (compression .eqs. "2")
$  then
$      dir 0*.GZ-uue
$  else
$      dir 0*.Z-uue
$  endif
$!
$  set default 'build_dir
$!
$!  Now deal with the grid and crossword files separate to the rest of the kit.
$!
$  write sys$output "CWORD-I-UUENNG, Creating UUencoded file of grd/xrd files"
$  copy cword_grdsxrds:*.* []/log
$  write sys$output "CWORD-I-RENAMG, Renaming all grds/xrds to *.*;1"
$  rename/nolog/noconfirm *.grd *.grd;1
$  rename/nolog/noconfirm *.xrd *.xrd;1
$!
$  define tape 'final_dir'grdsxrds.tar
$  vms2tar cv *.grd,*.xrd
$!
$  set default 'final_dir
$!
$  if (compression .eqs. "2")
$  then
$      if (f$search("grdsxrds.tar-GZ") .nes. "")
$      then
$          delete/log grdsxrds.tar-GZ;*
$      endif
$      gzip grdsxrds.tar
$      uuencode grdsxrds.tar-GZ "grdsxrds.tar.gz" grdsxrds.GZ-uue
$  else
$      if (f$search("grdsxrds.tar-Z") .nes. "")
$      then
$          delete/log grdsxrds.tar-Z;*
$      endif
$      compress grdsxrds.tar grdsxrds.tar-Z
$      uuencode grdsxrds.tar-Z "grdsxrds.tar.Z" grdsxrds.Z-uue
$  endif
$!
$  write sys$output "CWORD-I-FINALF, Directory listing of final grd/xrd kit files"
$  if (compression .eqs. "2")
$  then
$      dir grdsxrds.GZ-uue
$  else
$      dir grdsxrds.Z-uue
$  endif
$!
$  set def [-]
$  define in_build_dir "0"
$!
$  write sys$output "CWORD-I-PURGEF, Purging files in ''final_dir'"
$  purge/log 'final_dir*.*
$  rename 'final_dir*.* 'final_dir*.*;1
$!
$  end_time = f$time ()
$  write sys$output "CWORD-S-FINISH, Time Cword build kit finished successfully ", end_time
$  exit
$!
$abort_cwordkit:
$!
$  if (f$trnlnm("in_build_dir") .eqs. "1") then set def [-]
$  end_time = f$time ()
$  write sys$output "CWORD-W-ABORTD, Time Cword build kit was user aborted ", end_time
$  exit 5
$!
$error_cwordkit:
$!
$  if (f$trnlnm("in_build_dir") .eqs. "1") then set def [-]
$  end_time = f$time ()
$  write sys$output "CWORD-E-ERRORD, Time Cword build kit exitted with error ", end_time
$  exit 5
