/*
 *++
 *  FACILITY:
 *
 *      defaults.c
 *
 *  ABSTRACT:
 *
 *	This module contains all the defaults file processing functionality.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      16 April 1994
 *--
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#if defined(__unix) || defined(__unix__)

#include <sys/file.h>
#include <sys/uio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>

#endif

#if defined(VMS) || defined(__VMS)

#include <unixio.h>
#include <file.h>

#endif

#if defined(__MSDOS__) || defined(MSDOS)

#include <io.h>
#include <conio.h>
#include <fcntl.h>
#include <sys\stat.h>

#endif

#ifdef DEFAULTS
#define MESSAGES_DEFINED 1 
#endif

#include "main.h"
#include "grid.h"
#include "generate.h"
#include "soundex.h"
#include "utils.h"
#include "messages.h"
#include "fileio.h"
#include "word.h"
#include "memman.h"
#include "match.h"
#include "strll.h"
#include "defaults.h"

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#ifndef DEFAULTS

/*
 *  Remember to be careful where to place this macro!
 */

#define CWORD_DEFAULTS_READ_ERROR(message, file_name, file_id) \
	    cword_messages_output_message ( \
			message, \
			(TRACE2 | SCREEN0), \
			file_name); \
	    close (file_id); \
	    return (CWORD_S_FAILURE)

/*
 *  Private function prototypes
 */

PRIVATE void cword_defaults__set_defaults ();

PRIVATE int cword_defaults__parse_line (
	char			*defaults_str,
	int			sizeof_return_str,
	char			*return_str);

/*
 *  Module global telling whether the defaults file has already been read or
 *  not.
 */

static cword_l_defaults_file_read = FALSE;



/*
 *  ?? Is this function needed at all as globals are automatically initialised
 *  to 0 upon creation ??
 */

PUBLIC void cword_defaults_init_defaults ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      cword_g_defaults_record
 *      cword_g_defaults_record.trace_level
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    /*
     *	memset the global defaults data structure before doing anything else.
     */

    memset (
	&cword_g_defaults_record, 
	0, 
	sizeof (CWORD_G_DEFAULTS_RECORD));

    /*
     *	Breaking the rule!  This default will be re-set back to its proper
     *	default value in cword_defaults__set_defaults but this code is
     *	necessary to allow certain output messages be seen (-default, -max,
     *	-min, -centre).
     */

    cword_g_defaults_record.trace_level = LEVEL3;
    cword_g_defaults_record.ring_bell = TRUE;

    return;

}



PRIVATE void cword_defaults__set_defaults ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function used to insert default values into the global default options
 *	record before more specific one are read from the defaults file.
 *
 *	With each added default option added to the default options file the
 *	"default" default should be also added in here.
 *
 *  FORMAL PARAMETERS:
 *
 *      None
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record
 *
 *  IMPLICIT OUTPUTS:
 *
 *      cword_g_defaults_record
 *
 *  FUNCTION VALUE:
 *
 *      None
 *--
 */

{

    int count;

    cword_messages_output_message (
		CWORD_DEFAULTSDEFINE,
		TRACE2);

    cword_g_defaults_record.x_dimensions = NORMAL_GRID_DIMS;
    cword_g_defaults_record.y_dimensions = NORMAL_GRID_DIMS;
    cword_g_defaults_record.symmetry_type = NO_SYMMETRY;
    cword_g_defaults_record.symmetry_check = FALSE;
    cword_g_defaults_record.unches_check = FALSE;
    cword_g_defaults_record.smallest_allowable_light = CWORD_SIZEOF_MIN_LIGHT;
    cword_g_defaults_record.automatic_light_generation = TRUE;
    cword_g_defaults_record.soundex_algorithm = SOUNDEX_MATCH_START;
    cword_g_defaults_record.words_per_dot = CWORD_SIZEOF_MAX_WORDS_PER_DOT;
    cword_g_defaults_record.write_log_file = FALSE;
    cword_g_defaults_record.max_light_choices = CWORD_SIZEOF_MAX_ARRAY_SIZE;
    cword_g_defaults_record.light_generation_algorithm = RANDOMWORDS;
    cword_g_defaults_record.check_spelling = TRUE;
    cword_g_defaults_record.trace_level = LEVEL1;
    cword_g_defaults_record.max_matches = CWORD_SIZEOF_MAX_ARRAY_SIZE;
    cword_g_defaults_record.max_light_iterations = CWORD_SIZEOF_MAX_LIGHT_ITER;
    cword_g_defaults_record.letter_weight_algorithm = CWORD_SIZEOF_DEF_WGHT_ALGO;
    cword_g_defaults_record.nextlight_select_algorithm = BEST_LIGHT;    
    cword_g_defaults_record.fill_from_where = FILL_FROM_WORD_FILE;
    cword_g_defaults_record.reset_after_num_iterations = CWORD_SIZEOF_RESET_FILLER;
    cword_g_defaults_record.webster_server_port = XWEBSTER_PORT;
    cword_g_defaults_record.ring_bell = TRUE;

    /*
     *	This is a simple array of masks which will be used to verify the
     *	existence of the component word files.
     *
     *	Word files existance will be from word_file_access[1] (for one letter
     *	words) to word_file_access[CWORD_SIZEOF_WORD] (for CWORD_SIZEOF_WORD
     *	letter words).
     */

    for (count = 0; count < CWORD_SIZEOF_WORD + 1; count++)
    {
	/*
	 *  From now on I will not be using word_file_access[0] (formerly
	 *  dict file mask).
	 */

	cword_g_defaults_record.word_file_access[count] = FALSE;
    }

    /*
     *	If defaults_file_spec isn't already defined (via the command line) then
     *	substitute in the default default file specification and continue as
     *	usual.
     */

    if (cword_g_defaults_record.defaults_file_spec[0] == (char) 0)
    {
	strcpy (cword_g_defaults_record.defaults_file_spec, CWORD_DEFAULTS_FILENAME);
    }

    strcpy (cword_g_defaults_record.temp_grid_name, GRID_NAME);
    strcpy (cword_g_defaults_record.temp_crossword_name, CROSSWORD_NAME);

    /*
     *	?? If valid word files prefixes are found in the defaults file then
     *	this list is first freed and then added to with the values found in the
     *	user defaults file ??
     */

    cword_g_defaults_record.word_file_prefix_list = (CWORD_T_STR_LL *) 0;

    cword_strll_add_str (
	SORTED | UNIQUE,
	WORDFILE_NAME,
	&cword_g_defaults_record.word_file_prefix_list);

    if (cword_g_defaults_record.word_file_prefix_list == (CWORD_T_STR_LL *) 0)
    {
	/*
	 *  ?? Output message and return error ??
	 */
    }

    strcpy (cword_g_defaults_record.webster_server_name, XWEBSTER_IP_ADDR);
    strcpy (cword_g_defaults_record.log_file_prefix, CWORD_LOG_FILE_PREFIX);
    strcpy (cword_g_defaults_record.username, CWORD_DEFAULT_USERNAME);

    return;

}



/*
 *  ?? Check whether a word file set is incorrect or missing from a set of two
 *  of more series of word files ??
 */

PUBLIC void cword_defaults_output_defaults ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to output the defaults found either in the defaults.dat file
 *	or just Cword defaults to the trace file.
 *
 *  FORMAL PARAMETERS:
 *
 *      None
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      void
 *--
 */

{

    CWORD_T_STR_LL *temp_ll,
		   *top_ll;

    char tempfile_str[CWORD_SIZEOF_FILESPEC + 1];

    int	count;

    BOOLEAN is_word_file_found;

    if (cword_l_defaults_file_read == FALSE)
    {
	cword_messages_output_message (
	    CWORD_USESYSTEMDEFAULTS,
	    (SCREEN0 | TRACE0));
    }
    else
    {
	cword_messages_output_message (
	    CWORD_USEUSERDEFAULTS,
	    (SCREEN0 | TRACE0),
	    cword_g_defaults_record.defaults_file_spec);
   }

    WRITE_TRACE ("## x_dimensions %d\n", cword_g_defaults_record.x_dimensions);
    WRITE_TRACE ("## y_dimensions %d\n", cword_g_defaults_record.y_dimensions);
    WRITE_TRACE ("## symmetry_type %d\n", cword_g_defaults_record.symmetry_type);
    WRITE_TRACE ("## symmetry_check %d\n", cword_g_defaults_record.symmetry_check);
    WRITE_TRACE ("## unches_check %d\n", cword_g_defaults_record.unches_check);
    WRITE_TRACE ("## smallest_allowable_light %d\n", cword_g_defaults_record.smallest_allowable_light);
    WRITE_TRACE ("## automatic_light_generation %d\n", cword_g_defaults_record.automatic_light_generation);
    WRITE_TRACE ("## soundex_algorithm is %d\n", cword_g_defaults_record.soundex_algorithm);
    WRITE_TRACE ("## words_per_dot is %d\n", cword_g_defaults_record.words_per_dot);
    WRITE_TRACE ("## write_log_file is %d\n", cword_g_defaults_record.write_log_file);
    WRITE_TRACE ("## max_light_choices is %d\n", cword_g_defaults_record.max_light_choices);
    WRITE_TRACE ("## light_generation_algorithm is %d\n", cword_g_defaults_record.light_generation_algorithm);
    WRITE_TRACE ("## check_spelling is %d\n", cword_g_defaults_record.check_spelling);
    WRITE_TRACE ("## trace_level is %d\n", cword_g_defaults_record.trace_level);
    WRITE_TRACE ("## max_matches is %d\n", cword_g_defaults_record.max_matches);
    WRITE_TRACE ("## max_light_iterations is %d\n", cword_g_defaults_record.max_light_iterations);
    WRITE_TRACE ("## letter_weight_algorithm is %d\n", cword_g_defaults_record.letter_weight_algorithm);
    WRITE_TRACE ("## nextlight_select_algorithm is %d\n", cword_g_defaults_record.nextlight_select_algorithm);
    WRITE_TRACE ("## fill_from_where is %d\n", cword_g_defaults_record.fill_from_where);
    WRITE_TRACE ("## reset_after_num_iterations is %d\n", cword_g_defaults_record.reset_after_num_iterations);
    WRITE_TRACE ("## webster_server_port is %d\n", cword_g_defaults_record.webster_server_port);
    WRITE_TRACE ("## ring_bell is %d\n", cword_g_defaults_record.ring_bell);

    WRITE_TRACE ("## defaults_file_spec is %s\n", cword_g_defaults_record.defaults_file_spec);
    WRITE_TRACE ("## temp_grid_name %s\n", cword_g_defaults_record.temp_grid_name);
    WRITE_TRACE ("## temp_crossword_name %s\n", cword_g_defaults_record.temp_crossword_name);

    temp_ll = cword_g_defaults_record.word_file_prefix_list;
    count = 0;

    while (temp_ll != (CWORD_T_STR_LL *) 0)
    {
	count++;

	WRITE_TRACE (
	    "## word_file_prefix[%d] %s\n",
	    count,
	    temp_ll->str);

	temp_ll = temp_ll->next;
    }

    WRITE_TRACE ("## webster_server_name %s\n", cword_g_defaults_record.webster_server_name);
    WRITE_TRACE ("## log_file_prefix %s\n", cword_g_defaults_record.log_file_prefix);
    WRITE_TRACE ("## username %s\n", cword_g_defaults_record.username);

    /*
     *	Taking out word file prefixes that have no associated files!
     */

    top_ll = cword_g_defaults_record.word_file_prefix_list;
    temp_ll = (CWORD_T_STR_LL *) 0;

    while (top_ll != (CWORD_T_STR_LL *) 0)
    {
	cword_strll_add_str_and_val (
	    UNSORTED | MULTIPLE,
	    top_ll->str_val,
	    top_ll->str,
	    &temp_ll);

	top_ll = top_ll->next;
    }

    top_ll = cword_g_defaults_record.word_file_prefix_list;
    while (top_ll != (CWORD_T_STR_LL *) 0)
    {
	if (cword_match_verify_file_prefix (top_ll->str) == FALSE)
	{
	    cword_messages_output_message (
		CWORD_S_NOWORDFILEFND,
		(TRACE0 | SCREEN0),
		top_ll->str);

	    if (cword_strll_del_str (
		    top_ll->str,
		    &temp_ll) != CWORD_S_NORMAL)
	    {
		printf (
		    "## Error deleting %s from word file prefix list\n",
		    top_ll->str);
	    }
	}

	top_ll = top_ll->next;
    }

    cword_strll_free_str_ll (&cword_g_defaults_record.word_file_prefix_list); 

    cword_g_defaults_record.word_file_prefix_list = temp_ll;

    /*
     *	?? Slight coupling problem here as we are also determining the values
     *	of some of cword's global variables (_g_.word_file_access).  I think
     *	it's better to leave them here for the time being as the coupling would
     *	be worse if I put them in the function which is used to determine all
     *	the other values in cword_g_defaults_record ??
     */

    /*
     *	Fill out word_file_access file masks and print any problems found!
     */

    /*
     *	?? Need to create a function from this code as it can also be used in
     *	charui.c (Alter Defaults) ??
     */

    for (count = 1; count < CWORD_SIZEOF_WORD + 1; count++)
    {
	while (TRUE == cword_match_get_word_file_name (
			    count,
			    tempfile_str))
	{
	    if (FALSE == cword_utils_does_file_exist (tempfile_str))
	    {
		cword_messages_output_message (
		    CWORD_WORDNOTEXIST,
		    TRACE2,
		    tempfile_str);
	    }
	    else
	    {
		cword_g_defaults_record.word_file_access[count] = TRUE;
	    }
	}
    }

    cword_match_get_word_file_name (RESET, 0);

    is_word_file_found = FALSE;
    for (count = 1; count < CWORD_SIZEOF_WORD + 1; count++)
    {
	if (cword_g_defaults_record.word_file_access[count] == TRUE)
	{
	    is_word_file_found = TRUE;
	    break;
	}
    }

    if (is_word_file_found == FALSE)
    {
	/*
	 *  ?? Beef this up a bit as this could be a very common error for new
	 *  users ??
	 */

	cword_messages_output_message (
	    CWORD_CHECKWORDDEFAULT,
	    (TRACE0 | SCREEN0));
    }

    return;

}



PUBLIC void cword_defaults_free_defaults ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None.
 *--
 */

{

    /*
     *  Free linked list of word_file prefixes.
     */

    cword_strll_free_str_ll (&cword_g_defaults_record.word_file_prefix_list);

    return;

}



/*
 *  ?? Loosen up defaults file processing to allow the following possible
 *  formats ??
 *
 *  X_DIMENSIONS	15	- sscanf (str, "%s %s", str1, str2);
 *
 *  or
 *
 *  X_DIMENSIONS = 15		- sscanf (str, "%s = %s", str1, str2);
 */

PRIVATE int cword_defaults__parse_line (
	char			*defaults_str,
	int			sizeof_return_str,
	char			*return_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Simple function to parse an input line from the Cword defaults file.
 *	It return via return_str the string to the right of the equals sign.
 *
 *  FORMAL PARAMETERS:
 *
 *	*defaults_str		(r) String containing line from defaults file.
 *	sizeof_return_str	(r) sizeof (*return_str).
 *	*return_str		(w) Return string contains string to the right of '=' sign.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *--
 */

{

    char *str_ptr = (char *) 0;

    int count;

    memset (
	return_str, 0,
	sizeof_return_str);

    /*
     *	This assumes the simple format DEFAULT_OPTION=DEFAULT_VALUE
     *
     *  Nothing else is supported yet!
     */

    str_ptr = strstr (defaults_str, "=");

    if (str_ptr == (char *) 0)
    {
	cword_messages_output_message (
	    CWORD_DEFREADERR,
	    (SCREEN0 | TRACE0));

	return (CWORD_S_FAILURE);
    }

    str_ptr++;

    count = 0;
    while (str_ptr[count] != (char) 0)
    {
	return_str[count] = str_ptr[count];
	count++;
    }

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_defaults_read_defaults ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to read the defaults file and stuff the data found into a
 *	globally accessible data structure.  The defaults file contains all the
 *	default setting which are of vital importance in the correct operation
 *	of cword.
 *
 *	I think it's essential to keep a direct mapping between the variable
 *	names in the defaults record (cword_g_defaults_record) and the
 *	character strings searched for in the defaults file.  Maybe an example
 *	will be more illustrative. e.g.
 *
 *	in the defaults file:
 *
 *	    X_DIMENSIONS=15
 *
 *	in the defaults structure after this function:
 *
 *	    cword_g_defaults_record.x_dimensions equals 15
 *
 *  FORMAL PARAMETERS:
 *
 *	None.
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record
 *	cword_l_defaults_file_read
 *
 *  IMPLICIT OUTPUTS:
 *
 *      cword_g_defaults_record
 *	cword_l_defaults_file_read
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully read defaults file.
 *	CWORD_S_FAILURE		Error with defaults file.
 *--
 */

{

    char defaults_str[CWORD_SIZEOF_MESSAGE+ 1],
	 temp_str[CWORD_SIZEOF_MESSAGE + 1];

    int defaults_file,
	read_status,
	temp_int,
	status;

    BOOLEAN is_word_file_prefix_found = FALSE;

    if (cword_l_defaults_file_read == TRUE)
    {
	/*
	 *  The defaults file has already been read as part of command line
	 *  processing.
	 */

	return (CWORD_S_NORMAL);
    }

    cword_defaults__set_defaults ();

    defaults_file = open (
			cword_g_defaults_record.defaults_file_spec, 
			O_RDONLY,
			777);

    if (defaults_file == EOF)
    {
	DEBUG_TRACE (
	    "Defaults file: %s not found\n", 
	    cword_g_defaults_record.defaults_file_spec);

	/*
	 *  NULLifying this variable as the file wasn't found!
	 */

	cword_g_defaults_record.defaults_file_spec[0] = (char) 0;

	CWORD_DEFAULTS_READ_ERROR(
	    CWORD_FILEREADERROR, 
	    cword_g_defaults_record.defaults_file_spec, 
	    defaults_file);
    }

    while ((read_status = cword_fileio_read_line (
    			defaults_file, 
    			defaults_str, 
    			sizeof (defaults_str))) > 0)
    {
	memset (temp_str, 0, sizeof (temp_str));

	/*
	 *  This will skip blank lines *or* lines starting with a '!', 
    	 *  a '#' or a ';' character.
	 */

    	/*
         *  ?? Define the characters that are allowable as comment
         *  characters ??
    	 */

	if ( (defaults_str[0] == '!') ||
	      (defaults_str[0] == '#') ||
	       (defaults_str[0] == ';') ||
	        (defaults_str[0] == (char) 0) )
	    continue;

	if (strstr (defaults_str, "SYMMETRY_TYPE") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if (temp_int == 1)
		    cword_g_defaults_record.symmetry_type = DEG_90_SYMMETRY;
		else if (temp_int == 2)
		    cword_g_defaults_record.symmetry_type = DEG_180_SYMMETRY;
		else if (temp_int == 3)
		    cword_g_defaults_record.symmetry_type = AXIAL_LR_SYMMETRY;
		else if (temp_int == 4)
		    cword_g_defaults_record.symmetry_type = AXIAL_RL_SYMMETRY;
		else if (temp_int == 5)
		    cword_g_defaults_record.symmetry_type = NO_SYMMETRY;
		else
		    cword_g_defaults_record.symmetry_type = DEG_90_SYMMETRY;
	    }
	}
	else if (strstr (defaults_str, "SYMMETRY_CHECK") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		cword_word_upper_str (temp_str);

		if (strcmp (temp_str, CWORD_TRUE_STR) == 0)
		    cword_g_defaults_record.symmetry_check= TRUE;
		else if (strcmp (temp_str, CWORD_FALSE_STR) == 0)
		    cword_g_defaults_record.symmetry_check= FALSE;
		else
		    cword_g_defaults_record.symmetry_check= TRUE;
	    }
	}
	else if (strstr (defaults_str, "UNCHES_CHECK") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		cword_word_upper_str (temp_str);

		if (strcmp (temp_str, CWORD_TRUE_STR) == 0)
		    cword_g_defaults_record.unches_check = 1;
		else if (strcmp (temp_str, CWORD_FALSE_STR) == 0)
		    cword_g_defaults_record.unches_check = 0;
		else
		    cword_g_defaults_record.unches_check = 1;
	    }
	}
	else if (strstr (defaults_str, "X_DIMENSIONS") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		if (temp_str[0] != (char) 0)
		    cword_g_defaults_record.x_dimensions = atoi (temp_str);
	    }
	}
	else if (strstr (defaults_str, "Y_DIMENSIONS") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		if (temp_str[0] != (char) 0)
		    cword_g_defaults_record.y_dimensions = atoi (temp_str);
	    }
	}
	else if (strstr (defaults_str, "SMALLEST_ALLOWABLE_LIGHT") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if (temp_int > 0)
		    cword_g_defaults_record.smallest_allowable_light = temp_int;
	    }
	}
	else if (strstr (defaults_str, "AUTOMATIC_LIGHT_GENERATION") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		cword_word_upper_str (temp_str);

		if (strcmp (temp_str, CWORD_TRUE_STR) == 0)
		    cword_g_defaults_record.automatic_light_generation = TRUE;
		else if (strcmp (temp_str, CWORD_FALSE_STR) == 0)
		    cword_g_defaults_record.automatic_light_generation = FALSE;
		else
		    cword_g_defaults_record.automatic_light_generation = TRUE;
	    }
	}
	else if (strstr (defaults_str, "TEMP_GRID_NAME") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		if (temp_str[0] != (char) 0)
		    strcpy (cword_g_defaults_record.temp_grid_name, temp_str); 
	    }
	}
	else if (strstr (defaults_str, "TEMP_CROSSWORD_NAME") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		if (temp_str[0] != (char) 0)
		    strcpy (cword_g_defaults_record.temp_crossword_name, temp_str); 
	    }
	}
	else if (strstr (defaults_str, "WORD_FILE_PREFIX") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		if (temp_str[0] != (char) 0)
		{
		    if (is_word_file_prefix_found == FALSE)
		    {
			is_word_file_prefix_found = TRUE;

			/*
			 *  Delete the default word_file_prefix list I'd set up
			 *  before starting a news one!
			 */

			cword_strll_free_str_ll (&cword_g_defaults_record.word_file_prefix_list);
		    }

		    DEBUG_TRACE (
			"## Add %s to word_file_prefix_list\n",
			temp_str);

		    /*
		     *	?? Use a comma delimied list mechanism to define the
		     *	word file prefixes ??
		     */

		    /*
		     *	?? Before adding the word_file_prefix I have to append
		     *	the absolute pathname onto the file as CD is now
		     *	supported in the UI ??
		     *
		     *	?? relative pathnames are possible if the following
		     *	rules are observed ??:
		     *
		     *	1. On Unix, if the first character of the
		     *	word_file_prefix isn't '/ then append the current
		     *	working directory onto the file_prefix as found in
		     *	cword.ini.
		     *
		     *	2. on VMS, if the file_prefix doesn't contain either
		     *	':', '$', '[', or ']' append the current working
		     *	directory onto the file_prefix as found in cword.ini.
		     */

		    cword_strll_add_str (
			SORTED | UNIQUE,
			temp_str,
			&cword_g_defaults_record.word_file_prefix_list);
		}
	    }
	}
	else if (strstr (defaults_str, "SOUNDEX_ALGORITHM") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if (temp_int == 1)
		    cword_g_defaults_record.soundex_algorithm = SOUNDEX_MATCH_START;
		else if (temp_int == 2)
		    cword_g_defaults_record.soundex_algorithm = SOUNDEX_MATCH_ALL;
	    }
	}
	else if (strstr (defaults_str, "WORDS_PER_DOT") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if (temp_int > CWORD_SIZEOF_MAX_WORDS_PER_DOT)
		    cword_g_defaults_record.words_per_dot = CWORD_SIZEOF_MAX_WORDS_PER_DOT;
		else if (temp_int < CWORD_SIZEOF_MIN_WORDS_PER_DOT)
		    cword_g_defaults_record.words_per_dot = CWORD_SIZEOF_MIN_WORDS_PER_DOT;
		else
		    cword_g_defaults_record.words_per_dot = temp_int;
	    }
	}
	else if (strstr (defaults_str, "WRITE_LOG_FILE") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		cword_word_upper_str (temp_str);

		if (strcmp (temp_str, CWORD_TRUE_STR) == 0)
		    cword_g_defaults_record.write_log_file = TRUE;
		else
		    cword_g_defaults_record.write_log_file = FALSE;
	    }
	}
	else if (strstr (defaults_str, "WEBSTER_SERVER_NAME") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		if (temp_str[0] != (char) 0)
		    strcpy (cword_g_defaults_record.webster_server_name, temp_str); 
	    }
	}
	else if (strstr (defaults_str, "WEBSTER_SERVER_PORT") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		cword_g_defaults_record.webster_server_port = temp_int;
	    }
	}
	else if (strstr (defaults_str, "MAX_LIGHT_CHOICES") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if ( (temp_int > CWORD_SIZEOF_MAX_ARRAY_SIZE) || (temp_int < 0) )
		    cword_g_defaults_record.max_light_choices = CWORD_SIZEOF_MAX_ARRAY_SIZE;
		else
		    cword_g_defaults_record.max_light_choices = temp_int;
	    }
	}
	else if (strstr (defaults_str, "LOG_FILE_PREFIX") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		if (temp_str[0] != (char) 0)
		    strcpy (cword_g_defaults_record.log_file_prefix, temp_str); 
	    }
	}
	else if (strstr (defaults_str, "LIGHT_GENERATION_ALGORITHM") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if (temp_int == 1)
		    cword_g_defaults_record.light_generation_algorithm = RANDOMWORDS;
		else
		    cword_g_defaults_record.light_generation_algorithm = BESTWORDS;
	    }
	}
	else if (strstr (defaults_str, "CHECK_SPELLING") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		cword_word_upper_str (temp_str);

		if (strcmp (temp_str, CWORD_TRUE_STR) == 0)
		    cword_g_defaults_record.check_spelling = TRUE;
		else if (strcmp (temp_str, CWORD_FALSE_STR) == 0)
		    cword_g_defaults_record.check_spelling = FALSE;
		else
		    cword_g_defaults_record.check_spelling = TRUE;
	    }
	}
	else if (strstr (defaults_str, "TRACE_LEVEL") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if (temp_int >= 3)
		    cword_g_defaults_record.trace_level = LEVEL3;
		else if (temp_int <= 0)
		    cword_g_defaults_record.trace_level = LEVEL0;
		else if (temp_int == 1)
		    cword_g_defaults_record.trace_level = LEVEL1;
		else if (temp_int == 2)
		    cword_g_defaults_record.trace_level = LEVEL2;
	    }
	}
	else if (strstr (defaults_str, "MAX_MATCHES") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if ( (temp_int > CWORD_SIZEOF_MAX_ARRAY_SIZE) || (temp_int == 0) )
		    cword_g_defaults_record.max_matches = CWORD_SIZEOF_MAX_ARRAY_SIZE;
		else 
		    cword_g_defaults_record.max_matches = temp_int;
	    }
	}
	else if (strstr (defaults_str, "MAX_LIGHT_ITERATIONS") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if (temp_int == 0)
		    cword_g_defaults_record.max_light_iterations = CWORD_SIZEOF_MAX_INT_VAL;
		else if (temp_int > 0)
		    cword_g_defaults_record.max_light_iterations = temp_int;
	    }
	}
	else if (strstr (defaults_str, "USERNAME") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		if (temp_str[0] != (char) 0)
		    strcpy (cword_g_defaults_record.username, temp_str);
	    }
	}
	else if (strstr (defaults_str, "LETTER_WEIGHT_ALGORITHM") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if ( (temp_int < 4) || (temp_int > 0) )
		{
		    /*
		     *  To include 1, 2 or 3 as defined in defaults.dat!
		     */

		    cword_g_defaults_record.letter_weight_algorithm = temp_int;
		}
	    }
	}
	else if (strstr (defaults_str, "NEXTLIGHT_SELECT_ALGORITHM") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if (temp_int == 4)
		    cword_g_defaults_record.nextlight_select_algorithm = SHORTEST_LIGHT;
		else if (temp_int == 3)
		    cword_g_defaults_record.nextlight_select_algorithm = LONGEST_LIGHT;
		else if (temp_int == 2)
		    cword_g_defaults_record.nextlight_select_algorithm = RANDOM_LIGHT;
		else
		    cword_g_defaults_record.nextlight_select_algorithm = BEST_LIGHT;
	    }
	}
	else if (strstr (defaults_str, "FILL_FROM_WHERE") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		cword_g_defaults_record.fill_from_where = FILL_FROM_WORD_FILE;

		if (temp_int == 2)
		{
		    /*
		     *	Override default setting if we are on VMS (get lights
		     *	from database)!
                     */

#if defined(VMS) || defined (__VMS)
		    cword_g_defaults_record.fill_from_where = FILL_FROM_DB;
#endif
		}
	    }
	}
	else if (strstr (defaults_str, "RESET_AFTER_NUM_ITERATIONS") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		temp_int = atoi (temp_str);

		if ( (temp_int <= 100) ||
                      (temp_int > cword_g_defaults_record.max_light_iterations) )
		    cword_g_defaults_record.reset_after_num_iterations = 250;
		else
		    cword_g_defaults_record.reset_after_num_iterations = temp_int;
	    }
	}
	else if (strstr (defaults_str, "RING_BELL") != 0)
	{
	    status = cword_defaults__parse_line (
			defaults_str,
			sizeof (temp_str),
			temp_str);

	    if (status == CWORD_S_NORMAL)
	    {
		cword_word_upper_str (temp_str);

		if (strcmp (temp_str, CWORD_TRUE_STR) == 0)
		    cword_g_defaults_record.ring_bell = TRUE;
		else if (strcmp (temp_str, CWORD_FALSE_STR) == 0)
		    cword_g_defaults_record.ring_bell = FALSE;
		else
		    cword_g_defaults_record.ring_bell = TRUE;
	    }
	}
	else
	{
	    cword_messages_output_message (
		CWORD_DEFAULTUNDEFINED,
		(SCREEN0 | TRACE0),
		defaults_str);
	}
    }

    if (read_status == -1)
    {
	CWORD_DEFAULTS_READ_ERROR(
	    CWORD_FILEREADERROR, 
	    cword_g_defaults_record.defaults_file_spec, 
	    defaults_file);
    }
    else
    {
	close (defaults_file);

	cword_messages_output_message (
	    CWORD_DEFREAD,
	    TRACE2,
	    cword_g_defaults_record.defaults_file_spec);
    }

    cword_l_defaults_file_read = TRUE;

    return (CWORD_S_NORMAL);

}



#else

main ()

{
    printf ("Test\n");

    exit (0);

}

#endif
