!  FACILITY
!  --------
!
!	descrip.mms
!
!  ABSTRACT
!  --------
!
!	This file contains the MMS commands which are necessary to build 
!	Cword on OpenVMS/VAX or OpenVMS/AXP.
!
! 	To build the VAX version use the command line:
!
!	$ MMS
!
!	To build the AXP version use the command line:
!
!	$ MMS/macro=axp=true 
!
!    	Both commands are case insensitive.  It is not possible, 
!       AFAIK, to build Cword on both VAX and AXP platforms using
!       this file with the simple command MMS.
!
!  AUTHORS
!  -------
!
!	James A. Lundon
!
!  CREATION DATE
!  -------------
!
!	5 September 1994 (The day after Limerick lost the All-Ireland)
!

!
!  This MMS file can be used on either OpenVMS/VAX or OPenVMS/AXP.  Either 
!  DEC C or VAX C can be used when building the OpenVMS/VAX version.
!
!  Minor changes would have to be made to get this MMS file working for 
!  gcc or some other VAX based compiler.
!

!
!  List of MMS macro options:
!
!    1.  AXP   = Builds AXP version (default VAX version).
!    2.  VAXC  = What compiler options should be used during build (VAX 
!	         version only).
!    3.  ANSI  = Compile in DEC C with /standard=ansi89 if the VAXC macro 
!                isn't set (VAX version only).
!    4.  DEBUG = compile a debug or non-debug version of Cword (VAX or AXP 
!                version)?
!    5.  NODB  = Do *not* include the database component of Cword (VAX or
!                AXP version).
!

CC = cc

.ifdef axp

!
!  Building on AXP.
!

COMPILER_OPTS   = /standard=vaxc/prefix=all

SCC             = $sql$pre
SCC_OPTS        = /cc/standard=vaxc/prefix=all

L_FILES         = , cword_s:vmsdecc.opt/opt, sys$library:sql$user.olb/library

.else

!
!  Building on VAX.
!

.ifdef vaxc

!
!  VAX C
!

COMPILER_OPTS	= /standard=vaxc

L_FILES         = , cword_s:vmsvaxc.opt/opt, cword_s:vmsucx.opt/opt, cword_s:vmssql.opt/opt

.else

!
!  DEC C
!

.ifdef ansi
COMPILER_OPTS	= /decc/standard=ansi89
.else
COMPILER_OPTS	= /decc/standard=vaxc
.endif

L_FILES         = , cword_s:vmsdecc.opt/opt, cword_s:vmssql.opt/opt

.endif

SCC             = $sql$pre60

.endif

.ifdef debug

DEBUG_OPT	= _d
C_OPTS		= /debug/noopt
L_OPT		= /debug/notrace

.else

C_OPTS		= /nodebug/opt
L_OPT		= /nodebug/notrace

.endif

!
!  Total Compiler/linker flags
!

C_FLAGS		= $(compiler_opts)$(c_opts)
SCC_FLAGS       = $(compiler_opts)$(scc_opts)
L_FLAGS		= $(l_opt)



!
!  This code is always executed first or last
!

.first
.ifdef axp
.ifdef nodb
	@ write sys$output "Building non-database AXP version of Cword"
.else
	@ write sys$output "Building database AXP version of Cword"
.endif
.else
.ifdef nodb
	@ write sys$output "Building non-database VAX version of Cword"
.else
	@ write sys$output "Building database VAX version of Cword"
.endif
.endif

.last
.ifdef axp
.ifdef nodb
	@ write sys$output "Finished building non-database AXP version of Cword"
.else
	@ write sys$output "Finished building database AXP version of Cword"
.endif
.else
.ifdef nodb
	@ write sys$output "Finished building non-database VAX version of Cword"
.else
	@ write sys$output "Finished building database VAX version of Cword"
.endif
.endif

.ifdef test
	/*
         *  ?? Obsolete ??
	 */

	@ axp_flag = 0
	@ vax_flag = 0
	@ if (f$getsyi ("HW_MODEL") .gt. 1023) then axp_flag = 1
	@ if (f$getsyi ("HW_MODEL") .le. 1023) then vax_flag = 1
	@ if (axp_flag) then write sys$output "AXP build"
	@ if (axp_flag) then SCC = "$sql$pre"
	@ if (axp_flag) then C_FLAGS = "/nodebug/opt/standard=vaxc"
	@ if (axp_flag) then L_FLAGS = "/nodebug/notrace"
	if (axp_flag) then L_FILES = ",sys$library:sql$user.olb"
	@ if (vax_flag) then write sys$output "VAX build"
	@ if (vax_flag) then SCC = "$sql$pre60"
	@ if (vax_flag) then C_FLAGS = "/decc/standard=vaxc/nodebug/opt"
	@ if (vax_flag) then L_FLAGS = "/nodebug/notrace"
	if (vax_flag) then L_FILES = ",cword_s:vmsdecc.opt/opt, cword_s:vmsucx.opt/opt, cword_s:vmssql.opt/opt"
	sh log L_FILES
	sh sym L_FILES
.endif



!
!  Get rid of object library structure as it's unnecessary...
!

CWORD_OBJS = cword_o:print$(DEBUG_OPT).obj, cword_o:rdb$(DEBUG_OPT).obj, -
             cword_o:grid$(DEBUG_OPT).obj, cword_o:memman$(DEBUG_OPT).obj, -
             cword_o:utils$(DEBUG_OPT).obj, cword_o:messages$(DEBUG_OPT).obj, -
             cword_o:fileio$(DEBUG_OPT).obj, cword_o:match$(DEBUG_OPT).obj, -
             cword_o:verify$(DEBUG_OPT).obj, cword_o:stats$(DEBUG_OPT).obj, -
             cword_o:soundex$(DEBUG_OPT).obj, cword_o:word$(DEBUG_OPT).obj, -
             cword_o:vms$(DEBUG_OPT).obj, cword_o:charui$(DEBUG_OPT).obj, -
             cword_o:webster$(DEBUG_OPT).obj, cword_o:anagram$(DEBUG_OPT).obj, -
             cword_o:generate$(DEBUG_OPT).obj, cword_o:defaults$(DEBUG_OPT).obj, -
	     cword_o:clue$(DEBUG_OPT).obj, cword_o:seed$(DEBUG_OPT).obj, -
	     cword_o:breakdic$(DEBUG_OPT).obj, cword_o:cl$(DEBUG_OPT).obj, -
             cword_o:strll$(DEBUG_OPT).obj, cword_o:main$(DEBUG_OPT).obj

!
!  MMS will execute the actions that the target Cword defines if no other action 
!  is defined.
!

CWORD :	cword_e:cword$(DEBUG_OPT).exe cword_e:breakdic$(DEBUG_OPT).exe
    @ write sys$output "Built Cword and breakdic"

!
!  Actions to build each exe, whether it be main.exe or any of the test harnesses.
!

cword_e:cword$(DEBUG_OPT).exe : $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
                  $(CWORD_OBJS) -
		  $(L_FILES)

cword_e:breakdic$(DEBUG_OPT).exe : cword_o:breakdic_main$(DEBUG_OPT).obj
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
                  cword_o:breakdic_main$(DEBUG_OPT).obj -
		  $(L_FILES)

clean :
    delete/nolog/noconfirm cword_o:*$(DEBUG_OPT).obj;
    @ write sys$output "Deleting all Cword objects"



!
!  Actions to create all object files.
!

cword_o:main$(DEBUG_OPT).obj : cword_s:main.c cword_s:main.h cword_s:messages.h
.ifdef nodb
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.else
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) /define=DB $(MMS$SOURCE)
.endif

cword_o:print$(DEBUG_OPT).obj : cword_s:print.c cword_s:print.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:rdb$(DEBUG_OPT).obj : cword_s:rdb.sc cword_s:rdb.h cword_s:messages.h
.ifdef nodb
    SCC $(scc_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.else
    SCC $(scc_flags) /object=$(MMS$TARGET_NAME) /define=DB $(MMS$SOURCE)
.endif
.ifdef debug_opt
.else
    $ delete/nolog/noconfirm cword_o:rdb$(DEBUG_OPT).c;*
.endif

cword_o:grid$(DEBUG_OPT).obj : cword_s:grid.c cword_s:grid.h cword_s:messages.h
.ifdef nodb
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.else
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) /define=DB $(MMS$SOURCE)
.endif

cword_o:match$(DEBUG_OPT).obj : cword_s:match.c cword_s:match.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:memman$(DEBUG_OPT).obj : cword_s:memman.c cword_s:memman.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:fileio$(DEBUG_OPT).obj : cword_s:fileio.c cword_s:fileio.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:utils$(DEBUG_OPT).obj : cword_s:utils.c cword_s:utils.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:messages$(DEBUG_OPT).obj : cword_s:messages.c cword_s:messages.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:verify$(DEBUG_OPT).obj : cword_s:verify.c cword_s:verify.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:stats$(DEBUG_OPT).obj : cword_s:stats.c cword_s:stats.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:soundex$(DEBUG_OPT).obj : cword_s:soundex.c cword_s:soundex.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:word$(DEBUG_OPT).obj : cword_s:word.c cword_s:word.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:vms$(DEBUG_OPT).obj : cword_s:vms.c cword_s:vms.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:charui$(DEBUG_OPT).obj : cword_s:charui.c cword_s:charui.h cword_s:messages.h
.ifdef nodb
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.else
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) /define=DB $(MMS$SOURCE)
.endif

cword_o:webster$(DEBUG_OPT).obj	: cword_s:webster.c cword_s:webster.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:anagram$(DEBUG_OPT).obj	: cword_s:anagram.c cword_s:anagram.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:generate$(DEBUG_OPT).obj : cword_s:generate.c cword_s:generate.h cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:defaults$(DEBUG_OPT).obj : cword_s:defaults.c cword_s:defaults.h cword_s:messages.h
    $(CC) $(c_flags) /define=MAIN /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:clue$(DEBUG_OPT).obj : cword_s:clue.c, cword_s:clue.h, cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:seed$(DEBUG_OPT).obj : cword_s:seed.c, cword_s:seed.h, cword_s:messages.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:breakdic$(DEBUG_OPT).obj : cword_s:breakdic.c cword_s:breakdic.h cword_s:main.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:cl$(DEBUG_OPT).obj : cword_s:cl.c cword_s:cl.h cword_s:main.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:strll$(DEBUG_OPT).obj : cword_s:strll.c cword_s:strll.h cword_s:main.h
    $(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

!
!  Standalone breakdic object.
!

cword_o:breakdic_main$(DEBUG_OPT).obj : cword_s:breakdic.c cword_s:breakdic.h cword_s:main.h
    $(CC) $(c_flags) /define=MAIN /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
