/*
 *  ?? Generate American type grids by specifying the blanks rather than the
 *  lights proper.  This scheme should mean less work for the crossword setter
 *  as most American grid contain large amounts of lights which make creating
 *  these types of grids with Cword very cumbersome and probably slower than
 *  the proposed scheme will take ??
 */

/*
 *  ?? Rotate a defined grid by 90 or 180 degrees, thus creating a new grid ??
 */

/*
 *++
 *  FACILITY:
 *
 *      generate.c
 *
 *  ABSTRACT:
 *
 *	Module containing the functionality controlling the generation of an
 *	empty grid definition/template.  This template will contain the empty
 *	light locations which can then be used by the light filler at some
 *	later stage.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      9 Nomember 1993
 *--
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <signal.h>
#include <setjmp.h>

#if defined(__unix) || defined(__unix__)

#include <sys/file.h>
#include <sys/uio.h>

#endif

#if defined(__MSDOS__) || defined(MSDOS)

#endif

#include "utils.h"
#include "fileio.h"
#include "main.h"
#include "os.h"
#include "generate.h"
#include "messages.h"
#include "charui.h"
#include "stats.h"
#include "verify.h"
#include "defaults.h"

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#ifndef GENERATE

#define QUIT_CHAR	'Q'
#define SAVE_CHAR	'S'
#define VERIFY_CHAR	'V'
#define RESET_CHAR	'R'
#define DELETE_CHAR     'D'
#define ADD_CHAR	'A'
#define LIST_CHAR	'L'
#define HELP_CHAR	'H'
#define INVALID_CHAR    '\0'

#define ACROSS_LIGHT	0X01
#define DOWN_LIGHT	0X02

#define HORIZ_STR	">"
#define VERTI_STR	"^"

#define GEN_CLUES_PER_LINE  3

typedef enum 
{
    ADD = 1, 
    DELETE
} ACTION;

PRIVATE int cword_generate__add_light_loc (
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__del_light_loc (
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__view_grid (
	int                     output_mask,
	CROSSWORD_RECORD	*crossword_record);

/*
 *  Both additions and deletions are handled in these ...action...() functions
 */

PRIVATE int cword_generate__action_90deg (
	ACTION			action,	
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__action_180deg (
	ACTION			action,	
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__action_lraxial (
	ACTION			action,	
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__action_rlaxial (
	ACTION			action,	
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__action_nodeg (
	ACTION			action,	
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE BOOLEAN cword_generate_is_light_found (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light);

PRIVATE BOOLEAN cword_generate__light_centre (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light);

PRIVATE BOOLEAN cword_generate__is_light_valid (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int 	    		length_of_light);

PRIVATE int cword_generate__get_grid_props (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__check_adjacency (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light);

PRIVATE int cword_generate__verify_coords (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light);

PRIVATE int cword_generate__get_light_coord (
	CROSSWORD_RECORD	*crossword_record,
	int			*x_coordinate,
	int			*y_coordinate,
	int			*light_direction,
	int			*length_of_light);

PRIVATE int cword_generate__verify_grid (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__tidy_grid (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__delete_lights (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light);

PRIVATE int cword_generate__delete_grid_opts (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__reset_grid (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__list_grid (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__add_lights (
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record);

PRIVATE int cword_generate__add_grid_opts (
	CROSSWORD_RECORD	*crossword_record);

PRIVATE void cword_generate__get_option (
	char 			*option);



/*
 *  Module globals -- Indexes of newest lights created and deleted.
 */

static CLUE_DETAILS cword_generate_l_new_add_light;

static CLUE_DETAILS cword_generate_l_new_del_light;

static BOOLEAN cword_generate_l_grid_updated;



/*
 *  Types of grid
 *  -------------
 *
 *  A. BLOCK ---|- 1. Ordinary block grid
 *              |
 *              |- 2. American block grid
 *              |
 *              |- 3. Novelty block grid
 *
 *  B. BARRED --|- 4. Barred grid (straight-up barred grid)
 *
 *  Properties of grid types
 *  ========================
 *
 *  1. Ordinary block grid
 *         X: any
 *         Y: any
 *         symmetry: any
 *	   unches: yes/no (dependent on initialisation file)
 *	   adjacency: Only internal adjancies allowed [whatever that means]
 *	   cell formation: Some checked, some unchecked, some blanks
 *
 *  2. American block grid
 *         X: any
 *         Y: any
 *         symmetry: any
 *	   unches: no
 *	   adjacency: yes
 *	   cell formation: All checked, none unchecked, some blanks (less than 17%)
 *
 *  3. Novelty block grid
 *         X: Y (X will always be equal to Y)
 *         Y: X (Y will always be equal to X)
 *         symmetry: 90 or 180 degree
 *	   unches: yes
 *	   adjacency: yes
 *	   cell formation: anything is allowed
 *
 *  4. Barred grid (straight-up barred grid
 *         X: anything goes
 *         Y: anything goes
 *         symmetry: none [ALWAYS!]
 *	   unches: yes
 *	   adjacency: yes
 *	   cell formation: some checked, some unchecked, *none* blanks
 *
 *  If any of the grid chosen have uneven dimensions then the selected symmetry
 *  value is overridden and none is selected to avoid any number of possible
 *  problems I've got with handling grids where X != Y.  Since I wrote this
 *  little application, everything I say goes :-).
 */



PRIVATE int cword_generate__add_light_loc (
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to add a valid light definition to the evolving crossword
 *	definition.
 *
 *  FORMAL PARAMETERS:
 *
 *	x_coordinate		(r) X coordinate of the light to be added
 *	y_coordinate		(r) Y coordinate of the light to be added
 *	light_direction		(r) Direction of the light to be added
 *	length_of_light		(r) Length of the answer of the light to be added
 *      *crossword_record	(w) Pointer to an instance of a crossword_record
 *
 *  IMPLICIT OUTPUTS:
 *
 *	cword_generate_l_grid_updated
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		    Successful completion
 *--
 */

{

    cword_messages_output_message (
	    CWORD_ADDLIGHTLOC,
	    (TRACE0 | SCREEN0),
	    x_coordinate, 
	    y_coordinate,
	    light_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
	    length_of_light);

    /*
     *  Check if maximum number of lights limit has reached!
     */

    if (crossword_record->clue_sum == CWORD_SIZEOF_CLUENUM)
    {
	cword_messages_output_message (
		CWORD_S_MAXNUMOFCLUES,
		SCREEN0);

	return (CWORD_S_FAILURE);
    }

    /*
     *	When the dimensions of the incoming grid are unequal sometimes the
     *	light being added already exists in the grid definition.  This will
     *	most likely occur when a symmetry other than none is being employed.
     *	This option is currently disabled as it has led to other problems in
     *	this code.
     */

    if (cword_generate_is_light_found (
		crossword_record,
		x_coordinate,
		y_coordinate,
		light_direction,
		length_of_light) == FALSE)
    {
	crossword_record->clue_details[crossword_record->clue_sum].x_coordinate   = x_coordinate;
	crossword_record->clue_details[crossword_record->clue_sum].y_coordinate   = y_coordinate;
	crossword_record->clue_details[crossword_record->clue_sum].answer_length  = length_of_light;
	crossword_record->clue_details[crossword_record->clue_sum].clue_direction = light_direction;

	crossword_record->clue_sum++;

	cword_generate_l_grid_updated = TRUE;
    }
    else
    {
	/*
	 *  This error can sometimes happen.  I should have a better check
	 *  procedure and not leave it to here.
	 */

	cword_messages_output_message (
		CWORD_S_LGTALREADTPRES,
		(TRACE0 | SCREEN0));
    }

    /*
     *	There is no need for any other information to be defined at this stage.
     */

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__del_light_loc (
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to delete a valid light definition from the evolving crossword
 *	definition.
 *
 *  FORMAL PARAMETERS:
 *
 *	x_coordinate		(r) x coordinate of the light to be added
 *	y_coordinate		(r) y coordinate of the light to be added
 *	light_direction		(r) direction of the light to be added
 *	length_of_light		(r) length of the answer of the light to be added
 *      *crossword_record	(w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *	cword_generate_l_grid_updated
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		    Succeeded to delete light from grid
 *	CWORD_S_FAILURE		    Failed to delete light from grid
 *--
 */

{

    int count,
	count1;

    BOOLEAN is_light_deleted = FALSE;

    cword_messages_output_message (
	    CWORD_S_DELLIGHTLOC,
	    (TRACE0 | SCREEN0),
	    x_coordinate, 
	    y_coordinate,
	    light_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
	    length_of_light);

    for (count = 0; count < crossword_record->clue_sum; count++)
    {
	if ( (crossword_record->clue_details[count].x_coordinate == x_coordinate) &&
	      (crossword_record->clue_details[count].y_coordinate == y_coordinate) &&
	       (crossword_record->clue_details[count].clue_direction == light_direction) &&
	        (crossword_record->clue_details[count].answer_length == length_of_light) )
	{
	    crossword_record->clue_details[count].x_coordinate = 0;
	    crossword_record->clue_details[count].y_coordinate = 0;
	    crossword_record->clue_details[count].clue_direction = 0;
	    crossword_record->clue_details[count].answer_length = 0;

	    is_light_deleted = TRUE;
	}
    }

    if (is_light_deleted == FALSE)
    {
	/*
	 *  This shouldn't happen!
	 */

	printf ("# Error deleting light -- couldn't find it :+(\n");

	return (CWORD_S_FAILURE);
    }

    /*
     *  Re-sort crossword_record after each deletion.
     */

    count1 = 0;

    for (count = 0; count < crossword_record->clue_sum; count++)
    {
	if (crossword_record->clue_details[count].x_coordinate != 0)
	{
	    if (count != count1)
	    {
		/*
		 *  Swap
		 */

		crossword_record->clue_details[count1].x_coordinate = 
		    crossword_record->clue_details[count].x_coordinate;
		crossword_record->clue_details[count1].y_coordinate = 
		    crossword_record->clue_details[count].y_coordinate;
		crossword_record->clue_details[count1].clue_direction = 
		    crossword_record->clue_details[count].clue_direction;
		crossword_record->clue_details[count1].answer_length = 
		    crossword_record->clue_details[count].answer_length;
	    }

	    count1++;
	}
	else
	{
	    /*
	     *	If crossword_record->clue_details[count].x_coordinate == 0 then
	     *	skip.
	     */
	}
    }

    /*
     *	Delete the rest of the light defintion before returning
     *	crossword_record.
     */

    for (count = count1; count < crossword_record->clue_sum; count++)
    {
	crossword_record->clue_details[count].x_coordinate = 0;
	crossword_record->clue_details[count].y_coordinate = 0;
	crossword_record->clue_details[count].clue_direction = 0;
	crossword_record->clue_details[count].answer_length = 0;
    }

    crossword_record->clue_sum--;

    cword_generate_l_grid_updated = TRUE;

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Maybe possible to consolidate this functionality with the other
 *  grid/crossword output functionality located in other modules ??
 */

/*
 *  ?? When the grid being created is greater that can be viewed in one page is
 *  there a way of making the "please press return to continue" disappear when
 *  the rest of the grid is being displayed.  This message upsets the
 *  continuity of the displaying of the grid to quite a large extent and it
 *  would make the creation process much more seamless if I could get rid of it
 *  via control character manipulation (but would it be portable enough to
 *  implement) ??
 */

PRIVATE int cword_generate__view_grid (
	int                     output_mask,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that outputs the current grid definition during grid creation.
 *
 *	This function captures the extra subtlety of outputting the checked
 *	cells in the defined grid.
 *
 *  FORMAL PARAMETERS:
 *
 *	output_mask		(r) Used to determine how output (screen/trace file) is handled
 *	*crossword_record	(r) Pointer to crossword details
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal Completion
 *	CWORD_S_FAILURE		Misc. error
 *--
 */

{

    CROSSWORD_GRID output_grid;

    char temp_str[CWORD_SIZEOF_LINE + 1],
	 temp_str1[CWORD_SIZEOF_LINE + 1],
	 temp_str2[CWORD_SIZEOF_LINE + 1],
	 temp_str3[CWORD_SIZEOF_LINE + 1],
	 temp_str4[CWORD_SIZEOF_LINE + 1],
	 create_str[CWORD_SIZEOF_LINE + 1];

    MY_DIV div_struct;

    int	count1,
	count2,
	length,
	across = 0,
	down = 0,
	line_num;

    /*
     *	Before the grid is re-outputted I want to allow the user see what
     *	happened previously.
     */

    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

    CWORD_OS_CLEAR_SCREEN ();

    if (crossword_record->symmetry == DEG_90_SYMMETRY)
	cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);
    else if (crossword_record->symmetry == DEG_180_SYMMETRY)
	cword_messages_get_message (CWORD_180SYMMETRYMSG, temp_str);
    else if (crossword_record->symmetry == AXIAL_LR_SYMMETRY)
	cword_messages_get_message (CWORD_AXIALLEFTRIGHTMSG, temp_str);
    else if (crossword_record->symmetry == AXIAL_RL_SYMMETRY)
	cword_messages_get_message (CWORD_AXIALRIGHTLEFTMSG, temp_str);
    else if (crossword_record->symmetry == NO_SYMMETRY)
	cword_messages_get_message (CWORD_NOSYMMETRYMSG, temp_str);
    else
    {
	printf ("# Error getting grid symmetry - cannot continue\n");

	return (CWORD_S_FAILURE);
    }

    /*
     *	Only use the SCREEN mask part of output_mask.
     */

    cword_messages_output_message (
	CWORD_LIGHTGRIDMSG,
	cword_messages_get_screen_mask (output_mask),
	CWORD_UNFILLED_CHAR, 
	CWORD_UNCHED_CHAR,
	CWORD_CHECKED_CHAR);

    cword_messages_output_message (
	CWORD_USING, 
	cword_messages_get_screen_mask (output_mask),
	temp_str);

    across = cword_stats_get_clue_sum_for (
    		ACROSS,
    		crossword_record);

    down = cword_stats_get_clue_sum_for (
    		DOWN,
    		crossword_record);

    /*
     *	?? Will use block_props even though the grid could be a barred grid,
     *	for the time being ??
     */

    cword_verify_get_block_props (
	crossword_record,
	output_grid);

    line_num = 3;
    for (count2 = 0; count2 < crossword_record->y_dimensions; count2++)
    {
	memset (temp_str1, 0, sizeof (temp_str1));

	if (count2 != 0)
	    sprintf (
		temp_str1,
		"%02u%s",
		crossword_record->y_dimensions - count2,
		HORIZ_STR);
	else
	    sprintf (
		temp_str1,
		"\n%02u%s",
		crossword_record->y_dimensions - count2,
		HORIZ_STR);
	
	length = strlen (temp_str1);

	for (count1 = 0; count1 < crossword_record->x_dimensions; count1++)
	{
	    temp_str1[length + count1] = 
             output_grid[count1][crossword_record->y_dimensions - 1 - count2];
	}

	/*
	 *  Extra messages useful during grid creation are inserted here!
	 *
	 *  ?? Extra messages (to the side of the grid being outputted) are not
	 *  being displayed if the grid being created is less than 8*8 in size ??
	 */

	if (count2 == 1)
	{
	    if (crossword_record->clue_sum != 0)
	    {
		cword_messages_get_message (
		    CWORD_GENUPDATEMSG1,
		    temp_str);

		sprintf (
		    create_str, 
		    temp_str,
		    crossword_record->clue_sum);
            }
	    else
	    {
		cword_messages_get_message (
		    CWORD_GENUPDATEMSG1A,
		    create_str);
	    }

	    strcat (temp_str1, create_str);
	}
	else if (count2 == 2)
	{
	    if (crossword_record->clue_sum != 0)
	    {
		cword_messages_get_message (
		    CWORD_GENUPDATEMSG2,
		    temp_str);

		strcat (temp_str1, temp_str);
	    }
	}
	else if (count2 == 3)
	{
	    if (crossword_record->clue_sum != 0)
	    {
		cword_messages_get_message (
		    CWORD_GENUPDATEMSG3,
		    temp_str);

		sprintf (
		    create_str, 
		    temp_str,
		    across,
		    CWORD_ACROSS_STR,
    		    across == 1 ? LIGHT_STR : LIGHTS_STR);

		strcat (temp_str1, create_str);
	    }
	}
	else if (count2 == 4)
	{
	    if (crossword_record->clue_sum != 0)
	    {
		cword_messages_get_message (
		    CWORD_GENUPDATEMSG3,
		    temp_str);

		sprintf (
		    create_str, 
		    temp_str,
		    down,
		    CWORD_DOWN_STR,
    		    down == 1 ? LIGHT_STR : LIGHTS_STR);

		strcat (temp_str1, create_str);
	    }
	}
	else if (count2 == 6)
	{
	    /*
	     *  ?? Add length of clue to message - if necessary ??
	     */

	    if (cword_generate_l_new_add_light.x_coordinate != 0)
	    {
		cword_messages_get_message (
		    CWORD_GENUPDATEMSG4,
		    temp_str);

		sprintf (
		    create_str,
		    temp_str,
		    cword_generate_l_new_add_light.x_coordinate,
		    cword_generate_l_new_add_light.y_coordinate,
		    cword_generate_l_new_add_light.clue_direction ? CWORD_A_CHAR: CWORD_D_CHAR);

		strcat (temp_str1, create_str);
	    }
	}
	else if (count2 == 7)
	{
	    /*
	     *  ?? Add length of clue to message - if necessary ??
	     */

	    if (cword_generate_l_new_del_light.x_coordinate != 0)
	    {
		cword_messages_get_message (
		    CWORD_GENUPDATEMSG5,
		    temp_str);

		sprintf (
		    create_str,
		    temp_str,
		    cword_generate_l_new_del_light.x_coordinate,
		    cword_generate_l_new_del_light.y_coordinate,
		    cword_generate_l_new_del_light.clue_direction ? CWORD_A_CHAR: CWORD_D_CHAR);

		strcat (temp_str1, create_str);
	    }
	}

	strcat (temp_str1, "\n");

	/*
	 *  This message actually does the outputting to the SCREEN/TRACE log
	 *  of the grid being created.
	 */

	cword_messages_output_message (
	    CWORD_USERDEFINEDSTRING,
	    output_mask,
	    temp_str1);

	line_num++;

	/*
	 *  ?? Need to be more precise, so that the grid is being scrolled
	 *  correctly ??
	 */

	if (crossword_record->y_dimensions < 38)
	{
	    /*
	     *  ?? Is this OK ??
	     */

	    cword_charui_check_page_break (
		NOT_FINISHED,
		CWORD_S_PRESSRETURN,
		&line_num);
	}
	else
	{
	    if (count2 == 20)
		cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
	}
    }

    memset (temp_str1, 0, sizeof (temp_str1));
    memset (temp_str2, 0, sizeof (temp_str2));
    memset (temp_str3, 0, sizeof (temp_str3));
    memset (temp_str4, 0, sizeof (temp_str4));

    /*
     *  The last few lines of output
     */

    strcpy (temp_str3, "--+");
    strcpy (temp_str1, "Y/|");
    strcpy (temp_str2, "/X|");

    for (count1 = 1; count1 <= crossword_record->x_dimensions; count1++)
    {
	if (count1 < 10)
	{
	    strcat (temp_str1, "0");
	    sprintf (
		temp_str4,
		"%1u",
		count1);
	    strcat (temp_str2, temp_str4);
	}
	else
	{
	    div_struct = cword_utils_div (count1, 10);

	    sprintf (
		temp_str4,
		"%1u",
		div_struct.quot);
	    strcat (temp_str1, temp_str4);	    
	    sprintf (
		temp_str4,
		"%1u",
		div_struct.rem);
	    strcat (temp_str2, temp_str4);
	}

	strcat (temp_str3, VERTI_STR);
    }

    strcat (temp_str3, "\n");
    strcat (temp_str1, "\n");
    strcat (temp_str2, "\n\n");

    cword_messages_output_message (
	CWORD_USERDEFINEDSTRING,
	output_mask,
	temp_str3);

    cword_messages_output_message (
	CWORD_USERDEFINEDSTRING,
	output_mask,
	temp_str1);

    cword_messages_output_message (
	CWORD_USERDEFINEDSTRING,
	output_mask,
	temp_str2);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__action_90deg (
	ACTION			action,	
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to help add or delete the light locations to complete a 90
 *	degree grid symmetry pattern given a single input light as guide.
 *
 *	Algorithm
 *	---------
 *
 *	This is the algorithm to decide how to fill the grid with the 90
 *	degree symmetric lights. 
 *
 *	N.B. Everything is based around the initial light being in quadrant 1.
 *  
 *	    Quad1, Quad2, Quad3, Quad4
 *	    ACROSS,DOWN, ACROSS, DOWN,
 *	    DOWN, ACROSS, DOWN, ACROSS 
 *
 *	    START, START, END, END,
 *	    START, END, END, START
 *
 *	These are the offsets to be calculated from each of the four corners of
 *	the puzzle to allow the light be positions in the correct location. The
 *	X and Y values are calculated from the Quad1 corner and is equal to:
 *
 *	    X = abs (x_coordinate - 1) 
 *	    Y = abs (y_coordinate - crossword_record->x_dimension)
 *
 *	    X, -Y, -X, Y,
 *	    -Y, -X, Y, X
 *  	    
 *  FORMAL PARAMETERS:
 *
 *	action			(r) ADD or DELETE
 *	x_coordinate		(r) x coordinate of the light to be added/deleted
 *	y_coordinate		(r) y coordinate of the light to be added/deleted
 *	light_direction		(r) direction of the light to be added/deleted
 *	length_of_light		(r) length of the answer of the light to be added/deleted
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_generate_l_new_add_light
 *	cword_generate_l_new_del_light
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Successful completion
 *--
 */

{

    char temp_str[CWORD_SIZEOF_LINE + 1];

    int	x,	
	y,
	direction,
	x_defecit,
	y_defecit,
	centre,
	status = CWORD_S_NORMAL;

    BOOLEAN is_balanced = FALSE,
	    light_through_centre = FALSE;

    cword_messages_get_message (
		CWORD_90SYMMETRYMSG, 
		temp_str);

    if (action == ADD)
    {
	cword_messages_output_message (
		    CWORD_ADDINGUSING, 
		    SCREEN0,
		    temp_str);
    }
    else
    {
	cword_messages_output_message (
		    CWORD_S_DELETINGUSING, 
		    SCREEN0,
		    temp_str);
    }

    if (cword_generate__light_centre (
		crossword_record,
		x_coordinate,
		y_coordinate,
		light_direction,
		length_of_light) == TRUE)
    {
	light_through_centre = TRUE;
    }

    /*
     *	?? 13*12 grid - X/Y/Dir/Len 7/8/D/4 should be OK to place in grid but
     *	isn't (can identify that it is a centre light) using 180 degree
     *	symmetry - bug ??
     */

    if (light_through_centre == TRUE)
    {
	/*
	 *  Check that the defined light doesn't double back in itself!
	 */

	if ((crossword_record->x_dimensions % 2) == 1) 
	{
	    centre = (int) ((float) (crossword_record->x_dimensions + 1) / (float) 2);

	    if (x_coordinate == centre) 
	    {
		/*
		 *  if y_coordinate goes through the centre then the light will
		 *  double back on itself causing the grid to be in error
		 */

		if ( (y_coordinate >= centre) && ((y_coordinate - length_of_light) <= centre) )
		{
		    status = CWORD_S_FAILURE;
		}
	    }
	    else
	    {
		/*
		 *  if x_coordinate goes through the centre then the light will
		 *  double back on itself causing the grid to be in error
		 */

		if ( (x_coordinate <= centre) && ((x_coordinate + length_of_light) >= centre) )
		{
		    status = CWORD_S_FAILURE;
		}
	    }

	    /*
	     *	Check if the inputted light is balanced around the centre
	     *	location - if so then we can continue but only at half power!
	     */

	    if (CWORD_S_FAILURE == status)
	    {
		/*
		 *  Inputted light is wrapped around the centre but to what
		 *  extent?
		 */

		if (x_coordinate == centre) 
		{
		    if ( (centre - (y_coordinate - length_of_light + 1)) == (y_coordinate - centre) )
		    {
			status = CWORD_S_NORMAL;

			is_balanced = TRUE;
		    }
		}
		else
		{
		    if ( (centre - x_coordinate) == ((x_coordinate + length_of_light - 1) - centre) )
		    {
			status = CWORD_S_NORMAL;

			is_balanced = TRUE;
		    }

		}
	    }
	}
    }

    if (CWORD_S_FAILURE == status)
    {
	/*
	 *  Continue as normal
	 */

	cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);

	cword_messages_output_message (
		CWORD_GENVIOLATE,
		(TRACE2 | SCREEN0),
		temp_str);

	return (CWORD_S_NORMAL);
    }

    /* ACROSS/DOWN start quadrant 1 */

    if (action == ADD)
    {
	cword_generate_l_new_add_light.x_coordinate = x_coordinate;
	cword_generate_l_new_add_light.y_coordinate = y_coordinate;
	cword_generate_l_new_add_light.answer_length = length_of_light;
	cword_generate_l_new_add_light.clue_direction = light_direction;

	cword_generate__add_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else
    {
	cword_generate_l_new_del_light.x_coordinate = x_coordinate;
	cword_generate_l_new_del_light.y_coordinate = y_coordinate;
	cword_generate_l_new_del_light.answer_length = length_of_light;
	cword_generate_l_new_del_light.clue_direction = light_direction;

	cword_generate__del_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }

    /*
     *	Calculate the 3 other grid locations where the lights are to be
     *	inserted into the grid.
     *
     *	Light lengths are the same for other locations but direction, x
     *	and y coordinates change accordingly.
     */

    x_defecit = abs (x_coordinate - 1);
    y_defecit = abs (y_coordinate - crossword_record->y_dimensions);

    if (light_direction == ACROSS)
    {
	if (is_balanced == TRUE)
	{
	    /* DOWN start quadrant 2 */
	    x = crossword_record->x_dimensions - y_defecit;
	    y = crossword_record->y_dimensions - x_defecit;
	    direction = DOWN;

	    if (action == ADD)
	    {
		cword_generate__add_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	    else
	    {
		cword_generate__del_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	}
	else
	{
	    /* DOWN start quadrant 2 */
	    x = crossword_record->x_dimensions - y_defecit;
	    y = crossword_record->y_dimensions - x_defecit;
	    direction = DOWN;

	    if (action == ADD)
	    {
		cword_generate__add_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	    else
	    {
		cword_generate__del_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }

	    if (FALSE == light_through_centre)
	    {
		/* ACROSS end quadrant 3 */
		x = crossword_record->x_dimensions - x_defecit - length_of_light + 1;
		y = 1 + y_defecit;
		direction = ACROSS;

		if (action == ADD)
		{
		    cword_generate__add_light_loc (
				    x,
				    y,
				    direction,
				    length_of_light,
				    crossword_record);
		}
		else
		{
		    cword_generate__del_light_loc (
				    x,
				    y,
				    direction,
				    length_of_light,
				    crossword_record);
		}
	    }

	    if (FALSE == light_through_centre)
	    {
		/* DOWN end quadrant 4 */
		x = 1 + y_defecit;
		y = 1 + x_defecit + length_of_light - 1;
		direction = DOWN;

		if (action == ADD)
		{
		    cword_generate__add_light_loc (
				    x,
				    y,
				    direction,
				    length_of_light,
				    crossword_record);
		}
		else
		{
		    cword_generate__del_light_loc (
				    x,
				    y,
				    direction,
				    length_of_light,
				    crossword_record);
		}
	    }
	}
    }
    else
    {
	if (is_balanced == TRUE)
	{
	    /* ACROSS end quadrant 2 */
	    x = crossword_record->x_dimensions - y_defecit - length_of_light + 1;
	    y = crossword_record->y_dimensions - x_defecit;
	    direction = ACROSS;

	    if (action == ADD)
	    {
		cword_generate__add_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	    else
	    {
		cword_generate__del_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	}
	else
	{
	    /* ACROSS end quadrant 2 */
	    x = crossword_record->x_dimensions - y_defecit - length_of_light + 1;
	    y = crossword_record->y_dimensions - x_defecit;
	    direction = ACROSS;

	    if (action == ADD)
	    {
		cword_generate__add_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	    else
	    {
		cword_generate__del_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }

	    if (FALSE == light_through_centre)
	    {
		/* DOWN end quadrant 3 */
		x = crossword_record->x_dimensions - x_defecit;
		y = 1 + y_defecit + length_of_light - 1;
		direction = DOWN;

		if (action == ADD)
		{
		    cword_generate__add_light_loc (
				    x,
				    y,
				    direction,
				    length_of_light,
				    crossword_record);
		}
		else
		{
		    cword_generate__del_light_loc (
				    x,
				    y,
				    direction,
				    length_of_light,
				    crossword_record);
		}
	    }

	    if (FALSE == light_through_centre)
	    {
		/* ACROSS start quadrant 4 */
		x = 1 + y_defecit;
		y = 1 + x_defecit;
		direction = ACROSS;

		if (action == ADD)
		{
		    cword_generate__add_light_loc (
				    x,
				    y,
				    direction,
				    length_of_light,
				    crossword_record);
		}
		else
		{
		    cword_generate__del_light_loc (
				    x,
				    y,
				    direction,
				    length_of_light,
				    crossword_record);
		}
	    }
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__action_180deg (
	ACTION			action,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to help add or delete the light locations to complete a 180
 *	degree grid symmetry pattern given a single input light as guide.
 *
 *	This function uses the code developed in ...90deg_loc () function to
 *	obtain the 180 degree lights.
 *
 *  FORMAL PARAMETERS:
 *
 * 	action			(r) ADD or DELETE
 *	x_coordinate		(r) x coordinate of the light to be added/deleted
 *	y_coordinate		(r) y coordinate of the light to be added/deleted
 *	light_direction		(r) direction of the light to be added/deleted
 *	length_of_light		(r) length of the answer of the light to be added/deleted
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_generate_l_new_add_light
 *	cword_generate_l_new_del_light
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Successful completion
 *--
 */

{

    char temp_str[CWORD_SIZEOF_LINE + 1];

    int x,	
	y,
	direction,
	x_defecit,
	y_defecit,
	status = CWORD_S_NORMAL,
	centre;

    BOOLEAN is_balanced = FALSE,
	    light_through_centre = FALSE;

    cword_messages_get_message (
		CWORD_180SYMMETRYMSG, 
		temp_str);

    if (action == ADD)
    {
	cword_messages_output_message (
		    CWORD_ADDINGUSING, 
		    SCREEN0,
		    temp_str);
    }
    else
    {
	cword_messages_output_message (
		    CWORD_S_DELETINGUSING, 
		    SCREEN0,
		    temp_str);
    }

    if (cword_generate__light_centre (
		crossword_record,
		x_coordinate,
		y_coordinate,
		light_direction,
		length_of_light) == TRUE)
    {
	light_through_centre = TRUE;
    }

    if (light_through_centre == TRUE)
    {
	/*
	 *  Check that the defined light doesn't double back in itself!
	 */

	if ((crossword_record->x_dimensions % 2) == 1)
	{
	    centre = (int) ((float) (crossword_record->x_dimensions + 1) / (float) 2);

	    if (x_coordinate == centre) 
	    {
		/*
		 *  if y_coordinate goes through the centre then the light will
		 *  double back on itself causing the grid to be in error
		 */

		if ( (y_coordinate >= centre) && ((y_coordinate - length_of_light) <= centre) )
		    status = CWORD_S_FAILURE;
	    }
	    else
	    {
		/*
		 *  if x_coordinate goes through the centre then the light will
		 *  double back on itself causing the grid to be in error
		 */

		if ( (x_coordinate <= centre) && ((x_coordinate + length_of_light) >= centre) )
		    status = CWORD_S_FAILURE;
	    }

	    /*
	     *	Check if the inputted light is balanced around the centre
	     *	location - if so then we can continue but only at half power!
	     */

	    if (CWORD_S_FAILURE == status)
	    {
		/*
		 *  Inputted light is wrapped around the centre but to what
		 *  extent?
		 */

		if (x_coordinate == centre) 
		{
		    if ( (centre - (y_coordinate - length_of_light + 1)) == (y_coordinate - centre) )
		    {
			status = CWORD_S_NORMAL;
			is_balanced = TRUE;
		    }
		}
		else
		{
		    if ( (centre - x_coordinate) == ((x_coordinate + length_of_light - 1) - centre) )
		    {
			status = CWORD_S_NORMAL;
			is_balanced = TRUE;
		    }

		}
	    }
	}
    }

    if (CWORD_S_FAILURE == status)
    {
	/*
	 *  Continue as normal
	 */

	cword_messages_get_message (CWORD_180SYMMETRYMSG, temp_str);

	cword_messages_output_message (
		CWORD_GENVIOLATE,
		(TRACE2 | SCREEN0),
		temp_str);

	return (CWORD_S_NORMAL);
    }

    /* 
     *  Add the originally defined light
     */

    if (action == ADD)
    {
	cword_generate_l_new_add_light.x_coordinate = x_coordinate;
	cword_generate_l_new_add_light.y_coordinate = y_coordinate;
	cword_generate_l_new_add_light.answer_length = length_of_light;
	cword_generate_l_new_add_light.clue_direction = light_direction;

	cword_generate__add_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else
    {
	cword_generate_l_new_del_light.x_coordinate = x_coordinate;
	cword_generate_l_new_del_light.y_coordinate = y_coordinate;
	cword_generate_l_new_del_light.answer_length = length_of_light;
	cword_generate_l_new_del_light.clue_direction = light_direction;

	cword_generate__del_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }

    if (is_balanced == FALSE)
    {
	/*
	 *  Calculate the other light location where the light is supposed to be
	 *  inserted into the crossword record.
	 *
	 *  Light length is the same but the direction, x and y coordinates will
	 *  change.
	 *
	 *  If the inputted light is balanced around the centre then don't
	 *  execute this!
	 */

	x_defecit = abs (x_coordinate - 1);
	y_defecit = abs (y_coordinate - crossword_record->y_dimensions);

	if (light_direction == ACROSS)
	{
	    /* 
	     *  light is ACROSS in quadrant 3 if original is in quadrant 1
	     */

	    x = crossword_record->x_dimensions - x_defecit - length_of_light + 1;
	    y = 1 + y_defecit;
	    direction = ACROSS;

	    if (action == ADD)
	    {
		cword_generate__add_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	    else
	    {
		cword_generate__del_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	}
	else
	{
	    /* 
	     *  light is DOWN in quadrant 3 if original is in quadrant 1
	     */

	    x = crossword_record->x_dimensions - x_defecit;
	    y = 1 + y_defecit + length_of_light - 1;
	    direction = DOWN;

	    if (action == ADD)
	    {
		cword_generate__add_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	    else
	    {
		cword_generate__del_light_loc (
				x,
				y,
				direction,
				length_of_light,
				crossword_record);
	    }
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__action_lraxial (
	ACTION			action,	
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to help add or delete the light locations to complete a top
 *	*L*eft bottom *R*ight axial grid symmetry pattern given a single input
 *	light as guide.
 *
 *  FORMAL PARAMETERS:
 *
 * 	action			(r) ADD or DELETE
 *	x_coordinate		(r) x coordinate of the light to be added/deleted
 *	y_coordinate		(r) y coordinate of the light to be added/deleted
 *	light_direction		(r) direction of the light to be added/deleted
 *	length_of_light		(r) length of the answer of the light to be added/deleted
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_generate_l_new_add_light
 *	cword_generate_l_new_del_light
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Successful completion
 *--
 */

{

    char temp_str[CWORD_SIZEOF_LINE + 1];

    int	x,	
	y,
	direction,
	x_offset,
	y_offset;

    cword_messages_get_message (
		CWORD_AXIALLEFTRIGHTMSG, 
		temp_str);

    if (action == ADD)
    {
	cword_messages_output_message (
		    CWORD_ADDINGUSING, 
		    SCREEN0,
		    temp_str);
    }
    else
    {
	cword_messages_output_message (
		    CWORD_S_DELETINGUSING, 
		    SCREEN0,
		    temp_str);
    }

    /* 
     *  Add the originally defined light
     */

    if (action == ADD)
    {
	cword_generate_l_new_add_light.x_coordinate = x_coordinate;
	cword_generate_l_new_add_light.y_coordinate = y_coordinate;
	cword_generate_l_new_add_light.answer_length = length_of_light;
	cword_generate_l_new_add_light.clue_direction = light_direction;

	cword_generate__add_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else
    {
	cword_generate_l_new_del_light.x_coordinate = x_coordinate;
	cword_generate_l_new_del_light.y_coordinate = y_coordinate;
	cword_generate_l_new_del_light.answer_length = length_of_light;
	cword_generate_l_new_del_light.clue_direction = light_direction;

	cword_generate__del_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }

    if (light_direction == ACROSS)
    {
	x_offset = crossword_record->x_dimensions - x_coordinate - length_of_light + 1;
	y_offset = crossword_record->y_dimensions - y_coordinate;

	x = y_offset + 1;
	y = length_of_light + x_offset;
	direction = DOWN;

	if (action == ADD)
	{
	    cword_generate__add_light_loc (
			    x,
			    y,
			    direction,
			    length_of_light,
			    crossword_record);
	}
	else
	{
	    cword_generate__del_light_loc (
			    x,
			    y,
			    direction,
			    length_of_light,
			    crossword_record);
	}
    }
    else
    {
	x_offset = crossword_record->x_dimensions - x_coordinate;
	y_offset = crossword_record->y_dimensions - y_coordinate;

	x = y_offset + 1;
	y = x_offset + 1;
	direction = ACROSS;

	if (action == ADD)
	{
	    cword_generate__add_light_loc (
			    x,
			    y,
			    direction,
			    length_of_light,
			    crossword_record);
	}
	else
	{
	    cword_generate__del_light_loc (
			    x,
			    y,
			    direction,
			    length_of_light,
			    crossword_record);
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__action_rlaxial (
	ACTION			action,	
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to help add or delete the light locations to complete a top
 *	*R*ight bottom *L*eft axial grid symmetry pattern given a single input
 *	light as guide.
 *
 *  FORMAL PARAMETERS:
 *
 * 	action			(r) ADD or DELETE
 *	x_coordinate		(r) x coordinate of the light to be added/deleted
 *	y_coordinate		(r) y coordinate of the light to be added/deleted
 *	light_direction		(r) direction of the light to be added/deleted
 *	length_of_light		(r) length of the answer of the light to be added/deleted
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_generate_l_new_add_light
 *	cword_generate_l_new_del_light
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Successful completion
 *--
 */

{

    char temp_str[CWORD_SIZEOF_LINE + 1];

    int x,	
	y,
	direction,
	x_offset,
	y_offset;

    cword_messages_get_message (
		CWORD_AXIALRIGHTLEFTMSG, 
		temp_str);

    if (action == ADD)
    {
	cword_messages_output_message (
		    CWORD_ADDINGUSING, 
		    SCREEN0,
		    temp_str);
    }
    else
    {
	cword_messages_output_message (
		    CWORD_S_DELETINGUSING, 
		    SCREEN0,
		    temp_str);
    }

    /* 
     *  Add the originally defined light
     */

    if (action == ADD)
    {
	cword_generate_l_new_add_light.x_coordinate = x_coordinate;
	cword_generate_l_new_add_light.y_coordinate = y_coordinate;
	cword_generate_l_new_add_light.answer_length = length_of_light;
	cword_generate_l_new_add_light.clue_direction = light_direction;

	cword_generate__add_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else
    {
	cword_generate_l_new_del_light.x_coordinate = x_coordinate;
	cword_generate_l_new_del_light.y_coordinate = y_coordinate;
	cword_generate_l_new_del_light.answer_length = length_of_light;
	cword_generate_l_new_del_light.clue_direction = light_direction;

	cword_generate__del_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }

    if (light_direction == ACROSS)
    {
	x_offset = crossword_record->x_dimensions - x_coordinate - length_of_light + 1;
	y_offset = crossword_record->y_dimensions - y_coordinate;

	x = crossword_record->x_dimensions - y_offset;
	y = crossword_record->y_dimensions - x_offset;
	direction = DOWN;

	if (action == ADD)
	{
	    cword_generate__add_light_loc (
			    x,
			    y,
			    direction,
			    length_of_light,
			    crossword_record);
	}
	else
	{
	    cword_generate__del_light_loc (
			    x,
			    y,
			    direction,
			    length_of_light,
			    crossword_record);
	}
    }
    else
    {
	x_offset = crossword_record->x_dimensions - x_coordinate;
	y_offset = crossword_record->y_dimensions - y_coordinate;

	x = crossword_record->x_dimensions - y_offset - length_of_light + 1;
	y = crossword_record->y_dimensions - x_offset;
	direction = ACROSS;

	if (action == ADD)
	{
	    cword_generate__add_light_loc (
			    x,
			    y,
			    direction,
			    length_of_light,
			    crossword_record);
	}
	else
	{
	    cword_generate__add_light_loc (
			    x,
			    y,
			    direction,
			    length_of_light,
			    crossword_record);
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__action_nodeg (
	ACTION			action,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to add the inputted light location only - no symmetry is
 *	present in this defined grid type.  No extra calculations are needed
 *	other than to call the appropriate addition or deletion function.
 *
 *  FORMAL PARAMETERS:
 *
 * 	action			(r) ADD or DELETE
 *	x_coordinate		(r) x coordinate of the light to be added/deleted
 *	y_coordinate		(r) y coordinate of the light to be added/deleted
 *	light_direction		(r) direction of the light to be added/deleted
 *	length_of_light		(r) length of the answer of the light to be added/deleted
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_generate_l_new_add_light
 *	cword_generate_l_new_del_light
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Successful completion
 *--
 */

{

    char temp_str[CWORD_SIZEOF_LINE + 1];

    cword_messages_get_message (
		CWORD_NOSYMMETRYMSG, 
		temp_str);

    if (action == ADD)
    {
	cword_messages_output_message (
		    CWORD_ADDINGUSING, 
		    SCREEN0,
		    temp_str);
    }
    else
    {
	cword_messages_output_message (
		    CWORD_S_DELETINGUSING, 
		    SCREEN0,
		    temp_str);
    }

    /* 
     *  Add the originally defined light
     */

    if (action == ADD)
    {
	cword_generate_l_new_add_light.x_coordinate = x_coordinate;
	cword_generate_l_new_add_light.y_coordinate = y_coordinate;
	cword_generate_l_new_add_light.answer_length = length_of_light;
	cword_generate_l_new_add_light.clue_direction = light_direction;

	cword_generate__add_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else
    {
	cword_generate_l_new_del_light.x_coordinate = x_coordinate;
	cword_generate_l_new_del_light.y_coordinate = y_coordinate;
	cword_generate_l_new_del_light.answer_length = length_of_light;
	cword_generate_l_new_del_light.clue_direction = light_direction;

	cword_generate__del_light_loc (
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    return (CWORD_S_NORMAL);

}



PRIVATE BOOLEAN cword_generate_is_light_found (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that checks whether the inputted light definition is already
 *	in the grid.  Returns TRUE if found else FALSE.
 *
 *  FORMAL PARAMETERS:
 *
 *	x_coordinate		(r) x coordinate of the light to be checked
 *	y_coordinate		(r) y coordinate of the light to be checked
 *	light_direction		(r) direction of the light to be checked
 *	length_of_light		(r) length of the answer of the light to be checked
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Light found in crossword definition
 *	FALSE			Light not found in crossword definition
 *--
 */

{

    int count1;

    BOOLEAN is_found = FALSE;

    for (count1 = 0; count1 < crossword_record->clue_sum; count1++)
    {
	if ( (x_coordinate == crossword_record->clue_details[count1].x_coordinate) &&
	      (y_coordinate == crossword_record->clue_details[count1].y_coordinate) &&
	       (length_of_light == crossword_record->clue_details[count1].answer_length) &&
	        (light_direction == crossword_record->clue_details[count1].clue_direction) )
	{
	    is_found = TRUE;

	    break;
	}
    }

    return (is_found);

}



/*
 *  ?? Doesn't correctly identify that the light 5/7/A/4 is a centre clue in a
 *  12*13 grid using 180 degree symmetry ??
 */

PRIVATE BOOLEAN cword_generate__light_centre (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that checks whether the inputted light definition cuts the
 *	centre of the grid.  Returns TRUE if it does else FALSE.  This is
 *	important as it will affect grid symmetry if such a light is added
 *	without adequate checking.
 *
 *  FORMAL PARAMETERS:
 *
 *	x_coordinate		(r) x coordinate of the light to be checked
 *	y_coordinate		(r) y coordinate of the light to be checked
 *	light_direction		(r) direction of the light to be checked
 *	length_of_light		(r) length of the answer of the light to be checked
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Inputted light is a centre light.
 *	FALSE			Inputted light isn't a centre light.
 *--
 */

{

    int centre;

    /*
     *	Assumes the light being checked is valid before checking whether it is
     *	a centre light.
     *
     *	Also assumes that the grid being dealt with is square in dimensions -
     *	not correct!
     */

    /*
     *	If the grid's dimensions are odd then a centre light is possible.
     */

    if ((crossword_record->x_dimensions % 2) == 1)
    {
	centre = (int) ((float) (crossword_record->x_dimensions + 1) / (float) 2);

	/*
	 *
	 */

	if ( (x_coordinate == centre) || (y_coordinate == centre) )
	{
	    if (light_direction == ACROSS)
	    {
		if (y_coordinate == centre)
		{
		    if ( ((x_coordinate + length_of_light - 1) >= centre) && (x_coordinate <= centre) )
		    {
			cword_messages_output_message (
				CWORD_CENTRELIGHT, 
				(SCREEN0 | TRACE2));

			return (TRUE);
		    }
		}
	    }
	    else
	    {
		if (x_coordinate == centre)
		{
		    if ( ((y_coordinate - length_of_light + 1) <= centre) && (y_coordinate >= centre) )
		    {
			cword_messages_output_message (
				CWORD_CENTRELIGHT, 
				(SCREEN0 | TRACE2));

			return (TRUE);
		    }
		}
	    }
	}
    }

    return (FALSE);

}



PRIVATE BOOLEAN cword_generate__is_light_valid (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light)
			
/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function performs a series of tests on an inputted light
 *	definition to verify that it is acceptable for addition/deletion
 *	to/from the grid being created.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r) pointer to an instance of a crossword_record
 *	x_coordinate		(r) x coordinate of the light to be added
 *	y_coordinate		(r) y coordinate of the light to be added
 *	light_direction		(r) direction of the light to be added
 *	length_of_light		(r) length of the answer of the light to be added
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Light is found to be valid	
 *	FALSE			Light is found to be invalid
 *--
 */

{

    CROSSWORD_GRID double_check;

    int count1,
	count2,
	temp;

    cword_messages_output_message (
	    CWORD_CHECKLIGHTVALID,
	    TRACE2,
	    x_coordinate, 
	    y_coordinate,
	    light_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
	    length_of_light);

    for (count1 = 0; count1 < CWORD_SIZEOF_CELLDIM; count1++)
	for (count2 = 0; count2 < CWORD_SIZEOF_CELLDIM; count2++)
	{
	    /*
	     *	?? Don't like the use of NOT_TAKEN ??
	     */

	    double_check[count1][count2] = NOT_TAKEN;
	}

    /* 
     *	Construct an array structure to check for any lights on the definition
     *	which may occupy the same across/down space => invalid configuration.
     */

    /* 
     *  Construct the check double_check array.
     */

    for (count1 = 0; count1 < crossword_record->clue_sum; count1++)
    {
	if (crossword_record->clue_details[count1].clue_direction == ACROSS)
	{
	    for (count2 = crossword_record->clue_details[count1].x_coordinate;
		  count2 < (crossword_record->clue_details[count1].x_coordinate + 
			     crossword_record->clue_details[count1].answer_length);
		   count2++)
	    {
		temp = crossword_record->clue_details[count1].y_coordinate;

		/*
		 *  Bitwise AND operation
		 */

		if (ACROSS_LIGHT & double_check[count2-1][temp-1])
		{
		    cword_messages_output_message (
			    CWORD_LIGHTSSAMEDIR,
			    (SCREEN0 | TRACE2),
			    count2, 
			    temp);

		    return (FALSE);
		}

		double_check[count2-1][temp-1] |= ACROSS_LIGHT;
	    }
	}
	else
	{
	    for (count2 = crossword_record->clue_details[count1].y_coordinate;
		  count2 > (crossword_record->clue_details[count1].y_coordinate -
			     crossword_record->clue_details[count1].answer_length);
		   count2--)
	    {
		temp = crossword_record->clue_details[count1].x_coordinate;

		/*
		 *  Bitwise AND operation
		 */
		
		if (DOWN_LIGHT & double_check[temp-1][count2-1])
		{
		    cword_messages_output_message (
			    CWORD_LIGHTSSAMEDIR,
			    (SCREEN0 | TRACE2),
			    temp, 
			    count2);

		    return (FALSE);
		}

		double_check[temp-1][count2-1] |= DOWN_LIGHT;
	    }
	}
    }

    /*
     *  Check with the new light now
     */

    if (light_direction == ACROSS)
    {
	temp = y_coordinate;
	for (count2 = x_coordinate; count2 < (x_coordinate + length_of_light); count2++)
	{
	    /*
	     *  Bitwise AND operation
	     */

	    if (ACROSS_LIGHT & double_check[count2-1][temp-1])
	    {
		cword_messages_output_message (
			CWORD_LIGHTSSAMEDIR,
			(SCREEN0 | TRACE2),
			count2, 
			temp);

		return (FALSE);
	    }

	    double_check[count2-1][temp-1] |= ACROSS_LIGHT;
	}
    }
    else
    {
	temp = x_coordinate;
	for (count2 = y_coordinate; count2 > (y_coordinate - length_of_light); count2--)
	{
	    /*
	     *  Bitwise AND operation
	     */
	    
	    if (DOWN_LIGHT & double_check[temp-1][count2-1])
	    {
		cword_messages_output_message (
			CWORD_LIGHTSSAMEDIR,
			(SCREEN0 | TRACE2),
			temp, 
			count2);

		return (FALSE);
	    }

	    double_check[temp-1][count2-1] |= DOWN_LIGHT;
	}
    }

    return (TRUE);

}



/*
 *  ?? This is the order that the user should be prompted for grid properties ??
 *
 *	1. X
 *	2. Y
 *	3. Grid type [if X != Y then block/barred/novelty only]
 *	4. Symmetry type [if X != Y then 180 or none]  [if american then 90 only]
 *
 *  ?? This will get rid of the restriction whereby novelty grid can have only
 *  _none_ symmetry and also other grid with uneven side having _none_ symmetry
 *  too ??
 */

/*
 *  ?? When adding an Across or Down light it should be possible to 
 *  specify that the light should have length of whatever is left
 *  between it and the side of the grid bounding it ??
 *
 *  e.g. Specify a light at (8,8) Across and then input '-' to specify 
 *  that the light will have length of (x_dimension - 8 + 1).
 */

PRIVATE int cword_generate__get_grid_props (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to obtain the user definition of the grid to be created.  Such
 *	things as grid dimensions, symmetry, and grid types are inputted and
 *	defined here.
 *
 *	This function fills the rarely used fields of the crossword record -
 *	symmetry and grid_type!
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(w) pointer to struct containing crossword details
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.x_dimensions
 *      cword_g_defaults_record.y_dimensions
 *	cword_g_defaults_record.smallest_allowable_light
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal Completion
 *--
 */

{

    char value_str[CWORD_SIZEOF_MESSAGE + 1],
	 temp_str[CWORD_SIZEOF_MESSAGE + 1],
	 temp1_str[CWORD_SIZEOF_MESSAGE + 1];

    int	temp_int;

    /*
     *  Find out the following (in order):
     *
     *  1. grid type
     *  2. grid symmetry
     *  3. X
     *  4. Y
     */

    cword_messages_output_message (CWORD_INPUTGRIDDIMQ, SCREEN0);

    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

    /*
     *  ***** Grid type *****
     */

    cword_messages_output_message (
	    CWORD_INPUTGRIDTYPEMSG,
	    SCREEN0);

    cword_messages_output_message (CWORD_01MSG, SCREEN0);
    cword_messages_output_message (CWORD_S_GRID, SCREEN0, CWORD_BLOCK_GRID_STR);
    cword_messages_output_message (CWORD_02MSG, SCREEN0);
    cword_messages_output_message (CWORD_S_GRID, SCREEN0, CWORD_BARRED_GRID_STR);
    cword_messages_output_message (CWORD_03MSG, SCREEN0);
    cword_messages_output_message (CWORD_S_GRID, SCREEN0, CWORD_AMERICAN_GRID_STR);
    cword_messages_output_message (CWORD_04MSG, SCREEN0);
    cword_messages_output_message (CWORD_S_GRID, SCREEN0, CWORD_NOVELTY_GRID_STR);

    cword_messages_output_message (
	    CWORD_INPUTGRIDTYPE, 
	    SCREEN0,
	    1);

    temp_int = (int) cword_charui_get_ch (FALSE) - 48; /* '1' - 48 == 1 *or* '2' - 48 == 2 */

    if (temp_int == 4)
    {
	cword_messages_output_message (
		CWORD_GRIDTYPECHOSEN,
		(TRACE0 | SCREEN0),
		CWORD_NOVELTY_GRID_STR);

	crossword_record->grid_type = NOVELTY_GRID;
    }
    else if (temp_int == 3)
    {
	cword_messages_output_message (
		CWORD_GRIDTYPECHOSEN,
		(TRACE0 | SCREEN0),
		CWORD_AMERICAN_GRID_STR);

	crossword_record->grid_type = AMERICAN_GRID;
    }
    else if (temp_int == 2)
    {
	cword_messages_output_message (
		CWORD_GRIDTYPECHOSEN,
		(TRACE0 | SCREEN0),
		CWORD_BARRED_GRID_STR);

	crossword_record->grid_type = BARRED_GRID;
    }
    else
    {
	cword_messages_output_message (
		CWORD_GRIDTYPECHOSEN,
		(TRACE0 | SCREEN0),
		CWORD_BLOCK_GRID_STR);

	crossword_record->grid_type = BLOCK_GRID;
    }

    /*
     *  If Novelty then anything goes!
     *	If Block then anything goes!
     *	If Barred then anything goes!
     *  
     *	If American then only 1 and 2 go!
     */

    /*
     *  ***** Symmetry type *****
     */

    cword_messages_output_message (CWORD_INPUTSYMMTYPEMSG, SCREEN0);

    cword_messages_output_message (CWORD_01MSG, SCREEN0);
    cword_messages_output_message (CWORD_90SYMMETRYMSG, SCREEN0);
    cword_messages_output_message (CWORD_S_SYMMETRY, SCREEN0);
    cword_messages_output_message (CWORD_02MSG, SCREEN0);
    cword_messages_output_message (CWORD_180SYMMETRYMSG, SCREEN0);
    cword_messages_output_message (CWORD_S_SYMMETRY, SCREEN0);

    if (crossword_record->grid_type != AMERICAN_GRID)
    {
	cword_messages_output_message (CWORD_03MSG, SCREEN0);
	cword_messages_output_message (CWORD_AXIALLEFTRIGHTMSG, SCREEN0);
	cword_messages_output_message (CWORD_S_SYMMETRY, SCREEN0);
	cword_messages_output_message (CWORD_04MSG, SCREEN0);
	cword_messages_output_message (CWORD_AXIALRIGHTLEFTMSG, SCREEN0);
	cword_messages_output_message (CWORD_S_SYMMETRY, SCREEN0);
	cword_messages_output_message (CWORD_05MSG, SCREEN0);
	cword_messages_output_message (CWORD_NOSYMMETRYMSG, SCREEN0);
	cword_messages_output_message (CWORD_S_SYMMETRY, SCREEN0);
    }

    cword_messages_output_message (
	CWORD_INPUTSYMMTYPE, 
	SCREEN0,
	(crossword_record->grid_type != AMERICAN_GRID) ? cword_g_defaults_record.symmetry_type : DEG_90_SYMMETRY);

    if (CWORD_S_NORMAL == cword_charui_get_str (
				DIGIT, 
				BLANK_OK,
				sizeof (value_str) - 1,
				value_str))
    {
	if (crossword_record->grid_type != AMERICAN_GRID)
	{
	    if ( (atoi (value_str) > 5) || (atoi (value_str) < 1) )
	    {
		cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);

		cword_messages_output_message (
				    CWORD_SYMMTYPECHOSEN, 
				    (TRACE0 | SCREEN0),
				    temp_str);

		crossword_record->symmetry = DEG_90_SYMMETRY;
	    }
	    else
	    {
		switch (atoi (value_str))
		{
		case 1:
		    cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    crossword_record->symmetry = DEG_90_SYMMETRY;
		    break;

		case 2:
		    cword_messages_get_message (CWORD_180SYMMETRYMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    crossword_record->symmetry = DEG_180_SYMMETRY;
		    break;

		case 3:
		    cword_messages_get_message (CWORD_AXIALLEFTRIGHTMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    crossword_record->symmetry = AXIAL_LR_SYMMETRY;
		    break;

		case 4:
		    cword_messages_get_message (CWORD_AXIALRIGHTLEFTMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    crossword_record->symmetry = AXIAL_RL_SYMMETRY;
		    break;

		case 5:
		    cword_messages_get_message (CWORD_NOSYMMETRYMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    crossword_record->symmetry = NO_SYMMETRY;
		    break;

		default:
		    cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    /*
		     *  Default value 
		     */

		    crossword_record->symmetry = cword_g_defaults_record.symmetry_type; 
		    break;
		}
	    }	
	}
	else
	{
	    if ( (atoi (value_str) > 3) || (atoi (value_str) < 1) )
	    {
		cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);

		cword_messages_output_message (
				    CWORD_SYMMTYPECHOSEN, 
				    (TRACE0 | SCREEN0),
				    temp_str);
		crossword_record->symmetry = DEG_90_SYMMETRY;
	    }
	    else
	    {
		switch (atoi (value_str))
		{
		case 1:
		    cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    crossword_record->symmetry = DEG_90_SYMMETRY;
		    break;

		case 2:
		    cword_messages_get_message (CWORD_180SYMMETRYMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    crossword_record->symmetry = DEG_180_SYMMETRY;
		    break;

		default:
		    cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);
		    cword_messages_output_message (
					CWORD_SYMMTYPECHOSEN, 
					(TRACE0 | SCREEN0),
					temp_str);

		    crossword_record->symmetry = DEG_90_SYMMETRY;
		    break;
		}
	    }
	}
    }
    else
    {
	cword_messages_get_message (CWORD_90SYMMETRYMSG, temp_str);

	cword_messages_output_message (
			    CWORD_SYMMTYPECHOSEN, 
			    (TRACE0 | SCREEN0),
			    temp_str);
	crossword_record->symmetry = DEG_90_SYMMETRY; /* Default value */
    }

    /*
     *  ***** X *****
     */

    cword_messages_output_message (
	    CWORD_INPUTGRIDX,
	    SCREEN0,
	    CWORD_X_CHAR,
	    cword_g_defaults_record.smallest_allowable_light, 
	    CWORD_SIZEOF_CELLDIM,
	    cword_g_defaults_record.x_dimensions);

    if (CWORD_S_NORMAL == cword_charui_get_str (
				DIGIT, 
				BLANK_OK,
				sizeof (value_str) - 1,
				value_str))
    {
	if (value_str[0] == (char) 0)
	    crossword_record->x_dimensions = cword_g_defaults_record.x_dimensions;
	else
	    crossword_record->x_dimensions = atoi (value_str);
    }
    else
	crossword_record->x_dimensions = cword_g_defaults_record.x_dimensions;

    if (crossword_record->x_dimensions > CWORD_SIZEOF_CELLDIM)
    {
	cword_messages_output_message (
		CWORD_MAXGRIDDIMS,
		SCREEN0,
		CWORD_SIZEOF_CELLDIM); 

	crossword_record->x_dimensions = CWORD_SIZEOF_CELLDIM;
    }
    else if (crossword_record->x_dimensions < cword_g_defaults_record.smallest_allowable_light)
    {
	cword_messages_output_message (
		CWORD_MINGRIDDIMS,
		SCREEN0,
		cword_g_defaults_record.smallest_allowable_light); 

	crossword_record->x_dimensions = cword_g_defaults_record.smallest_allowable_light;
    }

    cword_messages_output_message (
	    CWORD_S_DIMSSETTO,
	    (TRACE0 | SCREEN0),
	    CWORD_X_CHAR,
	    crossword_record->x_dimensions);

    /*
     *  ***** Y *****
     */

    if (crossword_record->grid_type != AMERICAN_GRID)
    {
	cword_messages_output_message (
		CWORD_INPUTGRIDY,
		SCREEN0,
		CWORD_Y_CHAR,
		cword_g_defaults_record.smallest_allowable_light, 
		CWORD_SIZEOF_CELLDIM,
		crossword_record->x_dimensions);

	if (CWORD_S_NORMAL == cword_charui_get_str (
				    DIGIT, 
				    BLANK_OK,
				    sizeof (value_str) - 1,
				    value_str))
	{
	    if (value_str[0] == (char) 0)
		crossword_record->y_dimensions = crossword_record->x_dimensions;
	    else
		crossword_record->y_dimensions = atoi (value_str);
	}
	else
	    crossword_record->y_dimensions = crossword_record->x_dimensions;

	if (crossword_record->y_dimensions > CWORD_SIZEOF_CELLDIM)
	{
	    cword_messages_output_message (
		    CWORD_MAXGRIDDIMS,
		    SCREEN0,
		    CWORD_SIZEOF_CELLDIM); 

	    crossword_record->y_dimensions = CWORD_SIZEOF_CELLDIM;
	}
	else
	if (crossword_record->y_dimensions < cword_g_defaults_record.smallest_allowable_light)
	{
	    cword_messages_output_message (
		    CWORD_MINGRIDDIMS,
		    SCREEN0,
		    cword_g_defaults_record.smallest_allowable_light); 

	    crossword_record->y_dimensions = cword_g_defaults_record.smallest_allowable_light;
	}

	cword_messages_output_message (
		CWORD_S_DIMSSETTO,
		(TRACE0 | SCREEN0),
		CWORD_Y_CHAR,
		crossword_record->x_dimensions);

	if (crossword_record->x_dimensions != crossword_record->y_dimensions)
	    cword_messages_output_message (
			CWORD_XNOTEQUALY, 
			SCREEN0,
			CWORD_X_CHAR,
			CWORD_Y_CHAR);
    }
    else
    {
	cword_messages_output_message (
		CWORD_S_AMERICANYDIMS,
		(TRACE0 | SCREEN0),
		CWORD_Y_CHAR,
		CWORD_X_CHAR,
		CWORD_AMERICAN_GRID_STR);

	crossword_record->y_dimensions = crossword_record->x_dimensions;
    }

    /*
     *  Cword has to reset the selected symmetry type to none if X != Y.
     *
     *	This is a major hack, but it has to be done so that the grid creation
     *	logic isn't completely violated!  I can't have people crashing Cword at
     *	every hands turn - adding lights that are invalid (in my mind and in
     *	cword's mind anyhow).
     */

    if (crossword_record->x_dimensions != crossword_record->y_dimensions)
    {
	cword_messages_output_message (
		CWORD_S_RESETSYMM,
		(TRACE0 | SCREEN0));

	crossword_record->symmetry = NO_SYMMETRY;
    } 

    /*
     *  Output the options the user has chosen before continuing.
     */

    if (crossword_record->grid_type == BLOCK_GRID)
	strcpy (temp_str, CWORD_BLOCK_GRID_STR);
    else if (crossword_record->grid_type == BARRED_GRID)
	strcpy (temp_str, CWORD_BARRED_GRID_STR);
    else if (crossword_record->grid_type == AMERICAN_GRID)
	strcpy (temp_str, CWORD_AMERICAN_GRID_STR);
    else 
	strcpy (temp_str, CWORD_NOVELTY_GRID_STR);

    if (crossword_record->symmetry == DEG_90_SYMMETRY)
	cword_messages_get_message (CWORD_90SYMMETRYMSG, temp1_str);
    else if (crossword_record->symmetry == DEG_180_SYMMETRY)
	cword_messages_get_message (CWORD_180SYMMETRYMSG, temp1_str);
    else if (crossword_record->symmetry == AXIAL_LR_SYMMETRY)
	cword_messages_get_message (CWORD_AXIALLEFTRIGHTMSG, temp1_str);
    else if (crossword_record->symmetry == AXIAL_RL_SYMMETRY)
	cword_messages_get_message (CWORD_AXIALRIGHTLEFTMSG, temp1_str);
    else if (crossword_record->symmetry == NO_SYMMETRY)
	cword_messages_get_message (CWORD_NOSYMMETRYMSG, temp1_str);
    else
	cword_messages_get_message (CWORD_NOSYMMETRYMSG, temp1_str);

    cword_messages_output_message (
		CWORD_GRIDDEFINITION,
		SCREEN0,
		crossword_record->x_dimensions,
		crossword_record->y_dimensions,	
		temp_str,
		temp1_str);

    return (CWORD_S_NORMAL);

}



/*
 *  ?? There is at least one bug in this code that may be hard to dig out
 *  without radically changing the code.  This is where the associated
 *  subsequent lights to the light being added causes the adjacency rule to be
 *  broken.  To check this properly, the two or four lights being added should
 *  all be added and then the resulting grid checked.  If the grid fails verify
 *  then the seed light is refused.  This is a fairly big overhead and really
 *  isn't worth it within the current design of this module ??
 */

PRIVATE int cword_generate__check_adjacency (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function will check the input light definition for suitability in
 *	the grid definition being built.  It will only check the input light
 *	definition and will not be used to check any of the other possible
 *	light combinations which arise from the symmetry option chosen.  I
 *	assume if the inputted light is OK then the rest of the lights being
 *	generated from it will also be OK.  This assumption will not always be
 *	correct though :-(.
 *
 *	This function is only used on block grid where adjacent lights are
 *	almost non existant.  It will check that the light being input will not
 *	cause possible mis-interpretation when the grid is being printed or
 *	being solved.  For example it will not allow a configuration whereby
 *	two ACROSS lights will be laid out one after the other:
 *
 *                   -------------
 *	             |XXXXXYYYYYY|
 *
 *	where X is light 1 and Y is light 2.  For the solver this scenario is
 *	confusing and it isn't allowed in the crossword puzzles I've seen.
 *
 *  FORMAL PARAMETERS:
 *
 *	*crossword_record	(r) pointer to instance of crossword record
 *	x_coordinate		(r) x coordinate of the light to be added
 *	y_coordinate		(r) y coordinate of the light to be added
 *	light_direction		(r) direction of the light to be added
 *	length_of_light		(r) length of the answer of the light to be added
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Input light is acceptable
 *	CWORD_S_FAILURE		Input light is not acceptable
 *--
 */

{

    int	    count1;

    BOOLEAN is_light_ok = TRUE,
	    is_adjacent_warn = FALSE;

    for (count1 = 0; count1 < crossword_record->clue_sum; count1++)
    {
	DEBUG_TRACE (
		"old x is %d, old y is %d - new x is %d, new y is %d - old dir is %d new dir is %d\n",
		crossword_record->clue_details[count1].x_coordinate,
		crossword_record->clue_details[count1].y_coordinate,
		x_coordinate,
		y_coordinate,
		crossword_record->clue_details[count1].clue_direction,
		light_direction);

	if (crossword_record->clue_details[count1].clue_direction == light_direction)
	{
	    /*
	     *  Both lights are in same direction
	     */

	    if (light_direction == ACROSS)
	    {
		if ( (((crossword_record->clue_details[count1].x_coordinate + 
			 crossword_record->clue_details[count1].answer_length - 1) == (x_coordinate - 1))
						       &&
			(crossword_record->clue_details[count1].y_coordinate == y_coordinate))
						||
		      (((x_coordinate + length_of_light - 1) == (crossword_record->clue_details[count1].x_coordinate -1))
						       &&
		       (y_coordinate == crossword_record->clue_details[count1].y_coordinate)))
		{
		    cword_messages_output_message (
			    CWORD_GENADJACENT,
			    (TRACE0 | SCREEN0),
			    light_direction == ACROSS ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		    is_light_ok = FALSE; 
		}
		else
		{
		    if ( (((crossword_record->clue_details[count1].y_coordinate == (y_coordinate + 1)) ||
			    (crossword_record->clue_details[count1].y_coordinate == (y_coordinate - 1)))
						       &&
			 ((crossword_record->clue_details[count1].x_coordinate >= x_coordinate) &&
			   (crossword_record->clue_details[count1].x_coordinate <= (x_coordinate + length_of_light - 1))))
						||
			(((y_coordinate == (crossword_record->clue_details[count1].y_coordinate + 1)) ||
			   (y_coordinate == (crossword_record->clue_details[count1].y_coordinate - 1)))
						       &&
			 ((x_coordinate >= crossword_record->clue_details[count1].x_coordinate) &&
			   (x_coordinate <= (crossword_record->clue_details[count1].x_coordinate + 
			 		      crossword_record->clue_details[count1].answer_length - 1)))) )
		    {
			if (is_adjacent_warn == FALSE)
			{
			    cword_messages_output_message (
				    CWORD_GENADJACWARN,
				    (TRACE2 | SCREEN2));

			    is_adjacent_warn = TRUE;
			}
		    }
		}
            }
	    else
	    {
		if ( (((crossword_record->clue_details[count1].y_coordinate -
			 crossword_record->clue_details[count1].answer_length + 1) == (y_coordinate + 1))
						       &&
			(crossword_record->clue_details[count1].x_coordinate == x_coordinate))
						||
		      (((y_coordinate - length_of_light + 1) == (crossword_record->clue_details[count1].y_coordinate + 1))
						       &&
		       (crossword_record->clue_details[count1].x_coordinate == x_coordinate)))
		{
		    cword_messages_output_message (
			    CWORD_GENADJACENT,
			    (TRACE2 | SCREEN0),
			    light_direction == ACROSS ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		    is_light_ok = FALSE; 
		}
		else
		{
		    if ( ((crossword_record->clue_details[count1].x_coordinate == (x_coordinate - 1)) ||
			   (crossword_record->clue_details[count1].x_coordinate == (x_coordinate + 1)))
						  &&
			 ((y_coordinate <= crossword_record->clue_details[count1].y_coordinate) &&
			   (y_coordinate >= (crossword_record->clue_details[count1].y_coordinate - 
					      crossword_record->clue_details[count1].answer_length + 1)))
						  ||
			 ((x_coordinate == (crossword_record->clue_details[count1].x_coordinate - 1)) ||
			   (x_coordinate == (crossword_record->clue_details[count1].x_coordinate + 1)))
						  &&
			 ((crossword_record->clue_details[count1].y_coordinate <= y_coordinate) &&
			   (crossword_record->clue_details[count1].y_coordinate >= (y_coordinate - 
										     length_of_light + 1))) )
		    {
			if (is_adjacent_warn == FALSE)
			{
			    cword_messages_output_message (
				    CWORD_GENADJACWARN,
				    (TRACE2 | SCREEN2));

			    is_adjacent_warn = TRUE;
		        }
		    }
		}
	    }
	}
	else
	{
	    /*
	     *  If lights are in opposite directions
	     */

	    if ( (crossword_record->clue_details[count1].clue_direction == ACROSS) && 
	          (light_direction == DOWN) )
            {
		if (
		    (crossword_record->clue_details[count1].x_coordinate == (x_coordinate + 1))
					      &&
		    (
		     (crossword_record->clue_details[count1].y_coordinate <= y_coordinate) &&
		     (crossword_record->clue_details[count1].y_coordinate >= (y_coordinate - 
									       length_of_light + 1))
		    )
		   )
		{
		    cword_messages_output_message (
			    CWORD_GENADJACERROR,
			    (TRACE2 | SCREEN0));

		    WRITE_TRACE ("# Problem - (OLD = ACROSS/HORIZONTAL/RIGHT - New = DOWN/VERTICAL/LEFT)\n");

		    is_light_ok = FALSE;
		}

		if (
		    ((crossword_record->clue_details[count1].x_coordinate +
		       crossword_record->clue_details[count1].answer_length - 1 + 1) == x_coordinate)
					      &&
		    (
		     (crossword_record->clue_details[count1].y_coordinate <= y_coordinate) &&
		     (crossword_record->clue_details[count1].y_coordinate >= (y_coordinate -
									       length_of_light + 1))
		    ) 
		   )
		{
		    cword_messages_output_message (
			    CWORD_GENADJACERROR,
			    (TRACE2 | SCREEN0));

		    WRITE_TRACE ("# Problem - (OLD = ACROSS/HORIZONTAL/LEFT - New = DOWN/VERTICAL/RIGHT)\n");
		    is_light_ok = FALSE;
		}

		if (
		    (crossword_record->clue_details[count1].y_coordinate == (y_coordinate + 1))
					      &&
		    (
		     (x_coordinate >= crossword_record->clue_details[count1].x_coordinate) &&
		     (x_coordinate <= (crossword_record->clue_details[count1].x_coordinate +
					crossword_record->clue_details[count1].answer_length - 1))
		    )
		   )
		{
		    cword_messages_output_message (
			    CWORD_GENADJACERROR,
			    (TRACE2 | SCREEN0));

		    WRITE_TRACE ("# Problem - (OLD = ACROSS/HORIZONTAL/ABOVE - New = DOWN/VERTICAL/BELOW)\n");
		    is_light_ok = FALSE;
		}

		if (
		    (crossword_record->clue_details[count1].y_coordinate == (y_coordinate - 
									      length_of_light + 1 - 1))
					      &&
		    (
		     (x_coordinate >= crossword_record->clue_details[count1].x_coordinate) &&
		     (x_coordinate <= (crossword_record->clue_details[count1].x_coordinate + 
					crossword_record->clue_details[count1].answer_length - 1))
		    )
		   )
		{
		    cword_messages_output_message (
			    CWORD_GENADJACERROR,
			    (TRACE2 | SCREEN0));

		    WRITE_TRACE ("# Problem - (OLD = ACROSS/HORIZONTAL/BELOW - New = DOWN/VERTICAL/ABOVE)\n");
		    is_light_ok = FALSE;
		}
	    }

	    if ( (crossword_record->clue_details[count1].clue_direction == DOWN) && 
	          (light_direction == ACROSS) )
            {
		if (
		    ((x_coordinate + length_of_light - 1 + 1) == 
		       crossword_record->clue_details[count1].x_coordinate)
					      &&
		    (
		     (y_coordinate <= crossword_record->clue_details[count1].y_coordinate) &&
		     (y_coordinate >= (crossword_record->clue_details[count1].y_coordinate -
					crossword_record->clue_details[count1].answer_length + 1))
		    )
		   )
		{
		    cword_messages_output_message (
			    CWORD_GENADJACERROR,
			    (TRACE2 | SCREEN0));

		    WRITE_TRACE ("# Problem - (OLD = DOWN/VERTICAL/RIGHT - New = ACROSS/HORIZONTAL/LEFT)\n");
		    is_light_ok = FALSE;
		}

		if (
		    (x_coordinate == (crossword_record->clue_details[count1].x_coordinate + 1))
					      &&
		    (
		     (y_coordinate <= crossword_record->clue_details[count1].y_coordinate) &&
		     (y_coordinate >= (crossword_record->clue_details[count1].y_coordinate - 
					crossword_record->clue_details[count1].answer_length + 1))
		    )
		   )
		{
		    cword_messages_output_message (
			    CWORD_GENADJACERROR,
			    (TRACE2 | SCREEN0));

		    WRITE_TRACE ("# Problem - (OLD = DOWN/VERTICAL/LEFT - New = ACROSS/HORIZONTAL/RIGHT)\n");
		    is_light_ok = FALSE;
		}

		if (
		    (y_coordinate == (crossword_record->clue_details[count1].y_coordinate - 
				       crossword_record->clue_details[count1].answer_length + 1 - 1))
					      &&
		    (
		     (crossword_record->clue_details[count1].x_coordinate >= x_coordinate) &&
		     (crossword_record->clue_details[count1].x_coordinate <= (x_coordinate + 
									       length_of_light - 1))
		    )
		   )
		{
		    cword_messages_output_message (
			    CWORD_GENADJACERROR,
			    (TRACE2 | SCREEN0));

		    WRITE_TRACE ("# Problem - (OLD = DOWN/VERTICAL/ABOVE - New = ACROSS/HORIZONTAL/BELOW)\n");
		    is_light_ok = FALSE;
		}

		if (
		    (y_coordinate == (crossword_record->clue_details[count1].y_coordinate + 1))
					      &&
		    (
		     (crossword_record->clue_details[count1].x_coordinate >= x_coordinate) &&
		     (crossword_record->clue_details[count1].x_coordinate <= (x_coordinate +
									       length_of_light - 1))
		    ) 
		   )
		{
		    cword_messages_output_message (
			    CWORD_GENADJACERROR,
			    (TRACE2 | SCREEN0));

		    WRITE_TRACE ("# Problem - (OLD = DOWN/VERTICAL/BELOW - New = ACROSS/HORIZONTAL/ABOVE)\n");
		    is_light_ok = FALSE;
		}
	    }
	}

	if (is_light_ok == FALSE)
	    break;
    }

    if (is_light_ok == TRUE)
	return (CWORD_S_NORMAL);
    else
	return (CWORD_S_FAILURE);

}



PRIVATE int cword_generate__verify_coords (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check that the light definition to be worked into the
 *	developing grid is valid before doing anything else.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r) pointer to instance of CROSSWORD_RECORD
 *	x_coordinate		(r) coordinate of X
 *	y_coordinate		(r) coordinate of Y
 *	light_direction		(r) light direction
 *	length_of_light		(r) length of light
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]

 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal Completion
 *	CWORD_S_FAILURE		Error with light inputs
 *--
 */

{

    /*
     *  Validation of inputted light sequence
     */

    if ( (x_coordinate <= 0) || (y_coordinate <= 0) )
    {
	cword_messages_output_message (
	    CWORD_S_DIMZERO,
	    (TRACE2 | SCREEN0),
	    CWORD_X_CHAR,
	    CWORD_Y_CHAR);

	return (CWORD_S_FAILURE);
    }

    if (x_coordinate > crossword_record->x_dimensions)
    {
	cword_messages_output_message (
		CWORD_XORYOVERALLOWED,
		(TRACE2 | SCREEN0),
		CWORD_X_CHAR,
		crossword_record->x_dimensions);

	return (CWORD_S_FAILURE);
    }

    if (y_coordinate > crossword_record->y_dimensions)
    {
	cword_messages_output_message (
		CWORD_XORYOVERALLOWED,
		(TRACE2 | SCREEN0),
		CWORD_Y_CHAR,
		crossword_record->y_dimensions);

	return (CWORD_S_FAILURE);
    }

    if (length_of_light < cword_g_defaults_record.smallest_allowable_light)
    {
	cword_messages_output_message (
		CWORD_LIGHTTOOSMALL,
		(TRACE2 | SCREEN0),
		cword_g_defaults_record.smallest_allowable_light);

	return (CWORD_S_FAILURE);
    }

    if (light_direction == ACROSS)
    {
	if ((x_coordinate + length_of_light - 1) > crossword_record->x_dimensions)
	{
	    cword_messages_output_message (
		    CWORD_S_OVERSHOOTMAX,
		    (TRACE2 | SCREEN0),
		    x_coordinate, 
		    y_coordinate,
		    CWORD_X_CHAR);

	    return (CWORD_S_FAILURE);
	}

	if ( ((crossword_record->symmetry != NO_SYMMETRY) &&
               (crossword_record->symmetry != DEG_180_SYMMETRY)) 
                                  &&
             ((crossword_record->y_dimensions - x_coordinate - 
                              length_of_light + 1) < 0) )
	{
	    cword_messages_output_message (
		CWORD_S_INVALIDLIGHT,
		SCREEN0);

	    return (CWORD_S_FAILURE);
	}
    }

    if (light_direction == DOWN)
    {
	if ((y_coordinate - length_of_light + 1) < 1)
	{
	    cword_messages_output_message (
		    CWORD_S_UNDERSHOOTMIN,
		    (TRACE2 | SCREEN0),
		    x_coordinate, 
		    y_coordinate,
		    CWORD_Y_CHAR);

	    return (CWORD_S_FAILURE);
	}

        if ( ((crossword_record->symmetry != NO_SYMMETRY) && 
               (crossword_record->symmetry != DEG_180_SYMMETRY))
                                  && 
	     ((crossword_record->x_dimensions - y_coordinate + length_of_light) >
	        crossword_record->x_dimensions) )
	{
	    cword_messages_output_message (
		CWORD_S_INVALIDLIGHT,
		SCREEN0);

	    return (CWORD_S_FAILURE);
	}
    }

    if (crossword_record->grid_type == BLOCK_GRID)
    {
	int status;

	status = cword_generate__check_adjacency (
		crossword_record,
		x_coordinate,
		y_coordinate,
		light_direction,
		length_of_light);

        if (CWORD_S_NORMAL != status)
	    return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? No defaults should be presented when adding a light, but when deleting,
 *  the light which was last added should be the default choice for deletion ??
 */

/*
 *  ?? Each part of the light being inputted should be checked after it's
 *  inputted and not at the end of the input phase when the full light
 *  definition has been provided ??
 *
 *  ?? Check with smallest_allowable_light too ??
 */

/*
 *  ?? The length of the inputted light should be checked with the existing
 *  word files and a message could be outputted if the length isn't found in
 *  the existing word files ??
 */

PRIVATE int cword_generate__get_light_coord (
	CROSSWORD_RECORD	*crossword_record,
	int			*x_coordinate,
	int			*y_coordinate,
	int			*light_direction,
	int			*length_of_light)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to obtain the user defined grid co-ordinates of the developing
 *	grid in a structured manner!
 *
 *  FORMAL PARAMETERS:
 *
 *	crossword_record	(r) pointer to instance of CROSSWORD_RECORD
 *	*x_coordinate		(w) coordinate of X
 *	*y_coordinate		(w) coordinate of Y
 *	*light_direction	(w) light direction
 *	*length_of_light	(w) length of light
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Received proper coordinates
 *--
 */

{

    char temp_str[11];

    /*
     *	Get s set of coordinates (valid or not) for a crossword light to be
     *	added to the grid.
     */

    cword_messages_output_message (
		CWORD_GETLIGHT,
		SCREEN0,
		CWORD_X_CHAR,
		crossword_record->x_dimensions);

    if (CWORD_S_NORMAL == cword_charui_get_str (
				DIGIT, 
				BLANK_OK,
				sizeof (temp_str) - 1,
				temp_str))
    {
	if (temp_str[0] == (char) 0)
	{
	    cword_messages_output_message (
			CWORD_S_NODEFAULTERR,
			SCREEN0);

	    *x_coordinate = *y_coordinate = *light_direction = *length_of_light = 0;
	    return (CWORD_S_FAILURE);
	}
	else
	    *x_coordinate  = atoi (temp_str);
    }
    else
    {
	cword_messages_output_message (
	    CWORD_S_GRIDINPUTERR,
	    SCREEN0);

	*x_coordinate = *y_coordinate = *light_direction = *length_of_light = 0;
	return (CWORD_S_FAILURE);
    }

    cword_messages_output_message (
		CWORD_GETLIGHT,
		SCREEN0,
		CWORD_Y_CHAR,
		crossword_record->y_dimensions);

    if (CWORD_S_NORMAL == cword_charui_get_str (
				DIGIT,
				BLANK_OK,
				sizeof (temp_str) - 1,
				temp_str))
    {
	if (temp_str[0] == (char) 0)
	{
	    cword_messages_output_message (
		CWORD_S_NODEFAULTERR,
		SCREEN0);

	    *x_coordinate = *y_coordinate = *light_direction = *length_of_light = 0;
	    return (CWORD_S_FAILURE);
	}
	else
	    *y_coordinate = atoi (temp_str);
    }
    else
    {
	cword_messages_output_message (
	    CWORD_S_GRIDINPUTERR,
	    SCREEN0);

	*x_coordinate = *y_coordinate = *light_direction = *length_of_light = 0;
	return (CWORD_S_FAILURE);
    }

    cword_messages_output_message (
		CWORD_GETLIGHTDIR,
		SCREEN0,
		CWORD_A_CHAR, 
		CWORD_D_CHAR);

    temp_str[0] = (char) toupper (cword_charui_get_ch (FALSE));

    if (temp_str[0] == CWORD_D_CHAR)
	*light_direction = DOWN;
    else if (temp_str[0] == CWORD_A_CHAR) 
	*light_direction = ACROSS;
    else
    {
	cword_messages_output_message (
		    CWORD_S_NODEFAULTERR,
		    SCREEN0);

	*x_coordinate = *y_coordinate = *light_direction = *length_of_light = 0;
	return (CWORD_S_FAILURE);
    }

    if (*light_direction == ACROSS)
    {
	cword_messages_output_message (
		    CWORD_GETLIGHTLEN,
		    SCREEN0,
		    crossword_record->x_dimensions - *x_coordinate + 1);
    }
    else
    {
	cword_messages_output_message (
		    CWORD_GETLIGHTLEN,
		    SCREEN0,
		    *y_coordinate);
    }

    if (CWORD_S_NORMAL == cword_charui_get_str (
				DIGIT,
				BLANK_OK,
				sizeof (temp_str) - 1,
				temp_str))
    {
	if (temp_str[0] == (char) 0)
	{
	    cword_messages_output_message (
			CWORD_S_NODEFAULTERR,
			SCREEN0);

	    *x_coordinate = *y_coordinate = *light_direction = *length_of_light = 0;

	    return (CWORD_S_FAILURE);
	}
	else
	{
	    /*
	     *	?? Check is the length of the inputted light OK here rather
	     *	than outside in another generate function ??
	     */

	    *length_of_light = atoi (temp_str);
	}
    }
    else
    {
	cword_messages_output_message (
	    CWORD_S_GRIDINPUTERR,
	    SCREEN0);

	*x_coordinate = *y_coordinate = *light_direction = *length_of_light = 0;
	return (CWORD_S_FAILURE);
    }

    if ( (*x_coordinate == 0) ||
	  (*y_coordinate == 0) ||
	   (*length_of_light == 0) )
    {
	cword_messages_output_message (CWORD_LIGHTVALUEZERO, (TRACE2 | SCREEN0));

	*x_coordinate = *y_coordinate = *light_direction = *length_of_light = 0;

	return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__verify_grid (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to control the verification of the grid each time a series of
 *	light definitions are added.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(w) pointer to instance of CROSSWORD_RECORD
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Logic of crossword grid is fine
 *	CWORD_S_FAILURE		Logic of crossword grid isn't OK.
 *--
 */

{

    int status;

    cword_messages_output_message (
	CWORD_S_CREATEVERIFY, 
	TRACE0);

    if (crossword_record->clue_sum == 0)
    {
	cword_messages_output_message (
		CWORD_S_NOLIGHTSDEFINED, 
		SCREEN0);

	return (CWORD_S_NORMAL);
    }

    cword_utils_complete_definition (crossword_record);

    cword_stats_gen_clue_nums (crossword_record);

    /*
     *  Whatever grid is chosen is then checked -- nothing else.
     */

    if (crossword_record->grid_type == BLOCK_GRID)
	status = cword_verify_block_grid_defn (crossword_record);
    else if (crossword_record->grid_type == BARRED_GRID)
	status = cword_verify_barred_grid_defn (crossword_record);
    else if (crossword_record->grid_type == AMERICAN_GRID)
	status = cword_verify_american_grid_defn (crossword_record);
    else
	status = cword_verify_novelty_grid_defn (crossword_record);

    if (CWORD_S_NORMAL == status)
    {
	cword_messages_output_message (
		    CWORD_S_USERGRIDVALID,
		    (TRACE0 | SCREEN0));

	return (CWORD_S_NORMAL);
    }
    else
    {
	cword_messages_output_message (
		    CWORD_S_USERGRIDINVALID,
		    (TRACE0 | SCREEN0));

        return (CWORD_S_FAILURE);
    }

}



PRIVATE int cword_generate__tidy_grid (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to tidy up the CROSSWORD_RECORD data structure before
 *	returning to the calling function through ...generate_grid ().
 *
 *	This function adds clue numbers to the grid definition and sorts the
 *	crossword before returning.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(w) pointer to instance of CROSSWORD_RECORD
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal Completion
 *	CWORD_S_FAILURE		Error recorded during tidy up and verify
 *--
 */

{

    char temp_str[CWORD_SIZEOF_MESSAGE + 1];

    int status;

    cword_messages_output_message (
	CWORD_S_CREATETIDY, 
	TRACE0);

    /*
     *	Check is there anything defined in the grid before deciding whether
     *	it should be saved or not.
     */

    if (crossword_record->clue_sum == 0)
    {
	cword_messages_output_message (
		    CWORD_S_NOLIGHTSDEFINED,
		    SCREEN0);

	return (CWORD_S_FAILURE);
    }

    /*
     *	Check if the grid type decided upon before creation is valid with the
     *	grid actually created.
     */

    status = cword_generate__verify_grid (crossword_record);

    if (CWORD_S_NORMAL != status)
    {
	cword_messages_output_message (
		    CWORD_S_GRIDNOTASDEFD1,
		    (TRACE0 | SCREEN0));

	cword_messages_output_message (
		    CWORD_S_GRIDNOTASDEFD2,
		    (TRACE0 | SCREEN0),
		    CWORD_Y_CHAR,
		    CWORD_N_CHAR,
		    CWORD_Y_CHAR);

	if (toupper (cword_charui_get_ch (FALSE)) == CWORD_N_CHAR)
	{
	    return (CWORD_S_FAILURE);
	}

	/*
	 *  Change the grid type of the crossword definition to the actual grid
	 *  definition.
	 */

	cword_messages_output_message (
		CWORD_S_CHANGEGRIDDEFN,
		(TRACE0 | SCREEN0));

	crossword_record->grid_type = cword_verify_get_grid_type (crossword_record);

	if (crossword_record->grid_type == BLOCK_GRID)
	    cword_messages_output_message (
		    CWORD_S_GRIDTYPECHANGED,
		    (TRACE0 | SCREEN0),
		    CWORD_BLOCK_GRID_STR);
	else if (crossword_record->grid_type == BARRED_GRID)
	    cword_messages_output_message (
		    CWORD_S_GRIDTYPECHANGED,
		    (TRACE0 | SCREEN0),
		    CWORD_BARRED_GRID_STR);
	else if (crossword_record->grid_type == AMERICAN_GRID)
	    cword_messages_output_message (
		    CWORD_S_GRIDTYPECHANGED,
		    (TRACE0 | SCREEN0),
		    CWORD_AMERICAN_GRID_STR);
	else
	    cword_messages_output_message (
		    CWORD_S_GRIDTYPECHANGED,
		    (TRACE0 | SCREEN0),
		    CWORD_NOVELTY_GRID_STR);
    }

    /*
     *  Sort the crossword before finishing with it!
     */

    cword_utils_sort_crossword (crossword_record);

    cword_messages_output_message (
	CWORD_SAVEGRIDEND,
	(TRACE0 | SCREEN0));

    cword_messages_output_message (
	CWORD_GRIDCALLED,
	SCREEN0,
	cword_g_defaults_record.temp_grid_name);

    if (CWORD_S_NORMAL == cword_charui_get_str (
			    ALL, 
			    BLANK_OK,
			    sizeof (temp_str) - 1,
			    temp_str))
    {
	if (temp_str[0] != (char) 0)
	{
	    cword_messages_output_message (
		    CWORD_GRIDTOBECALLED,
		    (TRACE0 | SCREEN0),
		    temp_str);

	    strcpy (crossword_record->name, temp_str);
	}
	else
	{
	    cword_messages_output_message (
		    CWORD_GRIDTOBECALLED,
		    (TRACE0 | SCREEN0),
		    cword_g_defaults_record.temp_grid_name);

	    strcpy (crossword_record->name, cword_g_defaults_record.temp_grid_name);
	}
    }
    else
    {
	cword_messages_output_message (
		CWORD_GRIDTOBECALLED,
		(TRACE0 | SCREEN0),
		cword_g_defaults_record.temp_grid_name);

	strcpy (crossword_record->name, cword_g_defaults_record.temp_grid_name);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__delete_lights (
	CROSSWORD_RECORD	*crossword_record,
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Overall function that controls the deletion of lights dependent on the
 *	symmetry of the defined grid.
 *
 *  FORMAL PARAMETERS:
 *
 *	x_coordinate		(r) x coordinate of the light to be added
 *	y_coordinate		(r) y coordinate of the light to be added
 *	light_direction		(r) direction of the light to be added
 *	length_of_light		(r) length of the answer of the light to be added
 *      *crossword_record	(w)pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Lights successfully deleted.
 *--
 */

{

    int status;

    /*
     *  Delete lights to evolving grid!
     *
     *	The mechanism for deleting adding lights is dependent on the grid type
     *	chosen earlier.
     */

    if (crossword_record->symmetry == DEG_90_SYMMETRY)
    {
	status = cword_generate__action_90deg (
			DELETE,
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else if (crossword_record->symmetry == DEG_180_SYMMETRY)
    {
	status = cword_generate__action_180deg (
			DELETE,
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else if (crossword_record->symmetry == AXIAL_LR_SYMMETRY)
    {
	status = cword_generate__action_lraxial (
			DELETE,	
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else if (crossword_record->symmetry == AXIAL_RL_SYMMETRY)
    {
	status = cword_generate__action_rlaxial (
			DELETE,
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else if (crossword_record->symmetry == NO_SYMMETRY)
    {
	status = cword_generate__action_nodeg (
			DELETE,
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }

    return (status);

}



PRIVATE int cword_generate__delete_grid_opts (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is called when a light definition is to be deleted.  It
 *	will check whether the definition is valid and if so it will delete it
 *	and its partners from the grid definition.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful
 *	CWORD_S_FAILRE		Failure
 *--
 */

{

    int x_coordinate,
	y_coordinate,
	light_direction,
	length_of_light,
	status;

    if (crossword_record->clue_sum == 0)
    {
	cword_messages_output_message (
		CWORD_S_NOLIGHTSDEFINED, 
		SCREEN0);

	return (CWORD_S_NORMAL);
    }

    status = cword_generate__get_light_coord (
		crossword_record,
		&x_coordinate,
		&y_coordinate,
		&light_direction,
		&length_of_light);

    if (status == CWORD_S_NORMAL)
    {
	/*
	 *  Check are the co-ordinates inputted already in the grid definition
	 *  being created.  If they aren't then this is an error.  If they are
	 *  then we can go ahead and delete them from the grid.
	 */

	if (cword_generate_is_light_found (
		    crossword_record,
		    x_coordinate,
		    y_coordinate,
		    light_direction,
		    length_of_light) == FALSE)
	{
	    cword_messages_output_message (
			CWORD_S_USERLHTNOTFOUND,
			(TRACE2 | SCREEN0));

	    return (CWORD_S_FAILURE);
	}
	else
	{
	    cword_generate__delete_lights (
		    crossword_record,
		    x_coordinate,
		    y_coordinate,
		    light_direction,
		    length_of_light);

	    return (CWORD_S_NORMAL);
	}
    }
    else
    {
	return (CWORD_S_FAILURE);
    }

}



PRIVATE int cword_generate__reset_grid (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to completely delete all defined lights from the grid.  This
 *	is a drastic action and should be taken with caution.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful
 *--
 */

{

    int count;

    cword_messages_output_message (
	CWORD_S_CREATERESET, 
	TRACE0);

    /*
     *  ?? Question user before continuing ??
     */

    if (crossword_record->clue_sum == 0)
    {
	cword_messages_output_message (
		CWORD_S_NOLIGHTSDEFINED, 
		SCREEN0);

	return (CWORD_S_NORMAL);
    }
    else
    {
	if (crossword_record->clue_sum == 1)
	{
	    cword_messages_output_message (
		    CWORD_S_RESETSINGLE, 
		    (TRACE0 | SCREEN0));
	}
	else
	{
	    cword_messages_output_message (
		    CWORD_S_RESETMULTIPLE, 
		    (TRACE0 | SCREEN0),
		    crossword_record->clue_sum);
	}
    }

    for (count = 0; count < crossword_record->clue_sum; count++)
    {
	crossword_record->clue_details[count].x_coordinate   = 0;
	crossword_record->clue_details[count].y_coordinate   = 0;
	crossword_record->clue_details[count].answer_length  = 0;
	crossword_record->clue_details[count].clue_direction = 0;	
    }

    crossword_record->clue_sum = 0;

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__list_grid (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that lists the currently defined lights in a format that is
 *	sometimes easier to read then that offered by viewing the grid alone.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r) Pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_generate_l_grid_updated
 *
 *  IMPLICIT OUTPUTS:
 *
 *      cword_generate_l_grid_updated
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion.
 *--
 */

{

    int count,
	count1,
	line_num;

    BOOLEAN are_over_99_clues;

    cword_messages_output_message (
	CWORD_S_CREATELIST, 
	TRACE0);

    if (crossword_record->clue_sum == 0)
    {
	cword_messages_output_message (
	    CWORD_S_NOLIGHTSDEFINED, 
	    SCREEN0);

	return (CWORD_S_NORMAL);
    }

    CWORD_OS_CLEAR_SCREEN ();


    if (cword_generate_l_grid_updated == TRUE)
    {
	cword_generate_l_grid_updated = FALSE;
    }

    /*
     *	?? Sort inputted lights into a form whereby it's possible to process
     *	them them in a more structured manner:
     *
     *  1. Sort into ACROSS and DOWN lights
     *  2. Sort from top left to bottom right  ??
     *
     *  cword_utils_sort_crossword() implicitly calls 
     *  cword_stats_gen_clue_nums() which is needed later on.
     */

	cword_utils_sort_crossword (crossword_record);

    /*
     *	Get the highest value clues for both the across and down clues for the
     *	user's reference.  This information can be especially useful if the
     *	user is transcribing a crossword grid from a newspaper or other
     *	magazine and wants to quickly verify whether all the light definitions
     *	have been inserted in the grid being created.
     *
     *	It is also extremely useful when printing out the light definitions!
     */

    if (cword_stats_get_high_clue_num (
	    BOTH,
	    crossword_record) > 99)
        are_over_99_clues = TRUE;
    else
        are_over_99_clues = FALSE;

    cword_messages_output_message (
	CWORD_S_CLUESDEFINED, 
	SCREEN0);

    /*
     *	Go through crossword definition outputting defined lights via
     *	cword_messages_output_message.
     */

    line_num = 2;
    for (count = 0; count < crossword_record->clue_sum; count++)
    {
	if ( (line_num == 0) || (count == 0) )
	{
	    cword_messages_output_message (
		CWORD_WAITINGENDED, 
		SCREEN0);

	    /*
	     *  Header information.
	     */

	    count1 = 0;
	    while (count1 < GEN_CLUES_PER_LINE)
	    {
		cword_messages_output_message (
		    CWORD_S_CREATOUTPUTHEAD,
		    SCREEN0);

		count1++;
	    }

	    cword_messages_output_message (
		CWORD_WAITINGENDED, 
		SCREEN0);
	    cword_messages_output_message (
		CWORD_WAITINGENDED, 
		SCREEN0);

	    line_num = 3;
	}

	cword_messages_output_message (
	    CWORD_S_CREATOUTPUTCLUE,
	    SCREEN0,
	    are_over_99_clues == FALSE ? 2 : 3, 
	    crossword_record->clue_details[count].clue_number,
	    crossword_record->clue_details[count].clue_direction 
				? CWORD_ACROSS_STR : CWORD_DOWN_STR,  
	    crossword_record->clue_details[count].x_coordinate,
	    crossword_record->clue_details[count].y_coordinate,
	    crossword_record->clue_details[count].answer_length);

	if ( (((count + 1) % GEN_CLUES_PER_LINE) == 0) && 
	      (count != 0) && 
               (count != (crossword_record->clue_sum - 1)) )
	{
	    /*
	     *	If (count modulus GEN_CLUES_PER_LINE) is zero then stick in a
	     *	carriage return.
	     */

	    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

	    line_num++;
	}

	(void) cword_charui_check_page_break (
		    NOT_FINISHED,
		    CWORD_S_PRESSRETURN,
		    &line_num);
    }

    /*
     *	?? 100% correct page breaking is not catered for outside the above
     *	loop.  This could lead to some small problems without display for some
     *	certain dimensioned grids ??
     */

    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);
    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

    cword_messages_output_message (
	CWORD_S_HIGHESTCLUES, 
	SCREEN0,
	CWORD_ACROSS_STR,
	cword_stats_get_high_clue_num (ACROSS, crossword_record),
	CWORD_DOWN_STR,
	cword_stats_get_high_clue_num (DOWN, crossword_record));

    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

    /*
     *	?? Analyse columns and rows of grid to output statistics about clues
     *	defined in columns and rows or grid ??
     */

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_generate__add_lights (
	int			x_coordinate,
	int			y_coordinate,
	int			light_direction,
	int			length_of_light,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Overall function that controls the addition of lights dependent on the
 *	symmetry of the defined grid.
 *
 *  FORMAL PARAMETERS:
 *
 *	x_coordinate		(r) x coordinate of the light to be added/deleted
 *	y_coordinate		(r) y coordinate of the light to be added/deleted
 *	light_direction		(r) direction of the light to be added/deleted
 *	length_of_light		(r) length of the answer of the light to be added/deleted
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *	CWORD_S_FAILRE		Some error adding lights to grid
 *--
 */

{

    int status;

    /*
     *  Add lights to evolving grid!
     *
     *	The mechanism for adding lights is dependent on the grid type chosen
     *	earlier.
     */

    if (crossword_record->symmetry == DEG_90_SYMMETRY)
    {
	status = cword_generate__action_90deg  (
			ADD,
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else if (crossword_record->symmetry == DEG_180_SYMMETRY)
    {
	status = cword_generate__action_180deg (
			ADD,
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else if (crossword_record->symmetry == AXIAL_LR_SYMMETRY)
    {
	status = cword_generate__action_lraxial (
			ADD,	
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else if (crossword_record->symmetry == AXIAL_RL_SYMMETRY)
    {
	status = cword_generate__action_rlaxial (
			ADD,
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }
    else if (crossword_record->symmetry == NO_SYMMETRY)
    {
	status = cword_generate__action_nodeg (
			ADD,
			x_coordinate,
			y_coordinate,
			light_direction,
			length_of_light,
			crossword_record);
    }

    return (status);

}



PRIVATE int cword_generate__add_grid_opts (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function will allow the user to add light definitions to the grid
 *	being built.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(r/w) pointer to an instance of a crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully added lights to evolving grid.
 *	CWORD_S_FAILURE		Error adding lights to grid.
 *--
 */

{

    int x_coordinate,
	y_coordinate,
	light_direction,
	length_of_light,
	status;

    /*
     *  Get a set of co-ordinates to insert into the grid
     */

    status = cword_generate__get_light_coord (
		crossword_record,
		&x_coordinate,
		&y_coordinate,
		&light_direction,
		&length_of_light);

    if (CWORD_S_NORMAL != status)
    {
	/*
	 *  Could have a static int set up that keeps track of the number of
	 *  invalid user inputs entered so that after a certain number of
	 *  consecutive incorrect inputs the user would be kicked out.
	 */

	return (CWORD_S_FAILURE);
    }

    /*
     *  More specific validation checks on the inputted light location
     */

    if (cword_generate__verify_coords (
		crossword_record,
		x_coordinate,
		y_coordinate,
		light_direction,
		length_of_light) != CWORD_S_NORMAL)
    {
	cword_messages_output_message (
	    CWORD_INVALIDLIGHTFOUND,
	    (TRACE2 | SCREEN0),
	    x_coordinate,
	    y_coordinate,
	    light_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
	    length_of_light);

	return (CWORD_S_FAILURE);
    }
    else
    {
	cword_messages_output_message (
	    CWORD_VALIDLIGHTFOUND,
	    (TRACE2 | SCREEN2),
	    x_coordinate,
	    y_coordinate,
	    light_direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
	    length_of_light);

	if (cword_generate__is_light_valid (
		    crossword_record,
		    x_coordinate,
		    y_coordinate,
		    light_direction,
		    length_of_light) == FALSE)
	{
	    cword_messages_output_message (
		CWORD_S_INVALIDLIGHT,
		SCREEN0);

	    return (CWORD_S_FAILURE);
	}
    }

    if (cword_generate_is_light_found (
		crossword_record,
		x_coordinate,
		y_coordinate,
		light_direction,
		length_of_light) == TRUE)
    {
	cword_messages_output_message (
		CWORD_LIGHTALREADYTHERE,
		(SCREEN2 | TRACE2),
		x_coordinate, 
		y_coordinate);

	return (CWORD_S_FAILURE);
    }

    status = cword_generate__add_lights (
		    x_coordinate,
		    y_coordinate,
		    light_direction,
		    length_of_light,
		    crossword_record);

    return (CWORD_S_NORMAL);

}



PRIVATE void cword_generate__get_option (
	char 			*option)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that allows user to input what he/she want to do next during
 *	grid generation.
 *
 *  FORMAL PARAMETERS:
 *
 *	*option			(w) Get the option that the user wants processed
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None.
 *--
 */

{

    char temp_ch;

    cword_messages_output_message (
	CWORD_S_GRIDCREATECOMM,
	SCREEN0,
	ADD_CHAR);

    temp_ch = cword_charui_get_ch (FALSE);

    if ( (toupper (temp_ch) == ADD_CHAR) || (temp_ch == (char) 0) )
    {
	/*
	 *  If nothing is entered default to ADD_CHAR!
	 */

	*option = ADD_CHAR;
    }
    else if (toupper (temp_ch) == QUIT_CHAR)
	*option = QUIT_CHAR;
    else if (toupper (temp_ch) == SAVE_CHAR)
	*option = SAVE_CHAR;
    else if (toupper (temp_ch) == VERIFY_CHAR)
	*option = VERIFY_CHAR;
    else if (toupper (temp_ch) == RESET_CHAR)
	*option = RESET_CHAR;
    else if (toupper (temp_ch) == DELETE_CHAR)
	*option = DELETE_CHAR;
    else if (toupper (temp_ch) == LIST_CHAR)
	*option = LIST_CHAR;
    else if ( (toupper (temp_ch) == HELP_CHAR) || (temp_ch == QUESTION_CHAR) )
	*option = HELP_CHAR;
    else
    {
	/*
	 *  Invalid response
	 */

	*option = INVALID_CHAR;
    }

    /*
     *  ?? Add output option to log file - TRACE3 ??
     */

    /*
     *  No return value really necessary.
     */

    return;

}



PRIVATE void cword_generate__output_help ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that will print out a little one liner on each command line
 *	option available during grid creation.
 *
 *  FORMAL PARAMETERS:
 *
 *      None!
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None!
 *--
 */

{

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	CWORD_S_GENERICHELP,
	SCREEN0);

    /*
     *	?? What to do about these messages to make their creation as dynamic as
     *	possible - probably nothing ??
     */

    printf ("## S - Save the completed grid to Cword grid file\n");
    printf ("## Q - Quit current grid creation without saving\n");
    printf ("## A - Add light (or lights) to evolving grid definition\n");
    printf ("## D - Delete light (or lights) from evolving grid definition\n");
    printf ("## L - List the lights currently created\n");
    printf ("## R - Reset/delete all light definitions in evolving grid\n");
    printf ("## V - Verify that the current grid conforms to selected grid properties\n");
    printf ("## H - Get this screen :-)\n");

    cword_messages_output_message (
	CWORD_WAITINGENDED,
	SCREEN0);

    return;

}



PUBLIC int cword_generate_create_grid (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function will supervise the creation of an empty grid used in
 *	creating crosswords.
 *
 *  FORMAL PARAMETERS:
 *
 *	*crossword_record	(w) Pointer to an instance of a crossword_record where 
 *				    grid will be defined and stored until it is written to an 
 *				    output file in the calling routine.
 *
 *  IMPLICIT OUTPUTS:
 *
 *	cword_generate_l_new_add_light
 *	cword_generate_l_new_del_light
 *	cword_generate_l_grid_updated
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(w) crossword_record to store the grid
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Successful completion
 *      CWORD_NOGRIDCREATED	Error creating grid
 *--
 */
{

    char option = (char) 0;

    int status;

    time_t start_time,
	   end_time;

    BOOLEAN is_first_time;

    /*
     *	Reset BOOLEAN each time function is executed!
     */

    is_first_time = TRUE;

    /*
     *	?? Problem displaying the first "\n" on the UI which I've not been able
     *	to figure out ??
     */
    cword_messages_output_message (
	CWORD_WAITINGENDED,
	SCREEN0);

    cword_messages_output_message (
	CWORD_WAITINGENDED,
	SCREEN0);

    cword_messages_output_message (
	CWORD_GRIDSTARTPHASE,
	(TRACE0 | SCREEN0));

    /*
     *  Make sure there is nothing in the crossword_record struct before starting.
     */

    memset (crossword_record, 0, sizeof (CROSSWORD_RECORD));

    cword_generate_l_new_add_light.x_coordinate = cword_generate_l_new_add_light.y_coordinate =
    cword_generate_l_new_add_light.answer_length = cword_generate_l_new_add_light.clue_direction =
    cword_generate_l_new_del_light.x_coordinate = cword_generate_l_new_del_light.y_coordinate =
    cword_generate_l_new_del_light.answer_length = cword_generate_l_new_del_light.clue_direction = 0;
    cword_generate_l_grid_updated = FALSE;

    start_time = time (0);

    /*
     *  Get the starter parameters needed to set up the grid properly
     */

    cword_generate__get_grid_props (crossword_record);

    do
    {
	/*
	 *  Only output the grid to the trace file if an ADD or a DELETE has
	 *  taken place.  It's only wasting user and CPU time to do it
	 *  otherwise.
	 */

	if ( (option == ADD_CHAR) || (option == DELETE_CHAR) )
	{
	    cword_generate__view_grid (
		(SCREEN0 | TRACE3),
		crossword_record);
	}
	else
	{
	    cword_generate__view_grid (
		SCREEN0,
		crossword_record);
	}

	if (is_first_time == TRUE)
	{
	    cword_messages_output_message (
		CWORD_S_HELPAVAILABLE,
		SCREEN0,
		HELP_CHAR);

	    is_first_time = FALSE;
	}

	/*
	 *  Get command line user option: Add, Delete, Verify, List etc.
	 */

	cword_generate__get_option (&option);

	switch (option)
	{
	    /*
	     *	Only CWORD_S_NORMAL / CWORD_S_FAILURE / CWORD_S_GRIDGENFINISHED
	     *	/ CWORD_S_GRIDGENQUIT allowed 
	     *
	     *  ?? Tidy up status values ??
	     */

	case (ADD_CHAR):

	    /*
	     *	?? Should set up a little checkpoint that will check if the
	     *	user has created a valid light definition within a pre-defined
	     *	number of choices/minutes.  Not very necessary but could be
	     *	nice for the beginner ??
	     */

	    status = cword_generate__add_grid_opts (crossword_record);

	    if (status != CWORD_S_NORMAL)
	    {
		/*
		 *  There is no purpose to reset the variable option other than
		 *  to prevent display of the grid in TRACE3 mode since the
		 *  ADD_CHAR option was chosen.
		 */

		/*
		 *  ?? Output message saying add failed ??
		 */

		option = (char) 0;
	    }

	    status = CWORD_S_NORMAL;

	    break;

	case (DELETE_CHAR):
	    status = cword_generate__delete_grid_opts (crossword_record);

	    if (status != CWORD_S_NORMAL)
	    {
		/*
		 *  ?? Output message saying delete failed ??
		 */
	    }

	    status = CWORD_S_NORMAL;

	    break;

	case (VERIFY_CHAR):
	    status = cword_generate__verify_grid (crossword_record);

	    status = CWORD_S_NORMAL;

	    break;

	case (SAVE_CHAR):
	    status = CWORD_S_GRIDGENFINISHED;

	    break;

	case (RESET_CHAR):
	    status = cword_generate__reset_grid (crossword_record);

	    status = CWORD_S_NORMAL;

	    break;

	case (LIST_CHAR):
	    status = cword_generate__list_grid (crossword_record);

	    status = CWORD_S_NORMAL;

	    break;

	case (HELP_CHAR):
	    cword_generate__output_help ();

	    status = CWORD_S_NORMAL;

	    break;

	case (QUIT_CHAR):
	    status = CWORD_S_GRIDGENQUIT; 

	    break;

	default:
	    cword_messages_output_message (
			CWORD_CHOICENOTVALID,
			SCREEN0);

	    status = CWORD_S_NORMAL;

	    break;
	}
    }
    while (status == CWORD_S_NORMAL);

    end_time = time (0);

    cword_messages_output_message (
		CWORD_S_CREATETIMEWAS,
		(TRACE2 | SCREEN0),
		end_time - start_time);

    if (CWORD_S_FAILURE == status)
    {
	/*
	 *  If code gets to here then some sort of error has occurred --
	 *  recover or not is upto the user.
	 */

	cword_messages_output_message (
		    CWORD_GRIDENDERR,
		    (TRACE0 | SCREEN0),
		    CWORD_Y_CHAR, 
		    CWORD_N_CHAR, 
		    CWORD_N_CHAR);

	if (toupper (cword_charui_get_ch (FALSE)) == CWORD_Y_CHAR)
	{ 
	    status = cword_generate__tidy_grid (crossword_record);

	    if (CWORD_S_NORMAL != status)
	    {
		/*
		 *  Make sure that crossword_record is blank before returning
		 *  an error.
		 */

		memset (crossword_record, 0, sizeof (CROSSWORD_RECORD));

		cword_messages_output_message (
			    CWORD_NOGRIDCREATED,
			    (TRACE0 | SCREEN0));

		status = CWORD_NOGRIDCREATED;
	    }
	}
	else
	{
	    cword_messages_output_message (
			CWORD_GRIDNONECREATED,
			(TRACE0 | SCREEN0));

	    memset (crossword_record, 0, sizeof (CROSSWORD_RECORD));

	    cword_messages_output_message (
			CWORD_NOGRIDCREATED,
			(TRACE0 | SCREEN0));

	    status = CWORD_NOGRIDCREATED;
	}
    }
    else if (CWORD_S_GRIDGENQUIT == status)
    {
	/*
	 *  Quitting grid creation - option to save before quitting if lights
	 *  defined.
	 */

	if (crossword_record->clue_sum == 0)
	{
	    cword_messages_output_message (
			CWORD_S_NOLIGHTSNOSAVE,
			(TRACE0 | SCREEN0));

	    status = CWORD_NOGRIDCREATED;
	}
	else
	{
	    cword_messages_output_message (
			CWORD_S_QUITGRIDCREATE,
			SCREEN0,
			CWORD_Y_CHAR, 
			CWORD_N_CHAR, 
			CWORD_Y_CHAR);

	    if (toupper (cword_charui_get_ch (FALSE)) == CWORD_N_CHAR)
	    {
		cword_messages_output_message (
			    CWORD_GRIDNONECREATED,
			    (TRACE0 | SCREEN0));

		memset (crossword_record, 0, sizeof (CROSSWORD_RECORD));

		cword_messages_output_message (
			    CWORD_NOGRIDCREATED,
			    (TRACE0 | SCREEN0));

		status = CWORD_NOGRIDCREATED;
	    }
	    else
	    {
		status = cword_generate__tidy_grid (crossword_record);

		if (CWORD_S_NORMAL != status)
		{
		    memset (crossword_record, 0, sizeof (CROSSWORD_RECORD));

		    cword_messages_output_message (
				CWORD_NOGRIDCREATED,
				(TRACE0 | SCREEN0));

		    status = CWORD_NOGRIDCREATED;
		}
	    }
	}
    }
    else
    {
	/*
	 *  Happy with grid creation.  The grid will be saved in the calling
	 *  function.
	 */

	status = cword_generate__tidy_grid (crossword_record);

	if (CWORD_S_NORMAL != status)
	{
	    memset (crossword_record, 0, sizeof (CROSSWORD_RECORD));

	    cword_messages_output_message (
			CWORD_NOGRIDCREATED,
			(TRACE0 | SCREEN0));

	    status = CWORD_NOGRIDCREATED;
	}
    }

    return (status);

}



#else

main ()

{

    exit (EXIT_SUCCESS);

}

#endif
