/*
 *  ?? Major problems with messages now that the messaging scheme has been
 *  tightened up ??
 *
 *  ?? Function using light_templates to check adjacent light letters that are
 *  invalid or not found in getstats.h.  This function will be used both in
 *  seed and grid to notify the user of possible fill deadends that will occur
 *  because of the proposed grid configuation ??
 */

/*
 *++
 *  FACILITY:
 *
 *      grid.c
 *
 *  ABSTRACT:
 *
 *	Module containing the light filling functionality of Cword.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:     
 *
 *	31 December 1992
 *--
 */

/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <math.h>

#if defined(__unix) || defined(__unix__)

#include <unistd.h>
#include <sys/file.h>
#include <sys/uio.h>

#endif

#if defined(VMS) || defined (__VMS)

#include <unixio.h>
#include <file.h>
#include <signal.h>

#endif

#if defined(__MSDOS__) || defined(MSDOS)

#include <io.h>

#endif

#ifdef GRID
#define MESSAGES_DEFINED 1 
#endif

#include "main.h"
#include "print.h"
#include "grid.h"
#include "fileio.h"
#include "memman.h"
#include "utils.h"
#include "messages.h"
#include "match.h"
#include "os.h"
#include "charui.h"
#include "defaults.h"
#include "seed.h"
#include "stats.h"
#include "strll.h"

#if defined(VMS) || defined(__VMS) || defined(__osf__)
#include "rdb.h"
#endif

/*
 *  Specially generated header file containing dictionary file statistics
 *  useful for checking letter adjacencies in still unfilled lights during
 *  light generation.
 *
 *  This header file was generated using getstats.c.  This file is available
 *  from the developer on request.
 */

#include "dicstats.h"

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#ifndef GRID

/*
 *  Constants used during the grid creation phase.
 */

#define NOTFINISHED	1	/* Light generator hasn't finished yet */
#define FINISHED	2	/* Light generator has finished */
#define UNFINISHED	3	/* Light generator has run out of time */
#define ERRORFINISHING	4	/* Light generator is in error */

#define UNFILLED_WEIGHT 0.01

#define NOWORDCHOSEN	-1

typedef struct 
{
    int		    index_rating;
    float	    weight;
} BEST_RECORD;

PRIVATE float cword_grid__update_weight (
	char			letter);

PRIVATE int cword_grid__select_word (
	int			select_type,
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE void cword_grid__sort (
        int			word_index,
	float			word_weight,
	BEST_RECORD		words_chosen[CWORD_SIZEOF_MAX_ARRAY_SIZE]);

PRIVATE int cword_grid__prune_light_ll (
	int			prune_type,
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE void cword_grid__enhance_template_word (
	char			*template_word);

PRIVATE int cword_grid__check_adjacency (
	int			best_word,
	char			*check_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_grid__get_word_match (
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE float cword_grid__word_stats (
	CWORD_T_GRID_STRUCT	*grid_record,
	int			word_number);

PRIVATE int cword_grid__is_grid_finished (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_grid_template_sameas_list (
	char			*template,
	CWORD_T_STR_LL		*matches_ll);

PRIVATE cword_grid__check_light_ll (
	char			*search_str,
	CWORD_T_STR_LL		*search_ll);

PRIVATE int cword_grid__check_for_ow_lights (
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_grid__select_next_light (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE float cword_grid__find_intersect_let (
	CWORD_T_LIGHT_STRUCT	*light1,
	CWORD_T_LIGHT_STRUCT	*light2);

PRIVATE int cword_grid__backtrack_grid (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE void cword_grid_clean_up_grid_rec (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE void cword_grid_set_up_record (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_grid__fill_grid_poss (
	CWORD_T_GRID_STRUCT	*grid_record);

#ifdef DB

#if defined(VMS) || defined(__VMS) || defined(__osf__)

PRIVATE int cword_grid__get_clues_from_db (
	CROSSWORD_RECORD        *crossword_record);

#endif

#endif

PRIVATE int cword_grid__reset_grid_struct (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_grid__percent_completed (
	CWORD_T_GRID_STRUCT	*grid_record);



PRIVATE float cword_grid__update_weight (
	char			letter)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to update the grid record with the weight attached to the
 *	inputted letter.
 *
 *  FORMAL PARAMETERS:
 *
 *      *letter			(r) letter to get weight for
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.letter_weight_algorithm
 *
 *  FUNCTION VALUE:
 *
 *      Weight accorded to the letter inputted to the function
 *--
 */

{

    float value;

    /*
     *	?? This method depends on selecting crossing letters in an answer which
     *	will more likely lead to a successful grid being generated.  The *BIG*
     *	problem is that it will conncentrate on the crossing letters which are
     *	of most value e.g. the letter 'e' in this case.  This means that there
     *	will be a concentration of letter 'e''s in the completed crossword.
     *	Not a very good idea on the whole. ??
     */

    if (cword_g_defaults_record.letter_weight_algorithm == EQUAL_WEIGHTS)
    {
	/*
	 *  Equal weights for all words!
	 */

	value = EQUAL_PROB;
    }
    else if (cword_g_defaults_record.letter_weight_algorithm == DICT_WEIGHTS)
    {
	/*
         *  Weights as per dicstats.h!
	 *
	 *  ?? Using A_PROB... from grid.h at the moment ??
	 */

	switch (letter)
	{
	case 'a':
	    value = A_PROB; break;
	case 'b':
	    value = B_PROB; break;
	case 'c':
	    value = C_PROB; break;
	case 'd':
	    value = D_PROB; break;
	case 'e':
	    value = E_PROB; break;
	case 'f':
	    value = F_PROB; break;
	case 'g':
	    value = G_PROB; break;
	case 'h':
	    value = H_PROB; break;
	case 'i':
	    value = I_PROB; break;
	case 'j':
	    value = J_PROB; break;
	case 'k':
	    value = K_PROB; break;
	case 'l':
	    value = L_PROB; break;
	case 'm':
	    value = M_PROB; break;
	case 'n':
	    value = N_PROB; break;
	case 'o':
	    value = O_PROB; break;
	case 'p':
	    value = P_PROB; break;
	case 'q':
	    value = Q_PROB; break;
	case 'r':
	    value = R_PROB; break;
	case 's':
	    value = S_PROB; break;
	case 't':
	    value = T_PROB; break;
	case 'u':
	    value = U_PROB; break;
	case 'v':
	    value = V_PROB; break;
	case 'w':
	    value = W_PROB; break;
	case 'x':
	    value = X_PROB; break;
	case 'y':
	    value = Y_PROB; break;
	case 'z':
	    value = Z_PROB; break;
	default:
	    value = 0.0; break;
	}
    }
    else if (cword_g_defaults_record.letter_weight_algorithm == SCRABBLE_WEIGHTS)
    {
	/*
         *  Weights as per Scrabble weightings!
	 */

	switch (letter)
	{
	case 'a':
	    value = SCRABBLE_PROB_A; break;
	case 'b':
	    value = SCRABBLE_PROB_B; break;
	case 'c':
	    value = SCRABBLE_PROB_C; break;
	case 'd':
	    value = SCRABBLE_PROB_D; break;
	case 'e':
	    value = SCRABBLE_PROB_E; break;
	case 'f':
	    value = SCRABBLE_PROB_F; break;
	case 'g':
	    value = SCRABBLE_PROB_G; break;
	case 'h':
	    value = SCRABBLE_PROB_H; break;
	case 'i':
	    value = SCRABBLE_PROB_I; break;
	case 'j':
	    value = SCRABBLE_PROB_J; break;
	case 'k':
	    value = SCRABBLE_PROB_K; break;
	case 'l':
	    value = SCRABBLE_PROB_L; break;
	case 'm':
	    value = SCRABBLE_PROB_M; break;
	case 'n':
	    value = SCRABBLE_PROB_N; break;
	case 'o':
	    value = SCRABBLE_PROB_O; break;
	case 'p':
	    value = SCRABBLE_PROB_P; break;
	case 'q':
            value = SCRABBLE_PROB_Q; break;
	case 'r':
	    value = SCRABBLE_PROB_R; break;
	case 's':
	    value = SCRABBLE_PROB_S; break;
	case 't':
            value = SCRABBLE_PROB_T; break;
	case 'u':
	    value = SCRABBLE_PROB_U; break;
	case 'v':
	    value = SCRABBLE_PROB_V; break;
	case 'w':
	    value = SCRABBLE_PROB_W; break;
	case 'x':
	    value = SCRABBLE_PROB_X; break;
	case 'y':
	    value = SCRABBLE_PROB_Y; break;
	case 'z':
	    value = SCRABBLE_PROB_Z; break;
	default:
	    value = 0.0; break;
	}
    }
    else
    {
	printf ("## Invalid letter_weight_algorithm chosen - ERROR\n");
    }

    return (value);

}



PRIVATE int cword_grid__select_word (
	int			select_type,
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to select the best word from the list of words found by the
 *	template matcher and pruned to size by the prune_light_ll function.
 *
 *  FORMAL PARAMETERS:
 *
 *	select_type		(r) Select the best light using either RANDOMWORDS/BESTWORDS
 *      best_word		(r) the word being worked on presently by the generator
 *	*grid_record		(w) grid record to be filled using crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.max_light_choices
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		word successfully chosen from template list
 *	CWORD_S_FAILURE		word unsuccessfully chosen from template list 
 *--
 */

{

    CWORD_T_STR_LL *temp_list = (CWORD_T_STR_LL *) 0;

    char best_match[CWORD_SIZEOF_CELLDIM + 1];

    int	count1 = 0,
	index = 0;

    float best_fit;

    /*
     *	Calculate the value of each of the listed words and insert the value in
     *	the data structure
     */

    /*
     *	If no CROSS_STR's exist in answer_template then override select_type to
     *	RANDOMWORDS as there is no point going with BESTWORDS since it will
     *	only mean selection of the first word on the list.
     */

    if (strstr (grid_record->grid_details[best_word].light_template, CROSS_STR) == (char *) 0)
    {
	/*
	 *  Override default select_type
	 */

	select_type = RANDOMWORDS;

	cword_messages_output_message (
		    CWORD_BESTTORANDOM,
		    SCREEN0,
		    grid_record->grid_details[best_word].light_template);
    }

    if (select_type == BESTWORDS)
    {
	best_fit = 0.0;
	temp_list = grid_record->grid_details[best_word].matches;

	while (temp_list != (CWORD_T_STR_LL *) 0)
	{
	    if (temp_list->str_val > best_fit)
	    {
		best_fit = temp_list->str_val;
		strcpy (best_match, temp_list->str);
		index = count1;
	    }

	    temp_list = (CWORD_T_STR_LL *) temp_list->next;
	    count1++;
	}

	if (best_fit != 0.0)
	{
	    strcpy (
		grid_record->grid_details[best_word].light, 
		best_match);
	}
	else
	{
	    /*
	     *  First word/light on list
	     */

	    strcpy (
		grid_record->grid_details[best_word].light, 
		grid_record->grid_details[best_word].matches->str);
	}

	cword_messages_output_message (
		    CWORD_BESTWORDS,
		    TRACE0,
		    (index + 1),
		    grid_record->grid_details[best_word].match_num,
		    best_fit);
    }
    else if (select_type == RANDOMWORDS)
    {
	if (grid_record->grid_details[best_word].match_num > cword_g_defaults_record.max_light_choices)
	{
	    index = (int) (((double) (cword_utils_get_random_number () * 
   		      cword_g_defaults_record.max_light_choices)) / (double) 100.0);
	}
	else
	{
	    temp_list = grid_record->grid_details[best_word].matches;

	    while (temp_list != (CWORD_T_STR_LL *) 0)
	    {
		temp_list=temp_list->next;
		count1++;
	    }

	    if (count1 > 1)
	    {
		index = (int) (((double) (cword_utils_get_random_number () * 
			  count1)) / (double) 100.0);
	    }
	    else if (count1 == 1)
	    {
		index = 0;
	    }
	    else
	    {
		cword_messages_output_message (CWORD_NOLIGHTSAVAIL, SCREEN0);

		return (CWORD_S_FAILURE);
	    }
	}

	count1 = 0;
	temp_list = grid_record->grid_details[best_word].matches;

	while (temp_list != (CWORD_T_STR_LL *) 0)
	{
	    if (count1 == index)
	    {
		strcpy (
		    grid_record->grid_details[best_word].light, 
		    temp_list->str);

		break;
	    }

	    temp_list = temp_list->next;
	    count1++;
	}

	cword_messages_output_message (
		    CWORD_RANDOMWORDS,
		    (TRACE3 | SCREEN3),
		    index + 1,
		    grid_record->grid_details[best_word].match_num);
    }
    else
    {
	printf ("## Invalid light selection type chosen\n");

	return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Use qsort the RTL function to do the sorting ??
 */

PRIVATE void cword_grid__sort (
        int			word_index,
	float			word_weight,
	BEST_RECORD		words_chosen[CWORD_SIZEOF_MAX_ARRAY_SIZE])

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to add the variables word_index and word_weight to
 *	words_chosen and then perform a bubble sort on the inputted
 *	words_chosen data type.
 *
 *      Implemeneted using Bubble sort.  Can be improved upon.
 *
 *  FORMAL PARAMETERS:
 *
 *	word_index,		integer index
 *	word_weight,		word index to add
 *	words_chosen		data structure containing indexes and weights
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.max_light_choices
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      NONE
 *--
 */

{

    float   temp_weight;

    int	    temp_index,
	    count;

    BOOLEAN sorted = FALSE;

    /*
     *  If not being called with word_index = 0
     */

    if (word_index != 0)
    {
	/*
	 *  Just replace the last element with the element details coming in.
	 *  Sort then to restore descending order...
	 */

	words_chosen[cword_g_defaults_record.max_light_choices - 1].index_rating = word_index;
	words_chosen[cword_g_defaults_record.max_light_choices - 1].weight = word_weight;
    }

    /*
     *  Sort elements with trusty bubble sort
     */

    while (sorted == FALSE)
    {
	sorted = TRUE;

	for (count = 0; count < (cword_g_defaults_record.max_light_choices - 1); count++)
	{
	    if (words_chosen[count].weight < words_chosen[count + 1].weight)
	    {
		sorted = FALSE;
		temp_index = words_chosen[count].index_rating;
		temp_weight = words_chosen[count].weight;
		words_chosen[count].index_rating = words_chosen[count + 1].index_rating;
		words_chosen[count].weight = words_chosen[count + 1].weight;		
		words_chosen[count + 1].index_rating = temp_index;		
		words_chosen[count + 1].weight = temp_weight;		
	    }
	}
    }

    return;

}



PRIVATE int cword_grid__prune_light_ll (
	int			prune_type,
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to prune the list of words received from the template matching
 *	utility.
 *
 *  FORMAL PARAMETERS:
 *
 *      best_word		(r) Word index presently being worked on.
 *	*grid_record		(w) Grid record to be filled using crossword_record.
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.max_light_choices
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Template list for best_word successfully pruned.
 *	CWORD_S_FAILURE		Teplmate list for best_word unsuccessfully pruned.
 *--
 */

{

    CWORD_T_STR_LL *temp_list = (CWORD_T_STR_LL *) 0,
		   *new_list = (CWORD_T_STR_LL *) 0;

    BEST_RECORD words_chosen[CWORD_SIZEOF_MAX_ARRAY_SIZE];

    int	best_x[CWORD_SIZEOF_MAX_ARRAY_SIZE],
    	count,
	count1,
	status,
	index,
	temp,
	elem_no;

    BOOLEAN is_word_found;

    cword_messages_output_message (
	CWORD_DELETEDISALLOW, 
	TRACE3);

    for (count1 = 0; count1 < CWORD_SIZEOF_MAX_ARRAY_SIZE; count1++)
    {
	words_chosen[count1].index_rating = 0;
	words_chosen[count1].weight = 0;
    }

    /*
     *	Clearing down the inputted list of any disallowed words before
     *	continuing.
     */

    /*
     *  NEW: Copy grid_record->grid_details[best_word].matches to new_list.
     */

    temp_list = grid_record->grid_details[best_word].matches;
    while (temp_list != (CWORD_T_STR_LL *) 0)
    {
	cword_strll_add_str_and_val (
	    UNSORTED | MULTIPLE,
	    temp_list->str_val,
	    temp_list->str,
	    &new_list);

	temp_list = temp_list->next;
    }

    /*
     *  NEW: Operate on new_list.
     */

    temp_list = grid_record->grid_details[best_word].matches;
    while (temp_list != (CWORD_T_STR_LL *) 0)
    {
	status = cword_grid_check_disallowed (
		    best_word,
		    temp_list->str,
		    grid_record);

	if (status != CWORD_S_NORMAL)
	{
	    /*
	     *  Delete from new list any disallowed strings found!
	     */

	    cword_strll_del_str (
		temp_list->str,
		&new_list);
	}

	temp_list = temp_list->next;
    }

    /*
     *  NEW: Delete/free grid_record->grid_details[best_word].matches.
     */

    cword_strll_free_str_ll (&grid_record->grid_details[best_word].matches);

    /*
     *  NEW: Assign new_list to grid_record->grid_details[best_word].matches.
     */

    grid_record->grid_details[best_word].matches = new_list;

    /*
     *	NEW: Get linked list string values before continuing grid filling.
     */

    temp_list = grid_record->grid_details[best_word].matches;
    while (temp_list != (CWORD_T_STR_LL *) 0)
    {
	count1 = 0;
	temp_list->str_val = 0.0;

	while (grid_record->grid_details[best_word].light_template[count1] != (char) 0)
	{
	    if (grid_record->grid_details[best_word].light_template[count1] == CROSS_CHAR)
	    {
		temp_list->str_val += cword_grid__update_weight (temp_list->str[count1]);
	    }

	    count1++;
	}

	temp_list = temp_list->next;
    }

    /* 
     *	Get number of elements in the list and put in elem_no after deletion of
     *	disallowed words.
     */

    grid_record->grid_details[best_word].match_num = elem_no = 
	cword_strll_number_str_ll (grid_record->grid_details[best_word].matches);

    if (elem_no >= 1)
	cword_messages_output_message (
		CWORD_POSSIBLECHOICES,
		(TRACE2 | SCREEN2),
		elem_no);
    else
    {
	/*
	 *  ?? What happens in this is encountered ??
	 */

	cword_messages_output_message (CWORD_NOPOSSIBLECHOICES, (TRACE3 | SCREEN3));
    }

    if (elem_no < cword_g_defaults_record.max_light_choices)
    {
	cword_messages_output_message  (CWORD_NOPRUNELIST, (TRACE3 | SCREEN3));

	return (CWORD_S_NORMAL);
    }
 
    cword_messages_output_message  (CWORD_PRUNELIST, (TRACE3 | SCREEN3));

    if (prune_type == RANDOMWORDS)
    {
	/*
	 *  Generate cword_g_defaults_record.max_light_choices random numbers
	 *  which will offset into the list of matched words inputted.  When
	 *  cword_g_defaults_record.max_light_choices unique random numbers
	 *  have been chosen, sort them and then construct the link list which
	 *  will replace the input link list with the randomly chosen
	 *  alternative list.
	 */

	cword_messages_output_message (CWORD_PRUNERANDOM, (TRACE2 | SCREEN2));

	/*
	 *  Get cword_g_defaults_record.max_light_choices randomly generated
	 *  words
	 */

	for (count = 0; count < cword_g_defaults_record.max_light_choices; count++)
	    best_x[count] = 0;

	/*
	 *  Generate unique indexes.
	 */

	if (cword_g_defaults_record.max_light_choices < elem_no)
	{
	    count = 0;
	    while (count < cword_g_defaults_record.max_light_choices)
	    {
		index = (int) (((double) (cword_utils_get_random_number () * 
                                  elem_no)) / (double) 100.0);

		is_word_found = FALSE;
		count1 = 0;
		while (count1 < count)
		{
		    if (index == best_x[count1])
		    {
			is_word_found = TRUE;
			break;
		    }

		    count1++;
		}

		if (is_word_found == FALSE)
		{
		    best_x[count++] = index;
		}
	    }

	    /*
	     *  Sort the words
	     */

	    is_word_found = FALSE;
	    while (is_word_found == FALSE)
	    {
		is_word_found = TRUE;

		for (count = 0; count < (cword_g_defaults_record.max_light_choices - 1); count++)
		{
		    if (best_x[count] > best_x[count + 1])
		    {
			is_word_found = FALSE;
			temp = best_x[count];
			best_x[count] = best_x[count + 1];
			best_x[count + 1] = temp;
		    }
		}

		if (is_word_found == TRUE)
		{
		    break;
		}
	    }
        }
	else
	{
	    /*
	     *  Put all elements of the list into the best_x data structure
	     *
	     *  cword_g_defaults_record.max_light_choices >= elem_no
	     *
	     *  Building a list greater than what is ordinarily needed
	     */

	    for (count = 0; count < elem_no; count++)
		best_x[count] = count;
	}

#ifdef DEBUG
	WRITE_TRACE ("### Random nums : ");
	for (count = 0; count < cword_g_defaults_record.max_light_choices; count++)
	{
	    WRITE_TRACE (
		"%03d ", 
		best_x[count]);
	}
	WRITE_TRACE ("\n");
#endif

	/*
	 *  Get the cword_g_defaults_record.max_light_choices number of words
	 *  needed from LL, release memory formerly assigned, and replace with
	 *  new list of words.
	 */

	temp_list = grid_record->grid_details[best_word].matches;
	count = count1 = 0;
	new_list = (CWORD_T_STR_LL *) 0;

	while (temp_list != (CWORD_T_STR_LL *) 0)
	{
	    if (count1 == best_x[count])
	    {
		DEBUG_TRACE (
		    "Word %s chosen randomly\n", 
		    temp_list->str);

		cword_strll_add_str_and_val (
		    SORTED | UNIQUE,
		    0.0,
		    temp_list->str,
		    &new_list);

#if 0
		/*
		 *  ??? Add value to link list element ???
		 */

		new_list = cword_memman_add_to_match_list (
				    temp_list->str,
				    (float) 0.0,
				    new_list);
#endif

		if (new_list == (CWORD_T_STR_LL *) 0)
		    return (CWORD_S_FAILURE);
		
		count++;

		if (count == cword_g_defaults_record.max_light_choices)
		    break; /* Finished */
	    }

	    temp_list = temp_list->next;
	    count1++;
	}

	cword_messages_output_message (CWORD_FOUNDRANDOMLIST, (TRACE2 | SCREEN2));

	cword_strll_free_str_ll (&grid_record->grid_details[best_word].matches);

	grid_record->grid_details[best_word].matches = new_list;

	grid_record->grid_details[best_word].match_num = 
	    cword_strll_number_str_ll (grid_record->grid_details[best_word].matches);
    }
    else if (prune_type == BESTWORDS)
    {
	/*
	 *  Run through the list and get the respective weights of each of the
	 *  words.  Select the words with the greatest weighting.  This will
	 *  mean setting up a temporary struct to store the weight ins and then
	 *  select the top cword_g_defaults_record.max_light_choices.  The
	 *  inputted link list must be freed up and a new one created.  Thus a
	 *  pointer to the link list has to be passed into the function to
	 *  obtain the correct results.
	 */

	cword_messages_output_message (CWORD_PRUNEWEIGHT, (TRACE2 | SCREEN2));

	/*
	 *  Get best cword_g_defaults_record.max_light_choices words according
	 *  to weighting algorithm
	 */

	count = 0;
	temp_list = grid_record->grid_details[best_word].matches;

	/*
	 *  ?? Very inefficient was of doing the sorting - has to be better way ??
	 */

	while (temp_list != (CWORD_T_STR_LL *) 0)
	{
	    if (count < cword_g_defaults_record.max_light_choices)
	    {
		words_chosen[count].index_rating = count;
		words_chosen[count].weight = temp_list->str_val;
	    }
	    else if (count == cword_g_defaults_record.max_light_choices)
	    {
		cword_grid__sort (
			    0,
			    (float) 0.0,
			    words_chosen);

		if (words_chosen[cword_g_defaults_record.max_light_choices - 1].weight < temp_list->str_val)
		{
		    cword_grid__sort (
				count,
				temp_list->str_val,
				words_chosen);
		}
	    }
	    else
	    {
		if (words_chosen[cword_g_defaults_record.max_light_choices - 1].weight < temp_list->str_val)
		{
		    cword_grid__sort (
				count,
				temp_list->str_val,
				words_chosen);
		}
	    }

	    count++;
	    temp_list = temp_list->next;
	}

	/*
	 *  Now get the best cword_g_defaults_record.max_light_choices.
	 */

	count = 0;
	temp_list = grid_record->grid_details[best_word].matches;
	new_list = (CWORD_T_STR_LL *) 0;

	while (temp_list != (CWORD_T_STR_LL *) 0)
	{
	    for (count1 = 0; count1 < cword_g_defaults_record.max_light_choices; count1++)
	    {
		if (words_chosen[count1].index_rating == count)
		{
		    cword_strll_add_str_and_val (
			SORTED | UNIQUE,
			temp_list->str_val,
			temp_list->str,
			&new_list);

#if 0
		    /*
		     *  ??? Add value to link list element ???
		     */

		    new_list = cword_memman_add_to_match_list (
					temp_list->str,
					temp_list->str_val,
					new_list);
#endif

		    if (new_list == (CWORD_T_STR_LL *) 0)
			return (CWORD_S_FAILURE);

		    break;
		}
	    }

	    count++;
	    temp_list = temp_list->next;	
	}    

	cword_messages_output_message (CWORD_WEIGHTRANDOMLIST, (TRACE2 | SCREEN2));

	cword_strll_free_str_ll (&grid_record->grid_details[best_word].matches);

	grid_record->grid_details[best_word].matches = new_list;

	grid_record->grid_details[best_word].match_num = 
	    cword_strll_number_str_ll (grid_record->grid_details[best_word].matches);
    }
    else
    {
	printf ("## Invalid light selection type chosen to prune light list with\n");

	return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE void cword_grid__enhance_template_word (
	char			*template_word)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is used to decrease the template matching search space.
 *
 *      At the moment it checks:
 *
 *	1)  if there are no letters in the input answer/light template then
 *	    this function will insert a letter from the lower part of the
 *	    alphabet into the template to speed up the word file search
 *
 *	2)  ...
 *
 *  FORMAL PARAMETERS:
 *
 *	*template_word		template string to be altered if required
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		    If the word is updated to reduce search space
 *	CWORD_S_FAILURE		    If the word is not updated to reduce search space
 *--
 */

{

    int count;

    BOOLEAN is_word_empty = TRUE;

    for (count = 0; 
	  (count < strlen (template_word) && ((template_word[count] == BLANK_CHAR) ||
                                               (template_word[count] == CROSS_CHAR))); 
	    count++)
        {};

    if (count != strlen (template_word))
	is_word_empty = FALSE;

    if (is_word_empty == TRUE)
    {
	/*
	 *  ?? What happens if a letter is chosen for the template for which
	 *  there is no match in the word file.  At the moment an error is
	 *  generated ??
	 */

	template_word[0] = (unsigned char) ( ((double) cword_utils_get_random_number () * 
                                               (double) CWORD_SIZEOF_ALPHABET) / 100.0) + 'a';

	cword_messages_output_message (
		CWORD_RULE1REDUCESEARCH,
		(TRACE2 | SCREEN2),
		template_word[0]);
    }

    return;

}



PRIVATE void cword_grid__display_pruned_list (
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    CWORD_T_STR_LL *temp_list;

    temp_list = grid_record->grid_details[best_word].matches;

    cword_messages_output_message (CWORD_LIGHTSLIST, SCREEN0);

    cword_strll_output_str_ll (
	temp_list,
	0);

    return;

}



PRIVATE int cword_grid__check_adjacency (
	int			best_word,
	char			*check_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Computationally a very expensive function to run during light
 *	generation!
 *
 *	This function will check the selected word and check is it acceptable
 *	in the grid that it will become part of.  If it doesn't create and
 *	possible deadlock situations then it will be accepted.
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    CWORD_T_GRID_STRUCT temp_grid_record;

    unsigned int count,
		 count1,
		 status;

    /*
     *  ?? Extrememly inefficient at the moment ??
     */

    /*
     *  ?? Algorithm ??:
     *  
     *	a. pretend that the inputted word is acceptable and build up new
     *	answer_templates 
     *  b. check these new answer_templates for adjacency -
     *	if they're are still unfilled 
     *	c. If a conflict is found then this match shouldn't be accepted
     *	into the list of possible matches to the light generator 
     *	d. The function should return CWORD_S_NORMAL if the match is
     *	acceptable, using the adjacency rule, else CWORD_S_FAILURE if it is
     *	not.
     */

    if (grid_record->iteration_no == 0)
	return (CWORD_S_NORMAL);

    temp_grid_record.clue_sum = grid_record->clue_sum;

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	temp_grid_record.grid_details[count].clue_number =
	    grid_record->grid_details[count].clue_number;
	temp_grid_record.grid_details[count].clue_direction =
	    grid_record->grid_details[count].clue_direction;
	temp_grid_record.grid_details[count].x_coordinate =
	    grid_record->grid_details[count].x_coordinate;
	temp_grid_record.grid_details[count].y_coordinate =
	    grid_record->grid_details[count].y_coordinate;
	temp_grid_record.grid_details[count].light_len = 
	    grid_record->grid_details[count].light_len;

	memset (
	    temp_grid_record.grid_details[count].light_template, 0, 
	    sizeof (grid_record->grid_details[count].light_template));

	strcpy (
		temp_grid_record.grid_details[count].light, 
		grid_record->grid_details[count].light);
    }

    strcpy (
	    temp_grid_record.grid_details[best_word].light, 
	    check_word);

    status = cword_grid_update_light_temps (
		CWORD_UPDATETOGRID,
		&temp_grid_record);

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

#ifdef DEBUG
    for (count = 0; count < temp_grid_record.clue_sum; count++)
    {
	printf (
	    "%d'th answer_template is now \"%s\"\n", 
	    count,
	    temp_grid_record.grid_details[count].light_template);
    }
#endif

    for (count = 0; count < temp_grid_record.clue_sum; count++)
    {
	if ( (temp_grid_record.grid_details[count].grid_entry == LIGHT_EMPTY) && 
	      (count != best_word) ) 
	{
	    /*
	     *  answer_template being checked hasn't been fully generated yet!
	     */

	    for (count1 = 0; 
		  count1 < temp_grid_record.grid_details[count].light_len - 1; 
		   count1++)
	    {
		if ( (temp_grid_record.grid_details[count].light_template[count1] != BLANK_CHAR) &&
		      (temp_grid_record.grid_details[count].light_template[count1 + 1] != BLANK_CHAR) && 
		       (temp_grid_record.grid_details[count].light_template[count1] != CROSS_CHAR) && 
		        (temp_grid_record.grid_details[count].light_template[count1 + 1] != CROSS_CHAR) )
		{
		    if (first_adjacency_array[temp_grid_record.grid_details[count].light_template[count1] - 'a']
					      [temp_grid_record.grid_details[count].light_template[count1 + 1] - 'a'] < 1000)
		    {
			printf (
			    "## < 1000 adjacencies found in dic for check_word %s adjacency being %c-%c\n",
			    check_word,
			    temp_grid_record.grid_details[count].light_template[count1],
			    temp_grid_record.grid_details[count].light_template[count1 + 1]);

			WRITE_TRACE (
			    "## < 1000 adjacencies found in dic for check_word %s adjacency being %c-%c\n",
			    check_word,
			    temp_grid_record.grid_details[count].light_template[count1],
			    temp_grid_record.grid_details[count].light_template[count1 + 1]);

			return (CWORD_S_FAILURE);
		    }
		}
	    }
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_grid__get_word_match (
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function given a template to return a link list containing the list of
 *	suitable matches.
 *
 *  FORMAL PARAMETERS:
 *
 *	??
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		word list/NULL word list obtained successfully
 *	CWORD_S_FAILURE		word list not obtained successfully
 *--
 */

{

    CWORD_T_STR_LL *light_ll = (CWORD_T_STR_LL *) 0,
	      *temp_light_ll = (CWORD_T_STR_LL *) 0;

    char temp_str[CWORD_SIZEOF_WORD + 1];

    int	status;

    cword_messages_output_message (
	    CWORD_PICKWORD,
	    (TRACE0 | SCREEN0),
	    grid_record->grid_details[best_word].clue_number, 
	    grid_record->grid_details[best_word].clue_direction 
				? CWORD_ACROSS_STR : CWORD_DOWN_STR,  
	    grid_record->grid_details[best_word].light_template);

    /*
     *	Function to decrease the search space of the template word in certain
     *	circumstances.
     */

    cword_grid__enhance_template_word (grid_record->grid_details[best_word].light_template);

    /*
     *  Call the template matching function
     */

    strcpy (temp_str, grid_record->grid_details[best_word].light_template);

    /*
     *  Where to get the lights from -- word file or rdb database.
     */

#ifdef DB

#if defined(VMS) || defined(__VMS) || defined(__osf__)

    if (cword_g_defaults_record.fill_from_where == FILL_FROM_WORD_FILE)
    {
	status = cword_match_template_list (
		    temp_str,
		    &light_ll);
    }
    else
    {
	status = cword_rdb_match_template_ll (
		    temp_str,
		    &light_ll);
    }

#else

    status = cword_match_template_list (
		temp_str,
		&light_ll);

#endif

#else

    status = cword_match_template_list (
		temp_str,
		&light_ll);

#endif

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    if (light_ll == (CWORD_T_STR_LL *) 0)
    {
	/* 
	 *  This status is OK - it will signify a LIGHT_DEADEND in that the matcher
	 *  didn't find a single match on the inputted template!
	 */

	grid_record->grid_details[best_word].matches = (CWORD_T_STR_LL *) 0;

	return (CWORD_S_NORMAL);
    }
    else
    {
	temp_light_ll = light_ll;

	/*
	 *  Assign the light_ll linked list to the
	 *  grid_record[best_word].matches.
	 */

	while (light_ll != 0)
	{
	    /*
	     *	?? Have to check whether the grid being generated is a
	     *	BLOCK/BARRED grid before running cword_grid__check_adjacency ??
             *
             *  ?? Don't like constant in "if" statement ??
	     */

	    if (TRUE)
	    {
#if 0
		/*
		 *  ?? Is it the best place to put check for adjacency
		 *  clashes ??
		 */

		if (cword_grid__check_adjacency (
				best_word,
				light_ll->word,
				grid_record) == CWORD_S_NORMAL)
#endif
		{
		    cword_strll_add_str_and_val (
			SORTED | UNIQUE,
			0.0,
			light_ll->str,
			&grid_record->grid_details[best_word].matches);

#if 0
		    /*
		     *  ??? Add value to link list element ???
		     */

		    grid_record->grid_details[best_word].matches = 
				    cword_memman_add_to_match_list (
					light_ll->str,
					(float) 0.0,
					grid_record->grid_details[best_word].matches);
#endif

		    if (grid_record->grid_details[best_word].matches == (CWORD_T_STR_LL *) 0)
			return (CWORD_S_FAILURE);
		}
#if 0
		else
		{
		    printf (
			    "### Possible match %s not acceptable as it breaks adjacency rules ###\n",
			    light_ll->word);
		}
#endif
	    }
	    
	    light_ll = (CWORD_T_STR_LL *) light_ll->next;
	}

	cword_strll_free_str_ll (&temp_light_ll);

	/*
	 *  Cuts the list of possible choices downto a manageable size by randomly
	 *  choosing the matches
	 */

	if (grid_record->grid_details[best_word].matches != (CWORD_T_STR_LL *) 0)
	{
	    status = cword_grid__prune_light_ll (
			cword_g_defaults_record.light_generation_algorithm,
			best_word,
			grid_record);

	    if (status != CWORD_S_NORMAL)
	    {
		return (CWORD_S_FAILURE);
	    }

	    /* 
	     *	If number of possible choices available less than certain value
	     *	then display the choices here so that the user can get a good
	     *	idea what's available before having to choose
	     */

	    if (cword_g_defaults_record.automatic_light_generation == FALSE)
	    {
		cword_grid__display_pruned_list (
				best_word,
				grid_record);
	    }
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE float cword_grid__word_stats (
	CWORD_T_GRID_STRUCT	*grid_record,
	int			word_number)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to get the proportion of the inputted word which is filled at
 *	the moment.  The word with the highest proportion completed (except a
 *	completed answer) is selected for generation next.  A completed answer
 *	returns 0.
 *
 *  FORMAL PARAMETERS:
 *
 *	*grid_record		(r) grid record dat structure
 *	word_number		(r) index of word to be checked 
 *
 *  FUNCTION VALUE:
 *
 *	Proportion of word yet to be filled by grid generator
 *--
 */

{

    float   word_value;

    int	    count,
	    letters_filled;

    count = letters_filled = 0;
    word_value = 0.0;

    while (grid_record->grid_details[word_number].light_template[count] != (char) 0)
    {
	if ( (grid_record->grid_details[word_number].light_template[count] != CROSS_CHAR) &&
	      (grid_record->grid_details[word_number].light_template[count] != BLANK_CHAR) )
	{
	    letters_filled++;
	}
        count++;
    }

    if (letters_filled < grid_record->grid_details[word_number].light_len)
    {
	word_value = (float) letters_filled / 
			 (float) grid_record->grid_details[word_number].light_len + 
			  grid_record->grid_details[word_number].light_len * UNFILLED_WEIGHT;
    }

    return (word_value);

}



PUBLIC int cword_grid_c_to_g_rec (
	CROSSWORD_RECORD	*crossword_record,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to translate a CROSSWORD_RECORD into a CWORD_T_GRID_STRUCT, or
 *	as much as possible into a CWORD_T_GRID_STRUCT.
 *
 *  FORMAL PARAMETERS:
 *
 *	*crossword_record	(r) crossword_record used to file grid_record
 *	*grid_record		(w) grid record to be filled using crossword_record
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		successful filling grid_record from crossword_record 
 *--
 */

{

    int	count;

    cword_messages_output_message (CWORD_CROSSTOGRID, TRACE2);

    strcpy (grid_record->name, crossword_record->file_name);

    /*
     *  ?? Possibly no answers yet available.  Seeding possibly though. ??
     */

    grid_record->clue_sum = crossword_record->clue_sum;

    grid_record->x_dimensions = crossword_record->x_dimensions;
    grid_record->y_dimensions = crossword_record->y_dimensions;
    grid_record->grid_type = crossword_record->grid_type;
    grid_record->symmetry = crossword_record->symmetry;

    /*
     *  No of iterations needed to get a solution from light filler 
     */

    grid_record->iteration_no = 0;

    grid_record->failures = (CWORD_T_STR_LL *) 0;

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	grid_record->grid_details[count].clue_number = 
		crossword_record->clue_details[count].clue_number;
	grid_record->grid_details[count].clue_direction = 
		crossword_record->clue_details[count].clue_direction;
	grid_record->grid_details[count].x_coordinate = 
		crossword_record->clue_details[count].x_coordinate;
	grid_record->grid_details[count].y_coordinate = 
		crossword_record->clue_details[count].y_coordinate;
	grid_record->grid_details[count].light_len = 
		crossword_record->clue_details[count].answer_length;	

	/*
	 *  Assuming no prior seeding has taken place before transfer to
	 *  CWORD_T_GRID_STRUCT format.
	 */

	grid_record->grid_details[count].grid_entry = LIGHT_EMPTY;

	if (crossword_record->clue_details[count].clue_answer[0] != (char) 0)
	{
	    if (strlen (crossword_record->clue_details[count].clue_answer) == 
	    	 crossword_record->clue_details[count].answer_length)
	    {
		strcpy (
		    grid_record->grid_details[count].light,
		    crossword_record->clue_details[count].clue_answer);
	    }
	}
	else
	{
	    memset (
		grid_record->grid_details[count].light, 
		0, 
		sizeof (grid_record->grid_details[count].light));
	}

	memset (
	    grid_record->grid_details[count].light_template, 0, 
	    sizeof (grid_record->grid_details[count].light_template));

	grid_record->grid_details[count].matches = (CWORD_T_STR_LL *) 0;
	grid_record->grid_details[count].match_num = 0;

	grid_record->grid_details[count].disallowed = (CWORD_T_STR_LL *) 0;
    }

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_grid_g_to_c_rec (
	CWORD_T_GRID_STRUCT	*grid_record,
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to translate a CWORD_T_GRID_STRUCT into a CROSSWORD_RECORD, or
 *	as much as possible into a CROSSWORD_RECORD.
 *
 *  FORMAL PARAMETERS:
 *
 *	*grid_record		(r) grid record to be filled using crossword_record
 *	*crossword_record	(w) crossword_record used to file grid_record
 *
 *  FUNCTION VALUE:
 *   
 *      CWORD_S_NORMAL		successful filling crossword_record from grid_record 
 *--
 */

{
    
    int count;

    cword_messages_output_message (CWORD_GRIDTOCROSS, TRACE2);

    strcpy (crossword_record->name, grid_record->name);
    strcpy (crossword_record->file_name, grid_record->name);
    crossword_record->remarks[0] = (char) 0;

    crossword_record->x_dimensions = grid_record->x_dimensions; 
    crossword_record->y_dimensions = grid_record->y_dimensions; 

    crossword_record->clue_sum = grid_record->clue_sum;
    crossword_record->grid_type = grid_record->grid_type;
    crossword_record->symmetry = grid_record->symmetry;

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	crossword_record->clue_details[count].clue_number = 
		grid_record->grid_details[count].clue_number;
	crossword_record->clue_details[count].clue_direction = 
		grid_record->grid_details[count].clue_direction;
	crossword_record->clue_details[count].x_coordinate = 
		grid_record->grid_details[count].x_coordinate;
	crossword_record->clue_details[count].y_coordinate = 
		grid_record->grid_details[count].y_coordinate;
	crossword_record->clue_details[count].answer_length= 
		grid_record->grid_details[count].light_len;	

	strcpy (
		crossword_record->clue_details[count].clue_description, 
		grid_record->grid_details[count].light);

	strcpy (
		crossword_record->clue_details[count].clue_answer, 
		grid_record->grid_details[count].light);
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Similar probably to verify function except for the use_best_word ??
 */

PUBLIC int cword_grid_get_grid_contents (
	int			use_best_word,
	CWORD_T_GRID_STRUCT	*grid_record,
	CROSSWORD_GRID		grid_contents)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CROSSWORD_GRID temp_grid_props;

    int count1,
	count2;

    DEBUG_TRACE ("new cword_grid_get_grid_contents\n");

    /*
     *  Initialise grid_contents.
     */

    for (count1 = 0; count1 < CWORD_SIZEOF_CELLDIM; count1++)
	for (count2 = 0; count2 < CWORD_SIZEOF_CELLDIM; count2++)
	{
	    grid_contents[count1][count2] = NOT_TAKEN;
	}

    /*
     *  Get grid properties before continuing.
     */

    cword_grid_get_grid_props (
	grid_record,
	temp_grid_props);

    /*
     *	Assume the grid definition is OK i.e. it's valid.
     *
     *	Get content for each cell in the grid and insert into grid_contents.
     */

    /*
     *	?? Bang out something that will allow two separate scenarios to be
     *	played out: ??
     *
     *	1. when this function is called with best_word > NOBESTWORDPHASE update the
     *	grid_contents data structure with CROSS_CHAR's only on checked cells and not
     *	on unchecked cells (which are left empty)!  The answer_template data
     *	structures are updated with the results of this function.
     *
     *	2. when this function is called with best_word == NOBESTWORDPHASE update the
     *	grid_contents data structure with CROSS_CHAR's on all used cells whether they
     *	are checked or unchecked.  The screen grid is displayed with the
     *	results of this function.  
     */

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	if (grid_record->grid_details[count1].clue_direction == ACROSS)
	{
	    for (count2 = 0; count2 < grid_record->grid_details[count1].light_len; count2++)
	    {
		if (grid_record->grid_details[count1].light[count2] != BLANK_CHAR)
		{
		    grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
			       [grid_record->grid_details[count1].y_coordinate-1] = 
				    (int) grid_record->grid_details[count1].light[count2];
		    DEBUG_TRACE (
			"grid_contents of light: %s x-val: %2d y-val: %2d is %c\n",
			 grid_record->grid_details[count1].light,
			 (grid_record->grid_details[count1].x_coordinate-1 + count2 +1),
			 (grid_record->grid_details[count1].y_coordinate-1 +1),
			 (char) grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
			       [grid_record->grid_details[count1].y_coordinate-1]);
		}
		else
		{
		    if (use_best_word == NOBESTWORDPHASE)
		    {
			if ( (temp_grid_props[grid_record->grid_details[count1].x_coordinate-1 + count2]
			       [grid_record->grid_details[count1].y_coordinate-1] == UNCH) &&
				(grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
				  [grid_record->grid_details[count1].y_coordinate-1] == NOT_TAKEN) )	  
			{
				grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
					    [grid_record->grid_details[count1].y_coordinate-1] = BLANK_CHAR;
			}
			else if ( (temp_grid_props[grid_record->grid_details[count1].x_coordinate-1 + count2]
				    [grid_record->grid_details[count1].y_coordinate-1] == CHECKED) &&
				     (grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
				       [grid_record->grid_details[count1].y_coordinate-1] == NOT_TAKEN) )	  
			{
				grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
					    [grid_record->grid_details[count1].y_coordinate-1] = CROSS_CHAR;
			}
		    }
		    else
		    {
			if ( (temp_grid_props[grid_record->grid_details[count1].x_coordinate-1 + count2]
			       [grid_record->grid_details[count1].y_coordinate-1] == UNCH) &&
				(grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
				  [grid_record->grid_details[count1].y_coordinate-1] == NOT_TAKEN) )	  
			{
			    if (use_best_word == count1)
				grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
					    [grid_record->grid_details[count1].y_coordinate-1] = UNKNOWN_CHAR;
			    else
				grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
					    [grid_record->grid_details[count1].y_coordinate-1] = BLANK_CHAR;
			}
			else if ( (temp_grid_props[grid_record->grid_details[count1].x_coordinate-1 + count2]
				    [grid_record->grid_details[count1].y_coordinate-1] == CHECKED) &&
				     (grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
				       [grid_record->grid_details[count1].y_coordinate-1] == NOT_TAKEN) )	  
			{
			    if (use_best_word == count1)
				grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
					    [grid_record->grid_details[count1].y_coordinate-1] = UNKNOWN_CHAR;
			    else
				grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
					    [grid_record->grid_details[count1].y_coordinate-1] = CROSS_CHAR;
			}
			else
			{
			    /*
			     *	Check if the cell in question is part of
			     *	use_best_word and also that the contents of the
			     *	cell isn't ASCII i.e. it already contains a
			     *	character.
			     */

			    if ( (use_best_word == count1) &&
				  (isalpha ((int) grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
					    [grid_record->grid_details[count1].y_coordinate-1]) == 0) )
			    {
				DEBUG_TRACE ("Slamming grid_contents with UNKNOWN_CHAR - ACROSS\n");

				grid_contents[grid_record->grid_details[count1].x_coordinate-1 + count2]
					    [grid_record->grid_details[count1].y_coordinate-1] = UNKNOWN_CHAR;
			    }
			}
		    }
		}
	    }
	}
	else
	{
	    for (count2 = 0; count2 < grid_record->grid_details[count1].light_len; count2++)
	    {
		if (grid_record->grid_details[count1].light[count2] != BLANK_CHAR)
		{
		    grid_contents[grid_record->grid_details[count1].x_coordinate-1]
			       [grid_record->grid_details[count1].y_coordinate-1 - count2] = 
				    (int) grid_record->grid_details[count1].light[count2];
		    DEBUG_TRACE (
			"grid_contents of light: %s x-val: %2d y-val: %2d is %c\n",
			 grid_record->grid_details[count1].light,
			 (grid_record->grid_details[count1].x_coordinate-1 +1),
			 (grid_record->grid_details[count1].y_coordinate-1 - count2 +1),
			 (char) grid_contents[grid_record->grid_details[count1].x_coordinate-1]
			       [grid_record->grid_details[count1].y_coordinate-count2]);
		}
		else
		{
		    if (use_best_word == NOBESTWORDPHASE)
		    {
			if ( (temp_grid_props[grid_record->grid_details[count1].x_coordinate-1]
			       [grid_record->grid_details[count1].y_coordinate-1 - count2] == UNCH) && 
				(grid_contents[grid_record->grid_details[count1].x_coordinate-1]
			          [grid_record->grid_details[count1].y_coordinate-1 - count2] == NOT_TAKEN) )
			{
			    grid_contents[grid_record->grid_details[count1].x_coordinate-1]
					[grid_record->grid_details[count1].y_coordinate-1 - count2] = BLANK_CHAR;
			}
			else if ( (temp_grid_props[grid_record->grid_details[count1].x_coordinate-1]
			            [grid_record->grid_details[count1].y_coordinate-1 - count2] == CHECKED) &&
				     (grid_contents[grid_record->grid_details[count1].x_coordinate-1]
			               [grid_record->grid_details[count1].y_coordinate-1 - count2] == NOT_TAKEN) )
			{
			    grid_contents[grid_record->grid_details[count1].x_coordinate-1]
					[grid_record->grid_details[count1].y_coordinate-1 - count2] = CROSS_CHAR;
			}
		    }
		    else
		    {
			if ( (temp_grid_props[grid_record->grid_details[count1].x_coordinate-1]
			       [grid_record->grid_details[count1].y_coordinate-1 - count2] == UNCH) && 
				(grid_contents[grid_record->grid_details[count1].x_coordinate-1]
			          [grid_record->grid_details[count1].y_coordinate-1 - count2] == NOT_TAKEN) )
			{
			    if (use_best_word == count1)
				grid_contents[grid_record->grid_details[count1].x_coordinate-1]
					    [grid_record->grid_details[count1].y_coordinate-1 - count2] = UNKNOWN_CHAR;
			    else
				grid_contents[grid_record->grid_details[count1].x_coordinate-1]
					    [grid_record->grid_details[count1].y_coordinate-1 - count2] = BLANK_CHAR;
			}
			else if ( (temp_grid_props[grid_record->grid_details[count1].x_coordinate-1]
			            [grid_record->grid_details[count1].y_coordinate-1 - count2] == CHECKED) &&
				     (grid_contents[grid_record->grid_details[count1].x_coordinate-1]
			               [grid_record->grid_details[count1].y_coordinate-1 - count2] == NOT_TAKEN) )
			{
			    if (use_best_word == count1)
				grid_contents[grid_record->grid_details[count1].x_coordinate-1]
					    [grid_record->grid_details[count1].y_coordinate-1 - count2] = UNKNOWN_CHAR;
			    else
				grid_contents[grid_record->grid_details[count1].x_coordinate-1]
					    [grid_record->grid_details[count1].y_coordinate-1 - count2] = CROSS_CHAR;
			}
			else
			{
			    /*
			     *	Check if the cell in question is part of
			     *	use_best_word and also that the contents of the
			     *	cell isn't ASCII i.e. it already contains a
			     *	character.
			     */

			    if ( (use_best_word == count1) && 
                                  (isalpha ((int) grid_contents[grid_record->grid_details[count1].x_coordinate-1]
				                 [grid_record->grid_details[count1].y_coordinate-1 - count2]) == 0) )
			    {
				DEBUG_TRACE ("Slamming grid_contents with UNKNOWN_CHAR - DOWN\n");

				grid_contents[grid_record->grid_details[count1].x_coordinate-1]
					    [grid_record->grid_details[count1].y_coordinate-1 - count2] = UNKNOWN_CHAR;
			    }
			}
		    }
		}
	    }
	}
    }

    /*
     *	Post processing: Convert grid_contents into a printable format before
     *	returning.
     */

    for (count1 = 0; count1 < grid_record->x_dimensions; count1++)
	for (count2 = 0; count2 < grid_record->y_dimensions; count2++)
	{
	    if (grid_contents[count1][count2] == NOT_TAKEN)
		grid_contents[count1][count2] = CWORD_UNFILLED_CHAR;
	    else if (grid_contents[count1][count2] == BLANK_CHAR)
		grid_contents[count1][count2] = CWORD_UNCHED_CHAR;
	    else if (grid_contents[count1][count2] ==  CROSS_CHAR)
		grid_contents[count1][count2] = CWORD_CHECKED_CHAR;
	}

#if 0
    /*
     *	This is the proper way to output to the user the two dimensional array
     *	containing the character cell contents of the grid found from
     *	grid_record.
     */

    for (count2 = 0; count2 < grid_record->y_dimensions; count2++)
    {
	printf ("## -> ");
	for (count1 = 0; count1 < grid_record->x_dimensions; count1++)
	{
	    printf (
		"%c",
		grid_contents[count1][grid_record->y_dimensions - 1 - count2]);
	}
	printf (" <- ##\n");
    }
    printf ("\n");
#endif

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Probably similar to verify function ??
 *
 *  ?? No support for barred grids yet ??
 *
 *  ?? Should I be calling this function so much - have a module global to
 *  handle the grid properties as they are not going to change during the life
 *  of the grid filler process ??
 */

PUBLIC int cword_grid_get_grid_props (
	CWORD_T_GRID_STRUCT	*grid_record,
	CROSSWORD_GRID		grid_properties)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Possible cell properties: NOT_TAKEN/UNCH/CHECKED.
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int count1,
	count2,
	temp1,
	temp2;

    BOOLEAN is_ok = TRUE;

    DEBUG_TRACE ("Calling new cword_grid_get_grid_props\n");

    /*
     *  Initialise grid_properties.
     */

    for (count1 = 0; count1 < CWORD_SIZEOF_CELLDIM; count1++)
	for (count2 = 0; count2 < CWORD_SIZEOF_CELLDIM; count2++)
	{
	    grid_properties[count1][count2] = NOT_TAKEN;
	}

    /*
     *	Assume the grid definition is OK i.e. it's valid.
     *
     *	Get NOT_TAKEN, UNCH, or CHECKED for each cell in the grid and insert
     *	into grid_properties.
     */

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	DEBUG_TRACE (
	    "Clue %d - %s x:%d y:%d len:%d ans:\"%s\" tpl:\"%s\"\n", 
	    grid_record->grid_details[count1].clue_number,
	    grid_record->grid_details[count1].clue_direction 
			? CWORD_ACROSS_STR : CWORD_DOWN_STR,
	    grid_record->grid_details[count1].x_coordinate,
	    grid_record->grid_details[count1].y_coordinate,
	    grid_record->grid_details[count1].light_len,
	    grid_record->grid_details[count1].light,
	    grid_record->grid_details[count1].light_template);

	if (grid_record->grid_details[count1].clue_direction == ACROSS)
	{
	    for (count2 = grid_record->grid_details[count1].x_coordinate;
		  count2 < (grid_record->grid_details[count1].x_coordinate + 
			     grid_record->grid_details[count1].light_len);
		   count2++)
	    {
		temp2 = grid_record->grid_details[count1].y_coordinate;

		switch (grid_properties[count2-1][temp2-1])
		{
		case NOT_TAKEN : 
		    grid_properties[count2-1][temp2-1] = UNCH;
		    break;

		case UNCH: 
		    grid_properties[count2-1][temp2-1] = CHECKED;
		    break;

		default:
		    printf ("# Error getting ACROSS composition of cell in grid\n");
		    is_ok = FALSE;
		    break;
		}

		if (is_ok == FALSE)
		{   
		    break;
		}

	    }
	}
	else
	{
	    for (count2 = grid_record->grid_details[count1].y_coordinate;
		  count2 > (grid_record->grid_details[count1].y_coordinate -
			     grid_record->grid_details[count1].light_len);
		   count2--)
	    {
		temp1 = grid_record->grid_details[count1].x_coordinate;

		switch (grid_properties[temp1-1][count2-1])
		{
		case NOT_TAKEN :
		    grid_properties[temp1-1][count2-1] = UNCH;
		    break;

		case UNCH:
		    grid_properties[temp1-1][count2-1] = CHECKED;
		    break;

		default:
		    printf ("# Error getting DOWN composition of cell in grid\n");
		    is_ok = FALSE;
		    break;
		}

		if (is_ok == FALSE)
		{   
		    break;
		}

	    }
	}
    }

#if 0
    /*
     *	This is the proper way to output to the user the two dimensional array
     *	containing the properties of the grid found from grid_record.
     */

    for (count2 = 0; count2 < grid_record->y_dimensions; count2++)
    {
	printf ("## -> ");

	for (count1 = 0; count1 < grid_record->x_dimensions; count1++)
	{
	    if (grid_properties[count1][grid_record->y_dimensions - 1 - count2] == UNCH)
		printf ("u");
	    else if (grid_properties[count1][grid_record->y_dimensions - 1 - count2] == CHECKED)
		printf ("c");
	    else if (grid_properties[count1][grid_record->y_dimensions - 1 - count2] == NOT_TAKEN)
		printf ("-");
	    else
		printf ("?");
	}

	printf (" <- ##\n");
    }
    printf ("\n");
#endif

    if (is_ok == FALSE)
	return (CWORD_S_FAILURE);
    else
	return (CWORD_S_NORMAL);

}



/*
 *  ?? Incorporate with seed__update_light_temps ??
 */

PUBLIC int cword_grid_update_light_temps (
	int			msg_id,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to update the newly found word(s) into grid_record data
 *	structure.
 *
 *      Format of the word types used here
 *      
 *              " A * " 5 letter word 
 *              / |  \
 *             /filled\
 *            / letter \
 *           /          \
 *      unimportant  unfilled crossed 
 *        letter           letter
 *
 *  FORMAL PARAMETERS:
 *
 *	msg_id			(r) msg_id To output to trace/screen
 *	*grid_record	    	(w) Grid record to be filled using crossword_record
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Updated the grid with the new answer successfully
 *--
 */

{

    CROSSWORD_GRID grid_contents,
		   grid_properties;

    int count1,
	count2,
	count3;

    if (msg_id != 0)
    {
	cword_messages_output_message (
	    msg_id, 
	    TRACE3);
    }

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	count2 = 0;
	while (count2 < grid_record->grid_details[count1].light_len)
	{
	    grid_record->grid_details[count1].light_template[count2] = BLANK_CHAR;
	    count2++;
	}
	grid_record->grid_details[count1].light_template[count2] = (char) 0;
    }

    /*
     *	Functions to fill grid_properties and grid_contents data structures.
     *
     *	From these (grid_properties, grid_contents) obtain the new
     *	answer_template values for the grid record.
     */

    cword_grid_get_grid_contents (
	NOBESTWORDPHASE,
	grid_record,
	grid_contents);

    cword_grid_get_grid_props (
	grid_record,
	grid_properties);

    /*
     *	Updating answer_template's.
     */

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	if (grid_record->grid_details[count1].clue_direction == ACROSS)
	{
	    count3 = 0;
	    for (count2 = grid_record->grid_details[count1].x_coordinate;
		  count2 < (grid_record->grid_details[count1].x_coordinate + 
			     grid_record->grid_details[count1].light_len);
		   count2++)
	    {
		if ( (grid_contents[count2-1][grid_record->grid_details[count1].y_coordinate-1] != CWORD_UNFILLED_CHAR) &&
		      (grid_contents[count2-1][grid_record->grid_details[count1].y_coordinate-1] != CWORD_UNCHED_CHAR) )
		{
		    if (grid_contents[count2-1][grid_record->grid_details[count1].y_coordinate-1] == CWORD_CHECKED_CHAR)
		    {
			grid_record->grid_details[count1].light_template[count3] = CROSS_CHAR;
		    }
		    else
		    {
			/*
			 *  Otherwise it has to be a letter!
			 */

			grid_record->grid_details[count1].light_template[count3] = 
				grid_contents[count2-1][grid_record->grid_details[count1].y_coordinate-1];
		    }

		    DEBUG_TRACE (
			"light_temps word %s x-val %2d y-val %2d\n", 
			grid_record->grid_details[count1].light_template,
			(count2-1), 
			(grid_record->grid_details[count1].y_coordinate-1));
		}
		else /* blank but ... */
		{
		    if (grid_properties[count2-1][grid_record->grid_details[count1].y_coordinate-1] == CHECKED)
		    {
			grid_record->grid_details[count1].light_template[count3] = CROSS_CHAR;
		    }
		}

		count3++;
	    }
	}
	else
	{
	    count3 = 0;
	    for (count2 = grid_record->grid_details[count1].y_coordinate;
		  count2 > (grid_record->grid_details[count1].y_coordinate -
			     grid_record->grid_details[count1].light_len);
		   count2--)
	    {
		if ( (grid_contents[grid_record->grid_details[count1].x_coordinate-1][count2-1] != CWORD_UNFILLED_CHAR) &&
		      (grid_contents[grid_record->grid_details[count1].x_coordinate-1][count2-1] != CWORD_UNCHED_CHAR) )
		{
		    if (grid_contents[grid_record->grid_details[count1].x_coordinate-1][count2-1] == CWORD_CHECKED_CHAR)
		    {
			grid_record->grid_details[count1].light_template[count3] = CROSS_CHAR;
		    }
		    else
		    {
			/*
			 *  Otherwise it has to be a letter!
			 */

			grid_record->grid_details[count1].light_template[count3] = 
				grid_contents[grid_record->grid_details[count1].x_coordinate-1][count2-1];
		    }

		    DEBUG_TRACE (
			"light_temps word %s x-val %2d y-val %2d\n", 
			grid_record->grid_details[count1].light_template,
			(grid_record->grid_details[count1].x_coordinate-1),
			(count2-1));
		}
		else /* blank but ... */
		{
		    if (grid_properties[grid_record->grid_details[count1].x_coordinate-1][count2-1] == CHECKED)
		    {
			grid_record->grid_details[count1].light_template[count3] = CROSS_CHAR;
		    }
		}

		count3++;
	    }
    	}
    }

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	DEBUG_TRACE (
	    "light index: %3u answer_template: \"%s\" answer: \"%s\"\n", 
	    count1, 
	    grid_record->grid_details[count1].light_template,
	    grid_record->grid_details[count1].light);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_grid__is_grid_finished (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Function to check whether the crosword generator has completed the crossword
 *
 *  FORMAL PARAMETERS:
 *
 *	*grid_record		(r) grid record to be filled using crossword_record
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		crossword is completed
 *	CWORD_S_FAILURE		crossword isn't finished - continue finding answers
 *--
 */

{

    int count1,
	count2;

    cword_messages_output_message (CWORD_ISGRIDFILLED, TRACE3);

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	count2 = 0;
	while (grid_record->grid_details[count1].light[count2] != (char) 0)
	{
	    if ( (grid_record->grid_details[count1].light[count2] == CROSS_CHAR) ||
		  (grid_record->grid_details[count1].light[count2] == BLANK_CHAR) )
            {
		cword_messages_output_message (CWORD_NORETURN, TRACE3);

		return (CWORD_S_FAILURE);
	    }
	    count2++;
	}
    }

    cword_messages_output_message (CWORD_YESRETURN, TRACE3);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_grid_check_disallowed (
	int			best_word,
	char			*selected_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Checks the selected word with the disallowed list of that word.
 *
 *  FORMAL PARAMETERS:
 *
 *      best_word		(r) the word being worked on presently by the generator
 *	selected_word		(r) word originaly selected but needs to be checked for validity
 *	*grid_record		(r) grid record to be filled using crossword_record
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Light is not already on disallowed list so it can be used.
 *	CWORD_S_FAILURE		Light is disallowed and thus generator should reject it.
 *--
 */

{

    CWORD_T_STR_LL *test_list = (CWORD_T_STR_LL *) 0;

    /*
     *	Not a good idea to have a message in this function as it is called many
     *	times during grid generation :-)
     */

    test_list = grid_record->grid_details[best_word].disallowed;

    while (test_list != (CWORD_T_STR_LL *) 0)
    {
	if (strcmp (selected_word, test_list->str) == 0)
	{
	    return (CWORD_S_FAILURE);
	}

	test_list = (CWORD_T_STR_LL *) test_list->next;

    }

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_grid_add_to_disallowed ( 
	int			word_mask,
	char			*disallowed_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to update the disallowed list in the grid_record data
 *	structure with disallowed_word.
 *
 *	Add to the disallowed list if the following conditions are true about
 *	the disallowed_word string:
 *
 *	(the word is the same size as the word selected) AND (the word is
 *	already in the crossword OR the word will lead to a deadend)
 *
 *  FORMAL PARAMETERS:
 *
 *      word_mask		(r) Word mask
 *	*disallowed_word	(r) Word to be added to the disallowed list
 *	*grid_record		(w) Grid record to be filled using crossword_record
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Added a word to the disallowed list successfully
 *--
 */

{

    CWORD_T_STR_LL *temp_list = (CWORD_T_STR_LL *) 0;
    
    int	count1,	
	present_already;

    if (word_mask == ALLWORDS)
    {
	/*
	 *  Update all word lists (equal in length - of course) in the grid
	 *  with disallowed word 
	 */

	cword_messages_output_message (
		CWORD_ADDALLDISALLOWED,
		TRACE3,
		disallowed_word);

	for (count1 = 0; count1 < grid_record->clue_sum; count1++)
	{
	    /* 
	     *  Firstly Check the length of the word
	     */

	    if (grid_record->grid_details[count1].light_len == strlen (disallowed_word))
	    {	    
		temp_list = grid_record->grid_details[count1].disallowed;

		present_already = FALSE;
		while (temp_list != (CWORD_T_STR_LL *) 0)
		{
		    if (strcmp (temp_list->str, disallowed_word) == 0)
		    {   
			present_already = TRUE;
			break;
		    }

		    temp_list = (CWORD_T_STR_LL *) temp_list->next;
		}

		if (present_already != TRUE)
		{
		    /*
		     *  Add disallowed_word to the disallowed list of the specific word
		     */

		    cword_strll_add_str (
			SORTED | UNIQUE,
			disallowed_word,
			&grid_record->grid_details[count1].disallowed);

		    if (grid_record->grid_details[count1].disallowed == (CWORD_T_STR_LL *) 0)
			return (CWORD_S_FAILURE);
		}
	    }
	}
    }
    else
    {
	/*
	 *  word only check
	 */

	cword_messages_output_message (
		CWORD_ADDONEDISALLOWED,
		TRACE3,
		disallowed_word, 
		grid_record->grid_details[word_mask].clue_number,  
		grid_record->grid_details[word_mask].clue_direction 
			    ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

	if (grid_record->grid_details[word_mask].light_len == strlen (disallowed_word))
	{
	    temp_list = grid_record->grid_details[word_mask].disallowed;

	    present_already = FALSE;
	    while (temp_list != 0)
	    {
		if (strcmp (temp_list->str, disallowed_word) == 0)
		{   
		    present_already = TRUE;
		    break;
		}

		temp_list = (CWORD_T_STR_LL *) temp_list->next;
	    }

	    if (present_already == FALSE)
	    {
		/*
		 *  Add disallowed_word to the disallowed list of the specific word
		 */

		DEBUG_TRACE (
		    "Adding disallowed word %s to clue %d %s details\n",
		    disallowed_word,
		    grid_record->grid_details[word_mask].clue_number,
		    grid_record->grid_details[word_mask].clue_direction 
			    ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		    cword_strll_add_str (
			SORTED | UNIQUE,
			disallowed_word,
			&grid_record->grid_details[word_mask].disallowed);

		if (grid_record->grid_details[word_mask].disallowed == (CWORD_T_STR_LL *) 0)
		    return (CWORD_S_FAILURE);
	    }
	}

	if (grid_record->grid_details[word_mask].matches != (CWORD_T_STR_LL *) 0)
	{
	    cword_strll_del_str (
		disallowed_word,
		&grid_record->grid_details[word_mask].matches);

#if 0
	    grid_record->grid_details[word_mask].matches = 
				cword_memman_del_from_match_ll (
				    disallowed_word,
				    grid_record->grid_details[word_mask].matches);
#endif

	    grid_record->grid_details[word_mask].match_num--;
	}
    }

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_grid_del_from_disallowed (
	int			word_mask,
	char			*disallowed_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Word successfully found and deleted
 *      CWORD_S_FAILURE		Word unsuccessfully found or some other error recorded
 *--
 */

{

    int count;

    if (word_mask == ALLWORDS)
    {
	for (count = 0; count < grid_record->clue_sum; count++)
	{
	    /*
	     *	This call may or many not be successful.
	     */

	    cword_strll_del_str (
		disallowed_word,
		&grid_record->grid_details[count].disallowed);
	}
    }
    else
    {
	/*
	 *  Should always be successful!
	 */

	cword_strll_del_str (
	    disallowed_word,
	    &grid_record->grid_details[word_mask].disallowed);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_grid_template_sameas_list (
	char			*template,
	CWORD_T_STR_LL		*matches_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check whether the current answet_template string is similar
 *	to the matches_ll link list.  If te list contains word which match
 *	template then there is no need to get more possible matches_ll from the
 *	word file.  If they are not the same then
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		template compared with match list
 *	CWORD_S_FAILURE		template didn't compare with match list
 *--
 */

{

    /*
     *	Compare the first element of matches_ll with the template string.
     */

    int count = 0;

    BOOLEAN words_same = TRUE;

    if (matches_ll == (CWORD_T_STR_LL *) 0)
    {
	return (CWORD_S_FAILURE);
    }

    while (template[count] != (char) 0)
    {
	if ((template[count] != (char) BLANK_CHAR) &&
	     (template[count] != (char) CROSS_CHAR))
	{
	    /*
	     *  i.e. template[count] == 'ASCII letter of some sort'
	     */

	    if (template[count] != matches_ll->str[count])
	    {
		DEBUG_TRACE (
		    "%c and %c not same in words %s and %s\n", 
		    template[count],
		    matches_ll->str[count],
		    template,
		    matches_ll->str);

		words_same = FALSE;
		break;
	    }
	}

        count++;
    }

    if (words_same == TRUE)
	return (CWORD_S_NORMAL);
    else
	return (CWORD_S_FAILURE);

}



PRIVATE cword_grid__check_light_ll (
	char			*search_str,
	CWORD_T_STR_LL		*search_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    CWORD_T_STR_LL *temp_ll;

    char temp_str[CWORD_SIZEOF_MESSAGE + 1];

    int count = 0;

    strcpy (temp_str, search_str);

    DEBUG_TRACE ("Checking \"%s\" against failures list\n", temp_str);

    /*
     *	Take out CROSS_CHAR characters from input string before checking string
     *	against failures linked list.
     */

    while (count < strlen (temp_str)) 
    {
	if (temp_str[count] == CROSS_CHAR)
	    temp_str[count] = BLANK_CHAR;

	count++;
    }

    temp_ll = search_ll;

    while (temp_ll != (CWORD_T_STR_LL *) 0)
    {
	if (strcmp (temp_ll->str, temp_str) == 0)
	{
	    cword_messages_output_message (
		    CWORD_TPLATEUSEDBEFRE,
		    (TRACE2 | SCREEN0),
		    temp_ll->str);

	    return (CWORD_S_FAILURE);
	}

	temp_ll = temp_ll->next;
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_grid_add_to_failures (
	char			*failure_str,
	CWORD_T_STR_LL		**failures_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    int count = 0;

    DEBUG_TRACE (
	"Add \"%s\" to failures list\n",
	failure_str);

    /*
     *	Take out CROSS_CHAR characters from input string before adding string
     *	to failures linked list.
     */

    while (count < strlen (failure_str)) 
    {
	if (failure_str[count] == CROSS_CHAR)
	    failure_str[count] = BLANK_CHAR;

	count++;
    }

    cword_strll_add_str (
	SORTED | UNIQUE,
	failure_str,
	failures_ll);

    if (*failures_ll == (CWORD_T_STR_LL *) 0)
	return (CWORD_S_FAILURE);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_grid__check_for_ow_lights (
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is used to notify the light generator if there have been
 *	illegal lights generated inadventently by the insertion of legal lights
 *	into checking locations of the illegal light in question.  If this is
 *	the case then the calling function will take the necessary action to
 *	rectify the matter.
 *
 *  FORMAL PARAMETERS:
 *
 *	best_word		(r) 
 *	*grid_record		(r) pointer to instance of a CWORD_T_GRID_STRUCT
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int count,
	count1,
	status;

    char temp_ch;

    BOOLEAN is_filled;

    /*
     *	Have to find the new answer_template situation to see whether any
     *	lights have been inadventently filled by the addition of the light
     *	indexed by best_word.
     */

    status = cword_grid_update_light_temps (
		CWORD_UPDATETOGRID,
		grid_record);

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if ( (grid_record->grid_details[count].grid_entry == LIGHT_EMPTY) &&
	       (count != best_word) )
	{
	    is_filled = TRUE;
	    for (count1 = 0; count1 < grid_record->grid_details[count].light_len; count1++)
	    {
		if ( (grid_record->grid_details[count].light_template[count1] == BLANK_CHAR) ||
		      (grid_record->grid_details[count].light_template[count1] == CROSS_CHAR) )
		{
		    is_filled = FALSE;
		    break;
		}
	    }

	    if (is_filled == TRUE)
	    {
		cword_messages_output_message (
			CWORD_OVERGENERATEMSG,
			(TRACE2 | SCREEN0),
			grid_record->grid_details[best_word].clue_number, 
			grid_record->grid_details[best_word].clue_direction 
				? CWORD_ACROSS_STR : CWORD_DOWN_STR,
			grid_record->grid_details[count].clue_number, 
			grid_record->grid_details[count].clue_direction
				? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		/*
		 *  ?? Accept this word or not = check spelling first of all ??
		 */

		status = cword_match_spell_check (grid_record->grid_details[count].light_template);

		if (status != CWORD_S_NORMAL)
		{
		    cword_messages_output_message (
			    CWORD_LIGHTSPELLERR,
			    SCREEN0,
			    grid_record->grid_details[count].light_template,
			    CWORD_Y_CHAR,
			    CWORD_N_CHAR,
			    CWORD_N_CHAR);

		    temp_ch = cword_charui_get_ch (FALSE);

		    if ( (temp_ch == (char) 0) || (tolower (temp_ch) == CWORD_N_CHAR) )
			temp_ch = CWORD_N_CHAR;
		}
		else
		    temp_ch = CWORD_Y_CHAR;

		if ( (toupper (temp_ch) == CWORD_Y_CHAR) ||
		      (CWORD_S_NORMAL == status) )
		{
		    strcpy (
			grid_record->grid_details[count].light,
			grid_record->grid_details[count].light_template);

		    cword_messages_output_message (
			    CWORD_OVERGENLIGHTADD,
			    (TRACE2 | SCREEN0),
			    grid_record->grid_details[count].light,
			    grid_record->grid_details[count].clue_number, 
			    grid_record->grid_details[count].clue_direction 
				    ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		    cword_messages_output_message (
			    CWORD_SELECTEDWORD,
			    (TRACE0 | SCREEN0),
			    grid_record->grid_details[count].clue_number, 
			    grid_record->grid_details[count].clue_direction 
				    ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
			    grid_record->grid_details[count].light);

		    status = cword_grid_add_to_disallowed (
				ALLWORDS,
				grid_record->grid_details[count].light,
				grid_record);    

		    if (status != CWORD_S_NORMAL)
		    {
			return (CWORD_S_FAILURE);
		    }

		    cword_messages_output_message (
			    CWORD_ITERATIONFINISHED,
			    (TRACE0),
			    grid_record->iteration_no);

		    /*
		     *	Update the grid record to tell in which order the crossword grid was
		     *	filled and also just to say that the light was properly generated by
		     *	the light generator.
		     */

		    grid_record->grid_details[count].grid_entry = grid_record->iteration_no;
		}
		else
		{
		    cword_messages_output_message (
			    CWORD_OVERGENLIGHTDEL,
			    (TRACE2 | SCREEN0),
			    grid_record->grid_details[count].light,
			    grid_record->grid_details[count].clue_number, 
			    grid_record->grid_details[count].clue_direction 
				    ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		    cword_messages_output_message (
			    CWORD_MATCHDONEFAIL,
			    (TRACE2 | SCREEN0),
			    grid_record->grid_details[best_word].light_template);

		    cword_messages_output_message (
			    CWORD_ADDTOFAILURES,
			    TRACE2,
			    grid_record->grid_details[best_word].light_template);

		    cword_grid_add_to_failures (
			    grid_record->grid_details[best_word].light_template,
			    &grid_record->failures);

		    for (count = 0; count < grid_record->grid_details[best_word].light_len; count++)
			grid_record->grid_details[best_word].light[count] = (char) BLANK_CHAR;

		    grid_record->grid_details[best_word].grid_entry = LIGHT_DEADEND;

		    return (CWORD_DEADENDALERT);
		}
	    }
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_grid__select_best_av (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to select the best unfilled light for generation.  The best
 *	unfilled light is selected by executing cword_grid__word_stats on each
 *	of the lights in the grid definition.  The index of the selected light
 *	is returned through the function return value.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(r) pointer to instance of CWORD_T_GRID_STRUCT
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    float selected_value = 0.0,
	  word_value = 0.0;

    int	count1,
	selected_light = NOWORDCHOSEN;

    /*
     *	?? Better algorithm possible by checking only those lights that are
     *	still LIGHT_EMPTY rather than the whole lot ?? 
     */

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	word_value = cword_grid__word_stats (
				grid_record,
				count1);

	if (word_value > selected_value)
	{
	    selected_light = count1;
	    selected_value = word_value;
	}
    }

    /*
     *  Return the index of the light which will now be generated
     */

    return (selected_light);

}



PRIVATE int cword_grid__select_long_av (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	A function to select the longest unfilled light for generation.  The
 *	index of the selected light is returned through the function return
 *	value.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(r) pointer to instance of CWORD_T_GRID_STRUCT
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int	selected_light = NOWORDCHOSEN,
	current_length = 0,
	count1;

    /*
     *	Iterate through the list and figure out which light is longest *and*
     *	still unfilled.
     */

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	if ( (grid_record->grid_details[count1].light_len > current_length) &&
              (grid_record->grid_details[count1].grid_entry == LIGHT_EMPTY) )
	{
	    current_length = grid_record->grid_details[count1].light_len;
	    selected_light = count1;
	}
    }

    /*
     *  Return the index of the light which will now be generated
     */

    return (selected_light);

}



PRIVATE int cword_grid__select_short_av (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	A function to select the shortest unfilled light for generation.  The
 *	index of the selected light is returned through the function return
 *	value.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(r) pointer to instance of CWORD_T_GRID_STRUCT
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int	selected_light = NOWORDCHOSEN,
	current_length = CWORD_SIZEOF_CELLDIM,
	count1;

    /*
     *	Iterate through the list and figure out which light is shortest *and*
     *	still unfilled.
     */

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	if ( (grid_record->grid_details[count1].light_len < current_length) &&
              (grid_record->grid_details[count1].grid_entry == LIGHT_EMPTY) )
	{
	    current_length = grid_record->grid_details[count1].light_len;
	    selected_light = count1;
	}
    }

    /*
     *  Return the index of the light which will now be generated
     */

    return (selected_light);

}



PRIVATE int cword_grid__select_random_av  (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	A function to select a random unfilled light for generation.  The index
 *	of the selected light is returned through the function return value.
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    int light_selected = NOWORDCHOSEN,
	no_of_iterations = 0,
	index;

    /*
     *	Keep randomly generating numbers until it hits on an index of a light
     *	that is still ungenerated.  Cool :-).
     */

    while (no_of_iterations < 5000) /* I know it's anonymous -- tough */    
    {
	index = (int) ((double) (cword_utils_get_random_number () * 
				  grid_record->clue_sum)) / 100.0;

	if (grid_record->grid_details[index].grid_entry == LIGHT_EMPTY)
	{
	    light_selected = index;
	    break;
	}

	no_of_iterations++;
    }

    if (no_of_iterations == 5000)
    {
	printf ("## Random number generator probably (99.99%%) doesn't work correcly\n");
	printf ("## Please investigate as it may need re-coding for this platform\n");
    }

    return (light_selected);

}



PRIVATE int cword_grid__select_next_light (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to find the next (acceptable) light to be filled in the grid
 *	defined by grid_record.
 *
 *  FORMAL PARAMETERS:
 *
 *	*grid_record		(r) grid record to be filled using crossword_record
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_g_defaults_record.automatic_light_generation 
 *      cword_g_defaults_record.light_generation_algorithm
 *      cword_g_defaults_record.nextlight_select_algorithm
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		A word was found with best chance of being matched
 *				successfully with adjoinging answers in the grid.
 *	CWORD_S_FAILURE		No word found suitable for crossword. Exitting.
 *--
 */

{

    char    temp_ch;

    int	    count,
	    status,
	    best_word = NOWORDCHOSEN;

    grid_record->iteration_no++;

    cword_messages_output_message (
	    CWORD_ITERATIONNO,
	    (TRACE2 | SCREEN0),
	    grid_record->iteration_no);

    cword_messages_output_message (CWORD_FINDNEXTLIGHT, TRACE2);

    /*
     *	Get the next most appropriate light to fill into the grid.  This is
     *	important as this choice influences in what direction and what shape
     *	the grid will be generated.
     */

    switch (cword_g_defaults_record.nextlight_select_algorithm)
    {
    case (BEST_LIGHT):
	/*
	 *  Still the best way of filling a grid at the moment
	 */

	best_word = cword_grid__select_best_av (grid_record);

	DEBUG_TRACE (
	    "Using cword_grid__select_best_av to select index %d\n",
	    best_word);

	break;

    case (RANDOM_LIGHT):
	/*
	 *  ?? Sloppy way of selecting the next light for filling -- many
	 *  problems with it unfortunately ??
	 */

	best_word = cword_grid__select_random_av (grid_record);

	DEBUG_TRACE (
	    "Using cword_grid__select_random_av to select index %d\n",
	    best_word);

	break;

    case (LONGEST_LIGHT):
	/*
	 *  ?? Can't think why this way would be any cleaner or better than a
	 *  random fill ??
	 */

	best_word = cword_grid__select_long_av (grid_record);

	DEBUG_TRACE (
	    "Using cword_grid__select_long_av to selected index %d\n",
	    best_word);

	break;

    case (SHORTEST_LIGHT):
	/*
	 *  ?? Can't think why this way would be any cleaner or better than a
	 *  random fill ??
	 */

	best_word = cword_grid__select_short_av (grid_record);

	DEBUG_TRACE (
	    "Using cword_grid__select_short_av to selected index %d\n",
	    best_word);

	break;

    default:
	printf ("## Error assigning next light for filling algorithm\n");
    }

    if (best_word == NOWORDCHOSEN)
    {
	return (CWORD_S_FAILURE);
    }

    /* 
     *  ?? Check that the best word is an allowable template to search for
     *  in the word files ??
     */

    /*
     *	?? Write function to check linked list for specific element - return
     *	NORMAL if present otherwise FAILURE.  Use the failures link list to
     *	check the status of such ??
     */

    /* 
     *	Checks the current best_word answer_template against the templates
     *	which have already failed to return any valid choices.
     */

    status = cword_grid__check_light_ll (
		    grid_record->grid_details[best_word].light_template,
		    grid_record->failures);

    if (CWORD_S_NORMAL != status)
    {
	/*
	 *  Found failures match template in the list of failures -> LIGHT_DEADEND
	 *  found
	 */

	cword_messages_output_message (
		CWORD_MATCHDONEFAIL,
		(TRACE2 | SCREEN0),
		grid_record->grid_details[best_word].light_template);

	grid_record->grid_details[best_word].grid_entry = LIGHT_DEADEND;

	return (CWORD_DEADENDALERT);
    }

    if (cword_g_defaults_record.trace_level == LEVEL3)
    {
	/*
	 *  Only output the current grid definition if trace_level is LEVEL3!
	 */

	CROSSWORD_GRID grid_contents;

        cword_grid_get_grid_contents (
	    best_word,
	    grid_record,
	    grid_contents);

	cword_charui_output_grid (
	    0,
	    (SCREEN3 | TRACE3),
	    TRUE,
	    grid_record->x_dimensions,
	    grid_record->y_dimensions,
	    grid_contents);
    }

    /*
     *	Is the answer_template the same as the list of words already in place
     *	if not then call get_word_match else use currently extant word list.
     */

    cword_messages_output_message (
	    CWORD_CHECKSEARCHAGAIN,
	    (TRACE2 | SCREEN0),
	    grid_record->grid_details[best_word].light_template);

    if (cword_grid_template_sameas_list (
			    grid_record->grid_details[best_word].light_template,
			    grid_record->grid_details[best_word].matches) == CWORD_S_FAILURE)
    {
	cword_messages_output_message (CWORD_YESRETURN, (TRACE2 | SCREEN0));

	cword_strll_free_str_ll (&grid_record->grid_details[best_word].matches);

#if 0
	cword_memman_free_match_list (&grid_record->grid_details[best_word].matches);
#endif

    	/*
	 *  If there is an error returned from this function then we can't grab
	 *  the words we need and thus have to exit grid generation 
	 */

	status = cword_grid__get_word_match (
		    best_word,
		    grid_record);

	if (status != CWORD_S_NORMAL)
	{
	    return (CWORD_S_FAILURE);
	}
    }
    else
    {
	cword_messages_output_message (CWORD_NORETURN, (TRACE2 | SCREEN0));
    }

    /*
     *  N.B No words available for the current choice => Backtrack
     */

    if (grid_record->grid_details[best_word].matches == (CWORD_T_STR_LL *) 0)
    {
	cword_messages_output_message (CWORD_DEADENDALERT, (TRACE2 | SCREEN0));

	grid_record->grid_details[best_word].grid_entry = LIGHT_DEADEND;

	cword_messages_output_message (
		CWORD_ADDTOFAILURES,
		TRACE2,
		grid_record->grid_details[best_word].light_template);

	cword_grid_add_to_failures (
		grid_record->grid_details[best_word].light_template,
		&grid_record->failures);

	return (CWORD_DEADENDALERT);
    }

    /*
     *	This function may not need to be called depending upon the strategy
     *	chosen to select words/lights with.
     */

    if (cword_g_defaults_record.automatic_light_generation == TRUE)
    {
	/*
	 *  Let the light generator select the new light for the grid
	 */

	status = cword_grid__select_word (
				cword_g_defaults_record.light_generation_algorithm,
				best_word,
				grid_record);

	if (status != CWORD_S_NORMAL)
	{
	    return (CWORD_S_FAILURE);
	}
    }
    else
    {
	/*
	 *  Let the user select the new light for the grid from a list found by
	 *  the template matcher and light generator!
	 */

	/*
	 *  ?? Make into separate function as too much detail is already
	 *  encapulated in here ??
	 */

	/*
	 *  ?? No need for the select word function to be called as the words
	 *  should be presented in sequential format for the user to choose
	 *  from ??
	 */

	status = cword_grid__select_word (
		     cword_g_defaults_record.light_generation_algorithm,
		     best_word,
		     grid_record);

	if (status != CWORD_S_NORMAL)
	{
	    return (CWORD_S_FAILURE);
	}

	cword_messages_output_message (
		CWORD_LIGHTACCEPTQ,
		SCREEN0,
		grid_record->grid_details[best_word].light, 
		CWORD_Y_CHAR,
		CWORD_N_CHAR,
		CWORD_Y_CHAR);

	temp_ch = cword_charui_get_ch (FALSE);

	if (temp_ch == (char) 0)
	    temp_ch = CWORD_Y_CHAR;

	while ( (toupper (temp_ch) != CWORD_Y_CHAR) &&
		 (grid_record->grid_details[best_word].matches != (CWORD_T_STR_LL *) 0) )
	{
	    if (grid_record->grid_details[best_word].light[0] != (char) 0)
	    {
		cword_strll_add_str (
		    SORTED | UNIQUE,
		    grid_record->grid_details[best_word].light,
		    &grid_record->grid_details[best_word].disallowed);

		if (grid_record->grid_details[best_word].disallowed == (CWORD_T_STR_LL *) 0)
		    return (CWORD_S_FAILURE);

		cword_strll_del_str (
		    grid_record->grid_details[best_word].light,
		    &grid_record->grid_details[best_word].matches);

#if 0
		grid_record->grid_details[best_word].matches = 
			    cword_memman_del_from_match_ll (
				grid_record->grid_details[best_word].light,
				grid_record->grid_details[best_word].matches);
#endif

		if (grid_record->grid_details[best_word].matches == (CWORD_T_STR_LL *) 0)
		    return (CWORD_S_FAILURE);

		grid_record->grid_details[best_word].match_num--;

		memset (
			grid_record->grid_details[best_word].light, 
			0,
			grid_record->grid_details[best_word].light_len);
	    }

	    if (grid_record->grid_details[best_word].matches != (CWORD_T_STR_LL *) 0)
	    {
		/*
		 *  Get another light if possible from backup list
		 */

		status = cword_grid__select_word (
					cword_g_defaults_record.light_generation_algorithm,
					best_word,
					grid_record);

		if (status != CWORD_S_NORMAL)
		{
		    return (CWORD_S_FAILURE);
		}

		cword_messages_output_message (
			CWORD_LIGHTACCEPTQ,
			SCREEN0,
			grid_record->grid_details[best_word].light, 
			CWORD_Y_CHAR,
			CWORD_N_CHAR,
			CWORD_Y_CHAR);

		temp_ch = cword_charui_get_ch (FALSE);
	    }
	    else
	    {
		/*
		 *  Get more words from template matcher
		 */

		cword_messages_output_message (CWORD_GETTINGMORELIGHTS, SCREEN0);

		status = cword_grid__get_word_match (
			    best_word,
			    grid_record);

		if (status != CWORD_S_NORMAL)
		{
		    return (CWORD_S_FAILURE);
		}

		if (grid_record->grid_details[best_word].matches == (CWORD_T_STR_LL *) 0)
		{
		    cword_messages_output_message (CWORD_DEADENDMSG, (TRACE2 | SCREEN0));

		    break;
		}
	    }
	}
    }

    /*
     *  ?? Possible deadend - Not quite right I think! ??
     */

    if (grid_record->grid_details[best_word].matches == (CWORD_T_STR_LL *) 0)
    {
	cword_messages_output_message (CWORD_DEADENDALERT, (TRACE2 | SCREEN0));

	for (count = 0; count < grid_record->grid_details[best_word].light_len; count++)
	    grid_record->grid_details[best_word].light[count] = (char) BLANK_CHAR;

	grid_record->grid_details[best_word].grid_entry = LIGHT_DEADEND;

	cword_messages_output_message (
		CWORD_ADDTOFAILURES,
		TRACE2,
		grid_record->grid_details[best_word].light_template);

	cword_grid_add_to_failures (
		grid_record->grid_details[best_word].light_template,
		&grid_record->failures);

	return (CWORD_DEADENDALERT);
    }

    /*
     *	Check that no lights have been inadventently filled.  This scenario
     *	isn't possible with most blocked grids but it is very possible with
     *	fully checked grids, and more rarely with barred grid where some of the
     *	words are fully checked.
     */

    status = cword_grid__check_for_ow_lights (
		best_word,
		grid_record);

    if (CWORD_DEADENDALERT == status)
    {
	return (status);
    }

    /*
     *	If we get down to here, everything is OK to continue.
     */

    cword_messages_output_message (
	    CWORD_SELECTEDWORD,
	    (TRACE0 | SCREEN0),
	    grid_record->grid_details[best_word].clue_number, 
	    grid_record->grid_details[best_word].clue_direction 
		    ? CWORD_ACROSS_STR : CWORD_DOWN_STR,
	    grid_record->grid_details[best_word].light);

    status = cword_grid_update_light_temps (
		CWORD_UPDATETOGRID,
		grid_record);

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    status = cword_grid_add_to_disallowed (
		ALLWORDS,
		grid_record->grid_details[best_word].light,
		grid_record);    

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    cword_messages_output_message (
	    CWORD_ITERATIONFINISHED,
	    TRACE0,
	    grid_record->iteration_no);

    /*
     *	Update the grid record to tell in which order the crossword grid was
     *	filled and also just to say that the light was properly generated by
     *	the light generator.
     */

    grid_record->grid_details[best_word].grid_entry = grid_record->iteration_no;

    return (CWORD_S_NORMAL);

}



PRIVATE float cword_grid__find_intersect_let (
	CWORD_T_LIGHT_STRUCT	*light1,
	CWORD_T_LIGHT_STRUCT	*light2)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    char    intersect_ch;

    int	    x_coord,
	    y_coord;

    if (light1->clue_direction == ACROSS)
    {
	x_coord = light2->x_coordinate;
	y_coord = light1->y_coordinate;
    }
    else
    {
	x_coord = light1->x_coordinate;
	y_coord = light2->y_coordinate;
    }

    DEBUG_TRACE (
	"Intersect location is located at (%d,%d)\n", 
	x_coord, 
	y_coord);

    /* 
     *  Found (x_coord, y_coord) - now find the intersecting letter
     */

    if (light1->clue_direction == ACROSS)
    {
	if (light1->grid_entry > LIGHT_EMPTY)
	{
	    /*
	     *  There is a fully found answer already in light1
	     */

	    intersect_ch = light1->light[x_coord - light1->x_coordinate];
	}
	else
	{
	    /*
	     *  There is a fully found answer already in light2
	     */

	    intersect_ch = light2->light[light2->y_coordinate - y_coord];
	}

	DEBUG_TRACE (
    	    "light1 = ACROSS -> intersect character is \'%c\'\n",
    	    intersect_ch);
    }
    else
    {
	if (light1->grid_entry > LIGHT_EMPTY)
	{
	    intersect_ch = light1->light[light1->y_coordinate - y_coord];
	}
	else
	{
	    intersect_ch = light2->light[x_coord - light2->x_coordinate];
	}

	DEBUG_TRACE (
    	    "light2 = ACROSS -> intersect character is \'%c\'\n",
    	    intersect_ch);
    }

    return (1.0 - cword_grid__update_weight (intersect_ch));

}



PRIVATE int cword_grid__backtrack_grid (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function will allow the grid generator to backtrack upon finding a
 *	grid which cannot be completed successfully.  This function will delete
 *	the most appropriate light generated, add this light to the disallowed
 *	list and continue generating.  If this function cannot decide on a
 *	suitably computer generated light to backtrack to then there are two
 *	options available:
 *
 *	    1.	Delete a seed (if any) and continue generating [Make a default
 *	    value - to delete seed or not].
 *
 *	    2.	Exit grid generation in disgrace (should never happen if there
 *	    is/are large word file(s) available and no user seeds - think about it
 *	    and you'll see the unlikelihood of this happening).
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(r/w) pointer an instance of a CWORD_T_GRID_STRUCT
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Seed found for backtracking 
 *--
 */

{

    /*
     *  Set grid_entry = LIGHT_EMPTY for the clue/light to be downdated
     *
     *  Update disallowed list for that particular clue/light to be downdated
     *
     *  Downdate the grid details to the scenario before the deleted light was
     *  included
     */

    float   temp_val = 0.0;

    int	    bad_light = -1,
	    temp_index = -1,
	    count = 0,
	    status = CWORD_S_NORMAL;

    /*
     *  Find LIGHT_DEADENDed clue
     */

    while (count < grid_record->clue_sum)
    {
	if (grid_record->grid_details[count].grid_entry == LIGHT_DEADEND)
	{
	    bad_light = count;
	    break;
	}
	count++;
    }

    if (bad_light == -1)
    {
	cword_messages_output_message (CWORD_DEADENDMSG, (TRACE2 | SCREEN0));

	return (CWORD_S_FAILURE);
    }

    /*
     *  Reset the bad light 
     */

    grid_record->grid_details[bad_light].grid_entry = LIGHT_EMPTY;

    /*
     *	No point just finding the most recently generated light as it may have
     *	been in a different "grid block" than the current problem light
     *	template.  We have to find the computer generated light which
     *	intersects the current problem light template and delete that rather
     *	than an innocent bystander on anothe block :-) 
     */

#ifdef MOST_RECENT

    /*
     *	Implementation of the deletion of the light most recently found by the
     *	light generator
     */

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if (grid_record->grid_details[count].grid_entry	> (int) temp_val)
	{
	    temp_index = count;
	    temp_val = (float) grid_record->grid_details[count].grid_entry;
	}
    }

#endif

    /*
     *	Find the most suitable light to delete from the grid definition
     */

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	/* 
	 *  The bad_light and the light being deleted/backtracked have to be in
	 *  opposite directions
	 */

	if ( (grid_record->grid_details[count].clue_direction != 
	       grid_record->grid_details[bad_light].clue_direction) &&
	      (count != bad_light) )
	{
	    if (grid_record->grid_details[count].clue_direction == ACROSS)
	    {
		if ( ((grid_record->grid_details[count].x_coordinate <=
		       grid_record->grid_details[bad_light].x_coordinate) &&
		      (grid_record->grid_details[bad_light].x_coordinate <=
		       (grid_record->grid_details[count].x_coordinate + 
		        grid_record->grid_details[count].light_len - 1))) &&
		     ((grid_record->grid_details[count].y_coordinate >=
		       (grid_record->grid_details[bad_light].y_coordinate -
		        grid_record->grid_details[bad_light].light_len + 1)) &&
		      (grid_record->grid_details[count].y_coordinate <=
		       grid_record->grid_details[bad_light].y_coordinate)) )
		{
		    DEBUG_TRACE (
			    "Intersect found between %d %s and %d %s\n",
			    (char *) grid_record->grid_details[count].clue_number, 
			    grid_record->grid_details[count].clue_direction 
						? CWORD_ACROSS_STR : CWORD_DOWN_STR,
			    (char *) grid_record->grid_details[bad_light].clue_number, 
			    grid_record->grid_details[bad_light].clue_direction 
						? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		    if (grid_record->grid_details[count].grid_entry > LIGHT_EMPTY)
		    {
			/* 
			 *  Find the intersecting letter of the two lights and
			 *  decide whether this letter is most likely causing
			 *  the deadend and delete accordingly
			 */

			if (cword_grid__find_intersect_let (
				&grid_record->grid_details[count],
				&grid_record->grid_details[bad_light]) > temp_val)
			{
			    temp_index = count;
			    temp_val = cword_grid__find_intersect_let (
					    &grid_record->grid_details[count],
					    &grid_record->grid_details[bad_light]);
			}
		    }
		}
		else
		{
		    DEBUG_TRACE (
			"count -> %d-%d and bad_light -> %d-%d\n",
			grid_record->grid_details[count].x_coordinate,
			grid_record->grid_details[count].y_coordinate,
			grid_record->grid_details[bad_light].x_coordinate,
			grid_record->grid_details[bad_light].y_coordinate);
		}
	    }
	    else
	    {
		if ( ((grid_record->grid_details[bad_light].x_coordinate <=
		       grid_record->grid_details[count].x_coordinate) &&
		      (grid_record->grid_details[count].x_coordinate <=
		       (grid_record->grid_details[bad_light].x_coordinate + 
		        grid_record->grid_details[bad_light].light_len - 1))) &&
		     ((grid_record->grid_details[bad_light].y_coordinate >=
		       (grid_record->grid_details[count].y_coordinate -
		        grid_record->grid_details[count].light_len + 1)) &&
		      (grid_record->grid_details[bad_light].y_coordinate <=
		       grid_record->grid_details[count].y_coordinate)) )
		{
		    DEBUG_TRACE (
			    "Intersect found between %d %s and %d %s\n",
			    (char *) grid_record->grid_details[count].clue_number, 
			    grid_record->grid_details[count].clue_direction 
						? CWORD_ACROSS_STR : CWORD_DOWN_STR,
			    (char *) grid_record->grid_details[bad_light].clue_number, 
			    grid_record->grid_details[bad_light].clue_direction 
						? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		    if (grid_record->grid_details[count].grid_entry > LIGHT_EMPTY)
		    {
			if (cword_grid__find_intersect_let (
				&grid_record->grid_details[count],
				&grid_record->grid_details[bad_light]) > temp_val)
			{
			    temp_index = count;
			    temp_val = cword_grid__find_intersect_let (
					    &grid_record->grid_details[count],
					    &grid_record->grid_details[bad_light]);
			}
		    }
		}
		else
		{
		    DEBUG_TRACE (
			"count -> %d-%d and bad_light -> %d-%d\n",
			grid_record->grid_details[count].x_coordinate,
			grid_record->grid_details[count].y_coordinate,
			grid_record->grid_details[bad_light].x_coordinate,
			grid_record->grid_details[bad_light].y_coordinate);
		}
	    }
	}
    }

    /*
     *	if temp_index == -1 then have to delete seed if allowable since no
     *	computer generated seed have been found for deletion!
     */

    if (temp_index == -1)
    {
	cword_messages_output_message (CWORD_DELETESEED, (TRACE2 | SCREEN0));

	/*
	 *  Find a suitable SEEDED candidate for deletion from the grid
	 *
	 *  There must be some to get into this situation.  If there are none
	 *  then exit (should be impossible if there are full word files
	 *  available)
	 *
	 *  ?? Have to decide whether the deletion of seeds is allowable ??
	 */ 

	for (count = 0; count < grid_record->clue_sum; count++)
	{
	    if (grid_record->grid_details[count].grid_entry == LIGHT_SEEDED)
	    {
		temp_index = count;

		cword_messages_output_message (
			CWORD_DELETESEEDFOUND,
			(TRACE2 | SCREEN0),
			grid_record->grid_details[temp_index].clue_number, 
			grid_record->grid_details[temp_index].clue_direction 
					    ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		break;
	    }
	}

	if (temp_index == -1)
	{
	    /*
	     *	If no seed is found for deletion we have to exit here with an
	     *	error which will propogate back upto the highest levels
	     */

	    cword_messages_output_message (CWORD_GRIDGIVEUP, (TRACE2 | SCREEN0));

	    return (CWORD_S_FAILURE);
	}
    }

    /*
     *  Process of backtracking now taking place
     */

    cword_messages_output_message (
		CWORD_BACKTRACKGRID,
		(TRACE2 | SCREEN0),
		grid_record->grid_details[temp_index].clue_number, 
		grid_record->grid_details[temp_index].clue_direction 
				    ? CWORD_ACROSS_STR : CWORD_DOWN_STR,  
		grid_record->grid_details[temp_index].light);

    grid_record->grid_details[temp_index].grid_entry = LIGHT_EMPTY;

    if (grid_record->grid_details[temp_index].match_num > 0)
    {
	DEBUG_TRACE ("Deleting \"%s\" from matches list\n", grid_record->grid_details[temp_index].light);

	cword_strll_del_str (
	    grid_record->grid_details[temp_index].light,
	    &grid_record->grid_details[temp_index].matches);

#if 0
	grid_record->grid_details[temp_index].matches = 
			    cword_memman_del_from_match_ll (
				grid_record->grid_details[temp_index].light,
				grid_record->grid_details[temp_index].matches);
#endif

	grid_record->grid_details[temp_index].match_num--;
    }

    DEBUG_TRACE (
	"Adding \"%s\" from disallowed list\n",
	grid_record->grid_details[temp_index].light);

    cword_strll_add_str (
	SORTED | UNIQUE,
	grid_record->grid_details[temp_index].light,
	&grid_record->grid_details[temp_index].disallowed);

    if (grid_record->grid_details[temp_index].disallowed == (CWORD_T_STR_LL *) 0)
	return (CWORD_S_FAILURE);

    cword_strll_free_str_ll (&grid_record->grid_details[temp_index].matches);

#if 0
    cword_memman_free_match_list (&grid_record->grid_details[temp_index].matches);
#endif

    for (count = 0; count < grid_record->grid_details[temp_index].light_len; count++)
    {
	grid_record->grid_details[temp_index].light[count] = BLANK_CHAR;
    }

    status = cword_grid_update_light_temps (
		CWORD_UPDATETOGRID,
		grid_record);

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE void cword_grid_clean_up_grid_rec (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    int count;

    cword_strll_free_str_ll (&grid_record->failures);

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	cword_strll_free_str_ll (&grid_record->grid_details[count].matches);

#if 0
	cword_memman_free_match_list (&grid_record->grid_details[count].matches);
#endif

	cword_strll_free_str_ll (&grid_record->grid_details[count].disallowed);
    }

    return;

}



PRIVATE void cword_grid_set_up_record (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    int	count;

    grid_record->clue_sum = 0;

    grid_record->x_dimensions = 0;
    grid_record->y_dimensions = 0;

    grid_record->iteration_no = 0;

    grid_record->failures = (CWORD_T_STR_LL *) 0;

    for (count = 0; count < CWORD_SIZEOF_CLUENUM; count++)
    {
	grid_record->grid_details[count].clue_number = 0;
	grid_record->grid_details[count].clue_direction = 0;
	grid_record->grid_details[count].x_coordinate = 0;
	grid_record->grid_details[count].y_coordinate = 0;
	grid_record->grid_details[count].light_len = 0;

	/*
	 *  Assuming no prior seeding has taken place before transfer to
	 *  CWORD_T_GRID_STRUCT format.
	 */

	grid_record->grid_details[count].grid_entry = LIGHT_EMPTY;

	memset (
	    grid_record->grid_details[count].light, 
	    0, 
	    sizeof (grid_record->grid_details[count].light));

	memset (
	    grid_record->grid_details[count].light_template, 0,
	    sizeof (grid_record->grid_details[count].light_template));

	grid_record->grid_details[count].matches = (CWORD_T_STR_LL *) 0;
	grid_record->grid_details[count].match_num = 0;
	grid_record->grid_details[count].disallowed = (CWORD_T_STR_LL *) 0;
    }

    return;

}



PRIVATE int cword_grid__fill_grid_poss (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check whether a filled grid is possible given the word
 *	files currently defined for the job of filling.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(r) pointer to instance of CWORD_T_GRID_STRUCT
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.word_file_access
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    /*
     *	1. Have to transfer grid record into crossword record and then get
     *	analysis of light lengths.
     *
     *	2. Check that the light lengths are all represented in the word files
     *	currently defined.
     *
     *	3. If light present in defined grid that isn't represented in the
     *	corresponding word files then check whether it's been user defined.  If
     *	not then we cannot continue else we can.
     */

    CROSSWORD_RECORD *temp_crossword_record;

    CWORD_T_LENFREQ_STRUCT light_freq;

    int	length,
	count1;

    BOOLEAN is_fill_possible = TRUE;

    temp_crossword_record = cword_memman_make_crossword_rec ();

    cword_grid_g_to_c_rec (
	grid_record,
	temp_crossword_record);

    cword_stats_get_clue_dist (
	    temp_crossword_record,
	    light_freq);

    cword_memman_free_cross_rec (temp_crossword_record);

    length = 0;
    while (length < CWORD_SIZEOF_CELLDIM)
    {
	if (light_freq[length] > 0)
	{
	    if (cword_g_defaults_record.word_file_access[length] == FALSE)
	    {
		printf (
		    "## No word file of length %d exists\n", 
		    length);

		/*
		 *  ?? Is this code necessary ??
		 */
    
		for (count1 = 0; count1 < grid_record->clue_sum; count1++)
		{
		    if (grid_record->grid_details[count1].light_len == length)
		    {
			if (grid_record->grid_details[count1].grid_entry != LIGHT_SEEDED)
			{
			    is_fill_possible = FALSE;
			}
		    }
		}
	    }

	    if (is_fill_possible == FALSE)
	    {
		printf ("## Cannot continue with fill :-(\n");

		break;
	    }
	}

	length++;
    }
 
    if (is_fill_possible == TRUE)
	return (CWORD_S_NORMAL);
    else
    {
	cword_messages_output_message (
		CWORD_S_GRIDFILLNOTPOSS,
		(TRACE2 | SCREEN0),
		length); 

	return (CWORD_S_FAILURE);
    }

}



#ifdef DB

#if defined(VMS) || defined(__VMS) || defined(__osf__)

PRIVATE int cword_grid__get_clues_from_db (
	CROSSWORD_RECORD        *crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to go off to the database and fetch the clues for each of the
 *	inputted lights.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(w) Pointer to a CROSSWORD_RECORD structure
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		successful in getting the clues from the db
 *	CWORD_S_FAILURE		unsuccessful in getting the clues from the db
 *--
 */

{

    /*
     *	Go though the crossword_record struct and query the db for the clue for
     *	each word.
     */

    int	count1,
	status;

    cword_messages_output_message (
	    CWORD_S_GETCLUESFROMDB,
	    (TRACE2 | SCREEN0));    

    /*
     *	?? A better way of handling the fetching of clue details from the db
     *	than currently implemented would be to input the whole crossword into
     *	the rdb function.  This rdb function will examine every piece of
     *	returned data for a match with a light.  If a match is found then the
     *	clue details are copied into the crossword_record, otherwise we keep
     *	going until the end of the cursor ??
     */

    for (count1 = 0; count1 < crossword_record->clue_sum; count1++)
    {
	printf (
	    "## Getting clue for %s\n", 
	    crossword_record->clue_details[count1].clue_answer);

	status = cword_rdb_get_clue (
		crossword_record->clue_details[count1].clue_answer,
		crossword_record->clue_details[count1].clue_description);

	if (CWORD_S_NORMAL != status)
	{
	    cword_messages_output_message (
		    CWORD_S_GRIDGETANSWER,
		    (TRACE2 | SCREEN0),
		    crossword_record->clue_details[count1].clue_answer);

	    break;
	}
    }

    /*
     *  ?? What happens if something goes wrong while fetching from db ??
     */

    cword_messages_output_message (
	    CWORD_S_GRIDGOTANSWERS,
	    (TRACE2 | SCREEN0));

    return (CWORD_S_NORMAL);

}

#endif

#endif



PRIVATE int cword_grid__reset_grid_struct (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    /*
     *	Clean down the grid_record and restart the generation of lights
     */

    printf ("##### Resetting grid for generation of lights #####\n");

    grid_record->iteration_no = 0;

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_grid__percent_completed (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to calculate - to the nearest percent - how much of the grid
 *	is filled.  This function is important when the user cannot visually
 *	see how much of the grid has been filled.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(r) pointer to instance of CWORD_T_GRID_STRUCT
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      Percentage of grid filled expressed as an integer quantity
 *--
 */

{

    int count,
	total_filled = 0;

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if (grid_record->grid_details[count].light[0] != BLANK_CHAR)
	    total_filled++;
    }

    return ((int) ((float) (total_filled * 100) / (float) grid_record->clue_sum));

}



/*
 *  ?? Badly placed function.  Should be most likely in clue.c ??
 */

PUBLIC int cword_grid_get_misc_details (
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion.
 *--
 */

{

    char crossword_str[CWORD_SIZEOF_LINE + 1];

    /*
     *	Give a a name for the crossword being created.  This will be
     *	independent of the name given to the file in which this crossword will
     *	be saved.
     */

    /*
     *	?? Limit size of crossword name and crossword remarks to 50 for
     *	printing purposes ??
     */

    cword_messages_output_message (
	CWORD_S_GRIDGETXRDNAME, 
	SCREEN0,
	crossword_record->name[0] == (char) 0 ? CWORD_NONE_STR : crossword_record->name);

    if (cword_charui_get_str (
	    ALL,
	    BLANK_OK,
	    50,
	    crossword_str) == CWORD_S_NORMAL)
    {
	if (crossword_str[0] != (char) 0)
	{
	    printf (
		"## Filled crossword is named : \"%s\"\n",
		crossword_str);

	    strcpy (crossword_record->name, crossword_str);
	}
	else
	{
	    printf ("## Filled crossword will take default name\n");
	}
    }
    else
    {
	printf ("## Filled crossword will not named\n");

	crossword_record->name[0] = (char) 0;
    }

    memset (crossword_str, 0, sizeof (crossword_str));

    cword_messages_output_message (
	CWORD_S_GRIDGETREMARKS, 
	SCREEN0,
	crossword_record->remarks[0] == (char) 0 ? CWORD_NONE_STR : crossword_record->remarks);

    if (cword_charui_get_str (
	    ALL,
	    BLANK_OK,
	    50,
	    crossword_str) == CWORD_S_NORMAL)
    {
	if (crossword_str[0] != (char) 0)
	{
	    printf (
		"## Filled crossword remarks are : \"%s\"\n",
		crossword_str);

	    strcpy (crossword_record->remarks, crossword_str);
	}
	else
	{
	    printf ("## Filled crossword will take default remarks\n");
	}
    }
    else
    {
	printf ("## Filled crossword has no remarks\n");

	crossword_record->remarks[0] = (char) 0;
    }

    return (CWORD_S_NORMAL);

}



PUBLIC BOOLEAN cword_grid_is_fully_seeded (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Grid is fully seeded.
 *	FALSE			Grid is not fully seeded.
 *--
 */

{

    unsigned int count;

    BOOLEAN is_grid_fully_seeded = TRUE;

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if (grid_record->grid_details[count].grid_entry != LIGHT_SEEDED)
	{
	    is_grid_fully_seeded = FALSE;

	    break;
	}
    } 

    if (is_grid_fully_seeded == TRUE)
	return (TRUE);
    else
	return (FALSE);

}



PUBLIC int cword_grid_fill_desc ( 
	CROSSWORD_RECORD	*crossword_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function which controls the filling with lights of the inputted
 *	crossword definition before outputting to a crossword file.
 *
 *  FORMAL PARAMETERS:
 *
 *	*crossword_record	(w) record into which the info about the grid
 *				    will be inputted
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		grid successfully filled
 *	CWORD_S_FAILURE		grid unsuccessfully filled
 *--
 */

{

    CWORD_T_GRID_STRUCT *grid_record;

    char first_str[CWORD_SIZEOF_MESSAGE + 1],
	 second_str[CWORD_SIZEOF_MESSAGE + 1],
	 third_str[CWORD_SIZEOF_MESSAGE + 1],
	 fourth_str[CWORD_SIZEOF_MESSAGE + 1];

    time_t start_time,
	   end_time;

    int	status,
	grid_status,
	count1,
	count2,
	total_iterations = 0;

    cword_messages_output_message (
	CWORD_GRIDMAKESTART,
	(TRACE2 | SCREEN0),
	crossword_record->name);

    /*
     *  All future work will take place in this specially constructed struct!
     */

    grid_record = cword_memman_make_grid_rec ();

    status = cword_stats_gen_clue_nums (crossword_record);

    if (CWORD_S_NORMAL != status)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *  Sort the crossword contents before conrinuing!
     */

    cword_utils_sort_crossword (crossword_record);

    /*
     *  Use a grid_record from now on
     */

    status = cword_grid_c_to_g_rec (
		crossword_record,
		grid_record);

    /*
     *	?? Sets up the answer and answer template for use with blanks.
     *	Initialisation purposes ONLY ??
     *
     *	Could alternatively use pre-defined blank or otherwise character
     *	strings/numerals to signify nothing being in those fields
     */

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	count2 = 0;
	while (count2 < grid_record->grid_details[count1].light_len)
	{
	    grid_record->grid_details[count1].light_template[count2] = BLANK_CHAR;
	    grid_record->grid_details[count1].light[count2] = BLANK_CHAR;
	    count2++;
	}
	grid_record->grid_details[count1].light_template[count2] = (char) 0;
	grid_record->grid_details[count1].light[count2] = (char) 0;
    }

    /*
     *	Use this function to allow user input own choices for words to be
     *	seeded in the grid.
     */

    status = cword_seed_input_seeds (grid_record);

    if (status != CWORD_S_NORMAL)
    {
	/*
	 *  ?? Either an error was recorded during seeding or filling is not
	 *  required in this instance ??
	 */

	if (CWORD_S_SEEDQUITNOW == status)
	{
	    return (status);
	}
	else
	    return (CWORD_S_FAILURE);
    }

    /*
     *  !FILL THE GRID if it's not already fully seeded by the user!
     */

    if (cword_grid_is_fully_seeded (grid_record) == TRUE)
    {
	CROSSWORD_GRID grid_contents;

	printf ("## Grid is fully seeded - No need to continue with filling phase\n");

	cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	/*
	 *  ?? Make into function ??
	 */

	cword_grid_clean_up_grid_rec (grid_record);

	cword_grid_get_grid_contents (
	    NOBESTWORDPHASE,
	    grid_record,
	    grid_contents);

	cword_charui_output_grid (
	    0,
	    (SCREEN0 | TRACE0),
	    FALSE,
	    grid_record->x_dimensions,
	    grid_record->y_dimensions,
	    grid_contents);

	/*
	 *  Revert back to the crossword record for return from function.
	 */

	status = cword_grid_g_to_c_rec (
		    grid_record,
		    crossword_record);

	cword_grid_get_misc_details (crossword_record);

	cword_utils_complete_definition (crossword_record);

	cword_memman_free_grid_rec (grid_record);

	return (CWORD_S_NORMAL);
    }

    /*
     *	?? Output a useful (and messages.h formatted) message describing how
     *	the grid is to be filled before it is filled :-) ??
     */

    if (cword_g_defaults_record.letter_weight_algorithm == EQUAL_WEIGHTS)
	strcpy (first_str, "## Equally weighted");
    else if (cword_g_defaults_record.letter_weight_algorithm == DICT_WEIGHTS)
	strcpy (first_str, "## Dicstats weighted");
    else if (cword_g_defaults_record.letter_weight_algorithm == SCRABBLE_WEIGHTS)
	strcpy (first_str, "## Scrabble weighted");

    if (cword_g_defaults_record.light_generation_algorithm == BESTWORDS)
	strcpy (second_str, "## Optimally found words");
    else
	strcpy (second_str, "## Randomly found words");

    if (cword_g_defaults_record.automatic_light_generation == TRUE)
	strcpy (third_str, "## Computer selected lights");
    else
	strcpy (third_str, "## User selected lights");

    if (cword_g_defaults_record.fill_from_where == FILL_FROM_WORD_FILE)
	strcpy (fourth_str, "## from word files");
    else
	strcpy (fourth_str, "## from Rdb database");

    /* 
     *	This message should really be going out to the trace file but isn't
     *	because of the possibility of it being written multiple times.
     */

    cword_charui_ring_bell (1);

    cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);
    cword_messages_output_message (
	    CWORD_S_HOWGRIDFILL,
	    (SCREEN0 | TRACE0),
	    first_str,
	    second_str,
	    third_str,
	    fourth_str);
    cword_messages_output_message (
	    CWORD_WAITINGENDED,
	    SCREEN0);

    /*
     *  Just pause for a second before starting light generation proper!
     */

    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

    status = cword_grid_update_light_temps (
		CWORD_ADDSEEDSTOGRID,
		grid_record);

    if (status != CWORD_S_NORMAL)
    {
	cword_memman_free_grid_rec (grid_record);

	return (CWORD_S_FAILURE);
    }

    /*
     *	Check if it's possible from the word files Cword has at its disposal
     *	whether it can generate a completed grid
     */

    status = cword_grid__fill_grid_poss (grid_record);

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *	While the light generator isn't finished continue to get lights from the
     *	word files, until the grid is completed, or some error condition is found
     *	that will not allow the light generator to go any further.
     */

    grid_status = NOTFINISHED;

    start_time = time (0);

    while (grid_status == NOTFINISHED)  /* Is the grid completed or not yet */
    {
	cword_messages_output_message (
		    CWORD_S_PERCENTFILLED,
		    (TRACE2 | SCREEN0),
		    cword_grid__percent_completed (grid_record));

	/*
	 *  Find the next light to be filled into the crossword grid.
	 */

	status = cword_grid__select_next_light (grid_record);

	if (status == CWORD_S_FAILURE)
	{
	     grid_status = ERRORFINISHING;
    	}
    	else
    	if (status == CWORD_DEADENDALERT) /* Backtrack now */
    	{
	    /*
	     *  Backtracking to continue
	     */

	    status = cword_grid__backtrack_grid (grid_record);

	    if (status != CWORD_S_NORMAL)
	    {
		 grid_status = ERRORFINISHING;
	    }
	}
	else
	{
	    /*
	     *	Check if the grid is completed.
	     */

	    status = cword_grid__is_grid_finished (grid_record);

	    if (status == CWORD_S_NORMAL)
	    {
		grid_status = FINISHED;
	    }
	}

	total_iterations++;

	DEBUG_TRACE (
	    "total_iterations is %d with grid_record->iteration_no equaling %d\n",
	    total_iterations,
	    grid_record->iteration_no);

	if (total_iterations > cword_g_defaults_record.max_light_iterations)
	{
	    grid_status = UNFINISHED;
	}

	if (grid_record->iteration_no > cword_g_defaults_record.reset_after_num_iterations)
	{
	    /*
	     *	If the grid generation phase hasn't completed after a certain
	     *	number of iterations then clean down the grid and start again
	     */

	    cword_grid__reset_grid_struct (grid_record);	    
	}
    }

    cword_grid_clean_up_grid_rec (grid_record);

    /*
     *  Get the total time taken to fill grid.
     *
     *  ?? What about if fill time is only 1 second ??
     */

    end_time = time (0);    

    cword_messages_output_message (
		CWORD_S_FILLTIMEWAS,
		(TRACE2 | SCREEN0),
		end_time - start_time);

    if (grid_status == FINISHED)
    {
	CROSSWORD_GRID grid_contents;

	/*
	 *  Wake up the user if he's fallen asleep.
	 */

	cword_charui_ring_bell (3);

	cword_messages_output_message (CWORD_GRIDFINISHED, (TRACE0 | SCREEN0));

	cword_grid_get_grid_contents (
	    NOBESTWORDPHASE,
	    grid_record,
	    grid_contents);

	cword_charui_output_grid (
	    0,
	    (SCREEN0 | TRACE0),
	    FALSE,
	    grid_record->x_dimensions,
	    grid_record->y_dimensions,
	    grid_contents);

	/*
	 *  Revert back to the crossword record for return from function.
	 */

	status = cword_grid_g_to_c_rec (
		    grid_record,
		    crossword_record);

#ifdef DB

#if defined(VMS) || defined(__VMS) || defined(__osf__)

	if (cword_g_defaults_record.fill_from_where == FILL_FROM_DB)
	{
	    /*
	     *	If we are using the database filler then go off to the database
	     *	and fetch the clues for the defined lights.
	     */

	    status = cword_grid__get_clues_from_db (crossword_record);
	}

#endif

#endif

	cword_grid_get_misc_details (crossword_record);

	cword_utils_complete_definition (crossword_record);

	cword_memman_free_grid_rec (grid_record);

	return (CWORD_S_NORMAL);
    }

    cword_memman_free_grid_rec (grid_record);

    if (grid_status == UNFINISHED)
    {
	cword_messages_output_message (
		    CWORD_ITERATIONSOVER,
		    (TRACE2 | SCREEN0));
    }
    else if (grid_status == ERRORFINISHING)
    {
	cword_messages_output_message (
		    CWORD_ERRUNDEFINED,
		    (TRACE2 | SCREEN0));
    }
    else
    {
	printf ("# Should not be here - grid_status is undefined\n");
    }

    return (CWORD_S_FAILURE);

}



#else

extern int main (
		int argv, 
		char *argc[])	

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Main function to test functionality of the grid generator for use in
 *	the Cword application set
 *
 *  FORMAL PARAMETERS:
 *
 *      argv		(r) number of parameters inputted
 *	*argc[]		(r) contains the parameters inputted at command line
 *--
 */
{

    CROSSWORD_RECORD	*crossword_record;
    CROSSWORD_GRID	crossword_grid;

    int			status;

#if 0
    if (argv < 2)
    {
	printf ("No file name entered\n");
	exit (EXIT_FAILURE);
    }
#endif

    CWORD_OS_CLEAR_SCREEN ();

#if 0
    status = cword_fileio_read_crossword (
			    argc[1],
			    crossword_record);

    if (status == CWORD_S_FAILURE)
    {
	exit (EXIT_FAILURE);
    }

#endif

    status = cword_fileio_read_grid_details (
		    "james1.grid",
		    crossword_record);

    status = cword_stats_gen_clue_nums (
		    crossword_record);

    if (status != CWORD_S_NORMAL)
    {
	exit (EXIT_FAILURE);
    }

    status = cword_grid_fill_desc (crossword_record);
    
    if (status != CWORD_S_NORMAL)
    {
	exit (EXIT_FAILURE);
    }
 
    status = cword_fileio_write_crossword (
		    "james1.input",
		    crossword_record);

    if (status == CWORD_S_FAILURE)
    {
	printf ("error in writing output file\n");
	exit (1);
    }    

    exit (0);

}

#endif
