#ifndef CWORD_GRID_LOADED
#define CWORD_GRID_LOADED 1

#include "main.h"

/*
 *  Relative values of each letters as per my definitive dictionary. 
 */

/*
 *  ?? *_PROB possibly redundant ??
 */

#define A_PROB 0.249637
#define B_PROB 0.053494
#define C_PROB 0.129122
#define D_PROB 0.095082
#define E_PROB 0.315557
#define F_PROB 0.033112
#define G_PROB 0.068844
#define H_PROB 0.079620
#define I_PROB 0.260694
#define J_PROB 0.004444
#define K_PROB 0.023087
#define L_PROB 0.163377
#define M_PROB 0.089523
#define N_PROB 0.203288
#define O_PROB 0.212328
#define P_PROB 0.096953
#define Q_PROB 0.004914
#define R_PROB 0.209281
#define S_PROB 0.214272
#define T_PROB 0.195760
#define U_PROB 0.108215
#define V_PROB 0.026353
#define W_PROB 0.019864
#define X_PROB 0.008945
#define Y_PROB 0.059107
#define Z_PROB 0.012023

/*
 *  Equal weight/probability for each letter
 */

#define EQUAL_PROB  0.100000

/*
 *  Scrabble letter weights
 */

#define SCRABBLE_PROB_A (float)  9 / (float) 100
#define SCRABBLE_PROB_B (float)  2 / (float) 100
#define SCRABBLE_PROB_C (float)  2 / (float) 100
#define SCRABBLE_PROB_D (float)  4 / (float) 100
#define SCRABBLE_PROB_E (float) 12 / (float) 100
#define SCRABBLE_PROB_F (float)  2 / (float) 100
#define SCRABBLE_PROB_G (float)  3 / (float) 100
#define SCRABBLE_PROB_H (float)  2 / (float) 100
#define SCRABBLE_PROB_I (float)  9 / (float) 100
#define SCRABBLE_PROB_J (float)  1 / (float) 100
#define SCRABBLE_PROB_K (float)  1 / (float) 100
#define SCRABBLE_PROB_L (float)  4 / (float) 100
#define SCRABBLE_PROB_M (float)  2 / (float) 100
#define SCRABBLE_PROB_N (float)  6 / (float) 100
#define SCRABBLE_PROB_O (float)  8 / (float) 100
#define SCRABBLE_PROB_P (float)  2 / (float) 100
#define SCRABBLE_PROB_Q (float)  1 / (float) 100
#define SCRABBLE_PROB_R (float)  6 / (float) 100
#define SCRABBLE_PROB_S (float)  4 / (float) 100
#define SCRABBLE_PROB_T (float)  6 / (float) 100
#define SCRABBLE_PROB_U (float)  4 / (float) 100
#define SCRABBLE_PROB_V (float)  2 / (float) 100
#define SCRABBLE_PROB_W (float)  2 / (float) 100
#define SCRABBLE_PROB_X (float)  1 / (float) 100
#define SCRABBLE_PROB_Y (float)  2 / (float) 100
#define SCRABBLE_PROB_Z (float)  1 / (float) 100

/*
 *  Size of the link list containing choices of words for currently inputted
 *  template
 */

#define CHOICE_LL_SIZE	    25

/*
 *  Define some constants used when outputting a grid/crossword definition
 */

#define OUTPUT_GRID	    1
#define OUTPUT_CROSSWORD    2

/*
 *  Letter weighting constants
 */

#define EQUAL_WEIGHTS	    1
#define DICT_WEIGHTS	    2
#define SCRABBLE_WEIGHTS    3

/*
 *  Prune list options
 */

#define RANDOMWORDS	    1
#define BESTWORDS	    2

/*
 *  Next light selection options
 */

#define BEST_LIGHT	    1
#define RANDOM_LIGHT	    2
#define LONGEST_LIGHT	    3
#define SHORTEST_LIGHT	    4

/*
 *  Fill from where constants
 */

#define FILL_FROM_WORD_FILE 1

#if defined(VMS) || defined(__VMS) || defined(__osf__)
#define FILL_FROM_DB        2
#endif

/*
 *  Possible values for the grid_entry field in CWORD_T_GRID_STRUCT struct
 *
 *  All other values for grid_entry should be positive and describe what
 *  iteration of the grid filler algorithm that the light in question got
 *  filled succesfully on.
 */

#define LIGHT_EMPTY     0
#define LIGHT_DEADEND   -1
#define LIGHT_SEEDED    -2

/*
 *  Used in cword_grid__out_evolving_grid to display the without highlighting
 *  the next best_word location
 */

#define	NOBESTWORDPHASE	-1
#define SEEDINGPHASE	-2

/*
 *  Used in cword_grid__update_disallowed to updates all light definitions that
 *  a word is to be disallowed - in deference to just one light definition
 */

#define ALLWORDS	-1

/*
 *  Template definitions used during seeding and filling.
 */

#define BLANK_CHAR      ' '
#define CROSS_CHAR      '*'
#define UNKNOWN_CHAR    '?'

#define CROSS_STR       "*"

/*
 *  Definitions for structs necessary for grid filling.
 */

typedef struct cword_t_light_struct
{
    unsigned int   clue_number;
    unsigned int   clue_direction;
    unsigned int   x_coordinate;
    unsigned int   y_coordinate;
    unsigned int   light_len;
    signed int	   grid_entry; /* Can be negative */

    char	   light[CWORD_SIZEOF_CELLDIM + 1];
    char	   light_template[CWORD_SIZEOF_CELLDIM + 1];

    CWORD_T_STR_LL *matches;
    unsigned int   match_num;

    CWORD_T_STR_LL *disallowed;
} CWORD_T_LIGHT_STRUCT;

typedef struct 
{
    char 	   name[CWORD_SIZEOF_LINE + 1];

    unsigned int   clue_sum;
    unsigned int   x_dimensions;
    unsigned int   y_dimensions;
    unsigned int   grid_type;
    unsigned int   symmetry;
    unsigned int   iteration_no;

    CWORD_T_STR_LL *failures;

    CWORD_T_LIGHT_STRUCT grid_details[CWORD_SIZEOF_CLUENUM];
} CWORD_T_GRID_STRUCT;

/*
 *  Lights make up grids and thus LIGHT_STRUCTs make up GRID_STRUCTs.
 */

/*
 *  PUBLIC function prototypes.
 */

PUBLIC BOOLEAN cword_grid_is_fully_seeded (
	CWORD_T_GRID_STRUCT	*grid_record);

PUBLIC int cword_grid_check_disallowed (
	int			best_word,
	char			*selected_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PUBLIC void cword_grid_output_grid (
	int			msg_id,
	int			best_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PUBLIC int cword_grid_update_light_temps (
	int			message_id,
	CWORD_T_GRID_STRUCT	*grid_record);

PUBLIC int cword_grid_add_to_disallowed ( 
	int			word_mask,
	char			*disallowed_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PUBLIC int cword_grid_del_from_disallowed (
	int			word_mask,
	char			*disallowed_word,
	CWORD_T_GRID_STRUCT	*grid_record);

PUBLIC int cword_grid_c_to_g_rec (
	CROSSWORD_RECORD	*crossword_record,
	CWORD_T_GRID_STRUCT	*grid_record);

PUBLIC int cword_grid_g_to_c_rec (
	CWORD_T_GRID_STRUCT	*grid_record,
	CROSSWORD_RECORD	*crossword_record);

PUBLIC int cword_grid_get_grid_contents (
	int			use_best_word,
	CWORD_T_GRID_STRUCT	*grid_record,
	CROSSWORD_GRID		grid_contents);

PUBLIC int cword_grid_get_grid_props (
	CWORD_T_GRID_STRUCT	*grid_record,
	CROSSWORD_GRID		grid_properties);

PUBLIC int cword_grid_fill_desc ( 
	CROSSWORD_RECORD	*crossword_record);

#endif
