#ifndef CWORD_MAIN_LOADED
#define CWORD_MAIN_LOADED 1

#include <stdlib.h>

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0  /* Success */
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1  /* Failure */
#endif

/*
 *  Macros for function prototypes/definitions.
 */

#define PUBLIC	    extern
#define PRIVATE	    static

/*
 *  Macros for GLOBAL/LOCAL variables.
 */

#define GLOBAL	    extern
#define LOCAL	    static

#undef TRUE
#undef FALSE

typedef enum
{
    FALSE = 0,
    TRUE
} BOOLEAN;

#if 0

#define BOOLEAN	    unsigned short

/* 
 *  Define TRUE and FALSE.
 */

#ifndef TRUE
#define TRUE	    1
#endif

#ifndef FALSE
#define FALSE	    0
#endif

#endif

/*
 *  Constant used to decide how to output messages to the OS.
 */

#if defined (VMS) || defined (__VMS)
#define NO_SCREEN	0x10000000
#else
#define NO_SCREEN	0x00000000
#endif

/*
 *  ?? Why here and not in fileio.c ??
 */

#define READ_FILE		1
#define WRITE_FILE		2

/*
 *  Naming convention for word files, rather than explicitly name each file.
 *  This is used in conjunction with the defaults file.
 *
 *  These formats should work for all OS's.
 */

#define WORDFILE_PREFIX		    "%s.%02d"

#define WORDFILE_NAME		    "words"

/*
 *  Default file suffixes used in the crossword application.
 */

#define CWORD_SUFFIX_XWORD_FILE	   ".xrd"
#define CWORD_SUFFIX_GRID_FILE	   ".grd"
#define CWORD_SUFFIX_DICT_FILE	   ".dic"
#define CWORD_SUFFIX_WORD_FILE	   ".%d"
#define CWORD_SUFFIX_PRNDAT_FILE   ".dat"
#define CWORD_SUFFIX_PRNPS_FILE    ".ps"
#define CWORD_SUFFIX_TRACE_FILE	   ".log"
#define CWORD_SUFFIX_SEED_FILE     ".sed"
#define CWORD_SUFFIX_TEMP_FILE     ".tmp"

/*
 *  Defined lengths of strings to be used in the application.
 */

#define CWORD_SIZEOF_WEB_OUTPUT	10000
#define CWORD_SIZEOF_MESSAGE	255
#define CWORD_SIZEOF_LINE	80
#define CWORD_SIZEOF_FILESPEC	255
#define CWORD_SIZEOF_TIME	30
#define CWORD_SIZEOF_ALPHABET	26
#define CWORD_SIZEOF_SOUNDEX	6

#define CWORD_SIZEOF_PAGE_X     80
#define CWORD_SIZEOF_BIGPAGE_X  132
#define CWORD_SIZEOF_PAGE_Y     68

/*
 *  The number of lines per screen before executing a ...pause ().
 */

#define CWORD_SIZEOF_LINESPERSCREEN (22)

/* 
 *  Default number of words read before outputting the character '.' to the
 *  screen as an aid to the user how fast the words are being read from the
 *  word files.
 *
 *  These maxs/mins can be overridden by the Cword defaults file!
 */

#define CWORD_SIZEOF_MIN_WORDS_PER_DOT	500
#define CWORD_SIZEOF_MAX_WORDS_PER_DOT	20000

/*
 *  Maximum (in both the X and Y direction) dimensions of grid that can be
 *  defined.
 *
 *  Basically limits the user to creating a grid of 40*40 or less.  I have
 *  never seen a crossword any bigger than this, so it should suffice.
 */

#define CWORD_SIZEOF_CELLDIM	40  

/*
 *  Maximum number of clues allowed in a crossword definition.
 */

#define CWORD_SIZEOF_CLUENUM	255

/*
 *  Maximum size of a clue in the crossword definition.
 */

#define CWORD_SIZEOF_CLUE	132

/*
 *  Maximum number of choices that can be returned at any one time - this
 *  constant is used to declare space that is then used for storage.  This
 *  storage can only be changed at compile time.
 */

#define CWORD_SIZEOF_MAX_ARRAY_SIZE 1000

/*
 *  Reset the grid if more than X iteration have been executed of the grid
 *  filler!
 */

#define CWORD_SIZEOF_RESET_FILLER   1000

/* 
 *  Maximum possible size of light that can be inserted into a crossword grid.
 */

#define CWORD_SIZEOF_WORD	CWORD_SIZEOF_CELLDIM

/*
 *  Constant used in conjunction with match and rdb modules.
 */

#define CWORD_SIZEOF_CHECK_EVERY    250

/*
 *  Crossword grid_types and symmetry constants.  These are importatn constants
 *  as there are read and written to and from the grid and crossword files.  If
 *  any new grid or symmetry types are added they have to be added to the
 *  bottom of these various lists.
 */

#define NOTDEFINED_GRID		0
#define BLOCK_GRID		1
#define BARRED_GRID		2
#define AMERICAN_GRID           3
#define NOVELTY_GRID            4
#define INVALID_GRID	        5

#define NOTDEFINED_SYMMETRY	0
#define	DEG_90_SYMMETRY		1
#define	DEG_180_SYMMETRY	2
#define	AXIAL_LR_SYMMETRY	3  /* Axial from top Left to bottom Right */
#define	AXIAL_RL_SYMMETRY	4  /* Axial from top Right to bottom Left */
#define NO_SYMMETRY		5
#define INVALID_SYMMETRY	6

#define BOTH			2
#define ACROSS			1
/*
 *  ?? Dangerous having DOWN as 0 ??
 */
#define DOWN			0

/*
 *  Constant used for assignment purposes.
 */

#define NOT_DEFINED		-1

/*	 
 *  These #defines are used for cells who are either NOTHING or TAKEN .
 */	 

#define NOT_TAKEN	 	0 /* or 0X00 */

/*
 *  Barred identifiers used to identify type of grid component.  Remember
 *  they're HEX!
 */

#define LEFT			0X0001
#define RIGHT			0X0002
#define TOP			0X0004
#define	BOTTOM			0X0008
#define TOP_BOTTOM		TOP | BOTTOM
#define LEFT_RIGHT		LEFT | RIGHT

/*
 *  Used in grid definition to signify if a particular cell is not taken, is an
 *  unch, or is a checked cell.
 */

#define	UNCH			0X0020 
#define CHECKED			0X0040 

/*
 *  String link list structure to contain single string elements.
 */

/*
 *  Link list structure containing double string elements.
 *
 *  ?? All typedef names should be less than 24 letters in length ??
 */

/*
 *  Use "unsigned char" instead of short or long to save space as short on
 *  VAX/AXP is two bytes in lengthh, whatever about MS-DOS ;-).
 */

typedef unsigned char CROSSWORD_GRID[CWORD_SIZEOF_CELLDIM][CWORD_SIZEOF_CELLDIM];

/* ?? CWORD_T_CROSSGRID_STRUCT; ?? */

typedef long CWORD_T_LENFREQ_STRUCT[CWORD_SIZEOF_CELLDIM];

/*
 *  ?? Sizes of data types differ from VMS to U*X to DOS.  I should make
 *  provision for this in here possibly ??
 */

typedef struct
{
    unsigned int clue_number;
    unsigned int clue_direction;
    unsigned int x_coordinate;
    unsigned int y_coordinate;
    unsigned int answer_length;

    /*
     *  ?? Dynamically allocate these strings as required ??
     */

    char clue_description[CWORD_SIZEOF_CLUE + 1];
    char clue_answer[CWORD_SIZEOF_CELLDIM + 1];
} CLUE_DETAILS;

/* ?? CWORD_T_CLUE_STRUCT; ?? */

typedef struct 
{
    char name[CWORD_SIZEOF_LINE + 1];
    char remarks[CWORD_SIZEOF_LINE + 1];
    char file_name[CWORD_SIZEOF_LINE + 1]; /* Default file prefix */

    unsigned int    x_dimensions;
    unsigned int    y_dimensions;
    unsigned int    clue_sum;

    unsigned int    grid_type;
    unsigned int    symmetry;

    BOOLEAN	    is_completely_specified;

    /*
     *  ?? Should dynamically allocate this ??
     */

    CLUE_DETAILS    clue_details[CWORD_SIZEOF_CLUENUM];
} CROSSWORD_RECORD;

/* ?? CWORD_T_CROSSWORD_STRUCT; ?? */

/*
 *  Clues make up crosswords thus CLUE_STRUCTs make up CROSSWORD_STRUCTs
 */



/*
 *  New linked list type.
 */

typedef struct cword_t_str_ll
{
    struct cword_t_str_ll *previous;

    char *str;
    float str_val;

    struct cword_t_str_ll *next;
} CWORD_T_STR_LL;

#endif
