/*
 *++
 *  FACILITY:
 *
 *      match.c
 *
 *  ABSTRACT:
 *
 *	Module that contains a set of functions that reads and processes the
 *	word files defined in the defaults files.
 *
 *  AUTHORS:
 *
 *      James A. Lundon. 
 *
 *  CREATION DATE:     
 *
 *	23 October 1992
 *--
 */

/*
 *  Include files
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <time.h>
#include <string.h>

#if defined(__unix) || defined(__unix__)

#include <sys/file.h>
#include <malloc.h>
#include <strings.h>
#include <unistd.h>

#endif

#if defined(__MSDOS__) || defined(MSDOS)

#endif

#ifdef MATCH
#define MESSAGES_DEFINED 1
#endif

#include "main.h" 
#include "match.h" 
#include "utils.h" 
#include "memman.h" 
#include "messages.h" 
#include "defaults.h"
#include "word.h"
#include "charui.h" 
#include "os.h" 
#include "strll.h"

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#ifndef MATCH

/*
 *  Forward function prototypes - PRIVATE only
 */

PRIVATE int cword_match__possible_fits (
	char			*match_str,
	CWORD_T_STR_LL		**str_ll);

PRIVATE int cword_match__get_intersect_list (
	char			*string1,
	char			*string2,
	CWORD_T_STR_LL		**intersect_ll);

PRIVATE int cword_match__fprintf_word (
	char			*word_str,
	FILE			*word_fp);



PRIVATE int cword_match__possible_fits (
	char			*match_str,
	CWORD_T_STR_LL		**str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function which find the words matching the input match_str and returns
 *	the matches in the link list str_ll for further processing.
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.max_matches
 *
 *  FORMAL PARAMETERS:
 *
 *	*match_str		(r) search string to be matched
 *	**str_ll		(w) link list of words which match
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Got list of suitable words
 *	CWORD_S_FAILURE		Other error
 *--
 */
{

    char word_str[CWORD_SIZEOF_WORD + 1];

    register long word_count = 0;

    int count, 
	status,
	str_len = strlen (match_str);

    BOOLEAN is_first_letter = FALSE,
	    is_word_ok = TRUE;

    *str_ll = (CWORD_T_STR_LL *) 0;

    cword_messages_output_message (
	CWORD_CLOCKBEFORE,
	TRACE2,
	clock ());

    /*
     *  Read word file for appropriate matching template words!
     */

    status = cword_match_read_word_file (
		FALSE, str_len,
		word_str);

    if (CWORD_S_NORMAL != status)
    {
	return (status);
    }

    /*
     *	?? If the first character of the string is specified then there should
     *	be a way of short circuiting the searching of the word files for valid
     *	choices ??
     *
     *  ?? This hold true of all the other word file searching functions ??
     */

#if 0
    if (match_str[0] != (char) SPACE_CHAR)
	is_first_letter = TRUE;
#endif

    cword_messages_output_message (
	CWORD_MATCHNOW,
	SCREEN0);

    do 
    {
	count = 0;
	is_word_ok = TRUE;

	while (count < str_len)
	{
	    if (match_str[count] == SPACE_CHAR)
	    {
		count++;
	    }
	    else
	    {
		if (match_str[count] != word_str[count])
		{
		    is_word_ok = FALSE;
		    break;
		}
		else
		{
		    count++;
		}
	    }
	}

	if (is_word_ok == TRUE)
	{
	    cword_strll_add_str (
		SORTED | UNIQUE,
		word_str,
		str_ll);

	    if (*str_ll == (CWORD_T_STR_LL *) 0)
	    {
		CWORD_MATCH_WAIT_AND_CLOCK;

		return (CWORD_S_FAILURE);
	    }

	    word_count++;

	    CWORD_MATCH_WORD_COUNT_CHECK(word_count);
	}

#if 0
	if (is_first_letter == TRUE)
	{
	    if ((word_count % CWORD_SIZEOF_CHECK_EVERY) == 0)
	    {
		if ((int) match_str[0] < (int) word_str[0])
		{
		    CWORD_MATCH_WAIT_AND_CLOCK;

		    return (CWORD_S_NORMAL);
		}
	    }
	}
#endif
    }
    while (CWORD_S_NORMAL == cword_match_read_word_file (FALSE, 0, word_str));

    CWORD_MATCH_WAIT_AND_CLOCK;

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_match_get_min_size (
	char			*check_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    int letters = 0,
	count = 0;

    while (check_str[count] != (char) 0)
    {
	if ( (isalpha ((int) check_str[count])) ||
	      (check_str[count] == (char) QUESTION_CHAR) ||
	       (check_str[count] == (char) SPACE_CHAR) )
	    letters++;

	count++;
    }

    DEBUG_TRACE ("Minimum search size of input string should be %d\n", letters);

    return (letters);

}



PUBLIC int cword_match_get_max_size (
	char			*check_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    int letters;

    /*
     *  ?? Is there any better way of doing this I wonder ??
     */

    letters = strlen (check_str);

    letters = CWORD_SIZEOF_WORD;

    DEBUG_TRACE ("Maximum search size of input string should be %d\n", letters);

    return (letters);

}



PUBLIC int cword_match_spell_check (
	char			*match_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      This function checks the spelling of the inputted word.
 *
 *	The success/failure of this function is displayed outside of the
 *	function itself!
 *
 *  FORMAL PARAMETERS:
 *
 *      *word_to_spell		(r) Check spelling of word from list
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Spelling as per word file(s) is found.
 *	CWORD_S_FAILURE		Spelling as per word file(s) is not found.
 *	CWORD_WORDREADERR	Error with word file
 *--
 */

{

    char word_str[CWORD_SIZEOF_WORD + 1];

    int	count = 0,
	status;

    cword_messages_output_message (
	CWORD_CLOCKBEFORE,
	TRACE2,
	clock ());

    while (match_str[count] != (char) 0)
    {
	match_str[count] = (char) tolower (match_str[count]);
	count++;
    }

    status = cword_match_read_word_file (
		FALSE, strlen (match_str),
		word_str);

    if (CWORD_S_NORMAL != status)
    {
	return (CWORD_WORDREADERR);
    }

    cword_messages_output_message (
	CWORD_MATCHNOW,
	SCREEN0);

    /*
     *	strcmp starts out with its return value being greater than 0, then 0,
     *	and then eventually being less than 0.
     */

    do 
    {
	count++;

	if (strcmp (match_str, word_str) != 0)
	{
#if 0
	    /*
	     *  Check we haven't gone past the entry (if any) in the word file
	     */

	    if ((count % CWORD_SIZEOF_CHECK_EVERY) == 0)
	    {
		if (strcmp (match_str, word_str) < 0)
		{
		    CWORD_MATCH_WAIT_AND_CLOCK;

		    /*
		     *	?? What about the alternative word files to be searched ??
		     */

		    return (CWORD_S_FAILURE);
		}
	    }   
#endif
	}
	else
	{
	    CWORD_MATCH_WAIT_AND_CLOCK;

	    return (CWORD_S_NORMAL);
	}
    }
    while (CWORD_S_NORMAL == cword_match_read_word_file (FALSE, 0, word_str));

    CWORD_MATCH_WAIT_AND_CLOCK;

    return (CWORD_S_FAILURE);

}



PUBLIC int cword_match_template_list (
	char			*search_str,
	CWORD_T_STR_LL		**str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function returning all the matches, with the search_str found in the
 *	word files, in the link list str_ll.
 *
 *  FORMAL PARAMETERS:
 *
 *	*search_str		(r) Search string
 *	**str_ll		(w) Link list of words to be returned
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Found list ot words
 *	CWORD_WORDTOOBIG	Too many letters specified in word
 *	CWORD_S_FAILURE		Error getting list of words matching input parameters
 *--
 */

{

    int status,
	count,
	str_len = strlen (search_str);

    *str_ll = (CWORD_T_STR_LL *) 0;

    if (str_len > CWORD_SIZEOF_WORD)
    {
	cword_messages_output_message (
	    CWORD_WORDTOOBIG,
	    (TRACE2 | SCREEN0));

	return (CWORD_S_FAILURE);
    }

    cword_word_preproc_match_str (search_str);

    cword_messages_output_message (
	CWORD_TEMPLATESEARCH,
	(TRACE2 | SCREEN0),
	search_str);

    /*
     *  Call the match function that will get the matches from the word file.
     */

    status = cword_match__possible_fits (
		search_str,
		str_ll);

    if (status != CWORD_S_NORMAL)
    {
	cword_strll_free_str_ll (str_ll);

	return (CWORD_S_FAILURE);
    }

    if (*str_ll == (CWORD_T_STR_LL *) 0)
    {
	cword_messages_output_message (
		CWORD_NOTEMPLATEMATCH,
		(TRACE2 | SCREEN0));

	cword_strll_free_str_ll (str_ll);

	return (CWORD_S_NORMAL);
    }
    else
    {
	/*
	 *  Sort the list before continuing especially if more than one word
	 *  file was used to construct the list.
	 */

	count = (int) cword_strll_number_str_ll (*str_ll);

    	cword_messages_output_message (
	    CWORD_CHOICESFOUND,
	    TRACE2,
	    count,
    	    count == 1 ? ITEMS_STR : ITEM_STR);
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? There are possibly some enhancements to be made to this function that
 *  will save CPU cycles but as to whether they are worth doing is another
 *  thing now that I'm using a multi-file system of getting words to match
 *  against.  The most obvious enhancement concerns first letter matching check
 *  whereby all words (assuming the word files are sorted in ascending
 *  alphabetic order) after the first letter are skipped when a comparision is
 *  completed ??
 */

PUBLIC int cword_match_generic_read_func (
	int			(*compare_function_ptr) (char *, char *),
	int			msg_id,
	char			*check_str,
	CWORD_T_STR_LL		**str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is used to read through the defined series of word
 *	file(s) with each word being checked against an inputted check_str with
 *	the function *compare_function_ptr being called to do the "matching".
 *	If CWORD_S_NORMAL is returned from the function defined then the word
 *	string is added to the str_ll which is passed back out through the
 *	parameter list.  Understand?
 *
 *  FORMAL PARAMETERS:
 *
 *	*compare_function_ptr	(r) Pointer to function to be called to do "compare"
 *	msg_id			(r) Message ID 
 *	*check_str		(r) String inputted to be check using *compare_function_ptr
 *	**str_ll		(w) Link list of matches found using *compare_function_ptr 
 *                                  and *check_str
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.max_matches
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *	CWORD_S_FAILURE		Some other error
 *--
 */

{

    char word_str[CWORD_SIZEOF_WORD + 1];

    register long word_count = 0;

    int count = 0,
	status;

    *str_ll = (CWORD_T_STR_LL *) 0;

    while (check_str[count] != (char) 0)
    {
	/*
	 *  Leave the characters 0-9, '%', '*', and '.' alone but lower all
	 *  other alphabetic characters.
	 */

	if ( (isalpha ((int) check_str[count]) != 0) ||
	      (check_str[count] == (char) QUESTION_CHAR) ||
	       (check_str[count] == (char) WILDCARD_CHAR) ||
	        (check_str[count] == (char) DOT_CHAR) )
	{
	    check_str[count] = (char) tolower (check_str[count]);
	}

	count++;
    }

    /*
     *	How to determine what the max and min sizes are for the check_str.
     */

    status = cword_match_read_word_files (
		cword_match_get_min_size (check_str),
		cword_match_get_max_size (check_str),
		word_str);

    if (CWORD_S_NORMAL != status)
    {
	return (CWORD_S_FAILURE);
    }

    cword_messages_output_message (
	CWORD_CLOCKBEFORE,
	TRACE2,
	clock ());

    cword_messages_output_message (
	msg_id,
	SCREEN0);

    cword_messages_output_message (
	CWORD_S_GENERICSTR,
	TRACE0,
	check_str);

    do 
    {
	/*
	 *  Function to be called is entered through the parameter list as a
	 *  function pointer.  This function is called with two parameters: the
	 *  string to be checked and the word string that is to be compared
	 *  with the check string.
	 */

	status = compare_function_ptr (
		    check_str,
		    word_str);

	if (CWORD_S_NORMAL == status)
	{
	    cword_strll_add_str (
		SORTED | UNIQUE,
		word_str,
		str_ll);

	    if (*str_ll == (CWORD_T_STR_LL *) 0)
	    {
		CWORD_MATCH_WAIT_AND_CLOCK;

		return (CWORD_S_FAILURE);
	    }

	    word_count++;

	    CWORD_MATCH_WORD_COUNT_CHECK(word_count);
	}
    }
    while (CWORD_S_NORMAL == cword_match_read_word_files (0, 0, word_str));

    cword_match_read_word_files (RESET, RESET, 0);

    CWORD_MATCH_WAIT_AND_CLOCK;

    if (*str_ll == (CWORD_T_STR_LL *) 0)
    {
	cword_messages_output_message (
	    CWORD_NOMATCHESFOUND,
	    SCREEN0);

	return (CWORD_S_FAILURE);
    }

    count = (int) cword_strll_number_str_ll (*str_ll);

    cword_messages_output_message (
    	CWORD_CHOICESFOUND,
	TRACE2,
	count,
    	count == 1 ? ITEMS_STR : ITEM_STR);

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Change function name to something which is distinct from the function
 *  cword_match_read_word_file ??
 */

PUBLIC int cword_match_read_word_files (
	int			min_size,
	int			max_size,
	char			*word_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Call cword_match_read_word_file for all word lengths between min_size
 *	and max_size inclusive.
 *
 *  FORMAL PARAMETERS:
 *
 *	min_size		(r) Minimum size of word file to check for words in (usually 0)
 *	max_size		(r) Maximum size of word file to check for words in (usually 0)
 *	*word_str		(w) Return string found in word files
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Successfully found string in word files.
 *	CWORD_S_FAILURE		Error finding string or end of word files found.
 *--
 */

{

    static int word_file_min = -1,
	       word_file_max = -1,
	       word_size_current = -1;

    static BOOLEAN is_any_word_found = FALSE;

    int status;

    BOOLEAN is_word_file_read = FALSE;

    /*
     *	?? No need for this RESETting as the other logic will cover the
     *	re-initialisation of the static variables contained in this function ??
     *
     *	?? This logic needs to be overhauled and optimised as it's far too
     *	complicated - even for the author himself ??
     */

    if ( (min_size == RESET) || (max_size == RESET) )
    {
	DEBUG_TRACE ("Resetting static variables in this function\n");

	word_file_min = -1;
	word_file_max = -1;
	word_size_current = -1;
	is_any_word_found = FALSE;

	cword_match_get_word_file_name (RESET, 0);

	return (CWORD_S_NORMAL);
    }

    if (min_size > max_size)
    {
	cword_messages_output_message (
	    CWORD_S_MINBIGGERMAX,
	    (SCREEN0 | TRACE0),
	    min_size,
	    max_size);

	word_str[0] = (char) 0;

    	return (CWORD_S_FAILURE);
    }

    if (word_file_max == -1)
    {
	DEBUG_TRACE (
	    "1. read_word_files - min is %u - min is %u\n",
	    min_size,
	    max_size);

	/*
	 *  First time around the block and need to initialise some static
	 *  variables for use while reading the word files!
	 */

	word_file_min = min_size;
	word_file_max = max_size;
        word_size_current = min_size;
	is_any_word_found = FALSE;

	cword_match_get_word_file_name (RESET, 0);

	DEBUG_TRACE ("1. First time reading word_file\n");

	status = cword_match_read_word_file (
		    FALSE, word_size_current,
		    word_str);

	is_word_file_read = TRUE;
    }
    else
    {
	/*
	 *  ?? Is this block of functionality called at all ??
	 */

	if ( (min_size != 0) || (max_size != 0) )
	{
	    DEBUG_TRACE (
		"2. read_word_files - min is %u - min is %u\n",
		min_size,
		max_size);

	    /*
	     *  Reset static variables!
	     */

	    word_file_min = min_size;
	    word_file_max = max_size;
	    word_size_current = min_size;
	    is_any_word_found = FALSE;

	    cword_match_get_word_file_name (RESET, 0);

	    DEBUG_TRACE ("2. First time reading word_file\n");

	    status = cword_match_read_word_file (
			FALSE, word_size_current,
			word_str);

	    is_word_file_read = TRUE;
	}
    }

    /*
     *	Read the word files of default length as usual, if it hasn't already
     *	been read above :-).
     */

    if (is_word_file_read == FALSE)
    {
	status = cword_match_read_word_file (
		    FALSE, 0,
		    word_str);
    }

    if (status != CWORD_S_NORMAL)
    {
	/*
	 *  Error with current word files or most likely the word files we are
	 *  currently reading are at EOF.
	 */

	if (word_size_current < word_file_max)
	{
	    word_size_current++;

	    DEBUG_TRACE ("word_size_current is %d\n", word_size_current);
	}
	else
	{
	    /*
	     *  Cannot read anymore from the word files(s) 
	     */

	    DEBUG_TRACE ("1. Resetting word_file_name and exitting\n");

	    cword_match_get_word_file_name (RESET, 0);

	    word_str[0] = (char) 0;

	    return (CWORD_S_FAILURE);
	}

	while (CWORD_S_NORMAL != cword_match_read_word_file (
				    FALSE, word_size_current,
				    word_str))
	{
	    if (word_size_current < word_file_max)
	    {
		word_size_current++;

		DEBUG_TRACE ("word_size_current is %d\n", word_size_current);
	    }
	    else
	    {
		/*
		 *  Cannot read anymore from the word files(s) 
		 */

		DEBUG_TRACE ("2. Resetting word_file_name and exitting\n");

		/*
		 *  Output whether any words were read at all during the
		 *  execution of this function.
		 */

		if (is_any_word_found == FALSE)
		{
		    cword_messages_output_message (
			CWORD_S_NOWORDFILERANGE,
			(SCREEN0 | TRACE0),
			word_file_min,
			word_file_max);
		}

		cword_match_get_word_file_name (RESET, 0);

		word_str[0] = (char) 0;

		return (CWORD_S_FAILURE);
	    }
	}

	DEBUG_TRACE ("1. Returning %s\n", word_str);

	is_any_word_found = TRUE;

	return (CWORD_S_NORMAL);
    }
    else
    {
	is_any_word_found = TRUE;

	return (CWORD_S_NORMAL);
    }

}



PUBLIC int cword_match_read_word_file (
	BOOLEAN			word_file_msg,
	int			word_length,
	char			*word_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Reads the selected word file one word at a time.
 *
 *  FORMAL PARAMETERS:
 *
 *	word_length		(r) Length of word file to search (!= 0 change current word file)
 *	*word_str		(w) Word string recovered from word file
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.words_per_dot
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Word obtained successfully from file
 *	CWORD_S_FAILURE		Unsuccessfully obtained word -> EOF
 *	CWORD_WORDREADERR	Error using word file(s)
 *--
 */

{

    char temp_spec[CWORD_SIZEOF_FILESPEC + 1];

    BOOLEAN is_word_file_found;

    static FILE	*word_fp = (FILE *) 0;

    /*
     *	The int data type is not enough space on a PC to store the possible
     *	total size of the word files.
     */

    static long	num_words = 0;

    /*
     *  Keeping an eye on the current word file length being processed.
     */

    static int file_size = 0;

    errno = 0;

    if (word_length != 0)
    {
	file_size = word_length;

	cword_messages_output_message (
		    CWORD_WORDFILEOPT,
		    TRACE3,
		    word_length);

    	/*
	 *  This if() statement is called when a word file is opened for
	 *  reading, otherwise this function will read from a word file already
	 *  opened.
    	 */

	/*
	 *  Re-initialise the file pointer and words counter.
	 */

	if (word_fp != (FILE *) 0)
	{
	    DEBUG_TRACE ("1. Closing word file\n");
	    fclose (word_fp);
	}
	else
	{
	    DEBUG_TRACE ("1. Closing word file not needed\n");
	}

	num_words = 0;

	/*
	 *  Call cword_match_get_word_file_name () to reset internal static
	 *  variables in this function before any word files are obtained.
	 */

	cword_match_get_word_file_name (
	    RESET,
	    temp_spec);

	/*
	 *  Call to get_word_file_name () should be successful whether
	 *  there are 0 or more word file sets available for use!
	 */

	is_word_file_found = FALSE;

	while (TRUE == cword_match_get_word_file_name (
			word_length,
			temp_spec))
	{
	    DEBUG_TRACE (
		"1. Opening word file: %s\n",
		temp_spec);

	    word_fp = fopen (temp_spec, "r");

	    if ((FILE *) 0 == word_fp)
	    {
		/*
		 *  ?? Do nothing ??
		 */
	    }
	    else
	    {
		is_word_file_found = TRUE;
		break;
	    }
	}

	if (is_word_file_found == FALSE)
	{
	    /*
	     *	Valid words file is not found.  Have to exit without getting
	     *	any words :-(
	     */

	    if (word_file_msg == TRUE)
	    {
		if (temp_spec[0] == (char) 0)
		{
		    cword_messages_output_message (
			    CWORD_S_NOWORDFILEAVAIL,
			    (TRACE2 | SCREEN0));
		}
		else
		{
		    cword_messages_output_message (
			    CWORD_NOWORDFILE,
			    (TRACE2 | SCREEN0),
			    temp_spec);
		}
	    }

	    cword_match_get_word_file_name (RESET, 0);
	    
	    return (CWORD_WORDREADERR);
	}
    }

    /*
     *	My definition of fgets is: Reads a line from the specified file pointer
     *	upto CWORD_SIZEOF_WORD + 1 *or*  upto and including the newline
     *	character whichever comes first.
     */

    /*
     *	?? fgets doesn't work for word files length CWORD_SIZEOF_WORD
     *	with the input size defined to be CWORD_SIZEOF_WORD + 1 ??
     *
     *	?? changing it to CWORD_SIZEOF_WORD + 2 does the trick, I think ??
     */

    if (fgets (
	    word_str,
	    CWORD_SIZEOF_WORD + 2,
	    word_fp) == (char *) 0) 
    {
	if (feof (word_fp) == 0)
	{
	    /*
	     *  End of file not reached but fgets reports problem
	     */

	    char message_str[CWORD_SIZEOF_MESSAGE + 1];

	    cword_messages_get_message (
		CWORD_S_WORDFILEPERROR,
		message_str);

	    perror (message_str);

	    cword_messages_output_message (
		    CWORD_WAITINGENDED,
		    (TRACE2 | SCREEN0));

	    cword_messages_output_message (
		    CWORD_WORDREADERR,
		    (TRACE2 | SCREEN0));

	    cword_match_get_word_file_name (
		RESET, 0);

	    return (CWORD_WORDREADERR);
	}

	/*
	 *  The file we are reading from is at EOF.  We have to get the next
	 *  word file, if any are available, and continue reading.
	 */

	/*
	 *  ?? Possibly not good enough to cover the general case.  It could be
	 *  a while () {} construct instead ??
	 */

	if (TRUE == cword_match_get_word_file_name (
			file_size,
			temp_spec))
	{
	    /*
	     *  Re-initialise the file pointer.
	     */

	    if (word_fp != (FILE *) 0)
	    {
		DEBUG_TRACE ("2. Closing word file\n");

		fclose (word_fp);
	    }
	    else
	    {
		DEBUG_TRACE ("2. Closing word file not needed\n");
	    }

	    DEBUG_TRACE (
		"2. Opening word file: %s\n",
		temp_spec);

	    word_fp = fopen (temp_spec, "r");

	    if ((FILE *) 0 == word_fp)
	    {
		if (word_file_msg == TRUE)
		{
		    char message_str[CWORD_SIZEOF_MESSAGE + 1];

		    cword_messages_get_message (
			CWORD_S_WORDFILEPERROR,
			message_str);

		    perror (message_str);

		    if (temp_spec[0] == (char) 0)
		    {
			cword_messages_output_message (
				CWORD_WAITINGENDED,
				(TRACE2 | SCREEN0));

			cword_messages_output_message (
				CWORD_S_NOWORDFILEAVAIL,
				(TRACE2 | SCREEN0));
		    }
		    else
		    {
			cword_messages_output_message (
				CWORD_WAITINGENDED,
				(TRACE2 | SCREEN0));

			cword_messages_output_message (
				CWORD_NOWORDFILE,
				(TRACE2 | SCREEN0),
				temp_spec);
		    }
		}

		cword_match_get_word_file_name (RESET, 0);

		return (CWORD_WORDREADERR);
	    }

	    /*
	     *	?? Not a particularly great implementation but it is probably
	     *	adequate ??
	     */

	    if (fgets (
		    word_str,
		    CWORD_SIZEOF_WORD + 1,
		    word_fp) == (char *) 0) 
	    {
		char message_str[CWORD_SIZEOF_MESSAGE + 1];

		cword_messages_get_message (
		    CWORD_S_WORDFILEPERROR,
		    message_str);

		perror (message_str);

		cword_messages_output_message (
			CWORD_WAITINGENDED,
			(TRACE2 | SCREEN0));

		cword_messages_output_message (
			CWORD_WORDREADERR,
			(TRACE2 | SCREEN0));

		cword_match_get_word_file_name (RESET, 0);

		return (CWORD_WORDREADERR);
	    }
	}
	else
	{
	    /*
	     *	The word files we are reading is at EOF so we return an error
	     *	status which tells the calling functions that we cannot
	     *	continue reading from the series of word files defined!
	     */

	    cword_match_get_word_file_name (RESET, 0);

	    return (CWORD_S_FAILURE);
	}
    }

    num_words++;

    if ((num_words % cword_g_defaults_record.words_per_dot) == 0)
    {
	cword_messages_output_message (
	    CWORD_WAITING,
	    SCREEN0);
    }

    word_str[strlen (word_str) - 1] = (char) 0;

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_match__get_intersect_list (
	char			*first_str,
	char			*second_str,
	CWORD_T_STR_LL          **intersect_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function called by cword_match_find_intersects.  It does most of the
 *	work concerning the obtaining of intersecting matches of the two
 *	inputted string - first_str and second_str.
 *
 *  FORMAL PARAMETERS:
 *
 *      *first_str		(r) string which intersects with second_str at the '?' 
 *	*second_str		(r) string which intersects with first_str at the '?'
 *	**intersect_ll		(w) Link list returning matches is sequence
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		List of matches found and returned through parameter list
 *	CWORD_S_FAILURE		Unsuccessful - memory error or no matches found.
 *--
 */

{

    CWORD_T_STR_LL *list1 = (CWORD_T_STR_LL *) 0,
	      *list2 = (CWORD_T_STR_LL *) 0,
	      *temp_list1 = (CWORD_T_STR_LL *) 0,
	      *temp_list2 = (CWORD_T_STR_LL *) 0;

    char cross1[27],
	 cross2[27],
	 crosscross[27],
	 original1[CWORD_SIZEOF_WORD + 1],
	 original2[CWORD_SIZEOF_WORD + 1];

    unsigned short loc1 = 100,
		   loc2 = 100,
		   count,
		   temp1,
		   temp2,
		   temp3,
		   include1,
		   include2,
		   numfound,
		   status;

    BOOLEAN is_found;

    *intersect_ll = (CWORD_T_STR_LL *) 0;

    memset (cross1, 0, sizeof (cross1));
    memset (cross2, 0, sizeof (cross2));
    memset (crosscross, 0, sizeof (crosscross));

    /*
     *	Getting the locations where the intersection takes place and stores
     *	them in loc1 and loc2.
     */

    for (count = 0; count < strlen (first_str); count++)
	if (first_str[count] == (char) QUESTION_CHAR)
	{
	    loc1 = count;
	    first_str[loc1] = (char) SPACE_CHAR;

	    break;
	}

    for (count = 0; count < strlen (second_str); count++)
	if (second_str[count] == (char) QUESTION_CHAR)
	{
	    loc2 = count;
	    second_str[loc2] = (char) SPACE_CHAR;

	    break;
	}

    if ( (loc1 == 100) || (loc2 == 100) )
    {
	printf ("# Error getting '?' character\n");

	return (CWORD_S_FAILURE);
    }

    strcpy (original1, first_str);
    strcpy (original2, second_str);

    /*
     *  Generate a link list of the matches for first_str
     */

    status = cword_match__possible_fits (
		first_str, 
		&list1);

    if (status != CWORD_S_NORMAL)
    {
	cword_strll_free_str_ll (&list1);

	return (CWORD_S_FAILURE);
    }

    /*
     *  Generate a link list of the matches for second_str
     */

    status = cword_match__possible_fits (
		second_str,
		&list2);

    if (status != CWORD_S_NORMAL)
    {
	cword_strll_free_str_ll (&list1);
	cword_strll_free_str_ll (&list2);

	return (CWORD_S_FAILURE);
    }

    /*
     *	Work now with the link lists found above.
     *
     *	Sort the outputs so that I can get a cross-reference array containing
     *	all letters in the loc1 of first_str and loc2 of second_str.
     */

    temp_list1 = list1;
    while (temp_list1 != 0)
    {
	temp1 = 0; include1 = TRUE;

	while (cross1[temp1] != (char) 0)
	{
	    if (temp_list1->str[loc1] == cross1[temp1])
	    {
		include1 = FALSE;
		break;
	    }

	    temp1++;
	}

	if (include1 == TRUE)
	{
	    temp1 = 0;
	    while (cross1[temp1] != (char) 0)
		temp1++;
	    cross1[temp1] = temp_list1->str[loc1];
	}

	temp_list1 = temp_list1->next;
    }

    temp_list2 = list2;
    while (temp_list2 != 0)
    {
	temp2 = 0; include2 = TRUE;

	while (cross2[temp2] != (char) 0)
	{
	    if (temp_list2->str[loc2] == cross2[temp2])
	    {
		include2 = FALSE;
		break;
	    }

	    temp2++;
	}

	if (include2 == TRUE)
	{
	    temp2 = 0;
	    while (cross2[temp2] != (char) 0)
		temp2++;
	    cross2[temp2] = temp_list2->str[loc2];
	}

	temp_list2 = temp_list2->next;
    }

    WRITE_TRACE (
	"# Cross reference array for \"%s\" contains the letters \"%s\"\n", 
	original1,
	cross1);

    WRITE_TRACE (
	"# Cross reference array for \"%s\" contains the letters \"%s\"\n", 
	original2,
	cross2);

    /*
     *  Cross reference array for identical letters.
     */

    temp1 = temp3 = 0;
    while (cross1[temp1] != (char) 0)
    {
	temp2 = 0;
	while (cross2[temp2] != (char) 0)
	{
	    if (cross2[temp2] == cross1[temp1])
	    {
		crosscross[temp3] = cross2[temp2];
		temp3++;
	    }
	    
	    temp2++;
	}

	temp1++;
    }

    WRITE_TRACE (
	"# Criscross reference array contains \"%s\"\n", 
	crosscross);

    if (crosscross[0] == (char) 0)
    {
	cword_messages_output_message (
	    CWORD_NOINTERSECTS,
	    (TRACE2 | SCREEN0),
	    original1, 
	    original2);

	cword_strll_free_str_ll (&list1);
	cword_strll_free_str_ll (&list2);

	return (CWORD_S_FAILURE);
    }

    /*
     *  Output numbr of combinations!
     */

    numfound = 0;
    temp_list1 = list1;
    while (temp_list1 != 0)
    {
	temp_list2 = list2;
	while (temp_list2 != 0)
	{
	    if (temp_list1->str[loc1] == temp_list2->str[loc2])
		numfound++;

	    if (numfound == cword_g_defaults_record.max_matches)
	    {
		cword_messages_output_message (
		    CWORD_MAXMATCHFOUND,
		    SCREEN0);

		break;
	    }

	    temp_list2 = temp_list2->next;
	}

	if (numfound == cword_g_defaults_record.max_matches)
	    break;

	temp_list1 = temp_list1->next;
    }

    cword_messages_output_message (
    	CWORD_S_NUMINTERSECTS,
	(TRACE2 | SCREEN0),
	numfound,
    	numfound == 1 ? PAIR_STR : PAIRS_STR);

    /*
     *	Appending intersecting matches to CWORD_T_STR_LL link list for later
     *	processing.
     */

    is_found = FALSE;
    temp_list1 = list1;
    numfound = 0;

    while (temp_list1 != 0)
    {
	temp_list2 = list2;

	while (temp_list2 != 0)
	{
	    if (temp_list1->str[loc1] == temp_list2->str[loc2])
	    {
		numfound++;

		is_found = TRUE;

		/*
		 *  Output is to be unsorted and also the elements can be non
		 *  unique.
		 */

		cword_strll_add_str (
		    UNSORTED | MULTIPLE,
		    temp_list1->str,
		    intersect_ll);

		if (*intersect_ll == (CWORD_T_STR_LL *) 0)
		{
		    cword_strll_free_str_ll (&list1);
		    cword_strll_free_str_ll (&list2);

		    cword_strll_free_str_ll (intersect_ll);

		    return (CWORD_S_FAILURE);
		}

		/*
		 *  Output is to be unsorted and also the elements can be non
		 *  unique.
		 */

		cword_strll_add_str (
		    UNSORTED | MULTIPLE,
		    temp_list2->str,
		    intersect_ll);

		if (*intersect_ll == (CWORD_T_STR_LL *) 0)
		{
		    cword_strll_free_str_ll (&list1);
		    cword_strll_free_str_ll (&list2);

		    cword_strll_free_str_ll (intersect_ll);

		    return (CWORD_S_FAILURE);
		}

		if (numfound == cword_g_defaults_record.max_matches)
		    break;
	    }

	    temp_list2 = temp_list2->next;
	}

	if (numfound == cword_g_defaults_record.max_matches)
	    break;

	temp_list1 = temp_list1->next;
    }

    if (is_found == FALSE)
	*intersect_ll = (CWORD_T_STR_LL *) 0;

    cword_strll_free_str_ll (&list1);
    cword_strll_free_str_ll (&list2);

    return (CWORD_S_NORMAL);

}   



PUBLIC int cword_match_find_intersects (
	char			*first_str,
	char			*second_str,
	CWORD_T_STR_LL		**intersect_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function using two parameters, e.g. input from user interface (or
 *	command line) e.g.
 *
 *	    first_str = "ja? s" 
 *	    second_str = "?ad  "
 * 
 *	Both parameters must have the QUESTION_CHAR character to signify where
 *	the intersection of the words takes place.  This function will then
 *	generate all word combinations which satisfy the constraints implicit
 *	on their properties.
 *
 *	Possible output could include:
 *
 *	jaMes Madly ...
 *
 *	With the intersecting letters in uppercase!
 *
 *  FORMAL PARAMETERS:
 *
 *      *first_str		(r) String which intersects with second_str at the ? 
 *	*second_str		(r) String which intersects with first_str at the ?
 *	**intersect_ll		(w) Linked list containing matches 
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Normal completion
 *	CWORD_S_FAILURE		Error getting intersects
 *--
 */

{

    int count,
    	status,
	questionmarks;

    *intersect_ll = (CWORD_T_STR_LL *) 0;

    /*
     *	Good amount of string parsing is needed to verify that the input
     *	strings were entered correctly!
     */

    /*
     *	?? Problems on Visual C++ checking strstr with 0 - see how it goes on
     *	other platforms ??
     */

    if ( (strstr (first_str, "?") == 0) || 
          (strstr (second_str, "?") == 0) )
    {
	cword_messages_output_message (
		CWORD_NUMQMARKSERR,
		(TRACE2 | SCREEN0));

    	return (CWORD_S_FAILURE);
    }

    if (strlen (first_str) > CWORD_SIZEOF_WORD)
    {
	cword_messages_output_message (
		CWORD_STRINGTOOLONG,
		(TRACE2 | SCREEN0),
		first_str);

    	return (CWORD_S_FAILURE);
    }

    if (strlen (second_str) > CWORD_SIZEOF_WORD)
    {
	cword_messages_output_message (
		CWORD_STRINGTOOLONG,
		(TRACE2 | SCREEN0),
		second_str);

    	return (CWORD_S_FAILURE);
    }

    questionmarks = 0;
    for (count = 0; count < strlen (first_str); count++)
	if (isalpha ((int) first_str[count]) != 0)
	    first_str[count] = (char) tolower (first_str[count]);
	else
	{
	    if (first_str[count] == (char) PERCENT_CHAR)
	    {
		first_str[count] = (char) SPACE_CHAR;
	    }
	    else if (first_str[count] == (char) DOT_CHAR) 
	    {
		first_str[count] = (char) SPACE_CHAR;
	    }
	    else if ( (first_str[count] == (char) QUESTION_CHAR) || 
                       (first_str[count] == (char) SPACE_CHAR) )
	    {
		if (first_str[count] == (char) QUESTION_CHAR)
		    questionmarks++;
	    }
	    else
	    {
		cword_messages_output_message (
		    CWORD_INVALIDCHAR,
		    SCREEN0,
		    first_str[count]);

		return (CWORD_S_FAILURE);
	    }
	}

    if (questionmarks > 1)
    {
	cword_messages_output_message (
	    CWORD_TOOMANYQMARKSERR,
	    (TRACE2 | SCREEN0),
	    first_str);

	return (CWORD_S_FAILURE);
    }

    questionmarks = 0;
    for (count = 0; count < strlen (second_str); count++)
	if (isalpha ((int) second_str[count]) != 0)
	{
	    second_str[count] = (char) tolower (second_str[count]);
	}
	else
	{
	    if (second_str[count] == (char) PERCENT_CHAR)
	    {
		second_str[count] = (char) SPACE_CHAR;
	    }
	    else if (second_str[count] == (char) DOT_CHAR)
	    {
		second_str[count] = (char) SPACE_CHAR;
	    }
	    else if ( (second_str[count] == (char) QUESTION_CHAR) || 
                       (second_str[count] == (char) SPACE_CHAR) ) 
	    {
		if (second_str[count] == (char) QUESTION_CHAR)
		    questionmarks++;
	    }
	    else
	    {
		cword_messages_output_message (
		    CWORD_INVALIDCHAR,
		    SCREEN0,
		    second_str[count]);

		return (CWORD_S_FAILURE);
	    }
	}

    if (questionmarks > 1)
    {
	cword_messages_output_message (
	    CWORD_TOOMANYQMARKSERR,
	    (TRACE2 | SCREEN0),
	    second_str);

	return (CWORD_S_FAILURE);
    }

    cword_messages_output_message (
	CWORD_INTERSECTWORDS,
	(TRACE2 | SCREEN0),
	first_str, 
	second_str);

    status = cword_match__get_intersect_list (
		first_str,
		second_str,
		intersect_ll);

    return (status);

}



PUBLIC BOOLEAN cword_match_get_word_file_name (
	int			wordfile_size,
	char			*word_spec)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function returning in word_spec the full directory specification of the
 *	word file required.
 *
 *	Remember the different formats for constructing a full directory/file
 *	specification on the OS's Cword will work on!
 *
 *  FORMAL PARAMETERS:
 *
 *      word_spec		(w) String in which current word file spec is returned
 *
 *  IMPLICIT INPUTS:
 *
 *	[@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Word file specification successfully found.
 *	FALSE			Word file specification unsuccessfully found.
 *--
 */

{

    char temp_str[CWORD_SIZEOF_FILESPEC + 1],
	 word_prefix[CWORD_SIZEOF_FILESPEC + 1];

    /*
     *	These static variables determine the current state of the word files
     *	that are to be fed to the calling function.
     */

    static CWORD_T_STR_LL *cur_file_ll = (CWORD_T_STR_LL *) 0;

    static signed short file_len = -1;

    static BOOLEAN is_last = FALSE;

    if (cword_g_defaults_record.word_file_prefix_list == (CWORD_T_STR_LL *) 0)
    {
	if (wordfile_size != RESET)
	    word_spec[0] = (char) 0;

	return (FALSE);
    }

    /*
     *	A complex piece of logic to determine which or whether we need to
     *	supply the calling function with the correct word file spec.
     */

    /*
     *  ?? Is reset really needed ??
     */

    if (wordfile_size == RESET)
    {
	/*
	 *  There must be a better way of doing this than what I've implemented
	 *  at this moment.
	 */

	DEBUG_TRACE ("***** Resetting all static variables *****\n");

	cur_file_ll = (CWORD_T_STR_LL *) 0;

	file_len = -1;

	is_last = FALSE;

	return (FALSE);
    }

    /*
     *	?? This whole system of determining which wordfile prefix we should
     *	return seems to be unnecessarily complex and could be rationalised to
     *	something *much* simpler.  I don't have the mental energy at the moment
     *	to break something that seems to be working fine though :-( ??
     */

    if (file_len == -1)
    {
	/*
	 *  In here the first time this function is ever called during the life
	 *  of the Cword process.
	 */

	DEBUG_TRACE ("file_len is -1 but changing to %d\n", wordfile_size);

	file_len = wordfile_size;
	cur_file_ll = cword_g_defaults_record.word_file_prefix_list;

	if (cur_file_ll != (CWORD_T_STR_LL *) 0)
	{
	    strcpy (word_prefix, cur_file_ll->str);

	    cur_file_ll = cur_file_ll->next;
	}
	else
	{
	    cur_file_ll = (CWORD_T_STR_LL *) 0;

	    word_spec[0] = (char) 0;

	    DEBUG_TRACE ("1. Returning no word file spec as none available\n");

	    return (FALSE);	    
	}
    }
    else
    {
	if (file_len == wordfile_size)
	{
	    /*
	     *	We have already used some of cur_file_ll and are now
	     *	getting another element of the list to return to the calling
	     *	function.
	     */

	    DEBUG_TRACE ("file_len is %d and == wordfile_size\n", file_len);

	    if (cur_file_ll != (CWORD_T_STR_LL *) 0)
	    {
		strcpy (word_prefix, cur_file_ll->str);

		cur_file_ll = cur_file_ll->next;
	    }
	    else
	    {
		cur_file_ll = (CWORD_T_STR_LL *) 0;

		word_spec[0] = (char) 0;

		DEBUG_TRACE ("2. Returning no word file spec as none available\n");

		return (FALSE);
	    }
	}
	else
	{
	    /*
	     *	First time in the "loop" for values of file_len !=
	     *	wordfile_size.  Not quite sure of the purpose or usefulness of
	     *	this functionality yet?
	     */

	    DEBUG_TRACE (
		    "file_len is %d and != wordfile_size %d\n", 
		    file_len, 
		    wordfile_size);

	    if (cur_file_ll != (CWORD_T_STR_LL *) 0)
	    {
		strcpy (word_prefix, cur_file_ll->str);

		cur_file_ll = cur_file_ll->next;

		if (cur_file_ll == (CWORD_T_STR_LL *) 0)
		{
		    DEBUG_TRACE ("is_last is TRUE\n");

		    is_last = TRUE;
		}
	    }
	    else
	    {
		if (is_last == TRUE)
		{
		    DEBUG_TRACE ("is_last is checked TRUE\n");

		    is_last = FALSE;

		    cur_file_ll = (CWORD_T_STR_LL *) 0;

		    word_spec[0] = (char) 0;

		    DEBUG_TRACE ("3. Returning no word file spec as none available\n");

		    return (FALSE);
		}
		else
		{
		    DEBUG_TRACE ("is_last is checked FALSE\n");

		    file_len = wordfile_size;
		    cur_file_ll = cword_g_defaults_record.word_file_prefix_list;

		    strcpy (word_prefix, cur_file_ll->str);

		    cur_file_ll = cur_file_ll->next;
		}
	    }
	}
    }

    DEBUG_TRACE (
	"Returning word_prefix %s\n",
	word_prefix);

    /*
     *  Create valid word file specification!
     */

    sprintf (
	word_spec,
	WORDFILE_PREFIX,	/* %s.%02d */
	word_prefix,
	wordfile_size);

    return (TRUE);

}



PUBLIC int cword_match_verify_file_prefix (
	char			*file_prefix_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Files found for inputted prefix.
 *	FALSE			No files found for inputted prefix.
 *--
 */

{

    CWORD_T_STR_LL *temp_ll = (CWORD_T_STR_LL *) 0;

    char tempfile_str[CWORD_SIZEOF_MESSAGE + 1]; 

    sprintf (
	tempfile_str,
	"%s.*",
	file_prefix_str);

    CWORD_OS_LIST_FILES (
	(char *) 0,
	tempfile_str,
	&temp_ll);

    /*
     *	?? Have to check the list: temp_ll for validity as it's possible that
     *	there are files other than word files in the list: input_prefix.*
     *	returned ??
     */

    if (temp_ll == (CWORD_T_STR_LL *) 0)
    {
	/*
	 *  No files.
	 */

	return (FALSE);
    }
    else
    {
	/*
	 *  Files found.
	 */

	cword_strll_free_str_ll (&temp_ll);
	
	return (TRUE);
    }

}



/*
 *  ?? Corresponding "delete word" from word file function should also be
 *  provided ??
 */

/*
 *  ?? Output number of word in altered word file upon completion ??
 */

PUBLIC int cword_match_add_word_to_file (
	char			*word_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to add a user defined word to a word file.
 *
 *	?? Support for selection of word file if multiple word files present ??
 *
 *  FORMAL PARAMETERS:
 *
 *      *word_str		(r) Word to be added to related word file of length *word_str.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully added word to word file.
 *	CWORD_S_FAILURE		Error adding word to word file.
 *--
 */

{

    char word_file_str[CWORD_SIZEOF_FILESPEC + 1];

    int status;

    if (strlen (word_str) < 1)
    {
	cword_messages_output_message (
	    CWORD_STRNULLMSG,
	    SCREEN0);

	return (CWORD_S_FAILURE);
    }
    
    if (strlen (word_str) > CWORD_SIZEOF_WORD)
    {
	cword_messages_output_message (
	    CWORD_S_STRTOOLONGMSG,
	    SCREEN0);

	return (CWORD_S_FAILURE);
    }

    if (cword_word_is_alphabetics_only (word_str) == FALSE)
    {
	cword_messages_output_message (
	    CWORD_INVALIDCHAR,
	    SCREEN0,
	    word_str);

	return (CWORD_S_FAILURE);
    }

    cword_word_lower_str (word_str);

    /*
     *  Check that the word doesn't already exist in the extant word files!
     *
     *	If FAILURE returned then the word hasn't been found so we can continue.
     */

    status = cword_match_spell_check (word_str);

    if (status == CWORD_S_NORMAL)
    {
	cword_messages_output_message (
	    CWORD_SPELLSUCCESS,
	    SCREEN0,
	    word_str);

	/*
	 *  ?? Error message is really necessary as it's an error ??
	 */

	return (CWORD_S_FAILURE);
    }
    else if (status == CWORD_WORDREADERR) 
    {
	/*
	 *  Continue for the time being as inputted word may have no word file
	 *  in which to be searched for.  It's unlikely to be an error - I
	 *  hope.
	 */
    }

    /*
     *	Word not found now decide how to add it to the existing word file of
     *	length strlen (word_str).
     */
    
    cword_messages_output_message (
	CWORD_S_ADDWORDTOFILE,
	(TRACE0 | SCREEN0),
	word_str,
	strlen (word_str));

    /*
     *  ?? Allow user to select which word file to add word to ??
     *
     *	?? Assume for the time being that it's always going to be the first on
     *	the list ?? 
     */

    if (cword_match_get_word_file_name (
	    strlen (word_str),
	    word_file_str) == FALSE)
    {
	printf ("# No word file specification returned - error adding word\n");

	return (CWORD_S_FAILURE);
    }

    /*
     *	Successfully found input word file specification now use it!
     */

    if (cword_match_update_word_file (
	    word_str,
	    word_file_str) == CWORD_S_FAILURE)
    {
	cword_messages_output_message (
	    CWORD_S_WORDTOFILEFAIL,
	    (TRACE0 | SCREEN0),
	    word_str,
	    word_file_str);

	return (CWORD_S_FAILURE);
    }
    else
    {
	cword_messages_output_message (
	    CWORD_S_WORDTOFILESUC,
	    (TRACE0 | SCREEN0),
	    word_str,
	    word_file_str);

	return (CWORD_S_NORMAL);
    }

}



PUBLIC int cword_match_update_word_file (
	char			*word_str,
	char			*word_file_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function (given word and word file specification) to write inputted
 *	word to output word file.
 *
 *  FORMAL PARAMETERS:
 *
 *	*word_str		(r) Word to write to word file *word_file_str.
 *	*word_file_str		(r) Word file specification.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		() Successfully wrote word to new word file.
 *	CWORD_S_FAILURE		() Error writing word to word file.
 *--
 */

{

    char temp_file_str[CWORD_SIZEOF_FILESPEC + 1],
	 message_str[CWORD_SIZEOF_MESSAGE + 1],
	 in_word_str[CWORD_SIZEOF_WORD + 2];

    FILE *in_fp,
         *temp_fp;

    int count;

    BOOLEAN is_new_word_written;

    if (cword_utils_does_file_exist (word_file_str) == TRUE)
    {
	/*
	 *  File exists continue with processing...
	 *
	 *  Open existing word file for reading and open output temporary file
	 *  for writing.
	 */

	errno = 0;
	in_fp = fopen (word_file_str, "r");

	if (in_fp == (FILE *) 0)
	{
	    cword_messages_get_message (
		CWORD_S_WORDFILEPERROR,
		message_str);

	    perror (message_str);

	    return (CWORD_S_FAILURE);
	}

	/*
	 *  Construct temporary file specification to store intermediate word
	 *  file material - of the form WORD_FILE.TMP.
	 */

	strcpy (temp_file_str, word_file_str);

	/*
	 *  ?? Assuming that temp_file_str contains a '.' ??
	 */

	count = strlen (temp_file_str) - 1;
	while ( (temp_file_str[count] != '.') && (count >= 0) )
	    count--;
	temp_file_str[count] = (char) 0;

	strcat (temp_file_str, CWORD_SUFFIX_TEMP_FILE);

	DEBUG_TRACE (
	    "Temporary word file spec: %s\n",
	    temp_file_str);

	/*
	 *  Delete any previous versions of temp_file_str before creating a new
	 *  one temporary file.
	 */

	if (cword_utils_does_file_exist (temp_file_str) == TRUE)
	    cword_utils_remove_file (temp_file_str);

	errno = 0;
	temp_fp = fopen (temp_file_str, "w");

	if (temp_fp == (FILE *) 0)
	{
	    cword_messages_get_message (
		CWORD_S_TEMPFILEPERROR,
		message_str);

	    perror (message_str);

	    return (CWORD_S_FAILURE);
	}

	/*
	 *  ?? DOT, DOT, DOT to display to the user than something is happening ??
	 */

	is_new_word_written = FALSE;
	while (feof (in_fp) == 0)
	{
	    errno = 0;
	    if (fgets (
		    in_word_str,
		    sizeof (in_word_str) - 1,
		    in_fp) == 0)
	    {
		if (feof (in_fp) != 0)
		{
		    /*
		     *	EOF really found.  check that the user defined word has
		     *	been added before finishing the housekeeping!
		     */

		    if (is_new_word_written == FALSE)
		    {
			/*
			 *  Write the new word to the temporary word file.
			 */

			if (cword_match__fprintf_word (
				word_str,
				temp_fp) == CWORD_S_FAILURE)
			{
			    fclose (in_fp);
			    fclose (temp_fp);
			    cword_utils_remove_file (temp_file_str);

			    return (CWORD_S_FAILURE);
			}

			/*
			 *  To make sure the new word is written to the output
			 *  word file *only* once.
			 */

			is_new_word_written = TRUE;
		    }

		    break;
		}
		else
		{
		    /*
		     *	Valid error found while reading input word file.
		     */

		    cword_messages_get_message (
			CWORD_S_WORDFILEPERROR,
			message_str);

		    perror (message_str);

		    fclose (in_fp);
		    fclose (temp_fp);
		    cword_utils_remove_file (temp_file_str);

		    return (CWORD_S_FAILURE);
		}
	    }

	    /*
	     *  Slam '\n' into EOL.
	     */

	    in_word_str[strlen (in_word_str) - 1] = (char) 0;

	    if (strcmp (in_word_str, word_str) > 0)
	    {
		/*
		 *  Found the location where the new word should be inserted.
		 *  Once it's written then write the input word too to the
		 *  output file.
		 */

		if (is_new_word_written == FALSE)
		{
		    /*
		     *	Write the new word to the temporary word file.
		     */

		    if (cword_match__fprintf_word (
			    word_str,
			    temp_fp) == CWORD_S_FAILURE)
		    {
			fclose (in_fp);
			fclose (temp_fp);
			cword_utils_remove_file (temp_file_str);

			return (CWORD_S_FAILURE);
		    }

		    /*
		     *	To make sure the new word is written to the output word
		     *	file *only* once.
		     */

		    is_new_word_written = TRUE;
		}

		if (cword_match__fprintf_word (
			in_word_str,
			temp_fp) == CWORD_S_FAILURE)
		{
		    fclose (in_fp);
		    fclose (temp_fp);
		    cword_utils_remove_file (temp_file_str);

		    return (CWORD_S_FAILURE);
		}
	    }
	    else
	    {
		/*
		 *  Write the word just inputted and continue.
		 */

		if (cword_match__fprintf_word (
			in_word_str,
			temp_fp) == CWORD_S_FAILURE)
		{
		    fclose (in_fp);
		    fclose (temp_fp);
		    cword_utils_remove_file (temp_file_str);

		    return (CWORD_S_FAILURE);
		}
	    }
	}

	fclose (in_fp);
	fclose (temp_fp);

	/*
	 *  Disable Control-C processing for the duration of the remove and
	 *  rename!
	 */

	cword_utils_manage_sigint (DISABLE);

	/*
	 *  Remove old word file first.
	 */

	if (cword_utils_remove_file (word_file_str) == FALSE)
	{
	    cword_messages_output_message (
		CWORD_S_DELWORDFILEERR,
		(TRACE0 | SCREEN0),
		word_file_str);

	    cword_utils_remove_file (temp_file_str);

	    cword_utils_manage_sigint (ENABLE);

	    return (CWORD_S_FAILURE);
	}

	/*
	 *  Rename new word file to old word file secondly.
	 */

	if (cword_utils_rename_file (temp_file_str, word_file_str) == FALSE)
	{
	    cword_messages_output_message (
		CWORD_S_RENWORDFILEERR,
		(TRACE0 | SCREEN0),
		temp_file_str,
		word_file_str);

	    cword_utils_manage_sigint (ENABLE);

	    /*
	     *	Leaving temp_word_file hanging around as the user will have to
	     *	perform a manual hack to prevent possible major loss of word
	     *	file data.
	     */

	    return (CWORD_S_FAILURE);
	}

	/*
	 *  Re-enable normal Control-C processing.
	 */

	cword_utils_manage_sigint (ENABLE);
    }
    else
    {
	/*
	 *  New word file to be created.
	 */

	temp_fp = fopen (word_file_str, "w+");

	if (temp_fp == (FILE *) 0)
	{
	    cword_messages_get_message (
		CWORD_S_TEMPFILEPERROR,
		message_str);

	    perror (message_str);

	    return (CWORD_S_FAILURE);
	}

	cword_messages_output_message (
	    CWORD_S_NEWWORDFILEMSG,
	    (TRACE0 | SCREEN0),
	    word_file_str);

	if (cword_match__fprintf_word (
		word_str,
		temp_fp) == CWORD_S_FAILURE)
	{
	    fclose (temp_fp);
	    cword_utils_remove_file (word_file_str);

	    return (CWORD_S_FAILURE);
	}

	/*
	 *  Nothing gets deleted as we are dealing with a new word file!
	 */

	fclose (temp_fp);
    }

    /*
     *	If the code gets down to here then everything is 100% OK!
     */

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_match__fprintf_word (
	char			*word_str,
	FILE			*word_fp)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to output word to word file given word to write and file
 *	pointer to write word to!
 *
 *  FORMAL PARAMETERS:
 *
 *	*word_str		(r) Word to output to word file.
 *	*word_fp		(r) file pointer to write word word_str to.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully wrote word to word file.
 *	CWORD_S_FAILURE		Error writing word to word file.
 *--
 */

{

    char message_str[CWORD_SIZEOF_MESSAGE + 1];

    errno = 0;
    if (fprintf (
	    word_fp,
	    "%s\n",
	    word_str) == -1)
    {
	cword_messages_get_message (
	    CWORD_S_TEMPFILEPERROR,
	    message_str);

	perror (message_str);

	return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



#else

extern int main (
	int			argc, 
	char			*argv[])

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This program tests various word file based functions.
 *
 *  FORMAL PARAMETERS:
 *
 *	argv[1]		(r) word whose spelling is to be checked
 *--
 */ 

{

    exit (EXIT_SUCCESS);

}

#endif
