/*
 *  ?? Control structure to allow for free'ing of memory from calling 
 *  procedure ??
 */

/*
 *++
 *  FACILITY:
 *
 *      memman.c
 *
 *  ABSTRACT:
 *
 *	Memory management routines for use in the Cword application.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:     
 *
 *	10 February 1993
 *--
 */

/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#if defined(__unix) || defined(__unix__)

#include <sys/file.h>
#include <malloc.h>

#endif


#if defined(__MSDOS__) || defined(MSDOS) 

/*
 *  ?? Which header is not needed ??
 */

#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>

#endif

#include "main.h"
#include "print.h"
#include "grid.h"
#include "utils.h"
#include "memman.h"
#include "anagram.h"
#include "messages.h"



#ifndef MEMMAN

PUBLIC CROSSWORD_RECORD *cword_memman_make_crossword_rec ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *	Pointer to instance of CROSSWORD_RECORD.
 *--
 */

{

    /*
     *	Calloc enough memeory to satisfy largest possible crossword definition.
     */

    DEBUG_TRACE ("Callocing crossword_record\n");

    return ((CROSSWORD_RECORD *) calloc (1, sizeof (CROSSWORD_RECORD)));

}



PUBLIC void cword_memman_free_cross_rec (
	CROSSWORD_RECORD	*crossword_record)
    
/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to completely de-allocate an instance of a CROSSWORD_RECORD.
 *
 *  FORMAL PARAMETERS:
 *
 *      *crossword_record	(w) pointer to a CROSSWORD_RECORD struct
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None!
 *--
 */

{

    if (crossword_record == (CROSSWORD_RECORD *) 0)
    {
	cword_messages_output_message (
	    CWORD_S_DEALLOCNULL,
	    SCREEN0);

	return;
    }

    DEBUG_TRACE ("Freeing crossword_record\n");

    /*
     *	Call the RTL function free to de-allocate any dynamic memory used in
     *	allocating crossword_record.
     */

    free (crossword_record);

    return;

}



PUBLIC CWORD_T_GRID_STRUCT *cword_memman_make_grid_rec ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *	Pointer to instance of CWORD_T_GRID_STRUCT.
 *--
 */

{

    /*
     *	Calloc enough memeory to satisfy largest possible grid definition.
     */

    DEBUG_TRACE ("Callocing CWORD_T_GRID_STRUCT\n");

    return ((CWORD_T_GRID_STRUCT *) calloc (1, sizeof (CWORD_T_GRID_STRUCT)));

}



PUBLIC void cword_memman_free_grid_rec (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to completely de-allocate an instance of a CWORD_T_GRID_STRUCT.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(w) pointer to a CWORD_T_GRID_STRUCT struct
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None!
 *--
 */

{

    if (grid_record == (CWORD_T_GRID_STRUCT *) 0)
    {
	cword_messages_output_message (
	    CWORD_S_DEALLOCNULL,
	    SCREEN0);

	return;
    }

    DEBUG_TRACE ("Freeing grid_record\n");

    /*
     *	Call the RTL function free to de-allocate any dynamic memory used in
     *	allocating grdi_record.
     */

    free (grid_record);

    return;

}



PUBLIC CWORD_T_STR_LL *cword_memman_create_str_ll (
	int			sizeof_word)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    CWORD_T_STR_LL *temp_ll;

    temp_ll = (CWORD_T_STR_LL *) calloc (1, sizeof (CWORD_T_STR_LL));

    if (temp_ll == (CWORD_T_STR_LL *) 0)
    {
	printf ("# Error making dynamic memory available\n");

	return ((CWORD_T_STR_LL *) 0);
    }

    temp_ll->previous = (CWORD_T_STR_LL *) 0;
    temp_ll->next = (CWORD_T_STR_LL *) 0;

    if (sizeof_word != 0)
    {
	/*
	 *  Include room for NULL terminator.
	 */

	temp_ll->str = (char *) calloc (1, (sizeof_word + 1));

	if (temp_ll->str == (char *) 0)
	{
	    free (temp_ll);

	    printf ("# Error making dynamic memory available\n");

	    return ((CWORD_T_STR_LL *) 0);
	}

	temp_ll->str[0] = (char) 0;
    }
    else
    {
	temp_ll->str = (char *) 0;
    }

    temp_ll->str_val = 0.0;

    return (temp_ll);

}



PUBLIC BOOLEAN cword_memman_free_str_ll (
	CWORD_T_STR_LL         **word_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Memory successfully de-allocated.
 *--
 */

{

    (*word_ll)->previous = (CWORD_T_STR_LL *) 0;
    (*word_ll)->next = (CWORD_T_STR_LL *) 0;

    memset (
	(*word_ll)->str, 
	0, 
	strlen ((*word_ll)->str));

    (*word_ll)->str_val = 0.0;

    /*
     *	Free string member before freeing the link list element.
     */

    if ((*word_ll)->str != (char *) 0)
    {
	free ((*word_ll)->str);
    }

    free (*word_ll);

    *word_ll = (CWORD_T_STR_LL *) 0;

    return (TRUE);

}



#else

extern int main (
		int argv, 
		char *argc[])	

{
}

#endif
