/*
 *++
 *  FACILITY:
 *
 *      messages.c
 *
 *  ABSTRACT:
 *
 *	This messaging protocol is completely my own version of a simple
 *	portable messaging system.  It can probably be improved upon, but,
 *	suffices for the needs of cword's cross-platform portability ambitions.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:     
 *
 *	24 February 1993
 *--
 */

/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdarg.h>
#include <string.h>

#include "main.h"
#include "utils.h"
#include "charui.h"
#include "defaults.h"

/*
 *  MESSAGES_DEFINED is used to allocate memory to the message array in
 *  messages.h.  All other module attach to this message array at link time.
 *  It's essential that this constant is defined once, and only once in the
 *  Cword application [usually in the module containing main ()], otherwise no
 *  message array will be allocated memory on the heap.
 */

#define MESSAGES_DEFINED 1

#include "messages.h"

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#ifndef MESSAGES



/*
 *  ?? What should the input data type be ??
 */

PUBLIC unsigned int cword_messages_get_trace_mask (
	unsigned int		output_mask)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRACE mask of input output_mask
 *--
 */

{

    unsigned int trace_mask = 0;

    if (output_mask | TRACE0)
	trace_mask |= TRACE0;

    if (output_mask | TRACE1)
	trace_mask |= TRACE1;

    if (output_mask | TRACE2)
	trace_mask |= TRACE2;

    if (output_mask | TRACE3)
	trace_mask |= TRACE3;

    DEBUG_TRACE (
	"Trace mask is %x\n",
	trace_mask);

    return (trace_mask);

}



PUBLIC unsigned int cword_messages_get_screen_mask (
	unsigned int		output_mask)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      SCREEN mask of input output_mask
 *--
 */

{

    unsigned int screen_mask = 0;

    if (output_mask | SCREEN0)
	screen_mask |= SCREEN0;

    if (output_mask | SCREEN1)
	screen_mask |= SCREEN1;

    if (output_mask | SCREEN2)
	screen_mask |= SCREEN2;

    if (output_mask | SCREEN3)
	screen_mask |= SCREEN3;

    DEBUG_TRACE (
	"Screen mask is %x\n",
	screen_mask);

    return (screen_mask);

}



PUBLIC int cword_messages_get_message (
	int			message_code,
	char			*message_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Function to obtain a message from the message file
 *
 *  FORMAL PARAMETERS:
 *
 *	message_code	    (r) ID of message to return in *message_str
 *	*message_str	    (w) String to return message corresponding to message_code
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */
{


    if (message_code > LAST_MESSAGE)
    {
	/*
	 *  Error if the message_code is greater than the greatest element of
	 *  the messages array.  If this happens then Cword probably needs
	 *  re-compiling, or the message file needs updating.
	 */

	cword_messages_output_message (
		    CWORD_LASTMESSAGEERR,
		    (TRACE2 | SCREEN0));

	message_str[0] = (char) 0;

	return (CWORD_S_FAILURE);
    }

    strcpy (message_str, cword_messages_l_msgs[message_code]);

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Include __FILE__ and __LINE__ within the current conventions ??
 *
 *  Define a macro that breaks ...output_message into two distinct
 *  functions.  The first defines the line and file identifiers of
 *  the log message being outputted.  The second actually calls the
 *  log function.  The log function then obtains the current line and 
 *  file values from the module internal variables set by the first
 *  function.
 */

PUBLIC int cword_messages_output_message (
	int			message_code,
	int			output_mask,
	...)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to output Cword messages (to screen or trace file) using
 *	variable argument lists!
 *
 *  FORMAL PARAMETERS:
 *
 *	message_code		(r) message ID code used to reference message array
 *	output_mask		(r) Bit mask telling function how to outputp message
 *	...			(r) variable argument list!
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.trace_level
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      length of output	Successful 
 *	0			Unsuccessful for unspecified reason
 *--
 */

{

    va_list args;

    char message_str[CWORD_SIZEOF_MESSAGE + 1],
	 output_str[CWORD_SIZEOF_MESSAGE + 1];

    int status;

    BOOLEAN is_message_outputted = FALSE;

    if (output_mask == 0)
    {
	/*
	 *  If there is nothing in output_mask there is no point continuing the
	 *  execution of this function.
	 */

	return (0);
    }

    status = cword_messages_get_message (
		message_code,
		message_str);

    if (status != CWORD_S_NORMAL)
	return (0);

    va_start (args, output_mask);

    /*
     *	?? What happens when one of the input args is a string with an embedded
     *	'%'?  It will not be outputted to the screen correctly as I'm using
     *	vsprintf/printf??
     *
     *	?? The calling function will have to preprocess the string so that it
     *	will be outputted correctly via cword_messages_output_message ??
     *
     *	?? What happens if the input parameters are longer than is allowed i.e.
     *	output string will overflow.  How to check this on the fly ??
     */

    /*
     *	?? No need for vsprintf at all as vprintf can be used for SCREEN*
     *	processing and vfprintf can be used if I can get access to the trace
     *	file file pointer ??
     *
     *	?? To access the trace file file pointer I'd have to write another
     *	function which would return the file pointer which I could then used
     *	for TRACE* processing ??
     *
     *  ?? Does away with output_str ??
     */

    if (vsprintf (
	    output_str,
	    message_str,
	    args) == EOF)
    {
	printf (
    	    "## Error with vsprintf() in cword_messages_output_message ##\n");

	return (0);
    }

    if (strlen (output_str) > sizeof (output_str) - 1)
    {
	printf (
    	    "## Message output overflowed - could cause memory corruption ##\n");

	return (0);
    }

    /*
     *  !!!!! SCREEN !!!!!
     */

    /*
     *  Output to screen via printf
     */

    status = 0;

    /*
     *  ?? Use something like ??:
     *
     *	vprintf (
     *	    message_str,
     *	    args);
     */

    if ( (output_mask & SCREEN0) && (cword_g_defaults_record.trace_level >= LEVEL0) )
    {
	status = printf (output_str);

	is_message_outputted = TRUE;
    }
    else if ( (output_mask & SCREEN1) && (cword_g_defaults_record.trace_level >= LEVEL1) )
    {
	status = printf (output_str);

	is_message_outputted = TRUE;
    }
    else if ( (output_mask & SCREEN2) && (cword_g_defaults_record.trace_level >= LEVEL2) )
    {
	status = printf (output_str);

	is_message_outputted = TRUE;
    }
    else if ( (output_mask & SCREEN3) && (cword_g_defaults_record.trace_level >= LEVEL3) )
    {
	status = printf (output_str);

	is_message_outputted = TRUE;
    }

    if (status == EOF)
    {
    	/*
 	 *  Problem with printf!
     	 */

	printf (
    	    "## Error with printf() in cword_messages_output_message ##\n");

	return (0);
    }

    /*
     *  !!!!! TRACE !!!!!
     */

    /*
     *  ?? Use something like ??:
     *
     *  WRITE_TRACE_VARARGS (
     *	    message_str,
     *	    args);
     */

    /*
     *  Output to trace log file via WRITE_TRACE.
     */

    status = CWORD_S_NORMAL;

    if ( (output_mask & TRACE0) && (cword_g_defaults_record.trace_level >= LEVEL0) )
	status = WRITE_TRACE (output_str);
    else if ( (output_mask & TRACE1) && (cword_g_defaults_record.trace_level >= LEVEL1) )
	status = WRITE_TRACE (output_str);
    else if ( (output_mask & TRACE2) && (cword_g_defaults_record.trace_level >= LEVEL2) )
	status = WRITE_TRACE (output_str);
    else if ( (output_mask & TRACE3) && (cword_g_defaults_record.trace_level >= LEVEL3) )
	status = WRITE_TRACE (output_str);
#if 0
    else
    {
	char output_str1[CWORD_SIZEOF_MESSAGE + 1];
	if ( (!(output_mask & SCREEN0) && 
	       !(output_mask & SCREEN1) && 
	        !(output_mask & SCREEN2) &&
                 !(output_mask & SCREEN3)) )
	{
	    /*
	     *  ?? This part of the if-else statement will disappear when all the
	     *  messages are traced properly through the code ??
	     */

	    sprintf (
		    output_str1,
		    "***** %s",
		    output_str);

	    status = WRITE_TRACE (output_str1);
        }
    }
#endif

    if (CWORD_S_NORMAL != status)
    {
	return (0);
    }

    if (is_message_outputted == TRUE)
    {
	/*
	 *  If the output message has been displayed to the screen then check
	 *  for CWORD-E and ring the bell if found in output_str
	 */

	if (strstr (output_str, "CWORD-E-") != 0)
	{
	    cword_charui_ring_bell (1);	    
	}

	if (strstr (output_str, "CWORD-F-") != 0)
	{
	    cword_charui_ring_bell (2);
	}
    }

    /*
     *  Making sure that output to the screen is flushed after writing as
     *  some U*X systems don't flush strout until a LF is entered
     */

    if (fflush (stdout) == EOF)
    {
	/*
	 *  ?? If there is a problem fflushing stdout then there's little
	 *  chance that this printf will work either :-( ??
	 */

	printf (
    	    "## Error with fflush() of stdout (standard output) ##\n");

	return (0);
    }

    va_end (args);

    /*
     *	If we get down to here then outputting has gone OK, therefore it's OK
     *	to return the length of the output_str through the return value.
     */

    return (strlen (output_str));

}

#else



extern int main()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      test_harness main function to check the functionality of the message facility
 *--
 */
{

    char message_str[CWORD_SIZEOF_MESSAGE];
    int	message_code,
	cword_status;

    OPEN_TRACE ("james.trace");

    message_code = CWORD_TEST;

    cword_status = cword_messages_output_message (
	    message_code,
	    (TRACE | SCREEN),
	    "string1",
	    "string2",
	    "string3",
	    "string4");

    CLOSE_TRACE ();

    exit (cword_status | NO_SCREEN);
}

#endif
