#ifndef CWORD_MESSAGES_LOADED
#define CWORD_MESSAGES_LOADED 1

#include "main.h"

/*
 *  Current version ID of Cword.
 */

#define CWORD_VERSION_ID	"Cword V1.01"

/*
 *  Bit masking helps to add more flexibilty to messaging mechanism.
 */

#define SCREEN0   0x0001
#define SCREEN1   0x0002
#define SCREEN2   0x0004
#define SCREEN3   0x0008

#define TRACE0    0x0010
#define TRACE1    0x0020
#define TRACE2    0x0040
#define TRACE3    0x0080

/*
 *  Used by defaults struct to define the level of information that is to be
 *  outputted to the user of cword.
 */

#define LEVEL0    0X0400
#define LEVEL1    0X0800
#define LEVEL2    0X1000
#define LEVEL3    0X2000

/* 
 *  PUBLIC forward function prototypes.
 */

PUBLIC unsigned int cword_messages_get_trace_mask (
	unsigned int		output_mask);

PUBLIC unsigned int cword_messages_get_screen_mask (
	unsigned int		output_mask);

PUBLIC int cword_messages_get_message (
	int			message_code,
	char			*message_str);

PUBLIC int cword_messages_output_message (
	int			message_code,
	int			output_mask,
	...);



/* 
 *  This section will contain all messages, which although not messages in this
 *  file's true sense, are modifible if another language is to be supported.
 */

#define CWORD_BLOCK_GRID_STR	"Block"
#define CWORD_BARRED_GRID_STR	"Barred"
#define CWORD_AMERICAN_GRID_STR	"American"
#define CWORD_NOVELTY_GRID_STR	"Novelty"
#define CWORD_INVALID_GRID_STR	"Invalid"

#define CWORD_ACROSS_STR	"Across"
#define CWORD_DOWN_STR		"Down"

#define CWORD_YES_STR		"Yes"
#define CWORD_NO_STR		"No"

#define CWORD_ON_STR		"On"
#define CWORD_OFF_STR		"Off"

#define CWORD_A_CHAR		'A'
#define CWORD_D_CHAR		'D'

#define CWORD_TRUE_STR		"TRUE"
#define CWORD_FALSE_STR		"FALSE"

#define CWORD_GRID_STR		"Grid"
#define CWORD_CLUES_STR		"Clues"

#define CWORD_N_CHAR		'N'

#define CWORD_DEBUG_STR		"DEBUG"

#define ENABLED_STR		"enabled"
#define NOT_ENABLED_STR		"not enabled"

#define CWORD_UNDEFINED_STR	"undefined"

#define CWORD_WORD_STR		"word"

#define CWORD_X_CHAR		'X'
#define CWORD_Y_CHAR		'Y'

#define CWORD_SEEDED_CHAR	'S'
#define CWORD_AVAIL_CHAR	'-'
#define CWORD_CLUED_CHAR	'C'
#define CWORD_NOTCLUED_CHAR     '-'

#define CWORD_UNFILLED_CHAR	'-'
#define CWORD_UNCHED_CHAR	'*'
#define CWORD_CHECKED_CHAR	'#'
#define CWORD_UNKNOWN_CHAR      '?'

#define CWORD_NONE_STR		"none"

#define CWORD_MAIN_STR		"MAIN"
#define CWORD_SUB_STR		"SUB"

#define ARE_STR			"are"
#define IS_STR			"is"

#define ITEM_STR                "item"
#define ITEMS_STR               "items"

#define PAIR_STR                "pair"
#define PAIRS_STR               "pairs"

#define LIGHT_STR               "light"
#define LIGHTS_STR 		"lights"

#define CLUE_STR		"clue"
#define CLUES_STR		"clues"

#define GRID_STR		"grid"
#define GRIDS_STR		"grids"

#define CROSSWORD_STR		"crossword"
#define CROSSWORDS_STR		"crosswords"

#if defined(VMS) || defined(__VMS) || defined(__osf__)

#define DECLARE_CURSOR_STR	"Declare Cursor"
#define OPEN_CURSOR_STR		"Open Cursor"
#define FETCH_CURSOR_STR	"Fetch Cursor"
#define CLOSE_CURSOR_STR	"Close Cursor"

#define SET_TRANSACTION_STR	"Set Transaction"
#define ROLLBACK_STR		"Rollback"
#define FINISH_STR		"Finish"
#define INSERT_STR		"Insert"
#define COMMIT_STR		"Commit"
#define DELETE_STR		"Delete"
#define DISCONNECT_STR		"Disconnect"
#define COUNTSTAR_STR		"COUNT(*)"

#endif

#define BLOCK_GRID_STR		"Block grid"
#define BARRED_GRID_STR		"Barred grid"
#define BLOCK_CROSSWORD_STR	"Block crossword"
#define BARRED_CROSSWORD_STR    "Barred crossword"



/*
 *  ?? Any better way of doing this than the present implementation ??
 *
 *  Not a very flexible method as it doesn't allow for grouping of messages
 *  very well.
 *
 *  I could write a program which, given an input series of messages in an array
 *  write the H file with the message identifiers corresponding to the material
 *  inside the brackets in the file.
 *
 *  ?? Used enum'ed type instead of current method of referencing messages ??
 *
 *  enum message_id
 *  {
 *      CWORD_NORMAL = 1,
 *      CWORD_ERROR  
 *  }
 *
 *  once CWORD_NORMAL is defined to a value of 1 then all the rest of the 
 *  enumerated types will have values one greater than the previous, thus 
 *  CWORD_ERROR will equal 2!
 */



/*
 *  Message text and message number to correspond to that message
 */

#define CWORD_S_NORMAL		1
#define CWORD_S_FAILURE		2

#define CWORD_NOTRACE		3
#define CWORD_TEST		4
#define CWORD_NOGRIDCREATED	5
#define CWORD_CONTINUE		6
#define CWORD_WRITEERROR	7
#define CWORD_MEMORYERROR	8
#define CWORD_ADDWORDERROR	9
#define CWORD_S_DEALLOCNULL     10
#define CWORD_WORDTOOBIG	11
#define CWORD_NOFILESFOUND	12
#define CWORD_PARSEERROR	13
#define CWORD_CHOICENOTVALID	14
#define CWORD_PRINTCOMMAND      15
#define CWORD_DBERROR		16
#define CWORD_FILEERROR		17
#define CWORD_PRINTERROR	18
#define CWORD_SERVERERROR	19
#define CWORD_WEBPARSEERROR	20
#define CWORD_S_CREATOUTPUTHEAD 21
#define CWORD_PARAMTOOSMALL	22
#define CWORD_ANAGRAMSTOOMANY	23
#define CWORD_S_REPORTPROBLEMS  24
#define CWORD_NOANAGRAMS	25
#define CWORD_ANAGRAMMSG	26
#define CWORD_S_DEFNSINDB       27
#define CWORD_ANAGRAMWARN	28
#define CWORD_ANAGRAMERR	29
#define CWORD_WEBSTERERR 	30
#define CWORD_OUTFILEMSG	31
#define CWORD_PRINTFILEMSG	32
#define CWORD_S_FILENOTEXIST	33
#define CWORD_PRINTFILESUCC	34
#define CWORD_PRINTFILEFAIL	35
#define CWORD_MAINBANNER	36
#define CWORD_ILLFILENAME	37
#define CWORD_TIMEERROR		38
#define CWORD_VMSDELETEFILE 	39
#define CWORD_PRINTMSG 	        40
#define CWORD_TRACEFILEERR	41
#define CWORD_TRACEFILEOPEN	42
#define CWORD_TRACEFILECLOSE	43
#define CWORD_SELECTPSTYPE	44
#define CWORD_PSTYPEERRPR	45
#define CWORD_XRDOUTFILE	46
#define CWORD_PSTXTMSG		47
#define CWORD_XRDREAD		48
#define CWORD_XRDWRITTEN	49
#define CWORD_GRDREAD		50
#define CWORD_GRDWRITTEN	51
#define CWORD_DEFREAD		52
#define CWORD_INVPRINTOPT	53
#define CWORD_PSWRITTEN		54
#define CWORD_TXTWRITTEN	55
#define CWORD_S_LOOSECLUESMSG	56
#define CWORD_IVDINPUT		57
#define CWORD_GETSTATSXRDMSG	58
#define CWORD_S_ISDBBUILD	59
#define CWORD_WEBCONNECT	60
#define CWORD_WEBQUERY		61
#define CWORD_WEBERR		62
#define CWORD_WEBOUTLEN		63
#define CWORD_WEBDISCONNECT	64
#define CWORD_WAITING		65
#define CWORD_WAITINGENDED	66
#define CWORD_WEBNOUTPUT	67
#define CWORD_RDBOPENDB		68
#define CWORD_RDBDEADLOCK	69
#define CWORD_RDBCONFLICT	70
#define CWORD_RDBUNDEFERR	71
#define CWORD_S_CLUEALREADYINDB	72
#define CWORD_RDBGENERICSUC	73
#define CWORD_RDBGENERICERR	74
#define CWORD_RDBNOTRANX	75
#define CWORD_BLANKMSG		76
#define CWORD_S_RDBENDOFSTREAM	77
#define CWORD_RDBADDSUCC	78
#define CWORD_RDBADDFAIL	79
#define CWORD_RDBREADSUCC       80
#define CWORD_RDBREADFAIL       81
#define CWORD_RDBDELSUCC	82
#define CWORD_S_RDBDELFAIL	83
#define CWORD_S_COMLINIGNORE	84
#define CWORD_S_COMLINEMAXMIN	85
#define CWORD_RDBINVTRANX	86
#define CWORD_RDBBADTRANX	87
#define CWORD_RDBINXCONSTR	88
#define CWORD_RDBINVINDEX	89
#define CWORD_RDBERR		90
#define CWORD_RDBOUTOFRANGE	91
#define CWORD_RDBADDCROSSSUC	92
#define CWORD_RDBADDCLUESUC	93
#define CWORD_S_RDBADDGRIDSUC	94
#define CWORD_RDBADDCROSSERR	95
#define CWORD_RDBADDCLUEERR	96
#define CWORD_RDBADDGRIDERR	97
#define CWORD_RDBADDANASUC	98
#define CWORD_RDBADDHOMSUC	99
#define CWORD_RDBADDSYNSUC      100
#define CWORD_RDBADDANAERR	101
#define CWORD_RDBADDHOMERR	102
#define CWORD_RDBADDSYNERR      103
#define CWORD_RDBGETANADESC	104
#define CWORD_RDBGETHOMODESC	105
#define CWORD_RDBGETSYMDESC	106
#define CWORD_SIGNALGENERATED	107
#define CWORD_S_GRIDGENFINISHED	108
#define CWORD_GRDFILEWRITE	109
#define CWORD_GRDFILELISTYES	110
#define CWORD_GRDFILELISTNO	111
#define CWORD_SAVEGRIDEND	112
#define CWORD_GRIDCALLED	113
#define CWORD_GRIDENDERR	114
#define CWORD_GRIDNONECREATED	115
#define CWORD_GRIDSTARTPHASE	116
#define CWORD_GRIDVALID		117
#define CWORD_GRIDENDYET	118
#define CWORD_VERINVANSNUM	119
#define CWORD_VERINVANSLOC	120
#define CWORD_VERINVSYMMETRY	121
#define CWORD_VERINVANSONGRID	122
#define CWORD_VERADJACUNCHES	123
#define CWORD_VERINVDIMSYM	124
#define CWORD_VERNOSYMDEF	125
#define CWORD_VERSYMMFAIL	126
#define CWORD_VERBLOCKPROP	127
#define CWORD_FILLXWORD		128
#define CWORD_XWORDFILEWRITE    129
#define CWORD_XWORDCLUEGEN	130
#define CWORD_WEBTOOMUCHDATA	131
#define CWORD_LASTMESSAGEERR	132
#define CWORD_GRIDMAKESTART	133
#define CWORD_GRIDFINISHED	134
#define CWORD_S_COMLINEINVD	135
#define CWORD_OUTPUTELEMS 	136
#define CWORD_ERRUNDEFINED	137
#define CWORD_NEWSVALSINXWORD	138
#define CWORD_MATCHNOSTRING	139
#define CWORD_MATCHMINLEN	140
#define CWORD_MATCHNOTARGETSTAT	141
#define CWORD_MATCHTARGETSTAT	142
#define CWORD_S_FILENOTFOUNDERR	143
#define CWORD_NOWORDFILE	144
#define CWORD_S_NODEFNSINDB	145
#define CWORD_WORDREADERR	146
#define CWORD_S_DELFROMDB       147
#define CWORD_WORDFILEOPT	148
#define CWORD_MATCHSUBANAGRAMS	149
#define CWORD_SUBANAGRAMSFOUND	150
#define CWORD_INVALIDCHAR	151
#define CWORD_S_INVALIDINPUT    152
#define CWORD_READINGWORDFILE	153
#define CWORD_NOINTERSECTS	154
#define CWORD_DEFAULTSDEFINE	155
#define CWORD_NUMQMARKSERR	156
#define CWORD_TOOMANYQMARKSERR	157
#define CWORD_STRINGTOOLONG	158
#define CWORD_INTERSECTWORDS	159
#define CWORD_MATCHNOW		160
#define CWORD_SUBANAGRAMSNOW	161
#define	CWORD_STRSTARTNOW	162
#define CWORD_STRMIDDLENOW	163
#define CWORD_STRENDNOW		164
#define CWORD_STRWILDNOW	165
#define CWORD_SOUNDEXNOW	166
#define CWORD_CENTRENOTFOUND	167
#define CWORD_MINSIZETOOBIG	168
#define CWORD_TEMPLATESEARCH	169
#define CWORD_FINISHMSG		170
#define CWORD_XRDFILELISTYES	171
#define CWORD_S_COMLINECENONLY	172
#define CWORD_S_NOWORDFILEFND	173
#define CWORD_DBNOTPORTABLE	174
#define CWORD_ANAGRAMINPUT	175
#define CWORD_ANAGRAMLIST	176
#define CWORD_HOMOPHONEINPUTIN	177
#define CWORD_HOMOPHONEINPUTDE	178
#define CWORD_HOMPHONELIST	179
#define CWORD_SYNONYMINPUTIN	180
#define CWORD_SYNONYMINPUTDE	181
#define CWORD_SYNONYMLIST	182
#define CWORD_INPUTSTRQ		183
#define CWORD_SPELLCHECKSTR	184
#define CWORD_SPELLSUCCESS	185
#define CWORD_SPELLFAILURE	186
#define CWORD_TEMPLATESTR	187
#define CWORD_ANAGRAMMINSTR	188
#define CWORD_ANAGRAMGETSTR	189
#define CWORD_ANAGRAMALWAYSSTR	190
#define CWORD_ANAGRAMALL	191
#define CWORD_SOUNDEXSTR	192
#define CWORD_ROTSTR		193
#define CWORD_WEBSTERSTR	194
#define CWORD_INTERSECTEXPLAIN	195
#define CWORD_INTERSECTSTR1	196
#define CWORD_INTERSECTSTR2	197
#define CWORD_INTERSECTLIST	198
#define CWORD_INPUTCHOICE	199
#define CWORD_S_INVALIDCHOICE	200
#define CWORD_S_PRESSRETURN	201
#define CWORD_NOMATCHESFOUND	202
#define CWORD_LENGREATERMAX	203
#define CWORD_TEMPLATEFAILMSG	204
#define CWORD_IVDDIGIT		205
#define CWORD_SOUNDEXSAME	206
#define CWORD_SOUNDEXDIFF	207
#define CWORD_NOSUBANAGRAMS	208
#define CWORD_NOTEMPLATEMATCH	209
#define CWORD_RETURNEDWORDS	210
#define CWORD_S_COMLINEUNSUPT	211
#define CWORD_CHOICESFOUND	212
#define CWORD_STRSTARTSTR	213
#define CWORD_STRMIDDLESTR	214
#define CWORD_STRENDSTR		215
#define CWORD_STRWILDSTR	216
#define CWORD_NOCOMPANAGRAMS	217
#define CWORD_COMPANAGRAMS	218
#define CWORD_WEBSTERWORD	219
#define CWORD_ROTSTROUT		220
#define CWORD_TRACENOTOPEN	221
#define CWORD_CLOCKBEFORE	222
#define CWORD_CLOCKAFTER	223
#define CWORD_COMPANAGRAMLIST	224
#define CWORD_SOUNDEXLIST	225
#define CWORD_TRACEWRITEERR	226
#define CWORD_SELECTEDWORD	227
#define CWORD_S_SEEDADDED	228
#define CWORD_ADDALLDISALLOWED	229
#define CWORD_ADDONEDISALLOWED	230
#define CWORD_MEMMANDELWORD	231
#define CWORD_ITERATIONNO	232
#define CWORD_ITERATIONFINISHED	233
#define CWORD_POSSIBLECHOICES	234
#define CWORD_PRUNELIST		235
#define CWORD_NOPRUNELIST	236
#define CWORD_PRUNERANDOM	237
#define CWORD_PRUNEWEIGHT	238
#define CWORD_RULE1REDUCESEARCH	239
#define CWORD_PICKWORD		240
#define CWORD_CLUESFORLIGHTING	241
#define CWORD_FILLINGNOW	242
#define CWORD_LIGHTNOTFOUND	243
#define CWORD_POSSIBLECHOICE	244
#define CWORD_CROSSTOGRID	245
#define CWORD_SEEDLIGHTS	246
#define CWORD_ADDSEEDSTOGRID	247
#define CWORD_UPDATETOGRID	248
#define CWORD_OUTPUTGRID	249
#define CWORD_FINDNEXTLIGHT	250
#define CWORD_FOUNDRANDOMLIST	251
#define CWORD_WEIGHTRANDOMLIST	252
#define CWORD_ISGRIDFILLED	253
#define CWORD_YESRETURN		254
#define CWORD_NORETURN		255
#define CWORD_CHECKSEARCHAGAIN	256
#define CWORD_BACKTRACKGRID	257
#define CWORD_GRIDTOCROSS	258
#define CWORD_DELETEDISALLOW	259
#define CWORD_DEADENDALERT	260
#define CWORD_DELETESEED	261
#define CWORD_GRIDGIVEUP	262
#define CWORD_SEEDLENGTHERR	263
#define CWORD_MATCHDONEFAIL	264
#define CWORD_LIGHTNUMBER	265
#define CWORD_LIGHTDIRECTION	266
#define CWORD_LIGHTDIRCHOSEN	267
#define CWORD_INPUTVDLIGHTNUM	268
#define CWORD_INVALIDLIGHTNUM	269
#define CWORD_LIGHTMATCHTHIS	270
#define CWORD_GRDOUTFILE	271
#define CWORD_GETSTATSGRDMSG    272
#define CWORD_ADDTOFAILURES     273
#define CWORD_PARSEFILEERROR	274
#define CWORD_FILEDIMSTOOBIG	275
#define CWORD_FILEWRITEERROR	276
#define CWORD_FILEREADERROR	277
#define CWORD_90SYMMETRYMSG	278
#define CWORD_180SYMMETRYMSG	279
#define CWORD_AXIALLEFTRIGHTMSG 280
#define CWORD_AXIALRIGHTLEFTMSG	281
#define CWORD_NOSYMMETRYMSG	282
#define CWORD_01MSG		283
#define CWORD_02MSG		284
#define CWORD_03MSG		285
#define CWORD_04MSG		286
#define CWORD_05MSG		287
#define CWORD_06MSG		288
#define CWORD_07MSG		289
#define CWORD_08MSG		290
#define CWORD_09MSG		291
#define CWORD_10MSG		292
#define CWORD_ADDLIGHTLOC	293
#define CWORD_LIGHTGRIDMSG	294
#define CWORD_USING		295
#define CWORD_ADDINGUSING	296
#define CWORD_CHECKLIGHTVALID	297
#define CWORD_LIGHTALREADYTHERE	298
#define CWORD_LIGHTSSAMEDIR	299
#define CWORD_CENTRELIGHT	300
#define CWORD_INPUTGRIDDIMQ	301
#define CWORD_INPUTGRIDX 	302
#define CWORD_INPUTGRIDY 	303
#define CWORD_MAXGRIDDIMS	304
#define CWORD_MINGRIDDIMS	305
#define CWORD_S_DIMSSETTO       306
#define CWORD_S_READFROMDBMSG   307
#define CWORD_XNOTEQUALY	308
#define CWORD_INPUTGRIDTYPEMSG	309
#define CWORD_GRIDTYPECHOSEN	310
#define CWORD_INPUTSYMMTYPEMSG	311
#define CWORD_INPUTSYMMTYPE	312
#define CWORD_SYMMTYPECHOSEN	313
#define CWORD_S_DIMZERO         314
#define CWORD_XORYOVERALLOWED	315
#define CWORD_LIGHTTOOSMALL	316
#define CWORD_S_OVERSHOOTMAX	317
#define CWORD_S_UNDERSHOOTMIN	318
#define CWORD_GETLIGHT		319
#define CWORD_S_DBTOFILEMSG     320
#define CWORD_GETLIGHTDIR	321
#define CWORD_GETLIGHTLEN	322
#define CWORD_LIGHTVALUEZERO	323
#define CWORD_VALIDLIGHTFOUND	324
#define CWORD_INVALIDLIGHTFOUND	325
#define CWORD_GRIDDEFINITION	326
#define CWORD_USERDEFINEDSTRING	327
#define CWORD_SYSTEMCALLMSG	328
#define CWORD_SYSTEMCALLRETURN	329
#define CWORD_BESTWORDS		330
#define CWORD_RANDOMWORDS	331
#define CWORD_BESTTORANDOM	332
#define CWORD_GENCLUENUMS	333
#define CWORD_SEEDLIGHTMSG	334
#define CWORD_S_WRITETODB       335
#define CWORD_SEEDSPELLERR	336
#define CWORD_SEEDSPELLOUT	337
#define CWORD_SEEDCELLLOCERR	338
#define CWORD_LIGHTALREADYINGRD	339
#define CWORD_LIGHTSLIST	340
#define CWORD_NOLIGHTSAVAIL	341
#define CWORD_S_DBSTATSMSG	342
#define CWORD_TPLATEUSEDBEFRE	343
#define CWORD_LIGHTACCEPTQ	344
#define CWORD_GETTINGMORELIGHTS	345
#define CWORD_DELETESEEDFOUND	346
#define CWORD_DEADENDMSG	347
#define CWORD_GRIDOUTPUTCLUE 	348
#define CWORD_NOPOSSIBLECHOICES	349
#define CWORD_OVERGENERATEMSG	350
#define CWORD_LIGHTSPELLERR	351
#define CWORD_OVERGENLIGHTADD	352
#define CWORD_OVERGENLIGHTDEL	353
#define CWORD_STDOUTERR		354
#define CWORD_S_RDBADDLIGHTSUC  355
#define CWORD_S_DEFNSINDNMSG    356
#define CWORD_WORDNOTEXIST	357
#define CWORD_CHECKWORDDEFAULT	358
#define CWORD_GENADJACENT	359
#define CWORD_GENADJACWARN	360
#define CWORD_INPUTGRIDTYPE	361
#define CWORD_GENVIOLATE	362
#define CWORD_GENADJACERROR	363
#define CWORD_MAXMATCHFOUND	364
#define CWORD_ITERATIONSOVER	365
#define CWORD_S_COMLINEMISSING	366
#define CWORD_S_NUMINTERSECTS	367
#define CWORD_USESYSTEMDEFAULTS	368
#define CWORD_USEUSERDEFAULTS	369
#define CWORD_OUTPUTUSERDEFS	370
#define CWORD_DEFREADERR	371
#define CWORD_DEFAULTUNDEFINED	372
#define CWORD_APPBUILTWHEN	373
#define CWORD_CHARCELLUIMSG	374
#define CWORD_EXITERRMSG	375
#define CWORD_EXITOKMSG		376
#define CWORD_SORTCROSSWORD	377
#define CWORD_TRACEFLUSHERR	378
#define CWORD_GRIDTOBECALLED	379
#define CWORD_SIGINTMSG		380
#define CWORD_SIGFPEMSG		381
#define CWORD_SIGSEGVMSG	382
#define CWORD_SIGBUSMSG		383
#define CWORD_SIGEXITMSG	384
#define CWORD_GRIDOPTERR	385
#define CWORD_GRDDEFNOTFOUND	386
#define CWORD_GRDINVALIDERR	387
#define CWORD_GRIDOUTPUTMSG	388
#define CWORD_PRINTOPTERR	389
#define CWORD_GENUPDATEMSG1	390
#define CWORD_GENUPDATEMSG1A	391
#define CWORD_GENUPDATEMSG2	392
#define CWORD_GENUPDATEMSG3	393
#define CWORD_GENUPDATEMSG4	394
#define CWORD_GENUPDATEMSG5	395
#define CWORD_VERINVTYPEMSG	396
#define CWORD_S_FREECELLFOUND   397
#define CWORD_STRNULLMSG	398
#define CWORD_DIROPENERRMSG     399
#define CWORD_NOTCODEDYET	400
#define CWORD_S_GRIDFILLNOTPOSS 401
#define CWORD_S_GRIDERROR	402
#define CWORD_S_HOWGRIDFILL	403
#define CWORD_S_TEMPSEARCHDB    404
#define CWORD_S_NOTEMPMATCHDB   405
#define CWORD_S_RDBGENERICSUCII 406
#define CWORD_S_RDBGENERICERRII 407
#define CWORD_S_RDBNOTINDB	408
#define CWORD_S_RDBDUPLICDEFN	409
#define CWORD_S_RDBATTACHERR	410
#define CWORD_S_RDBXRDDELETED	411
#define CWORD_S_RDBGETGRDSUCC   412
#define CWORD_S_RDBGETGRDFAIL   413
#define CWORD_S_RDBADDGRDSUCC   414
#define CWORD_S_RDBADDGRDFAIL   415
#define CWORD_S_GRIDGETANSWER   416
#define CWORD_S_GRIDGOTANSWERS  417
#define CWORD_S_NOCLUEFORLIGHT  418
#define CWORD_S_GETCLUESFROMDB  419
#define CWORD_S_RDBGETCLUEDB    420
#define CWORD_S_RDBPUTCLUEDB    421
#define CWORD_S_RDBGETLIGHTSTR  422
#define CWORD_S_RDBPUTLIGHTSTR  423
#define CWORD_S_RDBPUTANSSTR    424
#define CWORD_S_RDBPUTANSERR    425
#define CWORD_S_RDBDUPDEFN	426
#define CWORD_S_NOTYETIMPLEMENT 427
#define CWORD_S_NOSEEDSDEFINED  428
#define CWORD_S_FILESEEDSTOGRID 429
#define CWORD_S_SEEDSFROMFILE   430
#define CWORD_S_SEEDNORESET     431
#define CWORD_S_PROMPTSEEDFILE  432
#define CWORD_S_SEEDFILENOTUSED 433
#define CWORD_S_SEEDFILEERR	434
#define CWORD_S_SEEDFILEDEFERR  435
#define CWORD_S_ADDINGSEEDMSG	436
#define CWORD_S_SEEDMISSED	437
#define CWORD_S_WRITESEEDTOFILE 438
#define CWORD_S_SEEDFILEWRITTEN 439
#define CWORD_S_SEEDFILEREAD    440
#define CWORD_S_GETCOMMSXRDMSG  441
#define CWORD_S_GETCOMMSGRDMSG  442
#define CWORD_S_PERCENTFILLED   443
#define CWORD_S_FILLTIMEWAS	444
#define CWORD_S_OVERWRITEFILE   445
#define CWORD_S_INPUTFILESPEC   446
#define CWORD_S_FILEINVALID     447
#define CWORD_S_FILEOVERWRITE   448
#define CWORD_S_RDBDELCLUEDB    449
#define CWORD_S_CLUESTARTTEXT   450
#define CWORD_S_CLUESALLDEFD	451
#define CWORD_S_CLUENUMBERMSG   452
#define CWORD_S_CLUEDIRECTMSG   453
#define CWORD_S_NULLCLUEENTERED 454
#define CWORD_S_ANSINCLUEMSG    455
#define CWORD_S_FIRSTCHARUPPER  456
#define CWORD_S_CLUEVALIDMSG	457
#define CWORD_S_NOUNDCLUESFND   458
#define CWORD_S_PROCESSINGCLUE	459
#define CWORD_S_NOCLUEYETUSRGEN 460
#define CWORD_S_CLUEQUITMSG     461
#define CWORD_S_CLUESAVEGENMSG  462
#define CWORD_S_SAVECLUESMSG	463
#define CWORD_S_NOTSAVECLUESMSG	464
#define CWORD_S_DISPLAYSOMECLUE 465
#define CWORD_S_DISPLAYALLCLUE  466
#define CWORD_S_REVIEWCLUESMSG  467
#define CWORD_S_ADDCLUEHEADMSG  468
#define CWORD_S_QUITCHANGESMADE 469
#define CWORD_S_QUITCONTINUE    470
#define CWORD_S_INVALIDCLUEINFO 471
#define CWORD_S_CLUEISBLANKS    472
#define CWORD_S_PROCESSCLUEMSG  473
#define CWORD_S_LIGHTCLUEMSG    474
#define CWORD_S_DEFAULTCLUEMSG  475
#define CWORD_S_CURRENTCLUEMSG  476
#define CWORD_S_INPUTCLUEMSG    477
#define CWORD_S_CLUELEFTALONE   478
#define CWORD_S_CLUENOTALLDEFN  479
#define CWORD_S_RESETCLUEMSG    480
#define CWORD_S_RESETNONETODO   481
#define CWORD_S_RESERCLUESUCC   482
#define CWORD_S_RESETCLUEISDEFN 483
#define CWORD_S_NOCLUESFOUNDMSG 484
#define CWORD_S_CLUECREATECOMM  485
#define CWORD_S_CLUELIGHTDIMS   486
#define CWORD_S_CLUEDIMSINPUT   487
#define CWORD_S_CLUEDIMSDEF     488
#define CWORD_S_CLUEDIMERR      489
#define CWORD_S_TOTALCLUEDESC   490
#define CWORD_S_DEFOPTCHOSEN    491
#define CWORD_S_CLUESNOTCOMP    492
#define CWORD_S_CLUEQUITUNSAVE  493
#define CWORD_S_GRIDFULLSPEC    494
#define CWORD_S_GRIDNOTFULLSPEC 495
#define CWORD_S_STATREPTCROSS   496
#define CWORD_S_STATREPTGRID    497
#define CWORD_S_STATDIMSARE     498
#define CWORD_S_GRIDCONNECT     499
#define CWORD_S_GRIDNOTCONNECT  500
#define CWORD_S_GRIDFOUNDMSG    501
#define CWORD_S_CHKUNCHKMSG     502
#define CWORD_S_CHKUNCHKBLKMSG  503
#define CWORD_S_GRDAFTERGRTURN  504
#define CWORD_S_XRDAFTERGRTURN  505
#define CWORD_S_TOTALLIGHTSMSG  506
#define CWORD_S_XRDFILESTATMSG  507
#define CWORD_S_GRDFILESTATMSG  508
#define CWORD_S_XRDFILEDETAILS  509
#define CWORD_S_GRDFILEDETAILS  510
#define CWORD_S_DETAILSFILEERR  511
#define CWORD_S_PRINTOFFSETSMSG 512
#define CWORD_S_PRINTPSOVERFLOW 513
#define CWORD_S_PRINTOVER80     514
#define CWORD_S_PRINTOVER132    515
#define CWORD_S_SYMMETRY        516
#define CWORD_S_SYSTEMCALLERR   517
#define CWORD_S_GRIDGETXRDNAME  518
#define CWORD_S_GRIDGETREMARKS  519
#define CWORD_S_TXTOFFSMALL     520
#define CWORD_S_TXTOFFLARGEX    521
#define CWORD_S_TXTOFFLARGEY    522
#define CWORD_S_TXTOFFNO        523
#define CWORD_S_COMLINENOTUNQ   524
#define CWORD_S_GRIDGENQUIT     525
#define CWORD_S_DELETINGUSING   526
#define CWORD_S_DELLIGHTLOC     527
#define CWORD_S_USERLHTNOTFOUND 528
#define CWORD_S_NODEFAULTERR    529
#define CWORD_S_NOLIGHTSDEFINED 530
#define CWORD_S_INVALIDLIGHT    531
#define CWORD_S_USERGRIDINVALID 532
#define CWORD_S_USERGRIDVALID   533
#define CWORD_S_RESETSINGLE     534
#define CWORD_S_RESETMULTIPLE   535
#define CWORD_S_NOLIGHTSNOSAVE  536
#define CWORD_S_QUITGRIDCREATE  537
#define CWORD_S_GRIDNOTASDEFD1  538
#define CWORD_S_GRIDNOTASDEFD2  539
#define CWORD_S_GRID		540
#define CWORD_S_AMERICANYDIMS	541
#define CWORD_S_XRDFILEINFO     542
#define CWORD_S_GRDFILEINFO     543
#define CWORD_S_LIGHTLENS       544
#define CWORD_S_LIGHTFREQS      545
#define CWORD_S_RESETSYMM       546
#define CWORD_S_CHANGEGRIDDEFN  547
#define CWORD_S_GRIDTYPECHANGED 548
#define CWORD_S_LGTALREADTPRES  549
#define CWORD_S_ZEROLENCLUE     550
#define CWORD_S_CHECKLETTERPROB 551
#define CWORD_S_MAXNUMOFCLUES   552
#define CWORD_S_UNCHESFOUND     553
#define CWORD_S_AMERICANBLANKS  554
#define CWORD_S_NOFREECELLFOUND 555
#define CWORD_S_SYMMNOTCORRECT  556
#define CWORD_S_SYMMCHANGE      557
#define CWORD_S_INVLDSYMMMSG    558
#define CWORD_S_MAINMENUCHOICE  559
#define CWORD_S_SUBMENUCHOICE0  560
#define CWORD_S_SUBMENUCHOICE1  561
#define CWORD_S_SUBMENUHEADER   562
#define CWORD_S_NONUMTRACEMSG   563
#define CWORD_S_MENUNUMTRACEMSG 564
#define CWORD_S_SUPERANAGRAMNOW 565
#define CWORD_S_SUPERANAGRAMSTR	566
#define CWORD_S_NOWORDFILEAVAIL 567
#define CWORD_S_GENERICSTR      568
#define CWORD_S_MINBIGGERMAX    569
#define CWORD_S_NOWORDFILERANGE 570
#define CWORD_S_SUBANAGRAMFOUND 571
#define CWORD_S_NOSOUNDEXFOUND  572
#define CWORD_S_SOUNDEX1MSG	573
#define CWORD_S_SOUNDEX2MSG	574
#define CWORD_S_WORDFILERATION  575
#define CWORD_S_FGETSPERRORMSG  576
#define CWORD_S_TRUNCATINGINPUT 577
#define CWORD_S_WORDFILEPERROR  578
#define CWORD_S_CREATETIMEWAS   579
#define CWORD_S_CREATOUTPUTCLUE 580
#define CWORD_S_WEBSTEROUT	581
#define CWORD_S_WEBQUERTFORMAT  582
#define CWORD_S_HIGHESTCLUES    583
#define CWORD_S_CLUESDEFINED    584
#define CWORD_S_GRIDCREATECOMM  585
#define CWORD_S_GRIDINPUTERR    586
#define CWORD_S_HELPAVAILABLE   587
#define CWORD_S_CREATEVERIFY    588
#define CWORD_S_CREATETIDY      589
#define CWORD_S_CREATERESET     590
#define CWORD_S_CREATELIST      591
#define CWORD_S_GENERICHELP     592
#define CWORD_S_CWORDCOMMANDS   593
#define CWORD_S_SEEDCREATECOMM  594
#define CWORD_S_SEEDDELETED     595
#define CWORD_S_SEEDFILLNOW     596
#define CWORD_S_SEEDQUITNOW     597
#define CWORD_S_SEEDVALIDCLUE   598
#define CWORD_S_SEEDINVALIDSEED 599
#define CWORD_S_SEEDSNONEAVAIL  600
#define CWORD_S_SEEDSLISTHEAD   601
#define CWORD_S_SEEDLISTSEEDS   602
#define CWORD_S_SEEDDELALLSEEDS 603
#define CWORD_S_SEEDVIEWGRID    604
#define CWORD_S_SEEDNOTSAVING   605
#define CWORD_S_SEEDSAVEQ       606
#define CWORD_S_SEEDSNONE	607
#define CWORD_S_SEEDDELETEFIRST 608
#define CWORD_S_SEEDFILENOTWRN  609
#define CWORD_S_MENUINPUT       610
#define CWORD_S_SEEDADDFIRST    611
#define CWORD_S_SEEDALSOGEN     612
#define CWORD_S_YESNOGENERIC    613
#define CWORD_S_SEEDFULLGRID    614
#define CWORD_S_DASHES          615
#define CWORD_S_SEEDOVERWRITE   616
#define CWORD_S_PRESSRETURNQUIT 617
#define CWORD_S_SINGLEWORDSTATS 618
#define CWORD_S_SINGLEWORDSTAT  619
#define CWORD_S_TOTALWORDSTAT   620
#define CWORD_S_GRANDWORDSTAT   621
#define CWORD_S_WORDFILESTATS   622
#define CWORD_S_BREAKUPDIC      623
#define CWORD_S_BREAKUPDICSPEC  624
#define CWORD_S_STRTOOLONGMSG   625
#define CWORD_S_ADDWORDTOFILE   626
#define CWORD_S_WORDTOFILESUC   627
#define CWORD_S_WORDTOFILEFAIL  628
#define CWORD_S_TEMPFILEPERROR  629
#define CWORD_S_NEWWORDFILEMSG  630
#define CWORD_S_DELETEFILEERR   631
#define CWORD_S_RENAMEFILEERR   632
#define CWORD_S_DELWORDFILEERR  633
#define CWORD_S_RENWORDFILEERR  634
#define CWORD_S_INPUTDEFCHOICE  635
#define CWORD_S_MODYDEFAULTMSG  636
#define CWORD_S_VIEWDEFAULTMSG  637
#define CWORD_S_PERRORSTR       638
#define CWORD_S_CHDIRMSG        639
#define CWORD_S_CHDIRSUCCESS    640
#define CWORD_S_CHDIRFAILURE    641
#define CWORD_S_TWINOUTPUTERR   642
#define CWORD_S_WORDFILEERR     643
#define CWORD_S_MENUTRACE       644
#define CWORD_S_RDBERRORTEXT    645
#define CWORD_S_RDBUNDEFINEDERR 646

/*
 *  When adding new message ID's please keep this constant upto date as it
 *  checks the allowable upper message bound value.
 */

#define LAST_MESSAGE		646



/*
 *  ?? When compiling on Visual C++ V1.5 remember to use the custom compiler
 *  option /Gt to get around a link error that would be generated by this large
 *  array of pointer to constant strings ??
 *
 *  ?? A link error is recorded when compiling with Borland C++ V3.1.  It
 *  concerns the _TEXT segment and is probably caused by statically defined
 *  data in the program being too big for the 64K default segment defined to
 *  store it.  I don't know how to overcome it at the moment ??
 *
 *  ?? To test on DOS/Windows without using this large array put "#if 0"'s
 *  around the message array and then alter the messages.c function accessing
 *  the array ??
 */

/* 
 *  ?? -S-, -I-, -W-, -E-, -F- ??
 */

/*
 *  I would normally place this array declaration in messages.c but it's more
 *  conveniently placed in here near where the constants which will index into
 *  this array are defined.  Usually, I dislike declaring memory in header
 *  files.
 */

#ifdef MESSAGES_DEFINED

static char *cword_messages_l_msgs[] =
{
    "",									/* NULL message */
    "CWORD-S-NORMAL, Normal Completion\n",				/* CWORD_NORMAL */
    "CWORD-E-ERRORS, General error found\n",				/* CWORD_ERROR */
    "CWORD-E-TRCERR, Error in trace file\n",				/* CWORD_NOTRACE */
    "CWORD-I-NEWMES, 1. \"%s\" 2. \"%s\" 3. \"%s\" 4. \"%s\"\n",	/* CWORD_TEST */
    "CWORD-W-NOGRID, No new grid created\n",				/* CWORD_NOGRIDCREATED */
    "CWORD-I_VERCON, Continue verifying crossword grid\n",		/* CWORD_CONTINUE */
    "CWORD-E-WRTERR, Write error\n",					/* CWORD_WRITEERROR */
    "CWORD-E-MRYERR, Memory allocation error\n",			/* CWORD_MEMORYERROR */
    "CWORD-E-ADDERR, Error adding word to link list\n",			/* CWORD_ADDWORDERROR */
    "CWORD-W-FRDBEF, Trying to de-allocate memory using NULL pointer\n",/* CWORD_S_DEALLOCNULL*/
    "CWORD-E-WRDBIG, Word inputted too large\n",			/* CWORD_WORDTOOBIG */
    "CWORD-I-NOFLEF, No files found\n",					/* CWORD_NOFILESFOUND */
    "CWORD-I-PARERR, Error parsing file name\n",			/* CWORD_PARSEERROR */
    "CWORD-E-INVALD, Invalid option chosen on menu\n",			/* CWORD_CHOICENOTVALID */
    "%s %s",							        /* CWORD_PRINTCOMMAND */
    "CWORD-E-DBERR, Rdb error\n",					/* CWORD_DBERROR */
    "CWORD-E-FLEERR, Error with file %s\n",				/* CWORD_FILEERROR */
    "CWORD-E-PRTERR, Error printing file %s\n",				/* CWORD_PRINTERROR */
    "\n\nCWORD-E-SERERR, Error with Webster server\n",			/* CWORD_SERVERERROR */
    "CWORD-E-WEBERR, Error parsing Webster output\n",			/* CWORD_WEBPARSEERROR */
    " Clue      Co-ords - Len",                                         /* CWORD_S_CREATOUTPUTHEAD */
    "CWORD-E-PARSML, Defined parameter in module %s too small\n",	/* CWORD_PARAMTOOSMALL */
    "CWORD-E-ANAERR, Too many sub-anagrams found - change default to increase\n",/* CWORD_ANAGRAMSTOOMANY */
    "CWORD-I-REPORT, Please report %s error to developer\n",            /* CWORD_S_REPORTPROBLEMS */
    "CWORD-E-NOANAG, No anagrams found for %s\n",			/* CWORD_NOANAGRAMS */
    "CWORD-I-ANAMSG, min length - %d and anagram - %s\n",		/* CWORD_ANAGRAMMSG */
    "The following %s are currently in the database:\n",                /* CWORD_S_DEFNSINDB */
    "CWORD-E-WRDDIC, Not possible to construct composite anagrams\n",	/* CWORD_ANAGRAMWARN */
    "CWORD-E-ANAERR, Incorrect paramters entered\n",			/* CWORD_ANAGRAMERR */
    "CWORD-E-WEBERR, Invalid character(s) entered in word - %s\n",	/* CWORD_WEBSTERERR */
    "Please input name of file to output [suffix %s] : ",		/* CWORD_OUTFILEMSG */
    "Please input name of file to print [suffix %s or %s] : ",		/* CWORD_PRINTFILEMSG */
    "CWORD-E-NOTFIL, File %s does not exist\n",				/* CWORD_S_FILENOTEXIST */
    "CWORD-I-PRTSUC, File %s successfully printed\n",			/* CWORD_PRINTFILESUCC */
    "CWORD-E-PRTFAL, File %s unsuccessfully printed\n",			/* CWORD_PRINTFILEFAIL */
    "\t\tSandman Software presents : %s\n",				/* CWORD_MAINBANNER */
    "CWORD-E-PRTFAL, Illegal file name entered: %s\n",			/* CWORD_ILLFILENAME */
    "CWORD-E-TIMERR, Error getting time - inserting default time\n",	/* CWORD_TIMEERROR */
    "delete/nolog/noconfirm %s;*",					/* CWORD_VMSDELETEFILE */
    "CWORD-I-PRNMSG, Printing file %s\n",			        /* CWORD_PRINTMSG */
    "CWORD-E-TRCNAM, Error no trace file name given\n",			/* CWORD_TRACEFILEERR */
    "CWORD-I-TRCOPN, Trace file %s opened on %s\n",			/* CWORD_TRACEFILEOPEN */
    "CWORD-I-TRCCLD, Trace file closed on %s\n",			/* CWORD_TRACEFILECLOSE */
    "CWORD-I-SLCTPS, Selecting output option - %s\n",			/* CWORD_SELECTPSTYPE */
    "CWORD-E-OUTERR, Incorrect output option chosen\n",			/* CWORD_PSTYPEERRPR */
    "CWORD-I-XRDOUT, Crossword file selected for outputting is %s\n", 	/* CWORD_XRDOUTFILE */
    "CWORD-I-PSTXTM, Writing %s part of crossword %s\n",		/* CWORD_PSTXTMSG */
    "CWORD-I-XRDRED, Crossword file %s read\n",				/* CWORD_XRDREAD */
    "CWORD-I-XRDWRT, Crossword file %s written\n",			/* CWORD_XRDWRITTEN */
    "CWORD-I-GRDRED, Grid file %s read\n",				/* CWORD_GRDREAD */
    "CWORD-I-GRDWRT, Grid file %s written\n",				/* CWORD_GRDWRITTEN */
    "CWORD-I-DEFRED, Configuration file %s read\n",			/* CWORD_DEFREAD */
    "CWORD-E-INVOPT, Invalid print option chosen\n",			/* CWORD_INVPRINTOPT */
    "CWORD-I-PSWRTN, Postscript file %s written\n",			/* CWORD_PSWRITTEN */
    "CWORD-I-TXTWRN, Text file %s written\n",				/* CWORD_TXTWRITTEN */
    "There %s %u unattached %s stored in the database\n",               /* CWORD_S_LOOSECLUESMSG */
    "CWORD-E-VERBAR, Invalid input entered\n",				/* CWORD_IVDINPUT */
    "Report on crossword definition [suffix %s] : ",			/* CWORD_GETSTATSXRDMSG */
    "CWORD-I-ISDBON, Database processing %s\n",                         /* CWORD_S_ISDBBUILD */
    "CWORD-I-WEBCON, Connecting to Webster server at IP address %s.",	/* CWORD_WEBCONNECT */
    "CWORD-I-WEBQRY, The query is - \"DEFINE %s\"\n",			/* CWORD_WEBQUERY */
    "\nCWORD-E-WEBERR, Webster error",                                  /* CWORD_WEBERR */
    "CWORD-E-WEBLEN, Length of return string from Webster %d\n",	/* CWORD_WEBOUTLEN */
    "CWORD-I-WEBCON, Disconnected from Webster\n",			/* CWORD_WEBDISCONNECT */
    ".",								/* CWORD_WAITING */
    "\n",								/* CWORD_WAITINGENDED */
    "CWORD-I-WEBEPY, No output received from Webster\n",		/* CWORD_WEBNOUTPUT */
    "CWORD-I-RDBODB, Opening database - please wait\n",			/* CWORD_RDBOPENDB */
    "CWORD-E-RDBDLK, Database deadlock error\n",			/* CWORD_RDBDEADLOCK */
    "CWORD-E-RDBCFT, Database conflict error\n",			/* CWORD_RDBCONFLICT */
    "CWORD-E-RDBUND, Database undefined error - error is: %s\n",	/* CWORD_RDBUNDEFERR */
    "CWORD-E-CLUIDB, Clue already in database\n",                       /* CWORD_S_CLUEALREADYINDB */
    "CWORD-I-RDBSUC, %s was successful\n",				/* CWORD_RDBGENERICSUC */
    "CWORD-E-RDBFAL, %s was unsuccessful\n",				/* CWORD_RDBGENERICERR */
    "CWORD-W-RDBNOX, No transaction outstanding\n",			/* CWORD_RDBNOTRANX */
    "CWORD-I-BNKMSG, %s\n",						/* CWORD_BLANKMSG */
    "CWORD-E-RDBEOS, No data or end of cursor found\n",	                /* CWORD_S_RDBENDOFSTREAM */
    "CWORD-I-DBSUCC, %s \"%s\" added successfully to database\n",       /* CWORD_RDBADDSUCC */
    "CWORD-E-DBXWFL, Error adding %s \"%s\" to database\n",             /* CWORD_RDBADDFAIL */
    "CWORD-I-DBXRSC, %s \"%s\" read successfully from database\n",      /* CWORD_RDBREADSUCC */
    "CWORD-E-DBXRFL, Error reading %s \"%s\" from database\n",	        /* CWORD_RDBREADFAIL */
    "CWORD-I-DBXDSC, %s \"%s\" deleted successfully from database\n",   /* CWORD_RDBDELSUCC */
    "CWORD-E-DBXDFL, Error deleting %s \"%s\" from database\n",	        /* CWORD_S_RDBDELFAIL */
    "CWORD-W-CLIGNR, All other non relevant command line options will be ignored\n",/* CWORD_S_COMLINIGNORE */
    "CWORD-W-MAXMIN, All command line parameters found will now be discarded\n",/* CWORD_S_COMLINEMAXMIN */
    "CWORD-E-DBIVTX, Invalid transaction state\n",			/* CWORD_RDBINVTRANX */
    "CWORD-E-DBBDTX, Bad Transaction error\n",				/* CWORD_RDBBADTRANX */
    "CWORD-E-DBIVCO, Constraint out of range error\n",			/* CWORD_RDBINXCONSTR */
    "CWORD-E-DBIVIN, Invalid index error \n",				/* CWORD_RDBINVINDEX */
    "CWORD-E-DBEROR, Generic rdb error: SQL_RDBERR\n",		        /* CWORD_RDBERR	*/
    "CWORD-E-DBOUTR, Out of range error\n",				/* CWORD_RDBOUTOFRANGE */
    "CWORD-I-DBADCR, Added crossword_description successfully\n",	/* CWORD_RDBADDCROSSSUC */
    "CWORD-I-DBADCL, Added clue_descriptions successfully\n",		/* CWORD_RDBADDCLUESUC */
    "CWORD-I-DBADGR, Added grid_descriptions successfully\n",		/* CWORD_RDBADDGRIDSUC */
    "CWORD-E-DBADCR, Error adding crossword_description\n",		/* CWORD_RDBADDCROSSERR */
    "CWORD-E-DBADCL, Error adding clue_description for %d %s\n",	/* CWORD_RDBADDCLUEERR */
    "CWORD-E-DBADGR, Error adding grid_description \n",			/* CWORD_RDBADDGRIDERR */
    "CWORD-I-DBADAN, Added \"%s\" to anagram_desc successfully\n",	/* CWORD_RDBADDANASUC */
    "CWORD-I-DBADHO, Added \"%s\" to homophone_desc successfully\n",	/* CWORD_RDBADDHOMSUC */
    "CWORD-I-DBADSY, Added \"%s\" to symonym_desc successfully\n",	/* CWORD_RDBADDSYNSUC */
    "CWORD-E-DBADAN, Error adding \"%s\" to anagram_description\n",	/* CWORD_RDBADDANAERR */
    "CWORD-E-DBADHO, Error adding \"%s\" to homophone_description\n",	/* CWORD_RDBADDHOMERR */
    "CWORD-E-DBADSY, Error adding \"%s\" to symonym_description\n",	/* CWORD_RDBADDSYNERR */
    "CWORD-E-DBADSY, Getting anagrams from the database\n",		/* CWORD_RDBGETANADESC */
    "CWORD-E-DBADSY, Getting homophones from the database\n",		/* CWORD_RDBGETHOMODESC */
    "CWORD-E-DBADSY, Getting synonyms from the database\n",		/* CWORD_RDBGETSYMDESC */
    "\n\nCWORD-F-SIGNAL, Signal generated\n",				/* CWORD_SIGNALGENERATED */
    "CWORD-F-GRDEND, User finished creating grid definition\n",		/* CWORD_S_GRIDGENFINISHED*/
    "Please input name of file to save grid definition to [%s%s] : ",	/* CWORD_GRDFILEWRITE */
    "CWORD-I-GRDLST, Grid files suitable for processing : \n",	        /* CWORD_GRDFILELISTYES */
    "CWORD-I-GRDNOF, There is no file suitable for filling\n",		/* CWORD_GRDFILELISTNO */
    "CWORD-I-GRDEND, Grid creation phase finished\n",			/* CWORD_SAVEGRIDEND */
    "What is the grid to be called [%s] : ",				/* CWORD_GRIDCALLED */
    "Error found in the creation of the grid - Write grid (%c/%c) [%c]: ",/* CWORD_GRIDENDERR */
    "CWORD-I-GRDNON, No grid file created\n",				/* CWORD_GRIDNONECREATED */
    "CWORD-I-GRDBGN, Starting grid creation procedure\n",               /* CWORD_GRIDSTARTPHASE */
    "CWORD-I-GRDVLD, Valid grid definition found\n",			/* CWORD_GRIDVALID */
    "Finished defining the grid yet (%c/%c) [%c] : ",			/* CWORD_GRIDENDYET */
    "CWORD-E-INVANS, The answer to clue %d %s is invalid\n",	        /* CWORD_VERINVANSNUM */
    "CWORD-E-ANSLOC, Location on grid of light %d %s is incorrect\n",	/* CWORD_VERINVANSLOC */
    "CWORD-E-INVSYM, Crossword has failed the grid symmetry check\n",	/* CWORD_VERINVSYMMETRY */
    "CWORD-E-IVANGR, Answers are incorrectly placed on grid\n",		/* CWORD_VERINVANSONGRID */
    "CWORD-E-ADJYNS, Adjacent unches at grid locations %d-%d and %d-%d\n",/* CWORD_VERADJACUNCHES */
    "CWORD-I-SYMDIM, %c and %c dimensions uneven - symmetry warning\n", /* CWORD_VERINVDIMSYM */
    "CWORD-I-ERRSYM, Error - no symmetry defined, grid not checked\n",	/* CWORD_VERNOSYMDEF */
    "CWORD-I-ERRSYM, Grid failed symmetry test\n",			/* CWORD_VERSYMMFAIL */
    "CWORD-I-ERRBLK, Error in %s at cell co-ordinate %d-%d\n",          /* CWORD_VERBLOCKPROP */
    "Input grid file to fill with answers [suffix %s] : ",              /* CWORD_FILLXWORD */
    "Input name of file to save crossword definition to [%s%s] : ",     /* CWORD_XWORDFILEWRITE */
    "Input crossword definition for clue generation [suffix %s] : ",    /* CWORD_XWORDCLUEGEN */
    "CWORD-E-WEBTMH, Too much data from Webster server - truncated\n",	/* CWORD_WEBTOOMUCHDATA */
    "CWORD-E-INTERR, Internal error - Message reference outside range\n",/* CWORD_LASTMESSAGEERR */
    "CWORD-I-LGTSTR, Light generation beginning on grid \"%s\"\n",	/* CWORD_GRIDMAKESTART */
    "CWORD-I-LGTEND, Light generation completed\n",			/* CWORD_GRIDFINISHED */
    "CWORD-E-COMLIN, Invalid input found on command line\n",            /* CWORD_S_COMLINEINVD */
    "CWORD-E-WRDELM, %u %s dealt with in outputting function\n",        /* CWORD_OUTPUTELEMS */
    "CWORD-E-ERRUND, Undefined error has occurred\n",			/* CWORD_ERRUNDEFINED */
    "CWORD-I-COMPLT, Complete empty crossword \"%s\" with temp values\n",/* CWORD_NEWSVALSINXWORD */
    "CWORD-W-MATNOS, No string entered\n",				/* CWORD_MATCHNOSTRING */
    "CWORD-E-MATMIN, Length greater than strlen of input string\n",	/* CWORD_MATCHMINLEN */
    "CWORD-I-TARGET, No centre letter : anagram size > %d : letters \"%s\"\n",/* CWORD_MATCHNOTARGETSTAT */
    "CWORD-I-TARGET, Centre letter \"%c\" : anagram size > %d : letters \"%s\"\n",/* CWORD_MATCHTARGETSTAT */
    "CWORD-E-NOTFND, File %s not found\n",                              /* CWORD_S_FILENOTFOUNDERR */
    "CWORD-E-NOWORD, Word file %s not available\n",			/* CWORD_NOWORDFILE */
    "CWORD-W-NOXRDD, No %s currently in the database\n",                /* CWORD_S_NODEFNSINDB */
    "CWORD-E-WRDERR, Error reading word file\n",			/* CWORD_WORDREADERR */
    "Input name of %s to be deleted from database : ",                  /* CWORD_S_DELFROMDB */
    "CWORD-I-WORDFL, Word file length %d chosen\n",			/* CWORD_WORDFILEOPT */
    "CWORD-I-GENSUB, Generating sub-anagrams using \"%s\"\n",		/* CWORD_MATCHSUBANAGRAMS */
    "CWORD-I-SUBFND, %d sub or full anagrams found\n",			/* CWORD_SUBANAGRAMSFOUND */
    "CWORD-E-INVCHR, Invalid character(s) in string \"%s\"\n",          /* CWORD_INVALIDCHAR */
    "CWORD-E-INVIPT, Invalid command \"%s\" entered\n",                 /* CWORD_S_INVALIDINPUT */
    "Reading word file.",						/* CWORD_READINGWORDFILE */
    "CWORD-I-NOINTS, No intersects found for \"%s\" and \"%s\"\n",	/* CWORD_NOINTERSECTS */
    "CWORD-I-SETDEF, Initing configuration defaults\n",                 /* CWORD_DEFAULTSDEFINE */
    "CWORD-E-QMARKS, One or other string does not contain \'?\' character\n",/* CWORD_NUMQMARKSERR */
    "CWORD-E-MMARKS, More than one question mark in string \"%s\"\n",	/* CWORD_TOOMANYQMARKSERR */
    "CWORD-E-TOOLNG, String \"%s\" is too long\n",			/* CWORD_STRINGTOOLONG */
    "CWORD-I-INTERS, Intersect \"%s\" with \"%s\"\n",			/* CWORD_INTERSECTWORDS */
    "Matching taking place.",						/* CWORD_MATCHNOW */
    "Getting sub-anagrams.",						/* CWORD_SUBANAGRAMSNOW */
    "Getting strstart words.",						/* CWORD_STRSTARTNOW */
    "Getting strmiddle words.",						/* CWORD_STRMIDDLENOW */
    "Getting strend words.",						/* CWORD_STRENDNOW */
    "Getting strwild words.",						/* CWORD_STRWILDNOW */
    "Getting soundex words.",						/* CWORD_SOUNDEXNOW */
    "CWORD-E-CRTENF, Centre letter \'%c\' not found in target string\n",/* CWORD_CENTRENOTFOUND */
    "CWORD-E-MINTOB, Size %d is greater than input string \"%s\"\n",	/* CWORD_MINSIZETOOBIG */
    "CWORD-I-TMPWRD, Template search using \"%s\" in word file\n",	/* CWORD_TEMPLATESEARCH */ 
    "CWORD-I-FINISH, Exiting application normally - Call again soon\n",	/* CWORD_FINISHMSG */
    "Crossword Files suitable for processing : \n",                     /* CWORD_XRDFILELISTYES */
    "CWORD-E-CHONLY, Centre is only allowed to be one character\n",	/* CWORD_S_COMLINECENONLY */
    "CWORD-E-NOWORD, No word files found for prefix \"%s\"\n",          /* CWORD_S_NOWORDFILEFND */
    "The Rdb functionality has not been ported from OpenVMS *yet*!\n",	/* CWORD_DBNOTPORTABLE */
    "Please input an anagram indicator : ",				/* CWORD_ANAGRAMINPUT */
    "List of anagram indicators include :\n",				/* CWORD_ANAGRAMLIST */
    "Please input a homophone indicator : ",				/* CWORD_HOMOPHONEINPUTIN */
    "Please input a homophone description : ",				/* CWORD_HOMOPHONEINPUTDE */
    "List of homophone indicators include :\n",				/* CWORD_HOMPHONELIST */
    "Please input a synonym indicator : ",				/* CWORD_SYNONYMINPUTIN */
    "Please input a synonym description : ",				/* CWORD_SYNONYMINPUTDE */
    "List of synonym indicators include :\n",				/* CWORD_SYNONYMLIST */
    "Please input string to search for : ",				/* CWORD_INPUTSTRQ */
    "Please input word to be spell checked : ",				/* CWORD_SPELLCHECKSTR */
    "CWORD-I-SPLSUC, The word \"%s\" is found in word files\n",		/* CWORD_SPELLSUCCESS */
    "CWORD-E-SPLERR, The word \"%s\" is *not* found in word files\n",	/* CWORD_SPELLFAILURE */
    "Please input string template to search for : ",			/* CWORD_TEMPLATESTR */
    "Please input min expected length of returned anagrams : ",         /* CWORD_ANAGRAMMINSTR */
    "Please input string from which to obtain anagrams of : ",		/* CWORD_ANAGRAMGETSTR */
    "Please input letter from string which will be in all anagrams [none] : ",/* CWORD_ANAGRAMALWAYSSTR */
    "CWORD-I-ANAALL, Anagrams returned will have all letters in search word\n", /* CWORD_ANAGRAMALL */
    "Please input string which to generate soundex equivalents of : ",  /* CWORD_SOUNDEXSTR */
    "Please input word to ROT13 decode : ",				/* CWORD_ROTSTR */
    "Please find the Webster's definition of which word : ",		/* CWORD_WEBSTERSTR */
    "Intersect: Input two words with the character ? where they cross\n", /* CWORD_INTERSECTEXPLAIN */
    "Please input intersect string 1 : ",				/* CWORD_INTERSECTSTR1 */
    "Please input intersect string 2 : ",				/* CWORD_INTERSECTSTR2 */
    "List of intersects include :\n",					/* CWORD_INTERSECTLIST */
    "\nPlease input user choice : ",					/* CWORD_INPUTCHOICE */
    "CWORD-E-INVCHC, Invalid selection entered\n",                      /* CWORD_S_INVALIDCHOICE */
    "Please press return to continue",					/* CWORD_S_PRESSRETURN */
    "CWORD-I-NOMCHS, No valid matches found\n",				/* CWORD_NOMATCHESFOUND */
    "CWORD-I-TOOLRG, length input greater than max length %d\n",	/* CWORD_LENGREATERMAX */
    "CWORD-E-NOTEMP, Error obtaining template match data\n",		/* CWORD_TEMPLATEFAILMSG */
    "CWORD-E-IVDNUM, Invalid character in number \"%s\"\n",		/* CWORD_IVDDIGIT*/
    "CWORD-I-SOUNDX, Getting soundex equivalents of \"%s\" (same starting letter)\n", /* CWORD_SOUNDEXSAME */
    "CWORD-I-SOUNDX, Getting soundex equivalents of \"%s\"\n",		/* CWORD_SOUNDEXDIFF */
    "CWORD-I-NOSUBA, No sub-anagrams found\n",				/* CWORD_NOSUBANAGRAMS */
    "CWORD-I-NOTEMP, No template matches found in word file\n",		/* CWORD_NOTEMPLATEMATCH */
    "List of words returned include :\n",				/* CWORD_RETURNEDWORDS */
    "CWORD-E-UNSUPT, Unsupported command line parameter: \"%s\" \n",    /* CWORD_S_COMLINEUNSUPT */
    "CWORD-I-NUMCHS, %u %s found\n",                                    /* CWORD_CHOICESFOUND */
    "CWORD-I-SRTSTR, Find words starting with \"%s\"\n",		/* CWORD_STRSTARTSTR */
    "CWORD-I-MIDSTR, Find words with \"%s\" in the middle\n",		/* CWORD_STRMIDDLESTR */
    "CWORD-I-ENDSTR, Find words ending in \"%s\"\n",			/* CWORD_STRENDSTR */
    "CWORD-I-WLDSTR, Find word matching \"%s\"\n",			/* CWORD_STRWILDSTR */
    "CWORD-I-NOANAF, No Composite anagrams found\n",			/* CWORD_NOCOMPANAGRAMS */
    "CWORD-I-ANAFND, %d composite anagrams found\n",			/* CWORD_COMPANAGRAMS */
    "CWORD-I-WEBSTR, The word to search for in Websters is \"%s\"\n",	/* CWORD_WEBSTERWORD */
    "CWORD-I-ROTSTR, The ROT'ed string : \"%s\"\n",                     /* CWORD_ROTSTROUT */
    "CWORD-I-TRCERR, Trace file not opened successfully\n",		/* CWORD_TRACENOTOPEN */
    "CWORD-I-CLOCKB, CPU Clock before search is %ld\n",			/* CWORD_CLOCKBEFORE */
    "CWORD-I-CLOCKA, CPU Clock after search is %ld\n",			/* CWORD_CLOCKAFTER */
    "List of composite anagrams include :\n",				/* CWORD_COMPANAGRAMLIST */
    "List of soundex matches include :\n",				/* CWORD_SOUNDEXLIST */
    "CWORD-I-TRCWRE, Error writing to trace file - exiting\n",		/* CWORD_TRACEWRITEERR */
    "CWORD-I-SWRDIS, Selected word for %d %s from generator is \"%s\"\n",/* CWORD_SELECTEDWORD */
    "CWORD-I-SEED+, Light \"%s\" seeded at %d %s\n",		        /* CWORD_S_SEEDADDED */
    "CWORD-I-DISALL, Adding \"%s\" to all lights disallowed list\n",	/* CWORD_ADDDISALLOWED */
    "CWORD-I-DISONE, Adding \"%s\" to %d %s's light disallowed list\n",	/* CWORD_ADDONEDISALLOWED */
    "CWORD-I-MEMDEL, Deleting \"%s\" from grid matches link list\n",    /* CWORD_MEMMANDELWORD */
    "CWORD-I-ITRNUM, Started iteration %d of light generation phase\n", /* CWORD_ITERATIONNO */
    "CWORD-I-ITRFIN, Finished iteration %d of light generation phase\n",/* CWORD_ITERATIONFINISHED */
    "CWORD-I-CHOICS, %d possible matches found in word list\n",		/* CWORD_POSSIBLECHOICES */
    "CWORD-I-PRUNEL, Prune matches list to manageable size\n",		/* CWORD_PRUNELIST */
    "CWORD-I-NOPRUN, Too few matches found to prune list any further\n",/* CWORD_NOPRUNELIST */
    "CWORD-I-PRNRAN, Pruning matching list using randomly selected words\n",/* CWORD_PRUNERANDOM */
    "CWORD-I-PRNWEI, Pruning matches list using weighted selected word\n",/* CWORD_PRUNEWEIGHT */
    "CWORD-I-RULE01, Insert \'%c\' as 1st letter of otherwise blank search word\n",/* CWORD_RULE1REDUCESEARCH */
    "CWORD-I-PICKWD, Picking word for %d %s with template: \"%s\"\n",   /* CWORD_PICKWORD */
    "The following lights can be seeded :\n",		                /* CWORD_CLUESFORLIGHTING */
    "CWORD-I-FILLIN, Computer generation of lights now commencing\n",	/* CWORD_FILLINGNOW */
    "CWORD-W-LIGTNF, Light %d %s not found in grid definition\n",	/* CWORD_LIGHTNOTFOUND */
    "CWORD-I-CHOICS, 1 possible match found in word list\n",		/* CWORD_POSSIBLECHOICE */
    "CWORD-I-CRTOGD, Translating crossword details to grid details\n",	/* CWORD_CROSSTOGRID */
    "CWORD-I-SEEDGD, Seed lights in grid before computer generation\n", /* CWORD_SEEDLIGHTS */
    "CWORD-I-ADSDGR, Added seed lights to blank grid before generation\n",/* CWORD_ADDSEEDSTOGRID */
    "CWORD-I-UPSDGR, Updating grid with generated light\n",	    	/* CWORD_UPDATETOGRID */
    "CWORD-I-OUTGRD, Outputting updated grid\n",			/* CWORD_OUTPUTGRID */
    "CWORD-I-NXTLHT, About to find next light for unfilled grid\n",     /* CWORD_FINDNEXTLIGHT */
    "CWORD-I-RNDLST, Obtained random selection of matches from list\n",	/* CWORD_FOUNDRANDOMLIST */
    "CWORD-I-WHTLST, Obtained weighed list of matches from list\n",	/* CWORD_WEIGHTRANDOMLIST */
    "CWORD-I-FILLED, Grid finished - ",					/* CWORD_ISGRIDFILLED */
    "yes\n",								/* CWORD_YESRETURN */
    "no\n",								/* CWORD_NORETURN */
    "CWORD-I-CHKSRH, Need to search word file for \"%s\" - ",		/* CWORD_CHECKSEARCHAGAIN */
    "CWORD-I-BCKTRK, Backtrack grid by deleting %d %s - \"%s\"\n",      /* CWORD_BACKTRACKGRID */
    "CWORD-I-GDTOCR, Translating grid details into crossword details\n",/* CWORD_GRIDTOCROSS */
    "CWORD-I-DELDUP, Deleting disallowed matches before continuing\n",  /* CWORD_DELETEDISALLOW */
    "CWORD-W-DEDEND, Deadend alert\n",					/* CWORD_DEADENDALERT */
    "CWORD-W-SEEDDL, About to try to delete seed from grid\n",          /* CWORD_DELETESEED */
    "CWORD-E-GIVEUP, Cannot fill grid and thus am giving up :-(\n",     /* CWORD_GRIDGIVEUP */
    "CWORD-E-SEDLEN, Light of incorrect length input\n",		/* CWORD_SEEDLENGTHERR */
    "CWORD-W-MCHDON, Match unsuccessfully completed on \"%s\" previously\n",/* CWORD_MATCHDONEFAIL */
    "Clue number which to seed (%d-%d) [no default] : ",                /* CWORD_LIGHTNUMBER */
    "Clue direction (%c/%c) [%c] : ",					/* CWORD_LIGHTDIRECTION */
    "CWORD-I-DEFDIR, %s direction chosen for light\n",			/* CWORD_LIGHTDIRCHOSEN */
    "CWORD-W-VLDNUM, Please input valid clue number to continue\n",     /* CWORD_INPUTVDLIGHTNUM */
    "CWORD-W-INVDNM, No clue found for either %d %s or %d %s\n",	/* CWORD_INVALIDLIGHTNUM */
    "CWORD-I-LIKTHS, User input light has to match template: \"%s\"\n", /* CWORD_LIGHTMATCHTHIS */
    "CWORD-I-GRDOUT, Grid file selected for outputting is %s\n",	/* CWORD_GRDOUTFILE */
    "Report on grid definition [suffix %s] : ",				/* CWORD_GETSTATSGRDMSG */
    "CWORD-I-ADDFAL, Adding \"%s\" to template failures list\n",	/* CWORD_ADDTOFAILURES */
    "CWORD-E-REDERR, Error parsing file %s properly\n",			/* CWORD_PARSEFILEERROR */
    "CWORD-E-DIMBIG, Dimensions in file %s too large for data structures\n",/* CWORD_FILEDIMSTOOBIG */
    "CWORD-E-WRTERR, Error writing to file %s\n",			/* CWORD_FILEWRITEERROR */
    "CWORD-E-REDERR, Error reading from file %s\n",			/* CWORD_FILEREADERROR */
    "90 degree",			     		                /* CWORD_90SYMMETRYMSG */
    "180 degree",					                /* CWORD_180SYMMETRYMSG */
    "axial - (top Left to bottom Right)",			        /* CWORD_AXIALLEFTRIGHTMSG */
    "axial - (top Right to bottom Left)",			        /* CWORD_AXIALRIGHTLEFTMSG */
    "no",							        /* CWORD_NOSYMMETRYMSG */ 
    "1. ",								/* CWORD_01MSG */
    "2. ",								/* CWORD_02MSG */
    "3. ",								/* CWORD_03MSG */
    "4. ",								/* CWORD_04MSG */
    "5. ",							    	/* CWORD_05MSG */
    "6. ",								/* CWORD_06MSG */
    "7. ",								/* CWORD_07MSG */
    "8. ",								/* CWORD_08MSG */
    "9. ",								/* CWORD_09MSG */
    "10. ",								/* CWORD_10MSG */
    "CWORD-I-ADDLIG, Adding light - loc: (%d,%d) dir: %s len: %d\n",	/* CWORD_ADDLIGHTLOC */
    "Current cell/grid details - blank (%c), unch (%c), checked (%c)\n",/* CWORD_LIGHTGRIDMSG */
    "Using : %s symmetry\n",						/* CWORD_USING */
    "CWORD-I-ADDING, Adding lights using : %s symmetry\n",	        /* CWORD_ADDINGUSING */
    "CWORD-I-CHKLIG, Checking validity of light - loc: (%d,%d) dir: %s len: %d\n",/* CWORD_CHECKLIGHTVALID */
    "CWORD-E-LIGTRE, Light already defined for location (%d,%d)\n",	/* CWORD_LIGHTALREADYTHERE */
    "CWORD-E-DBLLIG, Multiple lights defined at location (%d,%d)\n",	/* CWORD_LIGHTSSAMEDIR */
    "CWORD-I-MIDLIG, Light cuts centre of grid - treat accordingly\n",	/* CWORD_CENTRELIGHT */
    "\nPlease input grid dimensions to be used in construction of grid :\n",/* CWORD_INPUTGRIDDIMQ */
    "%c (%d to %d) [%d] : ",						/* CWORD_INPUTGRIDX */
    "%c (%d to %d) [%d] : ",						/* CWORD_INPUTGRIDY */
    "CWORD-W-MAXDIM, Maximum grid dimensions is currently %d\n",	/* CWORD_MAXGRIDDIMS */
    "CWORD-W-MINDIM, Minimum grid dimensions is currently %d\n",	/* CWORD_MINGRIDDIMS */
    "CWORD-I-DIMSET, %c dimensions of grid set to %u\n",                /* CWORD_S_DIMSSETTO */
    "Input name of %s to be read from database : ",                     /* CWORD_S_READFROMDBMSG */
    "CWORD-W-X!=Y, Warning: Undefined behaviour can result when %c != %c\n",/* CWORD_XNOTEQUALY */
    "Select the grid type from the list of choices :\n",                /* CWORD_INPUTGRIDTYPEMSG */
    "CWORD-I-GRDPKD, %s grid format chosen\n",				/* CWORD_GRIDTYPECHOSEN */
    "\nSelect the grid symmetry from the list of choices :\n",		/* CWORD_INPUTSYMMTYPEMSG */
    "\nPlease input selected grid symmetry [%d] : ",			/* CWORD_INPUTSYMMTYPE */
    "CWORD-I-SYMCSN, Grid symmetry chosen : %s symmetry\n",		/* CWORD_SYMMTYPECHOSEN	*/
    "CWORD-E-DIMXER, Invalid - %c or %c equals 0\n",                    /* CWORD_S_DIMZERO */
    "CWORD-E-XORYOR, %c greater than allowed dimension %d\n",           /* CWORD_XORYOVERALLOWED */
    "CWORD-E-LGHTTS, Length of defined light smaller than allowed %d\n",/* CWORD_LIGHTTOOSMALL */
    "CWORD-E-XOVERS, Light defined at (%d,%d) overshoots %c maximum\n", /* CWORD_S_OVERSHOOTMAX */
    "CWORD-E-YUNDRS, Light defined at (%d,%d) undershoots %c minimum\n",/* CWORD_S_UNDERSHOOTMIN */
    "Input %c co-ordinate of light to be created (1-%u) : ",            /* CWORD_GETLIGHT */
    "Input file spec to save %s to [suffix %s] : ",                     /* CWORD_S_DBTOFILEMSG */
    "Input the proposed light direction (%c/%c) : ",	                /* CWORD_GETLIGHTDIR */
    "Input the proposed light length (1-%d) : ",			/* CWORD_GETLIGHTLEN */
    "CWORD-E-LHTZRO, Light definition contains invalid value - please retry\n",/* CWORD_LIGHTVALUEZERO */
    "CWORD-I-LHTALW, Allowed grid location - loc: (%d,%d) dir: %s len: %d\n",/* CWORD_VALIDLIGHTFOUND */
    "CWORD-E-LHTDIS, Disallowed grid location - loc: (%d,%d) dir: %s len: %d\n",/* CWORD_INVALIDLIGHTFOUND */
    "\nYou will create a %d by %d %s grid using %s symmetry\n\n",       /* CWORD_GRIDDEFINITION */
    "%s",								/* CWORD_USERDEFINEDSTRING */
    "CWORD-I-SYSTEM, Calling OS using \"%s\" - on your own from here\n",/* CWORD_SYSTEMCALLMSG */
    "CWORD-I-SYSBCK, Back from the wilds of system () safely\n",        /* CWORD_SYSTEMCALLRETURN */
    "CWORD-I-BESTWD, Best word chosen indexed (%d of %d) weight %f\n",	/* CWORD_BESTWORDS */
    "CWORD-I-RANDWD, Random word chosen indexed (%d of %d)\n",		/* CWORD_RANDOMWORDS */
    "CWORD-I-CHANGE, Changing selection mechanism for \"%s\" to random\n",/* CWORD_BESTTORANDOM */
    "CWORD-I-GENNUM, Generating clue numbers for grid\n",		/* CWORD_GENCLUENUMS */
    "Input seed light (%d letters long) : ",                            /* CWORD_SEEDLIGHTMSG */
    "Input %s file to write to database [suffix %s] : ",                /* CWORD_S_WRITETODB */
    "CWORD-W-SEEDSP, Seed spelt incorrectly - Continue (%c/%c) [%c] : ",/* CWORD_SEEDSPELLERR */
    "CWORD-W-SPLOUT, User seed not accepted\n",				/* CWORD_SEEDSPELLOUT */
    "CWORD-E-SEDCEL, Error at cell (%d,%d) on a %s clue\n",		/* CWORD_SEEDCELLLOCERR */
    "CWORD-E-LGTGRD, Light already in grid definition - not allowed\n", /* CWORD_LIGHTALREADYINGRD */
    "List of lights obtained from template matcher for selection:\n",   /* CWORD_LIGHTSLIST */
    "CWORD-W-NOLGTS, No lights available for selection\n",		/* CWORD_NOLIGHTSAVAIL */
    "\nPlease wait while polling the database...\n",                    /* CWORD_S_DBSTATSMSG */
    "CWORD-W-TPLTUS, Template \"%s\" unsuccessfully used previously\n", /* CWORD_TPLATEUSEDBEFRE */
    "Accept the generated light \"%s\" (%c/%c) [%c] : ",		/* CWORD_LIGHTACCEPTQ */
    "CWORD-I-MORLGT, Getting more matches from the well\n",		/* CWORD_GETTINGMORELIGHTS */
    "CWORD-W-DELSED, Found a suitable seed %d %s for deletion to continue\n",/* CWORD_DELETESEEDFOUND */
    "CWORD-E-DEDEND, Deadend found during grid generation - cannot continue\n",/* CWORD_DEADENDMSG */
    " %*u %6.6s (%c)",                                                  /* CWORD_GRIDOUTPUTCLUE */
    "CWORD-W-NOCHSC, No possible matches found in word list\n",		/* CWORD_NOPOSSIBLECHOICES */
    "CWORD-W-OVRGEN, Generating %d %s will also generate %d %s\n",	/* CWORD_OVERGENERATEMSG */
    "CWORD-W-LGHTSP, Light \"%s\" spelt incorrectly - Continue (%c/%c) [%c] : ",/* CWORD_LIGHTSPELLERR */
    "CWORD-I-ADDGEN, Added extra light \"%s\" to %d %s\n",		/* CWORD_OVERGENLIGHTADD */
    "CWORD-W-DELGEN, Extra light \"%s\" not accepted in location %d %s\n",/* CWORD_OVERGENLIGHTDEL */
    "CWORD-E-STDOUT, Error writing to standard output\n",		/* CWORD_STDOUTERR */
    "CWORD-I-DBADCL, Added light_descriptions successfully\n",          /* CWORD_S_RDBADDLIGHTSUC */
    "There %s %u %s in the database\n",                                 /* CWORD_S_DEFNSINDNMSG */
    "CWORD-W-WRDEXT, Word file %s not found\n",        	    	        /* CWORD_WORDNOTEXIST */
    "CWORD-E-CHKWRD, Valid word files *not* found - check configuration\n",/* CWORD_CHECKWORDDEFAULT */
    "CWORD-E-ADJACT, New light adjacent to old light in %s direction\n",/* CWORD_GENADJACENT */
    "CWORD-W-ADJWRN, Creating an internal adjacency\n",                 /* CWORD_GENADJACWARN */
    "Please input selected grid type [%d] : ",                          /* CWORD_INPUTGRIDTYPE */
    "CWORD-E-VIOLAT, Cannot accept new lights - violates %s symmetry\n",/* CWORD_GENVIOLATE */
    "CWORD-E-ADJERR, New light not accepted because of adjacency\n",	/* CWORD_GENADJACERROR */
    "CWORD-I-MAXMAT, Maximum number of matches found - returning successfully\n",/* CWORD_MAXMATCHFOUND */
    "CWORD-W-ITOVER, Maximum light generator iterations executed - stopping\n", /* CWORD_ITERATIONSOVER	*/
    "CWORD-E-MISSNG, Missing input on command line\n",                  /* CWORD_S_COMLINEMISSING */
    "CWORD-I-NUMINT, %d matching %s of words found\n",                  /* CWORD_S_NUMINTERSECTS */
    "CWORD-W-SYSDEF, Using system defaults - configuration file not read\n",/* CWORD_USESYSTEMDEFAULTS */
    "CWORD-I-USRDEF, Using defaults found in configuration file %s\n",	/* CWORD_USEUSERDEFAULTS */
    "CWORD-I-OUTDEF, List/values of Cword configurations being used:\n",/* CWORD_OUTPUTUSERDEFS */
    "CWORD-E-DEFRED, Error reading defaults file\n",			/* CWORD_DEFREADERR */
    "CWORD-W-DEFUND, Undefined default - \"%s\" - not processed\n",	/* CWORD_DEFAULTUNDEFINED */
    "CWORD-I-BLTWHN, %s was built at %s %s\n",				/* CWORD_APPBUILTWHEN */
    "CWORD-I-CHARUI, Using character cell UI\n",			/* CWORD_CHARCELLUIMSG */
    "CWORD-W-EXTERR, Exitting with error status\n",			/* CWORD_EXITERRMSG */
    "CWORD-I-EXITOK, Exitting with successful status\n",		/* CWORD_EXITOKMSG */
    "CWORD-I-SRTCRX, Sorting crossword \"%s\"\n",                       /* CWORD_SORTCROSSWORD */
    "CWORD-E-TRCFLH, Error flushing trace file contents to disk\n",	/* CWORD_TRACEFLUSHERR */
    "CWORD-I-GRDCAL, Grid is to be called \"%s\"\n",			/* CWORD_GRIDTOBECALLED */
    "CWORD-W-SIGINT, User generated Control-C captured\n",              /* CWORD_SIGINTMSG */
    "CWORD-W-SIGFPE, Floating point exception\n",                       /* CWORD_SIGFPEMSG */
    "CWORD-W-SIGSEG, Segmentation violation exception\n",               /* CWORD_SIGSEGVMSG */
    "CWORD-W-SIGBUS, Bus error exception captured\n",			/* CWORD_SIGBUSMSG */
    "CWORD-E-SIGEXT, Exiting Cword abnormally\n",                       /* CWORD_SIGEXITMSG */
    "CWORD-E-GRDERR, Invalid print option chosen\n",			/* CWORD_GRIDOPTERR */
    "CWORD-E-GRDNFD, Valid grid definition not found\n",                /* CWORD_GRDDEFNOTFOUND */
    "CWORD-E-GRDINV, Invalid grid definition found\n",	                /* CWORD_GRDINVALIDERR */
    "CWORD-I-GRDOUT, Grid file %s written\n",                           /* CWORD_GRIDOUTPUTMSG */
    "CWORD-E-PRTERR, Invalid print option chosen\n",                    /* CWORD_PRINTOPTERR */
    "  Total lights so far created: %d",                                /* CWORD_GENUPDATEMSG1 */
    "  No lights so far created",                                       /* CWORD_GENUPDATEMSG1A */
    "  Made up of:",                                                    /* CWORD_GENUPDATEMSG2 */
    "  %d %s %s",                                                       /* CWORD_GENUPDATEMSG3 */
    "  Latest addition: (%d,%d) dir: %c",                               /* CWORD_GENUPDATEMSG4 */
    "  Latest deletion: (%d,%d) dir: %c",                               /* CWORD_GENUPDATEMSG5 */
    "CWORD-E-INVTYP, Invalid crossword type inputted\n",                /* CWORD_VERINVTYPEMSG */
    "CWORD-W-FRECEL, Free cell(s) found\n",                             /* CWORD_S_FREECELLFOUND */
    "CWORD-E-STRNUL, Inputted string is empty\n",                       /* CWORD_STRNULLMSG */
    "CWORD-E-DIRERR, Error opening directory %s\n",                     /* CWORD_DIROPENERRMSG */
    "CWORD-W-NOTCOD, This functionality is not coded just yet\n",       /* CWORD_NOTCODEDYET */
    "CWORD-E-NOTPOS, Grid fill error - no word file length %d available\n",/* CWORD_GRIDFILLNOTPOSS */
    "CWORD-E-GRDERR, Serious error found during grid fill\n",		/* CWORD_S_GRIDERROR */
    "CWORD-I-HOWGRD, The grid will be filled using:\n%s %s and\n%s %s\n",/* CWORD_S_HOWGRIDFILL */
    "CWORD-I-TMPWRD, Template search using \"%s\" in database\n",       /* CWORD_S_TEMPSEARCHDB */
    "CWORD-I-NOTEMP, No template matches found in database\n",		/* CWORD_S_NOTEMPMATCHDB */
    "CWORD-I-RDBSUC, %s %s was successful\n",				/* CWORD_S_RDBGENERICSUCII */
    "CWORD-E-RDBFAL, %s %s was unsuccessful\n",				/* CWORD_S_RDBGENERICERRII */
    "CWORD-W-XRDNOT, %s \"%s\" not in database\n",                      /* CWORD_S_RDBNOTINDB */
    "CWORD-E-DUPLIC, %s \"%s\" already in database\n",                  /* CWORD_S_RDBDUPLICDEFN */
    "CWORD-E-ATTACH, Error attaching to database\n",                    /* CWORD_S_RDBATTACHERR */
    "CWORD-I-XRDDEL, Crossword \"%s\" deleted from database\n",		/* CWORD_S_RDBXRDDELETED */
    "CWORD-I-GRDSUC, Grid \"%s\" obtained from database\n",             /* CWORD_S_RDBGETGRDSUCC */
    "CWORD-I-GRDFAL, Error getting grid \"%s\" from database\n",        /* CWORD_S_RDBGETGRDFAIL */
    "CWORD-I-GRDSUC, Grid \"%s\" added to database\n",                  /* CWORD_S_RDBADDGRDSUCC */
    "CWORD-I-GRDFAL, Error add grid \"%s\" to database\n",              /* CWORD_S_RDBADDGRDFAIL */
    "CWORD-E-NOANWR, No clue found in database for answer \"%s\"\n",    /* CWORD_S_GRIDGETANSWER */
    "CWORD-I-GOTANS, Added clues successfully from db to filled grid\n",/* CWORD_S_GRIDGOTANSWERS */
    "CWORD-W-NOCLUE, No clue was found for the light \"%s\"\n",         /* CWORD_S_NOCLUEFORLIGHT */
    "CWORD-I-CLUEDB, Getting clues details from db - please wait\n",    /* CWORD_S_GETCLUESFROMDB */
    "CWORD-I-GETCLU, Get a clue from the database using clue answer\n", /* CWORD_S_RDBGETCLUEDB */
    "CWORD-I-PUTCLU, Put a clue into database\n",                       /* CWORD_S_RDBPUTCLUEDB */
    "Please input clue answer which to search for : ",                  /* CWORD_S_RDBGETLIGHTSTR */
    "Please input clue answer to be inserted into database : ",         /* CWORD_S_RDBPUTLIGHTSTR */
    "Please input clue to be inserted into database : ",                /* CWORD_S_RDBPUTANSSTR */
    "CWORD-E-CLUERR, Error adding clue/light for \"%s\" to db\n",       /* CWORD_S_RDBPUTANSERR */
    "CWORD-E-DUPXRD, %s possibly already in database\n",                /* CWORD_S_RDBDUPDEFN */
    "CWORD-W-NOTIMP, This functionality has not yet been implemented\n",/* CWORD_S_NOTYETIMPLEMENT */
    "CWORD-I-NOSEED, No user-defined seeds added to grid before generation\n",/* CWORD_S_NOSEEDSDEFINED */
    "CWORD-I-FILSED, Adding seeds from file to grid definition\n",      /* CWORD_S_FILESEEDSTOGRID */
    "CWORD-I-SEEDSF, Getting pre-defined seeds for grid from %s\n",     /* CWORD_S_SEEDSFROMFILE */
    "CWORD-W-NOSEED, Seeds not reset as positive answer not found\n",   /* CWORD_S_SEEDNORESET */
    "Read file containing seeds for grid (%c/%c) [%c] : ",              /* CWORD_S_PROMPTSEEDFILE */
    "CWORD-I-SEDNOT, Seed file %s not used\n",                          /* CWORD_S_SEEDFILENOTUSED */
    "CWORD-E-SFILEE, Error getting seeds from file %s\n",               /* CWORD_S_SEEDFILEERR */
    "CWORD-E-SEDFIL, Seed file %s not compatible with current grid\n",  /* CWORD_S_SEEDFILEDEFERR */
    "CWORD-I-SEDADD, Adding seed \"%s\" to %d %s\n",                    /* CWORD_S_ADDINGSEEDMSG */
    "CWORD-W-SEDMIS, Not all seeds in file were added to grid\n",       /* CWORD_S_SEEDMISSED */
    "CWORD-I-WRTSED, Writing user generated seeds to %s\n",             /* CWORD_S_WRITESEEDTOFILE */
    "CWORD-I-SEDWRT, Seed file %s written successfully\n",              /* CWORD_S_SEEDFILEWRITTEN */
    "CWORD-I-SEDRED, Seed file %s read successfully\n",                 /* CWORD_S_SEEDFILEREAD */
    "Comments on crossword definition [suffix %s] : ",			/* CWORD_S_GETCOMMSXRDMSG */
    "Comments on grid definition [suffix %s] : ",                       /* CWORD_S_GETCOMMSGRDMSG */
    "CWORD-I-PERFIL, %d percent of grid filled\n",                      /* CWORD_S_PERCENTFILLED */
    "CWORD-I-FILTIM, Time taken to fill grid was %d seconds\n",         /* CWORD_S_FILLTIMEWAS */
    "%s already exists.  Overwrite (%c,%c) [%c] : ",                    /* CWORD_S_OVERWRITEFILE */
    "Input a valid file spec (other than %s) [suffix %s] : ",           /* CWORD_S_INPUTFILESPEC */
    "CWORD-E-FILEXD, Valid file spec not entered - cannot continue\n",  /* CWORD_S_FILEINVALID */
    "CWORD-W-OVWFIL, *Warning* overwriting file %s\n",                  /* CWORD_S_FILEOVERWRITE */
    "CWORD-I-GETCLU, Delete a clue from the database using clue answer\n",/* CWORD_S_RDBDELCLUEDB */
    "Input clue descriptions for any part of the following crossword:\n",/* CWORD_S_CLUESTARTTEXT */
    "\n\nCWORD-W-ALLCUD, ** All clues have been user-defined **\n\n",   /* CWORD_S_CLUESALLDEFD */
    "Please input valid clue number (%d-%d) [%d] : ",                   /* CWORD_S_CLUENUMBERMSG */
    "Please input valid clue direction (%c/%c) [%c] : ",                /* CWORD_S_CLUEDIRECTMSG */
    "CWORD-W-NULCLU, Clue not accepted as nothing was entered\n",       /* CWORD_S_NULLCLUEENTERED */
    "CWORD-W-ANINCL, Clue not acceptable as answer is embedded\n",      /* CWORD_S_ANSINCLUEMSG */
    "CWORD-W-1STCHR, First character \'%c\' converted to uppercase\n",  /* CWORD_S_FIRSTCHARUPPER */
    "CWORD-I-VLDCLU, Valid clue description found for %d %s\n",         /* CWORD_S_CLUEVALIDMSG */
    "CWORD-W-NOUNDC, No undefined clues found so defaulting to %d %s\n",/* CWORD_S_NOUNDCLUESFND */
    "Processing clue : %d %s\n\n",                                      /* CWORD_S_PROCESSINGCLUE */
    "CWORD-I-NOCLUS, No clues have been user generated yet\n",          /* CWORD_S_NOCLUEYETUSRGEN */
    "CWORD-W-CLUEQT, Quitting clue generation phase without saving\n",  /* CWORD_S_CLUEQUITMSG */
    "CWORD-I-CLUESV, Save and quit clue generation phase\n",            /* CWORD_S_CLUESAVEGENMSG */
    "CWORD-I-SAVCLU, Saving crossword definition as it has changed\n",  /* CWORD_S_SAVECLUESMSG */
    "CWORD-I-NOSAVC, Unnecessary to save crossword as no changes made\n",/* CWORD_S_NOTSAVECLUESMSG */
    "%d %s: \"%.50s...\" - \"%s\"\n",                                  /* CWORD_S_DISPLAYSOMECLUE */
    "%d %s: \"%s\" - \"%s\"\n",                                        /* CWORD_S_DISPLAYALLCLUE */
    "Reviewing current clue definitions:\n\n",                         /* CWORD_S_REVIEWCLUESMSG */
    "Adding/Altering clue descriptions\n\n",                           /* CWORD_S_ADDCLUEHEADMSG */
    "CWORD-W-CHGMAD, Changes have been made to crossword definition\n",/* CWORD_S_QUITCHANGESMADE */
    "Do you want to continue quitting (%c/%c) [%c] : ",                /* CWORD_S_QUITCONTINUE */
    "CWORD-E-INVCLU, Invalid clue information entered\n",              /* CWORD_S_INVALIDCLUEINFO */
    "\nCWORD-E-CLUEPY, clue has no alphanumeric chars - disallowed\n", /* CWORD_S_CLUEISBLANKS */
    "Processing clue : %d %s\n\n",                                     /* CWORD_S_PROCESSCLUEMSG */
    "Light to clue : %s\n",                                            /* CWORD_S_LIGHTCLUEMSG */
    "Default clue description is : \"%s\"\n\n",                        /* CWORD_S_DEFAULTCLUEMSG */
    "Current clue description is : \"%s\"\n\n",                        /* CWORD_S_CURRENTCLUEMSG */
    "Please input a valid clue description :\n -> ",                   /* CWORD_S_INPUTCLUEMSG */
    "CWORD-W-CLUALN, Clue is unchanged as no description entered\n",   /* CWORD_S_CLUELEFTALONE */
    "CWORD-I-NOTALL, Not all the clue descriptions have been defined\n",/* CWORD_S_CLUENOTALLDEFN */
    "Resetting user-defined contents of clue description to default\n\n",/* CWORD_S_RESETCLUEMSG */
    "CWORD-W-NOTODO, No clues available for resetting at the moment\n",/* CWORD_S_RESETNONETODO */
    "CWORD-I-RESETS, Resetting clue description for %d %s\n",          /* CWORD_S_RESERCLUESUCC */
    "CWORD-W-UNDCLU, Selected clue is undefined and cannot be reset\n",/* CWORD_S_RESETCLUEISDEFN */
    "CWORD-W-NOCLFD, No clue found so processing cannot take place\n", /* CWORD_S_NOCLUESFOUNDMSG */
    "Enter clue generation command [%c] : ",                           /* CWORD_S_CLUECREATECOMM */
    "\n\nNow getting light characteristics:\n\n",                      /* CWORD_S_CLUELIGHTDIMS */
    "Input light dimensions to be used in clue description (1 to %d) [%d] : ",/* CWORD_S_CLUEDIMSINPUT */
    "CWORD-I-LIGDEF, Defaulting light length to current value %d\n",   /* CWORD_S_CLUEDIMSDEF */
    "CWORD-W-CLUERR, Error getting correct length - please try again\n",/* CWORD_S_CLUEDIMERR */
    "\n\nTotal clue description is now : \"%s\"\n",                    /* CWORD_S_TOTALCLUEDESC */
    "CWORD-I-DEFOPT, Default option chosen\n",                         /* CWORD_S_DEFOPTCHOSEN */
    "\nCWORD-W-NOTCOM, Crossword not fully defined\n\n",               /* CWORD_S_CLUESNOTCOMP */
    "CWORD-W-QUTUNS, Quitting clue generation without saving result\n",/* CWORD_S_CLUEQUITUNSAVE */
    "Inputted crossword is fully specified\n\n",                       /* CWORD_S_GRIDFULLSPEC */
    "Inputted crossword is not fully specified\n\n",                   /* CWORD_S_GRIDNOTFULLSPEC */
    "Report on crossword definition \"%s\" :\n\n",                     /* CWORD_S_STATREPTCROSS */
    "Report on grid definition \"%s\" :\n\n",                          /* CWORD_S_STATREPTGRID */
    "Dimensions are %d by %d\n\n",                                     /* CWORD_S_STATDIMSARE */
    "Grid is not disjoint\n",                                          /* CWORD_S_GRIDCONNECT */
    "Grid is disjoint - possibly incorrect\n",                         /* CWORD_S_GRIDNOTCONNECT */
    "%s grid found\n",                                                 /* CWORD_S_GRIDFOUNDMSG */
    "Inputted grid is %d percent checked, and %d percent unchecked\n", /* CWORD_S_CHKUNCHKMSG */
    "Inputted grid is %d percent checked, %d percent unchecked and %d percent blank\n",/* CWORD_S_CHKUNCHKBLKMSG */
    "Press return to view the current grid definition",                /* CWORD_S_GRDAFTERGRTURN */
    "Press return to view the current crossword grid",                 /* CWORD_S_XRDAFTERGRTURN */
    "Total number of lights defined is %d\n\n",                        /* CWORD_S_TOTALLIGHTSMSG */
    "File name         | Crossword Name    %c dims  %c dims   Clue no    Completed\n",/* CWORD_S_XRDFILESTATMSG */
    "File name         | Grid Name    X dims   Y dims   Light no  Grid Type\n",/* CWORD_S_GRDFILESTATMSG */
    "%-17s | %-15s     %2d       %2d      %3d        %3s\n",           /* CWORD_S_XRDFILEDETAILS */
    "%-17s | %-12s   %2d       %2d       %3d      %s\n",               /* CWORD_S_GRDFILEDETAILS */
    "%-17s | Error getting relevant information\n",                    /* CWORD_S_DETAILSFILEERR */
    "CWORD-I-OFFSET, PS print offsets are X=%d and Y=%d for %s\n",     /* CWORD_S_PRINTOFFSETSMSG */
    "CWORD-W-OVRFLW, PS page not sufficently large for output\n",      /* CWORD_S_PRINTPSOVERFLOW */
    "CWORD-W-OVER80, Output is > 80 columns - print appropriately\n",  /* CWORD_S_PRINTOVER80 */
    "CWORD-W-OVR132, Output is > 132 columns - print problems possible\n",/* CWORD_S_PRINTOVER132 */
    " symmetry\n",                                                     /* CWORD_S_SYMMETRY */
    "CWORD-E-SYSTEM, ** Error calling system () **\n",                 /* CWORD_S_SYSTEMCALLERR */
    "Input a title for the completed crossword [%s] : ",               /* CWORD_S_GRIDGETXRDNAME */
    "Input some remarks for the completed crossword [%s] : ",          /* CWORD_S_GRIDGETREMARKS */
    "CWORD-I-SMALL, TXT print offsets for %c < %d is %d\n",            /* CWORD_S_TXTOFFSMALL */
    "CWORD-I-LARGEX, TXT print offsets for %c > %d and < %d is %d\n",  /* CWORD_S_TXTOFFLARGEX */
    "CWORD-I-LARGEY, TXT print offsets for %c >= %d is %d\n",          /* CWORD_S_TXTOFFLARGEY */
    "CWORD-I-NOOFF, No TXT print offset in action for %c\n",           /* CWORD_S_TXTOFFNO */
    "CWORD-E-NOTUNQ, Command line parameter not unique enough: \"%s\"\n",/* CWORD_S_COMLINENOTUNQ */
    "CWORD-I-GGQUIT, Quitting grid creation\n",                        /* CWORD_S_GRIDGENQUIT */
    "CWORD-I-DELING, Deleting lights using : %s symmetry\n",	       /* CWORD_S_DELETINGUSING */
    "CWORD-I-DELLIG, Deleting light - loc: (%d,%d) dir: %s len: %d\n", /* CWORD_S_DELLIGHTLOC */
    "CWORD-E-NOTFND, Specified light not found in grid\n",             /* CWORD_S_USERLHTNOTFOUND */
    "CWORD-E-NODEFS, No default for this option - cannot continue\n",  /* CWORD_S_NODEFAULTERR */
    "CWORD-W-NOLHTS, No lights defined - cannot continue with choice\n",/* CWORD_S_NOLIGHTSDEFINED */
    "CWORD-E-INVLGH, Invalid light co-ordinates entered\n",            /* CWORD_S_INVALIDLIGHT */
    "CWORD-W-INVALD, Grid definition is *in*valid for type sought\n",  /* CWORD_S_USERGRIDINVALID */
    "CWORD-I-VALIDG, Grid definition is valid for type sought\n",      /* CWORD_S_USERGRIDVALID */
    "CWORD-I-RESET, Resetting grid definition containing 1 light\n",   /* CWORD_S_RESETSINGLE */
    "CWORD-I-RESET, Resetting grid definition containing %d lights\n", /* CWORD_S_RESETMULTIPLE */
    "CWORD-W-NOSAVE, No lights defined so no grid will be created\n",  /* CWORD_S_NOLIGHTSNOSAVE */
    "Save grid before quitting (%c/%c) [%c] : ",                       /* CWORD_S_QUITGRIDCREATE */
    "CWORD-W-NOTSAM, Grid *does not* correspond to type requested\n",  /* CWORD_S_GRIDNOTASDEFD1 */
    "Continue saving it anyhow (%c/%c) [%c] : ",                       /* CWORD_S_GRIDNOTASDEFD2 */
    "%s grid\n",                                                       /* CWORD_S_GRID */
    "CWORD-I-GRDYST, Setting %c dimensions equal to %c for %s grid\n", /* CWORD_S_AMERICANYDIMS */
    "Info on crossword file(s) %s from default directory:\n",          /* CWORD_S_XRDFILEINFO */
    "Info on grid file(s) %s from default directory:\n",               /* CWORD_S_GRDFILEINFO */
    "Light lengths     : ",                                            /* CWORD_S_LIGHTLENS */
    "Light frequencies : ",                                            /* CWORD_S_LIGHTFREQS */
    "CWORD-W-RESETS, Resetting symmetry type to no symmetry\n",        /* CWORD_S_RESETSYMM */
    "CWORD-W-CHANGE, Changing grid type for input definition\n",       /* CWORD_S_CHANGEGRIDDEFN */
    "CWORD-I-GRDTYE, Grid type changed to: %s\n",                      /* CWORD_S_GRIDTYPECHANGED */
    "CWORD-W-LGHPRS, Light already present in grid - possible error\n",/* CWORD_S_LGTALREADTPRES */
    "CWORD-I-ZERLEN, Special case: no light info will be added to clue\n",/* CWORD_S_ZEROLENCLUE */
    "CWORD-E-CHKLET, Problem with character %c in light %s\n",         /* CWORD_S_CHECKLETTERPROB */
    "CWORD-W-MAXCLU, Max num of lights reached - Save grid and exit\n",/* CWORD_S_MAXNUMOFCLUES */
    "CWORD-E-UNCHES, Unches found in grid definition\n",               /* CWORD_S_UNCHESFOUND */
    "CWORD-I-BLKAMR, Proportion of blanks over %d percent\n",          /* CWORD_S_AMERICANBLANKS */
    "CWORD-W-NOFREE, No free cells found in grid definition\n",        /* CWORD_S_NOFREECELLFOUND */
    "CWORD-W-SYMNOT, Symmetry selected is incorrect for grid type created\n",/* CWORD_S_SYMMNOTCORRECT */
    "CWORD-I-SYMCHG, Changing symmetry to: %s symmetry\n",             /* CWORD_S_SYMMCHANGE */
    "CWORD-E-INVSYM, Invalid symmetry found\n",                        /* CWORD_S_INVLDSYMMMSG */
    "\t\t%2u.  %s\n",                                                  /* CWORD_S_MAINMENUCHOICE */
    "\t\t(%u)  %s\n",                                                  /* CWORD_S_SUBMENUCHOICE0 */
    "\t\t(%u) %s\n",                                                   /* CWORD_S_SUBMENUCHOICE1 */
    "\t%u.  %s\n\n",                                                   /* CWORD_S_SUBMENUHEADER */
    "MENU:\nMENU: (%s) chosen\nMENU:\n",                               /* CWORD_S_NONUMTRACEMSG */
    "MENU:\nMENU: %s menu option %d chosen\nMENU:\n",                  /* CWORD_S_MENUNUMTRACEMSG */
    "Getting super anagrams.",                                         /* CWORD_S_SUPERANAGRAMNOW */
    "Please input string for super-anagramming : ",                    /* CWORD_S_SUPERANAGRAMSTR */
    "CWORD-E-NOWORD, No word files available for this word length\n",  /* CWORD_S_NOWORDFILEAVAIL */
    "CWORD-I-GENERC, String being checked is %s\n",                    /* CWORD_S_GENERICSTR */
    "CWORD-E-MIN>MX, Error: min word file value %u is greater than max %u\n",/* CWORD_S_MINBIGGERMAX */
    "CWORD-E-NOWORD, No word files found between length %u and %u\n",  /* CWORD_S_NOWORDFILERANGE  */
    "CWORD-I-SUBFND, 1 sub or full anagram found\n",                   /* CWORD_S_SUBANAGRAMFOUND */
    "CWORD-I-NOSNDX, No soundex matches found\n",                      /* CWORD_S_NOSOUNDEXFOUND */
    "CWORD-I-SNDEX1, Selected word is %s - soundex template is %s\n",  /* CWORD_S_SOUNDEX1MSG */
    "CWORD-I-SNDEX2, Selected word is %s - soundex template is %s\n",  /* CWORD_S_SOUNDEX2MSG */
    "CWORD-W-RATION, Rationalised word file list as duplicates found\n",/* CWORD_S_WORDFILERATION */
    "CWORD-E-FGETSE, fgets returns error",                             /* CWORD_S_FGETSPERRORMSG */
    "CWORD-W-TRUNIN, Truncating input to %d characters\n",             /* CWORD_S_TRUNCATINGINPUT */
    "CWORD-E-WORDPE, Error reading word file",                         /* CWORD_S_WORDFILEPERROR */
    "CWORD-I-CRETIM, Time taken to create grid was %d seconds\n",      /* CWORD_S_CREATETIMEWAS */
    " %*u %6s (%2d,%2d) - %2d ",                                       /* CWORD_S_CREATOUTPUTCLUE */
    "CWORD-I-WEBOUT, Number of lines of output from Webster is %u\n",  /* CWORD_S_WEBSTEROUT */
    "DEFINE %s\015\012",                                               /* CWORD_S_WEBQUERTFORMAT */
    " Highest %s clue is numbered %u - Highest %s clue is numbered %u\n",/* CWORD_S_HIGHESTCLUES */
    "Clues already defined in the evolving grid include:\n",           /* CWORD_S_CLUESDEFINED */
    "Enter grid creation command [%c] : ",                             /* CWORD_S_GRIDCREATECOMM */
    "CWORD-E-INTERR, Error getting light information - cannot continue\n",/* CWORD_S_GRIDINPUTERR */
    "       ***** Press %c for help on all commands available *****\n",/* CWORD_S_HELPAVAILABLE */
    "CWORD-I-VERIFY, Verifying evolving grid for correctness\n",       /* CWORD_S_CREATEVERIFY */
    "CWORD-I-TIDY, Tidy evolving grid before saving\n",                /* CWORD_S_CREATETIDY */
    "CWORD-I-RESET, Delete/reset all clues in evolving grid\n",        /* CWORD_S_CREATERESET */
    "CWORD-I-LIST, List all clues in evolving grid to screen\n",       /* CWORD_S_CREATELIST */
    "Here is a short description of each command available:\n\n",      /* CWORD_S_GENERICHELP */
    "The following commands can be used (upper/lower case) :\n\n",     /* CWORD_S_CWORDCOMMANDS */
    "Enter seed generation command [%c] : ",                           /* CWORD_S_SEEDCREATECOMM */
    "CWORD-I-SEED-, Light \"%s\" unseeded at %d %s\n",                 /* CWORD_SEEDDELETED */
    "CWORD-I-FILLNW, Finished seeding - start grid fill immediately\n",/* CWORD_S_SEEDFILLNOW */
    "CWORD-I-QUITNW, Quitting seeding - returning to menu\n",          /* CWORD_S_SEEDQUITNOW */
    "CWORD-I-VLDCLU, Valid clue number has to be entered before continuing\n",/* CWORD_S_SEEDVALIDCLUE */
    "CWORD-W-INVDSD, Error obtaining valid seed - cannot continue\n",  /* CWORD_S_SEEDINVALIDSEED */
    "CWORD-W-NOSEED, No seeds available - cannot continue\n",          /* CWORD_S_SEEDSNONEAVAIL */
    "Lights that have been seeded include:\n\n",                       /* CWORD_S_SEEDSLISTHEAD */
    "Seed for %*u %6.6s is \"%s\"\n",                                  /* CWORD_S_SEEDLISTSEEDS */
    "CWORD-W-DELALL, Delete all seeded lights from current grid (%c/%c) [%c] : ",/* CWORD_S_SEEDDELALLSEEDS */
    "Displaying current seeds in grid view:\n",                        /* CWORD_S_SEEDVIEWGRID */
    "CWORD-W-NOTSAV, Not saving grid changes before continuing\n",     /* CWORD_S_SEEDNOTSAVING */
    "Crossword has been updated - save seeds for later use (%c/%c) [%c] : ",/* CWORD_S_SEEDSAVEQ */
    "CWORD-I-NOSEED, No lights seeded in current grid\n",              /* CWORD_S_SEEDSNONE */
    "CWORD-W-DELFRT, Light selected is already seeded - delete first\n",/* CWORD_S_SEEDDELETEFIRST */
    "CWORD-I-NOTWRN, Seed file previously read will not be altered\n", /* CWORD_S_SEEDFILENOTWRN */
    "CWORD-I-MENUIN, Menu input was %c\n",                             /* CWORD_S_MENUINPUT */
    "CWORD-W-ADDFRT, Light selected is not seeded - add first\n",      /* CWORD_S_SEEDADDFIRST */
    "CWORD-W-ALSGEN, %u %s will contain \"%s\" if %u %s is accepted\n",/* CWORD_S_SEEDALSOGEN */
    "Is this acceptable (%c/%c) [%c] : ",                              /* CWORD_S_YESNOGENERIC */
    "\n       ** Grid is fully seeded  -  Grid is fully seeded **\n\n",/* CWORD_S_SEEDFULLGRID */
    "--------------------------------------------------------------------------------\n",/* CWORD_S_DASHES */
    "CWORD-E-OVRWRT, Seed not valid - checking letters incorrect\n",   /* CWORD_S_SEEDOVERWRITE */
    "Press <return> to continue or %c <return> to quit: ",             /* CWORD_S_PRESSRETURNQUIT */
    "Word file %20s has %5u words\n",                                  /* CWORD_S_SINGLEWORDSTATS */
    "Word file %20s has but %1u word\n",                               /* CWORD_S_SINGLEWORDSTAT */
    " Total number of words in word file(s) of length %2u is %6u\n",   /* CWORD_S_TOTALWORDSTAT */
    "  Total words currently accessible in word files is %u\n",        /* CWORD_S_GRANDWORDSTAT */
    "Outputting word file statistics:\n\n",                            /* CWORD_S_WORDFILESTATS */
    "Break up which input dictionary file : ",                         /* CWORD_S_BREAKUPDIC */
    "File to break up is: %s\n",                                       /* CWORD_S_BREAKUPDICSPEC */
    "CWORD-E-STRTOO, Inputted string is too large\n",                  /* CWORD_S_STRTOOLONGMSG */
    "CWORD-I-ADDWRD, Adding word %s to word file length %u\n",         /* CWORD_S_ADDWORDTOFILE */
    "CWORD-I-WRDADD, Successfully added %s to word file %s\n",         /* CWORD_S_WORDTOFILESUC */
    "CWORD-E-WRDADD, Error adding %s to word file %s\n",               /* CWORD_S_WORDTOFILEFAIL */
    "CWORD-E-TMPERR, Error opening temporary file",                    /* CWORD_S_TEMPFILEPERROR */
    "CWORD-I-NEWWRD, New word file %s created\n",                      /* CWORD_S_NEWWORDFILEMSG */
    "CWORD-E-DELERR, Error deleting %s\n",                             /* CWORD_S_DELETEFILEERR */
    "CWORD-E-RENERR, Error renaming %s to %s\n",                       /* CWORD_S_RENAMEFILEERR */
    "CWORD-E-DELWRD, Error deleting word file %s - cannot continue\n", /* CWORD_S_DELWORDFILEERR */
    "CWORD-E-RENWRD, Error renaming %s to %s - serious error\n",       /* CWORD_S_RENWORDFILEERR */
    "Please input default no. to alter (0 to return to the menu) : ",  /* CWORD_S_INPUTDEFCHOICE */
    "Select a user default to be altered from the following list:\n\n",/* CWORD_S_MODYDEFAULTMSG */
    "The following configurations are currently in use:\n\n",          /* CWORD_S_VIEWDEFAULTMSG */
    "\nCWORD-E-PERROR, Message returned from RTL call is",             /* CWORD_S_PERRORSTR */
    "CWORD-I-CHDIRM, Attempting to change directory\n",                /* CWORD_S_CHDIRMSG */
    "CWORD-I-CHDIRS, Successfully changed directory to %s\n",          /* CWORD_S_CHDIRSUCCESS */
    "CWORD-E-CHDIRE, Error changing directory to %s\n",                /* CWORD_S_CHDIRFAILURE */
    "CWORD-E-TWNOUT, Error outputting matches as uneven number found\n",/* CWORD_S_TWINOUTPUTERR */
    "CWORD-E-WRDERR, Error adding %s to word file list\n",             /* CWORD_S_WORDFILEERR */
    "MENU: option %u (%s) selected",                                   /* CWORD_S_MENUTRACE */
    "CWORD-E-RDBTXT, Error: \"%s\"\n",                                 /* CWORD_S_RDBERRORTEXT */
    "CWORD-E-RDBUND, Undefined Rdb error found\n"                     /* CWORD_S_RDBUNDEFINEDERR */
};

#endif

#endif
