/*
 *++
 *  FACILITY:
 *
 *      seed.c
 *
 *  ABSTRACT:
 *
 *	Module containing all the functionality to support seeding of grid
 *	definitions before grid filling.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      19 January 1995
 *--
 */


/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "main.h"
#include "messages.h"
#include "grid.h"
#include "utils.h"
#include "stats.h"
#include "charui.h"
#include "fileio.h"
#include "word.h"
#include "os.h"
#include "match.h"
#include "memman.h"
#include "defaults.h"

#if defined(VMS) || defined (__VMS)

#include <unixio.h>
#include <signal.h>

#endif

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;

/*
 *  This assumes that the crossword has less than 100 clues or more
 *  specifically less than 100 across/down.
 */

#define LIGHTS_PER_LINE	5



#define ADD_CHAR	'A'
#define DELETE_CHAR     'D'
#define LIST_CHAR	'L'
#define RESET_CHAR      'R'
#define VIEW_CHAR       'V'
#define FILL_CHAR       'F'
#define QUIT_CHAR       'Q'
#define HELP_CHAR	'H'
#define INVALID_CHAR    '\0'

#define NOT_VALID	-1

#define BLANK_CH	(char) ' '

/*
 *  Whether to QUIT or FILL in cword_seed__finish_seeding?
 */

#define FILL		TRUE
#define QUIT		FALSE

/*
 *  For addition or deletion purposes
 */

#define ADDITION	TRUE
#define DELETION	FALSE



/*
 *  Private function prototypes.
 */

PRIVATE void cword_seed__set_light_blank (
	int			length,
	char			*light_str);

PRIVATE int cword_seed__is_seed_filled (
	int			length,
	char			*light_str);

PRIVATE BOOLEAN cword_seed__is_light_blank (
	int			length,
	char			*light_str);

PRIVATE BOOLEAN cword_seed_is_seed_available (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__read_seed_grid (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__write_seed_grid (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE void cword_seed__output_light_opts (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__get_seed_index (
	BOOLEAN			is_addition,
	CWORD_T_GRID_STRUCT	*grid_record,
	int			*index);

PRIVATE int cword_seed__check_light_temps (
	int			index,
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__check_is_seed_ok (
	int			index,
	char			*light_str,
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__get_seed_str (
	int			index,
	CWORD_T_GRID_STRUCT	*grid_record,
	char			*light_str);

PRIVATE int cword_seed__cal_grid_contents (
	CWORD_T_GRID_STRUCT	*grid_record,
	CROSSWORD_GRID		grid_contents);

PRIVATE BOOLEAN cword_seed__do_seeds_coexist (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__update_light_temps (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__add_seed (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__delete_seed (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__list_seeds (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__reset_seeds (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE int cword_seed__view_seed_grid (
	CWORD_T_GRID_STRUCT	*grid_record);

PRIVATE void cword_seed__output_help ();

PRIVATE void cword_seed__get_option (
	char			*option);

PRIVATE int cword_seed__finish_seeding (
	int			quit_or_fill,
	CWORD_T_GRID_STRUCT	*grid_record);

/*
 *  Module global to record whether the current grid definition has been
 *  updated or downdated.
 */

static BOOLEAN cword_seed_l_grid_updated;

static BOOLEAN cword_seed_l_seed_file_read;



PUBLIC int cword_seed_get_high_clue_num (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      Highest numbered clue in current grid_record.
 *--
 */

{

    CROSSWORD_RECORD *crossword_record;

    int high_num = NOT_VALID;

    crossword_record = cword_memman_make_crossword_rec ();

    cword_grid_g_to_c_rec (
	grid_record,
	crossword_record);

    high_num = cword_stats_get_high_clue_num (
		    BOTH,
		    crossword_record);

    /*
     *  Not needed anymore!
     */

    cword_memman_free_cross_rec (crossword_record);
    
    return (high_num);

}



PRIVATE void cword_seed__set_light_blank (
	int			length,
	char			*light_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Blank out the inputted indexed grid_record light.
 *
 *  FORMAL PARAMETERS:
 *
 *	*light_str		(w) String to be "blanked" during seeding!
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None!
 *--
 */

{

    int count;

    /*
     *  Set each element of light_str to BLANK_CH.
     */

    for (count = 0; count < length; count++)
	light_str[count] = BLANK_CH;

    return;

}



PRIVATE int cword_seed__is_seed_filled (
	int			length,
	char			*light_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int count;

    BOOLEAN is_filled = TRUE;

    for (count = 0; count < length; count++)
    {
	if (light_str[count] == BLANK_CH)
	{
	    is_filled = FALSE;

	    break;
	}
    }

    if (is_filled == TRUE)
	return (TRUE);
    else
	return (FALSE);

}



PRIVATE BOOLEAN cword_seed__is_light_blank (
	int			length,
	char			*light_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Checks whether the inputted indexed grid record light is blank or not!
 *
 *  FORMAL PARAMETERS:
 *
 *	length,			() length of input light.
 *	*light_str		() Input light to check for blankness.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Light is blank!
 *	FALSE			Light is filled!
 *--
 */

{

    int count;

    BOOLEAN is_blank = TRUE;

    for (count = 0; count < length; count++)
    {
	if (light_str[count] != BLANK_CH)
	{
	    is_blank = FALSE;

	    break;
	}
    }

    if (is_blank == TRUE)
	return (TRUE);
    else
	return (FALSE);

}



PRIVATE BOOLEAN cword_seed_is_seed_available (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check whether there is at least one seeded light in
 *	grid_record.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(r) Pointer to CWORD_T_GRID_STRUCT structure.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			There are lights seeded in inputted grid_record.
 *	FALSE			There are no lights seeded in input grid_record.
 *--
 */

{

    int count;

    BOOLEAN is_seed_avail = FALSE;

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if (grid_record->grid_details[count].grid_entry == LIGHT_SEEDED)
	{
	    is_seed_avail = TRUE;

	    break;
	}
    }

    if (is_seed_avail == TRUE)
	return (TRUE);
    else
	return (FALSE);

}



PRIVATE int cword_seed__add_seed_loc (
	char			*seed_str,
	int			index,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Seed successfully added.
 *	CWORD_S_FAILURE		Seed unsuccessfully added.
 *--
 */

{

    int status;

    if ( (cword_seed__is_light_blank (
	    grid_record->grid_details[index].light_len,
	    grid_record->grid_details[index].light) == FALSE)
			||
          (grid_record->grid_details[index].grid_entry == LIGHT_SEEDED) )
    {
	printf (
	    "# Seed light %d %s has already been seeded or is otherwise occupied\n",
	    grid_record->grid_details[index].clue_number, 
	    grid_record->grid_details[index].clue_direction 
		    ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

	return (CWORD_S_FAILURE);
    }

    if (strlen (seed_str) != grid_record->grid_details[index].light_len)
    {
	printf (
	    "# Length of input is not equal to that required of %u %s\n",
	    grid_record->grid_details[index].clue_number, 
	    grid_record->grid_details[index].clue_direction 
		    ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

	return (CWORD_S_FAILURE);
    }

    /*
     *  Put seed in grid_record!
     */

    strcpy (grid_record->grid_details[index].light, seed_str);

    /*
     *  Add the light to disallowed list.
     */

    status = cword_grid_add_to_disallowed (
		ALLWORDS,
		grid_record->grid_details[index].light,
		grid_record);

    if (status != CWORD_S_NORMAL)
    {
	cword_seed__set_light_blank (
	    grid_record->grid_details[index].light_len,
	    grid_record->grid_details[index].light);

	return (CWORD_S_FAILURE);
    }

    /*
     *	Clue is seeded and cannot be struck from the grid definition during
     *	generation.
     */

    grid_record->grid_details[index].grid_entry = LIGHT_SEEDED;

    cword_messages_output_message (
	CWORD_S_SEEDADDED,
	(TRACE0 | SCREEN0),
	seed_str,
	grid_record->grid_details[index].clue_number, 
	grid_record->grid_details[index].clue_direction 
		? CWORD_ACROSS_STR : CWORD_DOWN_STR);

    cword_seed_l_grid_updated = TRUE;

    cword_seed__update_light_temps (grid_record);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__delete_seed_loc (
	int			index,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Seed successfully deleted.
 *	CWORD_S_FAILURE		Seed unsuccessfully delted.
 *--
 */

{

    char temp_str[CWORD_SIZEOF_CELLDIM + 1];

    int status;

    if ( (cword_seed__is_light_blank (
	    grid_record->grid_details[index].light_len,
	    grid_record->grid_details[index].light) == TRUE) 
		    || 
          (grid_record->grid_details[index].grid_entry != LIGHT_SEEDED) )
    {
	printf (
	    "# Seed light %u %s is already deleted or was otherwise never added\n",
	    grid_record->grid_details[index].clue_number, 
	    grid_record->grid_details[index].clue_direction 
		    ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

	return (CWORD_S_FAILURE);
    }

    strcpy (temp_str, grid_record->grid_details[index].light);

    cword_seed__set_light_blank (
	grid_record->grid_details[index].light_len,
	grid_record->grid_details[index].light);

    grid_record->grid_details[index].grid_entry = LIGHT_EMPTY;

    /*
     *	?? Not yet written and is a bug at the moment whereby it's not possible
     *	to re-add a seed that was previously deleted in the same seeding
     *	session ??
     */

    status = cword_grid_del_from_disallowed (
		ALLWORDS,
		temp_str,
		grid_record);

    cword_messages_output_message (
	CWORD_S_SEEDDELETED,
	(TRACE0 | SCREEN0),
	temp_str,
	grid_record->grid_details[index].clue_number, 
	grid_record->grid_details[index].clue_direction 
		? CWORD_ACROSS_STR : CWORD_DOWN_STR);

    cword_seed_l_grid_updated = TRUE;

    cword_seed__update_light_temps (grid_record);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__write_seed_grid (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function is used to write all user defined seeds - which were user
 *	inputted before the light generation was begun - to a text file.  This
 *	file can then be re-used when a light generation is re-initiated on the
 *	same grid sometime in the future.  This functionality is particularly
 *	useful in the event that it wasn't possible to fill a grid with the
 *	configuration of user lights specified.
 *
 *  FORMAL PARAMETERS:
 *
 *	*grid_record		(r) Pointer to a CWORD_T_GRID_STRUCT containing seeds 
 *				    to be written to file
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *--
 */

{

    /*
     *  1. Transfer grid_record struct to crossword_record struct 
     *     (no loss on information)
     *	2. write crossword_record struct to pre-defined filespec with
     * 	   particular format which will only be read correctly by companion
     *	   function: cword_seed__read_seed_grid.
     */

    CROSSWORD_RECORD *crossword_record;

    char seed_file_spec[CWORD_SIZEOF_FILESPEC + 1];

    int status;

    crossword_record = cword_memman_make_crossword_rec ();

    /*
     *  Construct the name of the seed file to write to.
     */

    sprintf (
	seed_file_spec,
	"%s%s",
	grid_record->name,
	CWORD_SUFFIX_SEED_FILE);

    /*
     *	?? Check if there's already a file of that name in the default
     *	directory ??
     */

    cword_messages_output_message (
	CWORD_S_WRITESEEDTOFILE,
	(TRACE0 | SCREEN0),
	seed_file_spec);

    /*
     *  First move grid_record to crossword_record.
     */

    status = cword_grid_g_to_c_rec (
		grid_record,
		crossword_record);

    if (CWORD_S_NORMAL != status)
    {
	cword_memman_free_cross_rec (crossword_record);

	return (status);
    }

    /*
     *  Write seeds to file if successful?
     */

    status = cword_fileio_write_seeds (
		seed_file_spec,
		crossword_record);

    if (CWORD_S_NORMAL != status)
    {
	cword_memman_free_cross_rec (crossword_record);

	return (status);
    }

    cword_memman_free_cross_rec (crossword_record);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__read_seed_grid (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	This function reads the seed file written by its companion function
 *	cword_seed__write_seed_grid and checks whether it is a valid seed file
 *	and then makes sure that the seed file is for the same grid as we are
 *	currently generating.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(w) Pointer to CWORD_T_GRID_STRUCT in which to store seeds 
 *				    from seed file.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *	cword_seed_l_seed_file_read
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *	CWORD_S_FAILURE		Error matching up seeds from file with grid
 *	file error		Error parsing seed file correctly
 *--
 */

{

    CROSSWORD_RECORD *crossword_record;

    char seed_file_spec[CWORD_SIZEOF_FILESPEC + 1],
	 temp_ch;

    int	status,
	count,
	count1,
	total,
	total_seeded;

    crossword_record = cword_memman_make_crossword_rec ();

    /*
     *  Construct the name of the seed file to read from.
     */

    sprintf (
	seed_file_spec,
	"%s%s",
	grid_record->name,
	CWORD_SUFFIX_SEED_FILE);

    /*
     *	Check that the seeds file is being used on the correct grid definition
     *	we are now trying to fill.
     */

    if (cword_utils_does_file_exist (seed_file_spec) == FALSE)
    {
	/*
	 *  Seeds file not found - cannot continue!
	 */

	cword_memman_free_cross_rec (crossword_record);

	return (CWORD_S_FAILURE);
    }
    else
    {
	cword_messages_output_message (
	    CWORD_S_SEEDSFROMFILE,
	    SCREEN2,
	    seed_file_spec);

	/*
	 *  Prompt whether the seed file should be read?
	 */

	cword_messages_output_message (
	    CWORD_S_PROMPTSEEDFILE,
	    SCREEN0,
	    CWORD_Y_CHAR,
	    CWORD_N_CHAR,
	    CWORD_Y_CHAR);

	temp_ch = cword_charui_get_ch (FALSE);

	if (toupper (temp_ch) == CWORD_N_CHAR)
	{
	    cword_messages_output_message (
		CWORD_S_SEEDFILENOTUSED,
		(TRACE2 | SCREEN0),
		seed_file_spec);

	    /*
	     *  Let user see that seed file will not be used!
	     */

	    cword_memman_free_cross_rec (crossword_record);

	    return (CWORD_S_FAILURE);
	}
    }

    /*
     *	Fill in as many of the seeds as possible into the input
     *	crossword_record.
     */

    status = cword_fileio_read_seeds (
		seed_file_spec,
		crossword_record);

    if (CWORD_S_NORMAL != status)
    {
	cword_messages_output_message (
	    CWORD_S_SEEDFILEERR,
	    (TRACE2 | SCREEN0),
	    seed_file_spec);

	cword_memman_free_cross_rec (crossword_record);

	return (CWORD_S_FAILURE);
    }

    /*
     *	Gotten seeds and now checking that the seeds we've gotten correspond to
     *	the grid definition that we are currently filling.
     */

    if ( (strcmp (grid_record->name, crossword_record->name) != 0) ||
          (grid_record->x_dimensions != crossword_record->x_dimensions) ||
           (grid_record->y_dimensions != crossword_record->y_dimensions) ||
            (grid_record->clue_sum != crossword_record->clue_sum) )
    {
	cword_messages_output_message (
	    CWORD_S_SEEDFILEDEFERR,
	    (TRACE2 | SCREEN0),
	    seed_file_spec);

	cword_memman_free_cross_rec (crossword_record);

	return (CWORD_S_FAILURE);
    }

    /*
     *	Put the details obtained from the seed file into the grid_record before
     *	continuing the seeding process.
     */

    count1 = total_seeded = 0;

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	/*
	 *  Check if the seeds read from temporary file are suitable for
	 *  insertion into the grid record
	 */

	count1 = 0;
	while (crossword_record->clue_details[count1].clue_number != 0)
	{
	    DEBUG_TRACE (
		"crossword - num %d dir %d x_coord %d y_coord %d light_len %d\n",
		crossword_record->clue_details[count1].clue_number,
		crossword_record->clue_details[count1].clue_direction,
		crossword_record->clue_details[count1].x_coordinate,
		crossword_record->clue_details[count1].y_coordinate,
		crossword_record->clue_details[count1].answer_length);

	    DEBUG_TRACE (
		"grid      - num %d dir %d x_coord %d y_coord %d light_len %d\n",
		grid_record->grid_details[count].clue_number,
		grid_record->grid_details[count].clue_direction,
		grid_record->grid_details[count].x_coordinate,
		grid_record->grid_details[count].y_coordinate,
		grid_record->grid_details[count].light_len);

	    if ( (crossword_record->clue_details[count1].clue_number == 
                  grid_record->grid_details[count].clue_number) &&
                  (crossword_record->clue_details[count1].clue_direction == 
                   grid_record->grid_details[count].clue_direction) &&
                   (crossword_record->clue_details[count1].x_coordinate == 
                    grid_record->grid_details[count].x_coordinate) &&
                    (crossword_record->clue_details[count1].y_coordinate == 
                     grid_record->grid_details[count].y_coordinate) &&
                     (crossword_record->clue_details[count1].answer_length == 
                      grid_record->grid_details[count].light_len) )
            {
		strcpy (
		    grid_record->grid_details[count].light,
		    crossword_record->clue_details[count1].clue_answer);

		cword_messages_output_message (
		    CWORD_S_ADDINGSEEDMSG,
		    (TRACE2 | SCREEN0),
		    grid_record->grid_details[count].light,
		    grid_record->grid_details[count].clue_number,
		    grid_record->grid_details[count].clue_direction ?
			CWORD_ACROSS_STR : CWORD_DOWN_STR);

		/*
		 *  Signify that the seed just read is actually a seed in the
		 *  eyes of the light generator.
		 */

		grid_record->grid_details[count].grid_entry = LIGHT_SEEDED;

		/*
		 *  Add the seed to the disallowed list!
		 */

		status = cword_grid_add_to_disallowed (
			    ALLWORDS,
			    grid_record->grid_details[count].light,
			    grid_record);

		total_seeded++;

		break;
	    }

	    count1++;
	}   
    }

    /*
     *	?? Don't understand what's happening here, or why ??
     */

    total = 0;
    while (crossword_record->clue_details[total].clue_number != 0)
	total++;

    if (total != total_seeded)
    {
	/*
	 *  Continue anyhow.
	 */

	cword_messages_output_message (
	    CWORD_S_SEEDMISSED,
	    (TRACE2 | SCREEN0));
    }

    cword_seed_l_seed_file_read = TRUE;

    /*
     *  Need to include this pause, I mean it!
     */

    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

    cword_memman_free_cross_rec (crossword_record);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__cal_grid_contents (
	CWORD_T_GRID_STRUCT	*grid_record,
	CROSSWORD_GRID		grid_contents)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Grid lights OK and grid_contents filled.
 *	FALSE			Grid lights not OK and grid_contents zero.
 *--
 */

{

    int count1,
	count2;

    for (count1 = 0; count1 < CWORD_SIZEOF_CELLDIM; count1++)
	for (count2 = 0; count2 < CWORD_SIZEOF_CELLDIM; count2++)
	    grid_contents[count1][count2] = NOT_TAKEN;

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	if (grid_record->grid_details[count1].clue_direction == ACROSS)
	{
	    for (count2 = 0; count2 < grid_record->grid_details[count1].light_len; count2++)
	    {
		if (grid_record->grid_details[count1].light[count2] != (int) BLANK_CH)
		{
		    if (grid_contents[grid_record->grid_details[count1].x_coordinate - 1 + count2]
			       [grid_record->grid_details[count1].y_coordinate - 1] == NOT_TAKEN)
		    {
			grid_contents[grid_record->grid_details[count1].x_coordinate - 1 + count2]
				   [grid_record->grid_details[count1].y_coordinate - 1] = 
					(int) grid_record->grid_details[count1].light[count2];
		    }
		    else
		    {
		        /* 
			 *  grid_contents[x][y] already has value within -
			 *  check whether the value is acceptable.
			 */

			if ( (grid_contents[grid_record->grid_details[count1].x_coordinate - 1 + count2]
				           [grid_record->grid_details[count1].y_coordinate - 1] != 
				    (int) grid_record->grid_details[count1].light[count2]) &&
			       (grid_contents[grid_record->grid_details[count1].x_coordinate - 1 + count2]
				              [grid_record->grid_details[count1].y_coordinate - 1] != (int) BLANK_CH) )
			{
			    cword_messages_output_message (
				    CWORD_SEEDCELLLOCERR,
				    SCREEN2,
				    (grid_record->grid_details[count1].x_coordinate - 1 + count2 + 1),
				    grid_record->grid_details[count1].y_coordinate,
				    CWORD_ACROSS_STR);

			    cword_messages_output_message (
				    CWORD_S_SEEDOVERWRITE,
				    SCREEN0);

			    for (count1 = 0; count1 < CWORD_SIZEOF_CELLDIM; count1++)
				for (count2 = 0; count2 < CWORD_SIZEOF_CELLDIM; count2++)
				    grid_contents[count1][count2] = NOT_TAKEN;

			    return (FALSE);
			}
			else
			{
			    grid_contents[grid_record->grid_details[count1].x_coordinate - 1 + count2]
				       [grid_record->grid_details[count1].y_coordinate - 1] = 
					    (int) grid_record->grid_details[count1].light[count2];
			}
		    }
		}
		else
		{
		    if ( (grid_contents[grid_record->grid_details[count1].x_coordinate - 1 + count2]
			       [grid_record->grid_details[count1].y_coordinate - 1] == NOT_TAKEN) ||
			   (grid_contents[grid_record->grid_details[count1].x_coordinate - 1 + count2]
			       [grid_record->grid_details[count1].y_coordinate - 1] == (int) BLANK_CH) )
		    {
			grid_contents[grid_record->grid_details[count1].x_coordinate - 1 + count2]
				    [grid_record->grid_details[count1].y_coordinate - 1] = (int) BLANK_CH;
		    }
		}
	    }
	}
	else
	{
	    for (count2 = 0; count2 < grid_record->grid_details[count1].light_len; count2++)
	    {
		if (grid_record->grid_details[count1].light[count2] != (int) BLANK_CH)
		{
		    if (grid_contents[grid_record->grid_details[count1].x_coordinate - 1]
			       [grid_record->grid_details[count1].y_coordinate - 1 - count2] == NOT_TAKEN)
		    {
			grid_contents[grid_record->grid_details[count1].x_coordinate - 1]
			    [grid_record->grid_details[count1].y_coordinate - 1 - count2] =
				(int) grid_record->grid_details[count1].light[count2];
		    }
		    else
		    {
		        /* 
			 *  grid_contents[x][y] already has value within -
			 *  check whether the value is acceptable.
			 */

			if ( (grid_contents[grid_record->grid_details[count1].x_coordinate - 1]
			                    [grid_record->grid_details[count1].y_coordinate - 1 - count2] != 
			       (int) grid_record->grid_details[count1].light[count2]) && 
			       (grid_contents[grid_record->grid_details[count1].x_coordinate - 1]
			                      [grid_record->grid_details[count1].y_coordinate - 1 - count2] != 
			        (int) BLANK_CH) )
			{
			    cword_messages_output_message (
				    CWORD_SEEDCELLLOCERR,
				    SCREEN2,
				    grid_record->grid_details[count1].x_coordinate,
				    (grid_record->grid_details[count1].y_coordinate - 1 - count2 + 1),
				    CWORD_DOWN_STR);

			    cword_messages_output_message (
				    CWORD_S_SEEDOVERWRITE,
				    SCREEN0);

			    for (count1 = 0; count1 < CWORD_SIZEOF_CELLDIM; count1++)
				for (count2 = 0; count2 < CWORD_SIZEOF_CELLDIM; count2++)
				    grid_contents[count1][count2] = NOT_TAKEN;

			    return (FALSE);
			}
			else
			{
			    grid_contents[grid_record->grid_details[count1].x_coordinate - 1]
				[grid_record->grid_details[count1].y_coordinate - 1 - count2] =
				    (int) grid_record->grid_details[count1].light[count2];
			}
		    }
		}
		else
		{
		    if ( (grid_contents[grid_record->grid_details[count1].x_coordinate - 1]
 		           [grid_record->grid_details[count1].y_coordinate - 1 - count2] == NOT_TAKEN) ||
			  (grid_contents[grid_record->grid_details[count1].x_coordinate - 1]
 		           [grid_record->grid_details[count1].y_coordinate - 1 - count2] == (int) BLANK_CH) ) 
		    {
			grid_contents[grid_record->grid_details[count1].x_coordinate - 1]
				    [grid_record->grid_details[count1].y_coordinate - 1 - count2] = (int) BLANK_CH;
		    }
		}
	    }
	}
    }

    return (TRUE);

}



PRIVATE BOOLEAN cword_seed__do_seeds_coexist (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			Defined seed co-exist fine!
 *	FALSE			Defined seeds do not co-exist!
 *--
 */

{


    CROSSWORD_GRID grid_contents;

    /*
     *	If cword_seed__cal_grid_contents returns TRUE then the grid lights
     *	co-exist OK.  If the function returns FALSE then they don't.  I don't
     *	care about grid_contents in this case!
     */

    return (cword_seed__cal_grid_contents (
		grid_record,
		grid_contents));

}
 


PRIVATE int cword_seed__update_light_temps (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *	CWORD_S_FAILURE		Error found with seeds.
 *--
 */

{

    CROSSWORD_GRID grid_contents;

    int count1,
	count2,
	count3;

    /*
     *	The return value of cword_seed__cal_grid_contents should be OK.  We are
     *	looking for the values containing in grid_contents instead this time!
     */

    if (cword_seed__cal_grid_contents (
	    grid_record,
	    grid_contents) == FALSE)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *	Fill up the answer template strings for the further use in the calling
     *	function.
     */

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	/*
	 *  Pad out the strings before you start!
	 */

	count2 = 0;
	while (count2 < grid_record->grid_details[count1].light_len)
	{
	    grid_record->grid_details[count1].light_template[count2] = BLANK_CH;
	    count2++;
	}
	grid_record->grid_details[count1].light_template[count2] = (char) 0;
    }

    for (count1 = 0; count1 < grid_record->clue_sum; count1++)
    {
	/*
	 *  Insert to seeding data to re-create the light templates.
	 */

	if (grid_record->grid_details[count1].clue_direction == ACROSS)
	{
	    count3 = 0;
	    for (count2 = grid_record->grid_details[count1].x_coordinate;
		  count2 < (grid_record->grid_details[count1].x_coordinate + 
			     grid_record->grid_details[count1].light_len);
		   count2++)
	    {

		if ( (grid_contents[count2 - 1][grid_record->grid_details[count1].y_coordinate - 1] != NOT_TAKEN) &&
		      (grid_contents[count2 - 1][grid_record->grid_details[count1].y_coordinate - 1] != (int) BLANK_CH) )
		{
		    grid_record->grid_details[count1].light_template[count3] = 
			    grid_contents[count2 - 1][grid_record->grid_details[count1].y_coordinate - 1];
		}
		count3++;
	    }
	}
	else
	{
	    count3 = 0;
	    for (count2 = grid_record->grid_details[count1].y_coordinate;
		  count2 > (grid_record->grid_details[count1].y_coordinate -
			     grid_record->grid_details[count1].light_len);
		   count2--)
	    {
		if ( (grid_contents[grid_record->grid_details[count1].x_coordinate - 1][count2 - 1] != NOT_TAKEN) &&
		      (grid_contents[grid_record->grid_details[count1].x_coordinate - 1][count2 - 1] != (int) BLANK_CH) )
		{
		    grid_record->grid_details[count1].light_template[count3] = 
			    grid_contents[grid_record->grid_details[count1].x_coordinate - 1][count2 - 1];
		}
		count3++;
	    }
    	}
    }

    for (count2 = 0; count2 < grid_record->clue_sum; count2++)
	DEBUG_TRACE (
	    "New light_template[%u] is \"%s\"\n",
	    count2,
	    grid_record->grid_details[count2].light_template);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__get_seed_index (
	BOOLEAN			is_addition,
	CWORD_T_GRID_STRUCT	*grid_record,
	int			*index)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that prompts user to input details about the seed to be
 *	operated upon.  It will ask for the clue number and if necessary clue
 *	direction.  This function will return the grid_record index of the clue
 *	chosen, if any!
 *
 *  FORMAL PARAMETERS:
 *
 *	*grid_record		(r) Pointer to a CWORD_T_GRID_STRUCT data structure.
 *	*index			(w) Index of clue to be operated upon.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Valid index found to add
 *	CWORD_S_FAILURE		Invalid index - cannot continue.
 *--
 */

{

    char temp_str[CWORD_SIZEOF_MESSAGE + 1],
	 temp_ch;

    int	count,
	clue_num,
	across,
	down,
	direction,
        status;

    int highest_clue;

    *index = NOT_VALID;

    /*
     *	Get highest clue number value for messages later in the function.
     */

    highest_clue = cword_seed_get_high_clue_num (grid_record);

    /*
     *	?? Function to get highest available light that hasn't yet been
     *	seeded as the default for CWORD_LIGHTNUMBER ??
     */

    cword_messages_output_message (
	CWORD_LIGHTNUMBER,
	SCREEN0,
	1, 
	highest_clue);

    /*
     *	Just get a valid integer without mapping it to a light location in the
     *	grid for the time being.
     */

    status = cword_charui_get_str (
		DIGIT,
		BLANK_OK,
		sizeof (temp_str) - 1,
		temp_str);

    if (status != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }
    else
    {
	if (temp_str[0] != (char) 0)
	{
	    clue_num = atoi (temp_str);
	}
	else
	{
	    cword_messages_output_message (
		CWORD_S_SEEDVALIDCLUE,
		SCREEN0);

	    return (CWORD_S_FAILURE);
	}
    }

    /*
     *	Check if there is a 24 Across or 24 Down before continuing.
     */

    /*
     *	?? Write a function that would accept index and grid_record and return
     *	one of four things:
     *
     *	    1. inputted index is an ACROSS clue			    - ACROSS
     *	    2. inputted index is an DOWN clue			    - DOWN 
     *	    3. Input index has both an ACROSS and DOWN clue	    - BOTH (ACROSS | DOWN)
     *	    4. inputted index has neither an ACROSS or DOWN clue    - NEITHER
     */

    down = FALSE; across = FALSE;
    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if (grid_record->grid_details[count].clue_number == clue_num)
	    if (grid_record->grid_details[count].clue_direction == ACROSS)
		across = TRUE;
	    else
		down = TRUE;
    }

    if ( (across == FALSE) && (down == FALSE) )
    {
	/* 
	 *  No valid clue found for clue number clue_num.
	 */

	cword_messages_output_message (
		    CWORD_INVALIDLIGHTNUM,
		    SCREEN0,
		    clue_num,
		    CWORD_ACROSS_STR,
		    clue_num, 
		    CWORD_DOWN_STR);

	return (CWORD_S_FAILURE);
    }

    if ( (across == TRUE) && (down == TRUE) )
    {
	/*
	 *  If the inputted clue number has both a Down and an Across clue then
	 *  let the user select which she/he wants.
	 */

	cword_messages_output_message (
		    CWORD_LIGHTDIRECTION,
		    SCREEN0,
		    CWORD_A_CHAR, 
		    CWORD_D_CHAR,
		    CWORD_A_CHAR);

	temp_ch = cword_charui_get_ch (FALSE);

	if (toupper (temp_ch) == CWORD_D_CHAR)
	    direction = DOWN;
	else
	{
	    /*
	     *  Everything else is ACROSS.
	     */

	    direction = ACROSS;
	}
    }
    else if (across == TRUE)
    {
	direction = ACROSS;
    }
    else if (down == TRUE)    
    {
	direction = DOWN;
    }

    cword_messages_output_message (
	    CWORD_LIGHTDIRCHOSEN,
	    SCREEN0,
	    direction ? CWORD_ACROSS_STR : CWORD_DOWN_STR);

    /*
     *	Searches grid record for grid_record index of the light selected for
     *	seeding.
     */

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if ( (grid_record->grid_details[count].clue_number == clue_num) &&
	      (grid_record->grid_details[count].clue_direction == direction) )
	{
	    /*
	     *	Check now wether we are in here for a DELETION or an ADDITION?
	     */

	    if (is_addition == ADDITION)
	    {
		if (grid_record->grid_details[count].grid_entry != LIGHT_SEEDED)
		{
		    *index = count;
		}
		else
		{
		    /*
		     *	If the user is selecting a clue for addition but it's
		     *	already seeded, then it first has to be deleted before
		     *	an addition can take place.
		     */

		    cword_messages_output_message (
			CWORD_S_SEEDDELETEFIRST,
			SCREEN0);

		    return (CWORD_S_FAILURE);
		}
	    }
	    else
	    {
		if (grid_record->grid_details[count].grid_entry == LIGHT_SEEDED)
		{
		    *index = count;
		}
		else
		{
		    /*
		     *	If the user is selecting a clue for deletion but it's
		     *	not seeded, then it first has to be added before an
		     *	deletion can take place.
		     */

		    cword_messages_output_message (
			CWORD_S_SEEDADDFIRST,
			SCREEN0);

		    return (CWORD_S_FAILURE);
		}
	    }

	    break;
	}
    }

    if (*index != NOT_VALID)
        return (CWORD_S_NORMAL);
    else
    {
	/*
	 *  Just in case index wasn't found!
	 */

        return (CWORD_S_FAILURE);
    }

}



PRIVATE int cword_seed__check_light_temps (
	int			index,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Continue on as normal :-).
 *	CWORD_S_FAILURE		Do not accept the seed :-(.
 *--
 */

{

    int count;

    /*
     *	Checking all the light templates for completeness as a by-product of
     *	adding seed to the grid>
     */

    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if ( (grid_record->grid_details[count].grid_entry != LIGHT_SEEDED) &&
	      (count != index) )
	{
	    /*
	     *  Check any unSEEDED templates for completeness.
	     */

	    if (cword_seed__is_seed_filled (
		    grid_record->grid_details[count].light_len,
		    grid_record->grid_details[count].light_template) == TRUE)
	    {
		char temp_ch;

		cword_messages_output_message (
		    CWORD_S_SEEDALSOGEN,
		    SCREEN0,
		    grid_record->grid_details[count].clue_number,
		    grid_record->grid_details[count].clue_direction 
					? CWORD_ACROSS_STR : CWORD_DOWN_STR,
		    grid_record->grid_details[count].light_template,
		    grid_record->grid_details[index].clue_number,
		    grid_record->grid_details[index].clue_direction 
					? CWORD_ACROSS_STR : CWORD_DOWN_STR);

		cword_messages_output_message (
		    CWORD_S_YESNOGENERIC,
		    SCREEN0,
		    CWORD_Y_CHAR,
		    CWORD_N_CHAR,
		    CWORD_N_CHAR);

		temp_ch = cword_charui_get_ch (FALSE);

		if (toupper (temp_ch) == CWORD_Y_CHAR)
		{
		    /*
		     *  ?? Not spell checked - Bug ??
		     */

		    if (cword_seed__add_seed_loc (
			    grid_record->grid_details[count].light_template,
			    count,
			    grid_record) != CWORD_S_NORMAL)
		    {
			return (CWORD_S_FAILURE);
		    }
		}
		else
		{
		    return (CWORD_S_FAILURE);
		}
	    }
	}
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__check_is_seed_ok (
	int			index,
	char			*light_str,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      cword_g_defaults_record.check_spelling
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    char temp_ch;

    int status;

    /*
     *	Check that the seed isn't already in the grid before continuing?
     */

    status = cword_grid_check_disallowed (
		index,
		light_str,
		grid_record);

    if (status != CWORD_S_NORMAL)
    {
	cword_messages_output_message (
	    CWORD_LIGHTALREADYINGRD,
	    SCREEN0);

	return (CWORD_S_FAILURE);
    }

    /*
     *	Put seed in grid_record temporarily to check that it doesn't clash with
     *	existing definitions!
     */

    strcpy (grid_record->grid_details[index].light, light_str);

    /*
     *  Check if the inputted light co-exists with its neighbours?
     */

    if (cword_seed__do_seeds_coexist (grid_record) == FALSE)
    {
	/*
	 *  Blank light and return error :-(.
	 */

	cword_seed__set_light_blank (
	    grid_record->grid_details[index].light_len,
	    grid_record->grid_details[index].light);

	return (CWORD_S_FAILURE);
    }

    /*
     *  Update the light_template strings with the successful result.
     */

    cword_seed__update_light_temps (grid_record);

    /*
     *	Further check that no other lights were inadventently completely filled
     *	as a result of adding the current seed.
     */

    status = cword_seed__check_light_temps (
		index,
		grid_record);

    if (CWORD_S_NORMAL == status)
    {
	/*
	 *  Check that the inputted light is spelt correctly, if necessary!
	 */

	if (cword_g_defaults_record.check_spelling == TRUE)
	{
	    status = cword_match_spell_check (light_str);

	    if (CWORD_S_NORMAL != status)
	    {
		cword_messages_output_message (
			CWORD_SEEDSPELLERR,
			SCREEN0,
			CWORD_Y_CHAR,
			CWORD_N_CHAR, 
			CWORD_Y_CHAR);

		temp_ch = cword_charui_get_ch (FALSE);

		if (toupper (temp_ch) == CWORD_N_CHAR)
		{
		    cword_seed__set_light_blank (
			grid_record->grid_details[index].light_len,
			grid_record->grid_details[index].light);

		    /*
		     *  Don't accept the light as is!
		     */

		    cword_messages_output_message (
			CWORD_SEEDSPELLOUT,
			SCREEN0);

		    return (CWORD_S_FAILURE);
		}
	    }
	}
	else
	{
	    /*
	     *	Probably the easiest way of handling the spell checking of user
	     *	inputted lights - not do it at all :-).
	     */
	}
    }
    else
    {
	/*
	 *  Cannot accept the seed as it as it causes other problems in grid
	 *  definition.
	 */

	cword_messages_output_message (
	    CWORD_SEEDSPELLOUT,
	    SCREEN0);

	cword_seed__set_light_blank (
	    grid_record->grid_details[index].light_len,
	    grid_record->grid_details[index].light);

	return (CWORD_S_FAILURE);
    }

    /*
     *	Let the calling function do the needful of correctly updating the grid
     *	definition.
     */

    cword_seed__set_light_blank (
	grid_record->grid_details[index].light_len,
	grid_record->grid_details[index].light);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__get_seed_str (
	int			index,
	CWORD_T_GRID_STRUCT	*grid_record,
	char			*light_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to obtain from the user the seed which to insert in the grid
 *	index already found.
 *
 *  FORMAL PARAMETERS:
 *
 *	index			() 
 *	*grid_record		() 
 *	*light_str		() 
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    int	status,
        count = 0;

    BOOLEAN is_template_empty = TRUE;

    light_str[0] = (char) 0;

    while (grid_record->grid_details[index].light_template[count] != (char) 0)
    {
        if (grid_record->grid_details[index].light_template[count] != 
               (char) BLANK_CHAR)
        {
            is_template_empty = FALSE;

            break;
        }

        count++;
    }

    if (is_template_empty == FALSE)
    {
        cword_messages_output_message (
    	    CWORD_LIGHTMATCHTHIS,
            SCREEN0,
            grid_record->grid_details[index].light_template);
    }

    cword_messages_output_message (
	CWORD_SEEDLIGHTMSG,
	SCREEN0,
	grid_record->grid_details[index].light_len);

    /*
     *	Input is dotted to help the user determine how long the desired input
     *	is to be.
     */

    cword_charui_highlight_dots (grid_record->grid_details[index].light_len);

    /*
     *	Assume that the length of light required is CWORD_SIZEOF_WORD as is
     *	seen below!
     */

    status = cword_charui_get_str (
		CHARACTER,
		BLANK_OK,
		CWORD_SIZEOF_WORD,
		light_str);

    if (status != CWORD_S_NORMAL)
    {
	cword_messages_output_message (
	    CWORD_S_SEEDINVALIDSEED,
	    SCREEN0);

	return (status);
    }

    /* 
     *  Translate inputted light to lower case for completeness 
     */

    if (strlen (light_str) != grid_record->grid_details[index].light_len) 
    {
	cword_messages_output_message (
	    CWORD_S_SEEDINVALIDSEED,
	    SCREEN0);

	light_str[0] = (char) 0;

	return (CWORD_S_FAILURE);
    }
    else
    {
        cword_word_lower_str (light_str);

	/*
	 *  Check that the seed isn't already appearing in the grid definition.
	 */

	status = cword_seed__check_is_seed_ok (
		    index,
		    light_str,
		    grid_record);

	return (status);
    }

}



PRIVATE void cword_seed__output_light_opts (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to display the lights that are available (or already filled)
 *	for seeing.  This gives the user an opportunity to decide which lights
 *	are to be seeded before filling begins.
 *
 *	This function is called each time the seeding menu is entered and
 *	executed.
 *
 *  FORMAL PARAMETERS:
 *
 *      *grid_record		(r) Pointer to instance of a CWORD_T_GRID_STRUCT.
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_g_defaults_record.letter_weight_algorithm
 *	cword_g_defaults_record.light_generation_algorithm
 *	cword_g_defaults_record.automatic_light_generation
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None as ther should be no problems encountered in this function
 *--
 */

{

    int count,
	line_num;

    BOOLEAN are_over_99_clues;

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (CWORD_CLUESFORLIGHTING, SCREEN0);

    /*
     *  ?? Not the best way of handling this I think ??
     *
     *  CWORD_SEEDED_CHAR - Seeded
     *	CWORD_AVAIL_CHAR - Available
     */

    are_over_99_clues = (cword_seed_get_high_clue_num (grid_record) > 99) ? TRUE : FALSE;

    line_num = 2;
    for (count = 0; count < grid_record->clue_sum; count++)
    {
	/*
	 *  It's a bit tight using LIGHTS_PER_LINE for crosswords over 99 clues
	 *  during seeding but should be OK until further notice.
	 */

	if ( !(count % ((are_over_99_clues == FALSE) ? LIGHTS_PER_LINE : (LIGHTS_PER_LINE))))
	{
	    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

	    line_num++;
	}

	(void) cword_charui_check_page_break (
		    NOT_FINISHED,
		    CWORD_S_PRESSRETURN,
		    &line_num);

	cword_messages_output_message (
	    CWORD_GRIDOUTPUTCLUE,
	    SCREEN0,
	    are_over_99_clues == FALSE ? 2 : 3,
	    grid_record->grid_details[count].clue_number,
	    grid_record->grid_details[count].clue_direction 
				? CWORD_ACROSS_STR : CWORD_DOWN_STR,  
	    (cword_seed__is_light_blank (
		grid_record->grid_details[count].light_len,
		grid_record->grid_details[count].light) == FALSE)
				? CWORD_SEEDED_CHAR : CWORD_AVAIL_CHAR);
    }

    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);
    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

    return;

}



/*
 *  ?? Make into generic function (probably in the module charui) to handle the
 *  output of simple one-line help ??
 *
 *  ?? Variable argument list consisting of an array of pointers to character
 *  strings containing the strings to be outputted.  Similar to argv in
 *  structure I think ??
 */

PRIVATE void cword_seed__output_help ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      None!
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None!
 *--
 */

{

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	CWORD_S_GENERICHELP,
	SCREEN0);

    printf ("## A - Add a new seed to the grid\n");
    printf ("## D - Delete an existing seed from the grid\n");
    printf ("## L - List all seeded lights in the grid\n");
    printf ("## R - Reset all seeded lights to NULL in the grid\n");
    printf ("## V - View grid with seeded lights\n");
    printf ("## F - Fill existing grid\n");
    printf ("## Q - Quit seeding and return to menu\n");
    printf ("## H - Get this screen :-) \n");

    cword_messages_output_message (
	CWORD_WAITINGENDED,
	SCREEN0);

    return;

}



/*
 *  ?? Function signature:
 *
 *	PRIVATE char get_option (
 *			int message_id,
 *			char *allowable_str) ??
 *
 */

/*
 *  ?? If numeric is inputted, I could assume that it is for an ADD command and
 *  act accordingly - difficult to do without the use of module global
 *  variables ??
 */

PRIVATE void cword_seed__get_option (
	char			*option)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None!
 *--
 */

{

    char temp_ch;

    cword_messages_output_message (
	CWORD_S_SEEDCREATECOMM,
	SCREEN0,
	ADD_CHAR);

    temp_ch = cword_charui_get_ch (FALSE);

    /*
     *  ?? Maybe put this function inside cword_charui_get_ch ??
     */

    if (temp_ch != (char) 0)
    {
	cword_messages_output_message (
	    CWORD_S_MENUINPUT,
	    TRACE2,
	    temp_ch);
    }

    if ( (toupper (temp_ch) == ADD_CHAR) || (temp_ch == (char) 0) )
    {
	/*
	 *  If nothing is entered default to ADD_CHAR!
	 */

	*option = ADD_CHAR;
    }
    else if (toupper (temp_ch) == QUIT_CHAR)
	*option = QUIT_CHAR;
    else if (toupper (temp_ch) == FILL_CHAR)
	*option = FILL_CHAR;
    else if (toupper (temp_ch) == VIEW_CHAR)
	*option = VIEW_CHAR;
    else if (toupper (temp_ch) == RESET_CHAR)
	*option = RESET_CHAR;
    else if (toupper (temp_ch) == DELETE_CHAR)
	*option = DELETE_CHAR;
    else if (toupper (temp_ch) == LIST_CHAR)
	*option = LIST_CHAR;
    else if ( (toupper (temp_ch) == HELP_CHAR) || (temp_ch == QUESTION_CHAR) )
	*option = HELP_CHAR;
    else
    {
	/*
	 *  Invalid response
	 */

	*option = INVALID_CHAR;
    }

    /*
     *  No return value really necessary.
     */

    return;

}



PRIVATE int cword_seed__add_seed (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Seed successfully added.
 *	CWORD_S_FAILURE		Error adding seed to evlving grid.
 *--
 */

{

    char seed_str[CWORD_SIZEOF_CELLDIM + 1];

    int index;

    /*
     *	Get the seed index by the user inputting clue number and clue
     *	direction.
     */

    if (cword_seed__get_seed_index (
	    ADDITION,
	    grid_record,
	    &index) != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *	Having successfully found the clue index the user then has to input a
     *	valid seed before continuing.
     */

    if (cword_seed__get_seed_str (
	    index,
	    grid_record,
	    seed_str) != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *	Once a valid index and valid seed string has been found, add it to the
     *	list of seeds in the evolving grid.
     */

    if (cword_seed__add_seed_loc (
	    seed_str,
	    index,
	    grid_record) != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__delete_seed (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Seed successfully deleted.
 *	CWORD_S_FAILURE		Seed unsuccessfully deleted.
 *--
 */

{

    int index;

    if (cword_seed_is_seed_available (grid_record) == FALSE)
    {
	cword_messages_output_message (
	    CWORD_S_SEEDSNONEAVAIL, 
	    SCREEN0);

	return (CWORD_S_NORMAL);
    }

    /*
     *	Get the seed index.
     */

    if (cword_seed__get_seed_index (
	    DELETION,
	    grid_record,
	    &index) != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *	Once a valid seed index has been found, the seed can be deleted!
     */

    if (cword_seed__delete_seed_loc (
	    index,
	    grid_record) != CWORD_S_NORMAL)
    {
	return (CWORD_S_FAILURE);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__list_seeds (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int count,
        line_num;

    BOOLEAN are_over_99_clues;

    if (cword_seed_is_seed_available (grid_record) == FALSE)
    {
	cword_messages_output_message (
	    CWORD_S_SEEDSNONEAVAIL, 
	    SCREEN0);

	return (CWORD_S_NORMAL);
    }

    CWORD_OS_CLEAR_SCREEN ();

    cword_messages_output_message (
	CWORD_S_SEEDSLISTHEAD, 
	SCREEN0);

    are_over_99_clues = (cword_seed_get_high_clue_num (grid_record) > 99) ? TRUE : FALSE;

    line_num = 2;
    for (count = 0; count < grid_record->clue_sum; count++)
    {
	if (grid_record->grid_details[count].grid_entry == LIGHT_SEEDED)
	{
	    cword_messages_output_message (
		CWORD_S_SEEDLISTSEEDS, 
		SCREEN0,
		are_over_99_clues == FALSE ? 2 : 3,
		grid_record->grid_details[count].clue_number, 
		grid_record->grid_details[count].clue_direction 
			? CWORD_ACROSS_STR : CWORD_DOWN_STR,
		grid_record->grid_details[count].light);

	    line_num++;

	    (void) cword_charui_check_page_break (
			NOT_FINISHED,
			CWORD_S_PRESSRETURN,
			&line_num);

	}
    }

    cword_messages_output_message (CWORD_WAITINGENDED, SCREEN0);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__reset_seeds (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    char temp_ch;

    if (cword_seed_is_seed_available (grid_record) == FALSE)
    {
	cword_messages_output_message (
	    CWORD_S_SEEDSNONEAVAIL, 
	    SCREEN0);

	return (CWORD_S_NORMAL);
    }

    cword_messages_output_message (
	CWORD_S_SEEDDELALLSEEDS, 
	SCREEN0,
	CWORD_Y_CHAR,
	CWORD_N_CHAR,
	CWORD_N_CHAR);

    temp_ch = cword_charui_get_ch (FALSE);

    if (toupper (temp_ch) == CWORD_Y_CHAR)
    {
	int count;

	for (count = 0; count < grid_record->clue_sum; count++)
	{
	    if (grid_record->grid_details[count].grid_entry == LIGHT_SEEDED)
	    {
		cword_seed__delete_seed_loc (
		    count,
		    grid_record);
	    }
	}
    }
    else
    {
	cword_messages_output_message (
	    CWORD_S_SEEDNORESET, 
	    SCREEN0);
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__view_seed_grid (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CROSSWORD_GRID grid_contents;

    if (cword_seed_is_seed_available (grid_record) == FALSE)
    {
	cword_messages_output_message (
	    CWORD_S_SEEDSNONEAVAIL, 
	    SCREEN0);

	return (CWORD_S_NORMAL);
    }

    cword_grid_get_grid_contents (
	NOBESTWORDPHASE,
	grid_record,
	grid_contents);

     /*
      *	 The message CWORD_S_SEEDVIEWGRID will be displayed before the grid is, 
      *	 using cword_charui_output_grid!
      */

    cword_charui_output_grid (
	CWORD_S_SEEDVIEWGRID,
	(SCREEN0 | TRACE3),
	FALSE,
	grid_record->x_dimensions,
	grid_record->y_dimensions,
	grid_contents);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_seed__finish_seeding (
	int			quit_or_fill,
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_seed_l_grid_updated
 *	cword_seed_l_seed_file_read
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion.
 *--
 */

{

    cword_messages_output_message (
	(quit_or_fill == FILL) ? CWORD_S_SEEDFILLNOW : CWORD_S_SEEDQUITNOW, 
	SCREEN0);

    /*
     *	If no lights are seeded then no need to do *anything*.
     */

    if (cword_seed_is_seed_available (grid_record) == FALSE)
    {
	/*
	 *  ?? What happens if there was a seed file inputted?  Do I delete it
	 *  or just ignore it altogether ??
	 */

	if (cword_seed_l_grid_updated == TRUE)
	{
	    DEBUG_TRACE ("Grid was altered during seeding\n");
	}

	if (cword_seed_l_seed_file_read == TRUE)
	{
	    cword_messages_output_message (
		CWORD_S_SEEDFILENOTWRN, 
		SCREEN2);
	}

	cword_messages_output_message (
	    CWORD_S_SEEDSNONE, 
	    (TRACE0 | SCREEN0));

	return (CWORD_S_NORMAL);
    }

    if (cword_seed_l_grid_updated == TRUE)
    {
	char temp_ch;

	 /*
	  *  There has been changes in the grid definition and these changes
	  *  need to be saved for possible further use - save grid.
	  */

	cword_messages_output_message (
	    CWORD_S_SEEDSAVEQ, 
	    SCREEN0,
	    CWORD_Y_CHAR,
	    CWORD_N_CHAR,
	    (quit_or_fill == FILL) ? CWORD_Y_CHAR : CWORD_N_CHAR);

	temp_ch = cword_charui_get_ch (FALSE);

	if (quit_or_fill == FILL)
	{
	    if (toupper (temp_ch) != CWORD_N_CHAR)
		cword_seed__write_seed_grid (grid_record);
	    else
		cword_messages_output_message (
		    CWORD_S_SEEDNOTSAVING, 
		    SCREEN0);
	}
	else
	{
	    if (toupper (temp_ch) == CWORD_Y_CHAR)
		cword_seed__write_seed_grid (grid_record);
	    else
		cword_messages_output_message (
		    CWORD_S_SEEDNOTSAVING, 
		    SCREEN0);
	}
    }
    else
    {
	/*
	 *  No change in grid so no need to do anything :-).
	 */
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Add a kill list option.  This option would allow the user to add words
 *  to the disallowed list from which the fill robot would check before
 *  deciding whether this light is valid for inclusion in the grid ??
 */

PUBLIC int cword_seed_input_seeds (
	CWORD_T_GRID_STRUCT	*grid_record)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to handle the input of seed words into the grid to be
 *	generated.  Temporary function to help with testing.
 *
 *  FORMAL PARAMETERS:
 *
 *	*grid_record		(r/w) Grid record containing lights to be seeded.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *	cword_seed_l_grid_updated
 *	cword_seed_l_seed_file_read
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Ready to fill grid after some seeding possibly.
 *	CWORD_S_SEEDQUITNOW	Quit seeding and return to menu.
 *	CWORD_S_FAILURE		Serious error recorded.  Cannot continue.
 *--
 */

{

    char option;

    int	status;

    int highest_clue;

    BOOLEAN is_first_time;

    cword_seed_l_grid_updated = FALSE;
    cword_seed_l_seed_file_read = FALSE;
    is_first_time = TRUE;

    cword_messages_output_message (CWORD_SEEDLIGHTS, TRACE2);

#if 0
    /*
     *	Sleep for 2 seconds '-) - gives the impression that something is taking
     *	place when in fact there's nothing.
     */

    cword_utils_sleep_for (2);
#endif

    /*
     *  Read seeds file if it exists?
     */

    status = cword_seed__read_seed_grid (grid_record);

    if (CWORD_S_NORMAL == status)
    {
	/*
	 *  If seed file is successfully read then come in here for a little
	 *  while!
	 */

	cword_seed__update_light_temps (grid_record);

	if (status != CWORD_S_NORMAL)
	{
	    return (CWORD_S_FAILURE);
	}
    }
    else
    {
        /*
	 *  No seed file read *or* none found!
	 */

	cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);
    }

    /*
     *	Get highest clue number value for messages later in the function.
     *
     *  ?? highest_clue not yet used ??
     */

    highest_clue = cword_seed_get_high_clue_num (grid_record);

    do 
    {
	/*
	 *  Output to the screen the lights which can be seeded each time this
	 *  loop is iterated.
	 */

	cword_seed__output_light_opts (grid_record);

        if (is_first_time == TRUE)
	{
	    /*
	     *  Only display this message once!
	     */

	    cword_messages_output_message (
		CWORD_S_HELPAVAILABLE, 
		SCREEN0,
		HELP_CHAR);

	    is_first_time = FALSE;
	}

	/*
	 *  Check and see is the grid fully seeded.
	 */

	if (cword_grid_is_fully_seeded (grid_record) == TRUE)
	{
	    cword_messages_output_message (
		CWORD_S_SEEDFULLGRID, 
		SCREEN0);
	}

	cword_seed__get_option (&option);

	switch (option)
	{
	case (ADD_CHAR):
	    status = cword_seed__add_seed (grid_record);

	    status = CWORD_S_NORMAL;

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    break;

	case (DELETE_CHAR):
	    status = cword_seed__delete_seed (grid_record);

	    status = CWORD_S_NORMAL;

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    break;

	case (LIST_CHAR):
	    status = cword_seed__list_seeds (grid_record);

	    status = CWORD_S_NORMAL;

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    break;

	case (RESET_CHAR):
	    status = cword_seed__reset_seeds (grid_record);

	    status = CWORD_S_NORMAL;

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    break;

	case (VIEW_CHAR):
	    status = cword_seed__view_seed_grid (grid_record);

	    status = CWORD_S_NORMAL;

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    break;

	case (FILL_CHAR):
	    status = cword_seed__finish_seeding (
			FILL,
			grid_record);

	    status = CWORD_S_SEEDFILLNOW;

	    break;

	case (QUIT_CHAR):
	    status = cword_seed__finish_seeding (
			QUIT,
			grid_record);

	    /*
	     *	No need for a pause here as we will be falling through back
	     *	into charui where the next _pause () will  being executed as a
	     *	result of being in here.
	     */

	    status = CWORD_S_SEEDQUITNOW;

	    break;

	case (HELP_CHAR):
	    cword_seed__output_help ();

	    cword_charui_pause_ui (CWORD_S_PRESSRETURN, FALSE);

	    status = CWORD_S_NORMAL;

	    break;

	default:
	    /*
	     *	?? Continue as if nothing happened.  Should I even include the
	     *	CHOICENOTVALID message ??
	     */

	    cword_messages_output_message (
			CWORD_CHOICENOTVALID,
			SCREEN0);

	    status = CWORD_S_NORMAL;

	    break;
	}
    }
    while (status == CWORD_S_NORMAL);

    if (status == CWORD_S_SEEDFILLNOW)
	return (CWORD_S_NORMAL);
    else if (status == CWORD_S_SEEDQUITNOW)
	return (status);
    else
    {
	/*
	 *  All other exceptions are just flat-out errors.
	 */

	return (CWORD_S_FAILURE);
    }

}
