/*
 *  ?? Add a function to sort the linked list or to get rid of duplicate 
 *  items on said list ??
 */

/*
 *++
 *  FACILITY:
 *
 *      strll.c
 *
 *  ABSTRACT:
 *
 *      Module containing character linked list functionality.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      20 March 1995
 *--
 */

/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#ifdef STRLL
#define MESSAGES_DEFINED 1
#endif

#include "main.h"
#include "utils.h"
#include "messages.h"
#include "memman.h"
#include "strll.h"
#include "charui.h"
#include "defaults.h"



#ifndef STRLL

PUBLIC int cword_strll_add_str_and_val (
	int			options_mask,
	float			add_str_val,
	char			*add_str,
	CWORD_T_STR_LL          **str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *  Options mask will take one or more of four values:
 *
 *	1. SORTED   - The link list will add the item in a sorted manner
 *	2. UNSORTED - The link list will add the item in an unsorted manner
 *	3. UNIQUE   - The link list will contain only one copy of the input string
 *	4. MULTIPLE - The link list can contain multiple copies of the input string
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Added input string to linked list.
 *	CWORD_S_FAILURE		Error adding input string to linked list.
 *--
 */

{

    CWORD_T_STR_LL *temp_ll;

    BOOLEAN is_inserted = FALSE;

    if (add_str == (char *) 0)
    {
	printf ("# Error adding as add_str is empty\n");

	return (CWORD_S_FAILURE);
    }

    if (options_mask & UNIQUE)
    {
	/*
	 *  Check whether add_str is already in the linked list before
	 *  continuing otherwise add possibly multiple times.
	 */

	if (cword_strll_is_str_in_str_ll (
		add_str,
		*str_ll) == TRUE)
	{
	    return (CWORD_S_NORMAL);
	}
    }
    else if (options_mask & MULTIPLE)
    {
	/*
	 *  Nothing is correct i.e. don't check wehether the string being added
	 *  is already in the linked list.
	 */
    }
    else
    {
	printf ("# No indication whether to add string UNIQUEly/MULTIPLEly\n");

    	return (CWORD_S_FAILURE);
    }

    DEBUG_TRACE (
	"# add_str: %s\n",
	add_str);

    cword_strll_goto_start_str_ll (str_ll);

    /*
     *  Created new link and insert string into it.
     */

    temp_ll = cword_memman_create_str_ll (strlen (add_str));

    if (temp_ll == (CWORD_T_STR_LL *) 0)
    {
	return (CWORD_S_FAILURE);
    }

    strcpy (temp_ll->str, add_str);

    temp_ll->str_val = add_str_val;

    if (*str_ll == (CWORD_T_STR_LL *) 0)
    {
	DEBUG_TRACE ("Adding to empty list\n");

	*str_ll = temp_ll;
    }
    else
    {
	/*
	 *  Use options_masks to add as SORTED or UNSORTED.
	 */

	if (options_mask & SORTED)
	{
	    /*
	     *  Search for right place to insert the new link into the already
	     *  extant linked list.
	     */

	    while (*str_ll != (CWORD_T_STR_LL *) 0)
	    {
		if (strcmp (add_str, (*str_ll)->str) < 0)
		{
		    /*
		     *  Add add_str to str_ll.
		     */

		    if ((*str_ll)->previous != (CWORD_T_STR_LL *) 0)
		    {
			/*
			 *	Go back one element and add if we are not at the start
			 *	of the list.
			 */

			*str_ll = (*str_ll)->previous;
		    }

#if 0
		    if ((*str_ll)->next == (CWORD_T_STR_LL *) 0)
		    {
			/*
			 *  ?? Add add_str to end of list ??
			 */

			/*
			 *  ?? Will never be called ??
			 */

			DEBUG_TRACE ("Add at end of list\n");

			temp_ll->next = (CWORD_T_STR_LL *) 0;
			temp_ll->previous = *str_ll;
			(*str_ll)->next = temp_ll;

			is_inserted = TRUE;
		    }
		    /* else */
#endif

		    if ((*str_ll)->previous == (CWORD_T_STR_LL *) 0)
		    {
			/*
			 *  Add add_str to start of the list.
			 */

			DEBUG_TRACE ("Add at start of list\n");

			temp_ll->next = *str_ll;
			temp_ll->previous = (CWORD_T_STR_LL *) 0; /* or (*str_ll)->previous */
			(*str_ll)->previous = temp_ll;

			is_inserted = TRUE;
		    }
		    else
		    {
			/*
			 *  Add add_str to middle of the list.
			 */

			DEBUG_TRACE ("Add in middle of list\n");

			temp_ll->next = (*str_ll)->next;
			temp_ll->previous = *str_ll;
			(*str_ll)->next->previous = temp_ll;
			(*str_ll)->next = temp_ll;

			is_inserted = TRUE;
		    }

		    break;	    
		}
		else if ((*str_ll)->next == (CWORD_T_STR_LL *) 0)
		{
		    /*
		     *  Add to the end of the list as we've check all previous
		     *  entries without success.
		     */

		    DEBUG_TRACE ("Add onto end of list\n");

		    temp_ll->next = (CWORD_T_STR_LL *) 0;
		    temp_ll->previous = *str_ll;
		    (*str_ll)->next = temp_ll;

		    is_inserted = TRUE;

		    break;
		}
		else
		{
		    *str_ll = (*str_ll)->next;
		}
	    }
	}
	else if (options_mask & UNSORTED)
	{
	    /*
	     *  ?? Add add_str to end of list ??
	     */

	    DEBUG_TRACE ("Add at end of list - UNSORTED\n");

	    /*
	     *  First go to the end of the list.
	     */

	    cword_strll_goto_end_str_ll (str_ll);

	    temp_ll->next = (CWORD_T_STR_LL *) 0;
	    temp_ll->previous = *str_ll;
	    (*str_ll)->next = temp_ll;

	    is_inserted = TRUE;
	}
	else
	{
	    /*
	     *	?? Don't allow silent additions to the linked list.  Make sure
	     *	that the user has to specify how the new element is to be added ??
	     */

	    printf ("# Error adding string: no sort type specified\n");
	}

	/*
	 *  If the new link hasn't been added then output error.
	 */

	if (is_inserted == FALSE)
	{
	    printf (
		"# Error adding: %s\n",
		add_str);
	}
    }

    cword_strll_goto_start_str_ll (str_ll);

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Add in a option to delete UNIQUEly or by MULTIPLEs ??
 */

PUBLIC int cword_strll_del_str (
	char			*del_str,
	CWORD_T_STR_LL         **str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully deleted string from linked list.
 *	CWORD_S_FAILURE		Error deleting string from linked list.
 *--
 */

{

    CWORD_T_STR_LL *temp_ll;

    if (del_str == (char *) 0)
    {
	printf ("# Error deleting as del_str is empty\n");

	return (CWORD_S_FAILURE);
    }

    if (cword_strll_is_str_ll_empty (*str_ll) == TRUE)
    {
	return (CWORD_S_FAILURE);
    }

    /*
     *  Check whether del_str is in the linked list before continuing.
     */

    if (cword_strll_is_str_in_str_ll (
	    del_str,
	    *str_ll) == FALSE)
    {
	printf (
	    "# word %s not found on list\n",
	    del_str);

	return (CWORD_S_FAILURE);
    }

    DEBUG_TRACE (
	"del_str: %s\n",
	del_str);

    /*
     *  ?? Add check whether the link has been successfully deleted ??
     */

    while ((*str_ll) != (CWORD_T_STR_LL *) 0)
    {
	if (strcmp ((*str_ll)->str, del_str) == 0)
	{
	    if (cword_strll_number_str_ll (*str_ll) == 1)
	    {
		DEBUG_TRACE ("Delete only item on list\n");

		cword_memman_free_str_ll (str_ll);
	    }
	    else
	    {
		/*
		 *  Delete del_str from str_ll.
		 */

		temp_ll = *str_ll;

		if ((*str_ll)->next == (CWORD_T_STR_LL *) 0)
		{
		    /*
		     *  ?? At the end of the list ??
		     */

		    DEBUG_TRACE ("Delete at end of list\n");

		    *str_ll = (*str_ll)->previous; 
		    (*str_ll)->next = (CWORD_T_STR_LL *) 0;
		}
		else if ((*str_ll)->previous == (CWORD_T_STR_LL *) 0)
		{
		    /*
		     *  ?? At the start of the list ??
		     */

		    DEBUG_TRACE ("Delete at start of list\n");

		    *str_ll = (*str_ll)->next;
		    (*str_ll)->previous = (CWORD_T_STR_LL *) 0;
		}
		else
		{
		    /*
		     *  ?? In the middle of the list ??
		     */

		    DEBUG_TRACE ("Delete in middle of list\n");

		    (*str_ll)->previous->next = (*str_ll)->next;
		    (*str_ll)->next->previous = (*str_ll)->previous;
		    *str_ll = (*str_ll)->previous;
		}

		DEBUG_TRACE (
		    "Deleting: %s from str_ll\n",
		    temp_ll->str);

		cword_memman_free_str_ll (&temp_ll);
	    }

	    break;
	}
	else
	{
	    *str_ll = (*str_ll)->next;
	}
    }

    cword_strll_goto_start_str_ll (str_ll);

    return (CWORD_S_NORMAL);

}



PUBLIC void cword_strll_free_str_ll (
	CWORD_T_STR_LL         **str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CWORD_T_STR_LL *temp_ll;

    while (*str_ll != (CWORD_T_STR_LL *) 0)
    {
	temp_ll = *str_ll;

	*str_ll = (*str_ll)->next;

	cword_memman_free_str_ll (&temp_ll);
    }

    *str_ll = (CWORD_T_STR_LL *) 0;

    return;

}



/*
 *  ?? ...count_str_ll ??
 */

PUBLIC unsigned long cword_strll_number_str_ll (
	CWORD_T_STR_LL         *str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CWORD_T_STR_LL *temp_ll = str_ll;

    unsigned long count = 0;

    cword_strll_goto_start_str_ll (&temp_ll);

    while (temp_ll != (CWORD_T_STR_LL *) 0)
    {
	count++;

	temp_ll = temp_ll->next;
    }

    return (count);

}



PUBLIC BOOLEAN cword_strll_is_str_in_str_ll (
	char			*check_str,
	CWORD_T_STR_LL         *str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CWORD_T_STR_LL *temp_ll = str_ll;

    if (check_str == (char *) 0)
    {
	printf ("# Error checking CHAR_LL with add_str\n");
    }    

    while (temp_ll != (CWORD_T_STR_LL *) 0)
    {
	if (strcmp (temp_ll->str, check_str) == 0)
	{
	    return (TRUE);
	}

	temp_ll = temp_ll->next;
    }

    return (FALSE);

}



PUBLIC BOOLEAN cword_strll_is_str_ll_empty (
	CWORD_T_STR_LL         *str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    if (str_ll == (CWORD_T_STR_LL *) 0)
    {
	return (TRUE);
    }
    else
    {
	return (FALSE);
    }

}



PUBLIC BOOLEAN cword_strll_goto_start_str_ll (
	CWORD_T_STR_LL         **str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CWORD_T_STR_LL *temp_ll;

    if (cword_strll_is_str_ll_empty (*str_ll) == TRUE)
    {
	return (TRUE);
    }    

    temp_ll = *str_ll;
    
    while (temp_ll->previous != (CWORD_T_STR_LL *) 0)
    {
	temp_ll = temp_ll->previous;
    }

    *str_ll = temp_ll;

    return (TRUE);

}



PUBLIC BOOLEAN cword_strll_goto_end_str_ll (
	CWORD_T_STR_LL         **str_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    CWORD_T_STR_LL *temp_ll;

    if (cword_strll_is_str_ll_empty (*str_ll) == TRUE)
    {
	return (TRUE);
    }    

    temp_ll = *str_ll;
    
    while (temp_ll->next != (CWORD_T_STR_LL *) 0)
    {
	temp_ll = temp_ll->next;
    }

    *str_ll = temp_ll;

    return (TRUE);

}



/*
 *  ?? Add an option that will number the link list elements e.g.
 *
 *  1. alert
 *  ..
 *  n. zelda ??
 */

PUBLIC int cword_strll_output_str_ll (
	CWORD_T_STR_LL         *str_ll,
	int			per_line)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to output the contents of a CWORD_T_STR_LL linked list
 *	dependent on the per_line parameter.
 *
 *  FORMAL PARAMETERS:
 *
 *      *str_ll		(r) CWORD_T_STR_LL link list
 *	per_line		(r) number of entries to be outputted per line of output
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *	CWORD_S_FAILURE		Error in completion
 *--
 */

{

    CWORD_T_STR_LL *temp_ll; 

    char output_str[CWORD_SIZEOF_LINE + 1];

    int	count,
	item_count = 0,
	line_count = 0,
	total = 0;

    /*
     *	?? Not happy with the implementation of output_str_ll at the moment as
     *	it should be possible to have functionality that will output in a nice
     *	regimented list all the items in the list ??
     *
     *	?? Get the length of the longest item in the list and then divide (this
     *	value + 1) into (CWORD_SIZEOF_LINE - 3), round down the result and
     *	output that many items per line to the screen ??
     */

    /*
     *	If value of per_line is 0 then output as many as possible to the line.
     */

    if (per_line == 0)
    {
#if 0
	/*
	 *  ?? If this scheme is to work then I will have to change how the
	 *  character strings containing the output is formed ??
	 */

	int longest = 0;

	temp_ll = str_ll;

	while (temp_ll != (CWORD_T_STR_LL *) 0)
	{
	    if (strlen (temp_ll->str) > longest)
		longest = strlen (temp_ll->str);

	    temp_ll = temp_ll->next;
	}

	if (longest == 0)
	{
	    printf ("# Trouble!\n");

	    return (CWORD_S_FAILURE);
	}

	item_count = (int) ((float) (CWORD_SIZEOF_LINE - 3) / (float) (longest + 1));
#endif

	/*
	 *  Put as many items as possible on the line - that's what it means I
	 *  think anyhow?
	 */

	 item_count = CWORD_SIZEOF_LINE;
    }
    else
	item_count = per_line;

    temp_ll = str_ll;

    while (temp_ll != (CWORD_T_STR_LL *) 0)
    {
	count = 0;
	memset (output_str, 0, sizeof (output_str));

	while ( (strlen (output_str) < (sizeof (output_str) - 1)) &&
		 (count < item_count) &&
		  (temp_ll != (CWORD_T_STR_LL *) 0) )
	{
	    /*
	     *  While (line not filled) or (count < item_count).
	     */

	    if (count == 0)
	    {
		if (strlen (temp_ll->str) > (sizeof (output_str) - 1))
		    return (CWORD_S_FAILURE);
		else
		{
		    strcpy (output_str, temp_ll->str);
		    count++;
		    total++;
		    temp_ll = temp_ll->next;
		}
	    }
	    else
	    {
		if ( (strlen (output_str) + strlen (temp_ll->str)) <
		      (CWORD_SIZEOF_LINE - 3) )
		{
		    strcat (output_str, " ");
		    strcat (output_str, temp_ll->str);

		    count++;
		    total++;
		    temp_ll = temp_ll->next;
		}
		else
		{
		    break;
		}
	    }
	}

	strcat (output_str, "\n");
	cword_messages_output_message (
		    CWORD_USERDEFINEDSTRING,
		    SCREEN0,
		    output_str);

	line_count++;

	/*
	 *  If there are still items to be displayed from the linked
	 *  list...
	 */

	if (cword_charui_check_page_break (
		(void *) temp_ll,
		CWORD_S_PRESSRETURNQUIT,
		&line_count) != CWORD_S_NORMAL)
	{
	    /*
	     *  User has decided that she/he has seen enough output and
	     *  wants to quit option.
	     */

	    break;
	}
    }

    cword_messages_output_message (
    	CWORD_OUTPUTELEMS,
	TRACE2,
	total,
    	total == 1 ? ITEM_STR : ITEMS_STR);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_strll_output_twin_str_ll (
	CWORD_T_STR_LL		*double_ll,
	int			per_line)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to output to standard output entries from a two element linked
 *	list in a pretty format.
 *
 *  FORMAL PARAMETERS:
 *
 *	double_ll		(r) Link list
 *	per_line		(r) Number of entries to be outputted per line of output
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		successful completion
 *	CWORD_S_FAILURE		error in completion
 *--
 */

{

    CWORD_T_STR_LL *temp_ll; 

    char output_str[CWORD_SIZEOF_LINE + 1];

    int	count,
	item_count = 0,
	line_count = 0,
	total = 0;

    /*
     *	Find out whether there is an even number of entries in the linked list
     *	before continuing.
     */

    if ( (cword_strll_number_str_ll (double_ll) % 2) != 0)
    {
	cword_messages_output_message (
	    CWORD_S_TWINOUTPUTERR,
	    SCREEN0);

	return (CWORD_S_FAILURE);
    }

    /* If value of per_line is 0 then output as many as possible to the line */

    if (per_line == 0)
	item_count = CWORD_SIZEOF_LINE;
    else
	item_count = per_line;

    temp_ll = double_ll;

    while (temp_ll != (CWORD_T_STR_LL *) 0)
    {
	count = 0;
	memset (output_str, 0, sizeof (output_str));

	while ( (strlen (output_str) < (sizeof (output_str) -1)) &&
		 (count < item_count) &&
		  (temp_ll != (CWORD_T_STR_LL *) 0) )
	{
	    /* 
	     *  While [line not filled] or [count < item_count].
	     */

	    if (count == 0)
	    {
		if ( (strlen (temp_ll->str) + strlen (temp_ll->next->str) + 1) > 
		      (sizeof (output_str) - 1) )
		{
		    return (CWORD_S_FAILURE);
		}
		else
		{
		    strcpy (output_str, temp_ll->str);
		    strcat (output_str, "-");
		    strcat (output_str, temp_ll->next->str);
		    count++;
		    total++;

		    /*
		     *  Skip two elements!
		     */

		    temp_ll = temp_ll->next->next;
		}
	    }
	    else
	    {
		strcat (output_str, " ");

		if ( (strlen (output_str) + strlen (temp_ll->str) + strlen (temp_ll->next->str) + 1) <= 
		      (sizeof (output_str) - 1) )
		{
		    strcat (output_str, temp_ll->str);
		    strcat (output_str, "-");
		    strcat (output_str, temp_ll->next->str);
		    count++;
		    total++;

		    /*
		     *  Skip two elements!
		     */

		    temp_ll = temp_ll->next->next;
		}
		else
		{
		    count += CWORD_SIZEOF_LINE;
		}
	    }
	}

	strcat (output_str, "\n");
	cword_messages_output_message (
	    CWORD_USERDEFINEDSTRING,
	    SCREEN0,
	    output_str);

	line_count++;

	/*
	 *  If there are still items to be displayed from the linked
	 *  list...
	 */

	if (cword_charui_check_page_break (
		(void *) temp_ll,
		CWORD_S_PRESSRETURNQUIT,
		&line_count) != CWORD_S_NORMAL)
	{
	    /*
	     *  User has decided that she/he has seen enough output and
	     *  wants to quit option.
	     */

	    return (CWORD_S_NORMAL);
	}
    }

    cword_messages_output_message (
    	CWORD_OUTPUTELEMS,
	TRACE2,
	total,
    	total == 1 ? ITEM_STR : ITEMS_STR);

    return (CWORD_S_NORMAL);

}



#else

CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;

/*
 *  ?? Delete much of this test harness when finished implementation and
 *  testing as it's only taking up space ??
 */

main (
	int			argv, 
	char			*argc[])

{

    CWORD_T_STR_LL *test_ll = (CWORD_T_STR_LL *) 0;

    /*
     *  Test the new bi-directional character linked list functionality.
     */

    printf ("Testing STRLL test harness\n");

}

#endif
