/*
 *  Simple program to purge the input text file of all tabs.  Every tab found
 *  in every input file will be replaced by TAB_WIDTH space characters.
 *
 *  This program will make input files more readable and easier to edit across
 *  multiple platforms!
 *
 *  A series of input files can be specified at the command line.  Currently no
 *  wildcards are accepted by the program (except when the wildcards are
 *  expanded first using the U*X shell of choice e.g. tcsh).
 *
 *  Author: James A. Lundon.
 *
 *  Date: 23 August 1995. 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define TAB_CHAR (char) '\t'

/*
 *  Define the number of spaces to be written when a tab is found
 *  in the input file.
 */

#define TAB_WIDTH 4

/*
 *  Define the character to replace TAB_WIDTH times in place of any
 *  tabs found in input file.
 */

#define SPACE_CH (char) ' '

main (
        int         argc,
        char        *argv[])

{

    FILE *input_fp,
         *output_fp;

    char out_file_str[101],
         input_ch;

    int count = 1,
        count1;

    short is_changed;

    if (argc < 2)
    {
        printf ("Usage: tabs_to_spaces [in_file]\n");

        exit (EXIT_FAILURE);
    }

    while (argv[count] != (char) 0)
    {
        if (access (argv[count], 4) != 0)
        {
            printf (
                "Access to %s is not valid\n",
                argv[count]);

            exit (EXIT_FAILURE);
        }

        input_fp = fopen (argv[count], "r");

        if (input_fp == (FILE *) 0)
        {
            printf (
                "Cannot open %s\n",
                argv[count]);

            exit (EXIT_FAILURE);
        }

        count1 = 0;
        sprintf (
            out_file_str,
            "%s%d",
            argv[count],
            count1);

        while (access (out_file_str, 0) == 0)
        {
            count1++;
            
            sprintf (
                out_file_str,
                "%s%d",
                argv[count],
                count1);        
        }

        output_fp = fopen (out_file_str, "w");

        if (input_fp == (FILE *) 0)
        {
            printf (
                "Cannot open %s\n",
                argv[count]);

            exit (EXIT_FAILURE);
        }

        is_changed = FALSE;
        while ((input_ch = fgetc (input_fp)) != EOF)
        {
            if (input_ch == TAB_CHAR)
            {
                is_changed = TRUE;
                for (count1 = 0; count1 < TAB_WIDTH; count1++)
                {
                    if (fputc (SPACE_CH, output_fp) == EOF)
                    {
                        printf ("Error writing to output file\n");

                        fclose (input_fp);
                        fclose (output_fp);

                        remove (out_file_str);
    
                        exit (EXIT_FAILURE);
                    }
                }
            }
            else
            {
                if (fputc (input_ch, output_fp) == EOF)
                {
                    printf ("Error writing to output file\n");

                    fclose (input_fp);
                    fclose (output_fp);

                    remove (out_file_str);

                    exit (EXIT_FAILURE);
                }            
            }
        }

        fclose (input_fp);
        fclose (output_fp);

        if (is_changed == TRUE)
        {
            if (remove (argv[count]) != 0)
            {
                printf (
                    "Error deleting %s\n",
                    argv[count]);

                exit (EXIT_FAILURE);
            }

            if (rename (out_file_str, argv[count]) != 0)
            {
                printf (
                    "Error renaming %s to %s\n",
                    out_file_str,
                    argv[count]);

                exit (EXIT_FAILURE);
            }
        }
        else
        {
            if (remove (out_file_str) != 0)
            {
                printf (
                    "Error deleting %s\n",
                    out_file_str);

                exit (EXIT_FAILURE);
            }        
        }

        if (is_changed == TRUE)
        {
            printf (
                "Processed %s\n",
                argv[count]);
        }
        else
        {
            printf (
                "Unchanged %s\n",
                argv[count]);        
        }

        count++;
    }

    exit (EXIT_SUCCESS);

}
