/*
 *++
 *  FACILITY:
 *
 *      unix.c
 *
 *  ABSTRACT:
 *
 *      UNIX specific functionality for Cword application.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      9 July 1993
 *--
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>

#ifndef __osf__
#include <sys/dir.h>
#else
#include <dirent.h>
#endif

#include "main.h"
#include "os.h"
#include "unix.h"
#include "memman.h"
#include "utils.h"
#include "strll.h"
#include "messages.h"



#ifndef _UNIX

#define CWORD_UNIX_C_CURRENT_DIR  "."
#define CWORD_UNIX_C_CLEAR_CMD	  "clear"
#define CWORD_UNIX_C_FORSLASH_STR "/"
#define CWORD_UNIX_C_FORSLASH_CH  '/'



PUBLIC int cword_unix_get_files (
	char			*dir_spec,
	char			*return_file_spec)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to return file specifications from an opened directory spec.
 *	Files from the same directory will continue to be returned until either
 *	a new directory spec is entered or the directory runs out of suitable
 *	files.
 *
 *  FORMAL PARAMETERS:
 *
 *      *dir_spec		(r) directory specification either full or 0
 *	*return_file_spec	(w) file specification of file in dir_spec
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Normal completion with file spec obtained
 *      CWORD_S_FAILURE 	error in completion	
 *--
 */

{

    static DIR *dir_p = (DIR *) 0;

#ifndef __osf__
    struct direct *file_name = (struct direct *) 0;
#else
    struct dirent *file_name = (struct dirent *) 0;
#endif
	    
    if (dir_spec != (char *) 0)
    {
	if (dir_p != (DIR *) 0)
	    closedir (dir_p);

	dir_p = opendir (dir_spec);

	if ((DIR *) 0 == dir_p)
	{
	    printf ("## Error opening directory - returning\n");

	    return_file_spec[0] = (char) 0;

	    return (CWORD_S_FAILURE);
	}
    }

    /*
     *	?? Warning: compiling this line given the warning "incompatible pointer
     *	type" on AIX ??
     */

    file_name = readdir (dir_p);

#ifndef __osf__
    if ((struct direct *) 0 == file_name)
#else
    if ((struct dirent *) 0 == file_name)
#endif
    {
	return_file_spec[0] = (char) 0;

	return (CWORD_S_NORMAL);
    }

    strcpy (return_file_spec, file_name->d_name);

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_unix_list_files (
	char			*directory,
	char		        *file_spec,
	CWORD_T_STR_LL		**file_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Wrapper function which gets a list of file specs from the *directory
 *	specification
 *
 *  FORMAL PARAMETERS:
 *
 *      *directory		(r) directory spec
 *	*file_spec		(r) file spec
 *	**file_ll		(w) link list of returned file specifications from 
 *				    *directory and matching *file_spec
 *
 *  FUNCTION VALUE:
 *
 *	CWORD_S_NORMAL		Normal completion
 *	CWORD_S_FAILURE		Error recorded
 *      CWORD_NOFILESFOUND	No files found 
 *--
 */

{

    char full_file_spec[CWORD_SIZEOF_FILESPEC + 1],
         full_dir_spec[CWORD_SIZEOF_FILESPEC + 1],
         refer_file_spec[CWORD_SIZEOF_FILESPEC + 1],
	 return_file_spec[CWORD_SIZEOF_FILESPEC + 1],
	 *tmp_str;

    int status;

    /*
     *	More processing needed here as the files are handled move functionally.
     */

    if (directory != (char *) 0)
    {
	strcpy (full_dir_spec, directory);

        strcpy (refer_file_spec, file_spec);
    }
    else
    {
        /*
	 *  Check file_spec in case there is a CWORD_UNIX_C_FORSLASH_STR in the
	 *  string.  If so then use the text upto the last
	 *  CWORD_UNIX_C_FORSLASH_STR as the directoy specification in which to
	 *  run cword_unix_get_files with.
         */

        if (strstr (file_spec, CWORD_UNIX_C_FORSLASH_STR) != (char *) 0)
        {
            /*
             *  Find directory first!
             */

            char temp_file_str[CWORD_SIZEOF_FILESPEC + 1];

            int count = strlen (file_spec) - 1;

            strcpy (temp_file_str, file_spec);

            while (temp_file_str[count] != CWORD_UNIX_C_FORSLASH_CH)
            {
                count--;
            }

            temp_file_str[count] = (char) 0;

            strcpy (full_dir_spec, temp_file_str);

            /*
             *  Find file next!
             */

            count = strlen (file_spec) - 1;

            strcpy (temp_file_str, file_spec);

            while (temp_file_str[count] != CWORD_UNIX_C_FORSLASH_CH)
            {
                count--;
            }
     
            tmp_str = &temp_file_str[++count];

            strcpy (refer_file_spec, tmp_str);
        }
        else
        {
            strcpy (full_dir_spec, CWORD_UNIX_C_CURRENT_DIR);

            strcpy (refer_file_spec, file_spec);
        }
    }
    
    *file_ll = (CWORD_T_STR_LL *) 0;
    return_file_spec[0] = (char) 0;

    status = cword_unix_get_files (
		full_dir_spec,
		return_file_spec);

    if (CWORD_S_FAILURE == status)
	return (CWORD_S_FAILURE);

    while ( (CWORD_S_NORMAL == status) && (return_file_spec[0] != (char) 0) )
    {
        if (cword_word_str_wild (
                refer_file_spec,
                return_file_spec) == CWORD_S_NORMAL)
        {
                sprintf (
		    full_file_spec,
		    "%s/%s",
		    full_dir_spec,
		    return_file_spec);

		cword_strll_add_str (
		    SORTED | UNIQUE,
		    full_file_spec,
		    file_ll);

		if (*file_ll == (CWORD_T_STR_LL *) 0)
		{
                    return (CWORD_S_FAILURE);
		}
	}

	status = cword_unix_get_files (
		    (char *) 0,
		    return_file_spec);
    }

    if (*file_ll != (CWORD_T_STR_LL *) 0)
	return (CWORD_S_NORMAL);
    else
	return (CWORD_NOFILESFOUND);

}



/*
 *  ?? Obsolete ??
 */

PUBLIC int cword_unix_file_exists (
	char			*file_spec)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Not coded yet!!!
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *
 *  SIDE EFFECTS:
 *
 *      [@description or none@]
 *
 *--
 */

{

    /*
     *  ?? Code this function ?? 
     */

    /* 
	int cword_unix_list_files (
	char		    *directory,
	char		    *file_spec,
	CWORD_T_STR_LL	    **file_ll)
     */

    printf ("## Not coded yet!\n");

    return (CWORD_S_NORMAL);

}



PUBLIC int cword_unix_get_time (  
	char			*time_string)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Function to return time to the calling function.
 *
 *  FORMAL PARAMETERS:
 *
 *      *time_string		(w) string to return time
 *
 *  IMPLICIT INPUTS:
 *
 *      None yet
 *
 *  IMPLICIT OUTPUTS:
 *
 *      None yet
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		normal completion
 *	CWORD_S_FAILURE		error gettin time
 *--
 */

{

    time_t seconds;

    time (&seconds);

    strcpy (time_string, (char *) ctime (&seconds));

    time_string[strlen(time_string) - 1] = (char) 0;

    return (CWORD_S_NORMAL);

}



PUBLIC void cword_unix_clear_screen ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Call system using the shell command "clear".  Not great but it should
 *	suffice.
 *
 *  FORMAL PARAMETERS:
 *
 *      None.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      None.
 *--
 */

{

#if 0

    /*
     *  Utilises curses processing to clear the screen!
     */

    initscr ();

    werase ();

    wrefresh ();

#endif

    int status;

    status = system (CWORD_UNIX_C_CLEAR_CMD);

    /*
     *  ?? Different Unix systems have different return values for system () ??
     */

    if ( (status == 127) || (status == -1) )
    {
	/*
	 *  system () not supported or other error recorded.
	 */

	cword_messages_output_message (
	    CWORD_S_SYSTEMCALLERR, 
	    (SCREEN0 | TRACE0));
    }

    return;

}



#else

main (
    int	    argc,
    char    *argv[])

{

    CWORD_T_STR_LL *file_ll;

    char file_name[255];

    int  status = CWORD_S_NORMAL,
	 count;

    while (TRUE)
    {

    	printf ("Input search path/file to return : ");
    	gets (file_name);

    	if ( file_name[0] == (char) 0 || 
    	      'q' == (char) tolower (file_name[0]) )
    	    break;

	status = cword_unix_list_files (
			".",
			file_name,
			&file_ll);

	if (CWORD_S_FAILURE == status)
	{
	    exit (EXIT_FAILURE);
	}

	if (CWORD_PARSEERROR == status)
	{
	    /*
	     *  Problem with core dump
	     */

	    cword_messages_output_message (
			    status,
			    SCREEN0);
	}

	if (CWORD_NOFILESFOUND == status)
	    printf ("\nNo files found\n\n");

	if (CWORD_S_NORMAL == status)
	{
	    printf ("\nListing of files found satisfying input string:\n");
	    count = 0;

	    while (file_ll != (CWORD_T_STR_LL *) 0)
	    {	
		printf ("%s\n", file_ll->word);
		file_ll = file_ll->next;
		count++;
	    }

	    if (count == 1)
		printf ("%d file found\n\n", count);
	    else
		printf ("%d files found\n\n", count);
	}

    }

    exit (EXIT_SUCCESS);

}

#endif
