/*
 *++
 *  FACILITY:
 *
 *      vms.c
 *
 *  ABSTRACT:
 *
 *      VMS specific functionality for Cword application.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      9 July 1993
 *--
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <descrip.h>
#include <rmsdef.h>
#include <ssdef.h>
#include <clidef.h>

#include "main.h"
#include "utils.h"
#include "memman.h"
#include "os.h"
#include "messages.h"
#include "defaults.h"
#include "strll.h"

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#ifndef _VMS

PRIVATE int cword_vms__get_files (
	char			*full_file_spec,
	char			*return_file_spec);



PRIVATE int cword_vms__get_files (
	char			*full_file_spec,
	char			*return_file_spec)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	The function opens a context for scanning a file list, given a filename
 *	full_file_spec.
 *
 *  FORMAL PARAMETERS:
 *
 *      *full_file_spec		(r) Directory/Files to search for - 0 means use the 
 *                                  current directory
 *	*return_file_spec	(w) File spec of next file found in the directory chosen
 *
 *  RETURN VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *	CWORD_S_FAILURE		Error finding file spec
 *
 *  SIDE EFFECTS:
 *
 *      A filescan context is maintained for returning each file in turn.
 *--
 */

{

    /* Two static variables needed to keep track of the returned file names */

    static char find_file_spec[CWORD_SIZEOF_FILESPEC + 1];

    static unsigned long int *ctxt_p;

    char filespec[CWORD_SIZEOF_FILESPEC + 1],
	 out_file_str[CWORD_SIZEOF_FILESPEC + 1],
	 msg[CWORD_SIZEOF_FILESPEC + 1];

    unsigned long int vms_status;

#pragma nostandard

    $DESCRIPTOR(filespec_dx, filespec);
    $DESCRIPTOR(out_file_str_dx, out_file_str);

    if (full_file_spec != (char *) 0)
    {
	strcpy (find_file_spec, full_file_spec);

	if (0 == ctxt_p)
	{
	    ctxt_p = calloc (1, sizeof(unsigned long int));
	}
	else
	{
	    if ( ((vms_status = lib$find_file_end (ctxt_p)) & 1) != 1 )
	    {
		return (CWORD_S_FAILURE);
	    }

	    *ctxt_p = 0;
	}
    }

    /*
     *  Initialise the descriptor holding the file find_file_spec.
     */

    strcpy (filespec_dx.dsc$a_pointer, find_file_spec);
    filespec_dx.dsc$w_length = strlen (find_file_spec);

    if ( ((vms_status = lib$find_file (
			    &filespec_dx, 
			    &out_file_str_dx,
			    ctxt_p,
			    0,			/* default	    */
			    0,			/* related	    */
			    0,			/* status	    */
			    0)) & 1) != 1 )
    {
	if ( (vms_status == RMS$_NMF) || (vms_status == RMS$_FNF) )
	{
	    return_file_spec[0] = (char) 0;
	}
	else
	{
	    return (CWORD_S_FAILURE);
	}
    }

    /*
     *  NULL terminate resultant string.
     */

    out_file_str[out_file_str_dx.dsc$w_length] = (char) 0;

    if ( (vms_status != RMS$_NMF) && (vms_status != RMS$_FNF) )
    {
	strcpy (return_file_spec, out_file_str);

    	cword_utils_trim_filespec (return_file_spec);

	return (CWORD_S_NORMAL);
    }
    else 
    {
	return (CWORD_S_FAILURE);
    }

#pragma standard

}



PRIVATE void cword_vms_slim_file_specs (
	CWORD_T_STR_LL		*file_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to take out the device:directory specification from the
 *	inputted file specification.  This extra information isn't needed at
 *	the moment in cword.
 *
 *  FORMAL PARAMETERS:
 *
 *      *file_ll		(r/w) List to take device:directory specs from
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      void - everything should go alright in here
 *--
 */

{

    char *str_ptr;

    while (file_ll != (CWORD_T_STR_LL *) 0)
    {
	str_ptr = strstr (file_ll->str, "]");
	str_ptr++;

	strcpy (file_ll->str, str_ptr);

	file_ll = file_ll->next;
    }

    return;

}




PUBLIC int cword_vms_list_files (
	char			*directory,
	char			*file_spec,
	CWORD_T_STR_LL		**file_ll)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function which wraps around cword_vms_get_files calling it successively
 *	until no more files are found.  It implements a version of list_files
 *	as functionally similar as possible.
 *
 *  FORMAL PARAMETERS:
 *
 *      *directory		(r) Directory to search for the files in
 *	*file_spec		(r) files_spec to search for
 *	**file_ll		(w) Link list to store returned file in
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *--
 */

{

    char full_file_spec[CWORD_SIZEOF_FILESPEC + 1],
	 return_file_spec[CWORD_SIZEOF_FILESPEC + 1];

    int	status;

    if (directory != (char *) 0)
    {
	if (file_spec != (char *) 0)
	    sprintf (
		full_file_spec,
		"%s%s",
		directory,
		file_spec);
	else
	    strcpy (full_file_spec, directory);
    }
    else
    {
	strcpy (full_file_spec, file_spec);
    }

    *file_ll = (CWORD_T_STR_LL *) 0;

    status = cword_vms__get_files (
		full_file_spec,
		return_file_spec);

    while (CWORD_S_NORMAL == status)
    {
	cword_strll_add_str (
	    SORTED | UNIQUE,
	    return_file_spec,
	    file_ll);

	if (*file_ll == (CWORD_T_STR_LL *) 0)
	{
	    return (CWORD_S_FAILURE);
	}
    
	status = cword_vms__get_files (
		    (char *) 0,
		    return_file_spec);
    }

    /*
     *	?? Slim return file names downto prefix.suffix with no
     *	device:[directory] included ??
     */

    cword_vms_slim_file_specs (*file_ll);

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Obsolete ??
 */

PUBLIC int cword_vms_file_exists (
	char			*file_spec)
    
/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check the existance of a specific file in the Operating
 *	system.
 *
 *  FORMAL PARAMETERS:
 *
 *      *file_spec		(r) Check for the existance of this file
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		File/files exist
 *	CWORD_S_FAILURE		File/files don't exist
 *--
 */

{

    CWORD_T_STR_LL *file_ll;

    if ( strstr (file_spec, "*") || strstr (file_spec, "%") )
    {
	cword_messages_output_message (
		    CWORD_ILLFILENAME,
		    (TRACE2 | SCREEN0),
		    file_spec);
	
	return (CWORD_S_FAILURE);
    }

    cword_vms_list_files (
	(char *) 0,
	file_spec,
	&file_ll);

    if (file_ll == (CWORD_T_STR_LL *) 0)
    {
	return (CWORD_S_FAILURE);
    }
    else
    {
	/*
	 *  Free any memory already used.
	 */

	cword_strll_free_str_ll (&file_ll);

	return (CWORD_S_NORMAL);
    }

}



PUBLIC int cword_vms_get_time (
	char			*time_string)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      VMS function to return time in time_string.
 *
 *  FORMAL PARAMETERS:
 *
 *      *time_string		(w) string containing time
 *
 *  IMPLICIT INPUTS:
 *
 *      None yet
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal Completion
 *	CWORD_S_FAILURE		Error getting system time
 *--
 */

{

    long status;

#pragma nostandard

    $DESCRIPTOR (time_desc, time_string); 

    /* First 30 characters used to get the time. */

    time_desc.dsc$w_length = 30;

    status = sys$asctim (
                 &time_desc.dsc$w_length,
                 &time_desc, 0, 0);

    /* Only need to return the first 17 characters of the returned time stamp */

    time_string[17] = (char) 0;

    if (status != SS$_NORMAL)
    {
	cword_messages_output_message (
		    CWORD_TIMEERROR,
		    (SCREEN0 | TRACE2));

	/*
	 *  Guess what happen around this time?
	 */

	strcpy (time_string, "21-OCT-1968 01:06");

	return (CWORD_S_FAILURE);
    }
    else
    {
	return (CWORD_S_NORMAL);
    }

#pragma standard

}



PUBLIC void cword_vms_clear_screen ()

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Using printf output DCL set of openVMS control characters to clear
 *	screen!
 *
 *  FORMAL PARAMETERS:
 *
 *      None.
 *
 *  FUNCTION VALUE:
 *
 *      None.
 *--
 */

{

    /*
     *  Simple printf statement to clear the screen on OpenVMS!
     */ 

    /*
     *	?? the \n contributes to the cleared screen being two lines further
     *	down than I'd like, I think ??
     *
     *  ?? Using printd 
     */

    printf (
	"%s%s", 
	"\033[H", 
	"\033[2J\n"); 

    return;

}



#else

main ()

{

#if 0
    CWORD_T_STR_LL *file_ll;

    char file_name[256],
    	 search_spec[256];

    int  status; 

    while (TRUE)
    {

    	printf ("Input search path/file to return : ");
    	gets (search_spec);

    	if ( search_spec[0] == (char) 0 || 
    	      'q' == (char) tolower (search_spec[0]) )
    	    break;

	status = cword_vms_list_files (
			    "sys$login:",
			    search_spec,
			    &file_ll);
    
	while (file_ll != (CWORD_T_STR_LL *) 0)
	{
	    printf ("%s\n", file_ll->str);
	    file_ll = file_ll->next;
	}

    }
#endif

    char    time_string[101];

    int	    status;

    status = cword_vms_get_time (time_string);

    if (status != CWORD_S_NORMAL)
	printf ("Error: ");

    printf ("time is %s\n", time_string);

    exit (EXIT_SUCCESS);

}

#endif
