$!  FACILITY :
$!
$!	vmsbuild.com
$!
$!  ABSTRACT :
$!
$!      This is the generic DCL command file to build Cword from scratch on
$!      an OpenVMS/VAX or OpenVMS/AXP system.
$!
$!      It does the following:
$!
$!          1. Calls vmssetup.com to define where Cword is, or will be 
$!             located.
$!          2. Creates the sub-directories Cword uses.
$!          3. Builds the Cword Rdb database.
$!          4. Builds the Cword executables, first figuring out whether
$!             this file is being executed on a VAX or AXP machine.
$!          5. Cleans up after building Cword by deleting all
$!             object files and moving all sources/headers/command 
$!             files/readmes to the [.sources] directory.  It leaves all 
$!             grid/crossword files in cword$root:[000000] together with 
$!             the Cword initialisation file.
$!
$!  AUTHORS :	
$!
$!	James A. Lundon			
$!
$!  CREATION DATE :
$!
$!	8 July 1994
$!
$  wo := write sys$output
$!
$!  Output current directory.
$!
$  wo "***** You are building Cword in: ", f$environment ("DEFAULT"), " *****"
$!
$!  Execute vmssetup.com before continuing with Cword build.
$!
$  @[]vmssetup.com
$!
$  if ($STATUS .eq. 5)
$  then
$      wo "CWORD-E-EXITED, Exiting VMSBUILD.COM as error recorded"
$      exit 5
$  endif
$!
$!  Has it stuck or has it been even set.
$!
$  if (f$trnlnm ("cword$root") .eqs. "NOT_SET")
$  then
$      write sys$output "CWORD-E-SETROOT, Please define valid cword$root - exiting"
$      exit 5
$  endif
$!
$!  Create the necessary sub-directories to store Cword files.
$!
$  on error then continue
$  create/directory/log cword$root:[sources]
$  create/directory/log cword$root:[exes]
$  create/directory/log cword$root:[words]
$  create/directory/log cword$root:[database]
$! 
$!  Build the Cword database using V4.2 or greater command file.
$!
$!  If you are using a version of rdb less than V4.2 please edit 
$!  makedb.com accordingly.
$!
$!  ?? Check does the logical cword_database point to something already 
$!  in existence ??
$!
$  inquire database_enabled "Build database enabled version of Cword (Y/N) [N] "
$  if (f$edit (database_enabled, "UPCASE")) .eqs. "Y")
$  then
$      inquire database_created "Is the Cword database already created (Y/N) [N] "
$      if (f$edit (database_created, "UPCASE")) .eqs. "Y")
$      then
$          wo "CWORD-W-MAKEDB, Database already exists"
$      else
$          @[]makedb.com
$!
$          if ($STATUS .eq. 5)
$          then
$              wo "CWORD-E-EXITED, Exiting VMSBUILD.COM as error recorded"
$              exit 5
$          endif
$      endif
$  else
$      wo "CWORD-I-NORDB, Cword will be built without database enhancements"
$  endif
$! 
$!  Check that the logical cword_database has been defined and 
$!  that hopefully the database created before Cword can be built.
$!
$  if (f$trnlnm ("cword_database") .eqs. "")
$  then
$      wo "CWORD-E-NOLOGC, Cword database logical not defined - cannot find database."
$      wo "CWORD-E-CREATE, Please create database with MAKEDB.COM before continuing."
$      exit 1
$  endif
$!
$!  Select which hardware platform to build Cword with.
$!
$  if f$getsyi ("HW_MODEL") .gt. 1023
$  then
$      wo "Building Cword on an OpenVMS/AXP machine"
$      AXP := 1
$  else
$      wo "Building Cword on an OpenVMS/VAX machine"
$      AXP := 0
$  endif
$!
$! set noverify.
$!
$  if (AXP .eq. 1)
$  then
$!
$!  Build on AXP platform using DECC.
$!
$!++
$!
$!  ********** N.B. **********
$!  Modify the flags below to whatever switches/compiler are 
$!  needed to build Cword on the target AXP system.
$!  ********** N.B. **********
$!
$  scc = "$sql$pre"
$  if (f$edit (database_enabled, "UPCASE")) .eqs. "Y")
$  then
$      compiler_flags = "cc/nodebug/opt/standard=vaxc/prefix=all /define=DB" 
$      scc_flags = "scc/cc/nodebug/opt/standard=vaxc /define=DB"
$  else
$      compiler_flags = "cc/nodebug/opt/standard=vaxc/prefix=all"
$      scc_flags = "scc/cc/nodebug/opt/standard=vaxc"
$  endif
$  link_flags = "link/nodebug/notrace"
$!
$  on warning then goto exit_failure
$  write sys$output ""
$  write sys$output "You can safely ignore any %CC-I-IMPLICITFUNC messages"
$  write sys$output ""
$  write sys$output "Building print..."
$  'compiler_flags' /object=cword_o:print.obj cword_s:print.c
$  write sys$output "Building memman..."
$  'compiler_flags' /object=cword_o:memman.obj cword_s:memman.c
$  write sys$output "Building messages..."
$  'compiler_flags' /object=cword_o:messages.obj cword_s:messages.c
$  write sys$output "Building stats..."
$  'compiler_flags' /object=cword_o:stats.obj cword_s:stats.c
$  write sys$output "Building verify..."
$  'compiler_flags' /object=cword_o:verify.obj cword_s:verify.c
$  write sys$output "Building anagram..."
$  'compiler_flags' /object=cword_o:anagram.obj cword_s:anagram.c
$  write sys$output "Building defaults..."
$  'compiler_flags' /object=cword_o:defaults.obj cword_s:defaults.c
$  write sys$output "Building generate..."
$  'compiler_flags' /object=cword_o:generate.obj cword_s:generate.c
$  write sys$output "Building word..."
$  'compiler_flags' /object=cword_o:word.obj cword_s:word.c
$  write sys$output "Building utils..."
$  'compiler_flags' /object=cword_o:utils.obj cword_s:utils.c
$  write sys$output "Building fileio..."
$  'compiler_flags' /object=cword_o:fileio.obj cword_s:fileio.c
$  write sys$output "Building match..."
$  'compiler_flags' /object=cword_o:match.obj cword_s:match.c
$  write sys$output "Building soundex..."
$  'compiler_flags' /object=cword_o:soundex.obj cword_s:soundex.c
$  write sys$output "Building clue..."
$  'compiler_flags' /object=cword_o:clue.obj cword_s:clue.c
$  write sys$output "Building seed..."
$  'compiler_flags' /object=cword_o:seed.obj cword_s:seed.c
$  write sys$output "Building breakdic..."
$  'compiler_flags' /object=cword_o:breakdic.obj cword_s:breakdic.c
$  write sys$output "Building cl..."
$  'compiler_flags' /object=cword_o:cl.obj cword_s:cl.c
$  write sys$output "Building strll..."
$  'compiler_flags' /object=cword_o:strll.obj cword_s:strll.c
$  write sys$output "Building webster..."
$  'compiler_flags' /object=cword_o:webster.obj cword_s:webster.c
$  write sys$output "Building vms..."
$  'compiler_flags' /object=cword_o:vms.obj cword_s:vms.c
$!
$  write sys$output "Building charui..."
$  'compiler_flags' /object=cword_o:charui.obj cword_s:charui.c
$  write sys$output "Building grid..."
$  'compiler_flags' /object=cword_o:grid.obj cword_s:grid.c
$  write sys$output "Building main..."
$  'compiler_flags' /object=cword_o:main.obj cword_s:main.c
$!
$!  Building the database interface module.
$!
$  write sys$output "Building rdb..."
$  'scc_flags' /object=cword_o:rdb.obj cword_s:rdb.sc
$  delete/nolog/noconfirm cword_o:rdb.c;
$!
$  write sys$output "Linking cword..."
$  'link_flags' /exe=cword_e:cword.exe -
		cword_o:main.obj, cword_o:print.obj, cword_o:grid.obj, -
		cword_o:memman.obj, cword_o:messages.obj, cword_o:stats.obj, -
		cword_o:verify.obj, cword_o:charui.obj, cword_o:anagram.obj, -
		cword_o:defaults.obj, cword_o:generate.obj, cword_o:word.obj, -
		cword_o:utils.obj, cword_o:fileio.obj, cword_o:match.obj, -
		cword_o:soundex.obj, cword_o:vms.obj, cword_o:rdb.obj, -
		cword_o:webster.obj, cword_o:clue.obj, cword_o:seed.obj, -
		cword_o:breakdic.obj, cword_o:cl.obj, cword_o:strll.obj, -
                sys$library:sql$user/lib
$!
$!  Building breakdic.
$!
$  write sys$output "Building breakdic..."
$  'compiler_flags' /define=MAIN /object=cword_o:breakdic_main.obj cword_s:breakdic.c
$  write sys$output "Linking breakdic..."
$  'link_flags' /exe=cword_e:breakdic.exe cword_s:breakdic_main.obj
$!++
$  else
$!
$!  Build on VAX platform using VAXC.
$!
$!++
$!
$!  ********** N.B. **********
$!  Modify the flags below to whatever switches/compiler 
$!  needed to build Cword on the target VAX system.
$!
$!  For example VAC C V3.2 OpenVMS/VAX:
$!
$!  compiler_flags = "cc/nodebug/opt/standard=noportable" 
$!  scc_flags = "scc/cc/nodebug/opt/standard=portable"
$!
$!  ********** N.B. **********
$!
$  scc = "$sql$pre60"
$  if (f$edit (database_enabled, "UPCASE")) .eqs. "Y")
$  then
$      compiler_flags = "cc/decc/standard=vaxc/nodebug/opt /define=DB"
$      scc_flags = "scc/cc/decc/standard=vaxc/nodebug/opt /define=DB"
$  else
$      compiler_flags = "cc/decc/standard=vaxc/nodebug/opt"
$      scc_flags = "scc/cc/decc/standard=vaxc/nodebug/opt"
$  endif
$  link_flags = "link/nodebug/notrace"
$
$  on warning then goto exit_failure
$  write sys$output ""
$  write sys$output "Building print..."
$  'compiler_flags' /object=cword_o:print.obj cword_s:print.c
$  write sys$output "Building memman..."
$  'compiler_flags' /object=cword_o:memman.obj cword_s:memman.c
$  write sys$output "Building messages..."
$  'compiler_flags' /object=cword_o:messages.obj cword_s:messages.c
$  write sys$output "Building stats..."
$  'compiler_flags' /object=cword_o:stats/obj cword_s:stats.c
$  write sys$output "Building verify..."
$  'compiler_flags' /object=cword_o:verify.obj cword_s:verify.c
$  write sys$output "Building webster..."
$  'compiler_flags' /object=cword_o:webster.obj cword_s:webster.c
$  write sys$output "Building anagram..."
$  'compiler_flags' /object=cword_o:anagram.obj cword_s:anagram.c
$  write sys$output "Building defaults..."
$  'compiler_flags' /object=cword_o:defaults.obj cword_s:defaults.c
$  write sys$output "Building generate..."
$  'compiler_flags' /object=cword_o:generate.obj cword_s:generate.c
$  write sys$output "Building word..."
$  'compiler_flags' /object=cword_o:word.obj cword_s:word.c
$  write sys$output "Building utils..."
$  'compiler_flags' /object=cword_o:utils.obj cword_s:utils.c
$  write sys$output "Building fileio..."
$  'compiler_flags' /object=cword_o:fileio.obj cword_s:fileio.c
$  write sys$output "Building match..."
$  'compiler_flags' /object=cword_o:match.obj cword_s:match.c
$  write sys$output "Building soundex..."
$  'compiler_flags' /object=cword_o:soundex.obj cword_s:soundex.c
$  write sys$output "Building clue..."
$  'compiler_flags' /object=cword_o:clue.obj cword_s:clue.c
$  write sys$output "Building seed..."
$  'compiler_flags' /object=cword_o:seed.obj cword_s:seed.c
$  write sys$output "Building breakdic..."
$  'compiler_flags' /object=cword_o:breakdic.obj cword_s:breakdic.c
$  write sys$output "Building cl..."
$  'compiler_flags' /object=cword_o:cl.obj cword_s:cl.c
$  write sys$output "Building strll..."
$  'compiler_flags' /object=cword_o:strll.obj cword_s:strll.c
$  write sys$output "Building vms..."
$  'compiler_flags' /object=cword_o:vms.obj cword_s:vms.c
$!
$  write sys$output "Building charui..."
$  'compiler_flags' /object=cword_o:charui.obj cword_s:charui.c
$  write sys$output "Building grid..."
$  'compiler_flags' /object=cword_o:grid.obj cword_s:grid.c
$  write sys$output "Building main..."
$  'compiler_flags' /object=cword_o:main.obj cword_s:main.c
$!
$!  Building the database interface module.
$!
$  write sys$output "Building rdb..."
$  'scc_flags' /object=cword_o:rdb.obj cword_s:rdb.sc
$  delete/nolog/noconfirm cword_o:rdb.c;
$!
$  write sys$output "Linking cword..."
$  'link_flags' /exe=cword_e:cword.exe -
        cword_o:main.obj, cword_o:print.obj, cword_o:grid.obj, cword_o:memman.obj, -
        cword_o:messages.obj, cword_o:stats.obj, cword_o:verify.obj, cword_o:charui.obj, -
        cword_o:anagram.obj, cword_o:defaults.obj, cword_o:generate.obj, cword_o:word.obj, -
        cword_o:utils.obj, cword_o:fileio.obj, cword_o:match.obj, cword_o:soundex.obj, -
        cword_o:vms.obj, cword_o:rdb.obj, cword_o:webster.obj, cword_o:clue.obj, -
	cword_o:seed.obj, cword_o:breakdic.obj, cword_o:cl.obj, cword_o:strll.obj, -
	cword_s:vmsdecc.opt/opt, -
        cword_s:vmsucx.opt/opt, -
        cword_s:vmssql.opt/opt
$!
$!  Building breakdic.
$!
$  write sys$output "Building breakdic..."
$  'compiler_flags' /define=MAIN /object=cword_o:breakdic_main.obj cword_s:breakdic.c
$  write sys$output "Linking breakdic..."
$  'link_flags' /exe=cword_e:breakdic.exe cword_o:breakdic_main.obj, cword_s:vmsdecc.opt/opt
$!++
$  endif
$!
$!  Clean up after build.
$!
$  wo "CWORD-I-MOVESC, Deleting objects created during Cword build phase"
$  delete/nolog/noconfirm cword_o:*.obj;*
$!
$!  Move all sources, and other files, into the directory [.sources].
$!
$  on error then continue
$!
$  wo "CWORD-I-MOVESC, Purging current build directory and moving sources to cword$root:[sources]"
$!
$  purge :== purge
$  purge/log
$  rename/nolog/noconfirm/exclude=(*.ini,*.grd,*.xrd,*.dir) cword_s:*.* cword$root:[sources]
$!
$!  Remind user about word files.
$!
$  wo "CWORD-I-BREAKD, Remember: run BREAKDIC to create Cword word files"
$!
$!  exit successfully.
$!
$  exit
$!
$ exit_failure:
$  wo "CWORD-E-EXITED, Exiting VMSBUILD.COM as error has been recorded"
$!
$!  exit unsuccessfully.
$!
$  exit
