!  FACILITY
!  --------
!
!	vmsbuild.mms
!
!  ABSTRACT
!  --------
!
!	This file contains the MMS commands which are necessary to build 
!	Cword application.
!
!       ***** This is the OpenVMS/VAX *development* MMS file *****
!
!	By default, Cword is built with Rdb enabled.  To disable 
!	building Cword with Rdb, define the macro nodb=true on the 
!       command line.
!
!  AUTHORS
!  -------
!
!	James A. Lundon
!
!  CREATION DATE
!  -------------
!
!	30 December 1992
!

!
!  List of MMS macro options:
!
!    1.  vaxc  = What compiler options should be used during build?
!    2.  ansi  = Compile with /standard=ansi89 if vaxc not set
!    3.  debug = Build a debug or non-debug version?
!    4.  load  = SCA analysis required?
!



!
!  No need to include the UCX object library when compiling the sockets 
!  code with DEC C.  The DEC C RTL will handle this detail independently.
!  !!The object library is necessary if linking under old VAX C!!
!

.ifdef vaxc
compiler_opt	= /standard=vaxc
compiler	= vaxc
.else

.ifdef ansi
compiler_opt	= /decc/standard=ansi89
.else
compiler_opt	= /decc/standard=vaxc
.endif

compiler	= decc
.endif

.ifdef debug
deb_name	= _d
c_opt		= /debug/noopt
l_opt		= /debug/notrace
.else
c_opt		= /nodebug/opt
l_opt		= /nodebug/notrace
.endif

.ifdef load
c_ana		= /analysis=$(mms$target_name)
.endif

c_flags		= $(compiler_opt)$(c_opt)$(c_ana)
l_flags		= $(l_opt)



!
!  This code is always executed first
!

.first
	@ write sys$output "Starting Cword build"

!
!  This code is always executed last with the SCA library being loaded if the
!  variable load is defined.
!  

.last

.ifdef load
	@ write sys$output "Start SCA load"
	@ $(SCA) load/delete cword_s:*.ana	
	@ write sys$output "End SCA load"
.else
	@ write sys$output "Ending Build"	
.endif



!
!  Object library approach to bundling multiple objects into one library
!
!  Only one problem and that is some library objects are replaced each time 
!  a build is done even if no changes have taken place
! 

!
!  I've slightly moved around the order in which Cword modules are compiled to 
!  facilitate more effective compilation.  This breaks the chronology of the 
!  development of source code modules for Cword.
!
!  FWIW: The only change is that the messages module was added into Cword after 
!  utils and before fileio!
!

CWORD_OBJS =	cword_o:messages$(deb_name).obj, cword_o:print$(deb_name).obj, -
		cword_o:rdb$(deb_name).obj, cword_o:grid$(deb_name).obj, -
		cword_o:memman$(deb_name).obj, cword_o:utils$(deb_name).obj, -
		cword_o:fileio$(deb_name).obj, cword_o:match$(deb_name).obj, -
		cword_o:verify$(deb_name).obj, cword_o:stats$(deb_name).obj, -
		cword_o:soundex$(deb_name).obj, cword_o:word$(deb_name).obj, -
		cword_o:vms$(deb_name).obj, cword_o:charui$(deb_name).obj, -
		cword_o:webster$(deb_name).obj, cword_o:anagram$(deb_name).obj, -
		cword_o:generate$(deb_name).obj, cword_o:defaults$(deb_name).obj, -
		cword_o:clue$(deb_name).obj, cword_o:seed$(deb_name).obj, -
		cword_o:breakdic$(deb_name).obj, cword_o:cl$(deb_name).obj, -
		cword_o:strll$(deb_name).obj



!
!  To test individual module functionality, there is a test harness
!  appended to the end of  each source code module.  This test harness is
!  built when any of the targets  defined below (other than MAIN) is
!  selected at the command line for processing.
!

!
!  Build Cword (with or without database).
!

ALL : cword_e:cword$(deb_name).exe, cword_e:breakdic$(deb_name).exe
.ifdef nodb
    @ write sys$output "Finished complete build of Cword executables (without rdb)"
.else
    @ write sys$output "Finished complete build of Cword executables (with rdb)"
.endif

BREAKDIC : cword_e:breakdic$(deb_name).exe
    @ write sys$output "Finished breakdic build"

GETSTATS : cword_e:getstats$(deb_name).exe
    @ write sys$output "Finished getstats build"

TABS_TO_SPACES : cword_e:tabtospc$(deb_name).exe
    @ write sys$output "Finished tabs_to_spaces build"

PRINT : cword_e:print_test$(deb_name).exe
    @ write sys$output "Finished print test harness build"

GRID : cword_e:grid_test$(deb_name).exe
    @ write sys$output "Finished grid test harness build"

MATCH : cword_e:match_test$(deb_name).exe
    @ write sys$output "Finished match test harness build"

MEMMAN : cword_e:memman_test$(deb_name).exe
    @ write sys$output "Finished memman test harness build"
			
FILEIO : cword_e:fileio_test$(deb_name).exe
    @ write sys$output "Finished fileio test harness build"

UTILS : cword_e:utils_test$(deb_name).exe
    @ write sys$output "Finished utils test harness build"

MESSAGES : cword_e:messages_test$(deb_name).exe
    @ write sys$output "Finished messages test harness build"

VERIFY : cword_e:verify_test$(deb_name).exe
    @ write sys$output "Finished verify test harness build"

STATS : cword_e:stats_test$(deb_name).exe
    @ write sys$output "Finished stats test harness build"

SOUNDEX : cword_e:soundex_test$(deb_name).exe
    @ write sys$output "Finished soundex test harness build"

WORD :	cword_e:word_test$(deb_name).exe
    @ write sys$output "Finished word test harness build"

VMS : cword_e:vms_test$(deb_name).exe
    @ write sys$output "Finished vms test harness build"

CHARUI : cword_e:charui_test$(deb_name).exe
    @ write sys$output "Finished charUI test harness build"

WEBSTER : cword_e:webster_test$(deb_name).exe
    @ write sys$output "Finished webster test harness build"

ANAGRAM : cword_e:anagram_test$(deb_name).exe
    @ write sys$output "Finished anagram test harness build"

GENERATE : cword_e:generate_test$(deb_name).exe
    @ write sys$output "Finished generate build"

DEFAULTS : cword_e:defaults_test$(deb_name).exe
    @ write sys$output "Finished defaults test harness build"

CLUE : 	cword_e:clue_test$(deb_name).exe
    @ write sys$output "Finished clue test harness build"

STRLL : cword_e:strll_test$(deb_name).exe
    @ write sys$output "Finished strll test harness build"



!
!  Actions to build each exe, whether it be main.exe or any of the test harnesses.
!

cword_e:cword$(deb_name).exe : cword_o:main$(deb_name).obj, $(CWORD_OBJS)	
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:main$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler).opt/opt, -
		cword_s:vmssql.opt/opt

cword_e:breakdic$(deb_name).exe : cword_o:breakdic_main$(deb_name).obj
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:breakdic_main$(deb_name).obj, -
		cword_s:vms$(compiler)/opt

cword_e:getstats$(deb_name).exe : cword_o:getstats$(deb_name).obj, cword_o:breakdic$(deb_name).obj
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:getstats$(deb_name).obj, -
		cword_o:breakdic$(deb_name).obj, -
		cword_s:vms$(compiler)/opt

cword_e:tabtospc$(deb_name).exe : cword_o:tabtospc$(deb_name).obj
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:tabtospc$(deb_name).obj, -
		cword_s:vms$(compiler)/opt

cword_e:print_test$(deb_name).exe : cword_o:print_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:print_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:rdb_test$(deb_name).exe	: cword_o:rdb_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:rdb_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vmssql/opt

cword_e:grid_test$(deb_name).exe : cword_o:grid_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:grid_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:match_test$(deb_name).exe : cword_o:match_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:match_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:memman_test$(deb_name).exe : cword_o:memman_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:memman_test$(deb_name).obj, -
		$(CWORD_OBJS),
		cword_s:vms$(compiler)/opt

cword_e:fileio_test$(deb_name).exe : cword_o:fileio_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:fileio_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:utils_test$(deb_name).exe : cword_o:utils_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:utils_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:messages_test$(deb_name).exe : cword_o:messages_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME)  -
		cword_o:messages_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:verify_test$(deb_name).exe : cword_o:verify_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:verify_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:stats_test$(deb_name).exe : cword_o:stats_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:stats_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:soundex_test$(deb_name).exe : cword_o:soundex_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:soundex_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:word_test$(deb_name).exe : cword_o:word_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:word_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:vms_test$(deb_name).exe : cword_o:vms_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:vms_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:charui_test$(deb_name).exe : cword_o:charui_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:charui_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vmssql/opt, -
    		cword_s:vms$(compiler)/opt

cword_e:webster_test$(deb_name).exe : cword_o:webster_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:webster_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vmssql/opt, -
    		cword_s:vms$(compiler)/opt

cword_e:anagram_test$(deb_name).exe : cword_o:anagram_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:anagram_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:generate_test$(deb_name).exe : cword_o:generate_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:generate_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:defaults_test$(deb_name).exe : cword_o:defaults_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:defaults_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:clue_test$(deb_name).exe : cword_o:clue_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:clue_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vms$(compiler)/opt

cword_e:strll_test$(deb_name).exe : cword_o:strll_test$(deb_name).obj, $(CWORD_OBJS)
    $(LINK) $(l_flags) /exe=$(MMS$TARGET_NAME) -
		cword_o:strll_test$(deb_name).obj, -
		$(CWORD_OBJS), -
		cword_s:vmssql.opt/opt, -
		cword_s:vms$(compiler)/opt



!
!  Actions to create the object files used by MMS.
!

!
!  There are four modlues currently using rdb (each of these has to have 
!  /define=RDB included to enable the rdb functions):
!
!   1. main
!   2. grid
!   3. charui
!   4. rdb (of course)
!
!  ?? There is a bug/limitation in this MMS file so that Cword has to 
!  be re-compiled from scratch if there is any doubt whether there are 
!  mixed modules currently in scope (db enabled/db not enabled) ??
!

cword_o:main$(deb_name).obj	:	cword_s:main.c, -
 					cword_s:main.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
.ifdef nodb
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.else
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) /define=DB $(MMS$SOURCE)
.endif

cword_o:print$(deb_name).obj	:	cword_s:print.c, -
					cword_s:print.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:rdb$(deb_name).obj	:	cword_s:rdb.sc, -
					cword_s:rdb.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
.ifdef nodb
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.else
	SCC $(c_flags) /object=$(MMS$TARGET_NAME) /define=DB $(MMS$SOURCE)
.endif
.ifdef debug
.else
    	$ delete/nolog/noconfirm cword_o:rdb$(deb_name).c;*
.endif

cword_o:grid$(deb_name).obj	:	cword_s:grid.c, -
					cword_s:main.h, - 
					cword_s:print.h, -
					cword_s:grid.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
.ifdef nodb
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.else
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) /define=DB $(MMS$SOURCE)
.endif

cword_o:match$(deb_name).obj	:	cword_s:match.c, -
					cword_s:match.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:memman$(deb_name).obj	:	cword_s:memman.c, -
					cword_s:memman.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:fileio$(deb_name).obj	:	cword_s:fileio.c, -
					cword_s:fileio.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:utils$(deb_name).obj 	:	cword_s:utils.c, -
					cword_s:utils.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:messages$(deb_name).obj	:	cword_s:messages.c, -
					cword_s:messages.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:verify$(deb_name).obj	:	cword_s:verify.c, -
					cword_s:verify.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:stats$(deb_name).obj	:	cword_s:stats.c, -
					cword_s:stats.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:soundex$(deb_name).obj	:	cword_s:soundex.c, -
					cword_s:soundex.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:word$(deb_name).obj	:	cword_s:word.c, -
					cword_s:word.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:vms$(deb_name).obj	:	cword_s:vms.c, -
					cword_s:vms.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:charui$(deb_name).obj	:	cword_s:charui.c, -
					cword_s:charui.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
.ifdef nodb
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.else
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) /define=DB $(MMS$SOURCE)
.endif

cword_o:webster$(deb_name).obj	:	cword_s:webster.c, -
					cword_s:webster.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:anagram$(deb_name).obj	:	cword_s:anagram.c, -
					cword_s:anagram.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:generate$(deb_name).obj	:	cword_s:generate.c, -
					cword_s:generate.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:defaults$(deb_name).obj	:	cword_s:defaults.c, -
					cword_s:defaults.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:clue$(deb_name).obj	:	cword_s:clue.c, -
					cword_s:clue.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:seed$(deb_name).obj	:	cword_s:seed.c, -
					cword_s:seed.h, -
				        cword_s:messages.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:breakdic$(deb_name).obj	:	cword_s:breakdic.c, -
					cword_s:breakdic.h, -
					cword_s:main.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:cl$(deb_name).obj	:	cword_s:cl.c, -
					cword_s:cl.h, -
					cword_s:main.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:strll$(deb_name).obj	:	cword_s:strll.c, -
					cword_s:strll.h, -
					cword_s:main.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:getstats$(deb_name).obj	:	cword_s:getstats.c, -
					cword_s:breakdic.h, -
					cword_s:main.h, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)


cword_o:tabtospc$(deb_name).obj	:	cword_s:tabtospc.c, -
					cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

!
!  Standalone executable => define MAIN to include main ()
!

cword_o:breakdic_main$(deb_name).obj :	cword_s:breakdic.c, -
					cword_s:breakdic.h, -
					cword_s:main.h
	$(CC) $(c_flags) /define=MAIN /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)



!
!  The user can change the /define command to suit the various #defines used in the code.
!
!  Each module with "_test" contains a main() function for testing module functionality.
!

cword_o:print_test$(deb_name).obj	:  cword_s:print.c, -
					   cword_s:print.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(PRINT) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:rdb_test$(deb_name).obj		:  cword_s:rdb.sc, -
					   cword_s:rdb.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	SCC $(c_flags) /define=DB /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
.ifdef debug
.else
    	$ delete/nolog/noconfirm cword_o:rdb_test$(deb_name).c;*
.endif

cword_o:grid_test$(deb_name).obj	:  cword_s:grid.c, -
					   cword_s:main.h, - 
					   cword_s:print.h, -
					   cword_s:grid.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(GRID) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:match_test$(deb_name).obj	:  cword_s:match.c, -
					   cword_s:match.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=MATCH /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:memman_test$(deb_name).obj	:  cword_s:memman.c, -
					   cword_s:memman.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=MEMMAN /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:fileio_test$(deb_name).obj	:  cword_s:filefileio.c, -
					   cword_s:fileio.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=FILEIO /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:utils_test$(deb_name).obj	:  cword_s:utils.c, -
				           cword_s:utils.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=UTILS /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:messages_test$(deb_name).obj	:  cword_s:messages.c, -
					   cword_s:messages.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=MESSAGES /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:verify_test$(deb_name).obj	:  cword_s:verify.c, -
					   cword_s:verify.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=VERIFY /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:stats_test$(deb_name).obj	:  cword_s:stats.c, -
					   cword_s:stats.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=STATS /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:soundex_test$(deb_name).obj	:  cword_s:soundex.c, -
					   cword_s:soundex.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(SOUNDEX) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:word_test$(deb_name).obj	:  cword_s:word.c, -
					   cword_s:word.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(SOUNDEX,SIMILAR,SOUNDEX_1) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:vms_test$(deb_name).obj	:	   cword_s:vms.c, -
					   cword_s:vms.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(_VMS) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:charui_test$(deb_name).obj :	   cword_s:charui.c, -
					   cword_s:charui.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(CHARUI) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:webster_test$(deb_name).obj :	   cword_s:webster.c, -
					   cword_s:webster.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(WEBSTER) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:anagram_test$(deb_name).obj :	   cword_s:anagram.c, -
					   cword_s:anagram.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(ANAGRAM) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:generate_test$(deb_name).obj :	   cword_s:generate.c, -
					   cword_s:generate.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(GENERATE) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:defaults_test$(deb_name).obj :	   cword_s:defaults.c, -
					   cword_s:defaults.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(DEFAULTS) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)

cword_o:strll_test$(deb_name).obj :	   cword_s:strll.c, -
					   cword_s:strll.h, -
					   cword_s:messages.h, -
					   cword_tools:vmsbuild.mms
	$(CC) $(c_flags) /define=(STRLL) /object=$(MMS$TARGET_NAME) $(MMS$SOURCE)
