/*
 *  ?? Add conditional flag whether to compile the Webster's server 
 *  code or not.  This work affects this code, the accompanying header 
 *  file and all MMS/make files.  It also affects build and user
 *  documentation ??
 */

/*
 *  FACILITY:
 *
 *	webster.c
 *
 *  ABSTRACT:
 *
 *	This is probably the most basic client possible to the WEBSTER
 *	dictionary server located on the machine pointed to be the IP address
 *	defined in the defaults file, or hardcoded into cword.
 *
 *	It allows the user input a word and will return the definition of that
 *	word according to Websters dictionary of 1963 to stdout.  Other server
 *	return scenarios are less flexibly covered in the code.
 *
 *  AUTHOR:
 *
 *	James A. Lundon
 *
 *  CREATION DATE:
 *
 *	22 September 1993
 */



/*
 *  ?? Problem finding function prototypes for send/socket/connect on ULTRIX!
 *  Include some of my own home brewed versions ??
 *
 *  I've found that the precedence of include files is important on some OS's!
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>

#if defined(__unix) || defined(__unix__)

#if !(defined (__hpux) || defined (hpux))

/* 
 *  This file screws up both gcc and c89/cc on HP-UX and more importantly 
 *  isn't needed by either :-).
 */

#include <arpa/inet.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netdb.h>

#endif

#if defined(VMS) || defined (__VMS)

/*
 *  ?? This should work for both VAX C and DEC C on VAX and DEC C on AXP ??
 */

#if defined(__DECC)

#include <unixio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netdb.h>

#elif defined(VAXC)

#include <unixio.h>
#include <signal.h>
#include <types.h>
#include <socket.h>
#include <netdb.h>

#include <in.h>
#include <inet.h>

#endif

#include <perror.h>

#endif

#if defined(__unix) || defined(__unix__)

#include <unistd.h>

#endif

#include "main.h"
#include "utils.h"
#include "messages.h"
#include "webster.h"
#include "defaults.h"
#include "charui.h"

extern CWORD_G_DEFAULTS_RECORD	cword_g_defaults_record;



#ifndef WEBSTER

/* 
 *  For DOS, exclude all code in this module except the 
 *  cword_webster_get_defn function 
 */

#if !(defined(__MSDOS__) || defined(MSDOS))

/*
 *  Private function prototypes
 */

PRIVATE int cword_webster__connect_server (
	int			*socket_fd);

PRIVATE int cword_webster__send_data (
	int			socket_fd,
	char			*webster_query);

PRIVATE int cword_webster__receive_data (
	int			socket_fd,
	char			*webster_output);

PRIVATE int cword_webster__output_defn (
	char			*output_str);

PRIVATE int cword_webster__decode_output (
	char			*webster_line);

PRIVATE int cword_webster__disconnect_server (
	int			socket_fd);



/*
 *  ?? Once Webster's is opened keep it opened until Cword exits ??
 */

PRIVATE int cword_webster__connect_server (
	int			*socket_fd)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to connect the process to a websters' server defined in 
 *      cword_g_defaults_record.webster_server_name/webster_server_port.
 *
 *  FORMAL PARAMETERS:
 *
 *      socket_fd		(w) socket descriptor connecting to the Xwebster
 *
 *  IMPLICIT INPUTS:
 *
 *	cword_g_defaults_record.webster_server_name
 *	cword_g_defaults_record.webster_server_port
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *	CWORD_SERVERERROR	server error
 *	
 *--
 */

{

    struct sockaddr_in *SockAddr;

    int status;

    SockAddr = (struct sockaddr_in *) calloc (sizeof (struct sockaddr), 1);

    SockAddr->sin_family = AF_INET;

    /*
     *	?? Don't know whether it works with IP name as well as IP address.
     *
     *	There was a bug in VAXC that meant that gethostbyname didn't work,
     *	however this should not affect the working of gethostbyname on other
     *	platforms ??
     */

    SockAddr->sin_addr.s_addr = 
        inet_addr (cword_g_defaults_record.webster_server_name);

    /*
     *	The port number is just as important as the address/name of the IP host
     *	being accessed.
     */

    SockAddr->sin_port = htons (cword_g_defaults_record.webster_server_port);

    /*
     *  Socket provides sequence, reliable, two-way connection based byte 
     *  streams with an available out-of-band data transmission machanism.
     */

    *socket_fd = socket (AF_INET, SOCK_STREAM, 0);

    errno = 0;
    cword_messages_output_message (
    	CWORD_WEBCONNECT,
	(TRACE2 | SCREEN0),
	cword_g_defaults_record.webster_server_name);

    status = connect (
    	         *socket_fd,
    		 SockAddr,
    		 sizeof(struct sockaddr_in));

    free (SockAddr);

    if (status != 0)
    {
	char perror_str[CWORD_SIZEOF_MESSAGE + 1];

	/*
	 *  Connect error.
	 */

	/*
	 *  Sometimes when the server is not available errno returns 65535 (on
	 *  OpenVMS).  This is valid and should be checked for:
	 *
	 *  From errno.h:
	 *
	 *  #define EVMSERR 65535 VMS-specific error code for 
         *  non-translatable errors from VMS to C 
	 *
	 */

	cword_messages_output_message (
	    CWORD_WEBERR,
	    (TRACE2 | SCREEN0));

	cword_messages_get_message (
	    CWORD_S_PERRORSTR,
	    perror_str);

	perror (perror_str);

	cword_messages_output_message (
	    CWORD_SERVERERROR,
	    (TRACE0 | SCREEN0));

	return (CWORD_SERVERERROR);
    }
    else
    {
	cword_messages_output_message (
		    CWORD_WAITINGENDED,
		    TRACE2);

	return (CWORD_S_NORMAL);
    }

}



PRIVATE int cword_webster__send_data (
	int			socket_fd,
	char			*webster_query)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Function to send query data to Xwebster.
 *
 *  FORMAL PARAMETERS:
 *
 *      socket_fd		(r) Socket descriptor
 *	*webster_query		(r) character string to send to Xwebster
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *	CWORD_SERVERERROR	Server error
 *--
 */

{

    int status;

    status = send (
		socket_fd,
		webster_query,
		strlen (webster_query), 0);

    if (status < 0)
    {
	char perror_str[CWORD_SIZEOF_MESSAGE + 1];

	/*
	 *  Send error.
	 */

	cword_messages_output_message (
	    CWORD_WEBERR,
	    (TRACE2 | SCREEN0));

	cword_messages_get_message (
	    CWORD_S_PERRORSTR,
	    perror_str);

	perror (perror_str);

	cword_messages_output_message (
	    CWORD_SERVERERROR,
	    (TRACE0 | SCREEN0));

	return (CWORD_SERVERERROR);
    }
    else
    {
	return (CWORD_S_NORMAL);
    }

}



PRIVATE int cword_webster__receive_data (
	int			socket_fd,
	char			*webster_output)
	
/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Function to read the output data from Xwebster query.
 *
 *  FORMAL PARAMETERS:
 *
 *      socket_fd		(r) socket descriptor
 *      *webster_output		(w) output from Xwebster request
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *      Other			Error
 *--
 */

{

    char out_str[CWORD_SIZEOF_MESSAGE + 1];

    int	status = 0,
	running_total = 0,
	first_time = TRUE;

    memset (out_str, 0, sizeof (out_str));
    first_time = 0;

    /*
     *	?? Major problem with BUFSIZ that meant that the default size for
     *	BUFSIZ is different between VAX and AXP implementations of the compiler ??
     */

    /*
     *	?? Completely re-work this functionality so that a huge output space to
     *	store the result isn't required ??
     */

    while ((status = read (
			socket_fd,
			out_str,
			CWORD_SIZEOF_MESSAGE + 1)) > 0)
    {
	out_str[CWORD_SIZEOF_MESSAGE] = (char) 0;

	if (FALSE == first_time)
	{
	    if (running_total + strlen (out_str) >= CWORD_SIZEOF_WEB_OUTPUT)
	    {
		strncat (
		    webster_output, 
		    out_str, 
		    (CWORD_SIZEOF_WEB_OUTPUT - running_total - 2));

		webster_output[CWORD_SIZEOF_WEB_OUTPUT - 2] = '\n';

		cword_messages_output_message (
			    CWORD_WAITINGENDED,
			    SCREEN0);

		cword_messages_output_message (
			    CWORD_WEBTOOMUCHDATA,
			    SCREEN0);

		return (CWORD_S_NORMAL);
	    }

	    running_total += strlen (out_str);

	    strcat (webster_output, out_str);
	}
	else
	{
	    if ((running_total + strlen (out_str)) >= CWORD_SIZEOF_WEB_OUTPUT)
	    {
		strncpy (
		    webster_output, 
		    out_str, 
		    (CWORD_SIZEOF_WEB_OUTPUT - running_total - 2));

		webster_output[CWORD_SIZEOF_WEB_OUTPUT - 2] = '\n';

		cword_messages_output_message (
			    CWORD_WAITINGENDED,
			    SCREEN0);

		cword_messages_output_message (
			    CWORD_WEBTOOMUCHDATA,
			    SCREEN0);

		return (CWORD_S_NORMAL);
	    }

	    running_total += strlen (out_str);

	    strcpy (webster_output, out_str);
	    first_time = FALSE;
	}

	if (status < CWORD_SIZEOF_MESSAGE)
	    break;

	cword_messages_output_message (
		    CWORD_WAITING,
		    SCREEN0);

	/*
	 *  ?? To allow data be returned properly - delay the read () ??
	 *
	 *  ?? Use a blocking read instead of a simple non-blocking read to
	 *  ensure that the data is returned correctly ??
	 *
	 *  ?? Use ioctl() for blocking reads from the BSD socket ??
	 *
	 *  Have to find out more about this type of read before implementing
	 *  it!
	 */

	cword_utils_sleep_for (2);

	memset (out_str, 0, sizeof (out_str));
    }

    if (status < 0)
    {
	char perror_str[CWORD_SIZEOF_MESSAGE + 1];

	/*
	 *  Read error.
	 */

	cword_messages_output_message (
	    CWORD_WEBERR,
	    (TRACE2 | SCREEN0));

	cword_messages_get_message (
	    CWORD_S_PERRORSTR,
	    perror_str);

	perror (perror_str);

	cword_messages_output_message (
	    CWORD_SERVERERROR,
	    (TRACE0 | SCREEN0));
    }

    cword_messages_output_message (
		CWORD_WAITINGENDED,
		SCREEN0);

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_webster__output_defn (
	char			*output_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to parse the string returned from the webster server and
 *	output it via cword_messages_output_message.
 *
 *  FORMAL PARAMETERS:
 *
 *      *output_str		(r/w) Webster output string to be displayed to stdout
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_MORMAL
 *--
 */

{

    char *str_ptr;

    int line_num = 0;

    int	count = 0,
	lines_in_str = 0;

    while (output_str[count] != (char) 0)
    {
	/*
	 *  Getting the number of lines returned from webster.
	 */

	if (output_str[count] == (char) '\n')
	    lines_in_str++;

	count++;
    }

    cword_messages_output_message (
	CWORD_S_WEBSTEROUT, 
	TRACE2,
	lines_in_str);

    /*
     *  Use strtok to obtain EOL breaks is output_str.
     *
     *  Remember that the output string is being altered!
     */

    str_ptr = strtok (output_str, "\n");

    while (str_ptr != (char *) 0)
    {
	cword_messages_output_message (
	    CWORD_USERDEFINEDSTRING, 
	    SCREEN0,
	    str_ptr);

	cword_messages_output_message (
	    CWORD_WAITINGENDED, 
	    SCREEN0);

	str_ptr = strtok (0, "\n");

	line_num++;

	/*
	 *  Checking whether we need to page break or not!  All the necessary
	 *  housekeeping is doing in this function.
	 */

	if (cword_charui_check_page_break (
		(void *) str_ptr,
		CWORD_S_PRESSRETURNQUIT,
		&line_num) == CWORD_S_FAILURE)
	{
	    break;
	}
    }

    return (CWORD_S_NORMAL);

}




PRIVATE int cword_webster__decode_output (
	char			*webster_line)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Function to decode the output from the Xwebster server used
 *
 *  FORMAL PARAMETERS:
 *
 *      webster_line		(r) string containing output from Xwebster
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *	CWORD_SERVERERROR	Server error
 *--
 */

{

    if (strlen (webster_line) != 0)
    {
	if (strstr (webster_line, "DEFINITION"))
	{
	    /*
	     *	?? Parse the output so that no more than a screen if info is
	     *	output at any one time ??
	     */

	    cword_webster__output_defn (webster_line);
	}
	else if (strstr (webster_line, "WILD 0"))
	{
	    printf ("# No words match the inputted string\n");
	}
	else if (strstr (webster_line, "WILD"))
	{
	    printf ("# Choice of possible words which the word you selected could be include:\n");
	    printf ("%s\n", webster_line);
	}
	else if (strstr (webster_line, "SPELLING 0"))
	{
	    printf ("# No such word in the dictionary\n");
	}
	else if (strstr (webster_line, "SPELLING"))
	{
	    printf ("# The word could be classified differently in the dictionary\n");
	}
	else if (strstr (webster_line, "MATCHS 0"))
	{
	    printf ("# No matching words found\n");
	}
	else if (strstr (webster_line, "MATCHS"))
	{
	    printf (
		"# No perfect match found for %s\n",
		webster_line);
	}
	else if (strstr (webster_line, "COMPLETION"))
	{
	    printf (
		"# Completion status found :\n",
		webster_line);
	}
	else if (strstr (webster_line, "AMBIGUOUS"))
	{
	    printf (
		"# Ambiguous meaning given for: %s\n",
		webster_line);
	}
	else if (strstr (webster_line, "ERROR RECOVERABLE"))
	{
	    printf ("# Error occurred but webster server can recover\n");
	}
	else if (strstr (webster_line, "ERROR FATAL"))
	{
	    /*
	     *  Close down the connection to the server
	     */

	    cword_messages_output_message (
		    CWORD_SERVERERROR,
		    (TRACE2 | SCREEN0));

	    return (CWORD_SERVERERROR);
	}
	else
	{
	    /*
	     *	Two other possible states - a NULL string or an unrecognised
	     *	token.
	     */

	    cword_messages_output_message (
		    CWORD_WEBPARSEERROR,
		    (TRACE2 | SCREEN0));
	}
    }
    else
    {
	cword_messages_output_message (
		CWORD_WEBNOUTPUT,
		(TRACE2 | SCREEN0));
    }

    return (CWORD_S_NORMAL);

}



PRIVATE int cword_webster__disconnect_server (
	int			socket_fd)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      disconnecting from the Xwebster server
 *
 *  FORMAL PARAMETERS:
 *
 *      socket_fd		(r) socket descriptor
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successful completion
 *	CWORD_SERVERERROR	server error
 *--
 */

{

    int status;

    status = close (socket_fd);

    if (status != 0)
    {
	char perror_str[CWORD_SIZEOF_MESSAGE + 1];

	/*
	 *  Close error.
	 */

	cword_messages_output_message (
	    CWORD_WEBERR,
	    (TRACE2 | SCREEN0));

	cword_messages_get_message (
	    CWORD_S_PERRORSTR,
	    perror_str);

	perror (perror_str);

	cword_messages_output_message (
	    CWORD_SERVERERROR,
	    (TRACE2 | SCREEN0));

	return (CWORD_SERVERERROR);
    }
    else
    {
	status = cword_messages_output_message (
		CWORD_WEBDISCONNECT,
		(TRACE2 | SCREEN0));

	return (CWORD_S_NORMAL);
    }	

}

#endif /* End of #if !(defined(__MSDOS__) || defined(MSDOS)) */



PUBLIC int cword_webster_get_defn (
	char			*webster_word)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

/* 
 *  Exclude webster specific code for execution on DOS 
 *
 *  ?? Research Winsocks - Similar TCP/IP functionality for DOS as is defined
 *  here ??
 */

#if !(defined(__MSDOS__) || defined(MSDOS))

    char webster_query[81],
	 temp_str[CWORD_SIZEOF_LINE + 1],
	 webster_output[CWORD_SIZEOF_WEB_OUTPUT]; /* Allowable size of return string */

    int socket_fd,
    	status;

    /*
     *	Check string first for * or % characters before contining.
     */

    if ( strstr (webster_word, "*")  || strstr (webster_word, "%") )
    {
	cword_messages_output_message (
		CWORD_WEBSTERERR, 
		(TRACE2 | SCREEN0), 
		webster_word);

	return (CWORD_WEBSTERERR);	
    }

    cword_messages_output_message (
	    CWORD_WEBSTERWORD, 
	    (TRACE2 | SCREEN0), 
	    webster_word);

    /*
     *  Opening server connection.
     */

    status = cword_webster__connect_server (&socket_fd);

    if (CWORD_S_NORMAL != status)
    {
	return (status);
    }

    /*
     *  Format inquiry to Xwebster.
     */

    cword_messages_get_message (
	CWORD_S_WEBQUERTFORMAT,
	temp_str);

    sprintf (
	webster_query, 
	temp_str,		/* "DEFINE %s\015\012", */
	webster_word);

    cword_messages_output_message (
	CWORD_WEBQUERY,
	TRACE2,
	webster_word);

    /*
     *  Read/Write to/from server.
     */

    status = cword_webster__send_data (
		socket_fd,
		webster_query);

    if (CWORD_S_NORMAL != status)
    {
	return (status);
    }

    /* 
     *  Get data from server from initial query.
     */

    memset (webster_output, 0, sizeof (webster_output));

    status = cword_webster__receive_data (
		socket_fd,
		webster_output);

    cword_messages_output_message (
	CWORD_WEBOUTLEN,
	TRACE2,
	strlen (webster_output));

    /*
     *  Decode the output data
     */

    status = cword_webster__decode_output (webster_output);

    if (CWORD_S_NORMAL != status)
    {
	return (status);
    }

    /*
     *	?? Is there any need to close down the webster server while running
     *	Cword as it's a bit expensive to restart the connection each time it's
     *	required ??
     */

    /*
     *  Close down server after use.
     */

    status = cword_webster__disconnect_server (socket_fd);

    if (CWORD_S_NORMAL != status)
    {
	return (status);
    }

    return (CWORD_S_NORMAL);

#else  /* else of #if !(defined(__MSDOS__) || defined(MSDOS)) */

    cword_messages_output_message (
	CWORD_S_NOTYETIMPLEMENT,
	SCREEN0);

    return (CWORD_S_NORMAL);

#endif  /* end of #if !(defined(__MSDOS__) || defined(MSDOS)) */

}

#else

main (
	int			argc,
	char			*argv[])

{

    int status;
    
    status = cword_webster_get_defn (argv[1]);

    exit (EXIT_SUCCESS);

}

#endif
