/*
 *++
 *  FACILITY:
 *
 *      word.c
 *
 *  ABSTRACT:
 *
 *	Misc. string related functions used to Cword defined here.
 *
 *  AUTHORS:
 *
 *      James A. Lundon
 *
 *  CREATION DATE:
 *
 *      21 June 1993
 *--
 */

/*
 *  Include Files
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#if defined(__unix) || defined(__unix__)

#include <strings.h>

#endif

#include "main.h"
#include "word.h"
#include "charui.h"
#include "utils.h"
#include "messages.h"

/*
 *  PRIVATE function prototypes.
 */

#define SLAM_CHAR	'Q'

PRIVATE void cword_word__slam_str (
	char			check_ch,
	char			*slam_str);



PRIVATE int cword_word__check_mask (
	char			check_ch,
	char			*temp_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that checks for the existance of the character check_ch in the
 *	string *temp_str.  If the character is found then the character is
 *	deleted from *temp_str.
 *
 *  FORMAL PARAMETERS:
 *
 *	check_ch		(r) Character to check for in string *temp_str
 *	*temp_str		(r/w) string to check for instance of check_ch
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Character check_ch found in *temp_str
 *	CWORD_S_FAILURE		Character check_ch *not* found in *temp_str
 *--
 */

{

    /*
     *	If check_ch is found in temp_str then delete the character in question
     *	from temp_str.
     */

    int count1 = 0,
	count2 = 0,
	len;

    if ( (temp_str[0] == (char) 0) || (check_ch == (char) 0) )
    {
	/*
	 *  If either of both strings are empty then return with legs between
	 *  tail.
	 */

	return (CWORD_S_FAILURE);
    }

    while (temp_str[count1] != (char) 0)
    {
	if (temp_str[count1] == check_ch)
	{
	    /*
	     *	Take character check_ch out of temp_str and re-org temp_str.
	     */

	    len = strlen (temp_str);

	    temp_str[count1] = (char) 0;

	    while (count2 < count1)
	    {
		count2++;
	    }

	    count2++;

	    while (count2 < len)
	    {
		temp_str[count1] = temp_str[count2];
		count1++; count2++;
	    }

	    temp_str[count1] = (char) 0;
	}
	else
	    count1++;
    }

    return (CWORD_S_FAILURE);

}



/*
 *  All functions below should have to have the format of the following:
 *
 *  PUBLIC int function_name (
 *          char		*user_defined_string_according_to_function,
 *	    char		*string_found_in_dictionary_or_word_file);
 *
 *  if they are expected to work with cword_match_generic_fnc_name correctly.
 */

/*
 *  ?? Consider words that are only a maximum of 5 letters longer than
 *  *check_str ??
 */

PUBLIC int cword_word_super_anagram_str (
	char			*check_str,
	char			*word_str)
    
/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function that reads the local dictionary and returns words in the
 *	dictionary which contain at least one instance of every character
 *	specified in *check_str.  
 *
 *	This function is potentially very useful creating crossword clues as it
 *	is therefore possible to construct words which will comprise of a
 *	superanagram of the word being clued.
 *
 *	An example:
 *
 *	If the user wants all words return which contain at least one instance
 *	of all the following letters [aeiou] then these words will be returned
 *	amongst others:
 *
 *	1. authorize
 *	2. jalousie
 *	3. sequoia
 *
 *  FORMAL PARAMETERS:
 *
 *	*check_str		(r) List of character that are to be checked for
 *      *word_str		(r) String to check for check_str
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    char temp_str[CWORD_SIZEOF_WORD + 1];

    int count1;

    if ( (word_str[0] == (char) 0) || (check_str[0] == (char) 0) )
    {
	/*
	 *  If either word_str or check_str or both is blank then we cannot
	 *  continue.
	 */

	return (CWORD_S_FAILURE);
    }

    if (strlen (word_str) < strlen (check_str))
	return (CWORD_S_FAILURE);

    strcpy (temp_str, check_str);

    count1 = 0;
    while (word_str[count1] != (char) 0)
    {
	cword_word__check_mask (
		word_str[count1],
		temp_str);

	if (temp_str[0] == (char) 0)
	{
	    return (CWORD_S_NORMAL);
	}

	count1++;
    }

    return (CWORD_S_FAILURE);

}



PUBLIC int cword_word_str_start (
	char			*check_str,
	char			*word_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Determines whether the string being searched for exists only at the
 *	beginning of the inputted word or not.
 *
 *  FORMAL PARAMETERS:
 *
 *      *check_str		(r) string containing character string to match
 *	*word_str		(r) word to search for check_str in
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		successfully matched
 *	CWORD_S_FAILURE		unsuccessfully matched
 *--
 */

{

    char temp_str[CWORD_SIZEOF_WORD + 1];

    if ( (word_str[0] == (char) 0) || (check_str[0] == (char) 0) )
    {
	/*
	 *  If either word_str or check_str or both is blank then we cannot
	 *  continue.
	 */

	return (CWORD_S_FAILURE);
    }

    if (strlen (check_str) > strlen (temp_str))
    {
	/*
	 *  If the inputted word_str is smaller than check_str then a match
	 *  isn't possible.
	 */

	return (CWORD_S_FAILURE);
    }

    strcpy (temp_str, word_str);

    /*
     *	If check_str is not valid then return CWORD_S_FAILURE.
     */
    
    temp_str[strlen (check_str)] = (char) 0;

    if (strcmp (check_str, temp_str) == 0)
    {
	/*
	 *  Found!
	 */

	return (CWORD_S_NORMAL);
    }
    else
    {
	/*
	 *  Not found!
	 */

	return (CWORD_S_FAILURE);
    }

}



PUBLIC int cword_word_str_middle (
	char			*check_str,
	char			*word_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Determines whether the string being searched for exists only in the
 *	middle of the inputted word or not.
 *
 *  FORMAL PARAMETERS:
 *
 *      *check_str		(r) string to match
 *	*word_str		(r) word to search for check_str in
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		successfully matched
 *	CWORD_S_FAILURE		unsuccessfully matched
 */

{

    if ( (word_str[0] == (char) 0) || (check_str[0] == (char) 0) )
    {
	/*
	 *  If either word_str or check_str or both is blank then we cannot
	 *  continue.
	 */

	return (CWORD_S_FAILURE);
    }

    if (strlen (check_str) > strlen (word_str))
    {
	/*
	 *  If the inputted word_str is smaller than check_str then a match
	 *  isn't possible.
	 */

	return (CWORD_S_FAILURE);
    }

    /*
     *  ?? Must be a quicker way of doing this ??
     */

    if (strstr (word_str, check_str) != 0)
    {
	if ( (word_str[0] != check_str[0]) && 
              (word_str[strlen (word_str) - 1] != check_str[strlen (check_str) - 1]) )
	{
	    /*
	     *  Found!
	     *
	     *	This must be true as we've found that check_str is contained in
	     *	word_str and also that word_str[0] != check_str[0] and
	     *	check_str[strlen(check_str)-1] != word_str[strlen(word_str)-1]
	     *	so therefore check_str is in word_str.
	     */

	    return (CWORD_S_NORMAL);
	}
	else
	{
	    /*
	     *  Found!
	     */

	    return (CWORD_S_FAILURE);	    
	}
    }
    else
    {
	return (CWORD_S_FAILURE);
    }

}



PUBLIC int cword_word_str_end (
	char			*check_str,
	char			*word_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Determines whether the string being searched for exists only at the
 *	end of the inputted word or not.
 *
 *  FORMAL PARAMETERS:
 *
 *      *check_str		(r) string containing character string to match
 *	*word_str		(r) word to search for check_str in
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully matched
 *	CWORD_S_FAILURE		Unsuccessfully matched
 *--
 */

{

    char *str_ptr;

    if ( (word_str[0] == (char) 0) || (check_str[0] == (char) 0) )
    {
	/*
	 *  If either word_str or check_str or both is blank then we cannot
	 *  continue.
	 */

	return (CWORD_S_FAILURE);
    }

    if (strlen (check_str) > strlen (word_str))
    {
	/*
	 *  If the inputted word_str is smaller than check_str then a match
	 *  isn't possible.
	 */

	return (CWORD_S_FAILURE);
    }

   str_ptr = word_str + strlen (word_str) - strlen (check_str);
 
    if (strcmp (check_str, str_ptr) == 0)
    {
	/*
	 *  Found!
	 */

	return (CWORD_S_NORMAL);
    }
    else
    {
	/*
	 *  Not found!
	 */

	return (CWORD_S_FAILURE);
    }

}



/*
 *  ?? Incorrect results when j??? is entered ??
 *
 *  ?? Bug also with *s? ??
 */

PUBLIC int cword_word_str_wild (
	char			*refer_str, 
	char			*check_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Generic wildcard string matching routine which will accept both *'s
 *	and ?'s characters in place of valid character to match.
 *
 *  FORMAL PARAMETERS:
 *
 *	*refer_str		()
 *	*check_str		()
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Successfully matched
 *	CWORD_S_FAILURE		Unsuccessfull matched
 *--
 */

{

    register char *p = refer_str, 
		  *s = check_str, 
		  c;

    int i;

    if ( (check_str[0] == (char) 0) || (refer_str[0] == (char) 0) )
    {
	/*
	 *  If either check_str or refer_str or both is blank then we cannot
	 *  continue.
	 */

	return (CWORD_S_FAILURE);
    }

    /*
     *  Loop on each character in the pattern string
     */

    while ((c = *(p++)) != 0)
    {
	if (c == '*')
	{
            /*
             *  Skip an arbitrary number to find a match.
             */

	    do
	    {
                /*
                 *  If we are at end of string here then user has trailing SOME
		 *  and so we have a match.
                 */

		if (!*p) 
		    return (CWORD_S_NORMAL);

                /*
                 *  Keep eating any other '*'s up as multiple ones count a one.
                 */
	    }
	    while (*p++ == '*');

	    p--;

	    if (!*s) 
		return (CWORD_S_FAILURE);

	    for (i = 0; s[i]; i++)
	    {
		/*
		 *  Always use the return value of a function - when it's defined
		 */

		if (cword_word_str_wild (p, &s[i]) == CWORD_S_NORMAL)
		    return (CWORD_S_NORMAL);
	    }
	}
	else if (c != *s++ && c != '?')
	{
	    return (CWORD_S_FAILURE);
	}
    }

    /*
     *	Exhausted string - if != 0 then no match else == 0 there is a
     *	match!
     */

    return (*s ? CWORD_S_FAILURE : CWORD_S_NORMAL);

}



/*
 *  ?? Case sensitivity ??
 */

PUBLIC void cword_word_rot_string (
	int			shift,
	unsigned char		*rot_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to ROT[shift] (usually ROT13) a string.  The character shift
 *	can be a user defined quantity but is most usually 13.  This function
 *	is most useful for obtaining the decoded version of an inputted string
 *	from any Usenet newsgroup for example.
 *
 *  FORMAL PARAMETERS:
 *
 *      shift			(r) numeric value to move each character along
 *	rot_str			(w) character to ROT[shift]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Normal completion
 *	CWORD_S_FAILURE		Error in ROT'ing the input string
 *--
 */

{

    int count;

    if (rot_str[0] == (char) 0)
    {
	return;
    }

    count = 0;

    while (rot_str[count] != (char) 0)
    {
    	/*
         *  Check that the new letter is in the ASCII range 97 to 122. If not 
     	 *  then format!  Don't forget spaces stay the same as usual.
         */

    	if (isalpha(rot_str[count]) != 0)
        {
	    /*
	     *	Move to lower case before operating on the string.
	     */

    	    rot_str[count] = (char) (tolower((char) rot_str[count]) + shift);

     	    if (rot_str[count] > (char) 122)
	    {
    		rot_str[count] -= 26;
	    }
    	    else if (rot_str[count] < (char) 97)
	    {
    		rot_str[count] += 26;
	    }
	    else
	    {
		/* Do nothing - leave the input character alone */
	    }
    	}

        count++;
    }

    cword_messages_output_message (
	    CWORD_ROTSTROUT,
	    (TRACE2 | SCREEN0),
	    rot_str);

    return;

}



PUBLIC BOOLEAN cword_word_is_alphabetics_only (
	char			*input_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check a string for alphabetics (a-z, A-Z) *only*.  If any
 *	non-alphabetics are found then return FALSE else return TRUE.
 *
 *  FORMAL PARAMETERS:
 *
 *	*input_str		(r)
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			String contain alphabetics only.
 *	FALSE			String contain at least one non-alphabetic character.
 *--
 */

{

    unsigned int count;

    if (input_str[0] == (char) 0)
    {
	/*
	 *  If the string is blank then it contains no characters ergo FALSE as
	 *  a return value.
	 */

	return (FALSE);
    }

    for (count = 0; count < strlen (input_str); count++)
    {
	if (isalpha ((int) input_str[count]) == 0)
	{
	    return (FALSE);
	}
    }

    return (TRUE);

}



PUBLIC BOOLEAN cword_word_is_digits_only (
	char			*input_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check a string for digits (0-9) *only*.  If any
 *	non-alphabetics are found then return FALSE else return TRUE.
 *
 *  FORMAL PARAMETERS:
 *
 *	*input_str		(r)
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      TRUE			String contain alphabetics only.
 *	FALSE			String contain at least one non-alphabetic character.
 *--
 */

{

    unsigned int count;

    if (input_str[0] == (char) 0)
    {
	/*
	 *  If the string is blank then it contains no characters ergo FALSE as
	 *  a return value.
	 */

	return (FALSE);
    }

    for (count = 0; count < strlen (input_str); count++)
    {
	if (isdigit ((int) input_str[count]) == 0)
	{
	    return (FALSE);
	}
    }

    return (TRUE);

}



PUBLIC void cword_word_del_lead_spaces (
	char			*input_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to delete any leading spaces that may be in the input string.
 *
 *  FORMAL PARAMETERS:
 *
 *	*input_str		(r/w) input/output string
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    char *str_ptr;

    if (input_str[0] == (char) 0)
    {
	/*
	 *  If the string is blank there's nothing we can do.
	 */

	return;
    }

    str_ptr = input_str;

    while (*str_ptr == (char) ' ')
	str_ptr++;

    strcpy (input_str, str_ptr);

    return;

}



PUBLIC void cword_word_del_trail_spaces (
	char			*input_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to delete any trailing spaces that may be in the input string.
 *
 *  FORMAL PARAMETERS:
 *
 *	*input_str		(r/w) input/output string
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      [@description or none@]
 *--
 */

{

    char *str_ptr;

    if (input_str[0] == (char) 0)
    {
	/*
	 *  If the string is blank there's nothing we can do.
	 */

	return;
    }

    str_ptr = &input_str[strlen (input_str) - 1];

    while ( (*str_ptr == (char) ' ') && (str_ptr != &input_str[0]) )
    {
	str_ptr--;
    }

    if (*str_ptr != (char) ' ')
    {
	str_ptr++;
	*str_ptr = (char) 0;
    }
    else
    {
	*str_ptr = (char) 0;
    }

    return;

}



/* ?? toupper_str ?? */

PUBLIC void cword_word_upper_str (
	char			*case_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to change to upper case a string.
 *
 *  FORMAL PARAMETERS:
 *
 *	*trunc_str		(r) pointer to string to be uppercased
 *--
 */
{

    if (case_str[0] == (char) 0)
    {
	/*
	 *  If the string is blank there's nothing we can do.
	 */

	return;
    }

    while (*case_str)
    {
	*case_str = (char) toupper (*case_str);
	case_str++;
    }
    
    return;

}



/* ?? tolower_str ?? */

PUBLIC void cword_word_lower_str (
	char			*case_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to change to lower case a string.
 *
 *  FORMAL PARAMETERS:
 *
 *	*trunc_str		(r) pointer to string to be uppercased
 *--
 */
{

    if (case_str[0] == (char) 0)
    {
	/*
	 *  If the string is blank there's nothing we can do.
	 */

	return;
    }

    while (*case_str)
    {
	*case_str = (char) tolower (*case_str);
	case_str++;
    }
    
    return;

}



PUBLIC int cword_word_truncate_str (
	char			*trunc_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to truncate all spaces from end of inputted string.
 *
 *  FORMAL PARAMETERS:
 *
 *	*trunc_str		(r) pointer to string to be truncated
 *--
 */
{

    char *ptr1,
         *ptr2;

    if (trunc_str[0] == (char) 0)
    {
	/*
	 *  If the string is blank there's nothing we can do.
	 */

	return (CWORD_S_NORMAL);
    }

    ptr1 = trunc_str;
    ptr2 = ptr1 + strlen (ptr1) - 1;

    if (!isspace (*ptr2))   
    {
        ptr2++;
        *ptr2 = (char) 0;
    }

    while ((ptr2 >= ptr1) && (isspace (*ptr2)))
    {
        ptr2--;
    };

    ptr2++;
    *ptr2 = (char) 0;

    return (CWORD_S_NORMAL);

}



PRIVATE void cword_word__slam_str (
	char			check_ch,
	char			*slam_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      Used in cword_word_preproc_match_str to check the inputted string.
 *
 *  FORMAL PARAMETERS:
 *
 *	check_ch		(r) Character to slam.
 *	*slam_str		(r/w) String to slam.
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int count,
	str_size = strlen (slam_str);

    if (str_size == 0)
	return;

    for (count = 0; count < str_size; count++)
    {
	if (slam_str[count] == (char) check_ch)
	    slam_str[count] = (char) SLAM_CHAR;
    }

    return;

}



/*
 *  ?? cword_word_check_str_against_mask
 */

PUBLIC int cword_word_check_input_str (
	unsigned long		char_mask,
	char			*check_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Check the inputted string with the inputted char_mask defined by the
 *	user, or by the particular situation currently being executed.
 *
 *  FORMAL PARAMETERS:
 *
 *	char_mask		(r) Mask to check string against.
 *	*check_str		(r) User defined string to check for validity.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  FUNCTION VALUE:
 *
 *      CWORD_S_NORMAL		Inputted string is OK.
 *	CWORD_S_FAILURE		Error with part or all of inputted string.
 *--
 */

{

    char temp_str[CWORD_SIZEOF_MESSAGE + 1];

    if (check_str[0] == (char) 0)
    {
	/*
	 *  Assume that a NULL string is OK.
	 */

	return (CWORD_S_NORMAL);
    }

    strcpy (temp_str, check_str);

    if ( !(char_mask & ALL) )
    {
	/*
	 *  If (char_mask !& ALL) then check the input string for invalid
	 *  characters.
	 *
	 *  Checks for various characters, other than alphabetics which are
	 *  allowed in the input string.
	 */

	if (char_mask & PERCENT)
	    cword_word__slam_str (PERCENT_CHAR, temp_str);

	if (char_mask & WILDCARD)
	    cword_word__slam_str (WILDCARD_CHAR, temp_str);

	if (char_mask & QUESTION)
	    cword_word__slam_str (QUESTION_CHAR, temp_str);

	if (char_mask & SPACE)
	    cword_word__slam_str (SPACE_CHAR, temp_str);

	if (char_mask & DASH)
	    cword_word__slam_str (DASH_CHAR, temp_str);

	if (char_mask & UNDERSCORE)
	    cword_word__slam_str (UNDERSCORE_CHAR, temp_str);

	if (char_mask & APOSTROPHE)
	    cword_word__slam_str (APOSTROPHE_CHAR, temp_str);

	if (char_mask & DOT)
	    cword_word__slam_str (DOT_CHAR, temp_str);

	if (char_mask & BACKSLASH)
	    cword_word__slam_str (BACKSLASH_CHAR, temp_str);

	if (char_mask & FORWARDSLASH)
	    cword_word__slam_str (FORWARDSLASH_CHAR, temp_str);

	if (char_mask & COLON)
	    cword_word__slam_str (COLON_CHAR, temp_str);

	if (char_mask & DOLLAR)
	    cword_word__slam_str (COLON_CHAR, temp_str);

	if (char_mask & OPENSQUARE)
	    cword_word__slam_str (OPENSQUARE_CHAR, temp_str);

	if (char_mask & CLOSESQUARE)
	    cword_word__slam_str (CLOSESQUARE_CHAR, temp_str);

	if (char_mask & SEMICOLON)
	    cword_word__slam_str (SEMICOLON_CHAR, temp_str);

	if (char_mask & DIGIT)
	{
	    int count,
		str_size;

	    /*
	     *	If DIGIT *only* is defined then only digits should be found in
	     *	string, otherwise there is an error in the string.
	     */

	    if (char_mask == DIGIT)
	    {
		cword_word_del_lead_spaces (temp_str);
		cword_word_del_trail_spaces (temp_str);

    		str_size = strlen (temp_str);
		for (count = 0; count < str_size; count++)
		{
		    if (isdigit ((int) temp_str[count]) != 0)
			temp_str[count] = (char) SLAM_CHAR;
		    else
		    {
			printf ("## [1] is returning CWORD_S_FAILURE\n");

			return (CWORD_S_FAILURE);
		    }
		}		
	    }
	    else
	    {
    		str_size = strlen (temp_str);
		for (count = 0; count < str_size; count++)
		{
		    if (isdigit((int) temp_str[count]) != 0)
			temp_str[count] = (char) SLAM_CHAR;
		}
	    }
	}

	DEBUG_TRACE (
	    "temp_str is \"%s\" in module word\n", 
	    temp_str);

	/*
	 *  CHARACTER at this juncture will clean up the verification of the
	 *  string and check all other characters in the temp_str string which
	 *  weren't verified previously.
	 */

	if (char_mask & CHARACTER)
	{
	    if (cword_word_is_alphabetics_only (temp_str) != CWORD_S_NORMAL)
	    {
		printf ("## [2] is returning CWORD_S_FAILURE\n");

		return (CWORD_S_FAILURE);
	    }
	}

	/*
	 *  ?? Final sanity check but there should be a better way to do it 
    	 *  than this ??
         *
	 *  ?? This validation depends on SLAM_CHAR being alphabetic ??
	 */

	DEBUG_TRACE (
	    "Processed input from user (temp_str): %s\n",
	    temp_str);

	if (cword_word_is_alphabetics_only (temp_str) != CWORD_S_NORMAL)
	{
	    printf ("## [3] is returning CWORD_S_FAILURE\n");

	    return (CWORD_S_FAILURE);
	}
    }
    else
    {
	/*
	 *  Accept without checking as (char_mask & ALL) == TRUE.
	 */
    }

    return (CWORD_S_NORMAL);

}



/*
 *  ?? Should check whether inputted string is blank and also that it contains
 *  - after pre-processing - only alphabetic characters or blanks ??
 *
 *  ?? If these requirements are to be included a return value will need to be
 *  incorporated ??
 */

PUBLIC void cword_word_preproc_match_str (
	char			*match_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *      [@tbs@]
 *
 *  FORMAL PARAMETERS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    int count,
	str_len = strlen (match_str);

    for (count = 0; count < str_len; count++)
    {
	/*
	 *  Translates PERCENT_CHAR, WILDCARD_CHAR, QUESTION_CHAR and DOT_CHAR
	 *  characters into SPACE_CHAR and then does a to lowercase on the
	 *  whole thing.
	 *
	 *  WILDCARD_CHAR only used during grid filling.
	 */

	if ( (match_str[count] == (char) PERCENT_CHAR) ||
	      (match_str[count] == (char) WILDCARD_CHAR) ||
	       (match_str[count] == (char) QUESTION_CHAR) ||
	        (match_str[count] == (char) DOT_CHAR) )
	    match_str[count] = (char) SPACE_CHAR;
    }

    cword_word_lower_str (match_str);

    return;

}



PUBLIC BOOLEAN cword_word_is_light_in_clue (
	char			*clue_str,
	char			*light_str)

/*
 *++
 *  FUNCTIONAL DESCRIPTION:
 *
 *	Function to check whether the clue answer (light) is in the clue
 *	description.  Some pre-processing is done before a match is searched
 *	for.
 *
 *  FORMAL PARAMETERS:
 *
 *      *clue_str		(r) Clue description.
 *	*light_str		(r) Clue answer (light) description.
 *
 *  IMPLICIT INPUTS:
 *
 *      [@description or none@]
 *
 *  IMPLICIT OUTPUTS:
 *
 *      [@description or none@]
 *
 *  {@function value or completion codes@}
 *
 *      [@description or none@]
 *--
 */

{

    char temp_clue_str[CWORD_SIZEOF_CLUE + 1];

    strcpy (temp_clue_str, clue_str);

    /*
     *	?? Other preprocessing should take place before changing string to
     *	lower case.   Things like taking out spaces and other punctuation marks
     *	should be first achieved before doing a strstr () ??
     */

    /*
     *  Change string to lower case and then strstr.
     */

    cword_word_lower_str (temp_clue_str);

    if (strstr (temp_clue_str, light_str) != (char *) 0)
    {
	return (TRUE);
    }

    return (FALSE);

}

